<?php 

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use \Utilities\Messages;

class CadastroDeManual extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $dataSourceName = 'servmanualm_manualciam_servicociafili_servicogen';
    protected $dataSourceManager;
    protected $parameterBagColumns = array();
    protected $messages;
    protected $dataSourceManual = 'manualciam';

    function __construct(CrudService $crudService, ParameterBag $parameterBag, $dataSourceManager, Messages $messages){
        $this->dataSourceManager = $dataSourceManager;
        $this->messages = $messages;
        parent::__construct($crudService, $parameterBag);
    }

    public function save(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'manualciam';
        $rows = array();
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();

        foreach ($request->getDataSet()->getRows() as $row) {
            $condition = array(
                array(
                    "value" => $row['CDFILIAL'],
                    "operator" => "=",
                    "columnName" => "CDFILIAL"
                ),
                array(
                    "value" => $row['CDCIA'],
                    "operator" => "=",
                    "columnName" => "CDCIA"
                ),
                array(
                    "value" => $row['NRORG'],
                    "operator" => "=",
                    "columnName" => "NRORG"
                )
            );
            $filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $newdataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $newRows = $newdataSet->getRows();
            $maxCode = 0;
            foreach ($newRows as $rowW) {
                if($rowW['CDMANCIA'] > $maxCode)
                    $maxCode = $rowW['CDMANCIA'];
            }
            $row['CDMANCIA'] = $maxCode + 1;
            //COLOCA O NUMERO COM 4 DIGITOS
            $row['CDMANCIA'] = sprintf("%04d", $row['CDMANCIA']);
        }
        parent::save($request, $response);
    }

    public function saveServico (DTO\Request\DataSet $request, DTO\Response $response){
        foreach ($request->getDataSet()->getRows() as $row) {
            $condition = array(
                array(
                    "columnName" => "NRORG",
                    "operator" => "=",
                    "value" => $row["NRORG"]
                ),
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row["CDFILIAL"]
                ),
                array(
                    "columnName" => "CDCIA",
                    "operator" => "=",
                    "value" => $row["CDCIA"]
                ),
                array(
                    "columnName" => "NRSERVICOCIAFILI",
                    "operator" => "=",
                    "value" => $row["NRSERVICOCIAFILI"]
                ),
                array(
                    "columnName" => "CDCLASSE",
                    "operator" => "=",
                    "value" => $row["CDCLASSE"]
                ),
                array(
                    "columnName" => "CDCICLO",
                    "operator" => "=",
                    "value" => $row["CDCICLO"]
                )
            );
            $conditionManual = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row["CDFILIAL"]
                ),
                array(
                    "columnName" => "CDCIA",
                    "operator" => "=",
                    "value" => $row["CDCIA"]
                ),
                array(
                    "columnName" => "CDMANCIA",
                    "operator" => "=",
                    "value" => $row["CDMANCIA"]
                )
            );
            
            //busca os serviços
            $filterCriteria= new FilterCriteria($this->dataSourceName, $condition);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSetS = $this->dataSourceManager->findBy($newFilterCriteria);
            $servicos = $dataSetS->getRows();
            
            //busca os manuais
            $filterCriteria = new FilterCriteria($this->dataSourceManual, $conditionManual);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSetM = $this->dataSourceManager->findBy($newFilterCriteria);
            $manuais = $dataSetM->getRows();

            //Cria novo request para cada Row antes de salvar
            $dataSet = array();
            array_push($dataSet, $row);
            $dataSetS = new DataSet($this->dataSourceName, $dataSet);
            $requestS = new DTO\Request\DataSet($dataSetS, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            
            if(empty($servicos)){
                if(($this->returnStrToTime($row['DTINIVGSEMA']) >= $this->returnStrToTime($manuais[0]['DTINIVGMA'])) && ($this->returnStrToTime($row['DTFIMVGSEMA']) <= $this->returnStrToTime($manuais[0]['DTFIMVGMA']))) {
                    parent::save($requestS, $response);
                }else{
                    return $response->setError(new DTO\Response\Error($this->messages->getMessage('APP-00008'), 1));
                }
            }else{
                if($row['__is_new']){
                    if(($this->returnStrToTime($row['DTINIVGSEMA']) >= $this->returnStrToTime($manuais[0]['DTINIVGMA'])) && ($this->returnStrToTime($row['DTFIMVGSEMA']) <= $this->returnStrToTime($manuais[0]['DTFIMVGMA']))){
                        foreach ($servicos as $registro){
                            if(!($this->returnStrToTime($row['DTINIVGSEMA']) > $this->returnStrToTime($registro["DTFIMVGSEMA"]) ||
                                 $this->returnStrToTime($row['DTFIMVGSEMA']) < $this->returnStrToTime($registro["DTINIVGSEMA"]))){
                                return $response->setError(new DTO\Response\Error($servicos[0]['DSSERVICOCIAFILI'].": ".$this->messages->getMessage('APP-00007')." ".$servicos[0]['DSMANUAL'].".", 1));
                            }
                        }
                        parent::save($requestS, $response);
                    }else{
                        return $response->setError(new DTO\Response\Error($this->messages->getMessage('APP-00008'), 1));
                    }
                }else{
                    if(($this->returnStrToTime($row['DTINIVGSEMA']) >= $this->returnStrToTime($manuais[0]['DTINIVGMA'])) && ($this->returnStrToTime($row['DTFIMVGSEMA']) <= $this->returnStrToTime($manuais[0]['DTFIMVGMA']))){
                        foreach ($servicos as $registro){
                            if($row['CDMANCIA'] !== $registro['CDMANCIA']){
                                if(!($this->returnStrToTime($row['DTINIVGSEMA']) > $this->returnStrToTime($registro["DTFIMVGSEMA"]) || $this->returnStrToTime($row['DTFIMVGSEMA']) < $this->returnStrToTime($registro["DTINIVGSEMA"]))){
                                    return $response->setError(new DTO\Response\Error($servicos[0]['DSSERVICOCIAFILI'].": ".$this->messages->getMessage('APP-00007')." ".$servicos[0]['DSMANUAL'].".", 1));
                                }
                            }
                        }
                        parent::save($requestS, $response);
                    }else{
                        return $response->setError(new DTO\Response\Error($this->messages->getMessage('APP-00008'), 1));
                    }           
                }
                 
            }
        }

    }

    private function returnStrToTime($data){
        return strtotime(str_replace('/', '-', $data));
    }

    public function returnColumns(){
        return $this->crudService->getColumnsToReturn();
    }

}