<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use Utilities\Messages;
use Doctrine\ORM\EntityManager;

class TabprecoseController extends \Teknisa\GER\Controllers\TeknisaCrud{

	const NAO_POSSUI_FILHOS = 0;
	protected $dataSourceName = 'tabprecosem';
	protected $dataSourceManager;
	protected $parameterBagColumns = array();
	protected $messages;
	protected $entityManager;
	protected $connection;

	function __construct(CrudService $crudService, ParameterBag $parameterBag, $dataSourceManager, Messages $messages, EntityManager $entityManager, $workflowService, $connection){
		$this->dataSourceManager = $dataSourceManager;
		$this->messages = $messages;
		$this->entityManager = $entityManager;
		$this->workflowService = $workflowService;
		$this->connection = $connection;
		parent::__construct($crudService, $parameterBag);
	}

	public function gerarPrecoProporcionalItensSelecionados(DTO\Request\DataSet $request, DTO\Response $response) {

		$checkedRows = $request->getDataSet()->getRows();

	}

	private function excluirMenuGrids($cdfilial, $cdcia, $nrtabpreco, $cdmancia) {

		$sql = "DELETE FROM TABPRECOMENUGRID WHERE CDCIA = :CDCIA AND CDFILIAL = :CDFILIAL AND NRTABPRECO = :NRTABPRECO AND CDMANCIA = :CDMANCIA";
   
		$stmt = $this->connection->prepare($sql);

		$stmt->execute(array(
			"CDFILIAL" => $cdfilial,
			"CDCIA" => $cdcia,
			"NRTABPRECO" => $nrtabpreco,
			"CDMANCIA" => $cdmancia
		));

	}
	
	private function verificaMenuGridComFilhos($cdfilial, $cdcia, $nrtabpreco, $cdmancia) {

		$sql = "SELECT COUNT(1) AS ITENS
  FROM PRECOSECIAM
 WHERE CDFILIAL = :CDFILIAL
   AND CDCIA = :CDCIA
   AND NRTABPRECO = :NRTABPRECO
   AND CDMANCIA = :CDMANCIA";
   
		$stmt = $this->connection->prepare($sql);

		$stmt->execute(array(
			"CDFILIAL" => $cdfilial,
			"CDCIA" => $cdcia,
			"NRTABPRECO" => $nrtabpreco,
			"CDMANCIA" => $cdmancia
		));

		$result = $stmt->fetch(\PDO::FETCH_ASSOC);

		if(isset($result['ITENS'])){
			if($result['ITENS'] != self::NAO_POSSUI_FILHOS) {
				return true;
			}else{
				return false;
			}
		}
		
		return false;

	}
	
	public function excluirMenuGridsTabelaDePrecos(DTO\Request\DataSet $request, DTO\Response $response) {

		try {
			
			$rows = $request->getDataSet()->getRows();
			$menuGrids = [];
			
			for($i=0; $i < count($rows); $i++) {
	
				$cdfilial = $rows[$i]['CDFILIAL'];
				$cdcia = $rows[$i]['CDCIA'];
				$nrtabpreco = $rows[$i]['NRTABPRECO'];
				$cdmancia = $rows[$i]['CDMANCIA'];
				$dsmanual = $rows[$i]['DSMANUAL'];

				if($this->verificaMenuGridComFilhos($cdfilial, $cdcia, $nrtabpreco, $cdmancia)) {
					$menuGrids['COM_FILHOS'][] = $rows[$i];
				}else{
					$menuGrids['SEM_FILHOS'][] = $rows[$i];
				}
	
			}
	
			if(isset($menuGrids['SEM_FILHOS'])) {
				for($j=0;$j < count($menuGrids['SEM_FILHOS']); $j++) {
						$cdcia = $menuGrids['SEM_FILHOS'][$j]['CDCIA'];
					$cdfilial = $menuGrids['SEM_FILHOS'][$j]['CDFILIAL'];
					$nrtabpreco = $menuGrids['SEM_FILHOS'][$j]['NRTABPRECO'];
					$cdmancia = $menuGrids['SEM_FILHOS'][$j]['CDMANCIA'];
					$this->excluirMenuGrids($cdfilial, $cdcia, $nrtabpreco, $cdmancia);
				}
			}
	
		}catch(\Exception $e) {
			return $response->setError(new Error($e->getMessage(), 500));
		}
		
		if(isset($menuGrids['COM_FILHOS'])) {
			$response->setError(new Error($menuGrids['COM_FILHOS'],1));
		}

	}
	
    private function salvarMenuGridTabelaDePrecos($manuais, $cdcia, $cdfilial, $nrtabpreco, $nrorg, $idativo,
        $dtinclusao, $dtultatu, $nrorginclusao, $cdoperinclusao, $nrorgultatu, $cdoperultatu) {

        $sql = "INSERT INTO TABPRECOMENUGRID (CDCIA, CDFILIAL, NRTABPRECO, CDMANCIA, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) VALUES (:CDCIA, :CDFILIAL, :NRTABPRECO, :CDMANCIA, :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        $stmt = $this->connection->prepare($sql);

        for($i=0; $i < count($manuais); $i++){

            $cdmancia = $manuais[$i];

            $stmt->execute(array(
                "CDCIA" => $cdcia, 
                "CDFILIAL" => $cdfilial, 
                "NRTABPRECO" => $nrtabpreco, 
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg, 
                "IDATIVO" => $idativo, 
                "DTINCLUSAO" => $dtinclusao, 
                "DTULTATU" => $dtultatu, 
                "NRORGINCLUSAO" => $nrorginclusao, 
                "CDOPERINCLUSAO" => $cdoperinclusao, 
                "NRORGULTATU" => $nrorgultatu, 
                "CDOPERULTATU" => $cdoperultatu
            ));

        }

    }

	public function salvarMenuGrid (DTO\Request\DataSet $request, DTO\Response $response){

		$rows = $request->getDataSet()->getRows();
		$cdcia = $rows[0]['CDCIA'];
		$cdfilial = $rows[0]['CDFILIAL'];
		$nrtabpreco = $rows[0]['NRTABPRECO'];
		$nrorg = $this->crudService->getEnvironment()->getNrOrg();
		$idativo = 'S';
		$dtinclusao = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		$dtultatu = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		$nrorginclusao = $this->crudService->getEnvironment()->getNrOrg();
		$cdoperinclusao = $this->crudService->getEnvironment()->getCdOperador();
		$nrorgultatu = $this->crudService->getEnvironment()->getNrOrg();
		$cdoperultatu = $this->crudService->getEnvironment()->getCdOperador();

		$manuais = [];

		foreach ($request->getDataSet()->getRows() as $row) {
			$manuais[] = $row['CDMANCIA'];
		}

		try{
			if (gettype($manuais) == "array") {
				$this->salvarMenuGridTabelaDePrecos($manuais, $cdcia, $cdfilial, $nrtabpreco, $nrorg, $idativo, $dtinclusao, $dtultatu, $nrorginclusao, $cdoperinclusao, $nrorgultatu, $cdoperultatu);
			}
		}catch(\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}

	}

	public function saveTabelaPreco (DTO\Request\DataSet $request, DTO\Response $response){
		
		foreach ($request->getDataSet()->getRows() as $row) {
			$condition = array(
				array(
					"columnName" => "CDFILIAL",
					"operator" => "=",
					"value" => $row["CDFILIAL"]
				),
				array(
					"columnName" => "CDCIA",
					"operator" => "=",
					"value" => $row["CDCIA"]
				)
			);
			$filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
	        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
	        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
	        $registrosEncontrados = $dataSet->getRows();

	        if(empty($registrosEncontrados)){
				parent::save($request, $response);
				$manuais = $row['DSMANUAL'];
		        $cdcia = $row['CDCIA'];
		        $cdfilial = $row['CDFILIAL'];
				$columnsToReturn = $this->returnColumns();
				$nrtabpreco = $columnsToReturn[0]['NRTABPRECO'];
		        $nrorg = $this->crudService->getEnvironment()->getNrOrg();
				$idativo = 'S';
		        $dtinclusao = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		        $dtultatu = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		        $nrorginclusao = $this->crudService->getEnvironment()->getNrOrg();
		        $cdoperinclusao = $this->crudService->getEnvironment()->getCdOperador();
		        $nrorgultatu = $this->crudService->getEnvironment()->getNrOrg();
		        $cdoperultatu = $this->crudService->getEnvironment()->getCdOperador();
	        	if (gettype($manuais) == "array") {
		        	$this->salvarMenuGridTabelaDePrecos($manuais, $cdcia, $cdfilial, $nrtabpreco, $nrorg, $idativo,
        $dtinclusao, $dtultatu, $nrorginclusao, $cdoperinclusao, $nrorgultatu, $cdoperultatu);
				}
	        }else{
	        	if($row['__is_new']){
	        		foreach ($registrosEncontrados as $registro){
		        		if(!($this->returnStrToTime($row['DTINIVIGTAB']) > $this->returnStrToTime($registro["DTFIMVIGTAB"]) || $this->returnStrToTime($row['DTFIMVIGTAB']) <  $this->returnStrToTime($registro["DTINIVIGTAB"]))){
							return $response->setError(new DTO\Response\Error('COC21000_TAB_PRECO_CAD_PERIODO', 1));
		        		}
		        	}
		        	parent::save($request, $response);

			        $manuais = $row['DSMANUAL'];
			        $cdcia = $row['CDCIA'];
			        $cdfilial = $row['CDFILIAL'];
					$columnsToReturn = $this->returnColumns();
					$nrtabpreco = $columnsToReturn[0]['NRTABPRECO'];
			        $nrorg = $this->crudService->getEnvironment()->getNrOrg();
					$idativo = 'S';
			        $dtinclusao = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
			        $dtultatu = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
			        $nrorginclusao = $this->crudService->getEnvironment()->getNrOrg();
			        $cdoperinclusao = $this->crudService->getEnvironment()->getCdOperador();
			        $nrorgultatu = $this->crudService->getEnvironment()->getNrOrg();
			        $cdoperultatu = $this->crudService->getEnvironment()->getCdOperador();
		        	if (gettype($manuais) == "array") {
			        	$this->salvarMenuGridTabelaDePrecos($manuais, $cdcia, $cdfilial, $nrtabpreco, $nrorg, $idativo,
	        $dtinclusao, $dtultatu, $nrorginclusao, $cdoperinclusao, $nrorgultatu, $cdoperultatu);
					}

	        	}else{
	        		foreach ($registrosEncontrados as $registro){
	        			if($row['NRTABPRECO'] !== $registro['NRTABPRECO']){
	        				if(!($this->returnStrToTime($row['DTINIVIGTAB']) > $this->returnStrToTime($registro["DTFIMVIGTAB"]) || $this->returnStrToTime($row['DTFIMVIGTAB']) <  $this->returnStrToTime($registro["DTINIVIGTAB"]))){
								return $response->setError(new DTO\Response\Error('COC21000_TAB_PRECO_CAD_PERIODO', 1));
			        		}
	        			}
		        	}
		        	$this->atualizaServicoTabela($request, $response);
		        	parent::save($request, $response);
	        	}
	        }
	    }
	}

	private function atualizaServicoTabela($request, $response){
		$connection = $this->entityManager->getConnection();
		$rows = $request->getDataSet()->getRows();
		$conjuntoParametros = array();
		foreach ($rows as $row) {

			array_push($conjuntoParametros, array(
				":P_VIGENCIAINITAB_OLD" => substr(strval($row['BACKUPROW']['DTINIVIGTAB']), 0, 10),
				":P_VIGENCIAFIMTAB_OLD" => substr(strval($row['BACKUPROW']['DTFIMVIGTAB']), 0, 10),
				":P_VIGENCIAINITAB" => substr(strval($row['DTINIVIGTAB']), 0,10),
				":P_VIGENCIAFIMTAB" => substr(strval($row['DTFIMVIGTAB']), 0, 10),
				":P_CDFILIAL" => $row['CDFILIAL'],
				":P_CDCIA" => $row['CDCIA'],
				":P_NRTABPRECO" => $row['NRTABPRECO']
			));

		}

		$sql = "UPDATE PRECOSECIAM
						   SET DTINIVIGSE = CASE WHEN DTINIVIGSE = :P_VIGENCIAINITAB_OLD THEN TO_DATE(:P_VIGENCIAINITAB, 'DD/MM/YYYY') WHEN DTINIVIGSE < :P_VIGENCIAINITAB THEN TO_DATE(:P_VIGENCIAINITAB, 'DD/MM/YYYY') ELSE DTINIVIGSE END,
						       DTFIMVIGSE = CASE WHEN DTFIMVIGSE = :P_VIGENCIAFIMTAB_OLD THEN TO_DATE(:P_VIGENCIAFIMTAB, 'DD/MM/YYYY') WHEN DTFIMVIGSE > :P_VIGENCIAFIMTAB THEN TO_DATE(:P_VIGENCIAFIMTAB, 'DD/MM/YYYY') ELSE DTFIMVIGSE END
						 WHERE CDFILIAL = :P_CDFILIAL
						   AND CDCIA = :P_CDCIA
						   AND NRTABPRECO = :P_NRTABPRECO";
		$conn = $this->entityManager->getConnection();
		$query = $conn->prepare($sql);

		foreach ($conjuntoParametros as $params) {
			try {
				$query->execute($params);
			} catch(\Exception $e) {
				return $response->setError(new Error($e->getMessage(), 500));
			}
		}

	}

	public function createWorkflow (DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$row = $request->getDataSet()->getRows()[1];
			$primaryKeys = array(
				'CDCIA' => $row['CDCIA'], 
				'CDFILIAL' => $row['CDFILIAL'],
				'NRORG' => $row['NRORG'],
				'NRTABPRECO' => $row['NRTABPRECO']
			);
			$dataToSave = $request->getDataSet()->getRows()[0];
			$nmitemwflow = $row['NMTABPRECO'];
			$dsmotivowflow = $row['DSMOTIVOACAO'];
			$nmtabelawflow = 'TABPRECOSEM';
			$idtipoperwflow = $row['IDTIPOPERWFLOW'];
			$nritemprocaprova = $row['NRITEMPROCAPROVA'];
			$nrprocessoaprova = $row['NRPROCESSOAPROVA'];
			$rowWorkflow = $this->workflowService->createRowToWorkFlowProcess((array)$dataToSave, $primaryKeys, $dsmotivowflow, $nmitemwflow, $nmtabelawflow, $idtipoperwflow, $nritemprocaprova, $nrprocessoaprova);
			$this->workflowService->createWorkFlowProcess($primaryKeys, (array)$dataToSave, $rowWorkflow/*, true, 'IDANALIZEATIV'*/);
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

    private function returnStrToTime($data){
  		return intval(strtotime(str_replace('/', '-', $data)));
  	}

    public function returnColumns(){
  		return $this->crudService->getColumnsToReturn();
  	}

}