// FILE: ./assets/js/utilities/COCUtil.js
/* global Util angular Configuration moment */
var COCUtil = function(templateManager, MessageService) {
    var trataNumerico = function(value, decimalSeparator) {
        if (typeof value == "number") {
            return value.toString().split(".").join(decimalSeparator);
        }
        return (value) ? value.toString() : value + '';
    };

    /*
     * Usar no FieldOnKeydown
     *
     */

    this.lengthCurrency = function(args) {
        var value;
        var fieldValue;
        var field = args.owner.field;
        var allowedKeys = ['Tab', 'ArrowLeft', 'ArrowRight', 'ArrowUp', 'ArrowDown', 'End', 'Home', 'Shift', 'Control', 'Delete', 'Backspace', 'Alt'];

        let decimalSeparator = ',';
        let thousandSeparator = '.';

        if (field.mask && field.mask.params && field.mask.params.decimal) {
            decimalSeparator = field.mask.params.decimal;
        }

        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = trataNumerico(field.widget.currentRow[field.name + '_inline'], decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        } else {
            fieldValue = trataNumerico(field.value(), decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        }

        fieldValue = fieldValue.toString();
        if (event.target.selectionStart !== event.target.selectionEnd) {
            fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
        }

        let posInsertChar = event.target.selectionStart;

        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        var last = Math.max(...[value.lastIndexOf(','), value.lastIndexOf('.')]);
        var separador;
        var decimalLength = value.split('-').join('').split('.').join('').split(',')[1] ? value.split('-').join('').split('.').join('').split(',')[1].length : 0;
        var precisionLength = 0;

        if (field.mask && field.mask.params && field.mask.params.precision) {
            precisionLength = field.mask.params.precision;
        }

        if (last > -1) {
            separador = value.substring(0, last).length >= field.maxLengthCurrency - precisionLength ? ',' : value[last];
            value = value.substr(0, last).concat(separador + value.substr(last + 1, value.length));
            value = value.split('.').join('');
            last = value.lastIndexOf(',');
            realLength = value.substr(0, last).length;
            decimalLength = value.substr(last + 1, value.length).length;
        }

        let isDecimalSeparator = (args.e.key === decimalSeparator);
        let allowDecimalSeparator = ((precisionLength > 0) && (!value.includes(decimalSeparator)));
        let posDecimalseparator = fieldValue.indexOf(decimalSeparator);
        if (isDecimalSeparator && !allowDecimalSeparator) {
            args.e.preventDefault();
        }

        if (field.mask && field.mask.params && field.mask.params.thousands) {
            thousandSeparator = field.mask.params.thousands;
        }
        let isThousandSeparator = (args.e.key === thousandSeparator);

        if (isThousandSeparator) {
            args.e.preventDefault();
        }

        let allowNegative = true;
        if (field.mask && field.mask.params) {
            allowNegative = field.mask.params.allowNegative;
        }
        allowNegative = (allowNegative && !value.includes('-'));
        let isNegativeSimbol = (args.e.key === '-');

        if (isNegativeSimbol && (!allowNegative || (event.target.selectionStart > 0))) {
            args.e.preventDefault();
        }
        if (!allowedKeys.includes(args.e.key) && !isNegativeSimbol && !isDecimalSeparator && !isThousandSeparator) {
            if (realLength >= field.maxLengthCurrency) {
                if ((posDecimalseparator < 0) || (posInsertChar <= posDecimalseparator))
                    args.e.preventDefault();
            }

            if ((posDecimalseparator >= 0) && (posInsertChar > posDecimalseparator) && (precisionLength == decimalLength)) {
                args.e.preventDefault();
            }
        }
    };

    this.blockDependentField = function(field, nameFields, modifyValidations = false, treatFirstNode = true) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        const widget = field.widget;
        let firstNode = nameFields[0];
        const fieldValue = field.value();
        let fieldFirstNode = widget.getField(firstNode);
        nameFields.forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                cField.readOnly = true;
                if (modifyValidations) changeValidations(cField);
            }
        });
        if (fieldFirstNode && treatFirstNode) {
            fieldFirstNode.readOnly = !fieldValue;
            if (modifyValidations) changeValidations(fieldFirstNode);
        }
        templateManager.updateTemplate();
    };

    function changeValidations(field) {
        field.validations = field.readOnly ? {} : {
            required: true
        };
    }

    this.updateFilterFields = function(fieldsUpdate, nameFilters, rowToSet, reloadField = false, widget = null) {
        nameFilters = typeof nameFilters === 'string' ? [nameFilters] : nameFilters;
        fieldsUpdate = Array.isArray(fieldsUpdate) ? fieldsUpdate : [fieldsUpdate];
        reloadField = typeof reloadField == 'string' ? [reloadField] : reloadField;
        fieldsUpdate.forEach(cField => {
            cField = typeof cField == 'string' ? widget.getField(cField) : cField;
            if (cField && cField.dataSourceFilter) {
                let oldDataSourceFilter = angular.copy(cField.dataSourceFilter);
                cField.dataSourceFilter.forEach(filter => {
                    if (nameFilters.indexOf(filter.name) > -1) {
                        filter.value = rowToSet[filter.name];
                    }
                });
                if (reloadField == true || (Array.isArray(reloadField) && reloadField.indexOf(cField.name) > -1)) this.reloadField(cField, oldDataSourceFilter);
            }
        });
    };

    this.reloadField = function(field, oldDataSourceFilter = null) {
        let reloaded = false;
        if (field) {
            if (field.dataSourceFilter && oldDataSourceFilter && !angular.equals(field.dataSourceFilter, oldDataSourceFilter)) {
                field.reload();
                reloaded = true;
            } else {
                field.reload();
                reloaded = true;
            }
        }
        return reloaded;
    };

    this.getValuesInRow = function(widget, fieldAndPropValue, emptyProps = [], emptyValue = '') { // {FIELDNAME: NAME_GET_ROW}
        let result = {};
        emptyProps = Array.isArray(emptyProps) ? emptyProps : [emptyProps];
        Object.keys(fieldAndPropValue).forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                let valueFields = Array.isArray(fieldAndPropValue[nameField]) ? fieldAndPropValue[nameField] : [fieldAndPropValue[nameField]];
                let fieldValue = cField.value();
                valueFields.forEach(valueField => {
                    result[valueField] = fieldValue ? widget.currentRow[valueField] : '';
                });
            }
        });
        emptyProps.forEach(prop => {
            result[prop] = emptyValue;
        });
        return result;
    };

    this.clearDataSourceFilter = function(widget, fields, fieldsToReload = [], flagIgnoreValueInFilter = 'T') {
        widget = typeof widget === 'string' ? templateManager.container.getWidget(widget) : widget;
        fieldsToReload = typeof fieldsToReload === 'string' ? [fieldsToReload] : fieldsToReload;
        fields = Array.isArray(fields) ? fields : [fields];
        fields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field && field.dataSourceFilter && field.dataSourceFilter.length) {
                field.dataSourceFilter.forEach(filter => {
                    if (flagIgnoreValueInFilter !== false) {
                        filter.value = filter.value === flagIgnoreValueInFilter ? filter.value : '';
                    } else {
                        filter.value = "";
                    }
                });
                if (fieldsToReload.indexOf(field.name) > -1) field.reload();
            }
        });
    };

    this.clearSelectMultiple = function(field) {
        let widget = field.widget;
        let row = widget.currentRow;
        if (field.typeClear === 'default') {
            field.clearValue();
            return;
        }
        if (field.valueField && field.descriptionField) {
            let {
                valueField,
                descriptionField
            } = field;
            row[valueField] = [];
            row[descriptionField] = [];
        } else if (field.valueFields) {
            let descriptionFields = field.descriptionField ? [field.descriptionField] : field.descriptionFields;
            field.valueFields.forEach(valueField => {
                row[valueField] = '';
            });
            descriptionFields.forEach(descriptionField => {
                row[descriptionField] = '';
            });
        } else if (field.outData) {
            Object.values(field.outData).forEach(outData => {
                row[outData] = '';
            });
        } else if (typeof field.clearValue == 'function') {
            field.clearValue();
        }
    };

    function changeFieldCheckbox(widget, field, nameFieldCheckbox) {
        let fieldCheckbox = widget.getField(nameFieldCheckbox);
        let row = widget.currentRow;
        if (fieldCheckbox) {
            row[nameFieldCheckbox] = field.readOnly ? fieldCheckbox.valueToTrue : fieldCheckbox.valueToFalse;
            fieldCheckbox.checked = field.readOnly;
            changeValidations(field);
        }
    }

    this.defaultClearField = function(field) {
        let widget = field.widget;
        let row = widget.currentRow;
        let valueToSet = "";
        if (field.valueField && field.descriptionField) {
            let {
                valueField,
                descriptionField
            } = field;
            row[valueField] = valueToSet;
            row[descriptionField] = valueToSet;
        } else if (field.outData) {
            Object.values(field.outData).forEach(itemRow => {
                row[itemRow] = valueToSet;
            });
        } else if (field.valueFields) {
            field.valueFields.forEach(itemRow => {
                row[itemRow] = valueToSet;
            });
        }
    };

    // Nota: Função não altera readOnly de campos checkbox (ou outros) pois pode haver situações que se queira apenas limpar o campo e não bloqueá-lo
    this.clearFields = function(widget, nameFields = [], fieldNameAndCheckbox = null) {
        let functionByTemplate = Object.freeze({
            "select-multiple.html": this.clearSelectMultiple,
            default: this.defaultClearField
        });
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        nameFields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field) {
                let functionClearField = Object.keys(functionByTemplate).find(template => {
                    return field.template.indexOf(template) > -1;
                });
                functionClearField = functionByTemplate[functionClearField] || functionByTemplate.default;
                functionClearField(field);
                if (fieldNameAndCheckbox) {
                    if (typeof fieldNameAndCheckbox == 'string' && fieldNameAndCheckbox[field.name]) {
                        changeFieldCheckbox(widget, field, fieldNameAndCheckbox);
                    } else if (typeof fieldNameAndCheckbox == 'object' && !Array.isArray(fieldNameAndCheckbox)) {
                        if (fieldNameAndCheckbox[field.name]) {
                            changeFieldCheckbox(widget, field, fieldNameAndCheckbox[field.name]);
                        }
                    }
                }
            }
        });
        templateManager.updateTemplate();
    };

    function getDatesInRow(widget, fieldStartDate, fieldEndDate) {
        let row = widget.currentRow;
        return {
            startDate: row[fieldStartDate.name],
            endDate: row[fieldEndDate.name]
        };
    }

    function isValidRange(startDate, endDate) {
        return !moment(endDate, 'DD/MM/YYYY').isBefore(moment(startDate, 'DD/MM/YYYY'));
    }

    this.onChangeDate = function(args, fieldStartDate, fieldEndDate, showMessage = true) {
        const currentField = args.owner.field;
        const widget = currentField.widget;
        fieldStartDate = typeof fieldStartDate == 'string' ? widget.getField(fieldStartDate) : fieldStartDate;
        fieldEndDate = typeof fieldEndDate == 'string' ? widget.getField(fieldEndDate) : fieldEndDate;
        if (fieldStartDate && fieldEndDate) {
            if (!fieldStartDate.value() || !fieldEndDate.value()) return;
            let {
                startDate,
                endDate
            } = getDatesInRow(widget, fieldStartDate, fieldEndDate);
            let isValidRangeDate = isValidRange(startDate, endDate);
            if (!isValidRangeDate) {
                fieldEndDate.clearValue(fieldEndDate);
                if (showMessage) MessageService.showMessageByCode("MSG_COC_DATAINICIAL_MENOR_DATAFINAL");
                return false;
            }
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("COCUtil", COCUtil);
});

// FILE: ./assets/js/utilities/FieldUtil.js
var FieldUtil = function() {

    var parent = this;

    //Limpa um campo e todos os valores no outData desse campo
    this.clearField = function(field) {
        field.setValue(null);
        if (field.outData) {
            for (var i in field.outData) {
                field.widget.currentRow[field.outData[i]] = null;
            }
        }
    };

    this.enableFilteredField = function(filterValues, filteredField) {

        //se algum dos filtros passados não estiver preenchido, o campo fica travado
        var areFiltersSet = filterValues.reduce(function(prev, current) {
            return Boolean(prev && (current || current === 0));
        }, true);

        if (areFiltersSet) {
            filteredField.readOnly = false;
        } else {
            filteredField.readOnly = true;
        }
    };

    this.toggleField = function(field, enable) {
        if (enable) {
            field.readOnly = false;
        } else {
            parent.clearField(field);
            field.readOnly = true;
        }
    };

    this.toggleFieldVisibility = function(field, enable) {
        if (enable) {
            field.isVisible = true;
        } else {
            parent.clearField(field);
            field.isVisible = false;
        }
    };

    this.changeTagRichText = function(fieldValue) {
        if (typeof(fieldValue) === "string") {
            var newFieldValue;

            if (fieldValue.substr(0, 3) == "<p>") {
                newFieldValue = fieldValue.replace(/<p>/g, "").replace(/<\/p>/g, "<br>");
            } else if (fieldValue.substr(0, 3) == "<p ") {
                newFieldValue = fieldValue.replace(/<p/g, "<span").replace(/<\/p>/g, "</span><br>");
            } else {
                newFieldValue = fieldValue;
            }

            if (fieldValue.substr(-4) == "</p>") {
                newFieldValue = newFieldValue.slice("", -4);
            }

            return newFieldValue;
        } else {
            return fieldValue;
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("FieldUtil", FieldUtil);
});

// FILE: ./assets/js/utilities/FilterUtil.js
var FilterUtil = function() {

    var parent = this;

    /*
     *retorna a própria função que será usada nas funções aplicadas sobre o array de filtros
     *se operator não for passado, a função retornada será uma default, 
     *que checa por ambos os tipos de igualdade, '=' e 'LIKE'
     *se um valor truthy for passado para negate, a função retornada terá os resultados invertidos,
     *retornando false para o filtro especificado e true em todas os outros filtros. Útil em Array.filter
     */
    this._getFilterSearchFunction = function(filterName, operator, negate) {

        //se negate for null, a expressão será comparada contra true
        //se for true, será comparada contra false
        negate = !negate;
        if (operator) {
            return function(item) {
                return ((item.name === filterName && item.operator === operator) == negate);
            };
        } else { //caso default
            return function(item) {
                return ((item.name === filterName && (item.operator === "=" ||
                    item.operator === "LIKE")) == negate);
            };
        }
    };

    this.updateFilter = function(filterableObject, filterValue, filterName, operator) {

        var filterObject = {
            "name": filterName,
            "value": filterValue,
            "operator": operator || "="
        };
        //se filterableObject não tiver filtro, cria-se um novo
        if (!Array.isArray(filterableObject.dataSourceFilter)) {
            filterableObject.dataSourceFilter = new Array(filterObject);
        }
        //se tiver, checa se já existe um filtro de igualdade para a propriedade e operador procurados
        else {
            var currentFilterIndex = filterableObject.dataSourceFilter.findIndex(
                parent._getFilterSearchFunction(filterName, operator));
            /*function(item){
                return (item.name===filterName && (item.operator === "=" ||
                                                     item.operator === "LIKE"));
            });*/
            //se sim, substitui-o
            if (currentFilterIndex !== -1) {
                filterableObject.dataSourceFilter[currentFilterIndex] = filterObject;
            }
            //se não, o adiciona à lista de filtros
            else {
                filterableObject.dataSourceFilter.push(filterObject);
            }
        }
    };

    //Retorna o valor de um filtro de igualdade com o nome "name" em filterArray
    this.getFilterValueByName = function(filterArray, name, operator) {

        if (!Array.isArray(filterArray)) {
            return "";
        }

        var desiredFilterObject = filterArray.find(
            parent._getFilterSearchFunction(name, operator)
        );
        if (desiredFilterObject) {
            return desiredFilterObject.value;
        } else {
            return "";
        }
    };

    //Retorna uma cópia de filterArrray sem o filtro de igualdade com o nome "name"
    this.removeFilterObjectByName = function(filterArray, name, operator) {

        if (!Array.isArray(filterArray)) {
            return filterArray;
        }

        var desiredFilterObject = filterArray.filter(
            parent._getFilterSearchFunction(name, operator, true)
        );
        return desiredFilterObject;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("FilterUtil", FilterUtil);
});

// FILE: ./assets/js/services/coc00710_agrupcompanhiaService.js
var coc00710_agrupcompanhiaService = function(restEngine) {

    this.salvaCompanhias = function(dataset) {
        return restEngine.post("coc#/crud/itgrupocia/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00710_agrupcompanhiaService", coc00710_agrupcompanhiaService);
});

// FILE: ./assets/js/services/coc00800_personalizacompanhiaService.js
var coc00800_personalizacompanhiaService = function(ScreenService, templateManager, restEngine) {

    this.findFilial = function() {

        return restEngine.post("coc#/crud/filial_filioper", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_CDFILIAL")
                }
            ]
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00800_personalizacompanhiaService", coc00800_personalizacompanhiaService);
});

// FILE: ./assets/js/services/coc00900_frequenciaciclogeralService.js
var coc00900_frequenciaciclogeralService = function(ScreenService, templateManager, restEngine) {

    this.novaFrequenciaCiclo = function(data) {

        return restEngine.post("coc#/frciclocia_anual/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.findFrequenciaCiclo = function(cdfilial, cdCia, dateInit, dateFinal) {

        return restEngine.post("coc#/frciclocia_anual", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdCia
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dateFinal
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dateInit
                }
            ]
        });
    };

    this.excluiFrequenciaCiclo = function(data) {
        return restEngine.post("coc#/frciclocia_cdciclo/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00900_frequenciaciclogeralService", coc00900_frequenciaciclogeralService);
});

// FILE: ./assets/js/services/coc00901_frequenciaciclovooService.js
var coc00901_frequenciaciclovooService = function(ScreenService, templateManager, restEngine) {

    this.novaFrequenciaCiclo = function(data) {

        return restEngine.post("coc#/frciclovoo_anual/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.findFrequenciaCiclo = function(Filial, cdCia, dateInit, dateFinal, nrvoo, cdtrecho) {

        return restEngine.post("coc#/frciclovoo_anual", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": Filial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdCia
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dateFinal
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dateInit
                },
                {
                    "name": "NRVOO",
                    "operator": "=",
                    "value": nrvoo
                },
                {
                    "name": "CDTRECHO",
                    "operator": "=",
                    "value": cdtrecho
                }
            ]
        });
    };

    this.excluiFrequenciaCiclo = function(data) {
        return restEngine.post("coc#/frciclovoo_cdciclo/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    //Frequencia de Voo de Classe
    this.novaFrequenciaCicloClasse = function(data) {

        return restEngine.post("coc#/frciclovcla_anual/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    //Frequencia de Servico
    this.novaFrequenciaCicloServico = function(data) {

        return restEngine.post("coc#/frcicloserm_anual/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.findFrequenciaCicloClasse = function(Filial, cdCia, dateInit, dateFinal, nrvoo, cdtrecho, cdclasse) {

        return restEngine.post("coc#/frciclovcla_anual", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": Filial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdCia
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dateFinal
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dateInit
                },
                {
                    "name": "NRVOO",
                    "operator": "=",
                    "value": nrvoo
                },
                {
                    "name": "CDTRECHO",
                    "operator": "=",
                    "value": cdtrecho
                },
                {
                    "name": "CDCLASSE",
                    "operator": "=",
                    "value": cdclasse
                }
            ]
        });
    };

    this.findFrequenciaCicloServico = function(cdCia, dateInit, dateFinal, nrvoo, cdtrecho, nrservicociafili, cdciclo, cdclasse, cdfilial) {

        return restEngine.post("coc#/frcicloserm_anual", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdCia
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dateFinal
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dateInit
                },
                {
                    "name": "NRVOO",
                    "operator": "=",
                    "value": nrvoo
                },
                {
                    "name": "CDTRECHO",
                    "operator": "=",
                    "value": cdtrecho
                },
                {
                    "name": "CDCLASSE",
                    "operator": "=",
                    "value": cdclasse
                },
                {
                    "name": "NRSERVICOCIAFILI",
                    "operator": "=",
                    "value": nrservicociafili
                }
            ]
        });
    };

    this.excluiFrequenciaCicloClasse = function(data) {
        return restEngine.post("coc#/frciclovcla_cdciclo/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.excluiFrequenciaCicloServico = function(data) {
        return restEngine.post("coc#/frcicloserm_anual/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00901_frequenciaciclovooService", coc00901_frequenciaciclovooService);
});

// FILE: ./assets/js/services/coc01000_servicogenericoService.js
var coc01000_servicogenericoService = function(restEngine) {

    //Com o serviço genérico, busca os serviços utilizados por alguma companhia
    this.getServByServGen = function(nrsgen, nrorg) {
        return restEngine.post("coc#/crud/servicociafili", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRSERVICOGEN",
                    "operator": "=",
                    "value": nrsgen
                },
                {
                    "name": "IDSITSERVICO",
                    "operator": "=",
                    "value": "1"
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }]
        });
    };

    //Com os códigos da companhias, busca os nomes para exibir
    this.getNmciaByCdcia = function(companhias) {
        return restEngine.post("coc#/crud/companhia", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDCIA",
                    "operator": "IN",
                    "value": companhias
                }]
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01000_servicogenericoService", coc01000_servicogenericoService);
});

// FILE: ./assets/js/services/coc01001_servico_companhiaService.js
var coc01001_servico_companhiaService = function(restEngine) {

    this.cadAuto = function(dataset) {
        return restEngine.post("coc#/crud/servclasse/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.cadAutoCiclos = function(dataset) {
        return restEngine.post("coc#/crud/servciclo/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.cadProdutosCia = function(dataset) {
        return restEngine.post("coc#/crud/produtocia/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01001_servico_companhiaService", coc01001_servico_companhiaService);
});

// FILE: ./assets/js/services/coc01400_receitaService.js
var coc01400_receitaService = function(ScreenService, templateManager, restEngine, localStorageEngine, InflightorGeneral) {

    this.solicitaWorkFlowComposicao = function(row) {
        return restEngine.post("coc#/salvaWorkflow", {
            "requestType": "Row",
            "row": row
        });
    };

    this.retornaDetalhesReceita = function(cdreceita) {

        return restEngine.post("coc#/crud/receitam_operador_companhia", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": cdreceita
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });

    };

    this.atualizaFormulario = function(cdreceita, nrvariacaorec) {

        return restEngine.post("coc#/crud/variacaorec_operador", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": cdreceita
                },
                {
                    "name": "NRVARIACAOREC",
                    "operator": "=",
                    "value": nrvariacaorec
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });

    };

    this.getProcessosWorkflow = function(processo, item) {
        return restEngine.post("coc#/crud/processos_workflow", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRPROCESSOAPROVA",
                    "operator": "=",
                    "value": processo
                },
                {
                    "name": "NRITEMPROCAPROVA",
                    "operator": "=",
                    "value": item
                }
            ]
        });
    };

    this.returnProduto = function(cdreceita, nrvariacaorec) {

        return restEngine.post("coc#/crud/proreceitam_produto", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": cdreceita
                },
                {
                    "name": "NRVARIACAOREC",
                    "operator": "=",
                    "value": nrvariacaorec
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });
    };

    this.returnSubReceita = function(cdreceita, nrvariacaorec) {

        return restEngine.post("coc#/crud/subreceitam_receita", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": cdreceita
                },
                {
                    "name": "NRVARIACAOREC",
                    "operator": "=",
                    "value": nrvariacaorec
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });
    };

    this.salvarAlteracaoVariacaoCompProd = function(data) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return localStorageEngine.saveInServer("coc#/crud/proreceitam/save", null, data, 1, 60000);

    };

    this.salvarAlteracaoVariacaoCompRec = function(data) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return localStorageEngine.saveInServer("coc#/crud/subreceitam", null, data, 1, 60000);

    };

    this.salvarUnidadesVariacao = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/saveUnidade", {
            "requestType": "Row",
            "row": row
        });

    };

    this.removeUnidadeVariacao = function(data) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/deletaUnidade", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.salvarNovaRecCompVariacao = function(data) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/saveSubReceita", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.atribuirUnidade = function(rowParaAtualizacao) {

        return restEngine.post("coc#/atribuirUnidade", {
            "requestType": "DataSet",
            "dataset": rowParaAtualizacao
        });

    };

    this.removeReceitaVariacao = function(receita, cdreceita, nrvariacaorec) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/
        var data = [{
            CDRECEITA: cdreceita,
            NRVARIACAOREC: nrvariacaorec,
            CDSUBRECEITA: receita.CODIGO,
            __is_new: false
        }];
        return restEngine.post("coc#/deleteSubReceita", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.novaVariacao = function(cdreceita, row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        var data = {
            CDRECEITA: cdreceita,
            CDOPERADOR: row.CDOPERADOR,
            CDOPERADOR_TO_SAVE: row.CDOPERADOR,
            NMVARIACAOREC: row.NMVARIACAOREC,
            QTRENDIMENTOREC: InflightorGeneral.valueToFloat(row.QTRENDIMENTOREC),
            NRTEMPOPREPARO: row.NRTEMPOPREPARO ? InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO) : null,
            IDATIVO: row.IDATIVO ? row.IDATIVO : "S",
            DTCUSTOVARIACAO: null,
            IDTIPOCUSTOVAR: null,
            NROPCOES: row.NROPCOES,
            NRVARIACAOREC: null,
            IDANALIZEATIV: row.IDANALIZEATIV ? row.IDANALIZEATIV : null,
            __is_new: true
        };

        return restEngine.post("coc#/variacaorec/novaVariacao", {
            "requestType": "Row",
            "row": data
        });

    };

    this.salvaEdicaoVariacaoRec = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/editaVariacao", {
            "requestType": "Row",
            "row": row
        });
    };

    this.removeVariacaoReceita = function(row) {

        return restEngine.post("coc#/solicita_workflow", {
            "requestType": "Row",
            "row": row
        });
    };
    this.removeVariacaoReceitasemWorkflow = function(data) {
        /*

to Do: Implementar processo de workFlow de aprovação.

*/
        return restEngine.post("coc#/verificaAlteracoesDelete", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.novaReceita = function(row) {
        /*
Alteração tabela de RECEITAM para processo de WorkFlow
New Column IDANALIZEATIV :
'S' - Aguardando Solicitação de Workflow
'P' - Solicitação gerada e Pendente
'A' - Solicitação Aprovada
'N' - Solicitação Negada
*/

        var data = {
            DSRECEITA: row.DSRECEITA,
            DSRECIN: row.DSRECIN ? row.DSRECIN : null,
            DSRECES: row.DSRECES ? row.DSRECES : null,
            SGUNIDADE: row.SGUNIDADE,
            NRNIVELDIFICULDADE: row.NRNIVELDIFICULDADE,
            CDSETOR: row.CDSETOR ? row.CDSETOR : null,
            CDCATEGORIA: row.CDCATEGORIA ? row.CDCATEGORIA : null,
            NRTEMPOPREPARO: row.NRTEMPOPREPARO ? InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO) : null,
            CDCIA: row.CDCIA ? row.CDCIA : null,
            IDRECFLEXIVEL: row.IDRECFLEXIVEL,
            IDANALIZEATIV: row.IDANALIZEATIV ? row.IDANALIZEATIV : null,
            IDATIVO: row.IDATIVO ? row.IDATIVO : "S",
            __new_code_pk: "RECEITAM",
            __table_pk: "CDRECEITA",
            __is_new_code: true,
            __new_code_length: 10,
            __is_new: true,
            __columns_to_return: ["CDRECEITA"],
            DTALTSTATUS: moment().format("DD/MM/YYYY H:m:s"),
            IDRECVATIV: "S"
        };

        return restEngine.post("coc#/crud/receitam/save", {
            "requestType": "Row",
            "row": data
        });
    };

    this.excluiReceita = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/solicita_workflow", {
            "requestType": "Row",
            "row": row
        });

    };

    this.excluiReceitasemWorkflow = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/
        return restEngine.post("coc#/receitam/delete", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 9000000);

    };

    this.addCategoria = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        var data = {
            NMCATEGORIA: row.NMCATEGORIA,
            __is_new: true
        };

        return restEngine.post("coc#/categoria/save", {
            "requestType": "Row",
            "row": data
        });

    };

    this.unidadesPossiveis = function(data) {

        return restEngine.post("coc#/crud/filireceita_filial", {
            "requestType": "FilterData",
            "filter": data
        });

    };

    this.salvarNovoProdCompVariacao = function(row) {
        return restEngine.post("coc#/solicita_workflow_variacao", {
            "requestType": "Row",
            "dataset": row
        });

    };
    this.salvarNovoProdCompVariacaosemWorkflow = function(data) {
        return restEngine.post("coc#/saveProduto", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.removeProdCompVariacao = function(produtos, cdreceita, nrvariacaorec) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/
        var data = [{
            CDRECEITA: cdreceita,
            NRVARIACAOREC: nrvariacaorec,
            CDPRODUTO: produtos.CODIGO,
            __is_new: false
        }];

        return restEngine.post("coc#/crud/proreceitam/remove", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.salvaEdicaoModopreprec = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/crud/modpreprecm/save", {
            "requestType": "Row",
            "row": row
        });

    };

    this.salvaNovoModopreprec = function(row) {

        /*

to Do: Implementar processo de workFlow de aprovação.

*/

        return restEngine.post("coc#/savePreparo", {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteModopreprec = function(row) {

        return restEngine.post("coc#/deletePreparo", {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveTesteReceita = function(row) {

        return restEngine.post("coc#/saveTesteReceita", {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteTesteReceita = function(row) {

        return restEngine.post("coc#/deleteTesteReceita", {
            "requestType": "Row",
            "row": row
        });
    };

    this.duplicaVariacaoReceita = function(row) {
        return restEngine.post("coc#/duplicavariacaoreceita", {
            "requestType": "Row",
            "row": row
        });
    };

    this.duplicaReceita = function(row) {
        var data = {
            CDCATEGORIA: row.CDCATEGORIA,
            CDRECEITA: row.CDRECEITA,
            CDSETOR: row.CDSETOR,
            DSRECEITA: row.DSRECEITA,
            DSRECES: row.DSRECES,
            DSRECIN: row.DSRECIN,
            IDATIVO: row.IDATIVO,
            IDRECFLEXIVEL: row.IDRECFLEXIVEL,
            IDRECVATIV: row.IDRECVATIV,
            NRTEMPOPREPARO: row.NRTEMPOPREPARO,
            QTRENDIMENTOREC: row.QTRENDIMENTOREC,
            NRNIVELDIFICULDADE: row.NRNIVELDIFICULDADE,
            SGUNIDADE: row.SGUNIDADE
        };

        return restEngine.post("coc#/duplicareceita", {
            "requestType": "Row",
            "row": data
        });
    };

    this.findCusto = function(data) {

        return restEngine.post("coc#/custo_produto_coc", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.updateSubReceita = function(data) {

        return restEngine.post("coc#/saveSubReceita", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.updateSubProd = function(data) {

        return restEngine.post("coc#/saveProduto", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.updateVariacaoRec = function(row) {

        return restEngine.post("coc#/crud/variacaorec/save", {
            "requestType": "Row",
            "row": row
        });

    };

    this.solicitaWorkFlow = function(row) {
        return restEngine.post("coc#/solicita_workflow", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 20000);
    };
    this.solicitaWorkFlowVariacao = function(row) {
        return restEngine.post("coc#/solicita_workflow_variacao", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 20000);
    };

    this.solicitaEdicaoWorkFlow = function(data) {
        return restEngine.post("coc#/solicita_edicao_workflow", {
            "requestType": "DataSet",
            "dataset": data
        }, null, null, null, null, 20000);
    };

    this.solicitaEdicaoWorkFlowVariacao = function(data) {
        return restEngine.post("coc#/solicita_edicao_workflow_variacao", {
            "requestType": "DataSet",
            "dataset": data
        }, null, null, null, null, 20000);
    };
    this.getValidChanges = function(processo) {
        return restEngine.post("coc#/crud/itemprocaprova", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRPROCESSOAPROVA",
                    "operator": "=",
                    "value": processo
                }
            ]
        });
    };
    this.getDadoswFlow = function(cdreceita, nometabela) {
        return restEngine.post("coc#/workflow_dadoswflow", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "DSPRIMARYKEY",
                    "operator": "LIKE",
                    "value": "%" + cdreceita + "%"
                },
                {
                    "name": "NMTABELAWFLOW",
                    "operator": "=",
                    "value": nometabela
                }
            ]
        });
    };

    this.saveClassificacao = function(dataset) {
        return restEngine.post("coc#/saveClassificacao", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.deleteClassificacao = function(dataset) {
        return restEngine.post("coc#/deleteClassificacao", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.validateChangeSituation = function(row) {
        return InflightorGeneral.callRestEngine("coc#/validate_change_situation", "Row", row)
            .then()
            .catch(function(error) {
                if (error)
                    ScreenService.showMessage(error.data.error);
            });
    };

    this.validateVariacao = function(row) {
        return restEngine.post("coc#/validateVariacao", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": row
                }
            ]
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01400_receitaService", coc01400_receitaService);
});

// FILE: ./assets/js/services/coc01410_cadastrosubgrupoService.js
var coc01410_cadastrosubgrupoService = function(ScreenService, templateManager, restEngine) {

    this.returnProduto = function(CDSUBGRU) {
        return restEngine.post("coc#/crud/prodsubgrum_produto", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDSUBGRU",
                    "operator": "=",
                    "value": CDSUBGRU
                }
            ]
        });
    };

    this.returnReceita = function(CDSUBGRU) {
        return restEngine.post("coc#/crud/recsubgrum_receitam", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDSUBGRU",
                    "operator": "=",
                    "value": CDSUBGRU
                }
            ]
        });
    };

    this.saveProduto = function(row) {
        return restEngine.request("coc#/crud/prodsubgrum_produto/save", {
            "requestType": "Row",
            "row": row
        });
    };
    this.saveReceita = function(row) {
        return restEngine.post("coc#/crud/recsubgrum_receitam/save", {
            "requestType": "Row",
            "row": row
        });
    };
    this.removeProduto = function(data) {
        return restEngine.request("coc#/crud/prodsubgrum_produto/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };
    this.removeReceita = function(data) {
        return restEngine.post("coc#/crud/recsubgrum_receitam/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01410_cadastrosubgrupoService", coc01410_cadastrosubgrupoService);
});

// FILE: ./assets/js/services/coc03600_menu_designService.js
var coc03600_menu_designService = function(restEngine) {
    var self = this;
    var workflowPendenteAprovacaoMenuGrid;

    this.retornaFlagWorkflowPendenteAprovacaoMenuGrid = function(row) {
        var rowPendencias = {
            NRPROCESSOAPROVA: 4,
            NRITEMPROCAPROVA: 23,
            IDSITWORKFLOW: 1,
            IDTIPOPERWFLOW: null,
            DSPRIMARYKEY: "NRORG=" + row.NRORG + ";CDMANCIA=" + row.CDMANCIA + ";CDCIA=" + row.CDCIA + ";CDFILIAL=" + row.CDFILIAL + ";"
        };
        this.hasPendingWorkflow(rowPendencias).then(function(qtd) {
            var pendente = false;
            if (qtd > 0)
                pendente = true;
            this.setWorkflowPendenteAprovacaoMenuGrid(pendente);
        }.bind(this));
    };

    this.setWorkflowPendenteAprovacaoMenuGrid = function(pendente) {
        workflowPendenteAprovacaoMenuGrid = pendente;
    };

    this.getWorkflowPendenteAprovacaoMenuGrid = function() {
        return workflowPendenteAprovacaoMenuGrid;
    };

    this.subgrupoDeleteWf = function(row) {
        return restEngine.post("coc#/subgrupoDeleteWf", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

    this.insertSubgrupoWF = function(row) {
        return restEngine.post("coc#/insertSubgrupoWF", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

    this.insertServicoWF = function(row) {
        return restEngine.post("coc#/insertServicoWF", {
            "requestType": "DataSet",
            "dataset": row
        }, null, null, null, null, 60000);
    };

    this.tryDeleteWithTransaction = function(row) {
        return restEngine.post("coc#/tryDeleteWithTransaction", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

    this.copiaCiclos = function(dataset) {
        return restEngine.post("coc#/copiaCiclos", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 60000);
    };

    this.copiaMenu = function(dataset) {
        return restEngine.post("coc#/copiaMenu", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 60000);
    };

    this.copiaServico = function(dataset) {
        return restEngine.post("coc#/copiaServico", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 60000);
    };

    this.copiaSubgrupo = function(dataset) {
        return restEngine.post("coc#/copiaSubgrupo", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 60000);
    };

    this.saveLista = function(dataset) {
        return restEngine.post("coc#/crud/lista/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveListaSub = function(dataset, param) {
        var route;
        if (param) {
            route = "coc#/saveLista";
        } else {
            route = "coc#/crud/lista/save";
        }
        return restEngine.post(route, {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.geraNovoCodigo = function() {
        return restEngine.post("coc#/geraNovoCodigo", {
            "requestType": "Empty"
        });
    };

    this.saveClasseManual = function(dataset) {
        return restEngine.post("coc#/crud/classemanual/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveManual = function(dataset) {
        return restEngine.post("coc#/cadastrodemanual/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.deleteServico = function(dataset) {
        return restEngine.post("coc#/crud/servmanualm/remove", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveManualEdit = function(dataset) {
        return restEngine.post("coc#/coc03600_save_menu", {
            "requestType": "Row",
            "row": dataset
        });
    };

    this.saveMenu = function(dataset) {
        return restEngine.post("coc#/crud/manualciam/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.deleteMontagem = function(dataset) {
        return restEngine.post("coc#/crud/montasubsemam/remove", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveCicloSub = function(dataset) {
        return restEngine.post("coc#/ciclosubgrusemam_ciclo/save", {
            "requestType": "DataSet",
            "dataset": dataset,
            "timeout": 99999999
        });
    };

    this.deleteManual = function(dataset) {
        return restEngine.post("coc#/crud/manualciam/remove", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveServManual = function(dataset) {
        return restEngine.post("coc#/crud/servmanualm/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveServicociafili = function(dataset) {
        return restEngine.post("coc#/crud/servicociafili/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.salvaIntMontagem = function(dataset) {
        return restEngine.post("coc#/crud/montasubsemam/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.saveSubgrupom = function(dataset) {
        return restEngine.post("coc#/crud/subgrupom/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.calculaCustoManual = function(dataset) {

        return restEngine.post("coc#/custo_manualciam", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 60000);
    };

    this.findMontaSubsemam = function(currentRow) {

        return restEngine.post("coc#/crud/montasubsemam", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": currentRow.CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": currentRow.NRSERVICOCIAFILI
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentRow.CDCLASSE
            }, {
                "name": "CDCICLO",
                "operator": "=",
                "value": currentRow.CDCICLO
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": currentRow.CDSUBGRU
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }]
        });

    };

    this.salvaSubPrin = function(dataset) {
        return restEngine.post("coc#/crud/subgruprincipalmenu/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.findSubgruPrincipal = function(currentRow) {

        return restEngine.post("coc#/crud/subgruprincipalmenu_subgrupom", {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": currentRow.CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": currentRow.NRSERVICOCIAFILI
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentRow.CDCLASSE
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": currentRow.CDSUBGRU
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": currentRow.NRSEQSUBGRUSEMAM
            }]
        });

    };

    this.returnProduto = function(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru) {

        return restEngine.post("coc#/crud/prodsubsemam_produto", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": nrservicociafili
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": cdclasse
            }, {
                "name": "CDCICLO",
                "operator": "=",
                "value": cdciclo
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": nrseqsubgrusemam
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": cdsubgru
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }]
        });

    };

    this.returnReceita = function(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru) {

        return restEngine.post("coc#/crud/recsubsemam_receita", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": nrservicociafili
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": cdclasse
            }, {
                "name": "CDCICLO",
                "operator": "=",
                "value": cdciclo
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": nrseqsubgrusemam
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": cdsubgru
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }]
        });

    };

    this.autenticar = function(user, senha) {

        return restEngine.request("coc#/autenticarPorAd", {
            "requestType": "FilterData",
            "filter": [{
                name: "user",
                operator: "=",
                value: user
            }, {
                name: "senha",
                operator: "=",
                value: senha
            }]
        });

    };

    this.removeReceita = function(receitas) {

        var data = [{
            "CDRECEITA": receitas.CODIGO,
            "CDFILIAL": receitas.CDFILIAL,
            "CDCIA": receitas.CDCIA,
            "CDMANCIA": receitas.CDMANCIA,
            "NRSERVICOCIAFILI": receitas.NRSERVICOCIAFILI,
            "CDCLASSE": receitas.CDCLASSE,
            "CDCICLO": receitas.CDCICLO,
            "NRSEQSUBGRUSEMAM": receitas.NRSEQSUBGRUSEMAM,
            "CDSUBGRU": receitas.CDSUBGRU,
            "NRORG": receitas.NRORG,
            "__is_new": false
        }];

        return restEngine.post("coc#/crud/recsubsemam/remove", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.removeProduto = function(produtos) {

        var data = [{
            "CDPRODUTO": produtos.CODIGO,
            "CDFILIAL": produtos.CDFILIAL,
            "CDCIA": produtos.CDCIA,
            "CDMANCIA": produtos.CDMANCIA,
            "NRSERVICOCIAFILI": produtos.NRSERVICOCIAFILI,
            "CDCLASSE": produtos.CDCLASSE,
            "CDCICLO": produtos.CDCICLO,
            "NRSEQSUBGRUSEMAM": produtos.NRSEQSUBGRUSEMAM,
            "CDSUBGRU": produtos.CDSUBGRU,
            "NRORG": produtos.NRORG,
            "__is_new": false
        }];

        return restEngine.post("coc#/crud/prodsubsemam/remove", {
            "requestType": "DataSet",
            "dataset": data
        });

    };
    this.saveProduto = function(produtos) {

        var data = [{
            "CDPRODUTO": produtos.CODIGO || produtos.CDPRODUTO,
            "CDFILIAL": produtos.CDFILIAL,
            "CDCIA": produtos.CDCIA,
            "CDMANCIA": produtos.CDMANCIA,
            "NRSERVICOCIAFILI": produtos.NRSERVICOCIAFILI,
            "QTPRODSUMA": produtos.QTD,
            "TXCOPRODSUMA": produtos.COMENTARIO,
            "VRCUSPRSUBSEMAM": "",
            "CDCLASSE": produtos.CDCLASSE,
            "CDCICLO": produtos.CDCICLO,
            "CDSUBGRU": produtos.CDSUBGRU,
            "NRSEQSUBGRUSEMAM": produtos.NRSEQSUBGRUSEMAM,
            "NRORG": produtos.NRORG,
            "QTGRAMATURA": produtos.QTGRAMATURA,
            "VRPESOGRAMATURA": produtos.VRPESOGRAMATURA,
            "NRORDEM": produtos.NRORDEM,
            "__is_new": produtos.__is_new,
            "changed": produtos.changed,
            "IDANALIZEATIV": produtos.IDANALIZEATIV
        }];

        return restEngine.post("coc#/crud/prodsubsemam/save", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.saveReceita = function(receitas) {

        var data = [{
            "CDRECEITA": receitas.CODIGO || receitas.CDRECEITA,
            "CDFILIAL": receitas.CDFILIAL,
            "CDCIA": receitas.CDCIA,
            "CDMANCIA": receitas.CDMANCIA,
            "NRSERVICOCIAFILI": receitas.NRSERVICOCIAFILI,
            "QTRECSUMA": receitas.QTD,
            "TXCOMRECSUMA": receitas.COMENTARIO,
            "CDCLASSE": receitas.CDCLASSE,
            "CDCICLO": receitas.CDCICLO,
            "CDSUBGRU": receitas.CDSUBGRU,
            "NRSEQSUBGRUSEMAM": receitas.NRSEQSUBGRUSEMAM,
            "VRCUSRESUBSEMAM": "",
            "NRORG": receitas.NRORG,
            "NRORDEM": receitas.NRORDEM,
            "QTGRAMATURA": receitas.QTGRAMATURA,
            "VRPESOGRAMATURA": receitas.VRPESOGRAMATURA,
            "__is_new": receitas.__is_new,
            "changed": receitas.changed,
            "IDANALIZEATIV": receitas.IDANALIZEATIV
        }];

        return restEngine.post("coc#/crud/recsubsemam/save", {
            "requestType": "DataSet",
            "dataset": data
        });

    };

    this.removeProporcao = function(row) {

        var data = [{
            CDFILIAL: row.CDFILIAL,
            CDCIA: row.CDCIA,
            CDMANCIA: row.CDMANCIA,
            NRSERVICOCIAFILI: row.NRSERVICOCIAFILI,
            NRSEQSUBGRUSEMAM: row.NRSEQSUBGRUSEMAM,
            CDCLASSE: row.CDCLASSE,
            CDCICLO: row.CDCICLO,
            CDSUBGRU: row.CDSUBGRU,
            QTPAXDE: row.QTPAXDE,
            QTPAXATE: row.QTPAXATE,
            NRORG: row.NRORG,
            __is_new: false
        }];

        return restEngine.post("coc#/crud/propsubgruse/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.validaMenuGrid = function(row) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": row.NRORG
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "IDSITUAPROPCIA",
                "operator": "=",
                "value": row.IDSITUAPROPCIA
            }
        ];
        return restEngine.post("coc#/validaMenuGrid", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.validaInformacoesDependentesMenuGrid = function(row) {
        return restEngine.post("coc#/validaInformacoesDependentesMenuGrid", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 999999);
    };

    this.salvaWorkflow = function(row) {
        return restEngine.post("coc#/salvaWorkflow", {
            "requestType": "Row",
            "row": row
        });
    };

    this.hasPendingWorkflow = function(row) {
        var filter = [
            {
                "name": "NRPROCESSOAPROVA",
                "operator": "=",
                "value": row.NRPROCESSOAPROVA
            },
            {
                "name": "IDSITWORKFLOW",
                "operator": "=",
                "value": row.IDSITWORKFLOW
            },
            {
                "name": "NRITEMPROCAPROVA",
                "operator": "=",
                "value": row.NRITEMPROCAPROVA ? "_" + row.NRITEMPROCAPROVA + "_" : "T"
            },
            {
                "name": "IDTIPOPERWFLOW",
                "operator": "=",
                "value": row.IDTIPOPERWFLOW ? "_" + row.IDTIPOPERWFLOW + "_" : "T"
            },
            {
                "name": "DSPRIMARYKEY",
                "operator": "=",
                "value": row.DSPRIMARYKEY
            }
        ];
        return restEngine.post("coc#/crud/quantidade_workflow", {
            "requestType": "FilterData",
            "filter": filter
        }).then(function(datasource) {
            return parseInt(datasource.dataset.quantidade_workflow[0].QTD_WFLOW);
        });
    };

    this.verificaWorkflowPendenteReceitaProduto = function(dataset) {
        return restEngine.post("coc#/verificaWorkflowPendenteReceitaProduto", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.verificaWorkflowPendenteMenuGrid = function(row) {
        return restEngine.post("coc#/verificaWorkflowPendenteMenuGrid", {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset.workflowPendente[0];
        });
    };

    //Funções relativas a salvar os html
    this.getHtmlTranslationTable = function(table, quoteStyle) {
        // source: https://github.com/kvz/locutus/tree/master/src/php/strings

        var entities = {};
        var hashMap = {};
        var decimal;
        var constMappingTable = {};
        var constMappingQuoteStyle = {};
        var useTable = {};
        var useQuoteStyle = {};

        // Translate arguments
        constMappingTable[0] = "HTML_SPECIALCHARS";
        constMappingTable[1] = "HTML_ENTITIES";
        constMappingQuoteStyle[0] = "ENT_NOQUOTES";
        constMappingQuoteStyle[2] = "ENT_COMPAT";
        constMappingQuoteStyle[3] = "ENT_QUOTES";

        useTable = !isNaN(table) ? constMappingTable[table] : table ? table.toUpperCase() : "HTML_SPECIALCHARS";

        useQuoteStyle = !isNaN(quoteStyle) ? constMappingQuoteStyle[quoteStyle] : quoteStyle ? quoteStyle.toUpperCase() : "ENT_COMPAT";

        if (useTable !== "HTML_SPECIALCHARS" && useTable !== "HTML_ENTITIES") {
            throw new Error("Table: " + useTable + " not supported");
        }

        entities["38"] = "&amp;";
        if (useTable === "HTML_ENTITIES") {
            entities["160"] = "&nbsp;";
            entities["161"] = "&iexcl;";
            entities["162"] = "&cent;";
            entities["163"] = "&pound;";
            entities["164"] = "&curren;";
            entities["165"] = "&yen;";
            entities["166"] = "&brvbar;";
            entities["167"] = "&sect;";
            entities["168"] = "&uml;";
            entities["169"] = "&copy;";
            entities["170"] = "&ordf;";
            entities["171"] = "&laquo;";
            entities["172"] = "&not;";
            entities["173"] = "&shy;";
            entities["174"] = "&reg;";
            entities["175"] = "&macr;";
            entities["176"] = "&deg;";
            entities["177"] = "&plusmn;";
            entities["178"] = "&sup2;";
            entities["179"] = "&sup3;";
            entities["180"] = "&acute;";
            entities["181"] = "&micro;";
            entities["182"] = "&para;";
            entities["183"] = "&middot;";
            entities["184"] = "&cedil;";
            entities["185"] = "&sup1;";
            entities["186"] = "&ordm;";
            entities["187"] = "&raquo;";
            entities["188"] = "&frac14;";
            entities["189"] = "&frac12;";
            entities["190"] = "&frac34;";
            entities["191"] = "&iquest;";
            entities["192"] = "&Agrave;";
            entities["193"] = "&Aacute;";
            entities["194"] = "&Acirc;";
            entities["195"] = "&Atilde;";
            entities["196"] = "&Auml;";
            entities["197"] = "&Aring;";
            entities["198"] = "&AElig;";
            entities["199"] = "&Ccedil;";
            entities["200"] = "&Egrave;";
            entities["201"] = "&Eacute;";
            entities["202"] = "&Ecirc;";
            entities["203"] = "&Euml;";
            entities["204"] = "&Igrave;";
            entities["205"] = "&Iacute;";
            entities["206"] = "&Icirc;";
            entities["207"] = "&Iuml;";
            entities["208"] = "&ETH;";
            entities["209"] = "&Ntilde;";
            entities["210"] = "&Ograve;";
            entities["211"] = "&Oacute;";
            entities["212"] = "&Ocirc;";
            entities["213"] = "&Otilde;";
            entities["214"] = "&Ouml;";
            entities["215"] = "&times;";
            entities["216"] = "&Oslash;";
            entities["217"] = "&Ugrave;";
            entities["218"] = "&Uacute;";
            entities["219"] = "&Ucirc;";
            entities["220"] = "&Uuml;";
            entities["221"] = "&Yacute;";
            entities["222"] = "&THORN;";
            entities["223"] = "&szlig;";
            entities["224"] = "&agrave;";
            entities["225"] = "&aacute;";
            entities["226"] = "&acirc;";
            entities["227"] = "&atilde;";
            entities["228"] = "&auml;";
            entities["229"] = "&aring;";
            entities["230"] = "&aelig;";
            entities["231"] = "&ccedil;";
            entities["232"] = "&egrave;";
            entities["233"] = "&eacute;";
            entities["234"] = "&ecirc;";
            entities["235"] = "&euml;";
            entities["236"] = "&igrave;";
            entities["237"] = "&iacute;";
            entities["238"] = "&icirc;";
            entities["239"] = "&iuml;";
            entities["240"] = "&eth;";
            entities["241"] = "&ntilde;";
            entities["242"] = "&ograve;";
            entities["243"] = "&oacute;";
            entities["244"] = "&ocirc;";
            entities["245"] = "&otilde;";
            entities["246"] = "&ouml;";
            entities["247"] = "&divide;";
            entities["248"] = "&oslash;";
            entities["249"] = "&ugrave;";
            entities["250"] = "&uacute;";
            entities["251"] = "&ucirc;";
            entities["252"] = "&uuml;";
            entities["253"] = "&yacute;";
            entities["254"] = "&thorn;";
            entities["255"] = "&yuml;";
        }

        if (useQuoteStyle !== "ENT_NOQUOTES") {
            entities["34"] = "&quot;";
        }
        if (useQuoteStyle === "ENT_QUOTES") {
            entities["39"] = "&#39;";
        }
        entities["60"] = "&lt;";
        entities["62"] = "&gt;";

        // ascii decimals to real symbols
        for (decimal in entities) {
            if (entities.hasOwnProperty(decimal)) {
                hashMap[String.fromCharCode(decimal)] = entities[decimal];
            }
        }

        return hashMap;
    };

    this.htmlEntityDecode = function(string, quoteStyle) {
        // source: https://github.com/kvz/locutus/tree/master/src/php/strings
        var tmpStr = "";
        var entity = "";
        var symbol = "";
        tmpStr = string.toString();

        var hashMap = self.getHtmlTranslationTable("HTML_ENTITIES", quoteStyle);
        if (hashMap === false) {
            return false;
        }

        delete(hashMap["&"]);
        hashMap["&"] = "&amp;";

        for (symbol in hashMap) {
            entity = hashMap[symbol];
            tmpStr = tmpStr.split(entity).join(symbol);
        }
        tmpStr = tmpStr.split("&#039;").join("'");

        return tmpStr;
    };

    this.htmlentities = function(string, quoteStyle, charset, doubleEncode) {

        // source: https://github.com/kvz/locutus/tree/master/src/php/strings
        var hashMap = self.getHtmlTranslationTable("HTML_ENTITIES", quoteStyle);

        string = string === null ? "" : string + "";

        if (!hashMap) {
            return false;
        }

        if (quoteStyle && quoteStyle === "ENT_QUOTES") {
            hashMap["'"] = "&#039;";
        }

        doubleEncode = doubleEncode === null || !!doubleEncode;

        var regex = new RegExp("&(?:#\\d+|#x[\\da-f]+|[a-zA-Z][\\da-z]*);|[" +
            Object.keys(hashMap)
            .join("")
            // replace regexp special chars
            .replace(/([()[\]{}\-.*+?^$|\/\\])/g, "\\$1") + "]",
            "g");

        return string.replace(regex, function(ent) {
            if (ent.length > 1) {
                return doubleEncode ? hashMap["&"] + ent.substr(1) : ent;
            }

            return hashMap[ent];
        });
    };

    this.pendencias_workflow = function(row) {
        return restEngine.post("coc#/crud/pendencias_workflow", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": row.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": row.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": row.CDMANCIA
                }
            ]
        }).then(function(callback) {
            return callback.dataset.pendencias_workflow.length;
        });
    };

    this.multipleDeleteCiclo = function(row) {
        return restEngine.post("coc#/coc03600_muliple_delete", {
            "requestType": "Row",
            "row": row
        });
    };

    this.verificaTabelaPreco = function(row) {
        return restEngine.post("coc#/crud/verifica_menugrid_tabelapreco", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": row.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": row.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": row.CDMANCIA
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });
    };

    this.removeItensComposicao = function(dataset) {
        return restEngine.post("coc#/coc03600_remove_items", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03600_menu_designService", coc03600_menu_designService);
});

// FILE: ./assets/js/services/coc03602_relatorios_do_manualService.js
var coc03602_relatorios_do_manualService = function(ScreenService, InflightorGeneral, ZHPromise) {
    this.verfImgCiclosubgrusemam = function(widget) {
        var deferred = ZHPromise.defer();
        if (widget.currentRow.HIDEIMG === "false") {
            InflightorGeneral.callRestEngine("coc#/crud/verf_img_ciclosubgrusemam", "FilterData", getFilter(widget)).then(function(callback) {
                if (callback.dataset.verf_img_ciclosubgrusemam.length) {
                    ScreenService.confirmMessage("Foram encontrados ciclos sem foto cadastrada. Deseja gerar o relatório?", "CONFIRMATION",
                        function() {
                            deferred.resolve();
                        },
                        function() {
                            deferred.reject();
                        }
                    );
                } else deferred.resolve();
            });
        } else deferred.resolve();
        return deferred.promise;
    };

    var getFilter = function(widget) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widget.currentRow.CDMANCIA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widget.currentRow.NRORG
            },
            {
                "name": "PATHTOFILTER",
                "operator": "=",
                "value": window.undefinedImageInFileServer
            }
        ];
        if (widget.currentRow.TIDSERVICOCLASSE == "N") {
            var nrservFilter = [];
            var cdclasseFilter = [];
            widget.getField("IDSERVICOCLASSE").dataSource.getCheckedRows().forEach(function(servico) {
                nrservFilter.push(servico.NRSERVICOCIAFILI);
                cdclasseFilter.push(servico.CDCLASSE);
            });
            filter.push({
                "name": "NRSERVICOCIAFILI",
                "operator": "IN",
                "value": nrservFilter
            }, {
                "name": "CDCLASSE",
                "operator": "IN",
                "value": cdclasseFilter
            });
        }
        if (widget.currentRow.DSSUBGRU !== "") {
            var cdsubgruFilter = [];
            var nrsubgruFilter = [];
            widget.getField("DSSUBGRU").dataSource.getCheckedRows().forEach(function(subgrupo) {
                cdsubgruFilter.push(subgrupo.CDSUBGRU);
                nrsubgruFilter.push(subgrupo.NRSEQSUBGRUSEMAM);
            });
            filter.push({
                "name": "CDSUBGRU",
                "operator": "IN",
                "value": cdsubgruFilter
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "IN",
                "value": nrsubgruFilter
            });
        }
        if (widget.currentRow.IDCICLO !== "") {
            var cdcicloFilter = [];
            widget.getField("IDCICLO").dataSource.getCheckedRows().forEach(function(ciclo) {
                cdcicloFilter.push(ciclo.CDCICLO);
            });
            filter.push({
                "name": "CDCICLO",
                "operator": "IN",
                "value": cdcicloFilter
            });
        }

        return filter;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03602_relatorios_do_manualService", coc03602_relatorios_do_manualService);
});

// FILE: ./assets/js/services/coc03610_custeio_menu_gridService.js
var coc03610_custeio_menu_gridService = function(restEngine) {

    this.aprovacusto = function(row) {
        return restEngine.post("coc#/aprovacaodocusteio", {
            "requestType": "Row",
            "row": row
        });
    };

    this.autenticar = function(user, senha) {
        return restEngine.request("coc#/autenticarPorAd", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "user",
                    operator: "=",
                    value: user
                },
                {
                    name: "senha",
                    operator: "=",
                    value: senha
                }
            ]
        });

    };

    this.returnProduto = function(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru) {

        return restEngine.post("coc#/crud/prodsubsemam_produto", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": nrservicociafili
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": cdclasse
            }, {
                "name": "CDCICLO",
                "operator": "=",
                "value": cdciclo
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": nrseqsubgrusemam
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": cdsubgru
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }]
        });

    };

    this.returnReceita = function(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru) {

        return restEngine.post("coc#/crud/recsubsemam_receita", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": nrservicociafili
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": cdclasse
            }, {
                "name": "CDCICLO",
                "operator": "=",
                "value": cdciclo
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": nrseqsubgrusemam
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": cdsubgru
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }]
        });

    };

    this.runProcedureCustoManual = function(row) {
        return restEngine.post("coc#/calcula_custo_manualciam", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 9999999);
    };

    this.getFooterValues = function(cdfilial, cdcia, cdmancia) {
        return restEngine.post("coc#/crud/manualciam", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: cdfilial
                },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: cdcia
                },
                {
                    name: "CDMANCIA",
                    operator: "=",
                    value: cdmancia
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ]
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03610_custeio_menu_gridService", coc03610_custeio_menu_gridService);
});

// FILE: ./assets/js/services/coc23100_geracaocustomanualService.js
var coc23100_geracaocustomanualService = function(InflightorGeneral, restEngine) {

    this.findVrIndice = function(cdindice, dtrefind) {
        return restEngine.post("coc#/crud/movindice", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDINDICE",
                    "operator": "=",
                    "value": cdindice
                },
                {
                    "name": "DTREFIND",
                    "operator": "=",
                    "value": dtrefind
                }
            ]
        });
    };

    this.checkCdIndice = function(cdindice) {
        return restEngine.post("coc#/crud/paramcomis", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDINDICEMC",
                    "operator": "=",
                    "value": cdindice
                }
            ]
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc23100_geracaocustomanualService", coc23100_geracaocustomanualService);
});

// FILE: ./assets/js/services/coc26000_cicloService.js
var coc26000_cicloService = function(InflightorGeneral, restEngine) {

    this.validaIdCiclo = function(idciclo, nrorg) {
        return restEngine.post("coc#/crud/ciclo", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "IDCICLO",
                    "operator": "=",
                    "value": idciclo
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }
            ]
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26000_cicloService", coc26000_cicloService);
});

// FILE: ./assets/js/services/coc26001_importacaoCustoService.js
var coc26001_importacaoCustoService = function(InflightorGeneral, restEngine) {

    this.uploadCsvFile = function(file, idSubs) {
        var dataSet = [file, {
            ID: idSubs
        }];
        return restEngine.post("coc#/uploadCsvFile", {
            "requestType": "DataSet",
            "dataset": dataSet
        }, null, null, null, null, 2000000);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26001_importacaoCustoService", coc26001_importacaoCustoService);
});

// FILE: ./assets/js/services/coc26003_integracaoSAPService.js
var coc26003_integracaoSAPService = function(restEngine) {
    this.filtraPrefixos = function(prefixo) {
        return restEngine.post("coc#/crud/servciclo_ciclo_servico", {
            "requestType": "FilterData",
            "filter": prefixo
        });
    };

    this.retornaCodigoMaximoIntegracao = function(prefixo) {
        return restEngine.post("coc#/crud/codigo_maximo_integracao", {
            "requestType": "FilterData",
            "filter": prefixo
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26003_integracaoSAPService", coc26003_integracaoSAPService);
});

// FILE: ./assets/js/services/coc26004_exportacaoSAPService.js
var coc26004_exportacaoSAPService = function(restEngine) {

    this.exportaServico = function(row) {

        return restEngine.post("coc#/downloadExportServ", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 600000000000000);

    };

    this.exportaDelivery = function(dataset) {

        return restEngine.post("coc#/downloadExportDel", {
            "requestType": "DataSet",
            "dataset": dataset
        }, null, null, null, null, 600000000000000);

    };

    this.verificaDeliveryExp = function(filter) {
        return restEngine.post("coc#/crud/deliverym", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.verificaDeliveryReexp = function(filter) {
        return restEngine.post("coc#/crud/deliverym", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26004_exportacaoSAPService", coc26004_exportacaoSAPService);
});

// FILE: ./assets/js/services/coc26004_importacaoProporcaoVariavelService.js
var coc26004_importacaoProporcaoVariavelService = function(InflightorGeneral, restEngine) {

    this.uploadCsvFile = function(file, idSubs, data) {
        var dataSet = [file, {
            ID: idSubs
        }, {
            DATA: data
        }];
        return restEngine.post("coc#/uploadCsvFile2", {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26004_importacaoProporcaoVariavelService", coc26004_importacaoProporcaoVariavelService);
});

// FILE: ./assets/js/services/ManutencaoCustoProdutoService.js
var ManutencaoCustoProdutoService = function(InflightorGeneral, restEngine) {

    this.salvaManutencao = function(dataset) {
        return restEngine.post("coc#/manutencaocustoproduto/salvamanutencaoproduto", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("ManutencaoCustoProdutoService", ManutencaoCustoProdutoService);
});

// FILE: ./assets/js/services/OperWorkflowService.js
var OperWorkflowService = function(restEngine) {

    this.getOperProfile = function(cdoperador) {

        cdoperador = cdoperador ? cdoperador : "OPERADOR_LOGADO";

        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": cdoperador
            }
        ];

        restEngine.post("coc#/crud/operitempaprova", {
            "requestType": "FilterData",
            "filter": filter
        }).then(function(callback) {
            if (callback.dataset.operitempaprova && callback.dataset.operitempaprova.length) {
                setLocalVar("OPERPROFILEWFLOW", callback.dataset.operitempaprova[0].IDTIPOACAOOPER);
            }
        });
    };

    this.processAprov = function(row) {

        return restEngine.post("coc#/processworkflow/aprov", {
            "requestType": "Row",
            "row": row
        });
    };

    this.processAprovWF = function(row) {

        return restEngine.post("coc#/processworkflow/processAprova", {
            "requestType": "Row",
            "row": row
        });
    };

    this.processAprovWFUnidade = function(row) {
        return restEngine.post("coc#/processworkflow/processAprovaUnidades", {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("OperWorkflowService", OperWorkflowService);
});

// FILE: ./assets/js/services/ProdutociaService.js
var ProdutociaService = function(InflightorGeneral, restEngine) {

    this.saveProdCia = function(dataset) {
        return restEngine.post("coc#/crud/produtocia_produto/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.deleteProdCia = function(dataset) {
        return restEngine.post("coc#/crud/produtocia_produto/remove", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("ProdutociaService", ProdutociaService);
});

// FILE: ./assets/js/services/TabelaPrecosService.js
var TabelaPrecosService = function(InflightorGeneral, restEngine) {

    this.findFilial = function() {

        return restEngine.post("coc#/crud/unidades", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_CDFILIAL")
                }
            ]
        });
    };
    this.getDadoswFlow = function(codigo, nometabela) {
        return restEngine.post("coc#/workflow_dadoswflow", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "DSPRIMARYKEY",
                    "operator": "LIKE",
                    "value": "%" + codigo + "%"
                },
                {
                    "name": "NMTABELAWFLOW",
                    "operator": "=",
                    "value": nometabela
                }
            ]
        });
    };

    this.getProcessosWorkflow = function(processo, item) {
        return restEngine.post("coc#/crud/processos_workflow_ativos", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRPROCESSOAPROVA",
                    "operator": "=",
                    "value": processo
                },
                {
                    "name": "NRITEMPROCAPROVA",
                    "operator": "=",
                    "value": item
                }
            ]
        });
    };

    this.saveTabelaPreco = function(row) {
        var data = [];
        data.push(row);
        return restEngine.post("coc#/tabprecose/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.saveServTab = function(row) {
        var data = [];
        data.push(row);
        return restEngine.post("coc#/precoseciam/save", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.aumentaPorcent = function(data) {
        return restEngine.post("coc#/precoseciam/porcent", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.uploadCsvFile = function(file, idSubs, currentRow) {
        var dataSet = [file, {
            ID: idSubs
        }, currentRow];
        return restEngine.post("coc#/uploadCsvItemFat", {
            "requestType": "DataSet",
            "dataset": dataSet
        }, null, null, null, null, 1800000);
    };

    this.savePrecPromo = function(currentRow) {
        return restEngine.post("coc#/precoseciam/savePrecPromo", {
            "requestType": "Row",
            "row": currentRow
        });
    };
    this.removePropVariavel = function(row) {

        var data = [{
            CDFILIAL: row.CDFILIAL,
            CDCIA: row.CDCIA,
            CDMANCIA: row.CDMANCIA,
            NRSERVICOCIAFILI: row.NRSERVICOCIAFILI,
            CDCLASSE: row.CDCLASSE,
            CDCICLO: row.CDCICLO,
            CDSUBGRU: row.CDSUBGRU,
            QTPAXDE: row.QTPAXDE,
            QTPAXATE: row.QTPAXATE,
            NRSEQPRECO: row.NRSEQPRECO,
            NRTABPRECO: row.NRTABPRECO,
            NRORG: row.NRORG,
            __is_new: false
        }];

        return restEngine.post("coc#/crud/proporcaopreco/remove", {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.getItensFaturamento = function(cdfilial, cdcia, nrtabpreco) {
        return restEngine.post("coc#/crud/precoseciam_servicociafili_servicogen_classe_ciclo", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdcia
                },
                {
                    "name": "NRTABPRECO",
                    "operator": "=",
                    "value": nrtabpreco
                }
            ]
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("TabelaPrecosService", TabelaPrecosService);
});

// FILE: ./assets/js/services/WorkflowService.js
var WorkflowService = function(restEngine, ZHPromise) {

    this.getNrdiasoperaproDefault = function(nrprocessoaprova, nritemprocaprova) {
        return restEngine.post("coc#/crud/itemprocaprova", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRPROCESSOAPROVA",
                    "operator": "=",
                    "value": nrprocessoaprova
                },
                {
                    "name": "NRITEMPROCAPROVA",
                    "operator": "=",
                    "value": nritemprocaprova
                }
            ]
        });
    };

    this.vinculaOperador = function(dataset) {
        return restEngine.post("coc#/crud/operitempaprova/save", {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.desvinculaOperador = function(rows) {
        return restEngine.post("coc#/crud/operitempaprova/remove", {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("WorkflowService", WorkflowService);
});

// FILE: ./assets/js/controllers/coc00001_lean_codeController.js
var coc00001_lean_codeController = function(InflightorGeneral) {

    var self = this;

    this.blockField = function(widget) {
        var fieldCode = widget.getField("CDLEANCODE");
        fieldCode.readOnly = true;
        delete fieldCode.validations.required;
    };

    this.freeField = function(widget) {
        var fieldCode = widget.getField("CDLEANCODE");
        fieldCode.readOnly = false;
        fieldCode.validations.required = true;
    };

    this.addLeanCode = function(args) {
        const widget = args.owner.widget;
        widget.newRow();
        self.freeField(widget);
    };

    this.editLeanCode = function(args) {
        const widget = args.owner.widget;
        InflightorGeneral.edit(widget);
        self.blockField(widget);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00001_lean_codeController", coc00001_lean_codeController);
});

// FILE: ./assets/js/controllers/coc00002_produto_companhia_controller.js
var coc00002_produto_companhia_controller = function(ScreenService, InflightorGeneral, ProdutociaService, restEngine, templateManager, MessageService) {

    var cdnvproduto;

    function getMainWidget() {
        return templateManager.container.getWidget('produtocompanhia');
    }

    this.filtraProdutosJaUtilizados = function(field) {

        var filtroProdutoAp = field.widget.container.getWidget("produtocompanhia").dataSourceFilter;
        var pilhaProdutos = [];

        filtroProdutoAp.forEach(function(condicao) {
            if (condicao.name === "NRORG") {
                condicao.value = getLocalVar("INFLIGHTOR_NRORGTRAB");
            }
        });
        var filterFieldProduto = [
            {
                name: "IDPRODATIVPR",
                operator: "=",
                value: "S"
            }
        ];
        restEngine.post("coc#/crud/produtocia_produto", {
            "requestType": "FilterData",
            "filter": filtroProdutoAp
        }).then(function(sucess) {
            if (sucess.dataset.produtocia_produto.length) {
                sucess.dataset.produtocia_produto.forEach(function(condicaoprodutocia) {
                    pilhaProdutos.push(condicaoprodutocia.CDPRODUTO);
                });
                filterFieldProduto.push({
                    name: "CDPRODUTO",
                    operator: "NOT_IN",
                    value: pilhaProdutos
                });
            }
            field.dataSourceFilter = filterFieldProduto;
            field.reload();

        });

    };

    this.uniToCia = function(field) {
        if (field.widget.currentRow.NMFILIAL != "")
            field.dataSourceFilter = [];
        var filtro = [{
            "name": "NRORG",
            "operator": "=",
            "value": field.widget.currentRow.NRORG
            }, {
            "name": "CDFILIAL",
            "operator": "=",
            "value": field.widget.currentRow.CDFILIAL
            }];
        field.dataSourceFilter = filtro;
        field.reload();
    };

    this.ProdutoPopup = function(widget) {
        var adicionaprodutocompanhia = widget.container.getWidget("adicionaprodutocompanhia");
        ScreenService.openPopup(adicionaprodutocompanhia);
        adicionaprodutocompanhia.getField("NMPRODUTO").clearValue();
    };

    this.limpaDatasourceData = function(widget) {
        var filtroFilialCia = filtroPreenchido(widget.dataSourceFilter);
        if (!filtroFilialCia.CDFILIAL && !filtroFilialCia.CDCIA) {
            widget.dataSource.data = [];
        }
    };

    var filtroPreenchido = function(dataSourceFilter) {
        var filialCompanhia = {
            CDFILIAL: false,
            CDCIA: false
        };
        dataSourceFilter.forEach(function(parametro) {
            if (parametro.name === "CDFILIAL") {
                filialCompanhia.CDFILIAL = temParametro(parametro, "name", "CDFILIAL");
            }
            if (parametro.name === "CDCIA") {
                filialCompanhia.CDCIA = temParametro(parametro, "name", "CDCIA");
            }
        });
        return filialCompanhia;
    };

    var temParametro = function(parametro, propriedade, valor) {
        return parametro.hasOwnProperty(propriedade) && parametro[propriedade] === valor;
    };

    this.salvaProdutoCia = function(widget) {
        if (!widget.isValid()) return;
        var produtos = widget.getField("NMPRODUTO").dataSource.checkedRows;

        if (produtos.length > 0) {

            var cdcia;
            var cdfilial;
            var rows = [];
            var parametrosDoFiltro = widget.container.getWidget("produtocompanhia").dataSourceFilter;

            parametrosDoFiltro.forEach(function(parametro) {
                if (parametro.name == "CDFILIAL") {
                    cdfilial = parametro.value;
                }
                if (parametro.name == "CDCIA") {
                    cdcia = parametro.value;
                }
            });

            produtos.forEach(function(produto) {
                rows.push({
                    CDPRODUTO: produto.CDPRODUTO,
                    CDFILIAL: cdfilial,
                    CDCIA: cdcia,
                    __is_new: true
                });
            });

            ProdutociaService.saveProdCia(rows).then(function() {
                widget.container.getWidget("produtocompanhia").reload();
                ScreenService.closePopup();
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });

        } else {
            MessageService.showMessageByCode("COC00002_NO_PRODUCTS_SELECTED");
        }

    };

    this.deleteProdutoCia = function(widget) {

        var produtos = widget.getCheckedRows();
        var cantRemove = [];

        produtos.forEach(function(produto) {
            cantRemove.push(produto.CDPRODUTO);
            produto.__is_new = false;
        });

        InflightorGeneral.callRestEngine("coc#/crud/subprodcia", "FilterData", [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": produtos[0].CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": produtos[0].CDCIA
            },
            {
                "name": "CDPRODUTO",
                "operator": "IN",
                "value": cantRemove
            }
        ]).then(function(callback) {
            if (callback.dataset.subprodcia.length === 0) {
                ProdutociaService.deleteProdCia(produtos).then(function(deleteCallback) {
                    ScreenService.showMessage(deleteCallback.messages[0].message);
                    widget.reload();
                    widget.unSelectAll();
                }, function(err) {
                    MessageService.showMessageByCode("COC00002_ERR_REMOVE_PRODUCT");
                });
            } else {
                MessageService.showMessageByCode("COC00002_ERR_REMOVE_PRODUCT_SUBPRODUCT");
            }

        }, function(err) {
            ScreenService.showMessage(err.data.error);
        });

    };

    this.exibeSubProdCia = function(widget) {

        cdnvproduto = widget.currentRow.CDNVPRODUTO;
        var produto = widget.currentRow;

        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": produto.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": produto.CDCIA
            },
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": produto.CDPRODUTO
            }
        ];

        widget.container.getWidget("subprodutocompanhia").dataSourceFilter = filter;
        widget.container.getWidget("subprodutocompanhia").reload();
        ScreenService.openSwipe(widget.container.getWidget("tabbedsubprodutocompanhia"));

    };

    this.preparaNovoRegistro = function(widget) {
        const widgetSubProduto = templateManager.container.getWidget("subprodutocompanhia");
        const mainWidget = getMainWidget();
        const row = mainWidget.currentRow;
        widget.dataSourceFilter.forEach(function(filtro) {
            switch (filtro.name) {
                case "CDFILIAL":
                    widget.currentRow.CDFILIAL = filtro.value;
                    break;
                case "CDCIA":
                    widget.currentRow.CDCIA = filtro.value;
                    break;
                case "CDPRODUTO":
                    widget.currentRow.CDPRODUTO = filtro.value;
                    break;
            }
        });
        const fieldProduto = widgetSubProduto.getField("NMPRODUTO");
        fieldProduto.readOnly = false;
        fieldProduto.validations.required = true;
        fieldProduto.dataSourceFilter.forEach(filter => {
            filter.value = row[filter.name];
        });
    };

    this.editaSubProduto = function(widget) {
        InflightorGeneral.edit(widget);
        var fieldSubProduto = widget.getField("NMPRODUTO");
        fieldSubProduto.readOnly = widget.mode === "edit";
        if (fieldSubProduto.readOnly) {
            delete fieldSubProduto.validations.required;
        }
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc00002_produto_companhia_controller.inactiveBranchWarning(args);";
    };

    this.applyInactiveStyleForSubGrup = function(widget) {
        widget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.onChangeUnit = function(args) {
        const fieldUnit = args.owner.field;
        const widget = args.owner.field.widget;
        const CDFILIAL = fieldUnit.value() ? widget.currentRow.CDFILIAL : "";
        const fieldCia = widget.getField('NMCIA');
        let filterUnit = fieldCia.dataSourceFilter.find(filter => filter.name === 'CDFILIAL');
        if (filterUnit) {
            filterUnit.value = CDFILIAL;
        }
        fieldCia.clearValue();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00002_produto_companhia_controller", coc00002_produto_companhia_controller);
});

// FILE: ./assets/js/controllers/coc00004_workflow_operador_controller.js
var coc00004_workflow_operador_controller = function(ScreenService, InflightorGeneral, OperWorkflowService, MessageService, templateManager) {
    var self = this;
    let trataMensagens = function(message) {
        if (message && message.length) {
            if (message.indexOf('COC00004_') >= 0) {
                MessageService.showMessageByCode(message);
            } else {
                ScreenService.showMessage(message);
            }
        }
    };

    var processosItens = [
        "PROCESSOAPROVA_1",
        "PROCESSOAPROVA_2",
        "PROCESSOAPROVA_3",
        "PROCESSOAPROVA_4",
        "ITEMPROCAPROVA_1_1",
        "ITEMPROCAPROVA_1_2",
        "ITEMPROCAPROVA_1_3",
        "ITEMPROCAPROVA_1_4",
        "ITEMPROCAPROVA_1_5",
        "ITEMPROCAPROVA_1_6",
        "ITEMPROCAPROVA_1_7",
        "ITEMPROCAPROVA_1_8",
        "ITEMPROCAPROVA_1_9",
        "ITEMPROCAPROVA_1_10",
        "ITEMPROCAPROVA_1_11",
        "ITEMPROCAPROVA_1_12",
        "ITEMPROCAPROVA_1_13",
        "ITEMPROCAPROVA_1_14",
        "ITEMPROCAPROVA_1_15",
        "ITEMPROCAPROVA_1_16",
        "ITEMPROCAPROVA_1_17",
        "ITEMPROCAPROVA_1_18",
        "ITEMPROCAPROVA_1_19",
        "ITEMPROCAPROVA_1_21",
        "ITEMPROCAPROVA_1_22",
        "ITEMPROCAPROVA_1_23",
        "ITEMPROCAPROVA_1_24",
        "ITEMPROCAPROVA_1_25",
        "ITEMPROCAPROVA_1_26",
        "ITEMPROCAPROVA_1_27",
        "ITEMPROCAPROVA_1_28",
        "ITEMPROCAPROVA_1_29",
        "ITEMPROCAPROVA_1_30",
        "ITEMPROCAPROVA_1_31",
        "ITEMPROCAPROVA_1_32",
        "ITEMPROCAPROVA_1_33",
        "ITEMPROCAPROVA_1_34",
        "ITEMPROCAPROVA_2_1",
        "ITEMPROCAPROVA_2_2",
        "ITEMPROCAPROVA_2_3",
        "ITEMPROCAPROVA_3_1",
        "ITEMPROCAPROVA_3_2",
        "ITEMPROCAPROVA_3_3",
        "ITEMPROCAPROVA_3_4",
        "ITEMPROCAPROVA_3_5",
        "ITEMPROCAPROVA_3_6",
        "ITEMPROCAPROVA_4_1",
        "ITEMPROCAPROVA_4_2",
        "ITEMPROCAPROVA_4_3",
        "ITEMPROCAPROVA_4_4",
        "ITEMPROCAPROVA_4_5",
        "ITEMPROCAPROVA_4_6",
        "ITEMPROCAPROVA_4_7",
        "ITEMPROCAPROVA_4_8",
        "ITEMPROCAPROVA_4_9",
        "ITEMPROCAPROVA_4_10",
        "ITEMPROCAPROVA_4_11",
        "ITEMPROCAPROVA_4_12",
        "ITEMPROCAPROVA_4_13",
        "ITEMPROCAPROVA_4_14",
        "ITEMPROCAPROVA_4_15",
        "ITEMPROCAPROVA_4_16",
        "ITEMPROCAPROVA_4_17",
        "ITEMPROCAPROVA_4_18",
        "ITEMPROCAPROVA_4_19",
        "ITEMPROCAPROVA_4_20",
        "ITEMPROCAPROVA_4_21",
        "ITEMPROCAPROVA_4_22",
        "ITEMPROCAPROVA_4_23",
        "ITEMPROCAPROVA_4_24",
        "ITEMPROCAPROVA_4_25",
        "ITEMPROCAPROVA_4_26",
        "ITEMPROCAPROVA_4_27"
    ];

    var processosItensTrad;
    var situacao;

    this.onEnterProfileOper = function() {
        OperWorkflowService.getOperProfile();
    };

    this.preparaDataSource = function(widget) {
        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length) {

            var setMensagens = function() {
                for (let i = 0; i < widget.dataSource.data.length; i++) {
                    let idx = processosItens.indexOf(widget.dataSource.data[i].NMPROCESSOAPROVA);
                    widget.dataSource.data[i].NMPROCESSOAPROVA = (idx >= 0) ? processosItensTrad[idx] : widget.dataSource.data[i].NMPROCESSOAPROVA;
                    idx = processosItens.indexOf(widget.dataSource.data[i].NMITEMPROCAPROVA);
                    widget.dataSource.data[i].NMITEMPROCAPROVA = (idx >= 0) ? processosItensTrad[idx] : widget.dataSource.data[i].NMITEMPROCAPROVA;

                    widget.dataSource.data[i].DSSITWORKFLOW = situacao[widget.dataSource.data[i].IDSITWORKFLOW];
                    widget.dataSource.data[i].DSINFORANTERIOR = widget.dataSource.data[i].DSINFORANTERIOR ? widget.dataSource.data[i].DSINFORANTERIOR.replace(/\\par[d]?/g, "").replace(/\{\*?\\[^{}]+}|[{}]|\\\n?[A-Za-z]+\n?(?:-?\d+)?[ ]?/g, "") : widget.dataSource.data[i].DSINFORANTERIOR;
                }
            };

            if (situacao && processosItensTrad) {
                setMensagens();
            } else {
                if (!situacao) {
                    MessageService.getMessagesByCode(['COC0004_PENDENTE', 'COC0004_N_APROVADO', 'COC0004_APROVADO']).then(function(situacoes) {
                        situacao = {
                            "1": situacoes[0],
                            "2": situacoes[1],
                            "3": situacoes[2]
                        };
                        if (!processosItensTrad) {
                            processosItensTrad = angular.copy(processosItens);
                            MessageService.getMessagesByCode(processosItensTrad).then(function() {
                                setMensagens();
                            });
                        } else {
                            setMensagens();
                        }
                    });
                } else {
                    if (!processosItensTrad) {
                        processosItensTrad = angular.copy(processosItens);
                        MessageService.getMessagesByCode(processosItensTrad).then(function() {
                            setMensagens();
                        });
                    } else {
                        setMensagens();
                    }
                }
            }
        }
    };

    this.changeFormAndLabelsActions = function(widget) {
        MessageService.getMessagesByCode([
            "COC00004_RECE_ALT",
            "COC00004_RECE_INC",
            "COC00004_RECE_REMO",
            "COC00004_PROD_ALT",
            "COC00004_PROD_INC",
            "COC00004_PROD_DEL",
            "COC00004_TABPRECO_ALT",
            "COC00004_TABPRECO_INC",
            "COC00004_TABPRECO_REMO",
            "COC00004_ITEM_ALT",
            "COC00004_ITEM_INC",
            "COC00004_ITEM_REMO",
            "COC00004_VARIACAO_REC_ALT",
            "COC00004_VARIACAO_REC_INC",
            "COC00004_VARIACAO_REC_REMO"
            ]).then(function(mensagens) {
            var tipoOper = {
                "1": {
                    "1": mensagens[0],
                    "2": mensagens[1],
                    "3": mensagens[2]
                },
                "2": {
                    "1": mensagens[3],
                    "2": mensagens[4],
                    "3": mensagens[5]
                },
                "3": {
                    "1": mensagens[6],
                    "2": mensagens[7],
                    "3": mensagens[8]
                },
                "4": {
                    "1": mensagens[9],
                    "2": mensagens[10],
                    "3": mensagens[11]
                },
                "5": {
                    "1": mensagens[12],
                    "2": mensagens[13],
                    "3": mensagens[14]
                }
            };

            widget.getField("NMITEMWFLOW").label = tipoOper[widget.currentRow.NRPROCESSOAPROVA][widget.currentRow.IDTIPOPERWFLOW];

            switch (widget.currentRow.IDTIPOPERWFLOW) {
                case "1":
                    widget.getField("DSINFORANTERIOR").isVisible = true;
                    widget.getField("DSINFORATUAL").isVisible = true;
                    break;
                case "2":
                    widget.getField("DSINFORANTERIOR").isVisible = false;
                    widget.getField("DSINFORATUAL").isVisible = false;
                    break;
                case "3":
                    widget.getField("DSINFORANTERIOR").isVisible = false;
                    widget.getField("DSINFORATUAL").isVisible = false;
                    break;
            }

            if (widget.currentRow.IDSITOPERWFLOW === "A" || widget.currentRow.IDSITOPERWFLOW === "N") {
                widget.getAction("APROV").isVisible = false;
                widget.getAction("REPRO").isVisible = false;
                widget.getAction("CIENT").isVisible = false;
            } else {
                self.loadOperatorProfile(widget);
            }

            if (widget.currentRow.NRPROCESSOAPROVA == 4 && widget.currentRow.NRITEMPROCAPROVA == 24) {
                widget.getAction("VIEW").isVisible = true;
            } else {
                widget.getAction("VIEW").isVisible = false;
            }
        });
    };

    this.loadOperatorProfile = function(widget) {

        if (widget.currentRow.IDSITWORKFLOW === "2" || widget.currentRow.IDSITWORKFLOW === "3") {
            widget.getAction("APROV").isVisible = false;
            widget.getAction("REPRO").isVisible = false;
            widget.getAction("CIENT").isVisible = false;
        } else {
            switch (widget.currentRow.IDTIPOACAOOPER) {
                case "A":
                    widget.getAction("APROV").isVisible = true;
                    widget.getAction("REPRO").isVisible = true;
                    widget.getAction("CIENT").isVisible = false;
                    widget.operProfile = "A";
                    break;
                case "C":
                    widget.getAction("APROV").isVisible = false;
                    widget.getAction("REPRO").isVisible = false;
                    widget.getAction("CIENT").isVisible = true;
                    widget.operProfile = "C";
                    break;
                default:
                    widget.getAction("APROV").isVisible = false;
                    widget.getAction("REPRO").isVisible = false;
                    widget.getAction("CIENT").isVisible = false;
                    widget.operProfile = null;
            }
        }

    };

    this.loadOperatorInvolved = function(widget) {

        var filter = [
            {
                "name": "NRWORKFLOW",
                "operator": "=",
                "value": widget.currentRow.NRWORKFLOW
            }
        ];

        widget.container.getWidget("operadoresEnvolvidos").dataSourceFilter = filter;
        widget.container.getWidget("operadoresEnvolvidos").reload();

    };

    this.aprovaWorkflowProcess = function(widget, situacao) {

        let widgetMotivo = templateManager.container.getWidget("motivoWidget");

        ScreenService.openPopup(widgetMotivo);

        widgetMotivo.currentRow.IDSITOPERWFLOW = situacao;

    };

    this.aprovaWorkflow = function(widget) {
        if (widget.isValid()) {
            var row;
            if (widget.operProfile === "C") {
                row = {
                    "NRWORKFLOW": widget.container.getWidget("workflowoperador").currentRow.NRWORKFLOW,
                    "NMITEMWFLOW": widget.container.getWidget("workflowoperador").currentRow.NMITEMWFLOW,
                    "IDSITOPERWFLOW": widget.operProfile,
                    "DSMOTIVOACAO": "Operador ciente",
                    "IDTIPOPERWFLOW": widget.container.getWidget("workflowoperador").currentRow.IDTIPOPERWFLOW,
                    "NRPROCESSOAPROVA": widget.container.getWidget("workflowoperador").currentRow.NRPROCESSOAPROVA,
                    "DSMOTIVOWFLOW": widget.container.getWidget("workflowoperador").currentRow.DSMOTIVOWFLOW,
                    "NRITEMPROCAPROVA": widget.container.getWidget("workflowoperador").currentRow.NRITEMPROCAPROVA,
                    "CDOPERADOR": getLocalVar("INFLIGHTOR_CDOPERADOR"),
                    "__is_new": false,
                    "changed": true
                };

            } else {

                row = {
                    "NRWORKFLOW": widget.container.getWidget("workflowoperador").currentRow.NRWORKFLOW,
                    "NMITEMWFLOW": widget.container.getWidget("workflowoperador").currentRow.NMITEMWFLOW,
                    "IDSITOPERWFLOW": widget.currentRow.IDSITOPERWFLOW,
                    "DSMOTIVOACAO": widget.currentRow.DSMOTIVOACAO,
                    "IDTIPOPERWFLOW": widget.container.getWidget("workflowoperador").currentRow.IDTIPOPERWFLOW,
                    "NRPROCESSOAPROVA": widget.container.getWidget("workflowoperador").currentRow.NRPROCESSOAPROVA,
                    "DSMOTIVOWFLOW": widget.container.getWidget("workflowoperador").currentRow.DSMOTIVOWFLOW,
                    "NRITEMPROCAPROVA": widget.container.getWidget("workflowoperador").currentRow.NRITEMPROCAPROVA,
                    "CDOPERADOR": getLocalVar("INFLIGHTOR_CDOPERADOR"),
                    "__is_new": false,
                    "changed": true
                };

            }
            if (row.NRPROCESSOAPROVA == "1" || row.NRPROCESSOAPROVA == "2") {

                if (row.NRPROCESSOAPROVA == 1 && row.NRITEMPROCAPROVA == 17 || row.NRPROCESSOAPROVA == 1 && row.NRITEMPROCAPROVA == 16) {

                    OperWorkflowService.processAprov(row).then(
                        function(callback) {
                            if (callback.dataset && callback.dataset.errorMessage) {
                                InflightorGeneral.showErrorMessages(widget, callback.dataset.errorMessage);
                            } else if (callback.dataset && callback.dataset.ListaErros) {
                                InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros);
                            } else {
                                trataMensagens(callback.messages[0].message);
                                ScreenService.closePopup();
                                ScreenService.closeSwipe();
                                widget.container.getWidget("workflowoperador").reload();
                            }
                        },
                        function(error) {
                            if (error.data && error.data.error) {
                                if (!InflightorGeneral.mensagemErroProcedure(error.data.error))
                                    trataMensagens(error.data.error);
                            } else {
                                MessageService.showMessageByCode('ERRO_INESPERADO');
                                console.log(error);
                            }
                        }
                    );

                } else if (row.NRPROCESSOAPROVA == 1 && row.NRITEMPROCAPROVA == 34) {
                    OperWorkflowService.processAprovWFUnidade(row).then(
                        function(callback) {
                            trataMensagens(callback.messages[0].message);
                            ScreenService.closePopup();
                            ScreenService.closeSwipe();
                            widget.container.getWidget("workflowoperador").reload();
                        },
                        function(error) {
                            if (error.data && error.data.error) {
                                if (!InflightorGeneral.mensagemErroProcedure(error.data.error))
                                    trataMensagens(error.data.error);
                            } else {
                                MessageService.showMessageByCode('ERRO_INESPERADO');
                                console.log(error);
                            }
                        }
                    );
                } else {
                    OperWorkflowService.processAprovWF(row).then(
                        function(callback) {
                            trataMensagens(callback.messages[0].message);
                            ScreenService.closePopup();
                            ScreenService.closeSwipe();
                            widget.container.getWidget("workflowoperador").reload();
                        },
                        function(error) {
                            if (error.data && error.data.error) {
                                trataMensagens(error.data.error);
                            } else {
                                MessageService.showMessageByCode('ERRO_INESPERADO');
                                console.log(error);
                            }
                        }
                    );
                }

            } else {

                OperWorkflowService.processAprov(row).then(
                    function(callback) {
                        if (callback.messages) {
                            trataMensagens(callback.messages[0].message);
                            ScreenService.closePopup();
                            ScreenService.closeSwipe();
                            widget.container.getWidget("workflowoperador").reload();
                        } else if (callback.dataset && callback.dataset.errorMessage) {
                            InflightorGeneral.showErrorMessages(widget, callback.dataset.errorMessage);
                        } else if (callback.dataset && callback.dataset.ListaErros) {
                            InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros);
                        }
                    },
                    function(error) {
                        if (error.data && error.data.error) {
                            trataMensagens(error.data.error);
                        } else {
                            MessageService.showMessageByCode('ERRO_INESPERADO');
                            console.log(error);
                        }
                    }
                );
            }
        }
    };

    this.workflowViewer = function(widget) {
        var params = {
            "NRPROCESSOAPROVA": widget.currentRow.NRPROCESSOAPROVA,
            "NRWORKFLOW": widget.currentRow.NRWORKFLOW
        };
        InflightorGeneral.openReport("coc#/viewworkflow", null, params);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00004_workflow_operador_controller", coc00004_workflow_operador_controller);
});

// FILE: ./assets/js/controllers/coc00006_voo_menu_designController.js
function coc00006_voo_menu_designController(ScreenService, InflightorGeneral) {

    var self = this;
    var cdfilial, cdcia, nmfilial, nmcia;

    this.setDefaultFilial = function(widgetVoo) {
        widgetVoo.getField("CDFILIAL").applyDefaultValue();
        widgetVoo.getField("NMFILIAL").applyDefaultValue();
        self.filtraCiaFilter(widgetVoo.__filterPopup);
    };

    this.gridAfterMoveRow = function(widgetVoo) {
        widgetVoo.currentRow.NMFILIAL = nmfilial;
        widgetVoo.currentRow.NMCIA = nmcia;

        var widgetServicos = widgetVoo.container.getWidget("widgetServicos");

        widgetServicos.isVisible = !!widgetVoo.currentRow.CDMANCIA;

        widgetServicos.dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: cdfilial
            },
            {
                name: "CDCIA",
                operator: "=",
                value: cdcia
            },
            {
                name: "NRVOO",
                operator: "=",
                value: widgetVoo.currentRow.NRVOO
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: widgetVoo.currentRow.CDMANCIA
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];

        if (widgetServicos.isVisible) {
            widgetServicos.reload();
        }
    };

    this.filtraCiaFilter = function(widgetVoo) {
        var fieldCia = widgetVoo.getField("NMCIA");
        if (widgetVoo.currentRow.CDFILIAL) {
            cdfilial = widgetVoo.currentRow.CDFILIAL;
            nmfilial = widgetVoo.currentRow.NMFILIAL;
            fieldCia.readOnly = false;
            fieldCia.dataSourceFilter = [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: cdfilial
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ];
            fieldCia.reload();
        } else {
            fieldCia.setValue("");
            fieldCia.readOnly = true;
        }
    };

    this.FilterBeforeApply = function(widgetVoo) {
        cdfilial = widgetVoo.currenRow.CDFILIAL;
        cdcia = widgetVoo.currenRow.CDCIA;

        var popupManual = widgetVoo.container.getWidget("popupManual");
        var fieldManual = widgetVoo.getField("DSMANUAL");
        var dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: cdfilial
            },
            {
                name: "CDCIA",
                operator: "=",
                value: cdcia
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        popupManual.dataSourceFilter = dataSourceFilter;
        popupManual.reload();
        fieldManual.dataSourceFilter = dataSourceFilter;
        fieldManual.reload();
    };

    this.filtraFilhosCia = function(widgetVoo) {
        var fieldsToEdit = [
            widgetVoo.getField("DSMANUAL"),
            widgetVoo.getField("CDCLASSE"),
            widgetVoo.getField("CDCICLO")
        ];
        if (widgetVoo.currentRow.CDCIA) {
            cdcia = widgetVoo.currentRow.CDCIA;
            nmcia = widgetVoo.currentRow.NMCIA;
            var dataSourceFilter = [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetVoo.currentRow.CDFILIAL
                },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: widgetVoo.currentRow.CDCIA
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ];
            fieldsToEdit.forEach(function(field) {
                field.readOnly = false;
                field.dataSourceFilter = dataSourceFilter;
                field.reload();
            });
        } else {
            fieldsToEdit.forEach(function(field) {
                field.setValue("");
                field.readOnly = true;
            });
        }
    };

    this.chamaPopupManual = function(widgetVoo) {
        var popupManual = widgetVoo.container.getWidget("popupManual");
        ScreenService.openPopup(popupManual);
        popupManual.isVisible = true;
        popupManual.activate();
    };

    this.cancelaPopupManual = function(popupManual) {
        ScreenService.closePopup();
        popupManual.isVisible = false;
        popupManual.parent.activate();
    };

    this.savePopupManual = function(popupManual) {
        var msg = "Deseja associar o manual " + popupManual.currentRow.DSMANUAL + " ao(s) voo(s) selecionado(s)?";
        ScreenService.confirmMessage(msg, "question", function() {
            var dataSetToSave = [];
            popupManual.parent.getCheckedRows().forEach(function(voo) {
                voo.__is_new = voo.CDMANCIA ? false : true;
                voo.CDMANCIA = popupManual.currentRow.CDMANCIA;
                dataSetToSave.push(voo);
            });
            InflightorGeneral.callRestEngine("coc#/crud/conexvoomanual/save", "DataSet", dataSetToSave).then(function() {
                popupManual.parent.reload().then(function() {
                    ScreenService.closePopup();
                    popupManual.isVisible = false;
                    popupManual.parent.activate();
                }, function() {
                    ScreenService.closePopup();
                    popupManual.isVisible = false;
                    popupManual.parent.activate();
                    popupManual.parent.reload();
                });
            }, function(erro) {
                var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao associar o manual.";
                ScreenService.showMessage(msg);
            });
        });
    };

    this.chamaPopupServicos = function(widgetServicos) {
        var popupServicos = widgetServicos.container.getWidget("popupServicos");
        popupServicos.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetServicos.parent.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetServicos.parent.currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widgetServicos.parent.currentRow.CDMANCIA
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": widgetServicos.parent.currentRow.NRVOO
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        popupServicos.reload().then(function() {
            ScreenService.openPopup(popupServicos);
            popupServicos.isVisible = true;
            popupServicos.activate();
        });
    };

    this.cancelaPopupServicos = function(popupServicos) {
        ScreenService.closePopup();
        popupServicos.isVisible = false;
        popupServicos.parent.activate();
    };

    this.savePopupServicos = function(popupServicos) {
        var dataSetToSave = [];
        popupServicos.getCheckedRows().forEach(function(servico) {
            servico.__is_new = true;
            dataSetToSave.push(servico);
        });
        InflightorGeneral.callRestEngine("coc#/crud/servconexvooma/save", "DataSet", dataSetToSave).then(function() {
            popupServicos.parent.reload().then(function() {
                ScreenService.closePopup();
                popupServicos.isVisible = false;
                popupServicos.parent.activate();
            }, function() {
                ScreenService.closePopup();
                popupServicos.isVisible = false;
                popupServicos.parent.activate();
                popupServicos.parent.reload();
            });
        }, function(erro) {
            var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao associar o(s) serviço(s).";
            ScreenService.showMessage(msg);
        });
    };

    this.edit = function(widgetVoo) {
        InflightorGeneral.edit(widgetVoo);
        widgetVoo.currentRow.__is_new = widgetVoo.currentRow.CDMANCIA ? false : true;
        widgetVoo.widgets[0].isVisible = false;
    };

    this.saveEdit = function(widgetVoo) {
        InflightorGeneral.saveCrud(widgetVoo).then(function() {
            widgetVoo.widgets[0].isVisible = true;
        });
    };

    this.cancelEdit = function(args) {
        var widgetVoo = args.owner.widget;
        InflightorGeneral.backOrCancel(args);
        widgetVoo.widgets[0].isVisible = true;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00006_voo_menu_designController", coc00006_voo_menu_designController);
});

// FILE: ./assets/js/controllers/coc00007_cadastro_de_chef_controller.js
var coc00007_cadastro_de_chef_controller = function(InflightorGeneral, MessageService) {

    this.ativaChefe = function(widget) {
        alterarChef(widget, "S");
    };

    this.desativaChefe = function(widget) {
        alterarChef(widget, "N");
    };

    function alterarChef(widget, IDCHEFERECEITAM) {
        widget.getCheckedRows().forEach(function(row) {
            row.IDCHEFERECEITAM = IDCHEFERECEITAM;
            row.CDOPERADOR_TO_UPDATE = row.CDOPERADOR;
        });
        InflightorGeneral.callRestEngine("coc#/operadorchef/save", "DataSet", widget.getCheckedRows()).then(function() {
            MessageService.showMessageByCode("MSG_CHANGE_SUCCESS");
        }, InflightorGeneral.callBackError);
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00007_cadastro_de_chef_controller", coc00007_cadastro_de_chef_controller);
});

// FILE: ./assets/js/controllers/coc00007_configuracoes_do_operador_controller.js
var coc00007_configuracoes_do_operador_controller = function(InflightorGeneral) {
    this.ativaChefe = function(widget) {
        widget.getCheckedRows().forEach(function(row) {
            row.IDCHEFERECEITAM = "S";
        });
        InflightorGeneral.callRestEngine("coc#/crud/operador/save", "DataSet", widget.getCheckedRows());
    };

    this.desativaChefe = function(widget) {
        widget.getCheckedRows().forEach(function(row) {
            row.IDCHEFERECEITAM = "N";
        });
        InflightorGeneral.callRestEngine("coc#/crud/operador/save", "DataSet", widget.getCheckedRows());
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00007_configuracoes_do_operador_controller", coc00007_configuracoes_do_operador_controller);
});

// FILE: ./assets/js/controllers/Coc00022_ManutencaoMenugrid_Controller.js
var Coc00022_ManutencaoMenugrid_Controller = function(ScreenService, InflightorGeneral, MessageService, templateManager, coc03600_menu_designService, maskEngine, FieldUtil, ZHPromise, JSONContainerLoader, metaDataFactory) {

    var self = this;
    //Copia a row, evitando o loop por causa das actions presentes nas actions laterais
    function copyRow(row) {
        var newRow = {};
        var itensIgnorados = ["__leftActions", "__rightActions"];
        for (let item in row) {
            if (!itemIgnorado(item, itensIgnorados))
                newRow[item] = angular.copy(row[item]);
        }
        return newRow;
    }

    function itemIgnorado(item, itensIgnorados) {
        var itemEncontrado = itensIgnorados.indexOf(item);
        return itemEncontrado == -1 ? false : true;
    }

    function getFilterWorkflow(row) {
        return [
            {
                name: "NRPROCESSOAPROVA",
                operator: "=",
                value: "4"
            },
            {
                name: "NRITEMPROCAPROVA",
                operator: "=",
                value: "T"
            },
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: row.CDMANCIA
            },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: row.NRSERVICOCIAFILI
            },
            {
                name: "CDCLASSE",
                operator: "=",
                value: row.CDCLASSE
            },
            {
                name: "CDSUBGRU",
                operator: "=",
                value: row.CDSUBGRU
            },
            {
                name: "NRSEQSUBGRUSEMAM",
                operator: "=",
                value: row.NRSEQSUBGRUSEMAM
            }
        ];
    }

    function getFilterValidateSubgrupo(row) {
        return [
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: row.CDMANCIA
            },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: row.NRSERVICOCIAFILI
            },
            {
                name: "CDCLASSE",
                operator: "=",
                value: row.CDCLASSE
            },
            {
                name: "CDSUBGRU",
                operator: "=",
                value: row.CDSUBGRU
            },
            {
                name: "NRSEQSUBGRUSEMAM",
                operator: "=",
                value: row.NRSEQSUBGRUSEMAM
            },
            {
                name: "IDATIVO",
                operator: "=",
                value: "N"
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
    }

    //Verifica se ja exite o CDSUBGRUS pendente de aprovação na inserção do menugrid
    function verifyHasWorkflow(row, widget) {
        var filter = getFilterWorkflow(row);
        return InflightorGeneral.callRestEngine("coc#/crud/coc00022_dadoswf_workflow_pendente_subgrupo", "FilterData", filter).then(
            function(callback) {
                var workflowPendente = callback.dataset.coc00022_dadoswf_workflow_pendente_subgrupo[0].QTDE;
                if (workflowPendente > 0)
                    return workflowPendente;
                else {
                    //Valida se existem pendencias para o Subgrupo selecionado 
                    //pelos campos de CDSUBGRU e NRSEQSUBGRUSEMAM
                    if (row.NRSEQSUBGRUSEMAM == "T")
                        if (widget.currentRow.NRSEQSUBGRUSEMAM && widget.currentRow.NRSEQSUBGRUSEMAM != "T")
                            row.NRSEQSUBGRUSEMAM = widget.currentRow.NRSEQSUBGRUSEMAM;
                    return InflightorGeneral.callRestEngine("coc#/crud/coc00022_count_subgrupo_bloqueado", "FilterData", getFilterValidateSubgrupo(row)).then(
                        function(callback) {
                            if (callback.dataset && callback.dataset.coc00022_count_subgrupo_bloqueado.length && callback.dataset.coc00022_count_subgrupo_bloqueado[0].QTDE)
                                return callback.dataset.coc00022_count_subgrupo_bloqueado[0].QTDE;
                            else
                                return 0;
                        }
                    );
                }
            }
        );
    }

    //verifica se existe algum processo de workflowPendente e executa as determinadas funções
    this.executeFunction = function(widget, callbackSuccess, paramsSuccess, callbackError, paramsError) {
        this.mountCurrentRowWithFilterData(widget.currentRow, getFilterRow(widget));

        var currentRow;

        if (widget.name == "grid_composicao") {
            currentRow = widget.getParent().currentRow;
        } else {
            currentRow = widget.currentRow;
        }

        verifyHasWorkflow(currentRow, widget).then(
            function(qtde) {
                if (qtde == 0) {
                    callbackSuccess.apply(null, paramsSuccess);
                } else {
                    if (callbackError) {
                        callbackError.apply(null, paramsError);
                    }
                    MessageService.showMessageByCode("COC00022_PENDING_APPROVAL");
                }
            }
        );
    };

    this.afterFilterApply = function(widget) {

    };

    function getFilterRow(widget) {
        let appliedFilter = widget.container.getWidget("widgetSubgrupo").appliedFilter || {};
        var filterRow = appliedFilter.FILTER_ROW || {};
        if (typeof(filterRow) === "string") {
            filterRow = JSON.parse(filterRow);
        }
        return filterRow;
    }

    this.changeField = function(field, arraySonFields, firstSonField) {
        for (var index in arraySonFields) {
            var fieldSon = field.widget.getField(arraySonFields[index]);
            fieldSon.clearValue();
            // fieldSon.reload();
            fieldSon.readOnly = true;
        }
        if (field.value()) {
            field.widget.getField(firstSonField).readOnly = false;
            field.widget.getField(firstSonField).reload();
        } else {
            field.widget.getField(firstSonField).readOnly = true;
        }
    };

    this.mountCurrentRowWithFilterData = function(row, filterRow) {
        row.CDFILIAL = filterRow.CDFILIAL;
        row.CDCIA = filterRow.CDCIA;
        row.CDMANCIA = filterRow.CDMANCIA;
        row.NRSERVICOCIAFILI = filterRow.NRSERVICOCIAFILI;
        row.CDCLASSE = filterRow.CDCLASSE;
    };

    function validDate(widget) {
        if (widget.currentRow.DTINIVGSUMA && widget.currentRow.DTFIMVGSUMA) {
            var filterRow = getFilterRow(widget);
            var dtIniManual = moment(filterRow.DTINIMANUAL.substr(0, 10), "DD/MM/YYYY");
            var dtFimManual = moment(filterRow.DTFIMMANUAL.substr(0, 10), "DD/MM/YYYY");

            var dtIniSubgrupo = moment(widget.currentRow.DTINIVGSUMA.substr(0, 10), "DD/MM/YYYY");
            var dtFimSubgrupo = moment(widget.currentRow.DTFIMVGSUMA.substr(0, 10), "DD/MM/YYYY");

            var validDtIniSubgrupo = (dtIniSubgrupo >= dtIniManual && dtIniSubgrupo <= dtFimManual);
            var validDtFimSubgrupo = (dtFimSubgrupo >= dtIniManual && dtFimSubgrupo <= dtFimManual);
            var valid = (dtIniSubgrupo <= dtFimSubgrupo);
            if (!valid) {
                MessageService.showMessageByCode("COC00022_DATE_CONTROL");
                return false;
            } else if (!validDtIniSubgrupo || !validDtFimSubgrupo) {
                var variables = {
                    dtIni: dtIniManual.format("DD/MM/YYYY"),
                    dtFim: dtFimManual.format("DD/MM/YYYY")
                };
                ScreenService.showMessage("A vigência do subgrupo extrapola a vigência do Menu Grid.<br>(__dtIni__ - __dtFim__)", "E", false, variables);
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    this.verifySaveSubgrupo = function(widget) {
        var qtdConfiguracao;

        if (widget.isValid()) {
            if (validDate(widget)) {
                this.mountCurrentRowWithFilterData(widget.currentRow, getFilterRow(widget));
                widget.currentRow.QTPROPSUBSEMAM = InflightorGeneral.valueToFloat(widget.currentRow.QTPROPSUBSEMAM);
                if (widget.currentRow.IDPROPSUBSEMAM == "2" || widget.currentRow.IDPROPSUBSEMAM == "4" || widget.currentRow.IDPROPSUBSEMAM == "5") {
                    qtdConfiguracao = parseFloat(widget.currentRow.QTPROPSUBSEMAM);
                    if (qtdConfiguracao <= 0) {
                        MessageService.showMessageByCode("COC00022_QTDE_GREATER_THAN_ZERO");
                    } else if (qtdConfiguracao > 0) {
                        InflightorGeneral.saveCrud(widget).then(
                            function() {
                                this.changeVisibleSonWidgets(widget, true);
                                widget.getField('DSSUBGRU').readOnly = false;
                            }.bind(this)
                        );
                    }
                } else {
                    InflightorGeneral.saveCrud(widget).then(
                        function() {
                            this.changeVisibleSonWidgets(widget, true);
                            widget.getField('DSSUBGRU').readOnly = false;
                        }.bind(this)
                    );
                }
            }
        } else {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        }
    };

    this.voltarView = function(args) {
        InflightorGeneral.backOrCancel(args);
        args.owner.widget.reload();
        args.owner.widget.activate();
    };

    this.goBackSubGrupo = function(widget) {
        var subgrupoWidget = widget.container.getWidget("widgetSubgrupo");
        ScreenService.closeSwipe().then(function() {
            subgrupoWidget.activate();
        });
    };

    //Abre popup para adicionar proporcao variavel do subgrupo, executando a validação da quantidade inicial desejada
    this.addProporcao = function(widget) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        var add = function(widget) {
            var popupProporcao = widget.container.getWidget("popupAddProporcao");
            InflightorGeneral.callRestEngine("coc#/crud/coc00022_maxQtpax_auxpropsubgruse", "FilterData", widget.dataSourceFilter).then(
                function(callback) {
                    ScreenService.openPopup(popupProporcao);
                    popupProporcao.newRow();
                    var maxQtpax = callback.dataset.coc00022_maxQtpax_auxpropsubgruse[0].MAX_QTDE;
                    popupProporcao.currentRow.QTPAXDE = parseInt(maxQtpax) + 1;
                }
            );
        }.bind(this);
        this.executeFunction(widget, add, [widget]);
    };

    this.editLastRegister = function(widget) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        var edit = function(widget) {
            if (widget.dataSource.data.length) {
                var popupProporcao = widget.container.getWidget("popupAddProporcao");
                InflightorGeneral.callRestEngine("coc#/crud/coc00022_auxpropsubgruse_last_row", "FilterData", widget.dataSourceFilter).then(
                    function(callback) {
                        ScreenService.openPopup(popupProporcao);
                        popupProporcao.newRow();
                        popupProporcao.currentRow = callback.dataset.coc00022_auxpropsubgruse_last_row[0];
                    }
                );
            } else {
                MessageService.showMessageByCode("COC00022_NO_CHANGE_DATA");
            }
        }.bind(this);
        this.executeFunction(widget, edit, [widget]);
    };

    this.onEnterSonWidgets = function(widget, widgetSubgrupo) {
        if (widget.mode == "list") {
            widgetSubgrupo = widget.getParent();
            InflightorGeneral.buildDataSourceFilter(widget.dataSourceFilter, widgetSubgrupo.currentRow);
            widget.reload();
        } else if (widget.mode == "view" && widgetSubgrupo && widget.name != widgetSubgrupo.name) {
            ScreenService.closeSwipe(1).then(function() {
                if (widgetSubgrupo.currentWidget) {
                    InflightorGeneral.buildDataSourceFilter(widgetSubgrupo.currentWidget.dataSourceFilter, widgetSubgrupo.currentRow);
                    widgetSubgrupo.currentWidget.reload().then(function() {
                        widgetSubgrupo.currentWidget.activate();
                    });
                }
            });
        }
    };

    this.changeVisibleSonWidgets = function(widget, isVisible) {
        var container = widget.container;
        container.getWidget("tabPropVar").isVisible = isVisible;
        container.getWidget("widgetProdutocia").isVisible = isVisible;
        container.getWidget("widgetCiclo").isVisible = isVisible;
    };

    this.editSubgrupo = function(widget) {
        var edit = function(widget) {
            widget.edit();
            widget.backupRow = angular.copy(widget.currentRow);
            this.changeVisibleSonWidgets(widget, false);
            widget.getField('DSSUBGRU').readOnly = true;
        }.bind(this);

        this.executeFunction(widget, edit, [widget]);
    };

    this.cancelaEdicaoSubgrupo = function(args) {
        this.changeVisibleSonWidgets(args.owner.widget, true);
        args.owner.widget.getField('DSSUBGRU').readOnly = false;
        InflightorGeneral.backOrCancel(args);
    };

    this.newSubgrupo = function(widget) {
        InflightorGeneral.newRow(widget);
        if (widget.mode == "edit") {
            widget.getField('DSSUBGRU').reload();
            this.changeVisibleSonWidgets(widget, false);
            this.alteraProporcao(widget);
        }
    };

    //cuida dos subgrupos e da proporção
    this.alteraProporcao = function(widget) {
        var fieldConfiguracao = widget.getField("QTPROPSUBSEMAM");
        fieldConfiguracao.template = "field/text-edit.html";
        fieldConfiguracao.label = "Configuração";
        fieldConfiguracao.isVisible = true;

        //DEFINE A NOVA MASK APENAS APÓS O RETORNO DA FUNCAO valorSemMask
        var maskAtual = widget.currentRow.IDPROPSUBSEMAM;

        var novoValor = valorSemMask(fieldConfiguracao.value(), maskAtual);

        switch (maskAtual) {
            case "2":
                fieldConfiguracao.label = "Configuração (V)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "0"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                fieldConfiguracao.isVisible = true;
                break;
            case "3":
                novoValor = null;
                fieldConfiguracao.isVisible = false;
                break;
            case "4":
                fieldConfiguracao.label = "Configuração (%)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "2"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                break;
            case "5":
                fieldConfiguracao.label = "Configuração (S)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "0"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                break;
            default:
                fieldConfiguracao.isVisible = false;
        }

        var novoField = metaDataFactory.fieldFactory(fieldConfiguracao, widget);
        novoField.setValue(novoValor);
        widget.fields.splice(widget.fields.indexOf(fieldConfiguracao), 1, novoField);

        ScreenService.updateTemplate();
    };

    var valorSemMask = function(valor, mask) {
        if (!valor || valor === "0") {
            return "0";
        } else {
            switch (mask) {
                case "3":
                    return 1;
                default:
                    return valor;
            }
        }
    };

    function setRowWithFilterData(row, dataSourceFilter) {
        for (let index in dataSourceFilter) {
            row[dataSourceFilter[index].name] = dataSourceFilter[index].value;
        }
    }

    /*
this.openPopupAjustDate = function(widget){
popupAjust = widget.container.getWidget('popupAjust');
popupAjust.isVisible = true;
ScreenService.openPopup(popupAjust);
};

this.openCalendarPopup = function(field){
var widget = field.widget;
widget.fieldName = field.name;
var calendarWidget = widget.container.getWidget('calendarWidget');
var filterRow = getFilterRow(widget);
var inicioVigenciaMenugrid = filterRow.DTINIMANUAL.substr(0,10);
var fimVigenciaMenugrid = filterRow.DTFIMMANUAL.substr(0,10);
calendarWidget.minDate = inicioVigenciaMenugrid;
calendarWidget.maxDate = fimVigenciaMenugrid;
ScreenService.openPopup(calendarWidget);
};

this.onEnterCalendarWidget = function(widget){
widget.currentDate = widget.getParent().currentRow[widget.getParent().fieldName].substr(0,10);
};

this.selectDateInWidget = function(widget){
if (typeof widget.currentRow == 'object'){
widget.getParent().currentRow = angular.copy(widget.currentRow);
this.onEnterCalendarWidget(widget);
} else if(typeof widget.currentRow == 'string'){
var date = angular.copy(widget.currentRow);
widget.getParent().currentRow[widget.getParent().fieldName] = date;
widget.getParent().currentRow.changed = true;
ScreenService.closePopup();
widget.getParent().activate();
}
widget.currentRow = '';
};

this.cancelEditAjustDate = function(widget){
ScreenService.closePopup();
var widgetSubgrupo = widget.container.getWidget('widgetSubgrupo');
widgetSubgrupo.view();
widgetSubgrupo.activate();
widget.isVisible = false;
};

this.onEnterAjustPopup = function(widget){
var widgetSubgrupo = widget.container.getWidget('widgetSubgrupo');
InflightorGeneral.buildDataSourceFilter(widget.dataSourceFilter, widgetSubgrupo.currentRow);
widget.reload();
widget.edit();
};
*/

    this.deleteSubgrupo = function(widget) {
        var deleteSg = function(widget) {
            ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question",
                function() {
                    InflightorGeneral.callRestEngine("coc#/coc00022_auxsubgrusemam/remove", "Row", widget.currentRow).then(
                        function() {
                            ScreenService.closeSwipe().then(
                                function() {
                                    widget.list();
                                    widget.reload();
                                }
                            );
                        },
                        function(error) {
                            InflightorGeneral.callbackError(error);
                        }
                    );
                }
            );
        }.bind(this);
        this.executeFunction(widget, deleteSg, [widget]);
    };

    this.saveNewProporcao = function(widget) {
        if (widget.isValid() && verifyQtProporcao(widget.currentRow)) {
            setRowWithFilterData(widget.currentRow, widget.getParent().dataSourceFilter);
            var save = function(widget) {
                InflightorGeneral.callRestEngine("coc#/crud/auxpropsubgruse/save", "Row", widget.currentRow).then(
                    function() {
                        ScreenService.closePopup();
                        widget.getParent().activate();
                        widget.getParent().reload();
                    }
                );
            };
            this.executeFunction(widget, save, [widget]);

        }
    };

    function verifyQtProporcao(row) {
        var valid = parseInt(row.QTPAXDE) <= parseInt(row.QTPAXATE);
        if (!valid) {
            MessageService.showMessageByCode("COC00022_QTDE_MIN_MAX");
        }
        return valid;
    }

    this.saveInlineEditProporcao = function(widget) {
        var saveInlineEdit = function(widget) {
            var editRows = widget.dataSource.data.filter(
                function(row) {
                    return row.changed;
                }
            );

            InflightorGeneral.callRestEngine("coc#/crud/auxpropsubgruse/save", "DataSet", editRows).then(
                function() {
                    widget.reload().then(
                        function() {
                            widget.list();
                        }
                    );
                },
                function(error) {
                    ScreenService.showMessage(error.data.error);
                }
            );
        }.bind(this);

        var callbackError = function(widget) {
            widget.rollbackInlineEdit();
            widget.list();
        };
        this.executeFunction(widget, saveInlineEdit, [widget], callbackError, [widget]);
    };

    this.deleteLastProporcao = function(widget) {
        if (widget.dataSource.data.length) {
            var deleteProp = function(widget) {
                ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question",
                    function() {
                        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
                        this.executeFunction(widget, confirmDeleteProporcao, [widget]);
                    }.bind(this)
                );
            }.bind(this);
            this.executeFunction(widget, deleteProp, [widget]);
        } else {
            MessageService.showMessageByCode("COC00022_NO_DATA_DELETE");
        }
    };

    function confirmDeleteProporcao(widget) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        InflightorGeneral.callRestEngine("coc#/coc00022_delete_last_proporcao", "Row", widget.currentRow).then(
            function() {
                widget.reload();
            },
            function(error) {
                ScreenService.showMessage(error.data.error);
            }
        );
    }

    function getMaxNrordemItens(filter) {
        return InflightorGeneral.callRestEngine("coc#/crud/coc00022_max_nrordem_auxprociasubsemam", "FilterData", filter)
            .then(
                function(callback) {
                    return callback.dataset.coc00022_max_nrordem_auxprociasubsemam[0].MAX_NRORDEM;
                }
            );
    }

    function openSwipeForNewProdCia(widget, nrordem) {
        widget.currentRow.NRORDEM = nrordem;
        InflightorGeneral.justInteger("#QTPRODUTOCIA, #NRORDEM");
    }

    this.editProdcia = function(widget) {
        var edit = function(widget) {
            widget.backupRow = angular.copy(widget.currentRow);
            widget.currentRow = angular.copy(widget.currentRow);
            widget.edit();
        }.bind(this);
        this.executeFunction(widget, edit, [widget]);
    };
    this.addProdCia = function(widget) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        var add = function(widget) {
            widget.newRow();
            var row = {};
            setRowWithFilterData(row, widget.dataSourceFilter);
            InflightorGeneral.onClickDependentField(widget.getField("NMPRODUTO"), row);
            if (widget.dataSource.data.length > 1) {
                getMaxNrordemItens(widget.dataSourceFilter).then(
                    function(nrordem) {
                        openSwipeForNewProdCia(widget, nrordem + 1);
                    }.bind(this)
                );
            } else {
                openSwipeForNewProdCia(widget, 1);
            }
        }.bind(this);
        this.executeFunction(widget, add, [widget]);
    };

    this.cancelSaveProdCia = function(args) {
        var widget = args.owner.widget;
        if (widget.inEditMode()) {
            if (widget.currentRow.__is_new) {
                ScreenService.closeSwipe().then(
                    function() {
                        widget.dataSource.data.pop();
                        widget.list();
                    }
                );
            } else {
                widget.currentRow = angular.copy(widget.backupRow);
                widget.view();
            }
        } else {
            widget.rollbackInlineEdit();
            widget.list();
        }
    };

    this.saveProdCia = function(widget) {
        if (widget.isValid()) {
            var row = {};
            setRowWithFilterData(row, widget.dataSourceFilter);
            verifyHasWorkflow(row, widget).then(
                function(qtde) {
                    if (qtde == 0) {
                        if (widget.inEditMode()) {
                            saveProdCiaOnSwipe(widget);
                        } else {
                            var changedRows = getChangedRows(widget.dataSource.data);
                            serviceSaveProdCia(widget, changedRows, true);
                            widget.reload();
                        }
                    } else {
                        MessageService.showMessageByCode("COC00022_PENDING_APPROVAL");
                    }
                }.bind(this),
                InflightorGeneral.callbackError
            );
        }
    };

    function getChangedRows(data) {
        var changedData = [];
        for (let index in data) {
            if (data[index].changed) {
                changedData.push(data[index]);
            }
        }
        return changedData;
    }

    function saveProdCiaOnSwipe(widget) {
        //valida ordenação antes de efetuar o salvamento
        if (widget.currentRow.__is_new) {
            setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        }
        serviceSaveProdCia(widget, [widget.currentRow], false);
    }

    function serviceSaveProdCia(widget, dataSave, isInlineEdit) {
        InflightorGeneral.callRestEngine("coc#/coc00022_auxprociasubsemam/save", "DataSet", dataSave).then(
            function() {
                if (isInlineEdit) {
                    widget.reload();
                    widget.list();
                } else {
                    if (widget.currentRow.__is_new) {
                        ScreenService.closeSwipe().then(
                            function() {
                                widget.reload();
                                widget.list();
                                widget.activate();
                            }
                        );
                    } else {
                        widget.view();
                        widget.reload();
                    }
                }
            },
            function(error) {
                if (error.data.error.indexOf("PHP Error occurred") >= 0) {
                    MessageService.showMessageByCode("NO_CONNECTION");
                } else {
                    InflightorGeneral.callbackError(error);
                }
            }
        );
    }

    this.moveRowCiclo = function(widget) {
        if (widget.currentWidget && widget.currentWidget.name == "grid_composicao" && widget.currentWidget.mode == "view") {
            ScreenService.closeSwipe(2).then(function() {
                //Close swipe master_detail Composição
                widget.defaultTab = widget.currentWidget.name;
                templateManager.updateTemplate();
                widget.currentWidget.lastActiveWidget = widget;
                self.onEnterSonWidgets(widget.currentWidget, widget);
            });
        } else {
            this.moveRowSubgrupo(widget);
        }
    };

    this.addCiclo = function(widget) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        var add = function(widget) {
            validateAirVisionField(widget);
            var row = {};
            setRowWithFilterData(row, widget.dataSourceFilter);
            InflightorGeneral.onClickDependentField(widget.getField("IDCICLO"), row);
            widget.currentRow.FGFOTOSUBSEMAM = window.undefinedImage;
            templateManager.updateTemplate();
            widget.newRow();
            widget.backupRow = widget.currentRow;
            this.setVisibleSonWidgetsCiclo(widget, false);
        }.bind(this);
        this.executeFunction(widget, add, [widget]);
    };

    this.editCiclo = function(widget) {
        var edit = function(widget) {
            validateAirVisionField(widget);
            InflightorGeneral.edit(widget);
            this.setVisibleSonWidgetsCiclo(widget, false);
        }.bind(this);
        this.executeFunction(widget, edit, [widget]);
    };

    function validateAirVisionField(widgetCiclo) {
        var filterRow = getFilterRow(widgetCiclo);
        widgetCiclo.getField("NRITAIRVISION").readOnly = (filterRow.IDUTIAIRVISION == "N");
    }

    this.setVisibleSonWidgetsCiclo = function(widget, isVisible) {
        widget.container.getWidget("grid_composicao").isVisible = isVisible;
    };

    this.saveCiclo = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.DSMONTASUBSEMAMHTML = FieldUtil.changeTagRichText(widget.currentRow.DSMONTASUBSEMAMHTML);
            var currentRow = widget.currentRow;
            if (currentRow.FGFOTOSUBSEMAM == window.undefinedImage) {
                currentRow.FGFOTOSUBSEMAM = null;
            }
            setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
            currentRow.PHOTO_UPLOAD = currentRow.PHOTO_UPLOAD ? currentRow.PHOTO_UPLOAD : [];
            // currentRow.DSMONTASUBSEMAMHTML = currentRow.DSMONTASUBSEMAMHTML ? coc03600_menu_designService.htmlentities(currentRow.DSMONTASUBSEMAMHTML) : "";
            widget.saveRow().then(
                function() {
                    if (currentRow.__is_new) {
                        ScreenService.closeSwipe().then(
                            function() {
                                widget.list();
                                widget.reload();
                            }
                        );
                    } else {
                        widget.view();
                        widget.reload([], false).then(function() {
                            widget.dataSource.data.forEach(register => {
                                if (register.CDCICLO == widget.currentRow.CDCICLO) {
                                    widget.currentRow.OLD_FGFOTOSUBSEMAM = register.OLD_FGFOTOSUBSEMAM;
                                    widget.currentRow.FGFOTOSUBSEMAM = register.FGFOTOSUBSEMAM == null || register.FGFOTOSUBSEMAM === window.filesPath ? window.undefinedImage : register.FGFOTOSUBSEMAM;
                                }
                            });

                        });
                        // currentRow.DSMONTASUBSEMAMHTML = currentRow.DSMONTASUBSEMAMHTML ? coc03600_menu_designService.htmlEntityDecode(currentRow.DSMONTASUBSEMAMHTML) : "";
                        if (widget.backupRow.DSMONTASUBSEMAMHTML != currentRow.DSMONTASUBSEMAMHTML) {
                            currentRow.POSSUI_MONTAGEM = "S";
                        }
                    }
                    widget.activate();
                    this.setVisibleSonWidgetsCiclo(widget, true);
                }.bind(this),
                InflightorGeneral.callbackError
            );
        }
    };

    this.addComposicao = function(widget, ID) {
        setRowWithFilterData(widget.currentRow, widget.dataSourceFilter);
        var add = function(widget, ID) {
            widget.newRow();
            widget.currentRow.ID = ID;
            if (widget.dataSource.data.length > 1) {
                var filter = angular.copy(widget.dataSourceFilter);
                filter.push({
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                });
                getMaxNrordem(filter).then(
                    function(maxOrdem) {
                        widget.currentRow.NRORDEM = maxOrdem + 1;
                        this.changSwipeEditComposicao(widget);
                    }.bind(this)
                );
            } else {
                widget.currentRow.NRORDEM = 1;
                this.changSwipeEditComposicao(widget);
            }
        }.bind(this);
        this.executeFunction(widget, add, [widget, ID]);
    };

    function getMaxNrordem(filter) {
        return InflightorGeneral.callRestEngine("coc#/crud/coc0002_maxNrordem_auxprodsubsemam_auxrecsubsemam", "FilterData", filter).then(
            function(callback) {
                var maxOrdem = callback.dataset.coc0002_maxNrordem_auxprodsubsemam_auxrecsubsemam[0].MAXNRORDEM;
                return maxOrdem;
            }
        );
    }

    function showFieldsOnForm(widget, fieldActive, fieldInactive) {
        fieldActive.isVisible = true;
        fieldActive.validations = {
            required: true
        };

        fieldInactive.isVisible = false;
        fieldInactive.validations = {};
        InflightorGeneral.onClickDependentField(fieldActive, widget.getParent().currentRow);
    }

    this.changSwipeEditComposicao = function(widget) {
        var currentRow = widget.currentRow;

        if (currentRow.__is_new) {
            var fieldActive;
            var fieldInactive;
            widget.getField("NOME").showOnForm = false;

            if (currentRow.ID == "S") {
                widget.getField("NMLEANCODE").isVisible = false;
                fieldActive = widget.getField("DSRECEITA");
                fieldInactive = widget.getField("NMPRODUTO");
            } else {
                widget.getField("NMLEANCODE").isVisible = true;
                fieldActive = widget.getField("NMPRODUTO");
                fieldInactive = widget.getField("DSRECEITA");
            }
            showFieldsOnForm(widget, fieldActive, fieldInactive);
        } else {
            widget.getField("NOME").showOnForm = true;
            widget.getField("NMPRODUTO").validations = {};
            widget.getField("NMPRODUTO").isVisible = false;
            widget.getField("DSRECEITA").validations = {};
            widget.getField("DSRECEITA").isVisible = false;
        }
    };

    this.moveRowComposicao = function(widget) {
        var currentRow = widget.currentRow;
        if (widget.mode == "view") {
            widget.getField("NMLEANCODE").isVisible = currentRow.ID == "P";
        }
    };

    function mapProdutoReceita(row) {
        if (row.ID == "P") {
            row.CDPRODUTO = row.CODIGO;
            row.QTPRODSUMA = row.QTD;
            row.TXCOPRODSUMA = row.COMENTARIO;
            row.NMPRODUTO = row.NOME;
        } else {
            row.CDRECEITA = row.CODIGO;
            row.QTRECSUMA = row.QTD;
            row.TXCOMRECSUMA = row.COMENTARIO;
            row.DSRECEITA = row.NOME;
        }
    }

    this.saveComposicao = function(widget) {
        var row = {};
        setRowWithFilterData(row, widget.dataSourceFilter);
        verifyHasWorkflow(row, widget).then(
            function(qtde) {
                if (qtde == 0) {
                    if (widget.inEditMode()) {
                        saveComposicaoOnSwipe(widget);
                    } else {
                        saveInlineEditComposicao(widget);
                    }
                } else {
                    this.cancelEditComposicao(widget);
                    MessageService.showMessageByCode("COC00022_PENDING_APPROVAL");
                }
            }.bind(this),
            InflightorGeneral.callbackError
        );
    };

    function getChangedRowsComposicao(rows) {
        var dataChanged = [];
        for (let index in rows) {
            var row = copyRow(rows[index]);
            if (row.changed) {
                mapProdutoReceita(row);
                dataChanged.push(row);
            }
        }
        return dataChanged;
    }

    function saveInlineEditComposicao(widget) {
        var changedRows = getChangedRowsComposicao(widget.dataSource.data);

        InflightorGeneral.callRestEngine("coc#/coc00022_auxprodsubsemam_auxrecsubsemam/save", "DataSet", changedRows).then(
            function() {
                widget.reload();
                widget.list();
            },
            function(error) {
                InflightorGeneral.callbackError(error);
            }
        );
    }

    function saveComposicaoOnSwipe(widget) {
        if (widget.isValid()) {
            if (InflightorGeneral.valueToFloat(widget.currentRow.QTD) != 0) {
                var currentRow = widget.currentRow;
                InflightorGeneral.setFloatAttrOnCurrentRow(currentRow, "QTD");
                InflightorGeneral.setFloatAttrOnCurrentRow(currentRow, "NRORDEM");
                InflightorGeneral.setFloatAttrOnCurrentRow(currentRow, "VRPESOGRAMATURA");
                InflightorGeneral.setFloatAttrOnCurrentRow(currentRow, "QTGRAMATURA");
                var rowToSave = copyRow(currentRow);
                mapProdutoReceita(rowToSave);
                setRowWithFilterData(rowToSave, widget.dataSourceFilter);
                InflightorGeneral.callRestEngine("coc#/coc00022_auxprodsubsemam_auxrecsubsemam/save", "DataSet", [rowToSave]).then(
                    function() {
                        if (currentRow.__is_new) {
                            ScreenService.closeSwipe().then(
                                function() {
                                    widget.list();
                                    widget.activate();
                                    widget.reload();
                                }
                            );
                        } else {
                            widget.view();
                            widget.reload();
                        }
                    },
                    function(error) {
                        InflightorGeneral.callbackError(error);
                    }
                );
            } else {
                MessageService.showMessageByCode("COC00022_QTDE_NOT_EQUAL_ZERO");
            }
        } else {
            MessageService.showMessageByCode("FILL_FIELDS_CORRECTLY");
        }
    }

    this.deleteComposicao = function(widget) {
        var excluir = function(widget) {
            ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question",
                function() {
                    var rowToDelete = copyRow(widget.currentRow);
                    mapProdutoReceita(rowToDelete);
                    var dataSourceName = rowToDelete.ID == "P" ? "auxprodsubsemam" : "auxrecsubsemam";
                    InflightorGeneral.callRestEngine("coc#/crud/" + dataSourceName + "/remove", "Row", rowToDelete).then(
                        function() {
                            ScreenService.closeSwipe().then(
                                function() {
                                    widget.list();
                                    widget.reload();
                                }
                            );
                        },
                        function(callback) {
                            InflightorGeneral.callbackError(callback);
                        }
                    );
                }.bind(this)
            );
        }.bind(this);
        this.executeFunction(widget, excluir, [widget]);
    };

    this.editComposicao = function(widget) {
        var edit = function(widget) {
            widget.backupRow = copyRow(widget.currentRow);
            var __leftActions = widget.currentRow.__leftActions;
            var __rightActions = widget.currentRow.__rightActions;
            widget.currentRow = copyRow(widget.currentRow);
            widget.currentRow.__rightActions = __rightActions;
            widget.currentRow.__leftActions = __leftActions;
            this.changSwipeEditComposicao(widget);
            widget.edit();
        }.bind(this);
        this.executeFunction(widget, edit, [widget]);
    };

    this.cancelEditComposicao = function(widget) {
        if (widget.inEditMode()) {
            var currentRow = widget.currentRow;
            if (currentRow.__is_new) {
                currentRow = {};
                widget.dataSource.data.pop();
                ScreenService.closeSwipe().then(
                    function() {
                        widget.list();
                    }
                );
            } else {
                var __leftActions = currentRow.__leftActions;
                var __rightActions = currentRow.__rightActions;
                widget.currentRow = angular.copy(widget.backupRow);
                widget.currentRow.__leftActions = __leftActions;
                widget.currentRow.__rightActions = __rightActions;
                widget.view();
            }
        } else {
            widget.rollbackInlineEdit();
            widget.list();
        }
    };

    this.openDuplicatePopup = function(widget) {
        if (InflightorGeneral.isWidgetFiltered(widget)) {
            var duplicatePopup = widget.container.getWidget("duplicatePopup");
            duplicatePopup.isVisible = true;
            ScreenService.openPopup(duplicatePopup);
            duplicatePopup.newRow();
            duplicatePopup.currentRow = angular.copy(getFilterRow(widget));
        } else {
            MessageService.showMessageByCode("COC00022_EXECUTE_FILTER");
        }
    };

    this.duplicateSubgrupo = function(widget) {
        if (widget.isValid()) {
            InflightorGeneral.callRestEngine("coc#/coc00022_duplicate", "Row", widget.currentRow).then(
                function() {
                    this.cancelPopup(widget);
                    widget.getParent().reload();
                }.bind(this),
                function(error) {
                    InflightorGeneral.callbackError(error);
                }
            );
        }
    };

    this.effect = function(widget) {
        var row = angular.copy(widget.currentRow);
        if (row.DTINIVGSUMA && row.DTFIMVGSUMA) {
            row.NRSEQSUBGRUSEMAM = "T";
            verifyHasWorkflow(row, widget).then(
                function(qtde) {
                    if (qtde == 0) {
                        var filter = [
                            {
                                name: "CDFILIAL",
                                operator: "=",
                                value: row.CDFILIAL
                            },
                            {
                                name: "CDCIA",
                                operator: "=",
                                value: row.CDCIA
                            },
                            {
                                name: "CDMANCIA",
                                operator: "=",
                                value: row.CDMANCIA
                            },
                            {
                                name: "NRSERVICOCIAFILI",
                                operator: "=",
                                value: row.NRSERVICOCIAFILI
                            },
                            {
                                name: "CDCLASSE",
                                operator: "=",
                                value: row.CDCLASSE
                            },
                            {
                                name: "CDSUBGRU",
                                operator: "=",
                                value: row.CDSUBGRU
                            },
                            {
                                name: "DTINI",
                                operator: "=",
                                value: row.DTINIVGSUMA.substr(0, 10)
                            },
                            {
                                name: "DTFIM",
                                operator: "=",
                                value: row.DTFIMVGSUMA.substr(0, 10)
                            }
                        ];
                        InflightorGeneral.callRestEngine("coc#/crud/coc00022_subgrusemam_date_validate", "FilterData", filter).then(
                            function(callback) {
                                var datasSobrepostas = callback.dataset.coc00022_subgrusemam_date_validate;
                                if (datasSobrepostas.length) {
                                    MessageService.showMessageByCode("COC00022_OVERLAPPING_DATES");
                                    //this.openPopupAjustDate(widget);
                                    var dataError = [];
                                    for (let index in datasSobrepostas) {
                                        dataError.push(datasSobrepostas[index].DATA_ERROR);
                                    }
                                    InflightorGeneral.showErrorMessages(widget, dataError, "Subgrupos com datas sobrepostas");
                                } else {
                                    var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
                                    let filterRow = getFilterRow(widgetSubgrupo);
                                    widget.currentRow.SERVICO_CLASSE = filterRow.DSSERVICOCIAFILI;
                                    this.tryEffectSubgrupo(widget);
                                }
                            }.bind(this),
                            function(error) {
                                InflightorGeneral.callbackError(error);
                            }
                        );
                    } else {
                        ScreenService.showMessage("Operação bloqueada. <br>Existe uma versão do subgrupo: " + widget.currentRow.DSSUBGRU + " em processo de aprovação.");
                    }
                }.bind(this), InflightorGeneral.callbackError
            );
        } else {
            MessageService.showMessageByCode("COC00022_FILL_DATE");
        }
    };

    this.tryEffectSubgrupo = function(widgetSubgrupo) {
        usaWf().then(
            function(usaWorkFlow) {
                var dataSave = getDataSaveEffect(widgetSubgrupo);
                dataSave.USA_WF = usaWorkFlow;
                if (usaWorkFlow) {
                    this.openPopupMotivoWf(widgetSubgrupo, dataSave);
                } else {
                    this.executeEffectSubgrupo(widgetSubgrupo, dataSave);
                }
            }.bind(this)
        );
    };

    function getDataSaveEffect(widget) {
        var filterRow = getFilterRow(widget);
        var row = angular.copy(widget.currentRow);
        for (let index in filterRow) {
            row[index] = filterRow[index];
        }
        row.DSSUBGRU = widget.currentRow.DSSUBGRU;
        row.DTFIMVGSUMA = widget.currentRow.DTFIMVGSUMA.substr(0, 10);
        row.DTINIVGSUMA = widget.currentRow.DTINIVGSUMA.substr(0, 10);
        return row;
    }

    this.openPopupMotivoWf = function(widgetSubgrupo, dataSave) {
        var popup = {
            "name": "motivoWf",
            "label": "Motivo",
            "dataSave": dataSave,
            "isVisible": true,
            "showAsPopup": true,
            "template": "widget/form.html",
            "formTemplate": "widget/form.html",
            "events": [
                {
                    "name": "WidgetOnEnter",
                    "code": "args.owner.newRow();"
                }
            ],
            "actions": [
                {
                    "name": "save",
                    "label": "Salvar",
                    "isVisible": true,
                    "showAsAction": "never_edit",
                    "hideIcon": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Coc00022_ManutencaoMenugrid_Controller.executeFunctionAfterWF(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "cancel",
                    "label": "Cancelar",
                    "isVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back_edit",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "DSMOTIVOACAO",
                    "label": "Motivo",
                    "maxlength": 2000,
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                }
            ]
        };
        popup.container = widgetSubgrupo.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(popup);
        MessageService.showMessageByCode("COC00022_OPERATION_WORKFLOW");
        ScreenService.openPopup(motivoWidgetInitialized);
    };

    this.executeFunctionAfterWF = function(widget) {
        if (widget.isValid()) {
            var dataSave = widget.dataSave;
            dataSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
            this.executeEffectSubgrupo(widgetSubgrupo, dataSave);
        } else {
            MessageService.showMessageByCode("COC00022_FILL_REASON_WORKFLOW");
        }
    };

    this.executeEffectSubgrupo = function(widgetSubgrupo, dataSave) {
        InflightorGeneral.callRestEngine("coc#/coc00022_effect_auxsubgrusemam", "Row", dataSave).then(
            function() {
                if (dataSave.USA_WF) {
                    ScreenService.closePopup();
                    ScreenService.closeSwipe().then(
                        function() {
                            widgetSubgrupo.activate();
                            widgetSubgrupo.reload();
                            MessageService.showMessageByCode("WORKFLOW_SUCCESS");
                        }
                    );
                } else {
                    ScreenService.closeSwipe().then(
                        function() {
                            MessageService.showMessageByCode("COC00022_EFFECTIVENESS");
                            widgetSubgrupo.list();
                            widgetSubgrupo.activate();
                            widgetSubgrupo.reload();
                        }
                    );
                }
            },
            function(error) {
                InflightorGeneral.callbackError(error);
            }
        );
    };
    /*
this.confirmChangeDates = function(widget){
var datasAlteradas = widget.dataSource.data.filter(function(row){return row.changed;});
InflightorGeneral.callRestEngine('coc#/coc00022_try_save_datas', 'DataSet', datasAlteradas).then(
function (){
var widgetSubgrupo = widget.container.getWidget('widgetSubgrupo');
this.tryEffectSubgrupo(widgetSubgrupo, datasAlteradas);
}.bind(this),
function (error){
InflightorGeneral.callbackError(error);
}
);
};
*/
    function usaWf() {
        var filter = [
            {
                name: "NRPROCESSOAPROVA",
                operator: "=",
                value: 4
            },
            {
                name: "NRITEMPROCAPROVA",
                operator: "=",
                value: 24
            }
        ];
        return InflightorGeneral.callRestEngine("coc#/crud/itemprocaprova", "FilterData", filter).then(
            function(callback) {
                var usaWf = callback.dataset.itemprocaprova.length ? callback.dataset.itemprocaprova[0].IDATIVO == "S" : false;
                return usaWf;
            },
            function(error) {
                InflightorGeneral.callbackError(error);
            }
        );
    }

    this.moveRowSubgrupo = function(widgetSubgrupo) {
        var activeWidget = widgetSubgrupo.container.getActiveWidget();
        if ((widgetSubgrupo.name == "widgetSubgrupo" && activeWidget.name != "widgetSubgrupo") && !isLastTabbed(widgetSubgrupo, activeWidget)) {
            ScreenService.closeSwipe().then(function() {
                widgetSubgrupo.currentWidget.activate();
                self.onEnterSonWidgets(widgetSubgrupo.currentWidget, widgetSubgrupo);
            });
        } else if (widgetSubgrupo.name != activeWidget.name) {
            widgetSubgrupo.currentWidget = activeWidget;
            widgetSubgrupo.defaultTab = activeWidget.name;
            templateManager.updateTemplate();
            activeWidget.lastActiveWidget = widgetSubgrupo;
            self.onEnterSonWidgets(activeWidget, widgetSubgrupo);
        } else {
            widgetSubgrupo.lastActiveWidget = widgetSubgrupo;
            widgetSubgrupo.currentWidget = widgetSubgrupo;
            widgetSubgrupo.defaultTab = widgetSubgrupo.name;
        }
    };

    function isLastTabbed(widget, activeWidget) {
        return widget.widgets.filter(function(tabbed) {
            return tabbed.name == activeWidget.name;
        }).length;
    }

    this.cancelPopup = function(widget) {
        ScreenService.closePopup();
        widget.isVisible = false;
        widget.getParent().activate();
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "Coc00022_ManutencaoMenugrid_Controller.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.selectReceita = function(field) {
        var row = field.widget.currentRow;
        var filterRow = getFilterRow(field.widget);
        if (row.CDCIAREC && (row.CDCIAREC != filterRow.CDCIA)) {
            MessageService.showMessageByCode("COC00022_EXCLUSIVE_RECIPE");
            field.clearValue();
        } else if (row.IDATIVO == "N") {
            MessageService.showMessageByCode("INACTIVE_RECIPE.");
            field.clearValue();
        }
    };

    this.selectProduto = function(field) {
        var row = field.widget.currentRow;
        if (row.IDPRODATIVPR == "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
            field.clearValue();
        }
    };

    this.applyInactiveStyleReceita = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "Coc00022_ManutencaoMenugrid_Controller.inactiveBranchWarningReceita(args);";
    };

    this.inactiveBranchWarningReceita = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_RECIPE");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.applyInactiveStyleProduto = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            },
            {
                "condition": "row.IDPRODATIVPR == 'S' && row.PRODORCAMENTO == 'S'",
                "cssClass": "inf-orange"
            }
        ];
        field.selectWidget.events[0].code = "Coc00022_ManutencaoMenugrid_Controller.inactiveBranchWarningProduto(args);";
    };

    this.inactiveBranchWarningProduto = function(args) {
        if (args.owner.currentRow.IDPRODATIVPR === "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        } else {
            args.owner.setSelected(args);
        }
    };
    //Exclusao multipla de itens da composicao
    this.deleteCheckedRowsComposicao = function(widget) {
        var excluir = function(widget) {
            ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question",
                function() {
                    var checkedRows = widget.dataSource.getCheckedRows().slice();
                    var dataToDelete = checkedRows.filter(filterProdutoReceita);
                    var prdToDelete = dataToDelete.filter(function(row) {
                        return row.ID == "P";
                    });
                    var recToDelete = dataToDelete.filter(function(row) {
                        return row.ID != "P";
                    });
                    var promises = [];
                    if (prdToDelete.length) {
                        promises.push(InflightorGeneral.callRestEngine("coc#/crud/auxprodsubsemam/remove", "DataSet", prdToDelete));
                    }
                    if (recToDelete.length) {
                        promises.push(InflightorGeneral.callRestEngine("coc#/crud/auxrecsubsemam/remove", "DataSet", recToDelete));
                    }
                    ZHPromise.all(promises)
                        .then(function() {
                            widget.reload();
                            widget.unSelectAll();
                            if (ScreenService.getSwipeLevel(widget) === 2) {
                                ScreenService.closeSwipe();
                            }
                            MessageService.showMessageByCode("DELETE_SUCCESS");
                        }, function(err) {
                            ScreenService.showMessage(err);
                        });
                }.bind(this)
            );
        }.bind(this);
        this.executeFunction(widget, excluir, [widget]);
    };

    function filterProdutoReceita(row) {
        if (row.ID == "P") {
            row.CDPRODUTO = row.CODIGO;
            row.QTPRODSUMA = row.QTD;
            row.TXCOPRODSUMA = row.COMENTARIO;
            row.NMPRODUTO = row.NOME;
        } else {
            row.CDRECEITA = row.CODIGO;
            row.QTRECSUMA = row.QTD;
            row.TXCOMRECSUMA = row.COMENTARIO;
            row.DSRECEITA = row.NOME;
        }
        delete row.__leftActions;
        delete row.__rightActions;
        return row;
    }

    this.saveSubgrupoPincipal = function(widget) {
        if (widget.isValid()) {
            var parentWidget = widget.container.getWidget("grid_subgrupos_principais");
            var dataSourceFilter = parentWidget.dataSourceFilter;
            var dataSave = getDataSaveSubgruposPrincipais(widget.currentRow, dataSourceFilter);
            InflightorGeneral.callRestEngine(parentWidget.dataSource.name + "/save", "DataSet", dataSave).then(
                function() {
                    ScreenService.closePopup();
                    parentWidget.activate();
                    parentWidget.reload();
                },
                InflightorGeneral.callbackError
            );
        }
    };

    function getDataSaveSubgruposPrincipais(row, dataSourceFilter) {
        var data = [];
        for (let index in row.DSSUBGRU) {
            var rowSave = {
                NRSEQSUBGRUSEMAMPRIN: row.DSSUBGRU[index].NRSEQSUBGRUSEMAM,
                CDSUBGRUPRIN: row.DSSUBGRU[index].CDSUBGRU,
                __is_new: true,
                changed: true
            };
            setRowWithFilterData(rowSave, dataSourceFilter);
            data.push(rowSave);
        }
        return data;
    }

    function setCurrentRow(widget) {
        var paramsNewCode = widget.paramsNewCode;
        if (paramsNewCode && paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk = paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
        }
        widget.currentRow.__is_new = widget.currentRow.__is_new ? widget.currentRow.__is_new : true;
    }

    /*
Adição de novos subgrupos dentro da tela de Manutenção do Menu Grid: 
lê o container da tela de Cadastro de Subgrupos genericos e fatora os
fields dentro do Popup de adição na tela de Manutenção do Menu Grid
evitando, assim, a repetição dos fields e garantindo que serão sempre 
os mesmos da tela de Cadastro. 
*/
    this.cancelNewSubgrupo = function() {
        ScreenService.closePopup();
    };
    this.saveNewSubgrupo = function(widget) {
        if (!widget.isValid())
            return;
        setCurrentRow(widget);
        widget.saveRow().then(function() {
            ScreenService.closePopup();
            widget.container.getWidget("widgetSubgrupo").getField("DSSUBGRU").reload();
        }, function(error) {
            InflightorGeneral.callbackError(error);
        });
    };
    this.addSubGrupo = function(field, widgetNewSubgrupo) {
        widgetNewSubgrupo.fields = [];
        widgetNewSubgrupo.currentRow = {};
        JSONContainerLoader.loadContainer("coc#coc01410_cadastrosubgrupo").then(function(containerJson) {
            //Loader container Cadastro de Subgrupo
            containerJson.widgets[1].fields.forEach(function(field) {
                metaDataFactory.fieldFactory(field, widgetNewSubgrupo, true);
                widgetNewSubgrupo.getField(field.name).setValue("");
            });
        });
        ScreenService.openPopup(widgetNewSubgrupo).then(function() {
            widgetNewSubgrupo.activate();
        });
    };

    this.setUndefinedImage = function(widget, propertyToReplace) {
        widget.dataSource.data.forEach(item => {
            let prop = item[propertyToReplace];
            if (prop == null || prop == window.filesPath) {
                item[propertyToReplace] = window.undefinedImage;
            }
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Coc00022_ManutencaoMenugrid_Controller", Coc00022_ManutencaoMenugrid_Controller);
});

// FILE: ./assets/js/controllers/coc00600_classeController.js
var coc00600_classeController = function(ScreenService, InflightorGeneral, MessageService) {

    var self = this;

    this.salvaClassePassageiro = function(widget) {
        InflightorGeneral.saveCrud(widget);
    };

    this.validaDsclasse = function(widget) {
        var filtroDescricao = [
            {
                "name": "DSCLASSE",
                "operator": "=",
                "value": widget.currentRow.DSCLASSE
            }
        ];

        InflightorGeneral.callRestEngine("coc#/crud/classe", "FilterData", filtroDescricao)
            .then(function(callback) {
                if (widget.currentRow.__is_new) {
                    if (callback.dataset.classe.length === 0) {
                        self.salvaClassePassageiro(widget);
                    } else {
                        MessageService.showMessageByCode("COC00600_CLASS_EXISTS");
                    }
                } else {
                    InflightorGeneral.callRestEngine("coc#/crud/classe", "FilterData", filtroDescricao)
                        .then(function() {
                            if (callback.dataset.classe.length === 0) {
                                self.salvaClassePassageiro(widget);
                            } else if (callback.dataset.classe.length === 1 && widget.backupRow.DSCLASSE === widget.currentRow.DSCLASSE) {
                                self.salvaClassePassageiro(widget);
                            } else {
                                MessageService.showMessageByCode("COC00600_CLASS_EXISTS");
                            }
                        });
                }
            }.bind(widget), function(err) {
                ScreenService.showMessage(err.data.error);
            });
    };

    this.validaSgclasse = function(widget) {
        if (widget.isValid()) {
            var filtroSigla = [
                {
                    "name": "SGCLASSE",
                    "operator": "=",
                    "value": widget.currentRow.SGCLASSE
                }
            ];

            InflightorGeneral.callRestEngine("coc#/crud/classe", "FilterData", filtroSigla)
                .then(function(callback) {
                    if (widget.currentRow.__is_new) {
                        if (callback.dataset.classe.length === 0) {
                            self.validaDsclasse(widget);
                        } else {
                            MessageService.showMessageByCode("COC00600_SIGLA_EXISTS");
                        }
                    } else {
                        InflightorGeneral.callRestEngine("coc#/crud/classe", "FilterData", filtroSigla)
                            .then(function() {
                                if (callback.dataset.classe.length === 0) {
                                    self.validaDsclasse(widget);
                                } else if (callback.dataset.classe.length === 1 && widget.backupRow.SGCLASSE === widget.currentRow.SGCLASSE) {
                                    self.validaDsclasse(widget);
                                } else {
                                    MessageService.showMessageByCode("COC00600_SIGLA_EXISTS");
                                }
                            });
                    }
                }.bind(widget), function(err) {
                    ScreenService.showMessage(err.data.error);
                });
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00600_classeController", coc00600_classeController);
});

// FILE: ./assets/js/controllers/coc00700_cadastrocompanhiaController.js
var coc00700_cadastrocompanhiaController = function(ScreenService, MessageService, FilterUtil, FieldUtil, InflightorGeneral, restEngine, templateManager) {

    var parent = this;

    this.setEndeclieFilter = function(widget) {
        var parentRow = widget.parent.currentRow;
        FilterUtil.updateFilter(widget, parentRow.CDCLIENTE, "CDCLIENTE");
        if (widget.mode != "edit" && widget.mode != "view") {
            widget.reload();
        }
    };

    this.setCurrentRowForSave = function(widget) {
        parent.copyCdclienteFromFilter(widget);
        parent.setTipoendeclieField(widget);
    };

    this.copyCdclienteFromFilter = function(widget) {
        var filteredCdCliente = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDCLIENTE");
        widget.currentRow.CDCLIENTE = filteredCdCliente;
    };

    this.setCiaFilter = function(widget) {
        var parentRow = widget.parent.currentRow;
        FilterUtil.updateFilter(widget, parentRow.CDCIA, "CDCIA");
        if (widget.mode != "edit" && widget.mode != "view") {
            widget.reload();
        }
    };

    this.copyCdciaFromFilter = function(widget) {
        var filteredCdCia = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDCIA");
        if (filteredCdCia)
            widget.currentRow.CDCIA = filteredCdCia;
    };

    this.enableEstadoField = function(widget) {
        var estadoField = widget.getField("NMESTADO");
        var paisField = widget.getField("CDPAIS");

        if (paisField.value()) {
            FieldUtil.toggleField(estadoField, true);
            FilterUtil.updateFilter(estadoField, paisField.value(), "CDPAIS");
            estadoField.reload();
        } else {
            FieldUtil.toggleField(estadoField, false);
            FieldUtil.clearField(estadoField);
        }
    };

    this.enableMunicipioField = function(widget) {
        var municipioField = widget.getField("NMMUNICIPIO");
        var paisField = widget.getField("CDPAIS");
        var estadoField = widget.getField("SGESTADO");

        if (estadoField.value()) {
            FieldUtil.toggleField(municipioField, true);
            FilterUtil.updateFilter(municipioField, paisField.value(), "CDPAIS");
            FilterUtil.updateFilter(municipioField, estadoField.value(), "SGESTADO");
            municipioField.reload();
        } else {
            FieldUtil.toggleField(municipioField, false);
            FieldUtil.clearField(municipioField);
        }
    };

    this.filterDataRowsFromSelect = function(field, primaryKey) {
        var widget = field.widget;
        var data = widget.dataSource.data;
        var newArray = data.filter(function(currentItem) {
            return currentItem[primaryKey];
        });
        var keyArray = newArray.map(function(currentItem) {
            return currentItem[primaryKey];
        });
        if (keyArray.length > 0) {
            FilterUtil.updateFilter(field, keyArray, primaryKey, "NOT_IN");
            field.reload();
        }
    };

    this.setTipoendeclieField = function(widget) {
        var editFieldValue = widget.getField("IDTPENDECLIE_EDIT").value();
        if (editFieldValue) {
            widget.currentRow.IDTPENDECLIE = editFieldValue;
        }
    };

    this.popDataToCurrentRow = function(widget) {
        var data = widget.dataSource.data;
        if (data)
            widget.currentRow = data[data.length - 1];
    };

    /*
this.filterDataRowsFromFieldDataByTipo = function(widget){
var viewFieldValue=widget.getField("IDTPENDECLIE").value();
var field=widget.getField("IDTPENDECLIE_EDIT");
field.setValue(viewFieldValue);
var widgetData=widget.dataSource.data;
var fieldData=field.dataSource.data;
var currentRow=widget.currentRow;

console.log(widgetData, fieldData, currentRow);

var newData=fieldData.filter(function(fieldDataItem){

var dataToRemove=widgetData.find(function(widgetDataItem){
//um elemento de fieldData só será selecionado para remoção
//se já houver um registro com o mesmo IDTPENDECLIE salvo
//e ou se está criando um novo registro (a current row estará vazia)
//ou o elemento não é o cujo valor corresponde ao IDTPENDECLIE sendo editado
if(fieldDataItem.value==widgetDataItem.IDTPENDECLIE &&
(!currentRow.IDTPENDECLIE || fieldDataItem.value!==currentRow.IDTPENDECLIE)){
return true;
}
return false;
});

if(dataToRemove) return false;
else return true;
});
console.log(newData);

field.dataSource.data=newData;
};*/

    this.filterDataRowsFromFieldDataByTipo = function(widget) {
        var field = widget.getField("IDTPENDECLIE_EDIT");
        var widgetData = widget.dataSource.data;
        var fieldData = field.dataSource.data;

        var newData = fieldData.filter(function(fieldDataItem) {

            var dataToRemove = widgetData.find(function(widgetDataItem) {
                if (fieldDataItem.value == widgetDataItem.IDTPENDECLIE) {
                    return true;
                }
                return false;
            });

            if (dataToRemove) return false;
            else return true;
        });

        field.dataSource.data = newData;
    };

    this.setTpendeclieVisibility = function(widget, enable) {
        if (enable) {
            widget.getField("IDTPENDECLIE_EDIT").isVisible = true;
            widget.getField("IDTPENDECLIE").showOnForm = false;
        } else {
            widget.getField("IDTPENDECLIE_EDIT").isVisible = false;
            widget.getField("IDTPENDECLIE").showOnForm = true;
        }
    };

    this.restoreTipoFieldData = function(widget) {
        var field = widget.getField("IDTPENDECLIE_EDIT");
        field.dataSource.data = [
            {
                "name": "Comercial",
                "value": "P"
            },
            {
                "name": "Cobrança",
                "value": "C"
            },
            {
                "name": "Internacional",
                "value": "E"
            },
            {
                "name": "Exportação",
                "value": "X"
            }
        ];
    };

    this.setLogoFieldSource = function(widget) {
        if (widget.currentRow && widget.currentRow.DSLOGOCIA) {
            widget.getField("DSLOGOCIA").isVisible = true;
            widget.getAction("EXCLUIR").isVisible = true;
        } else {
            widget.getField("DSLOGOCIA").isVisible = false;
            widget.getAction("EXCLUIR").isVisible = false;
        }
    };

    this.abrePopup = function(popup) {
        popup.dataSourceFilter[0].value = popup.parent.parent.currentRow.CDCIA;
        popup.activate();
        popup.reload();
    };

    this.vinculaCompanhiaFilial = function(popupUnidade) {
        var datasetToSave = [];
        popupUnidade.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            datasetToSave.push(row);
        });
        restEngine.post("coc#/crud/ciafili/save", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            ScreenService.closePopup();
            popupUnidade.container.getWidget("ciafili").reload();
            popupUnidade.container.getWidget("ciafili").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.vinculaCompanhiaLeanCode = function(popupLeanCode) {
        var datasetToSave = [];
        popupLeanCode.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            row.IDBLOQUEIA = "N";
            datasetToSave.push(row);
        });
        restEngine.post("coc#/crud/leancodecia/save", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            ScreenService.closePopup();
            popupLeanCode.container.getWidget("leancodecia").reload();
            popupLeanCode.container.getWidget("leancodecia").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.validaActionLeanCode = function(widgetLeanCode) {
        if (widgetLeanCode.getCheckedRows().length > 1) {
            widgetLeanCode.getAction("BLOQUEIA").isVisible = true;
            widgetLeanCode.getAction("DESBLOQUEIA").isVisible = true;
            widgetLeanCode.getAction("ALTERNA").isVisible = false;
        } else {
            widgetLeanCode.getAction("ALTERNA").isVisible = true;
            widgetLeanCode.getAction("BLOQUEIA").isVisible = false;
            widgetLeanCode.getAction("DESBLOQUEIA").isVisible = false;
        }
    };

    this.alteraLeanCode = function(widgetLeanCode, valor) {
        var datasetToSave = [];
        widgetLeanCode.getCheckedRows().forEach(function(row) {
            row.IDBLOQUEIA = valor;
            datasetToSave.push(row);
        });
        restEngine.post("coc#/crud/leancodecia/save", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            widgetLeanCode.reload();
            widgetLeanCode.activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.alternaLeanCode = function(widgetLeanCode) {
        var datasetToSave = [];
        widgetLeanCode.getCheckedRows().forEach(function(row) {
            if (row.IDBLOQUEIA == "S") {
                row.IDBLOQUEIA = "N";
            } else {
                row.IDBLOQUEIA = "S";
            }
            datasetToSave.push(row);
        });
        restEngine.post("coc#/crud/leancodecia/save", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            widgetLeanCode.reload();
            widgetLeanCode.activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.vinculaCompanhiaLeanCode = function(popupLeanCode) {
        var datasetToSave = [];
        popupLeanCode.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            row.IDBLOQUEIA = "N";
            datasetToSave.push(row);
        });
        restEngine.post("coc#/crud/leancodecia/save", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            ScreenService.closePopup();
            popupLeanCode.container.getWidget("leancodecia").reload();
            popupLeanCode.container.getWidget("leancodecia").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.atualizaLogocia = function(popupLogocia) {
        var datasetToSave = [
            {
                PHOTO_VIEW: popupLogocia.currentRow.PHOTO_VIEW,
                FILE_NAME: popupLogocia.currentRow.FILE_NAME,
                NOVA: !popupLogocia.container.getWidget("logocia").getField("DSLOGOCIA").isVisible,
                CDCIA: popupLogocia.dataSourceFilter[0].value
            }
        ];
        restEngine.post("coc#/logocia/upload", {
            "requestType": "DataSet",
            "dataset": datasetToSave
        }).then(function() {
            ScreenService.closePopup();
            popupLogocia.container.getWidget("logocia").reload();
            popupLogocia.container.getWidget("logocia").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.desvinculaCompanhiaFilial = function(widgetUnidade) {
        var msg = "Deseja desvincular da companhia a(s) unidades(es) selecionada(s)?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), msg, function() {
            restEngine.post("coc#/crud/ciafili/remove", {
                "requestType": "DataSet",
                "dataset": widgetUnidade.getCheckedRows()
            }).then(function() {
                widgetUnidade.reload();
                widgetUnidade.activate();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.desvinculaCompanhiaLeanCode = function(widgetLeanCode) {
        var msg = "Deseja desvincular da companhia o(s) Lean Code(s) selecionado(s)?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), msg, function() {
            restEngine.post("coc#/crud/leancodecia/remove", {
                "requestType": "DataSet",
                "dataset": widgetLeanCode.getCheckedRows()
            }).then(function() {
                widgetLeanCode.reload();
                widgetLeanCode.activate();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.limpaLogocia = function(widgetLogocia) {
        var msg = "Deseja remover o arquivo de imagem selecionado?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), msg, function() {
            restEngine.post("coc#/logocia/limpa", {
                "requestType": "Row",
                "row": {
                    "CDCIA": widgetLogocia.dataSourceFilter[0].value,
                    "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB"),
                    "__is_new": false
                }
            }).then(function() {
                widgetLogocia.reload();
                widgetLogocia.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    this.onUploadPhoto = function(widget, photoUpload) {
        setTimeout(function() {
            if (isEmpty(photoUpload)) {
                self.onUploadPhoto(widget, photoUpload);
            }
            widget.currentRow.PHOTO_VIEW = photoUpload[0].b64File;
            widget.currentRow.FILE_NAME = photoUpload[0].name;
            templateManager.updateTemplate();
        }, 500);
    };

    var isEmpty = function(value) {
        return !value || value.length === 0;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00700_cadastrocompanhiaController", coc00700_cadastrocompanhiaController);
});

// FILE: ./assets/js/controllers/coc00710_agrupcompanhiaController.js
var coc00710_agrupcompanhiaController = function(ScreenService, InflightorGeneral, coc00710_agrupcompanhiaService) {

    this.openSwipeCia = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.container.getWidget("grid_companhias").isVisible = true;
            widget.container.getWidget("grid_companhias").dataSourceFilter = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    name: "NRGRUPOCIA",
                    operator: "=",
                    value: widget.currentRow.NRGRUPOCIA
                }
            ];
            widget.container.getWidget("grid_companhias").reload();
        } else {
            widget.container.getWidget("grid_companhias").isVisible = false;
        }
    };

    this.addAgrupamento = function(widget) {
        InflightorGeneral.newRow(widget);
        if (widget.mode == "edit") {
            widget.container.getWidget("grid_companhias").isVisible = false;
        }
    };

    this.editAgrup = function(widget) {
        widget.edit();
        widget.container.getWidget("grid_companhias").isVisible = false;
    };

    this.salvaAgrup = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            widget.container.getWidget("grid_companhias").isVisible = true;
        });
    };

    this.removeVariasRows = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            widget.removeCheckedRows().then(
                function success() {
                    return ScreenService.i18n("Excluído com sucesso!");
                },
                function error(error) {
                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                        let msg = ScreenService.i18n("Exclusão não permitida. Registro filho encontrado!");
                        ScreenService.showMessage(msg);
                    } else if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente");
                        ScreenService.showMessage(msg);
                    }
                }
            );
        });
    };

    this.salvaCompanhias = function(widget) {
        if (widget.isValid()) {
            widget.getField("NMCIA").selectWidget.getCheckedRows().forEach(function(row) {
                row.NRGRUPOCIA = widget.container.getWidget("grid_agrupcia").currentRow.NRGRUPOCIA;
                row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                row.IDATIVO = "S";
                row.__is_new = true;
            });
            var send = widget.getField("NMCIA").selectWidget.getCheckedRows();
            coc00710_agrupcompanhiaService.salvaCompanhias(send).then(
                function success() {
                    ScreenService.successNotification(ScreenService.i18n("Salvo com sucesso!"));
                    ScreenService.closePopup();
                    widget.container.getWidget("grid_companhias").reload();
                },
                function error(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                }
            );
        }
    };

    this.backWidgetCompanhias = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.container.getWidget("grid_agrupcia").activate();
        });
    };

    this.FieldOnChangeNmcia = function(field) {
        field.dataSourceFilter = [
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                name: "NRGRUPOCIA",
                operator: "=",
                value: field.widget.container.getWidget("grid_agrupcia").currentRow.NRGRUPOCIA
            }
        ];
        field.reload();
    };

    this.openPopupAddCias = function(widget) {
        ScreenService.openPopup(widget.container.getWidget("widget_addCia"));
        widget.container.getWidget("widget_addCia").currentRow = [];
        widget.container.getWidget("widget_addCia").getField("NMCIA").clearValue();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00710_agrupcompanhiaController", coc00710_agrupcompanhiaController);
});

// FILE: ./assets/js/controllers/coc00800_personalizacompanhiaController.js
var coc00800_personalizacompanhiaController = function(TabelaPrecosService, MessageService, FilterUtil, InflightorGeneral, restEngine, Colors, ZHPromise, ScreenService, templateManager) {

    var self = this;
    var cdfilial;

    this.addEquip = function(field) {
        field.widget.activate();
        field.widget.newRow();
        field.widget.getField("SGEQUIP").readOnly = false;
        field.widget.widgets[0].isVisible = false;
    };

    this.savePrefixo = function(widget) {

        if (widget.parent.currentRow.QTTOTPAX < parseInt(widget.currentRow.QTTOTPAXPREF) || widget.parent.currentRow.QTVIATEQUI < parseInt(widget.currentRow.QTVIATPRE)) {

            var msg = "A quantidade total de passageiros ou viaturas do prefixo excede a da aeronave. Deseja continuar?";
            ScreenService.confirmMessage(msg, "question", function() {
                InflightorGeneral.saveCrud(widget);
            });
        } else InflightorGeneral.saveCrud(widget);

    };

    this.filterPrefixo = function(widget) {

        var filtro = widget.currentRow;
        var filtroEquip = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": filtro.NRORG
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filtro.CDFILIAL
            },
            {
                "name": "CDEQUIP",
                "operator": "=",
                "value": filtro.CDEQUIP
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": filtro.CDCIA
            }
        ];
        widget.widgets[0].dataSourceFilter = filtroEquip;
        widget.widgets[0].reload();
    };

    this.desvincularCia = function(widget) {
        var msg = "Deseja desvincular da filial a(s) " + widget.getCheckedRows().length + " companhia(s) selecionada(s)?";
        ScreenService.confirmMessage(msg, "question", function() {
            InflightorGeneral.callRestEngine("coc#/crud/ciafili_companhia_ciclo/remove", "DataSet", widget.getCheckedRows()).then(function() {
                MessageService.showMessageByCode("COC00800_DESCONTINUED_CIA");
                widget.reload();
                widget.unSelectAll();
            }, function() {
                MessageService.showMessageByCode("COC00800_CIA_SELECTED_ERROR");
            });
        });

    };

    function getMainWidget() {
        return templateManager.container.getWidget('coc00800_personalizacompanhia');
    }

    this.addPopup = function(widget) {
        var dataset = [];
        const mainWidget = getMainWidget();
        const filterRow = mainWidget.appliedFilter.FILTER_ROW;
        var rows = widget.dataSource.checkedRows;
        rows.forEach(function(obj) {
            dataset.push({
                "CDFILIAL": filterRow.CDFILIAL,
                "CDCIA": obj.CDCIA,
                "DSROTACICLO": null,
                "CDCICLO": null,
                "VRPERCONVOO": null,
                "__is_new": true,
                "changed": true
            });
        });

        InflightorGeneral.callRestEngine("coc#/crud/companhia_ciafili_naovinculadas/save", "DataSet", dataset).then(function() {
            MessageService.showMessageByCode("COC00800_CIA_LINKED");
            ScreenService.closePopup();
            widget.reload();
            widget.container.widgets[1].reload();
        }, (error) => {
            ScreenService.showMessage(error.data.error, 'error');
        });

    };

    this.backPopup = function() {
        ScreenService.closePopup();
    };

    function setVisibilityComponents(widget) {
        widget.getAction('ADD').isVisible = true;
    }

    this.filterPopup = function(widget) {
        var cdfilial = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widget.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }
        ];
        setVisibilityComponents(widget);

    };

    this.filterCiclo = function(field) {
        var row = field.widget.currentRow;
        var filter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": row.NRORG
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            }
        ];
        field.dataSourceFilter = filter;
    };

    this.setCdCiainFieldCiclo = function(widget) {
        FilterUtil.updateFilter(widget.getField("IDCICLO"), widget.currentRow.CDCIA, "CDCIA");
        widget.getField("IDCICLO").reload();
    };

    this.setFilialbyFilter = function(widget) {
        cdfilial = widget.currentRow.CDFILIAL;
    };

    this.cancelaAlteracoes = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        templateManager.updateTemplate();
    };

    this.saveAlteracoes = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            widget.commitInlineEdit();
            widget.list();
            widget.activate();
            templateManager.updateTemplate();
        });
    };

    this.deleteRows = function(widget) {
        widget.currentRow = widget.dataSource.checkedRows;
        var deferred = ZHPromise.defer();
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            widget.removeRow().then(function() {
                widget.reload().then(function() {
                    widget.activate();
                });
                deferred.resolve();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

    this.deleteEquipamento = function(args) {
        var widget = args.owner.widget;
        var deferred = ZHPromise.defer();
        var rowToRemove = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "CDEQUIP",
                "operator": "=",
                "value": widget.currentRow.CDEQUIP
            }
        ];
        args.owner.dataset = rowToRemove;
        var remove = args.owner.dataset;
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            remove.removeRow().then(function() {
                widget.reload().then(function() {
                    widget.activate();
                });
                deferred.resolve();
                ScreenService.closeSwipe();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

    this.isVisibleforEdit = function(args) {
        args.owner.owner.widget.getAction('BACK').showAsAction = "back_view";
    };

    this.cancelaAlteracoes = function(widget) {
        widget.getAction('BACK').showAsAction = "back_view|back";
        widget.rollbackInlineEdit();
        widget.list();
        templateManager.updateTemplate();
    };

    this.saveAlteracoes = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            widget.commitInlineEdit();
            widget.list();
            widget.activate();
            templateManager.updateTemplate();
            widget.getAction('BACK').isVisible = true;
        });
    };

    this.FieldOnChangeFiltro = function(args) {
        var widget = args.owner.owner.widget;
        if (widget.mode != "edit") {
            if (args.row.CDFILIAL) {
                widget.getField("SGCIA").dataSource.name = "coc#/crud/ciafili_companhia";
                if (!widget.getField("SGCIA").dataSourceFilter[1])
                    FilterUtil.updateFilter(widget.getField("SGCIA"), cdfilial, "CDFILIAL");
                widget.getField("SGCIA").reload();
            } else {
                widget.getField("SGCIA").dataSourceFilter[1].value = "";
                widget.getField("SGCIA").reload();
                MessageService.showMessageByCode("SELECT_UNIT");
            }
        } else {
            widget.getField("SGCIA").dataSource.name = "coc#/crud/companhia";
            widget.getField("SGCIA").dataSourceFilter.pop();
            widget.getField("SGCIA").reload();
            widget.currentRow.CDFILIAL = cdfilial;
        }
    };

    this.setCiagrupoInRow = function(widget) {
        var currentRow = widget.currentRow;
        if (currentRow.QTPRODSU)
            currentRow.QTPRODSU = InflightorGeneral.valueToFloat(currentRow.QTPRODSU);
        if (currentRow.QTRECSUB)
            currentRow.QTRECSUB = InflightorGeneral.valueToFloat(currentRow.QTRECSUB);
        if (!currentRow.CDCIA)
            currentRow.CDCIA = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDCIA");
        if (!currentRow.CDFILIAL)
            currentRow.CDFILIAL = getMainWidget().appliedFilter.FILTER_ROW.CDFILIAL;
        if (widget.name == "prefixo")
            currentRow.CDEQUIP = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDEQUIP");
    };

    this.setVisibleforTabbed = function(widgets, value, exception) {
        widgets.forEach(function(widget) {
            if (exception) {
                if (exception.name != widget.name)
                    widget.isVisible = value;
            } else {
                widget.isVisible = value;
            }
            if (widget.name == "FILTER")
                widget.isVisible = false;
        });
    };

    this.filterRowsFromSelect = function(field, primaryKey) {
        var widget = field.widget;
        var data = widget.dataSource.data;
        var keyArray = data.map(function(currentItem) {
            return currentItem[primaryKey];
        });
        keyArray.pop();
        if (keyArray.length > 0)
            FilterUtil.updateFilter(field, keyArray, primaryKey, "NOT_IN");
        field.reload();
    };

    this.readOnlySigla = function(widget, value) {
        widget.getField("SGCIA").readOnly = value;
    };

    //Rotina para manipulação de Cores no field NRCORCICCIA para a Tabbed Ciclo 

    this.setCicloColor = function(element, row) {
        if (row) {
            setTimeout(function() {
                if (element.hasClass("td zh-standard-column  grid-align-left"))
                    element.css("background-color", row.NRCORCICCIA);
                element.children("span").remove();
                element.append("<div class='rectangle'></div>");
                element.find("p").css("background-color", row.NRCORCICCIA);
            }, 100);
        }
    };

    this.setCicloColorAdd = function(element, row) {
        if (row) {
            setTimeout(function() {
                if (element.hasClass("td zh-standard-column  grid-align-left"))
                    element.css("background-color", row.NRCORCICLO);
                element.children("span").remove();
                element.append("<div class='rectangle'></div>");
                element.find("p").css("background-color", row.NRCORCICLO);
            }, 100);
        }
    };

    this.afterMoveRow = function(row) {

        row.CORCICLO = " ";
        setTimeout(function() {
            if (row.NRCORCICCIA) {
                if (row.NRCORCICCIA.toString().charAt(0) != "#")
                    row.NRCORCICCIA = Colors.TColorToHex(row.NRCORCICCIA);
                $("#CORCICLO > div").css("background-color", row.NRCORCICCIA);
            }
        }, 150);

    };

    this.saveCiclo = function(widget) {

        if (widget.isValid()) {
            var r, g, b;
            if (widget.currentRow.NRCORCICCIA.toString().charAt(0) != "#") {
                r = widget.currentRow.NRCORCICCIA._r.toString();
                g = widget.currentRow.NRCORCICCIA._g.toString();
                b = widget.currentRow.NRCORCICCIA._b.toString();
                if (r.indexOf(".") != -1)
                    r = r.substr(0, r.indexOf("."));
                if (g.indexOf(".") != -1)
                    g = g.substr(0, g.indexOf("."));
                if (b.indexOf(".") != -1) {
                    b = b.substr(0, b.indexOf("."));
                }
                r = parseInt(r);
                g = parseInt(g);
                b = parseInt(b);

                widget.currentRow.NRCORCICCIA = r | (g << 8) | (b << 16);
            } else {
                var h = widget.currentRow.NRCORCICCIA.substring(1, 7);
                r = parseInt((h).substring(0, 2), 16);
                g = parseInt((h).substring(2, 4), 16);
                b = parseInt((h).substring(4, 6), 16);
                widget.currentRow.NRCORCICCIA = r | (g << 8) | (b << 16);
            }
            self.saveCrud(widget);
        }

    };

    this.saveCrud = function(widget) {
        var deferred = ZHPromise.defer();
        if (widget.isValid()) {
            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
            }

            widget.saveRow().then(function() {
                if (widget.currentRow.__is_new) {
                    widget.list(true);
                    ScreenService.goBack();
                    widget.reload();
                } else {
                    widget.view();
                }
                self.afterMoveRow(widget.currentRow);
                widget.activate();
                deferred.resolve();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
                deferred.reject(errorMessage);
            });
        } else {
            // @todo create messages class
            deferred.reject("Preencha todos os campos corretamente!");
        }

        return deferred.promise;
    };

    this.RGBToTColor = function(r, g, b) {
        var Tcolor = r | (g << 8) | (b << 16);
        return Tcolor;
    };

    this.changeColor = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.NRCORCICCIA.toString().charAt(0) != "#")
                row.NRCORCICCIA = Colors.TColorToHex(row.NRCORCICCIA);
        });

    };

    this.changeColorAdd = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.NRCORCICLO.toString().charAt(0) != "#")
                row.NRCORCICLO = Colors.TColorToHex(row.NRCORCICLO);
        });

    };

    this.save = function(widget) {

        restEngine.post("coc#/ciafili_companhia_ciclo/save", {
            "requestType": "Row",
            "row": widget.currentRow
        }).then(function() {
            self.setVisibleforTabbed(widget.widgets, true, widget);
            widget.reload();
            widget.view();
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.saveRows = function(widget) {
        var rows = widget.dataSource.getCheckedRows();
        const filterRow = getMainWidget().appliedFilter.FILTER_ROW;
        var currentParent = widget.container.getWidget("coc00800_personalizacompanhia").currentRow;
        var data = [];
        var widgetSave;
        if (rows.length === 0) {
            MessageService.showMessageByCode("COC00800_ADD_CLASS");
        } else {
            rows.forEach(function(row) {
                if (widget.name == "selectClasse") {
                    data.push({
                        "CDFILIAL": filterRow.CDFILIAL,
                        "CDCIA": currentParent.CDCIA,
                        "CDCLASSE": row.CDCLASSE,
                        "DSCLASSECIA": row.DSCLASSE,
                        "__is_new": true,
                        "changed": true
                    });
                } else {
                    data.push({
                        "CDFILIAL": filterRow.CDFILIAL,
                        "CDCIA": currentParent.CDCIA,
                        "CDCICLO": row.CDCICLO,
                        "IDCICLO": row.IDCICLO,
                        "NRCORCICLO": row.NRCORCICLO,
                        "NRCORCICCIA": row.NRCORCICLO,
                        "__is_new": true,
                        "changed": true
                    });
                }
            });
            if (widget.name == "selectClasse")
                widgetSave = widget.container.getWidget("classe");
            if (widget.name == "selectCiclo")
                widgetSave = widget.container.getWidget("ciclo");
            widgetSave.dataSource.data = data;
            widgetSave.dataSource.save().then(function() {
                widgetSave.reload();
                widgetSave.activate();
                ScreenService.closePopup();
                rows = [];

            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
        }
    };

    this.findCia = function(args) {
        var fieldCia = args.row;
        var widgetPersonCia = args.owner.getParent("coc00800_personalizacompanhia");
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": fieldCia.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": 1
            }
        ];
        widgetPersonCia.dataSourceFilter = filter;
        widgetPersonCia.reload();
    };

    //funcao para carregar os dados de outra row quando a view estiver aberta 
    this.changeTabs = function(args) {
        args.owner.widgets.forEach(function(widget) {

            widget.dataSourceFilter = [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": args.row.NRORG
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": args.row.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": args.row.CDCIA
                }
            ];

            widget.reload();
        });
    };

    this.openPopupCiclo = function(widget) {
        openLinkPopup(widget, "ciclo");
    };

    this.openPopupClasse = function(widget) {
        openLinkPopup(widget, "classe");
    };

    var openLinkPopup = function(widget, popupKey) {
        var widgetCia = widget.container.getWidget("coc00800_personalizacompanhia");
        var ciaFilter = getCiaFilter(widgetCia);

        var popupItens = getLinkPopupItens(widget, popupKey);

        var popupWidget = popupItens.popupWidget;
        var tableName = popupItens.tableName;
        var tableRoute = popupItens.tableRoute;
        var tableKey = popupItens.tableKey;

        InflightorGeneral.callRestEngine(tableRoute, "FilterData", ciaFilter).then(
            function(callback) {
                var linkedItens = [];
                var data = callback.dataset[tableName];
                for (let index in data) {
                    linkedItens.push(data[index][tableKey]);
                }
                var filter = [
                    {
                        name: tableKey,
                        operator: "NOT_IN",
                        value: linkedItens
                    }
                ];
                popupWidget.dataSourceFilter = (linkedItens.length > 0) ? filter : [];
                popupWidget.reload().then(
                    function() {
                        ScreenService.openPopup(popupWidget);
                        popupWidget.unSelectAll();
                    }
                );
            }
        );
    };

    var getCiaFilter = function(widgetCia) {
        var ciaFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: widgetCia.currentRow.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: widgetCia.currentRow.CDCIA
            }
        ];
        return ciaFilter;
    };

    var getLinkPopupItens = function(widget, popupKey) {
        var popupItens = {
            popupWidget: {},
            tableName: "",
            tableRoute: "",
            tableKey: ""
        };
        switch (popupKey) {
            case "ciclo":
                popupItens.popupWidget = widget.container.getWidget("selectCiclo");
                popupItens.tableName = "ciclocia";
                popupItens.tableRoute = "coc#/crud/ciclocia";
                popupItens.tableKey = "CDCICLO";
                break;
            case "classe":
                popupItens.popupWidget = widget.container.getWidget("selectClasse");
                popupItens.tableName = "classecia";
                popupItens.tableRoute = "coc#/crud/classecia";
                popupItens.tableKey = "CDCLASSE";
                break;
        }
        return popupItens;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00800_personalizacompanhiaController", coc00800_personalizacompanhiaController);
});

// FILE: ./assets/js/controllers/coc00900_frequenciaciclogeralController.js
function coc00900_frequenciaciclogeralController(ScreenService, templateManager, coc00900_frequenciaciclogeralService, Colors, restEngine, InflightorGeneral, MessageService) {
    var Filial;
    var cdCia;
    var self = this;
    var dateStart;
    var intervalCiclo;
    var data = [];
    var dateIn;
    var dateEnd;
    var date;
    var identificador;

    var mensagensGlobais = {};

    this.getMensagensGlobais = function() {
        MessageService.getMessagesByCode(['COC00900_DOM', 'COC00900_SEG', 'COC00900_TER',
                                          'COC00900_QUA', 'COC00900_QUI', 'COC00900_SEX', 'COC00900_SAB']).then(function(mensagens) {
            mensagensGlobais.dom = mensagens[0];
            mensagensGlobais.seg = mensagens[1];
            mensagensGlobais.ter = mensagens[2];
            mensagensGlobais.qua = mensagens[3];
            mensagensGlobais.qui = mensagens[4];
            mensagensGlobais.sex = mensagens[5];
            mensagensGlobais.sab = mensagens[6];
        });
    };
    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("COC00900_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    this.selecionaCompanhia = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("calendar");

        Filial = current.CDFILIAL;
        cdCia = current.CDCIA;

        gridWidget.getActions()[0].isVisible = true;
        gridWidget.activate();
        gridWidget.reload();

    };

    this.initDefaultMensal = function(widget) {
        self.setDataSelectDate(widget);
        widget.getField("TROCAINICIAL").applyDefaultValue();
        widget.currentRow.TROCAFINAL = mensagensGlobais.dom;
        widget.currentRow.id = 1;
        widget.currentRow.idtroca = 0;
    };

    this.initDefaultSemanal = function(widget) {
        self.setDataSelectDate(widget);
        widget.currentRow.TROCASEMANAL = mensagensGlobais.dom;
    };

    this.setDataSelectDate = function(widget) {
        var data = [
            {
                "value": mensagensGlobais.dom,
                "id": 0
            },
            {
                "value": mensagensGlobais.seg,
                "id": 1
            },
            {
                "value": mensagensGlobais.ter,
                "id": 2
            },
            {
                "value": mensagensGlobais.qua,
                "id": 3
            },
            {
                "value": mensagensGlobais.qui,
                "id": 4
            },
            {
                "value": mensagensGlobais.sex,
                "id": 5
            },
            {
                "value": mensagensGlobais.sab,
                "id": 6
            }
        ];
        if (widget.name == "frequenciaciclogeral_semanal")
            widget.getField("TROCASEMANAL").dataSource.data = data;
        else {
            widget.getField("TROCAFINAL").dataSource.data = data;
        }
    };
    this.filtraRegistros = function(widget, year) {
        // var year = widget.getYear();
        if (!year) {
            var newYear = new Date();
            self.filtraCalendar(widget, newYear.getFullYear());
        }
        self.filtraCalendar(widget, year);
    };

    this.setFilterForField = function(field) {
        var widget = field.widget;
        field.dataSourceFilter[1].value = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow.CDCIA;
        field.reload();
    };

    this.openSwipe = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
    };

    this.setYearForField = function(widget) {
        widget.fields[0].setValue(widget.currentRow.YEARS);
        widget.fields[0].reload();
        self.filtraCalendar(widget, widget.currentRow.YEARS);
    };

    this.openAssociacao = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow = widget.currentRow;
        ScreenService.closePopup(widget.container.getWidget("associarFrequencia"));
        swipeWidget.parent = widget.container.getWidget("associarFrequencia");
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        swipeWidget.activate();
        swipeWidget.widgets[0].edit();
        swipeWidget.fields[0].applyDefaultValue();
        swipeWidget.fields[1].applyDefaultValue();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
        swipeWidget.parent = widget.container.getWidget("coc00900_frequenciaciclogeral");
    };

    this.associarFrequencia = function(widget) {
        ScreenService.openPopup(widget.container.getWidget("associarFrequencia"));
        widget.container.getWidget("associarFrequencia").activate();
        widget.container.getWidget("associarFrequencia").reload();

    };

    this.setModeYear = function(widget) {
        var calendarMode = widget.getField("year").calendarMode;
        let widgetCalendar = widget.container.factoredWidgets.filter(function(factored) {
            return factored.name === "Widget" && factored.template == "widget/calendar.html";
        });
        widgetCalendar.forEach(function(calendar) {
            calendar.calendarMode = calendarMode;
        });
    };

    this.filtraCalendar = function(widget, year) {
        var calendar = widget.container.getWidget("calendar");
        var data = [];
        if (year) {
            // data = self.preencheCrossTab(setDataSource);
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00900_frequenciaciclogeralService.findFrequenciaCiclo(Filial, cdCia, dateInit, dateFinal).then(function(callback) {

                var ciclos = callback.dataset.frciclocia_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                // ciclos.forEach(function(dataset){
                // 	var dateFormat = moment(dataset.DTCICLOCIA.slice(0,10), "DD/MM/YYYY");
                // 	data.push({
                // 		EIXO_X: dataset.EIXO_X,
                // 		EIXO_Y: moment(dataset.EIXO_Y, 'MM').format('MMM'),
                // 		NRCORCICLO: Colors.TColorToHex(dataset.NRCORCICLO),
                // 		IDCICLO: dataset.IDCICLO
                // 	});

                // 	// dateFormat = ;
                // 	// setDataSource.TITLE = dataset.CDCICLO;
                // 	// setDataSource.DATE = angular.copy(dateFormat);
                // });

                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        } //if year
    };

    function filtraCicloBanco(dia, mes, ciclos) {
        return ciclos.filter(function(ciclo) {
            return ciclo.EIXO_X == dia && ciclo.EIXO_Y == mes;
        });
    }

    function pad(value, length) {
        return (value.toString().length < length) ? pad("0" + value, length) : value;
    }

    this.preencheCrossTab = function(setDataSource) {
        var day = 1;
        var month = 1;
        var data = [];
        for (let aux = 0; aux < 372; aux++) {
            setDataSource.EIXO_X = day;
            setDataSource.EIXO_Y = month;
            setDataSource.NRCORCICLO = "";
            setDataSource.IDCICLO = "";
            day++;
            if (day > 31) {
                day = 1;
                month++;
            }
            data.push(setDataSource);
            setDataSource = {};
        }
        return data;
    };

    this.closeSwipe = function(widget) {
        var parentWidget = widget.container.getWidget("coc00900_frequenciaciclogeral");
        ScreenService.closeSwipe().then(function() {
            parentWidget.activate();
        });
    };

    this.addCiclo = function(widget) {

        var gridWidget = widget.container.getWidget("popup_cadastro");

        var widgetsNames = ["diario", "semanal", "quinzenal", "mensal"];

        widgetsNames.forEach(function(nome) {
            var currentWidget = widget.container.getWidget("frequenciaciclogeral_" + nome);
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
            currentWidget.getField("MULTIPLECICLO").reload();
            currentWidget.newRow();
        });
        ScreenService.openSwipe(gridWidget);

    };
    this.openExclui = function(widget) {

        var formWidget = widget.container.getWidget("popup_excluir");
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
        formWidget.getField("MULTIPLECICLO").reload();
        formWidget.newRow();
        formWidget.reload();
        ScreenService.openPopup(formWidget);

    };

    this.cancelExcluir = function(widget) {
        ScreenService.closePopup();
        var swipeWidget = widget.container.getWidget("calendar");
        swipeWidget.activate();
    };
    this.cancelAdd = function(widget) {
        ScreenService.closeSwipe();
        var swipeWidget = widget.container.getWidget("calendar");
        swipeWidget.activate();
        // swipeWidget.list();
    };

    this.reloadDias = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                valueDay.setValue(0);
                valueDay.reload();
            }
        }

    };

    this.validaQtdDias = function(widget) {

        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                dateIn = date.start;
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;
            }
        }
    };

    this.setFnQuinzenal = function(widget) {

        var interval = widget.getField("INTERVAL");
        var date = interval.value();
        var valueTroca = widget.getField("INQUINZENAL").value();
        if (valueTroca) {
            widget.getField("FNQUINZENAL").setValue(valueTroca + 15);
            widget.getField("FNQUINZENAL").reload();
        }
    };
    this.subDate = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                //Verifica o maior valor valido de acordo com o mês
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;

                //Diferença entre DateIn e DateFn 
                var finalDate = moment(dateFn, "DD/MM/YYYY");
                var startDate = moment(dateIn, "DD/MM/YYYY");
                var difDate = finalDate.diff(startDate, "days");
                if (widget.name == "frequenciaciclogeral_quinzenal") {
                    if (difDate < 15) {
                        MessageService.showMessageByCode("COC00900_CICLO_QUINZENAL_PERIODO_MENOR");
                    }
                }
                if (widget.name == "frequenciaciclogeral_mensal" && difDate < 29) {
                    MessageService.showMessageByCode("COC00900_CICLO_MENSAL_PERIODO_MENOR");
                }
            }
        }

    };

    this.filtraTroca = function(widget) {

        var filtraRADIO = widget.getField("RADIOTROCA");
        var semanaInField = widget.getField("TROCAINICIAL");
        var semanaFnField = widget.getField("TROCAFINAL");
        var diaField = widget.getField("INQUINZENAL");
        var itemSelect = filtraRADIO.value();

        if (itemSelect == 1) {
            //Exibe DIAMENSAL spin 
            diaField.isVisible = true;
            semanaInField.isVisible = false;
            semanaFnField.isVisible = false;

        }
        if (itemSelect == 2) {
            //Exibe TROCAINICIAL e TROCAFINAL 
            semanaInField.isVisible = true;
            semanaFnField.isVisible = true;
            diaField.isVisible = false;
        }

        filtraRADIO.reload();

    };

    this.setDefaultTab = function(tabbedWidget, name) {
        tabbedWidget.defaultTab = name;
    };

    this.iniciaValues = function(row, name) {
        dateStart = row.INTERVAL.start;
        dateEnd = row.INTERVAL.end;
        dateIn = moment(dateStart, "DD/MM/YYYY");
        dateEnd = moment(dateEnd, "DD/MM/YYYY");
        //Diferença data inicial -> data final
        intervalCiclo = dateEnd.diff(dateStart, "days");
        data = [];
        date = dateIn.format("DD/MM/YYYY");
        identificador = name;
    };

    this.salveFreqCiclo = function(widget) {
        var row = widget.currentWidget.currentRow;
        var nameWidget = widget.currentWidget.name;
        if (row.INTERVAL.start && row.INTERVAL.end && row.MULTIPLECICLO) {
            switch (nameWidget) {
                case "frequenciaciclogeral_diario":
                    if (row.DURACAOCICLODIAS) {
                        this.salvaDiario(row, widget);
                    } else {
                        MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
                    }
                    break;
                case "frequenciaciclogeral_semanal":
                    this.salvaSemanal(row, widget);
                    break;
                case "frequenciaciclogeral_quinzenal":
                    this.salvaQuinzenal(row, widget);
                    break;
                case "frequenciaciclogeral_mensal":
                    this.salvaMensal(row, widget);
                    break;
                default:
                    MessageService.showMessageByCode("ERRO_INESPERADO");
            }
        } else {
            MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.preencheData = function(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador) {
        var dtInicialQuin = moment(date, "DD/MM/YYYY");
        var dtFinalQuin = angular.copy(dtInicialQuin).add(15, "days");
        dtFinalQuin = dtFinalQuin.format("DD");
        dtInicialQuin = dtInicialQuin.format("DD");
        var permiteTroca = 0;
        for (let aux = 0; aux < intervalCiclo; aux++) {
            if (identificador == "Diario" || identificador == "Semanal") {
                if (alterCiclo == duracaoCiclo) {
                    alterCiclo = 0;
                    qtdCiclo++;
                    if (qtdCiclo == maxCiclo) {
                        qtdCiclo = 0;
                    }
                }
            } else if (identificador == "Quinzenal") {
                var alterDateQuin = moment(date, "DD/MM/YYYY");
                alterDateQuin = alterDateQuin.format("DD");
                if (permiteTroca) {
                    if (dtInicialQuin == alterDateQuin || dtFinalQuin == alterDateQuin) {
                        qtdCiclo++;
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                }
            }
            var rows = {
                CDCICLO: cdciclo[qtdCiclo],
                CDCIA: cdCia,
                CDFILIAL: Filial,
                NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                DTCICLOCIA: date,
                __is_new: true
            };
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = alterDate;
            data.push(rows);
            alterCiclo++;
            permiteTroca++;
        }
    };

    this.salvaDiario = function(row, widget) {

        self.iniciaValues(row, "Diario");

        intervalCiclo += 1;
        if (intervalCiclo < row.DURACAOCICLODIAS) {
            MessageService.showMessageByCode("COC00900_DURACAO_MAIOR_Q_PERMITIDO", {
                "0": intervalCiclo
            });
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            exibeDate = exibeDate.format("L");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var qtdCiclo = 0;
                var maxCiclo = row.MULTIPLECICLO.length;
                var duracaoCiclo = row.DURACAOCICLODIAS;
                var alterCiclo = 0;
                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaSemanal = function(row, widget) {

        self.iniciaValues(row, "Semanal");

        var editInterval = intervalCiclo;
        editInterval /= 7;
        intervalCiclo += 1;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 0;
        editInterval = Math.round(editInterval);
        var duracaoCiclo = row.DURACAOCICLO * 7;
        if (intervalCiclo < 7) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_PERIODO_MENOR");
        } else if (editInterval < row.DURACAOCICLO) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_DURACAO_MAIOR");
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            exibeDate = exibeDate.format("L");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var diaSemana = row.TROCASEMANAL.toString();
                var diaInicial = dateIn.format("d");
                while (diaSemana.toUpperCase() != diaInicial.toUpperCase()) {
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    date = moment(alterDate, "DD/MM/YYYY");
                    diaInicial = date.format("d");
                }
                diaInicial = moment(date, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(diaInicial, "days") + 1;
                date = diaInicial.format("DD/MM/YYYY");

                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaQuinzenal = function(row, widget) {

        self.iniciaValues(row, "Quinzenal");

        var dtInTroca = row.INQUINZENAL;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 1;
        var dataInicial = dateIn.format("DD");
        var duracaoCiclo = 15;

        for (let value = 2; value <= dtInTroca; value++) {
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = moment(alterDate, "DD/MM/YYYY");
            dataInicial = date.format("DD");
        }
        dataInicial = moment(date, "DD/MM/YYYY");
        intervalCiclo = dateEnd.diff(dataInicial, "days") + 1;
        date = dataInicial.format("DD/MM/YYYY");
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
            self.salvaFrequencia(data, widget);
        });
    };

    this.salvaMensalporDiaSemana = function(intervalCiclo, cdciclo, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador) {

        var qtdCiclo = 0;
        var alterCiclo = 0;
        var qtdMes = 0;
        var trocaSemana = row.id;
        var permiteTroca = 0;

        if (intervalMensal === 0 || maxCiclo == 1 || (duracaoCiclo > intervalMensal)) {
            identificador = "Mensal";
            self.preencheData(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
        } else {
            for (let aux = 0; aux <= intervalCiclo; aux++) {
                if (identificador == "MensalDia") {
                    if (trocaDia == dateTroca) {
                        qtdMes++;
                        if (qtdMes == duracaoCiclo) {
                            qtdCiclo++;
                            qtdMes = 0;
                        }
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                } else if (identificador == "MensalSemana") {
                    if (intervalMensal > 0) {
                        if (permiteTroca) {
                            var startOfMonth = alterDate.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                            var studyDate = moment(startOfMonth).add(trocaSemana, "w");
                            if (studyDate.month() == startOfMonth.month()) {
                                studyDate = studyDate.subtract(1, "w");
                            }
                            studyDate = studyDate.format("DD/MM/YYYY");
                            if (date == studyDate) {
                                qtdCiclo++;
                            }
                            if (qtdCiclo == maxCiclo) {
                                qtdCiclo = 0;

                            }
                        }
                    }
                }
                var rows = {
                    CDCICLO: row.MULTIPLECICLO[qtdCiclo],
                    CDCIA: cdCia,
                    CDFILIAL: Filial,
                    NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                    DTCICLOCIA: date,
                    __is_new: true
                };
                var newDate = moment(date, "DD/MM/YYYY");
                var alterDate = newDate.add(1, "days").format("DD/MM/YYYY");
                date = alterDate;
                alterDate = moment(alterDate, "DD/MM/YYYY");
                data.push(rows);
                var dateTroca = newDate.format("DD");
                var maxTrocaMes = moment(date, "DD/MM/YYYY");
                maxTrocaMes = moment(newDate).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(maxTrocaMes, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                intervalMensal = newDate.startOf("month").diff(dateIn.startOf("month"), "month");
                permiteTroca++;
                if (maxDayFn < trocaDia) {
                    trocaDiaAnt = trocaDia;
                    trocaDia = maxDayFn;
                } else if (maxDayFn >= row.INQUINZENAL) {
                    trocaDia = row.INQUINZENAL;
                } else {
                    trocaDia = trocaDiaAnt;
                }
            }
        }

    };

    this.salvaMensal = function(row, widget) {

        self.iniciaValues(row, "Mensal");
        var opcaoTroca = row.RADIOTROCA;
        var trocaDia = row.INQUINZENAL;
        dateStart = moment(dateStart, "DD/MM/YYYY");
        var alterDia = 0;
        var trocaDiaAnt = trocaDia;
        var duracaoCiclo = row.DURACAOCICLO;
        var maxCiclo = row.MULTIPLECICLO.length;
        var intervalMensal = 0;
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            if (opcaoTroca == 1) {
                //Troca Diário 
                while (trocaDia != alterDia) {
                    var newDate = moment(dateIn, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    dateIn = alterDate;
                    alterDia = newDate.format("DD");
                    var monthInterval = moment(dateIn, "DD/MM/YYYY");
                    var month = moment(monthInterval, "DD/MM/YYYY");
                    monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                    var maxDay = moment(monthInterval, "DD/MM/YYYY");
                    var maxDayFn = maxDay.format("DD");
                    if (trocaDia >= maxDayFn) {
                        trocaDia = maxDayFn;
                    }

                }
                dateIn = moment(dateIn, "DD/MM/YYYY");
                dateIn = dateIn.subtract(1, "days").format("DD/MM/YYYY");
                dateIn = moment(dateIn, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(dateIn, "days") + 1;
                date = dateIn.format("DD/MM/YYYY");
                intervalMensal = dateEnd.diff(dateStart, "month");
                identificador = "MensalDia";

                self.salvaMensalporDiaSemana(intervalCiclo, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador);

            } else if (opcaoTroca == 2) {
                //Troca Semanal 
                /* row.id -> 1 - Primeiro(a)
2 -  Segundo(a)
3 - Terceiro(a)
4 -   Quarto(a)
*/
                var trocaSemana = row.id;
                var startOfMonth = angular.copy(dateIn);
                startOfMonth.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                var studyDate = moment(startOfMonth.format("DD/MM/YYYY"), "DD/MM/YYYY").add(trocaSemana, "w");
                if (studyDate.month() == startOfMonth.month()) {
                    studyDate = studyDate.subtract(1, "w");
                }
                var verificaDate = studyDate.format("DD/MM/YYYY");
                verificaDate = moment(verificaDate, "DD/MM/YYYY");
                if (verificaDate < dateIn) {
                    dateIn = dateIn.add(1, "month");
                    var startOfMonthIf = dateIn.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                    var studyDateIf = moment(startOfMonthIf).add(trocaSemana, "w");
                    if (studyDateIf.month() == startOfMonthIf.month()) {
                        studyDateIf = studyDateIf.subtract(1, "w");
                    }
                }

                intervalMensal = dateEnd.diff(studyDate, "month");
                date = studyDate.format("DD/MM/YYYY");
                identificador = "MensalSemana";
                self.salvaMensalporDiaSemana(intervalCiclo + 1, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador);

            }
            self.salvaFrequencia(data, widget);
        });

    };

    this.salvaFrequencia = function(data, widget) {
        coc00900_frequenciaciclogeralService.novaFrequenciaCiclo(data).then(function() {
            MessageService.showMessageByCode("COC00900_CAD_AUTO_SUCES");
            var setWidget = widget.container.getWidget("calendar");
            ScreenService.closeSwipe();
            self.filtraRegistros(setWidget, setWidget.currentRow.year);
            setWidget.reload().then(function() {
                setWidget.activate();
            });
            widget.container.getWidget("coc00900_frequenciaciclogeral").reload();
        }, function(calback) {
            if (calback.data)
                trataMensagem(calback.data.error);
        });
    };

    this.preencheDataExcluir = function(widget) {
        var dataset = [];
        var currentRow = widget.currentRow;
        var swipeWidget = widget.container.getWidget("calendar");
        self.iniciaValues(currentRow, widget.name);
        intervalCiclo += 1;
        if (!currentRow.INTERVAL) {
            MessageService.showMessageByCode("COC00900_PREENCHA_INTERVALO_EXCL");
        } else if (!currentRow.MULTIPLECICLO) {
            MessageService.showMessageByCode("COC00900_CICLOS_N_INF");
        } else {
            if (currentRow.MULTIPLECICLO.length == 1) {
                for (let aux = 0; aux < intervalCiclo; aux++) {

                    var rows = {
                        CDCICLO: currentRow.MULTIPLECICLO[0],
                        CDCIA: cdCia,
                        CDFILIAL: Filial,
                        NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                        DTCICLOCIA: date,
                        __is_new: false
                    };
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("L");
                    date = alterDate;
                    dataset.push(rows);
                }
            } else {
                var qtdCiclo = currentRow.MULTIPLECICLO.length;
                qtdCiclo--;
                var ciclo = currentRow.MULTIPLECICLO;
                while (qtdCiclo >= 0) {
                    date = dateIn.format("DD/MM/YYYY");
                    for (let cont = 0; cont <= intervalCiclo; cont++) {

                        var row = {
                            CDCICLO: ciclo[qtdCiclo],
                            CDCIA: cdCia,
                            CDFILIAL: Filial,
                            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                            DTCICLOCIA: date,
                            __is_new: false

                        };
                        var newDateQtd = moment(date, "DD/MM/YYYY");
                        var alterDateQtd = moment(newDateQtd).add(1, "days").format("DD/MM/YYYY");
                        date = alterDateQtd;
                        dataset.push(row);
                    }
                    self.iniciaValues(currentRow, widget.name);
                    qtdCiclo--;
                }
            }
            self.excluiFrequencia(dataset, swipeWidget, dateStart, dateEnd);
        }
    };

    this.excluiFrequencia = function(data, widget, dateStart, dateEnd) {
        var exibeDate = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(dateEnd, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_EXCL_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            coc00900_frequenciaciclogeralService.excluiFrequenciaCiclo(data).then(function() {
                MessageService.showMessageByCode("COC00900_EXCL_AUTO_SUCES");
                ScreenService.closePopup();
                var setWidget = widget.container.getWidget("calendar");
                self.filtraRegistros(widget, setWidget.currentRow.year);
                setWidget.reload().then(function() {
                    setWidget.activate();
                });
                widget.container.getWidget("coc00900_frequenciaciclogeral").reload();
            }, function(calback) {
                if (calback.data)
                    trataMensagem(calback.data.error);
            });
        });
    };

    this.setCicloColor = function(element, row) {
        if (row.NRCORCICLO) {
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            //var status = Util.isUndefinedOrNull(row.importStatus)? row.status : row.importStatus;
            //var hasError = Util.isUndefinedOrNull(row.importHasError)? row.hasError : row.importHasError;
            //var grid = $(".th.grid-align-left");
            // for (let i in grid) {
            // 	if (typeof grid[i] == "object") {
            // 		var attr = grid[i].getAttribute('data-zh-field-name');
            // 		if (attr && attr.search('EIXO_X_') != -1) {
            // 			console.log(grid[i]);
            // 		}
            // 	}
            // }
            // if (hasError) {
            // } else if (!hasError && status === 100) {
            //     element.append("<div class='circle-success'></div>");
            // } else if (!hasError && status >=0 && status < 100) {
            //     element.append("<div class='progress progress-striped import-progress active'> <div class='progress-bar import-progress-bar' role='progressbar' aria-valuenow='" + status + "' aria-valuemin='0' aria-valuemax='100' style='width: " + status + "%" + "'> </div> </div>");
            // }
        }
    };

    this.abrePopupImportacao = function(widget) {
        var rowWidgetCompanhia = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow;
        var widgetPopupImportacao = widget.container.getWidget("popup_importacao");
        var fieldCompanhia = widgetPopupImportacao.getField("NMCIA");
        widget.newRow();

        var filtroCompanhiaImportacao = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowWidgetCompanhia.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "<>",
                "value": rowWidgetCompanhia.CDCIA
            }
        ];

        fieldCompanhia.dataSourceFilter = filtroCompanhiaImportacao;
        fieldCompanhia.reload();
        widgetPopupImportacao.newRow();
        ScreenService.openPopup(widgetPopupImportacao);
    };

    this.importaFrequenciaCiclo = function(widget) {
        if (widget.isValid()) {
            if (moment(widget.currentRow.DTFINAL, "DD/MM/YYYY").isBefore(moment(widget.currentRow.DTINICIAL, "DD/MM/YYYY"))) {

                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {

                var widgetTabbed = widget.container.getWidget("coc00900_frequenciaciclogeral_swipe");
                var rowPopup = widget.currentRow;
                var companhiaDestino = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow.CDCIA;
                var cdfilial = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow.CDFILIAL;

                var rowParaImportacao = rowPopup;
                rowParaImportacao.CDCIADESTINO = companhiaDestino;
                rowParaImportacao.CDFILIAL = cdfilial;

                restEngine.post("coc#/importaFrequenciaGeral", {
                    "requestType": "Row",
                    "row": rowParaImportacao
                }).then(function() {
                    MessageService.showMessageByCode("COC00900_IMPORT_SUCES");
                    widget.container.getWidget("calendar").view();
                    self.filtraCalendar(widgetTabbed, widgetTabbed.currentRow.year);
                    ScreenService.closePopup(true);
                }, function(error) {
                    if (error.data)
                        trataMensagem(error.data.error);
                });

            }
        }
    };

    this.abrePopupExportacao = function(widget) {
        var rowWidgetCompanhia = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow;
        var widgetPopupExportacao = widget.container.getWidget("popup_exportacao");
        var fieldCompanhia = widgetPopupExportacao.getField("NMCIA");
        widget.newRow();

        var filtroCompanhiaExportacao = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowWidgetCompanhia.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "<>",
                "value": rowWidgetCompanhia.CDCIA
            }
        ];

        fieldCompanhia.dataSourceFilter = filtroCompanhiaExportacao;
        fieldCompanhia.reload();
        widgetPopupExportacao.newRow();
        ScreenService.openPopup(widgetPopupExportacao);
    };

    this.exportaFrequenciaCiclo = function(widget) {
        if (widget.isValid()) {
            if (moment(widget.currentRow.DTFINAL, "DD/MM/YYYY").isBefore(moment(widget.currentRow.DTINICIAL, "DD/MM/YYYY"))) {

                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {

                var rowPopup = widget.currentRow;
                var companhiaOrigem = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow.CDCIA;
                var cdfilial = widget.container.getWidget("coc00900_frequenciaciclogeral").currentRow.CDFILIAL;

                var rowParaExportacao = rowPopup;
                rowParaExportacao.CDCIAORIGEM = companhiaOrigem;
                rowParaExportacao.CDFILIAL = cdfilial;

                restEngine.post("coc#/exportaFrequenciaGeral", {
                    "requestType": "Row",
                    "row": rowParaExportacao
                }).then(function() {
                    MessageService.showMessageByCode("COC00900_EXPORT_SUCES");
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe();
                    widget.container.getWidget("coc00900_frequenciaciclogeral").activate();
                }, function(error) {
                    if (error.data)
                        trataMensagem(error.data.error);
                });

            }
        }
    };

    this.habilitaFieldData = function(field) {
        var row = field.widget.currentRow;
        if (field.widget.currentRow.DTINICIAL) {
            habilitaFields(row, field, ["DTFINAL"], true);
        } else {
            habilitaFields(row, field, ["DTFINAL"], false);
        }

    };

    var habilitaFields = function(row, field, fieldsDependentes, habilita) {
        var filho;
        if (habilita) {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = false;
            }
        } else {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = true;
                field.widget.getField(fieldsDependentes[filho]).value("");
            }
        }
    };

    this.closePopup = function(widget) {
        ScreenService.closePopup(true);
        widget.container.getWidget("calendar").view();
        widget.container.getWidget("calendar").activate();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00900_frequenciaciclogeralController", coc00900_frequenciaciclogeralController);
});

// FILE: ./assets/js/controllers/coc00900_mapa_de_cicloController.js
var coc00900_mapa_de_cicloController = function(InflightorGeneral, ScreenService) {

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        args.owner.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
    };

    this.unidadeToCompanhia = function(field) {
        var cdfilial = field.widget.currentRow.CDFILIAL;
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        field.widget.getField("NMCIA").dataSourceFilter = filter;
        field.widget.getField("NMCIA").reload();
    };

    this.report = function(widget) {

        if (widget.isValid()) {
            var mes = moment(widget.currentRow.DTMESANO, "MMMM [de] YYYY");
            var inicioMes = moment(mes, "MMMM [de] YYYY").startOf("month").format("DD/MM/YYYY");
            var finalMes = moment(mes, "MMMM [de] YYYY").endOf("month").format("DD/MM/YYYY");

            widget.currentRow.DATAINI = inicioMes;
            widget.currentRow.DATAFIM = finalMes;

            InflightorGeneral.openReport("coc#/mapaCicloReport", null, widget.currentRow).then(function() {
                widget.activate();
                widget.reload();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data)
                    errorMessage = error.data.error;
                ScreenService.showMessage(errorMessage);
            });
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00900_mapa_de_cicloController", coc00900_mapa_de_cicloController);
});

// FILE: ./assets/js/controllers/coc00901_frequenciaciclovooController.js
function coc00901_frequenciaciclovooController(ScreenService, templateManager, coc00901_frequenciaciclovooService, Colors, coc00800_personalizacompanhiaService, InflightorGeneral, MessageService) {

    var cdCia;
    var Filial;
    var self = this;
    var dateStart;
    var intervalCiclo;
    var data = [];
    var dateIn;
    var dateEnd;
    var date;
    var identificador;
    var cdtrecho;
    var nrvoo;
    var cdclasse;
    var operator = "";

    var mensagensGlobais = {};

    this.getMensagensGlobais = function() {
        MessageService.getMessagesByCode(['COC00900_DOM', 'COC00900_SEG', 'COC00900_TER',
                                          'COC00900_QUA', 'COC00900_QUI', 'COC00900_SEX', 'COC00900_SAB']).then(function(mensagens) {
            mensagensGlobais.dom = mensagens[0];
            mensagensGlobais.seg = mensagens[1];
            mensagensGlobais.ter = mensagens[2];
            mensagensGlobais.qua = mensagens[3];
            mensagensGlobais.qui = mensagens[4];
            mensagensGlobais.sex = mensagens[5];
            mensagensGlobais.sab = mensagens[6];
        });
    };

    this.selecionaCompanhia = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("calendarVoo");

        Filial = current.CDFILIAL;
        cdCia = current.CDCIA;

        gridWidget.getActions()[0].isVisible = true;
        gridWidget.activate();
        gridWidget.reload();
    };

    this.associarFrequencia = function(widget) {
        var widgetPopup = widget.container.getWidget("associarFrequencia");
        ScreenService.openPopup(widgetPopup);
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            }
        ];
        widgetPopup.dataSourceFilter.data = filter;
        widgetPopup.activate();
        widgetPopup.reload();
    };

    this.openAssociacao = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        widget.container.getWidget("coc00901_frequenciaciclovoo").currentRow = widget.currentRow;
        ScreenService.closePopup(widget.container.getWidget("associarFrequencia"));
        swipeWidget.parent = widget.container.getWidget("associarFrequencia");
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        // swipeWidget.activate();
        swipeWidget.widgets[0].edit();
        swipeWidget.fields[0].applyDefaultValue();
        swipeWidget.fields[1].applyDefaultValue();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
        swipeWidget.parent = widget.container.getWidget("coc00900_frequenciaciclogeral");
        operator = "";
        // swipeWidget.widgets[0].view();
    };

    this.setFilterForField = function(field) {
        var widget = field.widget;
        var currentRow = widget.container.getWidget("coc00901_frequenciaciclovoo").currentRow;
        field.dataSourceFilter[1].value = currentRow.CDCIA;
        field.dataSourceFilter[3].value = currentRow.NRVOO;
        field.dataSourceFilter[4].value = currentRow.CDTRECHO;
        field.reload();
    };

    this.setYearForField = function(widget) {
        widget.fields[0].setValue(widget.currentRow.YEARS);
        widget.fields[0].reload();
        self.filtraCalendar(widget, widget.currentRow.YEARS);
    };
    this.setFilterforClasse = function(widget) {
        var parentWidget = widget.container.getWidget("coc00901_frequenciaciclovoo");
        var currentWidget = parentWidget.currentRow;
        var data = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": Filial
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentWidget.CDCIA
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": currentWidget.NRVOO
            },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": currentWidget.CDTRECHO
            }
        ];
        widget.dataSourceFilter = data;
        widget.reload();
    };

    this.setDefaultFilial = function() {
        if (!ScreenService.getPopupOpen().currentRow.CDFILIAL) {
            coc00800_personalizacompanhiaService.findFilial().then(function(callback) {
                var filial = callback.dataset.unidades;
                var widgetPopup = ScreenService.getPopupOpen();
                widgetPopup.currentRow.CDFILIAL = filial[0].CDFILIAL;
                widgetPopup.currentRow.NMFILIAL = filial[0].NMFILIAL;
            });
        }
    };

    this.initDefaultMensal = function(widget) {
        self.setDataSelectDate(widget);
        widget.getField("TROCAINICIAL").applyDefaultValue();
        widget.getField("TROCAFINAL").applyDefaultValue();
        widget.currentRow.id = 1;
        widget.currentRow.idtroca = 0;
    };

    this.initDefaultSemanal = function(widget) {
        self.setDataSelectDate(widget);
        widget.currentRow.TROCASEMANAL = mensagensGlobais.dom;
    };

    this.setDataSelectDate = function(widget) {
        var data = [
            {
                "value": mensagensGlobais.dom,
                "id": 0
            },
            {
                "value": mensagensGlobais.seg,
                "id": 1
            },
            {
                "value": mensagensGlobais.ter,
                "id": 2
            },
            {
                "value": mensagensGlobais.qua,
                "id": 3
            },
            {
                "value": mensagensGlobais.qui,
                "id": 4
            },
            {
                "value": mensagensGlobais.sex,
                "id": 5
            },
            {
                "value": mensagensGlobais.sab,
                "id": 6
            }
        ];
        if (widget.name == "frequenciaciclogeral_semanal")
            widget.getField("TROCASEMANAL").dataSource.data = data;
        else {
            widget.getField("TROCAFINAL").dataSource.data = data;
        }
    };

    this.filtraRegistros = function(widget, year) {
        // var year = widget.getYear();
        if (!year) {
            var newYear = new Date();
            self.filtraCalendar(widget, newYear.getFullYear());
        }
        self.filtraCalendar(widget, year);
    };

    this.openSwipe = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
        operator = "";
    };

    this.openSwipeClass = function(swipeWidget) {
        // self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        // swipeWidget.activate();
        self.filtraCalendarClasse(swipeWidget, moment().format("YYYY"));
    };

    this.filtraCalendar = function(widget, year) {
        var calendar = widget.container.getWidget("calendarVoo");
        var parentWidget = widget.container.getWidget("coc00901_frequenciaciclovoo");
        nrvoo = parentWidget.currentRow.NRVOO;
        cdtrecho = parentWidget.currentRow.CDTRECHO;
        var data = [];
        if (year) {
            // data = self.preencheCrossTab(setDataSource);
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00901_frequenciaciclovooService.findFrequenciaCiclo(Filial, cdCia, dateInit, dateFinal, parentWidget.currentRow.NRVOO, parentWidget.currentRow.CDTRECHO).then(function(callback) {

                var ciclos = callback.dataset.frciclovoo_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                // ciclos.forEach(function(dataset){
                // 	var dateFormat = moment(dataset.DTCICLOVOO.slice(0,10), "DD/MM/YYYY");
                // 	data.push({
                // 		EIXO_X: dataset.EIXO_X,
                // 		EIXO_Y: moment(dataset.EIXO_Y, 'MM').format('MMM'),
                // 		NRCORCICLO: Colors.TColorToHex(dataset.NRCORCICLO),
                // 		IDCICLO: dataset.IDCICLO
                // 	});

                // 	// dateFormat = ;
                // 	// setDataSource.TITLE = dataset.CDCICLO;
                // 	// setDataSource.DATE = angular.copy(dateFormat);
                // });

                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        } //if year
    };

    function filtraCicloBanco(dia, mes, ciclos) {
        return ciclos.filter(function(ciclo) {
            return ciclo.EIXO_X == dia && ciclo.EIXO_Y == mes;
        });
    }

    function pad(value, length) {
        return (value.toString().length < length) ? pad("0" + value, length) : value;
    }

    this.filtraCalendarClasse = function(widget, year) {
        var calendar = widget.container.getWidget("calendarClasse");
        var parentWidget = widget.container.getWidget("coc00901_frequenciaciclovooclasse");
        cdclasse = parentWidget.currentRow.CDCLASSE;
        var data = [];
        if (year) {
            // data = self.preencheCrossTab(setDataSource);
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00901_frequenciaciclovooService.findFrequenciaCicloClasse(Filial, cdCia, dateInit, dateFinal, nrvoo, cdtrecho, cdclasse).then(function(callback) {

                var ciclos = callback.dataset.frciclovcla_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        } //if year
    };

    this.closeSwipe = function(widget) {
        var parentWidget = widget.container.getWidget("coc00901_frequenciaciclovoo");
        widget.lastActiveWidget = parentWidget;
        ScreenService.closeSwipe().then(function() {
            if (operator == "modified")
                parentWidget.reload();
            parentWidget.activate();
        });
    };

    this.addCiclo = function(widget) {

        var gridWidget = widget.container.getWidget("popup_cadastro");

        var widgetsNames = ["diario", "semanal", "quinzenal", "mensal"];

        widgetsNames.forEach(function(nome) {
            var currentWidget = widget.container.getWidget("frequenciaciclogeral_" + nome);
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
            currentWidget.getField("MULTIPLECICLO").reload();
            currentWidget.newRow();
        });
        ScreenService.openSwipe(gridWidget);

    };

    this.openExclui = function(widget) {

        var formWidget = widget.container.getWidget("popup_excluir");
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
        formWidget.getField("MULTIPLECICLO").reload();
        formWidget.newRow();
        formWidget.reload();
        ScreenService.openPopup(formWidget);

    };

    this.cancelExcluir = function(widget) {
        ScreenService.closePopup();
        var swipeWidget = widget.container.getWidget("calendarVoo");
        swipeWidget.activate();
        // swipeWidget.view();
    };

    this.cancelAdd = function(widget) {
        ScreenService.closeSwipe();
        var swipeWidget = widget.container.getWidget("calendarVoo");
        swipeWidget.activate();
        // swipeWidget.list();
    };

    this.reloadDias = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                valueDay.setValue(0);
                valueDay.reload();
            }
        }

    };

    this.validaQtdDias = function(widget) {

        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                dateIn = date.start;
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;
            }
        }
    };

    this.setFnQuinzenal = function(widget) {

        var interval = widget.getField("INTERVAL");
        var date = interval.value();
        var valueTroca = widget.getField("INQUINZENAL").value();
        if (valueTroca) {
            widget.getField("FNQUINZENAL").setValue(valueTroca + 15);
            widget.getField("FNQUINZENAL").reload();
        }
    };

    this.subDate = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                //Verifica o maior valor valido de acordo com o mês
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;

                //Diferença entre DateIn e DateFn 
                var finalDate = moment(dateFn, "DD/MM/YYYY");
                var startDate = moment(dateIn, "DD/MM/YYYY");
                var difDate = finalDate.diff(startDate, "days");
                if (widget.name == "frequenciaciclogeral_quinzenal") {
                    if (difDate < 15) {
                        MessageService.showMessageByCode("COC00900_CICLO_QUINZENAL_PERIODO_MENOR");
                    }
                }
                if (widget.name == "frequenciaciclogeral_mensal" && difDate < 29) {
                    MessageService.showMessageByCode("COC00900_CICLO_MENSAL_PERIODO_MENOR");
                }
            }
        }

    };

    this.filtraTroca = function(widget) {

        var filtraRADIO = widget.getField("RADIOTROCA");
        var semanaInField = widget.getField("TROCAINICIAL");
        var semanaFnField = widget.getField("TROCAFINAL");
        var diaField = widget.getField("INQUINZENAL");
        var itemSelect = filtraRADIO.value();

        if (itemSelect == 1) {
            //Exibe DIAMENSAL spin 
            diaField.isVisible = true;
            semanaInField.isVisible = false;
            semanaFnField.isVisible = false;

        }
        if (itemSelect == 2) {
            //Exibe TROCAINICIAL e TROCAFINAL 
            semanaInField.isVisible = true;
            semanaFnField.isVisible = true;
            diaField.isVisible = false;
        }

        filtraRADIO.reload();

    };

    this.setDefaultTab = function(tabbedWidget, name) {
        tabbedWidget.defaultTab = name;
    };

    this.iniciaValues = function(row, name) {
        dateStart = row.INTERVAL.start;
        dateEnd = row.INTERVAL.end;
        dateIn = moment(dateStart, "DD/MM/YYYY");
        dateEnd = moment(dateEnd, "DD/MM/YYYY");
        //Diferença data inicial -> data final
        intervalCiclo = dateEnd.diff(dateStart, "days");
        data = [];
        date = dateIn.format("DD/MM/YYYY");
        identificador = name;
    };

    this.salveFreqCiclo = function(widget) {
        var row = widget.currentWidget.currentRow;
        var nameWidget = widget.currentWidget.name;
        if (row.INTERVAL.start && row.INTERVAL.end && row.MULTIPLECICLO) {
            switch (nameWidget) {
                case "frequenciaciclogeral_diario":
                    if (row.DURACAOCICLODIAS) {
                        this.salvaDiario(row, widget);
                    } else {
                        MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
                    }
                    break;
                case "frequenciaciclogeral_semanal":
                    this.salvaSemanal(row, widget);
                    break;
                case "frequenciaciclogeral_quinzenal":
                    this.salvaQuinzenal(row, widget);
                    break;
                case "frequenciaciclogeral_mensal":
                    this.salvaMensal(row, widget);
                    break;
                default:
            }
        } else {
            MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.salvaDiario = function(row, widget) {

        self.iniciaValues(row, "Diario");

        intervalCiclo += 1;
        if (intervalCiclo < row.DURACAOCICLODIAS) {
            MessageService.showMessageByCode("COC00900_DURACAO_MAIOR_Q_PERMITIDO", {
                "0": intervalCiclo
            });
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            exibeDate = exibeDate.format("L");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var qtdCiclo = 0;
                var maxCiclo = row.MULTIPLECICLO.length;
                var duracaoCiclo = row.DURACAOCICLODIAS;
                var alterCiclo = 0;
                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaSemanal = function(row, widget) {

        self.iniciaValues(row, "Semanal");

        var editInterval = intervalCiclo;
        editInterval /= 7;
        intervalCiclo += 1;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 0;
        editInterval = Math.round(editInterval);
        var duracaoCiclo = row.DURACAOCICLO * 7;
        if (intervalCiclo < 7) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_PERIODO_MENOR");
        } else if (editInterval < row.DURACAOCICLO) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_DURACAO_MAIOR");
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            exibeDate = exibeDate.format("L");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var diaSemana = row.TROCASEMANAL.toString();
                var diaInicial = dateIn.format("d");
                while (diaSemana.toUpperCase() != diaInicial.toUpperCase()) {
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    date = moment(alterDate, "DD/MM/YYYY");
                    diaInicial = date.format("d");
                }
                diaInicial = moment(date, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(diaInicial, "days") + 1;
                date = diaInicial.format("DD/MM/YYYY");

                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaQuinzenal = function(row, widget) {

        self.iniciaValues(row, "Quinzenal");

        var dtInTroca = row.INQUINZENAL;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 1;
        var dataInicial = dateIn.format("DD");
        var duracaoCiclo = 15;

        for (let value = 2; value <= dtInTroca; value++) {
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = moment(alterDate, "DD/MM/YYYY");
            dataInicial = date.format("DD");
        }
        dataInicial = moment(date, "DD/MM/YYYY");
        intervalCiclo = dateEnd.diff(dataInicial, "days") + 1;
        date = dataInicial.format("DD/MM/YYYY");
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
            self.salvaFrequencia(data, widget);
        });
    };

    this.salvaMensalporDiaSemana = function(intervalCiclo, cdciclo, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador) {

        var qtdCiclo = 0;
        var alterCiclo = 0;
        var qtdMes = 0;
        var trocaSemana = row.id;
        var permiteTroca = 0;

        if (intervalMensal === 0 || maxCiclo == 1 || (duracaoCiclo > intervalMensal)) {
            identificador = "Mensal";
            self.preencheData(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador);
        } else {
            for (let aux = 0; aux <= intervalCiclo; aux++) {
                if (identificador == "MensalDia") {
                    if (trocaDia == dateTroca) {
                        qtdMes++;
                        if (qtdMes == duracaoCiclo) {
                            qtdCiclo++;
                            qtdMes = 0;
                        }
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                } else if (identificador == "MensalSemana") {
                    if (intervalMensal > 0) {
                        if (permiteTroca) {
                            var startOfMonth = alterDate.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                            var studyDate = moment(startOfMonth).add(trocaSemana, "w");
                            if (studyDate.month() == startOfMonth.month()) {
                                studyDate = studyDate.subtract(1, "w");
                            }
                            studyDate = studyDate.format("DD/MM/YYYY");
                            if (date == studyDate) {
                                qtdCiclo++;
                            }
                            if (qtdCiclo == maxCiclo) {
                                qtdCiclo = 0;

                            }
                        }
                    }
                }
                var rows = {
                    CDCICLO: row.MULTIPLECICLO[qtdCiclo],
                    CDCIA: cdCia,
                    CDTRECHO: cdtrecho,
                    NRVOO: nrvoo,
                    CDFILIAL: Filial,
                    NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                    DTCICLOVOO: date,
                    __is_new: true
                };
                var newDate = moment(date, "DD/MM/YYYY");
                var alterDate = newDate.add(1, "days").format("DD/MM/YYYY");
                date = alterDate;
                alterDate = moment(alterDate, "DD/MM/YYYY");
                data.push(rows);
                var dateTroca = newDate.format("DD");
                var maxTrocaMes = moment(date, "DD/MM/YYYY");
                maxTrocaMes = moment(newDate).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(maxTrocaMes, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                intervalMensal = newDate.startOf("month").diff(dateIn.startOf("month"), "month");
                permiteTroca++;
                if (maxDayFn < trocaDia) {
                    trocaDiaAnt = trocaDia;
                    trocaDia = maxDayFn;
                } else if (maxDayFn >= row.INQUINZENAL) {
                    trocaDia = row.INQUINZENAL;
                } else {
                    trocaDia = trocaDiaAnt;
                }
            }
        }

    };

    this.salvaMensal = function(row, widget) {

        self.iniciaValues(row, "Mensal");
        var opcaoTroca = row.RADIOTROCA;
        var trocaDia = row.INQUINZENAL;
        dateStart = moment(dateStart, "DD/MM/YYYY");
        var alterDia = 0;
        var trocaDiaAnt = trocaDia;
        var duracaoCiclo = row.DURACAOCICLO;
        var maxCiclo = row.MULTIPLECICLO.length;
        var intervalMensal = 0;
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            if (opcaoTroca == 1) {
                //Troca Diário 
                while (trocaDia != alterDia) {
                    var newDate = moment(dateIn, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    dateIn = alterDate;
                    alterDia = newDate.format("DD");
                    var monthInterval = moment(dateIn, "DD/MM/YYYY");
                    var month = moment(monthInterval, "DD/MM/YYYY");
                    monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                    var maxDay = moment(monthInterval, "DD/MM/YYYY");
                    var maxDayFn = maxDay.format("DD");
                    if (trocaDia >= maxDayFn) {
                        trocaDia = maxDayFn;
                    }

                }
                dateIn = moment(dateIn, "DD/MM/YYYY");
                dateIn = dateIn.subtract(1, "days").format("DD/MM/YYYY");
                dateIn = moment(dateIn, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(dateIn, "days") + 1;
                date = dateIn.format("DD/MM/YYYY");
                intervalMensal = dateEnd.diff(dateStart, "month");
                identificador = "MensalDia";

                self.salvaMensalporDiaSemana(intervalCiclo, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador);

            } else if (opcaoTroca == 2) {
                //Troca Semanal 
                /* row.id -> 1 - Primeiro(a)
2 -  Segundo(a)
3 - Terceiro(a)
4 -   Quarto(a)
*/
                var trocaSemana = row.id;
                var startOfMonth = angular.copy(dateIn);
                startOfMonth.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                var studyDate = moment(startOfMonth.format("DD/MM/YYYY"), "DD/MM/YYYY").add(trocaSemana, "w");
                if (studyDate.month() == startOfMonth.month()) {
                    studyDate = studyDate.subtract(1, "w");
                }
                var verificaDate = studyDate.format("DD/MM/YYYY");
                verificaDate = moment(verificaDate, "DD/MM/YYYY");
                if (verificaDate < dateIn) {
                    dateIn = dateIn.add(1, "month");
                    var startOfMonthIf = dateIn.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                    var studyDateIf = moment(startOfMonthIf).add(trocaSemana, "w");
                    if (studyDateIf.month() == startOfMonthIf.month()) {
                        studyDateIf = studyDateIf.subtract(1, "w");
                    }
                }

                intervalMensal = dateEnd.diff(studyDate, "month");
                date = studyDate.format("DD/MM/YYYY");
                identificador = "MensalSemana";
                self.salvaMensalporDiaSemana(intervalCiclo + 1, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador);

            }
            self.salvaFrequencia(data, widget);
        });

    };

    this.salvaFrequencia = function(data, widget) {
        coc00901_frequenciaciclovooService.novaFrequenciaCiclo(data).then(function() {
            MessageService.showMessageByCode("COC00900_CAD_AUTO_SUCES");
            var setWidget = widget.container.getWidget("calendarVoo");
            ScreenService.closeSwipe();
            self.filtraRegistros(setWidget, setWidget.container.getWidget("coc00900_frequenciaciclo_voo_swipe").currentRow.year);
            setWidget.reload().then(function() {
                setWidget.activate();
                setWidget.view();
                operator = "modified";
            });
        }, function(calback) {
            ScreenService.showMessage(calback.data.error);
        });
    };

    this.preencheData = function(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador) {
        var dtInicialQuin = moment(date, "DD/MM/YYYY");
        var dtFinalQuin = angular.copy(dtInicialQuin).add(15, "days");
        dtFinalQuin = dtFinalQuin.format("DD");
        dtInicialQuin = dtInicialQuin.format("DD");
        var permiteTroca = 0;
        for (let aux = 0; aux < intervalCiclo; aux++) {
            if (identificador == "Diario" || identificador == "Semanal") {
                if (alterCiclo == duracaoCiclo) {
                    alterCiclo = 0;
                    qtdCiclo++;
                    if (qtdCiclo == maxCiclo) {
                        qtdCiclo = 0;
                    }
                }
            } else if (identificador == "Quinzenal") {
                var alterDateQuin = moment(date, "DD/MM/YYYY");
                alterDateQuin = alterDateQuin.format("DD");
                if (permiteTroca) {
                    if (dtInicialQuin == alterDateQuin || dtFinalQuin == alterDateQuin) {
                        qtdCiclo++;
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                }
            }
            var rows = {
                CDCICLO: cdciclo[qtdCiclo],
                CDCIA: cdCia,
                CDFILIAL: Filial,
                NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                DTCICLOVOO: date,
                NRVOO: nrvoo,
                CDTRECHO: cdtrecho,
                __is_new: true
            };
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = alterDate;
            data.push(rows);
            alterCiclo++;
            permiteTroca++;
        }
    };

    this.preencheDataExcluir = function(widget) {
        var dataset = [];
        var currentRow = widget.currentRow;
        var swipeWidget = widget.container.getWidget("calendarVoo");
        self.iniciaValues(currentRow, widget.name);
        intervalCiclo += 1;
        if (!currentRow.INTERVAL) {
            MessageService.showMessageByCode("COC00900_PREENCHA_INTERVALO_EXCL");
        } else if (!currentRow.MULTIPLECICLO) {
            MessageService.showMessageByCode("COC00900_CICLOS_N_INF");
        } else {
            if (currentRow.MULTIPLECICLO.length == 1) {
                for (let aux = 0; aux < intervalCiclo; aux++) {

                    var rows = {
                        CDCICLO: currentRow.MULTIPLECICLO[0],
                        CDCIA: cdCia,
                        NRVOO: nrvoo,
                        CDTRECHO: cdtrecho,
                        CDFILIAL: Filial,
                        NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                        DTCICLOVOO: date,
                        __is_new: false
                    };
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("L");
                    date = alterDate;
                    dataset.push(rows);
                }
            } else {
                var qtdCiclo = currentRow.MULTIPLECICLO.length;
                qtdCiclo--;
                var ciclo = currentRow.MULTIPLECICLO;
                while (qtdCiclo >= 0) {
                    date = dateIn.format("DD/MM/YYYY");
                    for (let cont = 0; cont <= intervalCiclo; cont++) {

                        var row = {
                            CDCICLO: ciclo[qtdCiclo],
                            CDCIA: cdCia,
                            NRVOO: nrvoo,
                            CDTRECHO: cdtrecho,
                            CDFILIAL: Filial,
                            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                            DTCICLOVOO: date,
                            __is_new: false

                        };
                        var newDateQtd = moment(date, "DD/MM/YYYY");
                        var alterDateQtd = moment(newDateQtd).add(1, "days").format("DD/MM/YYYY");
                        date = alterDateQtd;
                        dataset.push(row);
                    }
                    self.iniciaValues(currentRow, widget.name);
                    qtdCiclo--;
                }
            }
            self.excluiFrequencia(dataset, swipeWidget, dateStart, dateEnd);
        }
    };

    this.excluiFrequencia = function(data, widget, dateStart, dateEnd) {
        var exibeDate = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(dateEnd, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_EXCL_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            coc00901_frequenciaciclovooService.excluiFrequenciaCiclo(data).then(function() {
                MessageService.showMessageByCode("COC00900_EXCL_AUTO_SUCES");
                ScreenService.closePopup();
                var setWidget = widget.container.getWidget("calendarVoo");
                self.filtraRegistros(widget, setWidget.container.getWidget("coc00900_frequenciaciclo_voo_swipe").currentRow.year);
                setWidget.reload().then(function() {
                    setWidget.activate();
                    setWidget.view();
                    operator = "modified";
                });
            }, function(calback) {
                ScreenService.showMessage(calback.data.error);
            });
        });
    };

    this.setCicloColor = function(element, row) {
        if (row.NRCORCICLO) {
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            // var status = Util.isUndefinedOrNull(row.importStatus) ? row.status : row.importStatus;
            // var hasError = Util.isUndefinedOrNull(row.importHasError) ? row.hasError : row.importHasError;
            // var grid = $(".th.grid-align-left");
            // for (let i in grid) {
            // 	if (typeof grid[i] == "object") {
            // 		var attr = grid[i].getAttribute('data-zh-field-name');
            // 		if (attr && attr.search('EIXO_X_') != -1) {
            // 			console.log(grid[i]);
            // 		}
            // 	}
            // }
            // if (hasError) {
            // } else if (!hasError && status === 100) {
            //     element.append("<div class='circle-success'></div>");
            // } else if (!hasError && status >=0 && status < 100) {
            //     element.append("<div class='progress progress-striped import-progress active'> <div class='progress-bar import-progress-bar' role='progressbar' aria-valuenow='" + status + "' aria-valuemin='0' aria-valuemax='100' style='width: " + status + "%" + "'> </div> </div>");
            // }
        }
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc00901_frequenciaciclovooController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        // if (args.owner.currentRow.IDATIVO === "N") {
        //     MessageService.showMessageByCode("INACTIVE_CIA");
        // } else {
        //     args.owner.setSelected(args);
        // }
        args.owner.setSelected(args);
    };

    this.unidadeToCompanhia = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.bloqueiaFieldFiltro = function(field, listaFieldName) {
        var contador = 0;
        var status;

        listaFieldName.forEach(function(fieldName) {
            if (field.value() && !contador) {
                status = false;
            } else {
                status = true;
            }

            field.widget.getField(fieldName).readOnly = status;
            field.widget.getField(fieldName).setValue("");
            contador++;
        });
    };

    this.filtraVoo = function(widget) {
        var widgetFilterCurrentRow = widget.container.getWidget("coc00901_frequenciaciclovoo").appliedFilter.FILTER_ROW;
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetFilterCurrentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetFilterCurrentRow.CDCIA
            },
            {
                "name": "CHAVE",
                "operator": "NOT_IN",
                "value": widget.container.getWidget("coc00901_frequenciaciclovoo").currentRow.NRVOO + "_" + widget.container.getWidget("coc00901_frequenciaciclovoo").currentRow.CDTRECHO
            }
        ];
        widget.getField("DSVOOTRECHO").dataSourceFilter = filtro;
        widget.getField("DSVOOTRECHO").reload();
    };

    this.importaFrequenciaCicloVoo = function(widget) {
        if (widget.isValid()) {
            if (moment(widget.currentRow.DTFIMVGMA, "DD/MM/YYYY").isBefore(moment(widget.currentRow.DTINIVGMA, "DD/MM/YYYY"))) {
                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {
                var currentRow = widget.currentRow;
                var widgetDestino = widget.container.getWidget("coc00901_frequenciaciclovoo");
                currentRow.CDCIA = widgetDestino.currentRow.CDCIA;
                currentRow.NRVOODES = widgetDestino.currentRow.NRVOO;
                currentRow.CDTRECHODES = widgetDestino.currentRow.CDTRECHO;
                currentRow.CDFILIAL = widgetDestino.currentRow.CDFILIAL;
                InflightorGeneral.callRestEngine("coc#/importaCicloVoo", "Row", currentRow).then(function() {
                    MessageService.showMessageByCode("COC00900_IMPORT_SUCES");
                    ScreenService.closePopup(true);
                    self.openSwipe(widget.container.getWidget("coc00900_frequenciaciclo_voo_swipe"), widgetDestino);
                }, function(err) {
                    ScreenService.showMessage(err.data.error);
                });
            }
        }
    };

    this.exportaFrequenciaCicloVoo = function(widgetDestino) {
        if (widgetDestino.isValid()) {
            if (moment(widgetDestino.currentRow.DTFIMVGMA, "DD/MM/YYYY").isBefore(moment(widgetDestino.currentRow.DTINIVGMA, "DD/MM/YYYY"))) {
                MessageService.showMessageByCode("COC00900_PERIODO_INV");
            } else {
                var widgetOrigem = widgetDestino.container.getWidget("coc00901_frequenciaciclovoo");
                widgetOrigem.currentRow.CDCIA = widgetDestino.getField("DSVOOTRECHO").dataSource.data[0].CDCIA;
                widgetOrigem.currentRow.CHAVE = widgetDestino.currentRow.DSVOOTRECHO;
                widgetOrigem.currentRow.DTINIVGMA = widgetDestino.currentRow.DTINIVGMA;
                widgetOrigem.currentRow.DTFIMVGMA = widgetDestino.currentRow.DTFIMVGMA;
                widgetOrigem.currentRow.CDTRECHODES = widgetDestino.currentRow.CDTRECHO;
                widgetOrigem.currentRow.CDFILIAL = widgetDestino.getField("DSVOOTRECHO").dataSource.data[0].CDFILIAL;
                InflightorGeneral.callRestEngine("coc#/exportaCicloVoo", "Row", widgetOrigem.currentRow).then(function() {
                    MessageService.showMessageByCode("COC00900_EXPORT_SUCES");
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe();
                    widgetOrigem.activate();
                }, function(err) {
                    ScreenService.showMessage(err.data.error);
                });
            }
        }
    };

    this.closePopup = function(widget) {
        ScreenService.closePopup(true);
        widget.container.getWidget("calendarVoo").activate();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00901_frequenciaciclovooController", coc00901_frequenciaciclovooController);
});

// FILE: ./assets/js/controllers/coc00902_frequenciaciclovclasseController.js
function coc00902_frequenciaciclovclasseController(ScreenService, templateManager, coc00901_frequenciaciclovooService, Colors, coc00800_personalizacompanhiaService, InflightorGeneral, MessageService) {

    //Existe um field calendar para setar o valor do CDTRECHO para o filter funcionar
    //O campo pode ser removido quando o outdata do filter puder aproveitar os dois valores que estão sendo passados
    var Filial;
    var cdCia;
    var self = this;
    var dateStart;
    var intervalCiclo;
    var data = [];
    var dateIn;
    var dateEnd;
    var date;
    var identificador;
    var cdtrecho;
    var nrvoo;
    var cdclasse;
    var operator = "";

    var mensagensGlobais = {};

    this.getMensagensGlobais = function() {
        MessageService.getMessagesByCode(['COC00900_DOM', 'COC00900_SEG', 'COC00900_TER',
                                          'COC00900_QUA', 'COC00900_QUI', 'COC00900_SEX', 'COC00900_SAB']).then(function(mensagens) {
            mensagensGlobais.dom = mensagens[0];
            mensagensGlobais.seg = mensagens[1];
            mensagensGlobais.ter = mensagens[2];
            mensagensGlobais.qua = mensagens[3];
            mensagensGlobais.qui = mensagens[4];
            mensagensGlobais.sex = mensagens[5];
            mensagensGlobais.sab = mensagens[6];
        });
    };

    this.selecionaCompanhia = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("calendarClasse");

        cdCia = current.CDCIA;
        Filial = current.CDFILIAL;

        gridWidget.getActions()[0].isVisible = true;
        gridWidget.activate();
        gridWidget.reload();

    };

    this.associarFrequencia = function(widget) {
        var widgetPopup = widget.container.getWidget("associarFrequencia");
        ScreenService.openPopup(widgetPopup);
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDCIA
            },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDTRECHO
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.NRVOO
            }
        ];
        widgetPopup.dataSourceFilter = filter;
        widgetPopup.activate();
        widgetPopup.reload();
    };

    this.openAssociacao = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow = widget.currentRow;
        ScreenService.closePopup(widget.container.getWidget("associarFrequencia"));
        swipeWidget.parent = widget.container.getWidget("associarFrequencia");
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        // swipeWidget.activate();
        swipeWidget.widgets[0].edit();
        swipeWidget.fields[0].applyDefaultValue();
        swipeWidget.fields[1].applyDefaultValue();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
        swipeWidget.parent = widget.container.getWidget("coc00900_frequenciaciclogeral");
        operator = "";
        // swipeWidget.widgets[0].view();
    };
    //Valida os readonly's do filtro
    this.UnidadeToCompanhia = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.CompanhiaToTrecho = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDFILIAL": currentRow.CDFILIAL,
            "CDCIA": currentRow.CDCIA,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);

    };

    this.setFilterForField = function(field) {
        var widget = field.widget;
        var currentRow = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow;
        field.dataSourceFilter[1].value = currentRow.CDCIA;
        field.dataSourceFilter[3].value = currentRow.NRVOO;
        field.dataSourceFilter[4].value = currentRow.CDTRECHO;
        field.reload();
    };

    this.setYearForField = function(widget) {
        widget.getField("year").setValue(widget.currentRow.YEARS);
        widget.getField("year").reload();
        self.filtraCalendarClasse(widget, widget.currentRow.YEARS);
    };

    this.setFilterforClasse = function(widget) {
        var parentWidget = widget.container.getWidget("coc00902_frequenciaciclovclasse");
        var currentWidget = parentWidget.currentRow;
        var data = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": Filial
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentWidget.CDCIA
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": currentWidget.NRVOO
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentWidget.CDCLASSE
            },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": currentWidget.CDTRECHO
            }
        ];
        widget.dataSourceFilter = data;
        widget.reload();
    };

    this.initDefaultMensal = function(widget) {
        self.setDataSelectDate(widget);
        widget.getField("TROCAINICIAL").applyDefaultValue();
        widget.getField("TROCAFINAL").applyDefaultValue();
        widget.currentRow.id = 1;
        widget.currentRow.idtroca = 0;
    };

    this.initDefaultSemanal = function(widget) {
        self.setDataSelectDate(widget);
        widget.currentRow.TROCASEMANAL = mensagensGlobais.dom;
    };

    this.setDataSelectDate = function(widget) {
        var data = [
            {
                "value": mensagensGlobais.dom,
                "id": 0
            },
            {
                "value": mensagensGlobais.seg,
                "id": 1
            },
            {
                "value": mensagensGlobais.ter,
                "id": 2
            },
            {
                "value": mensagensGlobais.qua,
                "id": 3
            },
            {
                "value": mensagensGlobais.qui,
                "id": 4
            },
            {
                "value": mensagensGlobais.sex,
                "id": 5
            },
            {
                "value": mensagensGlobais.sab,
                "id": 6
            }
        ];
        if (widget.name == "frequenciaciclogeral_semanal")
            widget.getField("TROCASEMANAL").dataSource.data = data;
        else {
            widget.getField("TROCAFINAL").dataSource.data = data;
        }
    };

    this.filtraRegistros = function(widget, year) {
        // var year = widget.getYear();
        if (!year) {
            var newYear = new Date();
            self.filtraCalendar(widget, newYear.getFullYear());
        }
        self.filtraCalendar(widget, year);
    };

    this.openSwipe = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendar(swipeWidget, moment().format("YYYY"));
        operator = "";
    };

    this.openSwipeClass = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        //var year = parseInt(swipeWidget.currentRow.year);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendarClasse(swipeWidget, moment().format("YYYY"));
        // operator = "";
    };

    this.filtraCalendar = function(widget, year) {
        var calendar = widget.container.getWidget("calendarClasse");
        var parentWidget = widget.container.getWidget("coc00902_frequenciaciclovclasse");
        nrvoo = parentWidget.currentRow.NRVOO;
        cdtrecho = parentWidget.currentRow.CDTRECHO;
        var data = [];
        if (year) {
            // data = self.preencheCrossTab(setDataSource);
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00901_frequenciaciclovooService.findFrequenciaCicloClasse(Filial, cdCia, dateInit, dateFinal, parentWidget.currentRow.NRVOO, parentWidget.currentRow.CDTRECHO, parentWidget.currentRow.CDCLASSE).then(function(callback) {

                var ciclos = callback.dataset.frciclovcla_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                // ciclos.forEach(function(dataset){
                // 	var dateFormat = moment(dataset.DTCICLOVCLA.slice(0,10), "DD/MM/YYYY");
                // 	data.push({
                // 		EIXO_X: dataset.EIXO_X,
                // 		EIXO_Y: moment(dataset.EIXO_Y, 'MM').format('MMM'),
                // 		NRCORCICLO: Colors.TColorToHex(dataset.NRCORCICLO),
                // 		IDCICLO: dataset.IDCICLO
                // 	});

                // 	// dateFormat = ;
                // 	// setDataSource.TITLE = dataset.CDCICLO;
                // 	// setDataSource.DATE = angular.copy(dateFormat);
                // });

                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        } //if year
    };

    function filtraCicloBanco(dia, mes, ciclos) {
        return ciclos.filter(function(ciclo) {
            return ciclo.EIXO_X == dia && ciclo.EIXO_Y == mes;
        });
    }

    function pad(value, length) {
        return (value.toString().length < length) ? pad("0" + value, length) : value;
    }

    this.filtraCalendarClasse = function(widget, year) {
        var calendar = widget.container.getWidget("calendarClasse");
        var parentWidget = widget.container.getWidget("coc00902_frequenciaciclovclasse");
        cdCia = parentWidget.currentRow.CDCIA;
        nrvoo = parentWidget.currentRow.NRVOO;
        cdtrecho = parentWidget.currentRow.CDTRECHO;
        cdclasse = parentWidget.currentRow.CDCLASSE;
        var data = [];
        if (year) {
            // data = self.preencheCrossTab(setDataSource);
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00901_frequenciaciclovooService.findFrequenciaCicloClasse(Filial, cdCia, dateInit, dateFinal, nrvoo, cdtrecho, cdclasse).then(function(callback) {

                var ciclos = callback.dataset.frciclovcla_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        } //if year
    };

    this.closeSwipe = function(widget) {
        var parentWidget = widget.container.getWidget("coc00902_frequenciaciclovclasse");
        widget.lastActiveWidget = parentWidget;
        ScreenService.closeSwipe().then(function() {
            if (operator == "modified")
                parentWidget.reload();
            parentWidget.activate();
        });
    };

    this.addCiclo = function(widget) {

        var gridWidget = widget.container.getWidget("popup_cadastro");

        var widgetsNames = ["diario", "semanal", "quinzenal", "mensal"];

        widgetsNames.forEach(function(nome) {
            var currentWidget = widget.container.getWidget("frequenciaciclogeral_" + nome);
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
            currentWidget.getField("MULTIPLECICLO").reload();
            currentWidget.newRow();
        });
        ScreenService.openSwipe(gridWidget);

    };

    this.openExclui = function(widget) {

        var formWidget = widget.container.getWidget("popup_excluir");
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
        formWidget.getField("MULTIPLECICLO").reload();
        formWidget.newRow();
        formWidget.reload();
        ScreenService.openPopup(formWidget);

    };

    this.cancelExcluir = function(widget) {
        ScreenService.closePopup();
        var swipeWidget = widget.container.getWidget("calendarClasse");
        swipeWidget.activate();
        // swipeWidget.view();
    };

    this.cancelAdd = function(widget) {
        ScreenService.closeSwipe();
        var swipeWidget = widget.container.getWidget("calendarClasse");
        swipeWidget.activate();
        // swipeWidget.list();
    };

    this.reloadDias = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                valueDay.setValue(0);
                valueDay.reload();
            }
        }

    };

    this.validaQtdDias = function(widget) {

        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                dateIn = date.start;
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;
            }
        }
    };

    this.setFnQuinzenal = function(widget) {

        var interval = widget.getField("INTERVAL");
        var date = interval.value();
        var valueTroca = widget.getField("INQUINZENAL").value();
        if (valueTroca) {
            widget.getField("FNQUINZENAL").setValue(valueTroca + 15);
            widget.getField("FNQUINZENAL").reload();
        }
    };

    this.subDate = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                //Verifica o maior valor valido de acordo com o mês
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;

                //Diferença entre DateIn e DateFn 
                var finalDate = moment(dateFn, "DD/MM/YYYY");
                var startDate = moment(dateIn, "DD/MM/YYYY");
                var difDate = finalDate.diff(startDate, "days");
                if (widget.name == "frequenciaciclogeral_quinzenal") {
                    if (difDate < 15) {
                        MessageService.showMessageByCode("COC00900_CICLO_QUINZENAL_PERIODO_MENOR");

                    }
                }
                if (widget.name == "frequenciaciclogeral_mensal" && difDate < 29) {
                    MessageService.showMessageByCode("COC00900_CICLO_MENSAL_PERIODO_MENOR");
                }
            }
        }

    };

    this.filtraTroca = function(widget) {

        var filtraRADIO = widget.getField("RADIOTROCA");
        var semanaInField = widget.getField("TROCAINICIAL");
        var semanaFnField = widget.getField("TROCAFINAL");
        var diaField = widget.getField("INQUINZENAL");
        var itemSelect = filtraRADIO.value();

        if (itemSelect == 1) {
            //Exibe DIAMENSAL spin 
            diaField.isVisible = true;
            semanaInField.isVisible = false;
            semanaFnField.isVisible = false;

        }
        if (itemSelect == 2) {
            //Exibe TROCAINICIAL e TROCAFINAL 
            semanaInField.isVisible = true;
            semanaFnField.isVisible = true;
            diaField.isVisible = false;
        }

        filtraRADIO.reload();

    };

    this.setDefaultTab = function(tabbedWidget, name) {
        tabbedWidget.defaultTab = name;
    };

    this.iniciaValues = function(row, name) {
        dateStart = row.INTERVAL.start;
        dateEnd = row.INTERVAL.end;
        dateIn = moment(dateStart, "DD/MM/YYYY");
        dateEnd = moment(dateEnd, "DD/MM/YYYY");
        //Diferença data inicial -> data final
        intervalCiclo = dateEnd.diff(dateStart, "days");
        data = [];
        date = dateIn.format("DD/MM/YYYY");
        identificador = name;
    };

    this.salveFreqCiclo = function(widget) {
        var row = widget.currentWidget.currentRow;
        var nameWidget = widget.currentWidget.name;
        if (row.INTERVAL.start && row.INTERVAL.end && row.MULTIPLECICLO) {
            switch (nameWidget) {
                case "frequenciaciclogeral_diario":
                    if (row.DURACAOCICLODIAS) {
                        this.salvaDiario(row, widget);
                    } else {
                        MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
                    }
                    break;
                case "frequenciaciclogeral_semanal":
                    this.salvaSemanal(row, widget);
                    break;
                case "frequenciaciclogeral_quinzenal":
                    this.salvaQuinzenal(row, widget);
                    break;
                case "frequenciaciclogeral_mensal":
                    this.salvaMensal(row, widget);
                    break;
                default:
                    MessageService.showMessageByCode("ERRO_INESPERADO");
            }
        } else {
            MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.salvaDiario = function(row, widget) {

        self.iniciaValues(row, "Diario");

        intervalCiclo += 1;
        if (intervalCiclo < row.DURACAOCICLODIAS) {
            MessageService.showMessageByCode("COC00900_DURACAO_MAIOR_Q_PERMITIDO", {
                "0": intervalCiclo
            });
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            var cdclasse = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow.CDCLASSE;
            exibeDate = exibeDate.format("L");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var qtdCiclo = 0;
                var maxCiclo = row.MULTIPLECICLO.length;
                var duracaoCiclo = row.DURACAOCICLODIAS;
                var alterCiclo = 0;
                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaSemanal = function(row, widget) {

        self.iniciaValues(row, "Semanal");

        var editInterval = intervalCiclo;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 0;
        var cdclasse = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow.CDCLASSE;
        var duracaoCiclo = row.DURACAOCICLO * 7;
        editInterval /= 7;
        intervalCiclo += 1;
        editInterval = Math.round(editInterval);

        if (intervalCiclo < 7) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_PERIODO_MENOR");
        } else if (editInterval < row.DURACAOCICLO) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_DURACAO_MAIOR");
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");

            exibeDate = exibeDate.format("L");
            exibeDateFn = exibeDateFn.format("L");

            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var diaSemana = row.TROCASEMANAL.toString();
                var diaInicial = dateIn.format("d");

                while (diaSemana.toUpperCase() != diaInicial.toUpperCase()) {
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    date = moment(alterDate, "DD/MM/YYYY");
                    diaInicial = date.format("d");
                }

                diaInicial = moment(date, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(diaInicial, "days") + 1;
                date = diaInicial.format("DD/MM/YYYY");

                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse);
                self.salvaFrequencia(data, widget);

            });
        }
    };

    this.salvaQuinzenal = function(row, widget) {

        self.iniciaValues(row, "Quinzenal");

        var dtInTroca = row.INQUINZENAL;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 1;
        var dataInicial = dateIn.format("DD");
        var duracaoCiclo = 15;
        var cdclasse = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow.CDCLASSE;

        for (let value = 2; value <= dtInTroca; value++) {
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = moment(alterDate, "DD/MM/YYYY");
            dataInicial = date.format("DD");
        }
        dataInicial = moment(date, "DD/MM/YYYY");
        intervalCiclo = dateEnd.diff(dataInicial, "days") + 1;
        date = dataInicial.format("DD/MM/YYYY");
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse);
            self.salvaFrequencia(data, widget);
        });
    };

    this.salvaMensal = function(row, widget) {

        self.iniciaValues(row, "Mensal");

        var opcaoTroca = row.RADIOTROCA;
        var trocaDia = row.INQUINZENAL;
        var alterDia = 0;
        var trocaDiaAnt = trocaDia;
        var duracaoCiclo = row.DURACAOCICLO;
        var maxCiclo = row.MULTIPLECICLO.length;
        var intervalMensal = 0;
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        var cdclasse = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow.CDCLASSE;
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");

        dateStart = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            if (opcaoTroca == 1) {
                //Troca Diário 
                while (trocaDia != alterDia) {
                    var newDate = moment(dateIn, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    dateIn = alterDate;
                    alterDia = newDate.format("DD");
                    var monthInterval = moment(dateIn, "DD/MM/YYYY");
                    var month = moment(monthInterval, "DD/MM/YYYY");
                    monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                    var maxDay = moment(monthInterval, "DD/MM/YYYY");
                    var maxDayFn = maxDay.format("DD");
                    if (trocaDia >= maxDayFn) {
                        trocaDia = maxDayFn;
                    }

                }
                dateIn = moment(dateIn, "DD/MM/YYYY");
                dateIn = dateIn.subtract(1, "days").format("DD/MM/YYYY");
                dateIn = moment(dateIn, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(dateIn, "days") + 1;
                date = dateIn.format("DD/MM/YYYY");
                intervalMensal = dateEnd.diff(dateStart, "month");
                identificador = "MensalDia";

                self.salvaMensalporDiaSemana(intervalCiclo, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse);

            } else if (opcaoTroca == 2) {
                //Troca Semanal 
                /* row.id -> 1 - Primeiro(a)
	 2 -  Segundo(a)
	 3 - Terceiro(a)
	 4 -   Quarto(a)
*/

                var trocaSemana = row.id;
                var startOfMonth = angular.copy(dateIn);
                var studyDate = moment(startOfMonth.format("DD/MM/YYYY"), "DD/MM/YYYY").add(trocaSemana, "w");
                var verificaDate = studyDate.format("DD/MM/YYYY");

                startOfMonth.utc().startOf("month").startOf("week").add(row.idtroca, "days");

                if (studyDate.month() == startOfMonth.month()) {
                    studyDate = studyDate.subtract(1, "w");
                }

                verificaDate = moment(verificaDate, "DD/MM/YYYY");

                if (verificaDate < dateIn) {
                    dateIn = dateIn.add(1, "month");
                    var startOfMonthIf = dateIn.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                    var studyDateIf = moment(startOfMonthIf).add(trocaSemana, "w");
                    if (studyDateIf.month() == startOfMonthIf.month()) {
                        studyDateIf = studyDateIf.subtract(1, "w");
                    }
                }

                intervalMensal = dateEnd.diff(studyDate, "month");
                date = studyDate.format("DD/MM/YYYY");
                identificador = "MensalSemana";
                self.salvaMensalporDiaSemana(intervalCiclo + 1, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse);
            }
            self.salvaFrequencia(data, widget);
        });

    };

    this.salvaMensalporDiaSemana = function(intervalCiclo, cdciclo, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse) {

        var qtdCiclo = 0;
        var alterCiclo = 0;
        var qtdMes = 0;
        var trocaSemana = row.id;
        var permiteTroca = 0;

        if (intervalMensal === 0 || maxCiclo == 1 || (duracaoCiclo > intervalMensal)) {
            identificador = "Mensal";
            self.preencheData(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse);
        } else {
            for (let aux = 0; aux <= intervalCiclo; aux++) {
                if (identificador == "MensalDia") {
                    if (trocaDia == dateTroca) {
                        qtdMes++;
                        if (qtdMes == duracaoCiclo) {
                            qtdCiclo++;
                            qtdMes = 0;
                        }
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                } else if (identificador == "MensalSemana") {
                    if (intervalMensal > 0) {
                        if (permiteTroca) {
                            var startOfMonth = alterDate.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                            var studyDate = moment(startOfMonth).add(trocaSemana, "w");
                            if (studyDate.month() == startOfMonth.month()) {
                                studyDate = studyDate.subtract(1, "w");
                            }
                            studyDate = studyDate.format("DD/MM/YYYY");
                            if (date == studyDate) {
                                qtdCiclo++;
                            }
                            if (qtdCiclo == maxCiclo) {
                                qtdCiclo = 0;

                            }
                        }
                    }
                }
                var rows = {
                    CDCICLO: row.MULTIPLECICLO[qtdCiclo],
                    CDCIA: cdCia,
                    CDTRECHO: cdtrecho,
                    NRVOO: nrvoo,
                    CDFILIAL: Filial,
                    NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                    DTCICLOVCLA: date,
                    CDCLASSE: cdclasse,
                    __is_new: true
                };
                var newDate = moment(date, "DD/MM/YYYY");
                var alterDate = newDate.add(1, "days").format("DD/MM/YYYY");
                date = alterDate;
                alterDate = moment(alterDate, "DD/MM/YYYY");
                data.push(rows);
                var dateTroca = newDate.format("DD");
                var maxTrocaMes = moment(date, "DD/MM/YYYY");
                maxTrocaMes = moment(newDate).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(maxTrocaMes, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                intervalMensal = newDate.startOf("month").diff(dateIn.startOf("month"), "month");
                permiteTroca++;
                if (maxDayFn < trocaDia) {
                    trocaDiaAnt = trocaDia;
                    trocaDia = maxDayFn;
                } else if (maxDayFn >= row.INQUINZENAL) {
                    trocaDia = row.INQUINZENAL;
                } else {
                    trocaDia = trocaDiaAnt;
                }
            }
        }

    };

    this.preencheData = function(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse) {
        var dtInicialQuin = moment(date, "DD/MM/YYYY");
        var dtFinalQuin = angular.copy(dtInicialQuin).add(15, "days");
        dtFinalQuin = dtFinalQuin.format("DD");
        dtInicialQuin = dtInicialQuin.format("DD");
        var permiteTroca = 0;
        for (let aux = 0; aux < intervalCiclo; aux++) {
            if (identificador == "Diario" || identificador == "Semanal") {
                if (alterCiclo == duracaoCiclo) {
                    alterCiclo = 0;
                    qtdCiclo++;
                    if (qtdCiclo == maxCiclo) {
                        qtdCiclo = 0;
                    }
                }
            } else if (identificador == "Quinzenal") {
                var alterDateQuin = moment(date, "DD/MM/YYYY");
                alterDateQuin = alterDateQuin.format("DD");
                if (permiteTroca) {
                    if (dtInicialQuin == alterDateQuin || dtFinalQuin == alterDateQuin) {
                        qtdCiclo++;
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                }
            }
            var rows = {
                CDCICLO: cdciclo[qtdCiclo],
                CDCIA: cdCia,
                CDFILIAL: Filial,
                NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                DTCICLOVCLA: date,
                CDCLASSE: cdclasse,
                NRVOO: nrvoo,
                CDTRECHO: cdtrecho,
                __is_new: true
            };
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = alterDate;
            data.push(rows);
            alterCiclo++;
            permiteTroca++;
        }
    };

    this.salvaFrequencia = function(data, widget) {
        coc00901_frequenciaciclovooService.novaFrequenciaCicloClasse(data).then(function() {
            MessageService.showMessageByCode("COC00900_CAD_AUTO_SUCES");
            var setWidget = widget.container.getWidget("calendarClasse");
            ScreenService.closeSwipe();
            self.filtraRegistros(setWidget, setWidget.container.getWidget("coc00902_frequenciaciclo_voo_swipe").currentRow.year);
            setWidget.reload().then(function() {
                setWidget.activate();
                setWidget.view();
                operator = "modified";
            });
        }, function(calback) {
            ScreenService.showMessage(calback.data.error);
        });
    };

    this.preencheDataExcluir = function(widget) {
        var dataset = [];
        var currentRow = widget.currentRow;
        var swipeWidget = widget.container.getWidget("calendarClasse");
        var parentMasterWidget = widget.container.getWidget("coc00902_frequenciaciclovclasse");
        cdclasse = parentMasterWidget.currentRow.CDCLASSE;
        self.iniciaValues(currentRow, widget.name);
        intervalCiclo += 1;
        if (!currentRow.INTERVAL) {
            MessageService.showMessageByCode("COC00900_PREENCHA_INTERVALO_EXCL");
        } else if (!currentRow.MULTIPLECICLO) {
            MessageService.showMessageByCode("COC00900_CICLOS_N_INF");
        } else {
            if (currentRow.MULTIPLECICLO.length == 1) {
                for (let aux = 0; aux < intervalCiclo; aux++) {

                    var rows = {
                        CDCICLO: currentRow.MULTIPLECICLO[0],
                        CDCIA: cdCia,
                        NRVOO: nrvoo,
                        CDTRECHO: cdtrecho,
                        CDFILIAL: Filial,
                        NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                        DTCICLOVCLA: date,
                        CDCLASSE: cdclasse,
                        __is_new: false
                    };
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    date = alterDate;
                    dataset.push(rows);
                }
            } else {
                var qtdCiclo = currentRow.MULTIPLECICLO.length;
                qtdCiclo--;
                var ciclo = currentRow.MULTIPLECICLO;
                while (qtdCiclo >= 0) {
                    for (let cont = 0; cont <= intervalCiclo; cont++) {

                        var row = {
                            CDCICLO: ciclo[qtdCiclo],
                            CDCIA: cdCia,
                            NRVOO: nrvoo,
                            CDTRECHO: cdtrecho,
                            CDFILIAL: Filial,
                            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                            DTCICLOVCLA: date,
                            CDCLASSE: cdclasse,
                            __is_new: false

                        };
                        var newDateQtd = moment(date, "DD/MM/YYYY");
                        var alterDateQtd = moment(newDateQtd).add(1, "days").format("DD/MM/YYYY");
                        date = alterDateQtd;
                        dataset.push(row);
                    }
                    self.iniciaValues(currentRow, widget.name);
                    qtdCiclo--;
                }
            }
            self.excluiFrequencia(dataset, swipeWidget, dateStart, dateEnd);
        }
    };

    this.excluiFrequencia = function(data, widget, dateStart, dateEnd) {
        var exibeDate = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(dateEnd, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_EXCL_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            coc00901_frequenciaciclovooService.excluiFrequenciaCicloClasse(data).then(function() {
                MessageService.showMessageByCode("COC00900_EXCL_AUTO_SUCES");
                ScreenService.closePopup();
                var setWidget = widget.container.getWidget("calendarClasse");
                self.filtraRegistros(widget, setWidget.container.getWidget("coc00902_frequenciaciclo_voo_swipe").currentRow.year);
                setWidget.reload().then(function() {
                    setWidget.activate();
                    setWidget.view();
                    operator = "modified";
                });
            }, function(calback) {
                ScreenService.showMessage(calback.data.error);
            });
        });
    };

    this.setCicloColor = function(element, row) {
        if (row.NRCORCICLO) {
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            // var status = Util.isUndefinedOrNull(row.importStatus) ? row.status : row.importStatus;
            // var hasError = Util.isUndefinedOrNull(row.importHasError) ? row.hasError : row.importHasError;
            // var grid = $(".th.grid-align-left");
            // for (let i in grid) {
            // 	if (typeof grid[i] == "object") {
            // 		var attr = grid[i].getAttribute('data-zh-field-name');
            // 		if (attr && attr.search('EIXO_X_') != -1) {
            // 			console.log(grid[i]);
            // 		}
            // 	}
            // }
            // if (hasError) {
            // } else if (!hasError && status === 100) {
            //     element.append("<div class='circle-success'></div>");
            // } else if (!hasError && status >=0 && status < 100) {
            //     element.append("<div class='progress progress-striped import-progress active'> <div class='progress-bar import-progress-bar' role='progressbar' aria-valuenow='" + status + "' aria-valuemin='0' aria-valuemax='100' style='width: " + status + "%" + "'> </div> </div>");
            // }
        }
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc00902_frequenciaciclovclasseController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        // if (args.owner.currentRow.IDATIVO === "N") {
        //     MessageService.showMessageByCode("INACTIVE_CIA");
        // } else {
        //     args.owner.setSelected(args);
        // }
        args.owner.setSelected(args);
    };

    this.bloqueiaFieldFiltro = function(field, listaFieldName) {
        var contador = 0;
        var status;

        listaFieldName.forEach(function(fieldName) {
            if (field.value() && !contador) {
                status = false;
            } else {
                status = true;
            }

            field.widget.getField(fieldName).readOnly = status;
            field.widget.getField(fieldName).setValue("");
            contador++;
        });
    };

    this.filtraVoo = function(widget) {
        var widgetFrequenciaCicloVoo = widget.container.getWidget("coc00902_frequenciaciclovclasse").currentRow;
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetFrequenciaCicloVoo.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetFrequenciaCicloVoo.CDCIA
            },
            {
                "name": "CHAVE",
                "operator": "NOT_IN",
                "value": widgetFrequenciaCicloVoo.NRVOO + "_" + widgetFrequenciaCicloVoo.CDTRECHO + "_" + widgetFrequenciaCicloVoo.CDCLASSE
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        widget.getField("DSVOOTRECHOCLASSE").dataSourceFilter = filtro;
        widget.getField("DSVOOTRECHOCLASSE").reload();
    };

    this.openImportar = function(widget) {
        ScreenService.openPopup(widget.container.getWidget("importar"));
        widget.container.getWidget("importar").newRow();
    };

    this.importaFrequenciaCicloVooClasse = function(widget) {
        if (widget.isValid()) {
            if (moment(widget.currentRow.DTFIMVGMA, "DD/MM/YYYY").isBefore(moment(widget.currentRow.DTINIVGMA, "DD/MM/YYYY"))) {
                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {
                var currentRow = widget.currentRow;
                var widgetDestino = widget.container.getWidget("coc00902_frequenciaciclovclasse");
                currentRow.CDCIA = widgetDestino.currentRow.CDCIA;
                currentRow.NRVOODES = widgetDestino.currentRow.NRVOO;
                currentRow.CDTRECHODES = widgetDestino.currentRow.CDTRECHO;
                currentRow.CDCLASSEDES = widgetDestino.currentRow.CDCLASSE;
                currentRow.CDFILIAL = widgetDestino.currentRow.CDFILIAL;
                InflightorGeneral.callRestEngine("coc#/importaCicloVooClasse", "Row", currentRow).then(function() {
                    MessageService.showMessageByCode("COC00900_IMPORT_SUCES");
                    ScreenService.closePopup(true);
                    self.openSwipeClass(widget.container.getWidget("coc00902_frequenciaciclo_voo_swipe"), widgetDestino);
                }, function(err) {
                    ScreenService.showMessage(err.data.error);
                });
            }
        }
    };

    this.openExportar = function(widget) {
        ScreenService.openPopup(widget.container.getWidget("exportar"));
        widget.container.getWidget("exportar").newRow();
    };

    this.exportaFrequenciaCicloVooClasse = function(widgetDestino) {
        if (widgetDestino.isValid()) {
            if (moment(widgetDestino.currentRow.DTFIMVGMA, "DD/MM/YYYY").isBefore(moment(widgetDestino.currentRow.DTINIVGMA, "DD/MM/YYYY"))) {
                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {
                var widgetOrigem = widgetDestino.container.getWidget("coc00902_frequenciaciclovclasse");
                widgetOrigem.currentRow.CDCIA = widgetDestino.getField("DSVOOTRECHOCLASSE").dataSource.data[0].CDCIA;
                widgetOrigem.currentRow.CHAVE = widgetDestino.currentRow.DSVOOTRECHOCLASSE;
                widgetOrigem.currentRow.DTINIVGMA = widgetDestino.currentRow.DTINIVGMA;
                widgetOrigem.currentRow.DTFIMVGMA = widgetDestino.currentRow.DTFIMVGMA;
                widgetOrigem.currentRow.CDTRECHODES = widgetDestino.currentRow.CDTRECHO;
                widgetOrigem.currentRow.CDFILIAL = widgetDestino.getField("DSVOOTRECHOCLASSE").dataSource.data[0].CDFILIAL;
                InflightorGeneral.callRestEngine("coc#/exportaCicloVooClasse", "Row", widgetOrigem.currentRow).then(function() {
                    MessageService.showMessageByCode("COC00900_EXPORT_SUCES");
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe();
                    widgetOrigem.activate();
                }, function(err) {
                    ScreenService.showMessage(err.data.error);
                });
            }
        }
    };

    this.closePopup = function(widget) {
        ScreenService.closePopup(true);
        widget.container.getWidget("calendarClasse").view();
        widget.container.getWidget("calendarClasse").activate();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00902_frequenciaciclovclasseController", coc00902_frequenciaciclovclasseController);
});

// FILE: ./assets/js/controllers/coc00903_frequenciacicloservico_controller.js
var coc00903_frequenciacicloservico_controller = function(ScreenService, InflightorGeneral, Colors, coc00901_frequenciaciclovooService, coc00800_personalizacompanhiaService, restEngine, MessageService, templateManager) {

    var Filial;
    var cdCia;
    var self = this;
    var dateStart;
    var intervalCiclo;
    var data = [];
    var dateIn;
    var dateEnd;
    var date;
    var identificador;
    var cdtrecho;
    var nrvoo;
    var nrservicociafili;
    var cdciclo;
    var cdclasse;
    var operator = "";

    var mensagensGlobais = {};

    this.getMensagensGlobais = function() {
        MessageService.getMessagesByCode(['COC00900_DOM', 'COC00900_SEG', 'COC00900_TER',
                                          'COC00900_QUA', 'COC00900_QUI', 'COC00900_SEX', 'COC00900_SAB']).then(function(mensagens) {
            mensagensGlobais.dom = mensagens[0];
            mensagensGlobais.seg = mensagens[1];
            mensagensGlobais.ter = mensagens[2];
            mensagensGlobais.qua = mensagens[3];
            mensagensGlobais.qui = mensagens[4];
            mensagensGlobais.sex = mensagens[5];
            mensagensGlobais.sab = mensagens[6];
        });
    };

    this.clearCheckdRows = function(widget) {
        if (widget.dataSource.data) {
            widget.unSelectAll();
        }
    };

    this.selecionaCompanhia = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("calendarClasse");

        cdCia = current.CDCIA;
        Filial = current.CDFILIAL;

        gridWidget.getActions()[0].isVisible = true;
        gridWidget.activate();
        gridWidget.reload();
    };

    this.associarFrequencia = function(widget) {
        var widgetPopup = widget.container.getWidget("associa_frequencia");
        ScreenService.openPopup(widgetPopup);
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": widget.currentRow.CDTRECHO
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": widget.currentRow.NRVOO
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": widget.currentRow.CDCLASSE
            },
        ];
        widgetPopup.dataSourceFilter.data = filter;
        widgetPopup.activate();
        widgetPopup.reload();
    };

    this.openAssociacao = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        widget.container.getWidget("coc00903_frequenciacicloservico").currentRow = widget.currentRow;
        ScreenService.closePopup(widget.container.getWidget("associa_frequencia"));
        swipeWidget.parent = widget.container.getWidget("associa_frequencia");
        ScreenService.openSwipe(swipeWidget);
        // var year = parseInt(swipeWidget.currentRow.year);
        // swipeWidget.activate();
        swipeWidget.widgets[0].edit();
        swipeWidget.fields[0].applyDefaultValue();
        swipeWidget.fields[1].applyDefaultValue();
        self.filtraCalendarServico(swipeWidget, moment().format("YYYY"));
        swipeWidget.parent = widget.container.getWidget("coc00903_frequenciacicloservico");
        operator = "";
        // swipeWidget.widgets[0].view();
    };

    this.UnidadeToCompanhia = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.CompanhiaToTrecho = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.setFilterForField = function(field) {
        var widget = field.widget;
        var currentRow = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow;
        var newFilter = [
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": currentRow.CDTRECHO
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": currentRow.NRVOO
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": currentRow.NRORG
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": currentRow.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentRow.CDCLASSE
            }
        ];
        widget.getField("YEARS").dataSourceFilter = newFilter;
        widget.getField("YEARS").reload();
    };

    this.setYearForField = function(widget) {
        widget.getField("year").setValue(widget.currentRow.YEARS);
        widget.getField("year").reload();
        self.filtraCalendarServico(widget, widget.currentRow.YEARS);
    };

    this.setFilterforClasse = function(widget) {
        var parentWidget = widget.container.getWidget("coc00903_frequenciacicloservico");
        var currentWidget = parentWidget.currentRow;
        var data = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": Filial
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentWidget.CDCIA
            },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": currentWidget.NRVOO
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": currentWidget.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentWidget.CDCLASSE
            },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": currentWidget.CDTRECHO
            }
        ];
        widget.dataSourceFilter = data;
        widget.reload();
    };

    function returnStringLang(value) {
        return ScreenService.i18n(value);
    }
    this.initDefaultMensal = function(widget) {
        self.setDataSelectDate(widget);
        widget.getField("TROCAINICIAL").applyDefaultValue();
        widget.getField("TROCAFINAL").applyDefaultValue();
        widget.currentRow.id = 1;
        widget.currentRow.idtroca = 0;
    };

    this.initDefaultSemanal = function(widget) {
        self.setDataSelectDate(widget);
        widget.currentRow.TROCASEMANAL = mensagensGlobais.dom;
    };

    this.setDataSelectDate = function(widget) {
        var data = [
            {
                "value": mensagensGlobais.dom,
                "id": 0
            },
            {
                "value": mensagensGlobais.seg,
                "id": 1
            },
            {
                "value": mensagensGlobais.ter,
                "id": 2
            },
            {
                "value": mensagensGlobais.qua,
                "id": 3
            },
            {
                "value": mensagensGlobais.qui,
                "id": 4
            },
            {
                "value": mensagensGlobais.sex,
                "id": 5
            },
            {
                "value": mensagensGlobais.sab,
                "id": 6
            }
        ];
        if (widget.name == "frequenciaciclogeral_semanal")
            widget.getField("TROCASEMANAL").dataSource.data = data;
        else {
            widget.getField("TROCAFINAL").dataSource.data = data;
        }
    };

    this.filtraRegistros = function(widget, year) {
        if (!year) {
            var newYear = new Date();
            self.filtraCalendarServico(widget, newYear.getFullYear());
        }
        self.filtraCalendarServico(widget, year);
    };

    this.openSwipe = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendarServico(swipeWidget, moment().format("YYYY"));
        operator = "";
    };

    this.openSwipeClass = function(swipeWidget, widget) {
        self.selecionaCompanhia(widget);
        ScreenService.openSwipe(swipeWidget);
        swipeWidget.activate();
        swipeWidget.widgets[0].view();
        self.filtraCalendarServico(swipeWidget, moment().format("YYYY"));
    };

    function filtraCicloBanco(dia, mes, ciclos) {
        return ciclos.filter(function(ciclo) {
            return ciclo.EIXO_X == dia && ciclo.EIXO_Y == mes;
        });
    }

    function pad(value, length) {
        return (value.toString().length < length) ? pad("0" + value, length) : value;
    }

    //Função que colore o calendario por serviço, ciclo  e classe
    this.filtraCalendarServico = function(widget, year) {
        var calendar = widget.container.getWidget("calendarClasse");
        var parentWidget = widget.container.getWidget("coc00903_frequenciacicloservico");
        var widgetAssociar = widget.container.getWidget("associa_frequencia");
        let cdfilial = parentWidget.appliedFilter.FILTER_ROW.CDFILIAL;
        cdCia = parentWidget.appliedFilter.FILTER_ROW.CDCIA;
        nrvoo = parentWidget.appliedFilter.FILTER_ROW.NRVOO;
        cdtrecho = parentWidget.appliedFilter.FILTER_ROW.CDTRECHO;
        nrservicociafili = parentWidget.currentRow.NRSERVICOCIAFILI ? parentWidget.currentRow.NRSERVICOCIAFILI : widgetAssociar.currentRow.NRSERVICOCIAFILI;
        cdclasse = parentWidget.currentRow.CDCLASSE ? parentWidget.currentRow.CDCLASSE : widgetAssociar.currentRow.CDCLASSE;

        var data = [];
        if (year) {
            var dateInit = moment(year, "YYYY").startOf("year").format("DD/MM/YYYY");
            var dateFinal = moment(year, "YYYY").endOf("year").format("DD/MM/YYYY");

            coc00901_frequenciaciclovooService.findFrequenciaCicloServico(cdCia, dateInit, dateFinal, nrvoo, cdtrecho, nrservicociafili, cdciclo, cdclasse, cdfilial).then(function(callback) {

                var ciclos = callback.dataset.frcicloserm_anual;
                for (let mes = 1; mes <= 12; mes++) {
                    for (let dia = 1; dia <= 31; dia++) {
                        var existeBanco = filtraCicloBanco(pad(dia, 2), pad(mes, 2), ciclos);
                        if (existeBanco.length) {
                            data.push({
                                EIXO_X: existeBanco[0].EIXO_X,
                                EIXO_Y: moment(existeBanco[0].EIXO_Y, "MM").format("MMM"),
                                NRCORCICLO: Colors.TColorToHex(existeBanco[0].NRCORCICLO),
                                IDCICLO: existeBanco[0].IDCICLO
                            });
                        } else {
                            data.push({
                                EIXO_X: pad(dia, 2),
                                EIXO_Y: moment(pad(mes, 2), "MM").format("MMM"),
                                NRCORCICLO: "",
                                IDCICLO: ""
                            });
                        }
                    }
                }
                calendar.dataSource.data = data;
                calendar.reload().then(function() {
                    calendar.activate();
                });
            });
        }
    };

    this.closeSwipe = function(widget) {
        var parentWidget = widget.container.getWidget("coc00903_frequenciacicloservico");
        widget.lastActiveWidget = parentWidget;
        ScreenService.closeSwipe().then(function() {
            if (operator == "modified")
                parentWidget.reload();
            parentWidget.activate();
        });
    };

    this.addCiclo = function(widget) {

        var gridWidget = widget.container.getWidget("popup_cadastro");
        var associaFrequencia = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow;

        var widgetsNames = ["diario", "semanal", "quinzenal", "mensal"];

        widgetsNames.forEach(function(nome) {
            var currentWidget = widget.container.getWidget("frequenciaciclogeral_" + nome);
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia ? cdCia : widget.container.getWidget("coc00903_frequenciacicloservico").appliedFilter.FILTER_ROW.CDCIA;
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[3].value = associaFrequencia.NRSERVICOCIAFILI;
            currentWidget.getField("MULTIPLECICLO").dataSourceFilter[4].value = associaFrequencia.CDCLASSE;
            currentWidget.getField("MULTIPLECICLO").reload();
            currentWidget.newRow();
        });
        ScreenService.openSwipe(gridWidget);

    };

    this.openExclui = function(widget) {

        var formWidget = widget.container.getWidget("popup_excluir");
        var associaFrequencia = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow;
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[1].value = cdCia;
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[3].value = associaFrequencia.NRSERVICOCIAFILI;
        formWidget.getField("MULTIPLECICLO").dataSourceFilter[4].value = associaFrequencia.CDCLASSE;
        formWidget.getField("MULTIPLECICLO").reload();
        formWidget.newRow();
        formWidget.reload();
        ScreenService.openPopup(formWidget);
    };

    this.cancelExcluir = function(widget) {
        ScreenService.closePopup();
        var swipeWidget = widget.container.getWidget("calendarClasse");
        swipeWidget.activate();
    };

    this.cancelAdd = function(widget) {
        ScreenService.closeSwipe();
        var swipeWidget = widget.container.getWidget("calendarClasse");
        swipeWidget.activate();
    };

    this.reloadDias = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                valueDay.setValue(0);
                valueDay.reload();
            }
        }
    };

    this.validaQtdDias = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                dateIn = date.start;
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;
            }
        }
    };

    this.setFnQuinzenal = function(widget) {
        var interval = widget.getField("INTERVAL");
        var date = interval.value();
        var valueTroca = widget.getField("INQUINZENAL").value();
        if (valueTroca) {
            widget.getField("FNQUINZENAL").setValue(valueTroca + 15);
            widget.getField("FNQUINZENAL").reload();
        }
    };

    this.subDate = function(widget) {
        var interval = widget.getField("INTERVAL");
        var valueDay = widget.getField("INQUINZENAL");
        var date = interval.value();
        if (date) {
            var dateIn = date.start;
            var dateFn = date.end;
            if (dateIn && dateFn) {
                //Verifica o maior valor valido de acordo com o mês
                var monthInterval = moment(dateIn, "DD/MM/YYYY");
                var month = moment(monthInterval, "DD/MM/YYYY");
                monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(monthInterval, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                valueDay.maxValue = maxDayFn;

                //Diferença entre DateIn e DateFn
                var finalDate = moment(dateFn, "DD/MM/YYYY");
                var startDate = moment(dateIn, "DD/MM/YYYY");
                var difDate = finalDate.diff(startDate, "days");
                if (widget.name == "frequenciaciclogeral_quinzenal") {
                    if (difDate < 15) {
                        MessageService.showMessageByCode("COC00900_CICLO_QUINZENAL_PERIODO_MENOR");
                    }
                }
                if (widget.name == "frequenciaciclogeral_mensal" && difDate < 29) {
                    MessageService.showMessageByCode("COC00900_CICLO_MENSAL_PERIODO_MENOR");
                }
            }
        }

    };

    this.filtraTroca = function(widget) {

        var filtraRADIO = widget.getField("RADIOTROCA");
        var semanaInField = widget.getField("TROCAINICIAL");
        var semanaFnField = widget.getField("TROCAFINAL");
        var diaField = widget.getField("INQUINZENAL");
        var itemSelect = filtraRADIO.value();

        if (itemSelect == 1) {
            //Exibe DIAMENSAL spin
            diaField.isVisible = true;
            semanaInField.isVisible = false;
            semanaFnField.isVisible = false;

        }
        if (itemSelect == 2) {
            //Exibe TROCAINICIAL e TROCAFINAL
            semanaInField.isVisible = true;
            semanaFnField.isVisible = true;
            diaField.isVisible = false;
        }

        filtraRADIO.reload();

    };

    this.setDefaultTab = function(tabbedWidget, name) {
        tabbedWidget.defaultTab = name;
    };

    this.iniciaValues = function(row, name) {
        dateStart = row.INTERVAL.start;
        dateEnd = row.INTERVAL.end;
        dateIn = moment(dateStart, "DD/MM/YYYY");
        dateEnd = moment(dateEnd, "DD/MM/YYYY");
        //Diferença data inicial -> data final
        intervalCiclo = dateEnd.diff(dateStart, "days");
        data = [];
        date = dateIn.format("DD/MM/YYYY");
        identificador = name;
    };

    this.preencheData = function(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador) {
        var dtInicialQuin = moment(date, "DD/MM/YYYY");
        var dtFinalQuin = angular.copy(dtInicialQuin).add(15, "days");
        dtFinalQuin = dtFinalQuin.format("DD");
        dtInicialQuin = dtInicialQuin.format("DD");
        var permiteTroca = 0;
        for (let aux = 0; aux < intervalCiclo; aux++) {
            if (identificador == "Diario" || identificador == "Semanal") {
                if (alterCiclo == duracaoCiclo) {
                    alterCiclo = 0;
                    qtdCiclo++;
                    if (qtdCiclo == maxCiclo) {
                        qtdCiclo = 0;
                    }
                }
            } else if (identificador == "Quinzenal") {
                var alterDateQuin = moment(date, "DD/MM/YYYY");
                alterDateQuin = alterDateQuin.format("DD");
                if (permiteTroca) {
                    if (dtInicialQuin == alterDateQuin || dtFinalQuin == alterDateQuin) {
                        qtdCiclo++;
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                }
            }
            var rows = {
                CDCICLO: cdciclo[qtdCiclo],
                CDCIA: cdCia,
                CDFILIAL: Filial,
                NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                DTCICLOSER: date,
                NRSERVICOCIAFILI: nrservicociafili,
                CDCLASSE: cdclasse,
                NRVOO: nrvoo,
                CDTRECHO: cdtrecho,
                __is_new: true
            };
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = alterDate;
            data.push(rows);
            alterCiclo++;
            permiteTroca++;
        }
    };

    this.preencheDataExcluir = function(widget) {
        var dataset = [];
        var currentRow = widget.currentRow;
        var rowToRemove = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow;
        var swipeWidget = widget.container.getWidget("calendarClasse");
        self.iniciaValues(currentRow, widget.name);

        if (!currentRow.INTERVAL.start || !currentRow.INTERVAL.end) {
            MessageService.showMessageByCode("COC00900_PREENCHA_INTERVALO_EXCL");
        } else {
            if (currentRow.MULTIPLECICLO.length === 0) {
                MessageService.showMessageByCode("COC00900_CICLOS_N_INF");
            } else {
                if (currentRow.MULTIPLECICLO.length == 1) {
                    for (let aux = 0; aux <= intervalCiclo; aux++) {

                        var rows = {
                            CDCICLO: currentRow.MULTIPLECICLO[0],
                            CDCIA: cdCia,
                            NRVOO: nrvoo,
                            CDTRECHO: cdtrecho,
                            CDFILIAL: Filial,
                            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                            DTCICLOSER: date,
                            NRSERVICOCIAFILI: rowToRemove.NRSERVICOCIAFILI,
                            CDCLASSE: rowToRemove.CDCLASSE,
                            __is_new: false
                        };
                        var newDate = moment(date, "DD/MM/YYYY");
                        var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                        date = alterDate;
                        dataset.push(rows);
                    }
                } else {
                    var qtdCiclo = currentRow.MULTIPLECICLO.length;
                    qtdCiclo--;
                    var ciclo = currentRow.MULTIPLECICLO;
                    while (qtdCiclo >= 0) {
                        for (let cont = 0; cont <= intervalCiclo; cont++) {
                            var row = {
                                CDCICLO: ciclo[qtdCiclo],
                                CDCIA: rowToRemove.CDCIA,
                                NRVOO: rowToRemove.NRVOO,
                                CDTRECHO: rowToRemove.CDTRECHO,
                                CDFILIAL: rowToRemove.CDFILIAL,
                                NRORG: rowToRemove.NRORG,
                                DTCICLOSER: date,
                                NRSERVICOCIAFILI: rowToRemove.NRSERVICOCIAFILI,
                                CDCLASSE: rowToRemove.CDCLASSE,
                                __is_new: false
                            };
                            var newDateQtd = moment(date, "DD/MM/YYYY");
                            var alterDateQtd = moment(newDateQtd).add(1, "days").format("DD/MM/YYYY");
                            date = alterDateQtd;
                            dataset.push(row);
                        }
                        self.iniciaValues(currentRow, widget.name);
                        qtdCiclo--;
                    }
                }
                self.excluiFrequencia(dataset, swipeWidget, dateStart, dateEnd);
            }
        }
    };

    this.excluiFrequencia = function(data, widget, dateStart, dateEnd) {
        var exibeDate = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(dateEnd, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_EXCL_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            coc00901_frequenciaciclovooService.excluiFrequenciaCicloServico(data).then(function() {
                MessageService.showMessageByCode("COC00900_EXCL_AUTO_SUCES");
                ScreenService.closePopup();
                var setWidget = widget.container.getWidget("calendarClasse");
                self.filtraRegistros(widget, setWidget.container.getWidget("coc00903_frequenciacicloservico_swipe").currentRow.year);
                setWidget.reload().then(function() {
                    setWidget.activate();
                    setWidget.view();
                    operator = "modified";
                });
            }, function(calback) {
                ScreenService.showMessage(calback.data.error);
            });
        });
    };

    this.setCicloColor = function(element, row) {
        if (row.NRCORCICLO) {
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
            element.css("border-bottom", "5px solid " + row.NRCORCICLO);
        }
    };

    this.filtraServico = function(field) {
        var currentRow = field.widget.container.getWidget("coc00903_frequenciacicloservico").appliedFilter.FILTER_ROW;

        var filtro = {
            "CDFILIAL": currentRow.CDFILIAL,
            "CDCIA": currentRow.CDCIA,
            "NRVOO": currentRow.NRVOO,
            "CDTRECHO": currentRow.CDTRECHO,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.bloqueiaFieldFiltro = function(field, listaFieldName) {
        var contador = 0;
        var status;

        listaFieldName.forEach(function(fieldName) {
            if (field.value() && !contador) {
                status = false;
            } else {
                status = true;
            }

            field.widget.getField(fieldName).readOnly = status;
            field.widget.getField(fieldName).setValue("");
            contador++;
        });
    };

    this.salveFreqCiclo = function(widget) {
        var row = widget.currentWidget.currentRow;
        var nameWidget = widget.currentWidget.name;
        cdCia = cdCia ? cdCia : widget.container.getWidget("coc00903_frequenciacicloservico").appliedFilter.FILTER_ROW.CDCIA;
        if (row.INTERVAL.start && row.INTERVAL.end && row.MULTIPLECICLO) {
            switch (nameWidget) {
                case "frequenciaciclogeral_diario":
                    if (row.DURACAOCICLODIAS) {
                        this.salvaDiario(row, widget);
                    } else {
                        MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
                    }
                    break;
                case "frequenciaciclogeral_semanal":
                    this.salvaSemanal(row, widget);
                    break;
                case "frequenciaciclogeral_quinzenal":
                    this.salvaQuinzenal(row, widget);
                    break;
                case "frequenciaciclogeral_mensal":
                    this.salvaMensal(row, widget);
                    break;
                default:
                    MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
            }
        } else {
            MessageService.showMessageByCode("COC00900_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.salvaDiario = function(row, widget) {

        self.iniciaValues(row, "Diario");

        intervalCiclo += 1;
        if (intervalCiclo < row.DURACAOCICLODIAS) {
            MessageService.showMessageByCode("COC00900_DURACAO_MAIOR_Q_PERMITIDO", {
                "0": intervalCiclo
            });
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
            var cdclasse = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCLASSE;
            var nrservicociafili = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRSERVICOCIAFILI;
            exibeDate = exibeDate.format("L");
            exibeDateFn = exibeDateFn.format("L");
            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var qtdCiclo = 0;
                var maxCiclo = row.MULTIPLECICLO.length;
                var duracaoCiclo = row.DURACAOCICLODIAS;
                var alterCiclo = 0;
                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse, nrservicociafili);
                self.salvaFrequencia(data, widget);
            });
        }
    };

    this.salvaSemanal = function(row, widget) {

        self.iniciaValues(row, "Semanal");

        var editInterval = intervalCiclo;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 0;
        var cdclasse = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCLASSE;
        var nrservicociafili = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRSERVICOCIAFILI;
        var duracaoCiclo = row.DURACAOCICLO * 7;
        editInterval /= 7;
        intervalCiclo += 1;
        editInterval = Math.round(editInterval);

        if (intervalCiclo < 7) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_PERIODO_MENOR");
        } else if (editInterval < row.DURACAOCICLO) {
            MessageService.showMessageByCode("COC00900_CICLO_SEMANAL_DURACAO_MAIOR");
        } else {
            var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
            var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");

            exibeDate = exibeDate.format("L");
            exibeDateFn = exibeDateFn.format("L");

            MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
                "0": exibeDate,
                "1": exibeDateFn
            }).then(function() {
                var diaSemana = row.TROCASEMANAL.toString();
                var diaInicial = dateIn.format("d");

                while (diaSemana != diaInicial) {
                    var newDate = moment(date, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    date = moment(alterDate, "DD/MM/YYYY");
                    diaInicial = date.format("d");
                }

                diaInicial = moment(date, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(diaInicial, "days") + 1;
                date = diaInicial.format("DD/MM/YYYY");

                self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse, nrservicociafili);
                self.salvaFrequencia(data, widget);

            });
        }
    };

    this.salvaQuinzenal = function(row, widget) {

        self.iniciaValues(row, "Quinzenal");

        var dtInTroca = row.INQUINZENAL;
        var qtdCiclo = 0;
        var maxCiclo = row.MULTIPLECICLO.length;
        var alterCiclo = 1;
        var dataInicial = dateIn.format("DD");
        var duracaoCiclo = 15;
        var cdclasse = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCLASSE;
        var nrservicociafili = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRSERVICOCIAFILI;

        for (let value = 2; value <= dtInTroca; value++) {
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = moment(alterDate, "DD/MM/YYYY");
            dataInicial = date.format("DD");
        }
        dataInicial = moment(date, "DD/MM/YYYY");
        intervalCiclo = dateEnd.diff(dataInicial, "days") + 1;
        date = dataInicial.format("DD/MM/YYYY");
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            self.preencheData(intervalCiclo, row.MULTIPLECICLO, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse, nrservicociafili);
            self.salvaFrequencia(data, widget);
        });
    };

    this.salvaMensal = function(row, widget) {

        self.iniciaValues(row, "Mensal");

        var opcaoTroca = row.RADIOTROCA;
        var trocaDia = row.INQUINZENAL;
        var alterDia = 0;
        var trocaDiaAnt = trocaDia;
        var duracaoCiclo = row.DURACAOCICLO;
        var maxCiclo = row.MULTIPLECICLO.length;
        var intervalMensal = 0;
        var exibeDate = moment(row.INTERVAL.start, "DD/MM/YYYY");
        var cdclasse = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCLASSE;
        var nrservicociafili = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRSERVICOCIAFILI;
        var exibeDateFn = moment(row.INTERVAL.end, "DD/MM/YYYY");

        dateStart = moment(dateStart, "DD/MM/YYYY");
        exibeDate = exibeDate.format("L");
        exibeDateFn = exibeDateFn.format("L");
        MessageService.confirmMessageByCode("COC00900_CONFIRM_GRAV_CICLO", {
            "0": exibeDate,
            "1": exibeDateFn
        }).then(function() {
            if (opcaoTroca == 1) {
                //Troca Diário 
                while (trocaDia != alterDia) {
                    var newDate = moment(dateIn, "DD/MM/YYYY");
                    var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
                    dateIn = alterDate;
                    alterDia = newDate.format("DD");
                    var monthInterval = moment(dateIn, "DD/MM/YYYY");
                    var month = moment(monthInterval, "DD/MM/YYYY");
                    monthInterval = moment(month).endOf("month").format("DD/MM/YYYY");
                    var maxDay = moment(monthInterval, "DD/MM/YYYY");
                    var maxDayFn = maxDay.format("DD");
                    if (trocaDia >= maxDayFn) {
                        trocaDia = maxDayFn;
                    }

                }
                dateIn = moment(dateIn, "DD/MM/YYYY");
                dateIn = dateIn.subtract(1, "days").format("DD/MM/YYYY");
                dateIn = moment(dateIn, "DD/MM/YYYY");
                intervalCiclo = dateEnd.diff(dateIn, "days") + 1;
                date = dateIn.format("DD/MM/YYYY");
                intervalMensal = dateEnd.diff(dateStart, "month");
                identificador = "MensalDia";

                self.salvaMensalporDiaSemana(intervalCiclo, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse, nrservicociafili);

            } else if (opcaoTroca == 2) {
                //Troca Semanal 
                /* row.id -> 1 - Primeiro(a)
2 -  Segundo(a)
3 - Terceiro(a)
4 -   Quarto(a)
*/
                var trocaSemana = row.id;
                var startOfMonth = angular.copy(dateIn);
                var studyDate = moment(startOfMonth.format("DD/MM/YYYY"), "DD/MM/YYYY").add(trocaSemana, "w");
                var verificaDate = studyDate.format("DD/MM/YYYY");

                startOfMonth.utc().startOf("month").startOf("week").add(row.idtroca, "days");

                if (studyDate.month() == startOfMonth.month()) {
                    studyDate = studyDate.subtract(1, "w");
                }

                verificaDate = moment(verificaDate, "DD/MM/YYYY");

                if (verificaDate < dateIn) {
                    dateIn = dateIn.add(1, "month");
                    var startOfMonthIf = dateIn.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                    var studyDateIf = moment(startOfMonthIf).add(trocaSemana, "w");
                    if (studyDateIf.month() == startOfMonthIf.month()) {
                        studyDateIf = studyDateIf.subtract(1, "w");
                    }
                }

                intervalMensal = dateEnd.diff(studyDate, "month");
                date = studyDate.format("DD/MM/YYYY");
                identificador = "MensalSemana";
                self.salvaMensalporDiaSemana(intervalCiclo + 1, row.MULTIPLECICLO, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse, nrservicociafili);
            }
            self.salvaFrequencia(data, widget);
        });

    };

    this.salvaMensalporDiaSemana = function(intervalCiclo, cdciclo, maxCiclo, duracaoCiclo, trocaDia, trocaDiaAnt, intervalMensal, row, identificador, cdclasse, nrservicociafili) {

        var qtdCiclo = 0;
        var alterCiclo = 0;
        var qtdMes = 0;
        var trocaSemana = row.id;
        var permiteTroca = 0;

        if (intervalMensal === 0 || maxCiclo == 1 || (duracaoCiclo > intervalMensal)) {
            identificador = "Mensal";
            self.preencheData(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse, nrservicociafili);
        } else {
            for (let aux = 0; aux <= intervalCiclo; aux++) {
                if (identificador == "MensalDia") {
                    if (trocaDia == dateTroca) {
                        qtdMes++;
                        if (qtdMes == duracaoCiclo) {
                            qtdCiclo++;
                            qtdMes = 0;
                        }
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                } else if (identificador == "MensalSemana") {
                    if (intervalMensal > 0) {
                        if (permiteTroca) {
                            var startOfMonth = alterDate.utc().startOf("month").startOf("week").add(row.idtroca, "days");
                            var studyDate = moment(startOfMonth).add(trocaSemana, "w");
                            if (studyDate.month() == startOfMonth.month()) {
                                studyDate = studyDate.subtract(1, "w");
                            }
                            studyDate = studyDate.format("DD/MM/YYYY");
                            if (date == studyDate) {
                                qtdCiclo++;
                            }
                            if (qtdCiclo == maxCiclo) {
                                qtdCiclo = 0;

                            }
                        }
                    }
                }
                var rows = {
                    CDCICLO: row.MULTIPLECICLO[qtdCiclo],
                    CDCIA: cdCia,
                    CDTRECHO: cdtrecho,
                    NRVOO: nrvoo,
                    CDFILIAL: Filial,
                    NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                    DTCICLOSER: date,
                    NRSERVICOCIAFILI: nrservicociafili,
                    CDCLASSE: cdclasse,
                    __is_new: true
                };
                var newDate = moment(date, "DD/MM/YYYY");
                var alterDate = newDate.add(1, "days").format("DD/MM/YYYY");
                date = alterDate;
                alterDate = moment(alterDate, "DD/MM/YYYY");
                data.push(rows);
                var dateTroca = newDate.format("DD");
                var maxTrocaMes = moment(date, "DD/MM/YYYY");
                maxTrocaMes = moment(newDate).endOf("month").format("DD/MM/YYYY");
                var maxDay = moment(maxTrocaMes, "DD/MM/YYYY");
                var maxDayFn = maxDay.format("DD");
                intervalMensal = newDate.startOf("month").diff(dateIn.startOf("month"), "month");
                permiteTroca++;
                if (maxDayFn < trocaDia) {
                    trocaDiaAnt = trocaDia;
                    trocaDia = maxDayFn;
                } else if (maxDayFn >= row.INQUINZENAL) {
                    trocaDia = row.INQUINZENAL;
                } else {
                    trocaDia = trocaDiaAnt;
                }
            }
        }

    };

    this.preencheData = function(intervalCiclo, cdciclo, qtdCiclo, maxCiclo, duracaoCiclo, alterCiclo, identificador, cdclasse, nrservicociafili) {
        var dtInicialQuin = moment(date, "DD/MM/YYYY");
        var dtFinalQuin = angular.copy(dtInicialQuin).add(15, "days");
        dtFinalQuin = dtFinalQuin.format("DD");
        dtInicialQuin = dtInicialQuin.format("DD");
        var permiteTroca = 0;
        for (let aux = 0; aux < intervalCiclo; aux++) {
            if (identificador == "Diario" || identificador == "Semanal") {
                if (alterCiclo == duracaoCiclo) {
                    alterCiclo = 0;
                    qtdCiclo++;
                    if (qtdCiclo == maxCiclo) {
                        qtdCiclo = 0;
                    }
                }
            } else if (identificador == "Quinzenal") {
                var alterDateQuin = moment(date, "DD/MM/YYYY");
                alterDateQuin = alterDateQuin.format("DD");
                if (permiteTroca) {
                    if (dtInicialQuin == alterDateQuin || dtFinalQuin == alterDateQuin) {
                        qtdCiclo++;
                        if (qtdCiclo == maxCiclo) {
                            qtdCiclo = 0;
                        }
                    }
                }
            }
            var rows = {
                CDCICLO: cdciclo[qtdCiclo],
                CDCIA: cdCia,
                CDFILIAL: Filial,
                NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                DTCICLOSER: date,
                NRSERVICOCIAFILI: nrservicociafili,
                CDCLASSE: cdclasse,
                NRVOO: nrvoo,
                CDTRECHO: cdtrecho,
                __is_new: true
            };
            var newDate = moment(date, "DD/MM/YYYY");
            var alterDate = moment(newDate).add(1, "days").format("DD/MM/YYYY");
            date = alterDate;
            data.push(rows);
            alterCiclo++;
            permiteTroca++;
        }
    };

    this.salvaFrequencia = function(data, widget) {
        var widgetFrequenciacicloservico = widget.container.getWidget("coc00903_frequenciacicloservico");
        widgetFrequenciacicloservico.currentRow = data[0];
        coc00901_frequenciaciclovooService.novaFrequenciaCicloServico(data).then(function() {
            MessageService.showMessageByCode("COC00900_CAD_AUTO_SUCES");
            var setWidget = widget.container.getWidget("calendarClasse");
            ScreenService.closeSwipe();
            self.filtraRegistros(setWidget, setWidget.container.getWidget("coc00903_frequenciacicloservico_swipe").currentRow.year);
            setWidget.reload().then(function() {
                setWidget.activate();
                setWidget.view();
                operator = "modified";
            });
        }, function(calback) {
            ScreenService.showMessage(calback.data.error);
        });
    };

    this.abrePopupImportacao = function(widget) {
        var rowWidgetItemFaturamento = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow;
        var widgetPopupImportacao = widget.container.getWidget("popup_importacao");
        var fieldItemFaturamento = widgetPopupImportacao.getField("DSITEMFATURAMENTO");
        widgetPopupImportacao.newRow();

        var filtroItemFaturamentoImportacao = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowWidgetItemFaturamento.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": rowWidgetItemFaturamento.CDCIA
            },
            {
                "name": "CHAVE",
                "operator": "<>",
                "value": rowWidgetItemFaturamento.NRVOO + "_" + rowWidgetItemFaturamento.NRSERVICOCIAFILI
            }
        ];

        fieldItemFaturamento.dataSourceFilter = filtroItemFaturamentoImportacao;
        fieldItemFaturamento.reload();
        ScreenService.openPopup(widgetPopupImportacao);
    };

    this.importaFrequenciaCiclo = function(widget) {
        if (widget.isValid()) {
            if (moment(widget.currentRow.DTFINAL, "DD/MM/YYYY").isBefore(moment(widget.currentRow.DTINICIAL, "DD/MM/YYYY"))) {

                MessageService.showMessageByCode("COC00900_PERIODO_INV");

            } else {

                var widgetTabbed = widget.container.getWidget("coc00903_frequenciacicloservico_swipe");
                var rowPopup = widget.currentRow;
                var itemFaturamentoDestino = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCIA;
                var vooDestino = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRVOO;
                var trechoDestino = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDTRECHO;
                var servicoDestino = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.NRSERVICOCIAFILI;
                var classeDestino = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDCLASSE;
                var cdfilial = widget.container.getWidget("coc00903_frequenciacicloservico").currentRow.CDFILIAL;

                var rowParaImportacao = rowPopup;
                rowParaImportacao.CDCIADESTINO = itemFaturamentoDestino;
                rowParaImportacao.NRVOODESTINO = vooDestino;
                rowParaImportacao.CDTRECHODESTINO = trechoDestino;
                rowParaImportacao.NRSERVICOCIAFILIDESTINO = servicoDestino;
                rowParaImportacao.CDCLASSEDESTINO = classeDestino;
                rowParaImportacao.CDFILIAL = cdfilial;

                restEngine.post("coc#/importaFrequenciaItemFaturamento", {
                    "requestType": "Row",
                    "row": rowParaImportacao
                }).then(function() {
                    MessageService.showMessageByCode("COC00900_IMPORT_SUCES");
                    widget.container.getWidget("calendarClasse").view();
                    self.filtraCalendarServico(widgetTabbed, widgetTabbed.currentRow.year);
                    ScreenService.closePopup(true);
                }, InflightorGeneral.treatAndShowError);
            }
        }
    };

    this.abrePopupExportacao = function(widget) {

        var rowFiltro = widget.appliedFilter.FILTER_ROW;
        var widgetPopupExportacao = widget.container.getWidget("popup_exportacao");
        var fieldAnoDestino = widgetPopupExportacao.getField("ANOS_DIPONIVEIS");
        var fieldVoosDestino = widgetPopupExportacao.getField("DSVOOTRECHO");

        widgetPopupExportacao.newRow();

        var filtroAnosExportacao = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowFiltro.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": rowFiltro.CDCIA
            }
        ];

        var filtroVooExportacao = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowFiltro.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": rowFiltro.CDCIA
            },
            {
                "name": "CHAVE",
                "operator": "<>",
                "value": rowFiltro.NRVOO + "_" + rowFiltro.CDTRECHO
            }
        ];

        fieldAnoDestino.dataSourceFilter = filtroAnosExportacao;
        fieldVoosDestino.dataSourceFilter = filtroVooExportacao;
        fieldAnoDestino.reload();
        fieldVoosDestino.reload();
        widgetPopupExportacao.selecionados = templateManager.container.getWidget("coc00903_frequenciacicloservico").dataSource.getCheckedRows();
        ScreenService.openPopup(widgetPopupExportacao);
    };

    this.exportaFrequenciaCiclo = function(widget) {

        if (widget.isValid()) {

            var rowPopUp = widget.currentRow;
            var itensFaturamento = widget.container.getWidget("coc00903_frequenciacicloservico");
            var itensFaturamentoSelecionados = widget.selecionados;
            var voosDestino = widget.getField("DSVOOTRECHO").dataSource.getCheckedRows();

            var chavesOrigim = [];

            var dataSetDestino = [];

            itensFaturamentoSelecionados.forEach(function(row) {
                chavesOrigim.push(row.CHAVE);
            });

            var dataSetOrigem = {
                "CDFILIAL": itensFaturamentoSelecionados[0].CDFILIAL,
                "CDCIA": itensFaturamentoSelecionados[0].CDCIA,
                "NRVOO": itensFaturamentoSelecionados[0].NRVOO,
                "CDTRECHO": itensFaturamentoSelecionados[0].CDTRECHO,
                "CHAVE": chavesOrigim,
                "NRORG": itensFaturamentoSelecionados[0].NRORG,
                "DTINICIAL": moment(rowPopUp.ANOS_DIPONIVEIS, "YYYY").startOf("year").format("DD/MM/YYYY"),
                "DTFINAL": moment(rowPopUp.ANOS_DIPONIVEIS, "YYYY").endOf("year").format("DD/MM/YYYY")
            };

            voosDestino.forEach(function(row) {
                dataSetDestino.push({
                    "CDFILIAL": row.CDFILIAL,
                    "CDCIA": row.CDCIA,
                    "NRVOO": row.NRVOO,
                    "CDTRECHO": row.CDTRECHO,
                    "NRORG": row.NRORG,
                    "DTINICIAL": moment(rowPopUp.ANOS_DIPONIVEIS, "YYYY").startOf("year").format("DD/MM/YYYY"),
                    "DTFINAL": moment(rowPopUp.ANOS_DIPONIVEIS, "YYYY").endOf("year").format("DD/MM/YYYY")
                });
            });

            restEngine.post("coc#/exportaFrequenciaItemFaturamento", {
                "requestType": "Row",
                "row": {
                    "DATASETORIGEM": dataSetOrigem,
                    "DATASETDESTINO": dataSetDestino
                }
            }).then(function() {
                MessageService.showMessageByCode("COC00900_EXPORT_SUCES");
                itensFaturamento.unSelectAll();
                ScreenService.closePopup(true);
                ScreenService.closeSwipe();
                itensFaturamento.activate();
            }, InflightorGeneral.treatAndShowError);

        }

    };

    this.habilitaFieldData = function(field) {
        var row = field.widget.currentRow;
        if (field.widget.currentRow.DTINICIAL) {
            habilitaFields(row, field, ["DTFINAL"], true);
        } else {
            habilitaFields(row, field, ["DTFINAL"], false);
        }

    };

    var habilitaFields = function(row, field, fieldsDependentes, habilita) {
        var filho;
        if (habilita) {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = false;
            }
        } else {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = true;
                field.widget.getField(fieldsDependentes[filho]).value("");
            }
        }
    };

    this.closePopup = function(widget) {
        ScreenService.closePopup(true);
        widget.container.getWidget("calendarClasse").activate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00903_frequenciacicloservico_controller", coc00903_frequenciacicloservico_controller);
});

// FILE: ./assets/js/controllers/coc00991_listagem_de_receitaController.js
var coc00991_listagem_de_receitaController = function(InflightorGeneral, ScreenService, templateManager, MessageService) {

    this.limpaDependentesCompanhia = function(field, camposDependentes) {
        var currentRow = field.widget.currentRow;
        var filtro = {
            "name": "CDCIA",
            "operator": "=",
            "value": currentRow.TCIA
        };

        for (let campo in camposDependentes) {
            field.widget.getField(camposDependentes[campo]).dataSourceFilter = field.widget.getField(camposDependentes[campo]).dataSourceFilter.filter(function(condicao) {
                return condicao.name != "CDCIA";
            });
            if (currentRow.TCIA == "T") {
                field.widget.getField(camposDependentes[campo]).value();
                currentRow[camposDependentes[campo]] = null;
                field.widget.getField(camposDependentes[campo]).dataSourceFilter.push(filtro);
                field.widget.getField(camposDependentes[campo]).reload();
            } else {
                field.widget.getField(camposDependentes[campo]).value();
                currentRow[camposDependentes[campo]] = null;
            }
        }

        InflightorGeneral.toggleReadOnlySelect(field, field.widget.getField("NMCIA"));

    };

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        args.owner.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
        args.owner.currentRow.NMFILIAL_SELECT = getLocalVar("INFLIGHTOR_NMFILIAL");
        this.FieldOnChangeReportName(args.owner, args.owner.currentRow);
        args.owner.newRow();
        args.owner.getField("IDSITUAPROPCIA").applyDefaultValue();
        args.owner.getField("NMFILIAL_SELECT").applyDefaultValue();
        args.owner.getField("CDFILIAL").applyDefaultValue();
        this.aplicaFiltroCompanhia(args.owner);
        this.aplicaFiltroManual(args.owner);
    };

    this.aplicaFiltroCompanhia = function(widget) {
        var currentRow = widget.currentRow;
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            }
        ];
        widget.getField("NMCIA").dataSourceFilter = filtro;
    };

    this.aplicaFiltroManual = function(widget) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.TCIA
            }
        ];
        widget.getField("DSMANUAL").dataSourceFilter = filtro;
    };

    this.verificaFiltro = function(args) {
        var filtros = args.owner.field.dataSourceFilter;
        filtros = filtros.filter(function(filtro) {
            return (filtro.name == "CDFILIAL" || filtro.name == "CDCIA") && filtro.value != "" && typeof filtro.value != "undefined" && filtro.value != null;
        });

        if (filtros.length != 2) {
            MessageService.showMessageByCode("FILL_UNIT");
            args.e.preventDefault();
        }
    };

    this.verificaFiltroCompanhia = function(args) {
        var filtros = args.owner.field.dataSourceFilter;
        filtros = filtros.filter(function(filtro) {
            return (filtro.name == "CDFILIAL") && filtro.value != "" && typeof filtro.value != "undefined" && filtro.value != null;
        });

        if (filtros.length < 1) {
            MessageService.showMessageByCode("FILL_UNIT");
            args.e.preventDefault();
        }
    };

    this.filtraReceita = function(field) {
        var menuGrids = listaMenuGrid(field.widget.getField("DSMANUAL"));
        field.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                "name": "IDSITUAPROPCIA",
                "operator": "=",
                "value": field.widget.currentRow.IDSITUAPROPCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": menuGrids
            }
        ];

        field.reload();
    };

    this.report = function(widget, popup, formato) {

        if (!widget.isValid()) { //Verifica se todos os campos estão preenchidos
            return;
        }

        var CDFILIAL = "";
        var CDRECEITA = "";
        var CDCIA = "";
        var rowMontada = {};
        var CDSETOR = "";
        var CDLEANCODE;
        var CDCATEGORIA;
        rowMontada.__format = formato;
        rowMontada.P_ZEBRADO = popup.currentRow.P_ZEBRADO;
        rowMontada.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
        rowMontada.REPORTNAME = widget.currentRow.REPORTNAME;
        rowMontada.IDSITUAPROPCIA = widget.currentRow.IDSITUAPROPCIA;

        if (widget.currentRow.NMFILIAL !== "") {
            CDFILIAL = "_";
            widget.currentRow.NMFILIAL.forEach(function(row) {
                CDFILIAL += row + "_";
            });
        }
        rowMontada.CDFILIAL = CDFILIAL;
        if (widget.currentRow.TFILIAL === "S") {
            if (!widget.currentRow.CDFILIAL) {
                rowMontada.CDFILIAL = "T";
            } else {
                rowMontada.CDFILIAL = widget.currentRow.CDFILIAL;
            }
        } else {
            if (widget.currentRow.NMFILIAL !== "") {
                CDFILIAL = "_";
                widget.currentRow.NMFILIAL.forEach(function(row) {
                    CDFILIAL += row + "_";
                });
            }
            rowMontada.CDFILIAL = CDFILIAL;
        }
        if (widget.currentRow.TCIA === "T") {
            rowMontada.CDCIA = "T";
        } else {
            if (widget.currentRow.NMCIA !== "") {
                CDCIA = "_";
                widget.currentRow.NMCIA.forEach(function(row) {
                    CDCIA += row + "_";
                });
            } else {
                if (!widget.currentRow.TCIA && !widget.currentRow.NMCIA) {
                    CDCIA = "T";
                }
            }
            rowMontada.CDCIA = CDCIA;
        }

        if (widget.currentRow.CDPRODUTO) {
            rowMontada.CDPRODUTO = widget.currentRow.CDPRODUTO;
        } else {
            rowMontada.CDPRODUTO = "";
        }

        if (widget.currentRow.TRECEITAM == "T") {
            rowMontada.CDRECEITA = "T";
        } else {
            if (widget.currentRow.DSRECEITA !== "") {
                CDRECEITA = "_";
                widget.currentRow.DSRECEITA.forEach(function(row) {
                    CDRECEITA += row + "_";
                });
            }
            rowMontada.CDRECEITA = CDRECEITA;
        }
        if (widget.currentRow.TSETOR === "S") {
            rowMontada.CDSETOR = "T";
        } else {
            if (widget.currentRow.NMSETOR !== "") {
                CDSETOR = "_";
                widget.currentRow.NMSETOR.forEach(function(row) {
                    CDSETOR += row + "_";
                });
            }
            rowMontada.CDSETOR = CDSETOR;
        }
        if (widget.currentRow.TCATEGORIA === "S") {
            rowMontada.CDCATEGORIA = "T";
        } else {
            if (widget.currentRow.NMCATEGORIA !== "") {
                CDCATEGORIA = "_";
                widget.currentRow.NMCATEGORIA.forEach(function(row) {
                    CDCATEGORIA += row + "_";
                });
            }
            rowMontada.CDCATEGORIA = CDCATEGORIA;
        }
        if (widget.currentRow.TLEANCODE === "S") {
            rowMontada.CDLEANCODE = "T";
        } else {
            if (widget.currentRow.NMLEANCODE !== "") {
                CDLEANCODE = "_";
                widget.currentRow.NMLEANCODE.forEach(function(row) {
                    CDLEANCODE += row + "_";
                });
            }
            rowMontada.CDLEANCODE = CDLEANCODE;
        }
        if (widget.currentRow.TMANCIA === "T") {
            rowMontada.CDCIACDMANCIA = "T";
        } else {
            rowMontada.CDCIACDMANCIA = this.getCdciaCdmancia(widget.currentRow.DSMANUAL);
        }
        if (widget.currentRow.REPORTNAME === "COC02600" && widget.currentRow.CDSUBRECEITA !== "")
            rowMontada.CDSUBRECEITA = widget.currentRow.CDSUBRECEITA;
        if (widget.currentRow.REPORTNAME === "COC02700" && widget.currentRow.UNPRODUTO !== "")
            rowMontada.UNPRODUTO = widget.currentRow.CDPRODUTO;
        if (widget.currentRow.REPORTNAME === "COC0I991")
            rowMontada.IDTIPOPERWFLOW = widget.currentRow.IDTIPOPERWFLOW;
        if (widget.currentRow.REPORTNAME === "COC02300") { //Relatório de Utilização de Receita na Companhia
            rowMontada.CDFILIAL = widget.currentRow.CDFILIAL;
            rowMontada.CDRECEITA = widget.currentRow.CDRECEITA;
        }
        if (widget.currentRow.REPORTNAME === "COC00992") { //Relatório de Utilização de Receita na Companhia
            rowMontada.CDFILIAL = widget.currentRow.CDFILIAL;
        }
        if (widget.currentRow.REPORTNAME === "COC0L991" || widget.currentRow.REPORTNAME === "COC0G991") {
            if (widget.currentRow.TFILIAL === "S")
                rowMontada.CDFILIAL = "T";
        }
        // rowMontada['CDCATEGORIA']  = 'T';
        rowMontada.IDPRODATIVPR = widget.currentRow.IDPRODATIVPR;

        InflightorGeneral.openReport("coc#/relatorioDeReceitas", widget.currentRow.REPORTNAME, rowMontada);

    };

    this.getCdciaCdmancia = function(dsmanual) {
        var rowsSelecionadas = "";
        dsmanual.forEach(function(item) {
            rowsSelecionadas += "_" + item.CDCIA + "-" + item.CDMANCIA;
        });
        rowsSelecionadas += "_";
        return rowsSelecionadas;
    };

    this.FieldOnChangeReportName = function(widget, row) {
        switch (row.REPORTNAME) {
            case "COC00991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0B991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = true;
                widget.getField("TSETOR").isVisible = true;
                widget.getField("NMCATEGORIA").isVisible = true;
                widget.getField("TCATEGORIA").isVisible = true;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0C991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = true;
                widget.getField("TSETOR").isVisible = true;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0J991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = true;
                widget.getField("TSETOR").isVisible = true;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0D991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = true;
                widget.getField("TSETOR").isVisible = true;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC00992":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = true;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = true;
                widget.getField("TRECEITAM").isVisible = true;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0E991":
                widget.getField("IDSITUAPROPCIA").isVisible = true;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = true;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("NMCIA").isVisible = true;
                widget.getField("TCIA").isVisible = true;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = true;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                break;

            case "COC0K991":
                widget.getField("IDSITUAPROPCIA").isVisible = true;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = true;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("NMCIA").isVisible = true;
                widget.getField("TCIA").isVisible = true;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = true;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                break;

            case "COC0F991":
            case "COC0H991": //Tempo de Mão de Obra por Receita
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = true;
                widget.getField("TRECEITAM").isVisible = true;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC0G991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                // widget.getField('NMFILIAL').isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                // widget.getField('TFILIAL').isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = true;
                widget.getField("TLEANCODE").isVisible = true;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                break;

            case "COC0L991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                // widget.getField('NMFILIAL').isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                // widget.getField('TFILIAL').isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = true;
                widget.getField("TLEANCODE").isVisible = true;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("TFILIAL").isVisible = true;
                break;

            case "COC0I991":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = true;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC02600":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = true;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC02700":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = true;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC02800":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;

            case "COC00993":
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = true;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TRECEITAM").isVisible = true;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("DSRECEITA").reload();
                break;

            case "COC02300":
                widget.getField("IDSITUAPROPCIA").isVisible = true;
                widget.getField("TMANCIA").isVisible = true;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = true;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = true;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("NMFILIAL_SELECT").isVisible = true;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("TFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = true;
                widget.getField("TCIA").isVisible = true;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                widget.getField("DSRECEITA").reload();
                break;

            default:
                widget.getField("IDSITUAPROPCIA").isVisible = false;
                widget.getField("TMANCIA").isVisible = false;
                widget.getField("DSRECEITAMG").isVisible = false;
                widget.getField("DSMANUAL").isVisible = false;
                widget.getField("DSRECEITA").isVisible = false;
                widget.getField("TRECEITAM").isVisible = false;
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("NMFILIAL_SELECT").isVisible = false;
                widget.getField("DSRECEITA_SELECT").isVisible = false;
                widget.getField("TFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = false;
                widget.getField("TCIA").isVisible = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("NMSETOR").isVisible = false;
                widget.getField("TSETOR").isVisible = false;
                widget.getField("NMCATEGORIA").isVisible = false;
                widget.getField("TCATEGORIA").isVisible = false;
                widget.getField("DSSUBRECEITA").isVisible = false;
                widget.getField("UNPRODUTO").isVisible = false;
                widget.getField("IDTIPOPERWFLOW").isVisible = false;
                widget.getField("IDPRODATIVPR").isVisible = false;
                break;
        }
        if (widget.getField("TFILIAL").isVisible) {
            widget.getField("TFILIAL").applyDefaultValue();
            widget.getField("NMFILIAL").readOnly = true;
            widget.getField("NMFILIAL").validations = {};
            widget.getField("NMFILIAL").setValue("");
        }
        if (widget.getField("TLEANCODE").isVisible) {
            widget.getField("TLEANCODE").applyDefaultValue();
            widget.getField("NMLEANCODE").readOnly = true;
            widget.getField("NMLEANCODE").validations = {};
            widget.getField("NMLEANCODE").setValue("");
        }
        if (widget.getField("TRECEITAM").isVisible) {
            widget.getField("TRECEITAM").applyDefaultValue();
            widget.getField("DSRECEITA").readOnly = true;
            widget.getField("DSRECEITA").validations = {};
            widget.getField("DSRECEITA").setValue("");
        }
        if (widget.getField("TMANCIA").isVisible) {
            widget.getField("TMANCIA").applyDefaultValue();
            widget.getField("DSMANUAL").readOnly = true;
            widget.getField("DSMANUAL").validations = {};
            widget.getField("DSMANUAL").setValue("");
        }
        if (widget.getField("TSETOR").isVisible) {
            widget.getField("TSETOR").applyDefaultValue();
            widget.getField("NMSETOR").readOnly = true;
            widget.getField("NMSETOR").validations = {};
            widget.getField("NMSETOR").setValue("");
        }
        if (widget.getField("TCATEGORIA").isVisible) {
            widget.getField("TCATEGORIA").applyDefaultValue();
            widget.getField("NMCATEGORIA").readOnly = true;
            widget.getField("NMCATEGORIA").validations = {};
            widget.getField("NMCATEGORIA").setValue("");
        }
        templateManager.updateTemplate();
    };

    this.fieldOnChangeFilial = function(field, camposDependentes) {
        var currentRow = field.widget.currentRow;
        var filtro = {
            "name": "CDFILIAL",
            "operator": "=",
            "value": field.widget.currentRow.CDFILIAL
        };
        for (let campo in camposDependentes) {
            field.widget.getField(camposDependentes[campo]).dataSourceFilter = field.widget.getField(camposDependentes[campo]).dataSourceFilter.filter(function(condicao) {
                return condicao.name != "CDFILIAL";
            });
            field.widget.getField(camposDependentes[campo]).value();
            currentRow[camposDependentes[campo]] = null;
            field.widget.getField(camposDependentes[campo]).dataSourceFilter.push(filtro);
            field.widget.getField(camposDependentes[campo]).reload();
        }
    };

    this.fieldOnChangeFilialMultiSelect = function(field, camposDependentes) {
        var currentRow = field.widget.currentRow;
        if (field.widget.currentRow.TFILIAL == "N") {
            field.widget.currentRow.CDFILIAL = "_" + field.widget.currentRow.NMFILIAL.join("_") + "_";
        } else {
            field.widget.currentRow.CDFILIAL = field.widget.currentRow.TFILIAL;
        }
        var filtro = {
            "name": "CDFILIAL",
            "operator": "=",
            "value": field.widget.currentRow.CDFILIAL
        };
        for (let campo in camposDependentes) {
            field.widget.getField(camposDependentes[campo]).dataSourceFilter = field.widget.getField(camposDependentes[campo]).dataSourceFilter.filter(function(condicao) {
                return condicao.name != "CDFILIAL";
            });
            field.widget.getField(camposDependentes[campo]).value();
            currentRow[camposDependentes[campo]] = null;
            field.widget.getField(camposDependentes[campo]).dataSourceFilter.push(filtro);
            field.widget.getField(camposDependentes[campo]).reload();

        }
    };

    this.fieldOnChangeCompanhia = function(field, camposDependentes) {
        var currentRow = field.widget.currentRow;
        if (field.widget.currentRow.TCIA == "N") {
            field.widget.currentRow.CDCIA = "_" + field.widget.currentRow.NMCIA.join("_") + "_";
        } else {
            field.widget.currentRow.CDCIA = field.widget.currentRow.TCIA;
        }
        var filtro = {
            "name": "CDCIA",
            "operator": "=",
            "value": field.widget.currentRow.CDCIA
        };
        for (let campo in camposDependentes) {
            field.widget.getField(camposDependentes[campo]).dataSourceFilter = field.widget.getField(camposDependentes[campo]).dataSourceFilter.filter(function(condicao) {
                return condicao.name != "CDCIA";
            });
            field.widget.getField(camposDependentes[campo]).value();
            currentRow[camposDependentes[campo]] = null;
            field.widget.getField(camposDependentes[campo]).dataSourceFilter.push(filtro);
            field.widget.getField(camposDependentes[campo]).reload();
        }
    };

    this.fieldOnChangeReceita = function(field) {
        field.widget.getField("DSMANUAL").clearValue();
        if (field.widget.currentRow.TCIA == "N") {
            field.widget.currentRow.CDCIA = "_" + field.widget.currentRow.NMCIA.join("_") + "_";
        } else {
            field.widget.currentRow.CDCIA = field.widget.currentRow.TCIA;
        }
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": field.widget.currentRow.CDCIA
            },
            {
                "name": "IDSITUAPROPCIA",
                "operator": "=",
                "value": field.widget.currentRow.IDSITUAPROPCIA
            }
        ];
        field.widget.getField("DSMANUAL").value();
        field.widget.currentRow.DSMANUAL = null;
        field.widget.getField("DSMANUAL").dataSourceFilter = filtro;
        field.widget.getField("DSMANUAL").reload();
    };

    this.FieldOnChangeCiaFili = function(args) {
        var msg = null;
        if (!args.row.NMFILIAL_SELECT) {
            if (args.row.NMFILIAL.length == 0)
                msg = "COC00991_CHOOSE_UNIT";
        }
        if (args.row.NMFILIAL.length > 1) {
            msg = "COC00991_CHOOSE_JUST_UNIT";
        }
        if (msg !== null) {
            MessageService.showMessageByCode(msg);
        }
        var filter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL ? args.row.CDFILIAL : args.row.NMFILIAL[0]
            }
        ];
        args.owner.owner.widget.getField("NMCIA").dataSourceFilter = filter;
        args.owner.owner.widget.getField("NMCIA").reload();
    };

    this.openCustomPopup = function(widget, popup) {
        if (widget.isValid()) {
            ScreenService.openPopup(popup);
        }
    };

    this.validaProduto = function(widget) {

        var status = widget.currentRow.IDPRODATIVPR;
        var filter;

        if (widget.currentRow.REPORTNAME == "COC0E991") {
            if (status != "T") {
                filter = [
                    {
                        name: "IDPRODATIVPR",
                        operator: "=",
                        value: status
                    },
                    {
                        name: "NRORG",
                        operator: "=",
                        value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                    },
                    {
                        name: "IDATIVO",
                        operator: "=",
                        value: "S"
                    }
                ];
            } else {
                filter = [
                    {
                        name: "NRORG",
                        operator: "=",
                        value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                    },
                    {
                        name: "IDATIVO",
                        operator: "=",
                        value: "S"
                    }
                ];
            }
        } else {
            filter = [
                {
                    name: "IDPRODATIVPR",
                    operator: "=",
                    value: "S"
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    name: "IDATIVO",
                    operator: "=",
                    value: "S"
                }
            ];
        }
        widget.getField("NMPRODUTO").dataSourceFilter = filter;
        widget.getField("NMPRODUTO").reload();
    };

    var listaMenuGrid = function(field) {

        var lista = "";
        var rows = field.selectWidget.getCheckedRows();

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i].CDMANCIA;
            } else {
                lista = lista + "_" + rows[i].CDMANCIA;
            }
        }
        return lista;
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00991_listagem_de_receitaController", coc00991_listagem_de_receitaController);
});

// FILE: ./assets/js/controllers/coc00999_relatorios_de_receitaController.js
var coc00999_relatorios_de_receitaController = function(coc00800_personalizacompanhiaService, metaDataFactory, JSONContainerLoader, templateManager, restEngine, ScreenService, ZhReports) {

    //Preenche o DataSource com "-" caso os campos estejam vazios
    this.preencheNMFILIAL = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (!row.NMCIA) {
                row.NMCIA = "-";
            }
            if (!row.NRNIVELDIFICULDADE) {
                row.NRNIVELDIFICULDADE = "-";
            }
            if (!row.NMCATEGORIA) {
                row.NMCATEGORIA = "-";
            }
        });
    };

    //*******************************GERAR RELATÓRIOS**********************************//
    var customReportFields = [{
        "id": 99999999,
        "name": "CUSTOMREPORTTYPE",
        "label": "",
        "class": 12,
        "readOnly": false,
        "isVisible": true,
        "isVisibleForm": true,
        "isVisibleGrid": false,
        "template": "field/radio-group.html",
        "events": [],
        "valueField": "value",
        "descriptionField": "name",
        "defaultOption": 0,
        "dataSource": {
            "data": [{
                "name": "PDF",
                "value": "PDF"
            }, {
                "name": "XLS",
                "value": "XLS"
            }]
        }
    }];

    var customReportActions = [{
        "name": "cancelar",
        "label": "Cancelar",
        "idVisible": true,
        "hideIcon": true,
        "showAsAction": "back",
        "events": [{
            "name": "ActionEvent",
            "code": "ScreenService.closePopup();"
        }]
    }, {
        "name": "confirmar",
        "label": "Confirmar",
        "idVisible": true,
        "hideIcon": true,
        "showAsAction": "never",
        "events": [{
            "name": "ActionEvent",
            "code": "coc00999_relatorios_de_receitaController.generateCustomReport(args.owner.widget);"
        }]
    }];

    var customReportWidget = {
        "id": 0,
        "name": "customReportWidget",
        "label": "Tipo de Relatório",
        "template": "widget/form.html",
        "container": {},
        "isVisible": true,
        "popupNoBlock": false,
        "fields": [],
        "actions": [],
        "events": []
    };

    var referenceWidget;
    var customReportParameters;

    this.openCustomReportPopup = function(widget, parameters) {

        customReportParameters = parameters || "";
        referenceWidget = widget;
        customReportWidget.container = widget.container;
        customReportWidget.fields = customReportFields;
        customReportWidget.actions = customReportActions;

        var customReportWidgetInitialized = metaDataFactory.widgetFactory(customReportWidget);

        ScreenService.openPopup(customReportWidgetInitialized);
    };

    this.generateCustomReport = function(widget) {

        var reportType = widget.currentRow.CUSTOMREPORTTYPE;
        var reportPath = "../" + customReportParameters;

        var successCallback = function() {
            ScreenService.closePopup();
        };

        var errorCallback = function(err) {
            ScreenService.showMessage(err);
        };

        var data = angular.copy(referenceWidget.dataSource.data);

        //coloca as labels ao inves de values no relatório
        referenceWidget.dataSource.data.forEach(function(row) {
            referenceWidget.getField("IDRECFLEXIVEL").dataSource.data.forEach(function(campo) {
                if (campo.value == row.IDRECFLEXIVEL)
                    row.IDRECFLEXIVEL = campo.label;
            });
            row.NRNIVELDIFICULDADE = referenceWidget.getField("NRNIVELDIFICULDADE").options[row.NRNIVELDIFICULDADE];
        });

        if (reportType == "PDF") {
            restEngine.requestMetaData(reportPath).then(function(reportMetaData) {
                ZhReports.openReport(reportMetaData.data, referenceWidget, function() {
                    referenceWidget.dataSource.data = data;
                }, errorCallback);
            });
        } else {
            restEngine.requestMetaData(reportPath).then(function(reportMetaData) {
                ZhReports.openReport(reportMetaData.data, referenceWidget, successCallback, errorCallback, reportType);
            });
        }

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc00999_relatorios_de_receitaController", coc00999_relatorios_de_receitaController);
});

// FILE: ./assets/js/controllers/coc01000_servicogenerico_controller.js
var coc01000_servicogenerico_controller = function(ScreenService, InflightorGeneral, coc01000_servicogenericoService) {

    this.salvaServicoGen = function(widget) {
        var nrsgen = widget.currentRow.NRSERVICOGEN;
        var nrorg = widget.currentRow.NRORG;

        if (widget.currentRow.IDATIVO === "N") {
            coc01000_servicogenericoService.getServByServGen(nrsgen, nrorg).then(function(callback) {
                if (callback.dataset.servicociafili.length === 0) {
                    if (widget.currentRow.VRPESOSE) {
                        InflightorGeneral.setFloatAttrOnCurrentRow(widget.currentRow, "VRPESOSE");
                    }
                    InflightorGeneral.saveCrud(widget);
                } else {
                    var companhias = [];
                    var nomes = [];
                    callback.dataset.servicociafili.forEach(function(companhia) {
                        companhias.push(companhia.CDCIA);
                    });
                    companhias = Array.from(new Set(companhias));
                    coc01000_servicogenericoService.getNmciaByCdcia(companhias).then(function() {
                        callback.dataset.companhia.forEach(function(ciaNome) {
                            nomes.push(ciaNome.NMCIA);
                        });
                        ScreenService.showMessage("Serviços utilizados na(s) companhia(s): \n" + nomes);
                    });
                }
            });
        } else {
            if (widget.currentRow.VRPESOSE) {
                InflightorGeneral.setFloatAttrOnCurrentRow(widget.currentRow, "VRPESOSE");
            }
            InflightorGeneral.saveCrud(widget);
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01000_servicogenerico_controller", coc01000_servicogenerico_controller);
});

// FILE: ./assets/js/controllers/coc01001_servico_companhiaController.js
var coc01001_servico_companhiaController = function(InflightorGeneral, ScreenService, MessageService, restEngine, templateManager, coc01001_servico_companhiaService, JSONContainerLoader, metaDataFactory, ZHPromise) {
    var msg;
    var self = this;

    this.setCicloColor = function(element, row) {

        if (row) {
            setTimeout(function() {
                element.find(".td:eq(3)").css("background-color", row.NRCORCICLO);
            }, 100);
        }

    };

    this.unidadeToCompanhia = function(field) {
        field.widget.getField("NMCIA").dataSourceFilter.forEach(function(filter) {
            if (field.widget.currentRow[filter.name]) {
                filter.value = field.widget.currentRow[filter.name];
            }
        });
        field.widget.getField("NMCIA").setValue("");
    };

    this.adicionarServGenerico = function(widget, field, widget_servGen) {
        ScreenService.closePopup();
        widget_servGen.fields = [];
        JSONContainerLoader.loadContainer("coc#coc01000_servicogenerico").then(function(data) {
            // Caso queira dar load na widget inteira.
            // widgetServ = metaDataFactory.widgetFactory(data.widgets[0], templateManager.container);
            data.widgets[1].fields.forEach(function(field) {
                metaDataFactory.fieldFactory(field, widget_servGen, true);
            });
        });
        widget_servGen.activate();
        ScreenService.openPopup(widget_servGen);
    };

    this.adicionarProdCia = function(widget_ProdCia) {
        widget_ProdCia.activate();
        ScreenService.openPopup(widget_ProdCia);
    };

    this.validaManualVigente = function(widget) {
        if (widget.currentRow.IDSITSERVICO === "1") {
            self.save(widget);
        } else {
            //Validação de Inativação do Serviço é feita no Backend
            // da aplicação seguindo novas regras de controle
            //19/07/2017
            self.save(widget);
            // InflightorGeneral.callRestEngine('coc#/crud/existe_manual_vigente', 'FilterData', [
            //     {
            //         "name": "NRSERVICOCIAFILI",
            //         "operator": "=",
            //         "value": widget.currentRow.NRSERVICOCIAFILI
            //     }, {
            //         "name": "CDFILIAL",
            //         "operator": "=",
            //         "value": widget.currentRow.CDFILIAL
            //     }, {
            //         "name": "CDCIA",
            //         "operator": "=",
            //         "value": widget.currentRow.CDCIA
            //     }
            // ]).then(function() {
            //     if (callback.dataset.existe_manual_vigente.length === 0) {
            //         InflightorGeneral.callRestEngine('coc#/crud/precoseciam', 'FilterData', [
            //             {
            //                 "name": "NRSERVICOCIAFILI",
            //                 "operator": "=",
            //                 "value": widget.currentRow.NRSERVICOCIAFILI
            //             }, {
            //                 "name": "CDFILIAL",
            //                 "operator": "=",
            //                 "value": widget.currentRow.CDFILIAL
            //             }, {
            //                 "name": "CDCIA",
            //                 "operator": "=",
            //                 "value": widget.currentRow.CDCIA
            //             }
            //         ]).then(function() {
            //             if (callback.dataset.precoseciam.length === 0) {
            //                 self.save(widget);
            //             } else {
            //                 ScreenService.showMessage('Serviço em tabela de preço não pode ser desativado!');
            //             }
            //         });
            //     } else {
            //         ScreenService.showMessage('Serviço em manual vigente não pode ser desativado!');
            //     }
            // }.bind(widget), function(err) {
            //     ScreenService.showMessage(err.data.error, 'error');
            // });
        }
    };

    this.salvarServGenerico = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            row.VRPESOSE = row.VRPESOSE.toString();
            row.VRPESOSE = row.VRPESOSE.concat();
            // row.VRPESOSE = row.VRPESOSE.replace(".", ",");
            row.VRPESOSE = InflightorGeneral.valueToFloat(row.VRPESOSE);
            row.__is_new = true;
            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
            }
            widget.dataSource.data = [widget.currentRow]; //permite salvar o form sem data no datasource pelo widget saverow
            widget.saveRow().then(
                function success() {
                    ScreenService.closePopup();
                    widget.fields.forEach(function(field) {
                        field.clearValue();
                    });
                    MessageService.showMessageByCode("SAVE_SUCCESS");
                    widget.deactivate();
                    widget.parent.activate();
                    templateManager.updateTemplate();
                },
                function error(error) {
                    if (error.data.error.indexOf("ORA-01722") !== -1) {
                        MessageService.showMessageByCode("COC01001_INVALID_WEIGHT");
                    } else if (error.data) {
                        msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        MessageService.showMessageByCode("NO_CONNECTION");
                    }
                }
            );
        }
    };

    this.salvarProdCia = function(widget) {
        if (widget.isValid()) {
            var CDCIA = 0;
            var CDFILIAL = 0;
            widget.container.getWidget("grid_servicos").dataSourceFilter.forEach(function(data) {
                if (data.name == "CDCIA")
                    CDCIA = data.value;
                if (data.name == "CDFILIAL")
                    CDFILIAL = data.value;
            });
            widget.currentRow.CDFILIAL = CDFILIAL;
            widget.currentRow.CDCIA = CDCIA;
            widget.currentRow.__is_new = true;
            var filterProdutocia = [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widget.currentRow.CDFILIAL
                },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: widget.currentRow.CDCIA
                },
                {
                    name: "CDPRODUTO",
                    operator: "=",
                    value: widget.currentRow.CDPRODUTO
                }
            ];
            InflightorGeneral.callRestEngine("coc#/crud/produtocia_produto", "FilterData", filterProdutocia).then(
                function(callback) {
                    if (callback.dataset.produtocia_produto.length > 0) {
                        MessageService.showMessageByCode("COC01001_PRODUCT_DUPLICATED");
                    } else {
                        coc01001_servico_companhiaService.cadProdutosCia([widget.currentRow]).then(function() {
                            ScreenService.closePopup();
                            widget.deactivate();
                            widget.parent.activate();
                            templateManager.updateTemplate();
                            MessageService.showMessageByCode("COC01001_ITEM_ADD_CIA");
                        }, InflightorGeneral.callbackError);
                    }
                }, InflightorGeneral.callbackError
            );
        }
    };

    this.deletaCiclo = function(widget) {
        if (widget.getCheckedRows().length > 1) {
            msg = "Deseja excluir os " + widget.getCheckedRows().length + " registros selecionados?";
        } else {
            msg = "Deseja excluir o registro? Classe: ''" + widget.getCheckedRows()[0].DSCLASSE + "'' Ciclo: ''" + widget.getCheckedRows()[0].IDCICLO + "''?";
        }
        ScreenService.confirmMessage(msg, "question", function() {
            widget.removeCheckedRows().then(
                function success() {
                    MessageService.showMessageByCode("DEL_SUCCESS");
                },
                function error(error) {
                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                        MessageService.showMessageByCode("COC01001_DELETE_ERROR_TABLE_PRICE");
                    } else if (error.data) {
                        msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        MessageService.showMessageByCode("NO_CONNECTION");
                    }
                }
            );
        }, function() {
            widget.unSelectAll();
            templateManager.updateTemplate();
        });
    };

    this.openSwipe = function(widget, tabbed) {
        tabbed.container.getWidget("grid_servicos").dataSourceFilter = [{
            name: "NRORG",
            operator: "=",
            value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        }, {
            name: "CDFILIAL",
            operator: "=",
            value: widget.parent.currentRow.CDFILIAL
        }, {
            name: "CDCIA",
            operator: "=",
            value: tabbed.parent.currentRow.CDCIA
        }];
        tabbed.container.getWidget("grid_servicos").getField("CDCIA").setValue(tabbed.parent.currentRow.CDCIA);
        tabbed.widgets[0].reload();
        tabbed.activate();
        ScreenService.openSwipe(tabbed);
    };

    this.edit = function(widget) {
        if (!widget.isNewRow()) {
            widget.backupRow = angular.copy(widget.currentRow);
            widget.auxRow = Util.clone(widget.currentRow);
            widget.template = "widget/form-view-edit.html";
            templateManager.updateTemplate();
            widget.edit();
            widget.parent.widgets[1].isVisible = false;
        }
    };

    this.abreCiclo = function(grid, tabbed, ciclo) {
        ciclo.dataSourceFilter = [{
            name: "NRORG",
            operator: "=",
            value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        }, {
            name: "CDFILIAL",
            operator: "=",
            value: grid.currentRow.CDFILIAL
        }, {
            name: "CDCIA",
            operator: "=",
            value: grid.currentRow.CDCIA
        }, {
            name: "CDCLASSE",
            operator: "=",
            value: grid.currentRow.CDCLASSE
        }, {
            name: "NRSERVICOCIAFILI",
            operator: "=",
            value: grid.currentRow.NRSERVICOCIAFILI
        }];
        ciclo.getField("CDFILIAL").defaultValue = grid.currentRow.CDFILIAL;
        ciclo.getField("CDCIA").defaultValue = grid.currentRow.CDCIA;
        ciclo.getField("CDCLASSE").defaultValue = grid.currentRow.CDCLASSE;
        ciclo.getField("NRORG").defaultValue = grid.currentRow.NRORG;
        ciclo.getField("NRSERVICOCIAFILI").defaultValue = grid.currentRow.NRSERVICOCIAFILI;
        ciclo.activate();
        ciclo.reload();
        ScreenService.openSwipe(tabbed);
    };

    this.openView = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.container.getWidget("view_servicos").fields = widget.fields;
            // widget.container.getWidget("view_servicos").fieldGroups = widget.fieldGroups;
            widget.container.getWidget("view_servicos").currentRow = widget.currentRow;
            widget.container.getWidget("view_servicos").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.currentRow.CDFILIAL
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.currentRow.CDCIA
            }];
            widget.container.getWidget("view_servicos").reload();
            widget.container.getWidget("tab_ciclo").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.currentRow.CDFILIAL
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.currentRow.CDCIA
            }, {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: widget.currentRow.NRSERVICOCIAFILI
            }];
            widget.container.getWidget("tab_ciclo").getField("NRSERVICOCIAFILI").defaultValue = widget.currentRow.NRSERVICOCIAFILI;
            widget.container.getWidget("tab_ciclo").getField("CDCIA").defaultValue = widget.currentRow.CDCIA;
            widget.container.getWidget("tab_ciclo").getField("CDFILIAL").defaultValue = widget.currentRow.CDFILIAL;
            widget.container.getWidget("tab_ciclo").getField("NRORG").defaultValue = getLocalVar("INFLIGHTOR_NRORGTRAB");
            widget.container.getWidget("tab_ciclo").reload();
            widget.container.getWidget("child_tabbed").activate();
            ScreenService.openSwipe(widget.container.getWidget("child_tabbed"));
        }
    };

    this.voltarView = function(widgetGrid) {
        widgetGrid.list();
        widgetGrid.activate();
        ScreenService.closeSwipe();
        templateManager.updateTemplate();
    };

    this.cancelaEdicao = function(args) {
        if (args.row.__is_new) {
            args.owner.widget.dataSource.data.pop();
            args.owner.widget.currentRow = {};
            args.owner.widget.list();
            args.owner.widget.activate();
            ScreenService.closeSwipe();
        } else {
            for (let i in args.owner.widget.backupRow) {
                args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
            }
            args.owner.widget.template = "widget/form-view.html";
            templateManager.updateTemplate();
            args.owner.widget.list();
            args.owner.widget.parent.widgets[1].isVisible = true;
        }
    };

    this.newRow = function(widget) {
        widget.backupRow = Util.clone(widget.currentRow);
        //widget.edit();
        widget.newRow();
        widget.auxRow = Util.clone(widget.currentRow);
        //ScreenService.openSwipe(widget);
    };

    this.abrePopup = function(widget, popup) {
        if (popup.name == "cad_autoCiclos") {
            popup.getField("NRSERVICOCIAFILI").defaultValue = widget.parent.parent.currentRow.NRSERVICOCIAFILI;
            popup.getField("CDCLASSE").defaultValue = widget.parent.parent.currentRow.CDCLASSE;
            popup.getField("CDCIA").defaultValue = widget.getField("CDCIA").defaultValue;
            popup.getField("VRPESOSE").defaultValue = widget.container.getWidget("grid_servicos").currentRow.VRPESOSE;
            popup.getField("CDFILIAL").defaultValue = widget.getField("CDFILIAL").defaultValue;
            popup.getField("IDCICLO").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.getField("CDFILIAL").defaultValue
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.getField("CDCIA").defaultValue
            }];

            popup.getField("DSCLASSE").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.getField("CDFILIAL").defaultValue
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.getField("CDCIA").defaultValue
            }];
        }
        popup.activate();
        popup.newRow();
        popup.edit();
        ScreenService.openPopup(popup);
    };

    this.backSwipe = function(widget) {
        ScreenService.closeSwipe();
        widget.parent.parent.activate();
        templateManager.updateTemplate();
    };

    this.backPopup = function(widget) {
        if (widget.name == "cad_autoCiclos") {
            widget.currentRow = [];
            widget.getField("IDCICLO").clearValue();
            ScreenService.closePopup();
            widget.parent.activate();
            templateManager.updateTemplate();
        }

        if (widget.name == "widget_servGen") {
            ScreenService.closePopup();
            widget.fields.forEach(function(field) {
                field.clearValue();
            });
            widget.deactivate();
            widget.parent.activate();
        }

        if (widget.name == "widget_ProdCia") {
            ScreenService.closePopup();
            widget.fields.forEach(function(field) {
                field.clearValue();
            });
            widget.deactivate();
            widget.parent.activate();
        }
        templateManager.updateTemplate();
    };

    this.mudaCampos = function(widget, field) {
        if (widget.currentRow.SITSERVGEN == "N") {
            MessageService.showMessageByCode("INACTIVE_SERVICE");
            field.openField();
            widget.currentRow.SITSERVGEN = null;
            widget.currentRow.DSSERVICOGEN = null;
            widget.currentRow.NRSERVICOGEN = null;
        } else {
            widget.currentRow.DSSERVICOCIAFILI = widget.currentRow.DSSERVICOGEN;
            widget.getField("SGPADSERV").value(widget.currentRow.SGPADSERV);
            widget.getField("SGPADSERV").reload();
            widget.auxRow = Util.clone(widget.currentRow);
        }
    };

    this.delete = function(widget) {
        msg = "Deseja excluir o serviço " + widget.currentRow.DSSERVICOGEN + "?";
        ScreenService.confirmMessage(msg, "question", function() {
            widget.removeRow().then(
                function success() {
                    ScreenService.closeSwipe();
                    widget.parent.parent.activate();
                    templateManager.updateTemplate();
                    widget.parent.parent.reload().then(function() {
                        return MessageService.showMessageByCode("DEL_SUCCESS");
                    });
                },
                function error(error) {
                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                        MessageService.showMessageByCode("COC01001_DELETE_ERROR_TABLE_CLASS");
                    } else if (error.data) {
                        msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        MessageService.showMessageByCode("NO_CONNECTION");
                    }
                }
            );
        });
    };

    this.salvaEditCiclos = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.changed) {
                row.VRPESOSE = row.VRPESOSE.toString();
                row.VRPESOSE = row.VRPESOSE.concat();
                row.VRPESOSE = row.VRPESOSE.replace(",", ".");
            }
        });

        InflightorGeneral.saveCrud(widget).then(function() {
            widget.commitInlineEdit();
            widget.list();
            widget.activate();
        }, function(erro) {
            var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao salvar edição!";
            ScreenService.showMessage(msg);
        });
    };

    this.saveCadAuto = function(widget) {
        if (widget.isValid()) {
            if (widget.name == "cad_autoCiclos") {
                widget.getField("IDCICLO").selectWidget.getCheckedRows().forEach(function(row) {
                    row.CDFILIAL = widget.getField("CDFILIAL").defaultValue;
                    row.CDCIA = widget.getField("CDCIA").defaultValue;
                    row.CDCLASSE = widget.currentRow.CDCLASSE;
                    row.VRPESOSE = widget.currentRow.VRPESOSE;
                    row.VRPESOSE = row.VRPESOSE.toString();
                    row.VRPESOSE = row.VRPESOSE.concat();
                    row.VRPESOSE = row.VRPESOSE.replace(",", ".");
                    row.NRSERVICOCIAFILI = widget.getField("NRSERVICOCIAFILI").defaultValue;
                    row.CDSERVICO = row.CDCICLO;
                    row.CDARVSERV = row.CDCICLO;
                    row.__is_new = true;
                });

                var sendd = widget.getField("IDCICLO").selectWidget.getCheckedRows();
                coc01001_servico_companhiaService.cadAutoCiclos(sendd).then(
                    function success() {
                        msg = ScreenService.i18n("Salvo com sucesso!");
                        ScreenService.showMessage(msg);
                        widget.getField("IDCICLO").clearValue();
                        ScreenService.closePopup();
                        widget.parent.activate();
                        templateManager.updateTemplate();
                        widget.parent.reload();
                    },
                    function error(error) {
                        if (error.data) {
                            msg = ScreenService.i18n(error.data.error);
                            ScreenService.showMessage(msg);
                        } else {
                            msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                            ScreenService.showMessage(msg);
                        }
                    }
                );
            }
        }
    };

    this.FieldOnChangeProduto = function(args) {
        var CDCIAA = 0;
        var CDFILIALL = 0;
        args.owner.owner.widget.dataSourceFilter.forEach(function(data) {
            if (data.name == "CDCIA")
                CDCIAA = data.value;
            if (data.name == "CDFILIAL")
                CDFILIALL = data.value;
        });
        if (CDCIAA === 0) {
            MessageService.showMessageByCode("COC01001_FILTER_CIA");
        } else {
            args.owner.owner.widget.getField("NMPRODUTO").dataSourceFilter[1].value = CDFILIALL;
            args.owner.owner.widget.getField("NMPRODUTO").dataSourceFilter[2].value = CDCIAA;
            args.owner.owner.widget.getField("NMPRODUTO").reload();
        }
    };

    this.save = function(widget) {
        var CDCIA = 0;
        var CDFILIAL = 0;
        widget.dataSourceFilter.forEach(function(data) {
            if (data.name == "CDCIA")
                CDCIA = data.value;
            if (data.name == "CDFILIAL")
                CDFILIAL = data.value;
        });
        if (CDCIA === 0) {
            MessageService.showMessageByCode("COC01001_FILTER_CIA");
        } else {
            if (widget.isValid()) {
                if (widget.currentRow.__is_new) {
                    widget.currentRow.CDFILIAL = CDFILIAL;
                    widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                    widget.currentRow.CDCIA = CDCIA;
                    widget.currentRow.CDSETOR = "";
                    // InflightorGeneral.callRestEngine('coc#/crud/servicociafili', 'FilterData', filterServicociafili).then(
                    //     function (){
                    // @todo - verificar a regra de negócio
                    // if(callback.dataset.servicociafili.length > 0) {
                    //     var variables = {
                    //         servico : callback.dataset.servicociafili[0].DSSERVICOCIAFILI
                    //     };
                    //     ScreenService.showMessage(ScreenService.i18n('Operação bloqueada!<br>Serviço genérico informado já está vinculado ao serviço: __servico__ ', variables));
                    // } else {
                    if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                        widget.currentRow.__using_unit = false;
                        widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
                        widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
                        widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                    }
                    /* Validação/Inserção de produtoCia removida na branch "TASK_BKP_61418_SERVICOS_DA_CIA_NAO_ESTA_SALVANDO" */
                    self.effectSave(widget, true);
                    // }
                    //     }, InflightorGeneral.callbackError
                    // );
                } else {
                    validateChangedStatus(widget).then(function() {
                        self.effectSave(widget, false);
                    });
                }
            }
        }
    };

    function validateChangedStatus(widget) {
        var deferred = ZHPromise.defer();
        if (widget.currentRow.IDTIPOSERVICO === "AMBOS" && widget.currentRow.IDSITSERVICO == "1") {
            deferred.resolve();
        } else {
            InflightorGeneral.callRestEngine("coc#/validateChangedStatusServico", "Row", widget.currentRow).then(
                function(callback) {
                    if (callback.dataset && callback.dataset.MENSSAGESERROR && callback.dataset.MENSSAGESERROR.length) {
                        //Lista de erros
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.MENSSAGESERROR, "Operação bloqueada");
                        deferred.reject();
                    } else
                        deferred.resolve(callback);
                    return deferred.promise;
                },
                function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    deferred.reject(errorMessage);
                    return deferred.promise;
                });
        }
        return deferred.promise;
    }

    this.effectSave = function(widget, isNew) {
        InflightorGeneral.saveCrud(widget, false)
            .then(
                function success() {
                    if (!isNew) {
                        widget.activate();
                        widget.getParent().widgets[1].isVisible = true;
                        widget.template = "widget/form-view.html";
                        templateManager.updateTemplate();
                        widget.list();
                    } else {
                        widget.activate();
                        widget.list();
                    }
                    widget.reload().then(function() {
                        return MessageService.showMessageByCode("SAVE_SUCCESS");
                    });
                },
                function error(error) {
                    widget.dataSource.name = "coc#/servicociafili_servicogen_produto";
                    if (error.data) {
                        msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        MessageService.showMessageByCode("NO_CONNECTION");
                    }
                }
            );
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc01001_servico_companhiaController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01001_servico_companhiaController", coc01001_servico_companhiaController);
});

// FILE: ./assets/js/controllers/coc01200_ficha_tecnicaController.js
var coc01200_ficha_tecnicaController = function(InflightorGeneral, ScreenService, COCUtil, MessageService) {

    var listaservicoclasse;

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        args.owner.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
    };

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        COCUtil.updateFilterFields(fieldCia, 'CDFILIAL', rowFilter);
        this.limpaField(fieldUnit);
        this.onChangeCia(fieldCia);

    };

    this.onChangeCia = function(fieldCia) {
        const widget = fieldCia.widget;
        let rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });
        rowFilter.CDMANCIA = '';
        this.limpaField(fieldCia);
        COCUtil.updateFilterFields(['DSMANUAL', 'IDCICLO'], ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowFilter, false, widget);
    };

    this.onChangeManual = function(fieldManual) {
        const widget = fieldManual.widget;
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA'
        });
        COCUtil.updateFilterFields('IDCICLO', ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowFilter, false, widget);
        this.limpaField(fieldManual);
    };

    this.limpaField = function(field) {
        if (field.name == "NMFILIAL") {
            field.widget.currentRow.NMCIA = "";
            field.widget.currentRow.DSMANUAL = "";
            field.widget.currentRow.IDCICLO = "";
            field.widget.currentRow.DSSUBGRU = "";
            field.widget.currentRow.NMSETOR = "";
            field.widget.currentRow.IDSERVICOCLASSE = "";
        }

        if (field.name == "NMCIA") {
            field.widget.currentRow.DSMANUAL = "";
            field.widget.currentRow.IDCICLO = "";
            field.widget.currentRow.DSSUBGRU = "";
            field.widget.currentRow.NMSETOR = "";
            field.widget.currentRow.IDSERVICOCLASSE = "";
        }

        if (field.name == "DSMANUAL") {
            field.widget.currentRow.IDCICLO = "";
            field.widget.currentRow.DSSUBGRU = "";
            field.widget.currentRow.NMSETOR = "";
            field.widget.currentRow.IDSERVICOCLASSE = "";
        }

        if (field.name == "IDSERVICOCLASSE") {
            field.widget.currentRow.IDCICLO = "";
            field.widget.currentRow.DSSUBGRU = "";
        }

        if (field.name == "NMSETOR") {
            field.widget.currentRow.DSSUBGRU = "";
            field.widget.currentRow.IDCICLO = "";
        }

        if (field.name == "DSSUBGRU") {
            field.widget.currentRow.IDCICLO = "";
        }

    };

    this.report = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.__locale = widget.currentRow.IDIOMA;
            if (widget.currentRow.TCDSETOR != "T") {
                var setor = listaSetor(widget.getField("NMSETOR"));
                widget.currentRow.CDSETOR = setor;
            } else {
                widget.currentRow.CDSETOR = "T";
            }

            if (!widget.currentRow.VOOREF) {
                widget.currentRow.VOOREF = "";
            }
            if (!widget.currentRow.DESTREF) {
                widget.currentRow.DESTREF = "";
            }

            widget.currentRow.FILESERVPATH = window.filesPath;

            if (widget.currentRow.TIDSERVICOCLASSE == "S") {
                widget.currentRow.SERVCLASSE = "T";
            } else if (widget.currentRow.IDSERVICOCLASSE !== "" && widget.currentRow.IDSERVICOCLASSE !== undefined) {
                var arrayToFilerServ = [];
                widget.getField("IDSERVICOCLASSE").dataSource.getCheckedRows().forEach(function(servico) {
                    arrayToFilerServ.push(servico.NRSERVICOCIAFILI + "-" + servico.CDCLASSE);
                });
                widget.currentRow.SERVCLASSE = arrayToFilerServ.join("_");
            }
            if (widget.currentRow.TSUBGRU == "S") {
                widget.currentRow.SUBGRUPO = "T";
                callReport(widget);
            } else if (widget.currentRow.DSSUBGRU !== "" && widget.currentRow.DSSUBGRU !== undefined) {
                var dtsSubgru = widget.getField("DSSUBGRU").dataSource.getCheckedRows();
                var subgrupos = "";
                dtsSubgru.forEach(function(subgru) {
                    subgrupos += subgru.CDSUBGRU + "-" + subgru.NRSEQSUBGRUSEMAM + "_";
                });
                widget.currentRow.SUBGRUPO = subgrupos;

                //
                // Remoção solicitada pela skychefs.
                // Este metodo buscava subgrupos dependentes dos selecionados.
                //
                // dtsSubgru.forEach(function(subgru){
                //     subgru.CDCICLO = widget.currentRow.CDCICLO;
                // });
                // InflightorGeneral.callRestEngine('coc#/getSubgruTree', 'DataSet', dtsSubgru).then(function(callback){
                //     widget.currentRow.SUBGRUPO = callback.dataset.subgrutree[0].SUBGRUPO;
                // });
                //

                callReport(widget);
            } else {
                MessageService.showMessageByCode("MSG_COC_SELECIONE_SUBGRUPO");
            }
        }
    };

    var callReport = function(widget) {
        InflightorGeneral.openReport("coc#/fichaTecnicaReport", null, widget.currentRow).then(function() {
            widget.activate();
        }, function(error) {
            if (error.data) {
                ScreenService.showMessage(error.data.error);
            } else {
                MessageService.showMessageByCode("TIMEOUT");
            }
        });
    };

    this.verifyFilterCiclo = function(args, changeFilter = false) {
        let fieldCiclo = args.owner.field;
        let row = args.row;
        let lastCheckedFilter = '';
        const msgsEmptyFilter = {
            CDFILIAL: 'MSG_COC_ESCOLHA_UNID_CICLO',
            CDCIA: 'MSG_COC_ESCOLHA_CIA_CICLO',
            CDMANCIA: 'MSG_COC_ESCOLHA_MANUAL_CICLO',
            CDSUBGRU: 'MSG_COC_ESCOLHA_SUBGRUPO_CICLO'
        };
        let hasEmptyFilter = !fieldCiclo.dataSourceFilter.every(filter => {
            if (changeFilter) {
                if (filter.name == 'CDSUBGRU') {
                    if (row.TSUBGRU != "S" && row.DSSUBGRU) {
                        var filtroSubGrupo = listaSubgruposDist(row);
                        filter.operator = 'IN';
                        filter.value = filtroSubGrupo;
                    } else {
                        filter.operator = "=";
                        filter.value = 'T';
                    }
                } else if (row[filter.name]) {
                    filter.value = row[filter.name];
                }
            }
            lastCheckedFilter = filter.name;
            return !!filter.value;
        });
        if (hasEmptyFilter && lastCheckedFilter) {
            MessageService.showMessageByCode(msgsEmptyFilter[lastCheckedFilter]);
        }
    };

    this.FieldOnChangeServico = function(args) {
        var menuGrid = args.owner.owner.widget.getField("DSMANUAL").value();

        var filtro = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: args.row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: args.row.CDMANCIA
            }
        ];

        if (args.row.CDFILIAL == "") {
            args.owner.owner.widget.getField("IDSERVICOCLASSE").dataSourceFilter = [];
            args.owner.owner.widget.getField("IDSERVICOCLASSE").reload();
            MessageService.showMessageByCode("MSG_COC_ESCOLHA_UNID_SERVICO");
            return;
        }
        if (args.row.CDCIA == "") {
            args.owner.owner.widget.getField("IDSERVICOCLASSE").dataSourceFilter = [];
            args.owner.owner.widget.getField("IDSERVICOCLASSE").reload();
            MessageService.showMessageByCode("MSG_COC_ESCOLHA_CIA_SERVICO");
            return;
        }
        if (menuGrid == "") {
            args.owner.owner.widget.getField("IDSERVICOCLASSE").dataSourceFilter = [];
            args.owner.owner.widget.getField("IDSERVICOCLASSE").reload();
            MessageService.showMessageByCode("MSG_COC_ESCOLHA_MANUAL_SERVICO");
            return;
        }

        args.owner.owner.widget.getField("IDSERVICOCLASSE").dataSourceFilter = filtro;
        args.owner.owner.widget.getField("IDSERVICOCLASSE").reload();
    };

    this.FieldOnChangeDssubgru = function(args) {
        var fieldServicoClasse = args.owner.field.widget.getField("IDSERVICOCLASSE");

        listaservicoclasse = args.row.TIDSERVICOCLASSE == "S" ? "T" : listaServicoClasseConcat(fieldServicoClasse);

        var filtro = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: args.row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: args.row.CDMANCIA
            },
            {
                name: "SERVICOCLASSE",
                operator: "=",
                value: listaservicoclasse
            }
        ];

        args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = filtro;
        args.owner.owner.widget.getField("DSSUBGRU").reload();
    };

    var listaServicoClasseConcat = function(field) {
        var lista = "";
        var rows = field.selectWidget.getCheckedRows();

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i].NRSERVICOCIAFILI + "-" + rows[i].CDCLASSE;
            } else {
                lista = lista + "_" + rows[i].NRSERVICOCIAFILI + "-" + rows[i].CDCLASSE;
            }
        }
        return lista;
    };

    var listaSubgruposDist = function(field) {

        var lista = "";

        var rowsArr = field.DSSUBGRU;
        var rows = [];
        for (let i in rowsArr) {
            rows.push(rowsArr[i].CDSUBGRU + "|" + rowsArr[i].NRSEQSUBGRUSEMAM);
        }

        for (let j = 0; j < rows.length; j++) {
            if (lista == "") {
                lista = rows[j];
            } else {
                lista = lista + "_" + rows[j];
            }
        }
        return lista;
    };

    var listaSetor = function(field) {

        var lista = "";
        var rows = field.selectWidget.getCheckedRows();

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i].CDSETOR;
            } else {
                lista = lista + "_" + rows[i].CDSETOR;
            }
        }
        return lista;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01200_ficha_tecnicaController", coc01200_ficha_tecnicaController);
});

// FILE: ./assets/js/controllers/coc01400_receitaController.js
var coc01400_receitaController = function(ScreenService, MessageService, templateManager, $http, ZHPromise, restEngine, InflightorGeneral, coc01400_receitaService, ValidationEngine) {

    var self = this;
    var cdreceita;
    var nrvariacaorec;
    var rowDuplicacao;
    var isNew;
    var rendBruto;
    var rendLiquido;
    var percentLiq;

    this.setDateInFilter = function(widget) {
        var dataSourceFilter = widget.dataSourceFilter;
        for (let index in dataSourceFilter) {
            var position = dataSourceFilter[index];
            if (position.name == "DTINCLUSAO") {
                position.value[1] += " 23:59:59";
            } else if (position.name == "DTULTATU") {
                position.value[1] += " 23:59:59";
            }
        }
    };

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.abreReceitasDoGrupo = function(args) {

        var swipeWidget = args.owner.container.getWidget("receitasDoGrupo");
        var filtro = [
            {
                "name": "NRARVOREREC",
                "operator": "=",
                "value": args.row.NRARVOREREC
            }
        ];

        if (!(args.row._hasChild)) {
            swipeWidget.dataSourceFilter = filtro;
            swipeWidget.reload().then(function() {
                ScreenService.openSwipe(swipeWidget);
            });
        }
    };

    function converteFieldsReceita(row, operator) {

        if (operator == "find") {
            switch (row.NRNIVELDIFICULDADE) {
                case 1:
                    row.NRNIVELDIFICULDADE = "Fácil";
                    break;
                case 2:
                    row.NRNIVELDIFICULDADE = "Médio";
                    break;
                case 3:
                    row.NRNIVELDIFICULDADE = "Díficil";
                    break;
            }
            switch (row.IDATIVO) {
                case "S":
                    row.IDATIVO = "Ativa";
                    break;
                case "N":
                    row.IDATIVO = "Inativa";
                    break;
            }
            switch (row.IDANALIZEATIV) {
                case "S":
                    row.IDANALIZEATIV = "Aguardando Solicitação de Workflow";
                    break;
                case "P":
                    row.IDANALIZEATIV = "Solicitação gerada e Pendente";
                    break;
                case "A":
                    row.IDANALIZEATIV = "Solicitação Aprovada";
                    break;
                case "N":
                    row.IDANALIZEATIV = "Solicitação Reprovada";
                    break;
            }
            if (row.NRTEMPOPREPARO)
                row.NRTEMPOPREPARO = InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO);
        } else if (operator == "save") {
            switch (row.NRNIVELDIFICULDADE) {
                case "Fácil":
                    row.NRNIVELDIFICULDADE = 1;
                    break;
                case "Médio":
                    row.NRNIVELDIFICULDADE = 2;
                    break;
                case "Díficil":
                    row.NRNIVELDIFICULDADE = 3;
                    break;
            }
            switch (row.IDATIVO) {
                case "Ativa":
                    row.IDATIVO = "S";
                    break;
                case "Inativa":
                    row.IDATIVO = "N";
                    break;
            }
            switch (row.IDANALIZEATIV) {
                case "Aguardando Solicitação de Workflow":
                    row.IDANALIZEATIV = "S";
                    break;
                case "Solicitação gerada e Pendente":
                    row.IDANALIZEATIV = "P";
                    break;
                case "Solicitação Aprovada":
                    row.IDANALIZEATIV = "A";
                    break;
                case "Solicitação Reprovada":
                    row.IDANALIZEATIV = "N";
                    break;
            }
        }
    }

    this.abreDetalheReceita = function(widget, openSwipe = true) {

        var swipeWidget = widget.container.getWidget("receita");
        var formDetalheReceita = widget.container.getWidget("detalheReceita");
        var currentRow = widget.currentRow;
        coc01400_receitaService.retornaDetalhesReceita(currentRow.CDRECEITA).then(function(callback) {
            var rows = callback.dataset.receitam_operador_companhia[0];
            converteFieldsReceita(rows, "find");
            formDetalheReceita.currentRow = rows;
            formDetalheReceita.container.getWidget("receitas").descriptionOrder = rows.DSRECEITA;
            formDetalheReceita.container.getWidget("receitas").currentRow = rows;
            if (formDetalheReceita.currentRow.IDATIVO == "Inativa" && formDetalheReceita.currentRow.IDANALIZEATIV == "Aguardando Solicitação de Workflow") {
                formDetalheReceita.getAction("WORKFLOW").isVisible = true;
            } else
                formDetalheReceita.getAction("WORKFLOW").isVisible = false;
            formDetalheReceita.reload().then(function() {
                if (openSwipe) {
                    ScreenService.openSwipe(swipeWidget);
                }
            });
        }, function(err) {
            ScreenService.errorNotification(err);
        });

    };

    this.atualizaFormularioVariacao = function(widget) {

        var formularioVariacao = widget.container.getWidget("detalhesvariacao");
        var currentRow = widget.currentRow;
        coc01400_receitaService.atualizaFormulario(currentRow.CDRECEITA, currentRow.NRVARIACAOREC).then(function(callback) {
            var rows = callback.dataset.variacaorec_operador[0];
            formularioVariacao.currentRow = rows;
            Object.assign(formularioVariacao.container.getWidget("variacoesReceita").currentRow, rows);
            formularioVariacao.reload();
        }, function(err) {
            ScreenService.errorNotification(err);
        });

    };

    this.setDataSourceForGrid = function(widget, cdreceita, nrvariacaorec) {
        if (widget.mode == "list") {
            var filterObj = {
                NRVARIACAOREC: nrvariacaorec,
                CDRECEITA: cdreceita
            };
            InflightorGeneral.buildDataSourceFilter(widget.dataSourceFilter, filterObj);
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    function setStripeFooter(data, widget) {
        var stripeFooter = widget.container.getWidget("stripefooter_composicao");
        rendBruto = 0;
        rendLiquido = InflightorGeneral.valueToFloat(widget.container.getWidget("variacoesReceita").currentRow.QTRENDIMENTOREC);
        data.forEach(function(row) {
            rendBruto += InflightorGeneral.valueToFloat(row.QTD);
        });
        stripeFooter.getField("QTRENDBRUTO").setValue(rendBruto.toFixed(3));
        stripeFooter.getField("QTPRODTCOMP").setValue(rendLiquido.toFixed(3));
        if (rendLiquido && rendBruto) {
            percentLiq = ((rendLiquido * 100) / rendBruto).toFixed(2);
        } else {
            percentLiq = 0;
        }
        stripeFooter.getField("QTPERCENTAPRO").setValue(percentLiq);
        stripeFooter.reload();
    }

    this.loadVersoes = function(widget) {

        cdreceita = widget.container.getWidget("receitas").currentRow.CDRECEITA;
        if (cdreceita && widget.container.getWidget("receita").currentWidget) {
            var widgetCurrent = widget.container.getWidget("receita").currentWidget;
            if (widget.container.getWidget("receita").currentWidget.name == "manualReceita") {
                self.onEnterDadosManual(widgetCurrent);
            } else {
                if (widget.container.getWidget("receita").currentWidget.name == "variacoesReceita") {
                    var filtro = [
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": cdreceita
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                        }
                    ];
                    if (widget.mode == "list" || !widget.__is_new) {
                        widgetCurrent.dataSourceFilter = filtro;
                        widgetCurrent.reload().then(function() {
                            widgetCurrent.activate();
                        });
                    }
                }
            }
        }

    };

    this.modEdit = function(widgetParent, widget) {
        InflightorGeneral.edit(widget);
        if (widgetParent)
            self.setVisibleforTabbed(widgetParent.widgets, false, widget);
        widget.template = widget.formTemplate = widget.templateToForm;
    };
    this.cancelModEdit = function(widgetParent, widget) {
        widget.view();
        if (widgetParent)
            self.setVisibleforTabbed(widgetParent.widgets, true);
        widget.template = widget.formTemplate = widget.templateToView;
        if (widget.name == 'detalheReceita') {
            this.onChangeExclusiv(widget.getField('ESCOLHECIAOUGRUPO'));
        }
    };

    this.setVisibleforTabbed = function(widgets, value, exception) {
        widgets.forEach(function(widget) {
            if (exception) {
                if (exception.name != widget.name)
                    widget.isVisible = value;
            } else {
                widget.isVisible = value;
            }
        });
    };

    this.detalheVersao = function(swipeWidget) {
        ScreenService.openSwipe(swipeWidget);
    };
    this.onEnterDetalhesVariacaoRec = function(widget) {
        if (!widget.__is_new)
            widget.currentRow = widget.container.getWidget("variacoesReceita").currentRow;
    };
    this.afterMoveRowVariacoes = function(widget) {
        if (!widget.container.getWidget("versao").hasActive()) {
            ScreenService.openSwipe(widget.container.getWidget("versao")).then(function() {
                widget.container.getWidget("versao").currentWidget = "detalhesvariacao";
            });
        } else {
            if (!widget.__is_new) {
                var widgetCurrent = widget.container.getWidget(widget.container.getWidget("versao").currentWidget);
                if (widgetCurrent) {
                    if (widgetCurrent.name == "detalhesvariacao")
                        self.onEnterDetalhesVariacaoRec(widgetCurrent);
                    else if (widgetCurrent.name == "composicaoprodutos" || widgetCurrent.name == "mododepreparo" || widgetCurrent.name == "unidade" || widgetCurrent.name == "testeReceita")
                        self.onEnterDadosVersao(widgetCurrent);
                    else
                        throw "widget is undefined";
                }
            }
        }
    };

    this.directRemove = function(widget) {
        widget.getCheckedRows().forEach(function(row) {
            var identificador = row.ID;
            widget.currentRow = row;
            if (identificador == "S") {
                self.removeRecCompVariacao(widget);
            } else if (identificador == "P") {
                self.excluiProdCompVariacao(widget);
            }
        });
    };

    this.onEnterDadosVersao = function(widget) {
        var row = widget.container.getWidget("variacoesReceita").currentRow;
        cdreceita = row.CDRECEITA;
        nrvariacaorec = row.NRVARIACAOREC;
        if (!widget.__is_new) {
            var filtro;
            if (widget.name == "composicaoprodutos" || widget.name == "mododepreparo") {
                if (widget.name == "mododepreparo") {
                    filtro = [
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": cdreceita
                        },
                        {
                            "name": "NRVARIACAOREC",
                            "operator": "=",
                            "value": nrvariacaorec
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                        }
                    ];
                    widget.dataSourceFilter = filtro;
                    widget.reload().then(function() {
                        widget.activate();
                    });
                } else {
                    self.setDataSourceForGrid(widget, cdreceita, nrvariacaorec);
                }
            }
            if (widget.name == "unidade" || widget.name == "testeReceita") {
                filtro = [
                    {
                        "name": "CDRECEITA",
                        "operator": "=",
                        "value": cdreceita
                    },
                    {
                        "name": "NRVARIACAOREC",
                        "operator": "=",
                        "value": nrvariacaorec
                    },
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                    }
                ];
                widget.dataSourceFilter = filtro;
                widget.reload().then(function() {
                    widget.activate();
                });
            }
        }

    };

    this.afterLoadDadosVersao = function(widget) {

        widget.activate();

    };

    this.setCurrentForm = function(widget) {
        var row = widget.dataSource.data;
        if (row.length) {
            widget.container.getWidget("mododepreparo").currentRow = row[0];
            widget.getAction("edit").isVisible = true;
            widget.getAction("add").isVisible = false;
            widget.getAction("ZH_TRASH").isVisible = true;
        } else {
            widget.getAction("edit").isVisible = false;
            widget.getAction("add").isVisible = true;
            widget.getAction("ZH_TRASH").isVisible = false;
        }

    };

    this.salvaEditReceita = function(widget) {

        var row = widget.currentRow;
        if (widget.isValid()) {
            if ((row.CDRECEITA === row.CDRECEITAPAI)) {
                MessageService.showMessageByCode("COC01400_RECIPE_EQUAL");
            } else {
                converteFieldsReceita(row, "save");
                restEngine.post("coc#/receitam/save", {
                    "requestType": "Row",
                    "row": angular.copy(row)
                }).then(function() {
                    if (row.__is_new) {
                        widget.list(true);
                        ScreenService.goBack();
                        widget.reload();
                    } else {
                        widget.view();
                        widget.getParent().getParent().reload();
                        widget.getParent().getParent().descriptionOrder = row.DSRECEITA;
                    }
                    widget.activate();
                    converteFieldsReceita(row, "find");
                });
                self.cancelModEdit(widget.container.getWidget("receita"), widget);
                self.setExcluRec(widget);
                templateManager.updateTemplate();
            }
        }

    };

    this.salvarAlteracoesProd = function(widget) {

        var data = widget.dataSource.data;

        coc01400_receitaService.salvarAlteracaoVariacaoCompProd(data).then(function() {

            widget.commitInlineEdit();
            widget.list();
            if (widget.mode == "list") {
                widget.container.getWidget("variacoesReceita").reload();
            }

        }, function(err) {

            if (err.data) {

                ScreenService.showMessage(err.data.error, "error");

            } else {

                MessageService.showMessageByCode("NO_CONNECTION");

            }

        });

    };

    this.salvarAlteracoesRec = function(widget, setCurrentRow) {
        var data = widget.dataSource.data;
        var receitas = [];
        var produtos = [];
        var promises = [];
        var rowProd;
        if (widget.currentRow.changed_QTD) {
            for (let i in data) {
                if (data[i].ID == "S" && data[i].changed_QTD) {
                    var row = {
                        CDSUBRECEITA: data[i].CODIGO,
                        CDRECEITA: data[i].CDRECEITA,
                        NRVARIACAOREC: data[i].NRVARIACAOREC,
                        QTRENDSUBREC: InflightorGeneral.valueToFloat(data[i].QTD),
                        NRORDSUBREC: data[i].ORDEM,
                        IDPREFERENCIA: widget.currentRow.IDPREFERENCIA,
                        DTINCLUSAO: moment().format("DD/MM/YYYY"),
                        VRCUSTOSUBREC: null,
                        __is_new: false
                    };
                    receitas.push(row);
                } else if (data[i].changed_QTD) {
                    rowProd = {
                        CDPRODUTO: data[i].CODIGO,
                        CDRECEITA: data[i].CDRECEITA,
                        NRVARIACAOREC: data[i].NRVARIACAOREC,
                        QTPRODREC: InflightorGeneral.valueToFloat(data[i].QTD),
                        NRORDPREC: data[i].ORDEM,
                        IDPREFERENCIA: widget.currentRow.IDPREFERENCIA,
                        DTINCLUSAO: moment().format("DD/MM/YYYY"),
                        VRCUSTOPROREC: null,
                        __is_new: false
                    };
                    produtos.push(rowProd);
                }
            }
        } else {
            if (widget.currentRow.ID === "P") {
                rowProd = {
                    CDPRODUTO: widget.currentRow.CODIGO,
                    CDRECEITA: data[i].CDRECEITA,
                    NRVARIACAOREC: data[i].NRVARIACAOREC,
                    QTPRODREC: InflightorGeneral.valueToFloat(widget.currentRow.QTD),
                    NRORDPREC: widget.currentRow.ORDEM,
                    IDPREFERENCIA: widget.currentRow.IDPREFERENCIA,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    VRCUSTOPROREC: null,
                    __is_new: false
                };
                produtos.push(rowProd);
            }
            if (widget.currentRow.ID === "S") {
                var rowSubrec = {
                    CDSUBRECEITA: widget.currentRow.CODIGO,
                    CDRECEITA: widget.container.getWidget("receitas").currentRow.CDRECEITA,
                    NRVARIACAOREC: data[i].NRVARIACAOREC,
                    QTRENDSUBREC: InflightorGeneral.valueToFloat(widget.currentRow.QTD),
                    NRORDSUBREC: widget.currentRow.ORDEM,
                    IDPREFERENCIA: widget.currentRow.IDPREFERENCIA,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    VRCUSTOSUBREC: null,
                    __is_new: false
                };
                receitas.push(rowSubrec);
            }
        }

        if (receitas.length) {
            promises.push(coc01400_receitaService.updateSubReceita(receitas));
        }
        if (produtos.length) {
            promises.push(coc01400_receitaService.updateSubProd(produtos));
        }

        ZHPromise.all(promises).then(function() {
            widget.commitInlineEdit();
            widget.list();
            widget.showCheckbox = true;
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
            widget.container.getWidget("variacoesReceita").reload().then(_ => {
                let widgetDetalhes = widget.container.getWidget('detalhesvariacao');
                let widgetVariacaoRec = widget.container.getWidget('variacoesReceita');
                let row = widgetVariacaoRec.dataSource.data.find(item => item.NRVARIACAOREC === widgetDetalhes.currentRow.NRVARIACAOREC);
                if (row && setCurrentRow) {
                    widgetVariacaoRec.currentRow = row;
                }
            });
            setStripeFooter(widget.container.getWidget("composicaoprodutos").dataSource.data, widget);

        }, function(err) {

            if (err.data) {

                ScreenService.showMessage(err.data.error, "error");

            } else {

                MessageService.showMessageByCode("NO_CONNECTION");

            }

        });

    };

    this.cancelaAlteracoes = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
    };

    this.addUnidade = function(widget) {

        var rowVariacao = widget.container.getWidget("unidade").dataSourceFilter;
        var cdreceita;

        for (let i in rowVariacao) {
            if (rowVariacao[i].name == "CDRECEITA") {
                cdreceita = rowVariacao[i].value;
            }
        }

        var data = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        widget.dataSourceFilter = data;
        widget.reload();
        ScreenService.openPopup(widget);
        widget.activate();
    };

    this.salvaUnidade = function(widget) {

        var row = widget.container.getWidget("variacoesReceita").currentRow;
        var unidade = widget.currentRow;
        var data = {
            CDRECEITA: row.CDRECEITA,
            NRVARIACAOREC: row.NRVARIACAOREC,
            CDFILIAL: unidade.CDFILIAL,
            DTINCLUSAO: moment().format("DD/MM/YYYY"),
            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
            __is_new: true
        };
        if (widget.isValid()) {
            coc01400_receitaService.salvarUnidadesVariacao(data).then(function() {
                widget.container.getWidget("unidade").reload();
                ScreenService.closePopup(widget);
                widget.container.getWidget("unidade").activate();
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                widget.currentRow.NRVARIACAOREC = widget.container.getWidget('variacoesReceita').currentRow.NRVARIACAOREC;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);

            }, function(callback) {
                InflightorGeneral.callbackError(callback);
            });
        }
    };

    this.removeUnidade = function(widget) {

        var unidades = widget.getCheckedRows();

        if (unidades.length > 0) {

            coc01400_receitaService.removeUnidadeVariacao(unidades).then(function() {
                widget.container.getWidget("unidade").reload();
                widget.container.getWidget("unidade").activate();
            });

        }

    };

    this.setUnMedida = function(widget) {
        var data = widget.dataSource.data;
        data.forEach(function(row) {
            row.SGUNIDADE = widget.container.getWidget("detalheReceita").currentRow.SGUNIDADE;
        });
    };

    this.addCompReceita = function(widget) {

        var filtro = [];
        var receitaPai = widget.container.getWidget("receitas").currentRow.CDRECEITA;
        var receitaJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

        filtro.push(receitaPai);

        if (receitaJaInc.length > 0 && receitaJaInc.ID == "S") {

            for (let i in receitaJaInc) {
                filtro.push(receitaJaInc[i].CODIGO);
            }

        }

        var dsfiltro = {
            "name": "CDRECEITA",
            "operator": "NOT_IN",
            "value": filtro
        };

        widget.getField("DSRECEITA").dataSourceFilter = dsfiltro;
        widget.getField("DSRECEITA").reload().then(function() {
            widget.newRow();
            ScreenService.openPopup(widget);
        });

    };

    this.getDataSourceFilterComposicao = function(widget) {
        return [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": widget.currentRow.CDRECEITA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
    };

    this.addNovaCompReceita = function(widget) {

        if (widget.isValid()) {
            var novasReceitas = widget.currentRow;
            if (novasReceitas.IDATIVO == "N") {
                return MessageService.showMessageByCode("COC01400_INACTIVE_RECIPE");
            } else {
                //Valida se a receita possui composição
                InflightorGeneral.callRestEngine("coc#/crud/coc01400_receitas_composicao", "FilterData", self.getDataSourceFilterComposicao(widget)).then(
                    function(callback) {
                        if (callback && callback.dataset.coc01400_receitas_composicao && callback.dataset.coc01400_receitas_composicao.length) {
                            var rowVariacaoReceita = widget.container.getWidget("variacoesReceita").currentRow;
                            var receitas = [];
                            var receitaPai = widget.container.getWidget("receitas").currentRow.CDRECEITA;
                            var receitaJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

                            if (novasReceitas.CDRECEITA == receitaPai) {
                                return MessageService.showMessageByCode("COC01400_ERROR_ADD_RECIPE");
                            }

                            for (let j in receitaJaInc) {
                                if (novasReceitas.CDRECEITA == receitaJaInc[j].CODIGO && receitaJaInc[j].ID == "S") {
                                    return ScreenService.showMessage("Não é possivel adicionar uma receita já existente. Receita " + receitaJaInc[j].NOME + " em duplicidade.");
                                }
                            }

                            if (!(novasReceitas.QTRENDSUBREC) || novasReceitas.QTRENDSUBREC === 0 || novasReceitas.QTRENDSUBREC == "0,000") {
                                return MessageService.showMessageByCode("QTDE_NOT_INFORMED");
                            }

                            if (!novasReceitas.NRORDSUBREC) {
                                novasReceitas.NRORDSUBREC = 1;
                            }

                            receitas.push({
                                CDSUBRECEITA: novasReceitas.CDRECEITA,
                                CDRECEITA: rowVariacaoReceita.CDRECEITA,
                                NRVARIACAOREC: rowVariacaoReceita.NRVARIACAOREC,
                                QTRENDSUBREC: InflightorGeneral.valueToFloat(novasReceitas.QTRENDSUBREC),
                                NRORDSUBREC: novasReceitas.NRORDSUBREC,
                                DTINCLUSAO: moment().format("DD/MM/YYYY"),
                                VRCUSTOSUBREC: null,
                                __is_new: true
                            });

                            coc01400_receitaService.salvarNovaRecCompVariacao(receitas).then(function() {
                                widget.container.getWidget("composicaoprodutos").reload().then(function() {
                                    ScreenService.closePopup(true);
                                    widget.container.getWidget("composicaoprodutos").activate();
                                    self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, getNrvariacaoFromWidgetFilter(widget.container.getWidget("composicaoprodutos")));
                                    widget.container.getWidget("composicaoprodutos").refreshWidget();
                                });
                                widget.currentRow.CDRECEITA = widget.container.getWidget('receitas').currentRow.CDRECEITA;
                                widget.currentRow.NRVARIACAOREC = widget.container.getWidget('variacoesReceita').currentRow.NRVARIACAOREC;
                                self.abreDetalheReceita(widget, false);
                                self.atualizaFormularioVariacao(widget);
                            });
                        } else {
                            ScreenService.showMessage(ScreenService.i18n("Impossível adicionar uma Receita sem Composição!"), "error");
                        }
                    }, InflightorGeneral.callbackError);
            }
        }

    };

    var getNrvariacaoFromWidgetFilter = function(widget) {
        var dtsFilter = widget.dataSourceFilter;
        var condition = dtsFilter.filter(function(item) {
            return item.name == "NRVARIACAOREC";
        });
        return condition[0].value;
    };

    this.removeRecCompVariacao = function(widget) {

        var receitas = widget.currentRow;

        coc01400_receitaService.removeReceitaVariacao(receitas, widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, widget.currentRow.NRVARIACAOREC).then(function() {
            widget.unSelectAll();
            widget.container.getWidget("composicaoprodutos").reload().then(function() {
                self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, widget.currentRow.NRVARIACAOREC);
                widget.container.getWidget("composicaoprodutos").activate();
            });
        });
        widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
        self.abreDetalheReceita(widget, false);
        self.atualizaFormularioVariacao(widget);

    };

    this.adicionaVariacao = function(widget) {

        var row = widget.currentRow;
        if (widget.isValid()) {
            coc01400_receitaService.novaVariacao(cdreceita, row).then(function() {
                widget.container.getWidget("variacoesReceita").reload().then(function() {
                    widget.container.getWidget("variacoesReceita").activate();
                    ScreenService.closeSwipe();
                });
            });
        }

    };

    this.removeVariacao = function(widget) {

        var variacaoRec = widget.currentRow;
        var data = [];

        data.push({
            CDRECEITA: variacaoRec.CDRECEITA,
            NRVARIACAOREC: variacaoRec.NRVARIACAOREC,
            NRORG: variacaoRec.NRORG,
            __is_new: false
        });
        coc01400_receitaService.removeVariacaoReceitasemWorkflow(data).then(function() {
            widget.container.getWidget("variacoesReceita").reload().then(function() {
                ScreenService.closeSwipe();
                widget.container.getWidget("variacoesReceita").activate();
            });
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
        }, function(erro) {
            if (erro.data && erro.data.error.indexOf("ORA-02292") !== -1) {
                MessageService.showMessageByCode("COC01400_DEL_NOT_PERMISSION");
            } else {
                ScreenService.showMessage(erro.data.error);
            }

        });

    };

    this.salvaEditVariacaoReceita = function(widget) {

        var data = widget.currentRow;
        widget.currentRow.OPERADOR_TO_SAVE = widget.currentRow.CDOPERADOR;
        widget.currentRow.QTRENDIMENTOREC = InflightorGeneral.valueToFloat(widget.currentRow.QTRENDIMENTOREC);
        coc01400_receitaService.salvaEdicaoVariacaoRec(data).then(function() {
            widget.container.getWidget("variacoesReceita").reload().then(function() {
                widget.currentRow = widget.container.getWidget("variacoesReceita").currentRow;
                widget.reload().then(function() {
                    widget.activate();
                    widget.backupRow = undefined;
                    self.cancelModEdit(widget.container.getWidget("versao"), widget);
                    converteFieldsReceita(data, "find");
                });
            });
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
        });

    };

    var errorValidations = function(error) {

        /*
to do: faltam cercar alguns erros que podem surgir caso a variação seja fk de alguma outra tabela.
*/

        var errors = {
            FKRELATIONVARIACAO: {
                desc: "Unidade",
                message: "COC01400_VARIATION_LINKED_UNIT"
            },
            FKRECSUB2VARIACAO: {
                desc: "receitas",
                message: "COC01400_VARIATION_HAS_RECIPES"
            },
            FKVARIPRORVARIACAO: {
                desc: "produtos",
                message: "COC01400_VARIATION_HAS_PRODUCTS"
            },
            FKRELATIONRECEITAM: {
                desc: "variacoes",
                message: "COC01400_RECIPES_HAS_VARIATION"
            }
        };

        if ((error.search("FKRELATIONVARIACAO")) > 0) {

            MessageService.showMessageByCode(errors.FKRELATIONVARIACAO.message);
            return null;

        } else if ((error.search("FKRECSUB2VARIACAO")) > 0) {

            MessageService.showMessageByCode(errors.FKRECSUB2VARIACAO.message);
            return null;

        } else if ((error.search("FKVARIPRORVARIACAO")) > 0) {

            MessageService.showMessageByCode(errors.FKVARIPRORVARIACAO.message);
            return null;

        } else if ((error.search("FKRELATIONRECEITAM")) > 0) {

            MessageService.showMessageByCode(errors.FKRELATIONRECEITAM.message);
            return null;

        } else {

            ScreenService.showMessage(error);
            return null;

        }

    };

    this.salvaNovaReceita = function(widget) {

        var row = widget.currentRow;

        if (widget.isValid()) {
            row.__is_new = true;
            converteFieldsReceita(row, "save");
            coc01400_receitaService.novaReceita(row).then(function() {
                ScreenService.closeSwipe();
                widget.container.getWidget("receitas").reload();
            });

        }

    };
    this.validaVariacaoReceita = function(row) {
        return coc01400_receitaService.validateVariacao(row).then(function(callback) {
            if (callback.dataset && callback.dataset.variacaorec_operador && callback.dataset.variacaorec_operador.length) {
                return "S";
            } else {
                return "N";
            }
        }, function(error) {
            return InflightorGeneral.callbackError(error);
        });
    };

    this.removeReceita = function(widget) {
        widget.currentRow.P_IDVIGENCIA = "T";

        ScreenService.confirmMessage(ScreenService.i18n("A exclusão apagará todas as variações e composições da receita. Confirmar operação?"), "question", function() {
            coc01400_receitaService.excluiReceitasemWorkflow(widget.currentRow).then(function(callback) {
                if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                    InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, "Operação bloqueada");
                } else {
                    widget.container.getWidget("receitas").reload().then(function() {
                        ScreenService.closePopup(true);
                        ScreenService.closeSwipe();
                        ScreenService.showMessage(ScreenService.i18n("Receita excluída com sucesso."), "success").then(function() {
                            widget.container.getWidget("receitas").activate();
                        });
                    });
                }
            }, function(err) {
                errorValidations(err.data.error);
            });
        });
    };

    this.adicionarCategoria = function(field) {
        ScreenService.openPopup(field.widget.container.getWidget("novaCategoria"));
    };

    this.addNovaCategoria = function(widget) {
        var row = widget.currentRow;
        if (widget.isValid()) {
            coc01400_receitaService.addCategoria(row).then(function() {
                widget.container.getWidget("novaReceita").getField("NMCATEGORIA").reload().then(function() {
                    ScreenService.closePopup();
                });
            });
        }
    };

    this.addProduto = function(widget) {

        var filtro = [];
        var produtoJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

        if (produtoJaInc.length > 0) {

            for (let i in produtoJaInc) {
                if (produtoJaInc[i].ID == "P")
                    filtro.push(produtoJaInc[i].CODIGO);
            }
            var dsfiltro = [{
                    "name": "CDPRODUTO",
                    "operator": "NOT_IN",
                    "value": filtro
            },
                {
                    "name": "IDPRODCOMISS",
                    "operator": "IS_NULL",
                    "value": ""
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    "name": "SGUNIDADE",
                    "operator": "IS_NOT_NULL",
                    "value": ""
            }
            ];

            widget.getField("NMPRODUTO").dataSourceFilter = dsfiltro;

        }

        widget.getField("NMPRODUTO").reload().then(function() {
            widget.newRow();
            ScreenService.openPopup(widget);
        });

    };

    this.addNovaCompProduto = function(widget) {

        if (widget.isValid()) {
            var novosProdutos = widget.currentRow;
            if (novosProdutos.IDATIVO == "N") {
                return MessageService.showMessageByCode("INACTIVE_PRODUCT");
            } else {
                var rowVariacaoReceita = widget.container.getWidget("variacoesReceita").currentRow;
                var receitas = [];
                var produtoJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

                for (let j in produtoJaInc) {
                    if (novosProdutos.CDPRODUTO == produtoJaInc[j].CODIGO) {
                        return ScreenService.showMessage("Não é possivel adicionar um Produto já existente. Produto " + produtoJaInc[j].NMPRODUTO + " em duplicidade.");
                    }
                }

                if (!(novosProdutos.QTPRODREC) || novosProdutos.QTPRODREC === 0 || novosProdutos.QTPRODREC == "0,000") {
                    return MessageService.showMessageByCode("QTDE_NOT_INFORMED");
                }

                if (!novosProdutos.NRORDPREC) {
                    novosProdutos.NRORDPREC = 1;
                }

                receitas.push({
                    CDPRODUTO: novosProdutos.CDPRODUTO,
                    CDRECEITA: rowVariacaoReceita.CDRECEITA,
                    NRVARIACAOREC: rowVariacaoReceita.NRVARIACAOREC,
                    QTPRODREC: InflightorGeneral.valueToFloat(novosProdutos.QTPRODREC),
                    NRORDPREC: novosProdutos.NRORDPREC,
                    SGGRUPORECFLEX: novosProdutos.SGGRUPORECFLEX,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    VRCUSTOPROREC: null,
                    __is_new: true
                });
                coc01400_receitaService.salvarNovoProdCompVariacaosemWorkflow(receitas).then(function() {
                    widget.container.getWidget("composicaoprodutos").reload().then(function() {
                        ScreenService.closePopup(true);
                        widget.container.getWidget("composicaoprodutos").activate();
                        self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), rowVariacaoReceita.CDRECEITA, rowVariacaoReceita.NRVARIACAOREC);
                        widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                        widget.currentRow.NRVARIACAOREC = widget.currentRow.NRVARIACAOREC || widget.container.getWidget('variacoesReceita').currentRow.NRVARIACAOREC;
                        self.abreDetalheReceita(widget, false);
                        self.atualizaFormularioVariacao(widget);
                    });

                });
            }
        }

    };

    this.excluiProdCompVariacao = function(widget) {
        var variacao = widget.currentRow.NRVARIACAOREC;
        var produtos = widget.currentRow;
        coc01400_receitaService.removeProdCompVariacao(produtos, widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, variacao).then(function() {
            widget.unSelectAll();
            widget.container.getWidget("composicaoprodutos").reload().then(function() {
                self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, widget.currentRow.NRVARIACAOREC);
                widget.container.getWidget("composicaoprodutos").activate();
            });
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
        });
    };

    this.validateToSave = function(widget) {
        if (widget.dataSource.data.length) {
            if (widget.currentRow.DSMPREPRECHTML && widget.currentRow.DSMPREPRECHTML.length) {
                isNew = false;
            } else {
                isNew = true;
            }
        } else {
            isNew = true;
        }
        return isNew;
    };

    this.salvaEditModopreprecVariacao = function(widget) {

        var row = widget.currentRow;
        if (widget.isValid()) {

            var rowVariacaoReceita = widget.container.getWidget("variacoesReceita").currentRow;

            row.CDRECEITA = rowVariacaoReceita.CDRECEITA;
            row.NRVARIACAOREC = rowVariacaoReceita.NRVARIACAOREC;
            row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
            row.__is_new = self.validateToSave(widget);
            row.DTINCLUSAO = moment().format("DD/MM/YYYY");
            row.DTULTATU = row.DTINCLUSAO;

            coc01400_receitaService.salvaNovoModopreprec(row).then(function() {
                widget.reload().then(function() {
                    widget.activate();
                });
                self.cancelModEdit(widget.container.getWidget("versao"), widget);
                widget.container.getWidget("versao").currentWidget = "detalhesvariacao";
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);
            });
        }

    };

    this.duplicarVariacao = function(widget) {

        var row = widget.currentRow;

        coc01400_receitaService.duplicaVariacaoReceita(row).then(function() {
            widget.container.getWidget("variacoesReceita").reload().then(function() {
                ScreenService.closeSwipe();
                widget.container.getWidget("variacoesReceita").activate();
            });
        }, function(err) {
            ScreenService.showMessage(err.message);
        });

    };
    this.openPopupDuplicacao = function(widget) {
        var widgetPopup = widget.container.getWidget("nomeReceitaDuplicada");
        ScreenService.openPopup(widgetPopup);
        widgetPopup.getField("DSRECEITA").value(rowDuplicacao.DSRECEITA);
    };

    this.salvaRowTemp = function(row) {
        rowDuplicacao = row;
    };

    this.deleteModpreprecm = function(widget) {
        var row = widget.currentRow;
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            coc01400_receitaService.deleteModopreprec(row).then(function() {
                self.onEnterDadosVersao(widget);
                widget.activate();
                self.setCurrentForm(widget);
                widget.getField("DSMPREPRECHTML").setValue("");
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.duplicarReceita = function(widget) {

        var row = widget.currentRow;
        rowDuplicacao.DSRECEITA = row.DSRECEITA;
        converteFieldsReceita(rowDuplicacao, "save");
        coc01400_receitaService.duplicaReceita(rowDuplicacao).then(function() {
            widget.container.getWidget("receitas").reload().then(function() {
                ScreenService.closePopup();
                ScreenService.closeSwipe();
                widget.container.getWidget("receitas").activate();
                rowDuplicacao = {};
            });
        }, function(err) {
            ScreenService.showMessage(err.message);
        });

    };

    this.salvaTesteReceita = function(widget) {
        var current = widget.container.getWidget("variacoesReceita").currentRow;
        widget.currentRow.CDRECEITA = current.CDRECEITA;
        widget.currentRow.NRVARIACAOREC = current.NRVARIACAOREC;
        InflightorGeneral.saveCrud(widget);
    };

    this.onEnterDadosReceita = function(widget) {

        var row = widget.container.getWidget("receitas").currentRow;
        var cdreceita = row.CDRECEITA;

        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            },
        ];

        widget.dataSourceFilter = filtro;
        if (widget.mode == "list") {
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    this.onEnterDadosManual = function(widget) {

        var row = widget.container.getWidget("receitas").currentRow;
        var cdreceita = row.CDRECEITA;

        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_CDFILIAL")
            }
        ];

        widget.dataSourceFilter = filtro;
        if (widget.mode == "list") {
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    this.cancelarEditReceitaTeste = function(args) {

        var currentRow = args.owner.widget.currentRow;

        if (currentRow.__is_new) {
            InflightorGeneral.backOrCancel(args);
            args.owner.widget.reload().then(function() {
                args.owner.widget.list();
            });
        } else {
            InflightorGeneral.backOrCancel(args);
        }

    };

    this.calculaCusto = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("composicaoprodutos");
        var variacao = widget.container.getWidget("variacoesReceita");
        var currentPai = gridWidget.dataSource.data;
        var DATACUSTO = moment(current.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY");
        var row;
        if (widget.isValid()) {

            var data = [];
            for (let i in currentPai) {
                row = {
                    DATACUSTO: DATACUSTO,
                    TIPOCUSTO: current.TIPOCUSTO,
                    CDFILIAL: getLocalVar("INFLIGHTOR_CDFILIAL"),
                    CODIGO: currentPai[i].CODIGO,
                    NRVARIACAO: nrvariacaorec,
                    ID: currentPai[i].ID
                };
                data.push(row);
            }
            coc01400_receitaService.findCusto(data).then(function(callback) {
                var custoAll = callback.dataset.composicao;
                self.updateVariacaoRec(current, variacao.currentRow);
                self.updateDataSourceforWidget(gridWidget, currentPai, custoAll);

            });
            //Find para o Custo Líquido do stripe footer
            var dataRec = [];
            row = {
                DATACUSTO: DATACUSTO,
                TIPOCUSTO: current.TIPOCUSTO,
                CDFILIAL: getLocalVar("INFLIGHTOR_CDFILIAL"),
                CODIGO: widget.container.getWidget("receitas").currentRow.CDRECEITA,
                NRVARIACAO: nrvariacaorec,
                ID: "S"
            };
            dataRec.push(row);
            coc01400_receitaService.findCusto(dataRec).then(function(callback) {
                var custoAll = callback.dataset.composicao;
                var custoUnitario = custoAll[0].CUSTO;
                var custoRend = custoAll[0].CUSTO * widget.container.getWidget("variacoesReceita").currentRow.QTRENDIMENTOREC;
                var stripeFooter = widget.container.getWidget("stripefooter_composicao");
                stripeFooter.getField("CUSRENDLIQ").setValue(custoRend.toFixed(3));
                stripeFooter.getField("CUSTOUN").setValue(custoUnitario.toFixed(3));
                stripeFooter.getField("QTRENDBRUTO").setValue(rendBruto.toFixed(3));
                stripeFooter.getField("QTPRODTCOMP").setValue(rendLiquido.toFixed(3));
                stripeFooter.getField("QTPERCENTAPRO").setValue(percentLiq);
                stripeFooter.reload();
            });
        }
    };

    this.updateDataSourceforWidget = function(gridWidget, currentPai, custoAll) {
        currentPai.forEach(function(datasetRow) {
            var custoCodigo = custoAll.filter(function(row) {
                return row.CODIGO == datasetRow.CODIGO;
            });
            datasetRow.CUSTO = InflightorGeneral.valueToFloat(custoCodigo[0].CUSTO) * InflightorGeneral.valueToFloat(datasetRow.QTD);
            datasetRow.CUSTOUN = datasetRow.CUSTO;
            if (datasetRow.ID == "P")
                datasetRow.CUSTO *= datasetRow.YIELD;
        });

        gridWidget.dataSource.data = currentPai;
        gridWidget.reload().then(function() {
            self.updateSubRecandProd(currentPai);
            ScreenService.closePopup();
            gridWidget.activate();
        });
    };

    this.updateSubRecandProd = function(data) {
        var dataRec = [];
        var dataProd = [];
        var promises = [];
        data.forEach(function(rows) {
            if (rows.ID == "S") {
                var row = {
                    CDSUBRECEITA: rows.CODIGO,
                    CDRECEITA: cdreceita,
                    NRVARIACAOREC: nrvariacaorec,
                    NRORDSUBREC: rows.ORDEM,
                    QTRENDSUBREC: rows.QTD,
                    VRCUSTOSUBREC: rows.CUSTO,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    __is_new: false
                };
                dataRec.push(row);
            } else {
                var rowprod = {
                    CDPRODUTO: rows.CODIGO,
                    CDRECEITA: cdreceita,
                    NRVARIACAOREC: nrvariacaorec,
                    QTPRODREC: rows.QTD,
                    NRORDPREC: rows.ORDEM,
                    SGGRUPORECFLEX: rows.SGGRUPORECFLEX,
                    VRCUSTOPROREC: rows.CUSTO,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    __is_new: false
                };
                dataProd.push(rowprod);
            }
        });
        promises.push(coc01400_receitaService.updateSubReceita(dataRec));
        promises.push(coc01400_receitaService.updateSubProd(dataProd));

        ZHPromise.all(promises).then(null, function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    this.controlExclusive = function(widget) {
        widget.getField("NMCIA").isVisible = false;
        widget.getField("NMCIA").showOnForm = false;
        widget.getField("NMCIA").showOnView = false;
        clearField(widget.getField("NMCIA"));
        widget.getField("NMCIA").validations = [];
        widget.getField("DSGRUPOCIA").isVisible = false;
        widget.getField("DSGRUPOCIA").showOnForm = false;
        widget.getField("DSGRUPOCIA").showOnView = false;
        clearField(widget.getField("DSGRUPOCIA"));
        widget.getField("DSGRUPOCIA").validations = [];
    };

    this.setExcluRec = function(widget) {
        var data = widget.currentRow;
        if (data.CDCIA) {
            data.ESCOLHECIAOUGRUPO = "1";
            widget.getField("NMCIA").isVisible = true;
            widget.getField("NMCIA").showOnForm = true;
            widget.getField("NMCIA").showOnView = true;
            widget.getField("NMCIA").validations = {
                required: true
            };
            widget.getField("DSGRUPOCIA").isVisible = false;
            widget.getField("DSGRUPOCIA").showOnForm = false;
            clearField(widget.getField("DSGRUPOCIA"));
            widget.getField("DSGRUPOCIA").validations = {
                required: false
            };
        } else if (data.NRGRUPOCIA) {
            data.ESCOLHECIAOUGRUPO = "2";
            widget.getField("DSGRUPOCIA").isVisible = true;
            widget.getField("DSGRUPOCIA").showOnForm = true;
            widget.getField("DSGRUPOCIA").showOnView = true;
            widget.getField("DSGRUPOCIA").validations = {
                required: true
            };
            widget.getField("NMCIA").isVisible = false;
            widget.getField("NMCIA").showOnForm = false;
            clearField(widget.getField("NMCIA"));
            widget.getField("NMCIA").validations = {
                required: false
            };
        } else {
            data.ESCOLHECIAOUGRUPO = "0";
            widget.getField("NMCIA").isVisible = false;
            widget.getField("NMCIA").showOnForm = false;
            widget.getField("NMCIA").showOnView = false;
            clearField(widget.getField("NMCIA"));
            widget.getField("NMCIA").validations = [];
            widget.getField("DSGRUPOCIA").isVisible = false;
            widget.getField("DSGRUPOCIA").showOnForm = false;
            widget.getField("DSGRUPOCIA").showOnView = false;
            clearField(widget.getField("DSGRUPOCIA"));
            widget.getField("DSGRUPOCIA").validations = [];
        }
    };

    function clearField(field) {
        if (field.clearByOutData) {
            field.clearValue();
        } else {
            field.setValue('');
        }
    }

    this.onChangeExclusiv = function(field) {
        if (field.value() == 1) {
            field.widget.getField("NMCIA").isVisible = true;
            field.widget.getField("NMCIA").showOnForm = true;
            field.widget.getField("NMCIA").validations = {
                required: true
            };
            field.widget.getField("DSGRUPOCIA").isVisible = false;
            field.widget.getField("DSGRUPOCIA").showOnForm = false;
            clearField(field.widget.getField("DSGRUPOCIA"));
            field.widget.getField("DSGRUPOCIA").validations = {
                required: false
            };
        } else if (field.value() == 2) {
            field.widget.getField("DSGRUPOCIA").isVisible = true;
            field.widget.getField("DSGRUPOCIA").showOnForm = true;
            field.widget.getField("DSGRUPOCIA").validations = {
                required: true
            };
            field.widget.getField("NMCIA").isVisible = false;
            field.widget.getField("NMCIA").showOnForm = false;
            clearField(field.widget.getField("NMCIA"));
            field.widget.getField("NMCIA").validations = {
                required: false
            };
        } else {
            field.widget.getField("NMCIA").isVisible = false;
            field.widget.getField("NMCIA").showOnForm = false;
            clearField(field.widget.getField("NMCIA"));
            field.widget.getField("NMCIA").validations = [];
            field.widget.getField("DSGRUPOCIA").isVisible = false;
            field.widget.getField("DSGRUPOCIA").showOnForm = false;
            clearField(field.widget.getField("DSGRUPOCIA"));
            field.widget.getField("DSGRUPOCIA").validations = [];
        }
        templateManager.updateTemplate();
    };

    this.updateVariacaoRec = function(current, currentVari) {

        var DATACUSTO = moment(current.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY");

        currentVari.DTCUSTOVARIACAO = DATACUSTO;
        currentVari.IDTIPOCUSTOVAR = current.TIPOCUSTO;

        coc01400_receitaService.updateVariacaoRec(currentVari).then(null, function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    var timeFormats = {
        "hh:mm": isValidHourAndMinutes
    };

    function isValidMinutesOrSeconds(minuteOrSecond) {

        if (parseInt(minuteOrSecond) > 59)
            return false;
        else
            return true;
    }

    function isValidHourAndMinutes(hourAndMinute) {
        if (hourAndMinute) {
            var timeArray = hourAndMinute.split(":");
            return isValidMinutesOrSeconds(timeArray[1]);
        } else
            return true;
    }

    var validTime = function(value, validationAttrs) {
        var validFunction = timeFormats[validationAttrs.format.toLowerCase()];

        return {
            "valid": validFunction(value),
            "message": ScreenService.i18n("Is not a valid time")
        };
    };

    ValidationEngine.registerValidation("isValidMinute", validTime);

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01400_receitaController", coc01400_receitaController);
});

// FILE: ./assets/js/controllers/coc01400_receitaWorkflowController.js
var coc01400_receitaWorkflowController = function(ScreenService, MessageService, templateManager, $http, ZHPromise, maskEngine, ZhUtil, restEngine, InflightorGeneral, coc01400_receitaService, ValidationEngine, coc01400_receitaController, FieldUtil) {

    var self = this;
    var cdreceita;
    var nrvariacaorec;
    var rowDuplicacao;
    var isNew;
    var rendBruto;
    var rendLiquido;
    var percentLiq;
    var data = [];
    var possuiWflow;

    function getRowToReport(widget) {
        return {
            CDCATEGORIA: "T",
            CDCIA: "T",
            CDFILIAL: getLocalVar("INFLIGHTOR_CDFILIAL"),
            CDLEANCODE: "T",
            CDMANCIA: "T",
            CDPRODUTO: "",
            CDRECEITA: "_" + widget.currentRow.CDRECEITA + "_",
            CDSETOR: "T",
            IDPRODATIVPR: "S",
            IDSITUAPROPCIA: "T",
            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
            P_ZEBRADO: "N",
            REPORTNAME: "COC00992",
            __format: "pdf"
        };
    }

    this.openWindowReport = function(widget) {
        InflightorGeneral.openReport("coc#/relatorioDeReceitas", "COC00992", getRowToReport(widget));
    };

    this.abreReceitasDoGrupo = function(args) {
        var swipeWidget = args.owner.container.getWidget("receitasDoGrupo");
        var filtro = [
            {
                "name": "NRARVOREREC",
                "operator": "=",
                "value": args.row.NRARVOREREC
            }];
        if (!(args.row._hasChild)) {
            swipeWidget.dataSourceFilter = filtro;
            swipeWidget.reload().then(function() {
                ScreenService.openSwipe(swipeWidget);
            });
        }
    };

    function converteFieldsReceita(row, operator) {

        if (operator == "find") {
            switch (row.NRNIVELDIFICULDADE) {
                case 1:
                    row.NRNIVELDIFICULDADE = "Fácil";
                    break;
                case 2:
                    row.NRNIVELDIFICULDADE = "Médio";
                    break;
                case 3:
                    row.NRNIVELDIFICULDADE = "Díficil";
                    break;
            }
            switch (row.IDATIVO) {
                case "S":
                    row.IDATIVO = "Ativa";
                    break;
                case "N":
                    row.IDATIVO = "Inativa";
                    break;
            }
            switch (row.IDANALIZEATIV) {
                case "S":
                    row.IDANALIZEATIV = "Aguardando Solicitação de Workflow";
                    break;
                case "P":
                    row.IDANALIZEATIV = "Solicitação gerada e Pendente";
                    break;
                case "A":
                    row.IDANALIZEATIV = "Solicitação Aprovada";
                    break;
                case "N":
                    row.IDANALIZEATIV = "Solicitação Reprovada";
                    break;
                case "I":
                    row.IDANALIZEATIV = "Não utiliza Workflow";
                    break;
            }
            if (row.NRTEMPOPREPARO)
                row.NRTEMPOPREPARO = InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO);
        } else if (operator == "save") {
            if (row.NRTEMPOPREPARO)
                row.NRTEMPOPREPARO = InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO);
            switch (row.NRNIVELDIFICULDADE) {
                case "Fácil":
                    row.NRNIVELDIFICULDADE = 1;
                    break;
                case "Médio":
                    row.NRNIVELDIFICULDADE = 2;
                    break;
                case "Díficil":
                    row.NRNIVELDIFICULDADE = 3;
                    break;
            }
            switch (row.IDATIVO) {
                case "Ativa":
                    row.IDATIVO = "S";
                    break;
                case "Inativa":
                    row.IDATIVO = "N";
                    break;
            }
            switch (row.IDANALIZEATIV) {
                case "Aguardando Solicitação de Workflow":
                    row.IDANALIZEATIV = "S";
                    break;
                case "Solicitação gerada e Pendente":
                    row.IDANALIZEATIV = "P";
                    break;
                case "Solicitação Aprovada":
                    row.IDANALIZEATIV = "A";
                    break;
                case "Solicitação Reprovada":
                    row.IDANALIZEATIV = "N";
                    break;
                case "Não utiliza Workflow":
                    row.IDANALIZEATIV = "I";
                    break;
            }
        }
    }

    this.abreDetalheReceita = function(widget, openSwipe = true) {

        var swipeWidget = widget.container.getWidget("receita");
        var formDetalheReceita = widget.container.getWidget("detalheReceita");
        var currentRow = widget.currentRow;
        coc01400_receitaService.retornaDetalhesReceita(currentRow.CDRECEITA).then(function(callback) {
            var rows = callback.dataset.receitam_operador_companhia[0];
            converteFieldsReceita(rows, "find");
            formDetalheReceita.currentRow = rows;
            formDetalheReceita.container.getWidget("receitas").descriptionOrder = rows.DSRECEITA;
            formDetalheReceita.container.getWidget("receitas").currentRow = rows;
            if (formDetalheReceita.currentRow.IDATIVO == "Inativa" && formDetalheReceita.currentRow.IDANALIZEATIV == "Aguardando Solicitação de Workflow") {
                formDetalheReceita.getAction("WORKFLOW").isVisible = true;
            } else {
                formDetalheReceita.getAction("WORKFLOW").isVisible = false;
            }
            formDetalheReceita.reload().then(function() {
                if (openSwipe) {
                    ScreenService.openSwipe(swipeWidget);
                }
            });
        }, function(err) {
            ScreenService.errorNotification(err);
        });

    };

    this.atualizaFormularioVariacao = function(widget) {

        var formularioVariacao = widget.container.getWidget("detalhesvariacao");
        var currentRow = widget.currentRow;
        coc01400_receitaService.atualizaFormulario(currentRow.CDRECEITA, currentRow.NRVARIACAOREC).then(function(callback) {
            var rows = callback.dataset.variacaorec_operador[0];
            formularioVariacao.currentRow = rows;
            formularioVariacao.container.getWidget("variacoesReceita").currentRow = rows;
            formularioVariacao.reload();
        }, function(err) {
            ScreenService.errorNotification(err);
        });

    };

    this.setDataSourceForGrid = function(widget, cdreceita, nrvariacaorec) {
        if (widget.mode == "list") {
            var filterObj = {
                NRVARIACAOREC: nrvariacaorec,
                CDRECEITA: cdreceita
            };
            InflightorGeneral.buildDataSourceFilter(widget.dataSourceFilter, filterObj);
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    this.setStripeFooterValues = function(widgetComposicao) {
        var stripeFooter = widgetComposicao.container.getWidget("stripefooter_composicao");
        var widgetVariacao = widgetComposicao.container.getWidget("detalhesvariacao");
        InflightorGeneral.callRestEngine("coc#/crud/coc01400_get_rendimentoBruto", "FilterData", widgetComposicao.dataSourceFilter).then(
            function(callback) {
                var rendLiquido = InflightorGeneral.valueToFloat(widgetComposicao.container.getWidget("variacoesReceita").currentRow.QTRENDIMENTOREC);
                stripeFooter.getField("QTPRODTCOMP").setValue(String(rendLiquido.toFixed(3)).replace(".", ","));
                var rendBruto = InflightorGeneral.valueToFloat(callback.dataset.coc01400_get_rendimentoBruto[0].QTD);
                stripeFooter.getField("QTRENDBRUTO").setValue(String(rendBruto.toFixed(3)).replace(".", ","));
                var percentLiq;
                if (rendLiquido && rendBruto) {
                    percentLiq = ((rendLiquido * 100) / rendBruto).toFixed(2);
                } else {
                    percentLiq = 0;
                }
                stripeFooter.getField("QTPERCENTAPRO").setValue(percentLiq);
            }
        );
        verificaReceitaFlex(stripeFooter);
        stripeFooter.currentRow.NROPCOES = widgetVariacao.currentRow.NROPCOES;
        if (widgetComposicao.container.getWidget("variacoesReceita").currentRow.DTCUSTOVARIACAO) {
            self.findCusto(widgetComposicao, false);
        } else {
            widgetComposicao.container.getWidget("stripefooter_composicao").getField("CUSRENDLIQ").value("");
            widgetComposicao.container.getWidget("stripefooter_composicao").getField("CUSTOUN").value("");
        }

    };

    this.loadClassificao = function(widget) {
        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": widget.container.getWidget("receitas").currentRow.CDRECEITA
            }
        ];
        widget.dataSourceFilter = filtro;
        widget.reload().then(function() {
            widget.activate();
        });
    };

    this.loadVersoes = function(widget) {

        cdreceita = widget.container.getWidget("receitas").currentRow.CDRECEITA;
        if (cdreceita && widget.container.getWidget("receita").currentWidget) {
            var widgetCurrent = widget.container.getWidget("receita").currentWidget;
            if (widget.container.getWidget("receita").currentWidget.name == "manualReceita") {
                self.onEnterDadosManual(widgetCurrent);
            } else {
                var filtro;
                if (widget.container.getWidget("receita").currentWidget.name == "variacoesReceita") {
                    widgetCurrent.dataSource.data = [];
                    templateManager.updateTemplate();
                    filtro = [
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": cdreceita
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                        }
                    ];
                    if (widget.mode == "list" || !widget.__is_new) {
                        widgetCurrent.dataSourceFilter = filtro;
                        widgetCurrent.reload().then(function() {
                            widgetCurrent.activate();
                        });
                    }
                } else if (widget.container.getWidget("receita").currentWidget.name == "classificacao") {
                    filtro = [
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": cdreceita
                        }
                    ];
                    if (widget.mode == "list" || !widget.__is_new) {
                        widgetCurrent.dataSourceFilter = filtro;
                        widgetCurrent.reload().then(function() {
                            widgetCurrent.activate();
                        });
                    }
                }
            }
        }

    };

    this.loadVersoesVariacaoReceita = function(widget) {
        widget.dataSource.data = [];
        templateManager.updateTemplate();
        const filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            }
        ];
        if (widget.mode == "list" || !widget.__is_new) {
            widget.dataSourceFilter = filtro;
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    function applyAlterData(widget, data, possuiWflow, row) {
        var deferred = ZHPromise.defer();
        var promises = [];
        var edicoes = angular.copy(data[1]);
        edicoes.forEach(function(edited) {
            promises.push(coc01400_receitaService.getProcessosWorkflow("1", edited[Object.keys(edited)]));
        });
        ZHPromise.all(promises).then(function(callback) {
            callback.forEach(function(call, key) {
                if (call.dataset.processos_workflow.length) {
                    if (call.dataset.processos_workflow[0].IDATIVO != "S") {
                        row[Object.keys(data[1][key])] = data[2][Object.keys(data[1][key])];
                        data[0][Object.keys(data[1][key])] = data[2][Object.keys(data[1][key])];
                        delete data[2][Object.keys(data[1][key])];
                        delete data[1][key];
                    } else {
                        if (!possuiWflow)
                            possuiWflow = true;
                    }
                }
            });
            if (possuiWflow)
                return deferred.resolve();
            else
                return deferred.reject();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
        return deferred.promise;
    }

    this.solicitaSalvaEditReceita = function(widget, data, possuiWflow, row, process) {
        applyAlterData(widget, data, possuiWflow, row).then(function() {
            self.solicitarAprovacaoReceita(widget, process);
        }, function() {
            converteFieldsReceita(row, "save");
            widget.currentRow = row;
            if (process == "edit_receita") {
                coc01400_receitaController.salvaEditReceita(widget);
            } else if (process == "edit_variacao") {
                coc01400_receitaController.salvaEditVariacaoReceita(widget);
            } else if (process == "edit_variacao_comp") {
                self.saveRendLiq(widget);
            }
        });
    };

    this.validaAtivacaoReceita = function(modifiedinRows) {
        var filter = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": modifiedinRows.CDRECEITA
            }
        ];

        return InflightorGeneral.callRestEngine("coc#/composicao_mododepreparo", "FilterData", filter)
            .then()
            .catch(function(error) {
                if (error)
                    ScreenService.showMessage(error.data.error);
            });
    };

    function changeSituation(modifiedinRows) {
        return Object.keys(modifiedinRows).indexOf("IDATIVO") === -1;
    }

    this.controlaEditReceita = function(widget, process) {
        data = [];
        var valida_modpreprecm, valida_composicao;
        valida_modpreprecm = valida_composicao = false;
        possuiWflow = false;
        if (widget.isValid()) {
            if (process == "edit_variacao_comp") {
                var popupRendLiq = widget.container.getWidget("rendimentoLiquido");
                widget.backupRow = angular.copy(widget.currentRow);
                widget.currentRow.QTRENDIMENTOREC = popupRendLiq.currentRow.QTRENDIMENTOREC;
            }
            widget.currentRow.QTRENDIMENTOREC = widget.currentRow.QTRENDIMENTOREC ? widget.currentRow.QTRENDIMENTOREC : 0;
            if (widget.currentRow.IDRECFLEXIVEL == "M" || widget.currentRow.IDRECFLEXIVEL == "P") {
                if (widget.currentRow.DSRECEITA.lastIndexOf("- FLEX") !== widget.currentRow.DSRECEITA.length - 6) {
                    widget.currentRow.DSRECEITA += " - FLEX";
                }
                if (widget.currentRow.DSRECIN.lastIndexOf("- FLEX") !== widget.currentRow.DSRECIN.length - 6) {
                    widget.currentRow.DSRECIN += " - FLEX";
                }
                if (widget.currentRow.DSRECES.lastIndexOf("- FLEX") !== widget.currentRow.DSRECES.length - 6) {
                    widget.currentRow.DSRECES += " - FLEX";
                }
            }

            var row = angular.copy(widget.currentRow);

            var backupRow = angular.copy(widget.backupRow);
            if (row.NRTEMPOPREPARO)
                row.NRTEMPOPREPARO = InflightorGeneral.valueToFloat(row.NRTEMPOPREPARO);
            if (process == "edit_receita") {
                if (row.ESCOLHECIAOUGRUPO == 1) {
                    row.NRGRUPOCIA = null;
                } else if (row.ESCOLHECIAOUGRUPO == 2) {
                    row.CDCIA = null;
                } else if (row.ESCOLHECIAOUGRUPO == 0) {
                    row.CDCIA = null;
                    row.NRGRUPOCIA = null;
                }
            }
            converteFieldsReceita(backupRow, "save");
            data.push(backupRow);
            var modifiedinRows = compareRows(widget.currentRow, widget.backupRow);
            converteFieldsReceita(modifiedinRows, "save");
            data.push(self.mappingModifications(modifiedinRows));
            if (modifiedinRows) {
                modifiedinRows.CDRECEITA = widget.currentRow.CDRECEITA;
                data.push(modifiedinRows);
                if (modifiedinRows.IDATIVO && widget.currentRow.IDATIVO == "Ativa") {
                    this.validaAtivacaoReceita(modifiedinRows).then(function(callback) {
                        var valida = callback.dataset.composicao_mododepreparo;
                        var modpreprecm = valida[0].QUANTIDADE;
                        var proreceitam = valida[1].QUANTIDADE;
                        var subreceitam = valida[2].QUANTIDADE;

                        if (widget.currentRow.IDATIVO == "Inativa") {
                            valida_modpreprecm = true;
                            valida_composicao = true;
                        } else {
                            valida_modpreprecm = modpreprecm;
                            if (proreceitam || subreceitam) {
                                valida_composicao = true;
                            }
                        }

                        if (valida_composicao && valida_modpreprecm) {
                            self.solicitaSalvaEditReceita(widget, data, possuiWflow, row, process);
                        } else if (!valida_modpreprecm) {
                            widget.currentRow = angular.copy(widget.backupRow);
                            MessageService.showMessageByCode("COC01400_RECIPE_NOT_PREPARE_METHOD");
                        } else if (!valida_composicao) {
                            widget.currentRow = angular.copy(widget.backupRow);
                            MessageService.showMessageByCode("COC01400_RECIPE_NO_COMPOSITION");
                        }
                    }, function(error) {
                        var errorMessage = "Timeout";
                        if (error.data)
                            errorMessage = error.data.error;
                        ScreenService.showMessage(errorMessage);
                    });
                } else {
                    if (changeSituation(modifiedinRows)) {
                        if (widget.currentRow.NROPCOES < 1 && (widget.currentRow.IDRECFLEXIVEL == "M" || widget.currentRow.IDRECFLEXIVEL == "P")) {
                            MessageService.showMessageByCode("COC01400_NUMBER_OPTIONS");
                        } else {
                            self.solicitaSalvaEditReceita(widget, data, possuiWflow, row, process);
                        }
                    } else {
                        widget.currentRow.P_IDVIGENCIA = "T";
                        coc01400_receitaService.validateChangeSituation(widget.currentRow).then(function(callback) {
                            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                                //Lista de erros gerados pela Procedure
                                InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, "Operação bloqueada");
                            } else {
                                //success
                                self.solicitaSalvaEditReceita(widget, data, possuiWflow, row, process);
                            }
                        }, function(error) {
                            InflightorGeneral.callbackError(error);
                        });
                    }
                }
            } else {
                //Não houve alterações na Receita
                widget.view();
                if (process == "edit_receita")
                    self.cancelModEdit(widget.container.getWidget("receita"), widget);
                else if (process == "edit_variacao")
                    self.cancelModEdit(widget.container.getWidget("versao"), widget);
            }
        } // Widget is valid
    };

    function validateToClearCheckedRows(field) {
        var valid = false;
        if (field.dataSource) {
            var checkedRows = field.dataSource.getCheckedRows();
            if (checkedRows && Array.isArray(checkedRows) && checkedRows.length) {
                valid = true;
            }
        }
        return valid;
    }

    this.controlaWorkflow = function(widget, item, process) {
        var widgetAutenticar = widget.container.getWidget("widget_autenticar");
        widget.currentRow.DSMPREPRECHTML = FieldUtil.changeTagRichText(widget.currentRow.DSMPREPRECHTML);
        if (process == "delete_receita") {
            ScreenService.confirmMessage("Confirmar a exclusão da Receita?",
                "question",
                function() {
                    ScreenService.openPopup(widgetAutenticar).then(function() {
                        widgetAutenticar.fields.forEach(function(field) {
                            if (field.defaultValue) {
                                field.applyDefaultValue();
                            } else {
                                field.setValue("");
                            }
                            if (validateToClearCheckedRows(field)) {
                                field.dataSource.unSelectAll();
                            }
                        });
                        widgetAutenticar.activate();
                    });
                });
        } else {
            validaProcessoWorkflow("1", item).then(function() {
                self.solicitarAprovacaoReceita(widget, process);
            }, function() {
                //Não passa por processo de workflow
                switch (process) {
                    case "delete_variacao":
                        coc01400_receitaController.removeVariacao(widget);
                        break;
                    case "new_produto":
                        coc01400_receitaController.addNovaCompProduto(widget);
                        break;
                    case "new_subreceita":
                        coc01400_receitaController.addNovaCompReceita(widget);
                        break;
                    case "delete_composicao":
                        coc01400_receitaController.directRemove(widget);
                        break;
                    case "new_modpreprecm":
                        coc01400_receitaController.salvaEditModopreprecVariacao(widget);
                        break;
                    case "delete_modprerpecm":
                        coc01400_receitaController.deleteModpreprecm(widget);
                        break;
                    case "new_unidade":
                        coc01400_receitaController.salvaUnidade(widget);
                        break;
                    case "delete_unidade":
                        InflightorGeneral.deleteCrudCheckedRows(widget);
                        break;
                }
            });
        }
    };

    this.confirmaAutenticacao = function(widget) {
        if (widget.isValid()) {

            autenticacaoUsuario(widget).then(function() {
                coc01400_receitaController.removeReceita(widget.container.getWidget("detalheReceita"));
            }).catch(function(callback) {
                InflightorGeneral.callbackError(callback);
            });

        }
    };

    this.fechaAuthPopup = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget("detalheReceita").activate();
    };

    function autenticacaoUsuario(widget) {
        var deferred = ZHPromise.defer();
        if (widget.isValid()) {
            var nomeUsuario = widget.currentRow.usuario;
            var senha = widget.currentRow.senha;

            restEngine.request("ate#/autenticarPorAd", {
                "requestType": "FilterData",
                "filter": [
                    {
                        name: "user",
                        operator: "=",
                        value: nomeUsuario
                    },
                    {
                        name: "senha",
                        operator: "=",
                        value: senha
                    }
                ]
            }).then(function() {
                deferred.resolve();
            }).catch(function(error) {
                deferred.reject(error.data.error);
            });
        }
        return deferred.promise;
    }

    this.controlaEdicao = function(widget) {
        validaProcessoWorkflow("1", "2").then(function() {
            self.solicitarAprovacaoReceita(widget, "delete_receita");
        }, function() {
            //Não passa por processo de workflow
            coc01400_receitaController.removeReceita(widget);
        });
    };

    this.validaEdicao = function(widget) {
        permiteEdicao(widget, "RECEITAM").then(function(callback) {
            if (callback) {
                self.modEdit(widget.container.getWidget("receita"), widget);
                coc01400_receitaService.getDadoswFlow(widget.currentRow.CDRECEITA, "RECEITAM").then(function(callback) {
                    var processos = callback.dataset.workflow_dadoswflow;
                    if (processos.length) {
                        processos.forEach(function(field) {
                            if (widget.getField(field.NMCAMPOWFLOW) && field.IDSITWORKFLOW == 1)
                                widget.getField(field.NMCAMPOWFLOW).readOnly = true;
                            else if (widget.getField(field.NMCAMPOWFLOW).name != "IDRECFLEXIVEL" || widget.getField(field.NMCAMPOWFLOW).name != "IDANALIZEATIV" || widget.getField(field.NMCAMPOWFLOW).name != "CDRECEITA")
                                widget.getField(field.NMCAMPOWFLOW).readOnly = false;
                        });
                    } else {
                        widget.fields.forEach(function(field) {
                            if (field.name == "IDRECFLEXIVEL" || field.name == "IDANALIZEATIV" || field.name == "CDRECEITA" || field.name == "CODRECEITA")
                                field.readOnly = true;
                            else
                                field.readOnly = false;
                        });
                    }
                });
            } else {
                MessageService.showMessageByCode("COC01400_RECIPE_DELETE_PROCESS");
            }
        });
    };

    function permiteEdicao(widget, tabela) {
        return coc01400_receitaService.getDadoswFlow(widget.currentRow.CDRECEITA, tabela).then(function(callback) {
            var validate = true;
            var processos = callback.dataset.workflow_dadoswflow;
            if (processos.length) {
                processos.forEach(function(field) {
                    if ((field.NMCAMPOWFLOW == "CDRECEITA" || field.NMCAMPOWFLOW == "NRVARIACAOREC") && field.IDSITWORKFLOW == 1)
                        validate = false;
                });
            }
            return validate;
        });
    }

    this.modEdit = function(widgetParent, widget) {
        InflightorGeneral.edit(widget);
        if (widget.name == "detalheReceita")
            coc01400_receitaController.onChangeExclusiv(widget.getField("ESCOLHECIAOUGRUPO"));
        if (widgetParent)
            self.setVisibleforTabbed(widgetParent.widgets, false, widget);
        widget.template = widget.formTemplate = widget.templateToForm;
    };

    this.cancelModEdit = function(widgetParent, widget) {
        if (widget.backupRow) {
            widget.currentRow = angular.copy(widget.backupRow);
            delete widget.backupRow;
        }
        widget.view();
        widget.activate();
        if (widgetParent) {
            self.setVisibleforTabbed(widgetParent.widgets, true);
        }
        widget.template = widget.formTemplate = widget.templateToView;
        if (widget.name == "detalheReceita")
            coc01400_receitaController.onChangeExclusiv(widget.getField("ESCOLHECIAOUGRUPO"));
    };

    this.setVisibleforTabbed = function(widgets, value, exception) {
        widgets.forEach(function(widget) {
            if (exception) {
                if (exception.name != widget.name)
                    widget.isVisible = value;
            } else {
                widget.isVisible = value;
            }
        });
    };

    this.validaEdicaoVariacao = function(widget) {
        permiteEdicao(widget, "VARIACAOREC").then(function(callback) {
            if (callback) {

                verificaReceitaFlex(widget);

                self.modEdit(widget.container.getWidget("versao"), widget);
                coc01400_receitaService.getDadoswFlow(widget.currentRow.CDRECEITA, "VARIACAOREC").then(function(callback) {
                    var processos = callback.dataset.workflow_dadoswflow;
                    if (processos.length) {
                        processos.forEach(function(field) {
                            if (widget.getField(field.NMCAMPOWFLOW) && field.IDSITWORKFLOW == 1)
                                widget.getField(field.NMCAMPOWFLOW).readOnly = true;
                            else if (widget.getField(field.NMCAMPOWFLOW).name != "CDOPERINCLUSAO" || widget.getField(field.NMCAMPOWFLOW).name != "DTINCLUSAO" || widget.getField(field.NMCAMPOWFLOW).name != "IDANALIZEATIV" || widget.getField(field.NMCAMPOWFLOW).name != "CDOPERULTATU" || widget.getField(field.NMCAMPOWFLOW).name != "DTULTATU")
                                widget.getField(field.NMCAMPOWFLOW).readOnly = false;
                        });
                    } else {
                        widget.fields.forEach(function(field) {
                            if (widget.getField(field.NMCAMPOWFLOW)) {
                                if (widget.getField(field.NMCAMPOWFLOW).name != "CDOPERINCLUSAO" || widget.getField(field.NMCAMPOWFLOW).name != "DTINCLUSAO" || widget.getField(field.NMCAMPOWFLOW).name != "IDANALIZEATIV" || widget.getField(field.NMCAMPOWFLOW).name != "CDOPERULTATU" || widget.getField(field.NMCAMPOWFLOW).name != "DTULTATU")
                                    field.readOnly = true;
                                else
                                    field.readOnly = false;
                            }
                        });
                    }
                });
            } else {
                MessageService.showMessageByCode("COC01400_RECIPE_DELETE_PROCESS");
            }
        });
    };

    this.detalheVersao = function(swipeWidget) {
        ScreenService.openSwipe(swipeWidget);
    };

    this.onEnterDetalhesVariacaoRec = function(widget) {
        if (!widget.__is_new && !widget.backupRow) {
            widget.currentRow = widget.container.getWidget("variacoesReceita").currentRow;
            converteFieldsReceita(widget.currentRow, "find");
        }
    };

    this.afterMoveRowVariacoes = function(widget) {
        let swipeIndex = ScreenService.getSwipeLevel(widget) + 1;
        let indexToClose = swipeIndex + 1;
        ScreenService.closeSwipe(indexToClose);

        if (!widget.container.getWidget("versao").hasActive()) {
            ScreenService.openSwipe(widget.container.getWidget("versao")).then(function() {
                if (widget.currentRow.IDATIVO == "N" && widget.currentRow.IDANALIZEATIV == "S") {
                    verificaReceitaFlex(widget.container.getWidget("detalhesvariacao"));
                    widget.container.getWidget("detalhesvariacao").getAction("WORKFLOW").isVisible = true;
                } else {
                    verificaReceitaFlex(widget.container.getWidget("detalhesvariacao"));
                    widget.container.getWidget("detalhesvariacao").getAction("WORKFLOW").isVisible = false;
                }
                converteFieldsReceita(widget.currentRow, "find");
            });
        } else {
            if (!widget.__is_new) {
                var widgetCurrent = widget.container.getWidget(widget.container.getWidget("versao").currentWidget);
                if (widgetCurrent) {
                    if (widgetCurrent.name == "detalhesvariacao") {
                        verificaReceitaFlex(widgetCurrent);
                        self.onEnterDetalhesVariacaoRec(widgetCurrent);
                    } else if (widgetCurrent.name == "composicaoprodutos" || widgetCurrent.name == "mododepreparo" || widgetCurrent.name == "unidade" || widgetCurrent.name == "testeReceita")
                        self.onEnterDadosVersao(widgetCurrent);
                    else
                        throw "widget is undefined";
                }
            }
        }
    };

    this.directRemove = function(widget) {
        var validate = true;
        var currentRowReceita = widget.container.getWidget("detalheReceita").currentRow;
        var widgetComposicao = widget.container.getWidget("composicaoprodutos");
        coc01400_receitaService.getDadoswFlow(currentRowReceita.CDRECEITA, "RECEITAM").then(function(callback) {
            var workflow = callback.dataset.workflow_dadoswflow;
            if (workflow.length) {
                workflow.forEach(function(row) {
                    if ((row.NRITEMPROCAPROVA == 1 || row.NRITEMPROCAPROVA == 2) && row.IDSITWORKFLOW == 1) {
                        MessageService.showMessageByCode("COC01400_RECIPE_WORKFLOW_PROCESS");
                        validate = false;
                    }
                });
            }
            if (validate) {
                widgetComposicao.getCheckedRows().forEach(function(row) {
                    var identificador = row.ID;
                    widgetComposicao.currentRow = row;
                    if (identificador == "S") {
                        return self.removeRecCompVariacao(widgetComposicao).then(function() {},
                            function(error) {
                                var errorMessage = "Timeout";
                                if (error.data) {
                                    errorMessage = error.data.error;
                                }
                                ScreenService.showMessage(errorMessage);
                            });
                    } else if (identificador == "P") {
                        return self.excluiProdCompVariacao(widgetComposicao).then(function() {},
                            function(error) {
                                var errorMessage = "Timeout";
                                if (error.data) {
                                    errorMessage = error.data.error;
                                }
                                ScreenService.showMessage(errorMessage);
                            });
                    }
                });
                widgetComposicao.activate();
            }
        });
    };

    this.onEnterDadosVersao = function(widget) {
        var row = widget.container.getWidget("variacoesReceita").currentRow;
        var widgetReceita = widget.container.getWidget("receitas");
        cdreceita = row.CDRECEITA;
        nrvariacaorec = row.NRVARIACAOREC;
        var controle = false;
        var filtro;
        if (!widget.__is_new) {
            if (widget.name == "composicaoprodutos" || widget.name == "mododepreparo") {
                if (widget.name == "mododepreparo") {
                    filtro = [
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": cdreceita
                        },
                        {
                            "name": "NRVARIACAOREC",
                            "operator": "=",
                            "value": nrvariacaorec
                        },
                        {
                            "name": "NRORG",
                            "operator": "=",
                            "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                        }
                    ];
                    widget.dataSourceFilter = filtro;
                    widget.reload().then(function() {
                        if (widget.dataSource.data.length)
                            if (widget.dataSource.data[0].DSMPREPRECHTML) {
                                widget.currentRow.DSMPREPRECHTML = widget.dataSource.data[0].DSMPREPRECHTML;
                                widget.getField("DSMPREPRECHTML").setValue(widget.dataSource.data[0].DSMPREPRECHTML);
                                controle = true;
                                widget.getField("DSMPREPRECHTML").reload();
                            }
                        if (!controle) {
                            widget.currentRow.DSMPREPRECHTML = "";
                            widget.getField("DSMPREPRECHTML").setValue("");
                            widget.getField("DSMPREPRECHTML").reload();
                        }
                        widget.activate();
                    });
                } else {
                    if (widgetReceita.currentRow.IDRECFLEXIVEL === "M" || widgetReceita.currentRow.IDRECFLEXIVEL === "P") {
                        widget.getField("IDPREFERENCIA").isVisible = true;
                        widget.getAction("removePreferencia").isVisible = true;
                        widget.getAction("definePreferencia").isVisible = true;
                    } else {
                        widget.getField("IDPREFERENCIA").isVisible = false;
                        widget.getAction("removePreferencia").isVisible = false;
                        widget.getAction("definePreferencia").isVisible = false;
                    }
                    self.setDataSourceForGrid(widget, cdreceita, nrvariacaorec);
                }
            }
            if (widget.name == "unidade" || widget.name == "testeReceita") {
                filtro = [
                    {
                        "name": "CDRECEITA",
                        "operator": "=",
                        "value": cdreceita
                    },
                    {
                        "name": "NRVARIACAOREC",
                        "operator": "=",
                        "value": nrvariacaorec
                    },
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                    }
                ];
                widget.dataSourceFilter = filtro;
                widget.reload().then(function() {
                    widget.activate();
                });
            }
        }
        widget.unSelectAll();
    };

    this.afterLoadDadosVersao = function(widget) {
        widget.activate();
    };

    this.setCurrentForm = function(widget) {
        var row = widget.dataSource.data;
        if (row.length) {
            widget.container.getWidget("mododepreparo").currentRow = row[0];
            widget.getAction("edit").isVisible = true;
            widget.getAction("add").isVisible = false;
            widget.getAction("ZH_TRASH").isVisible = true;
        } else {
            widget.getAction("edit").isVisible = false;
            widget.getAction("add").isVisible = true;
            widget.getAction("ZH_TRASH").isVisible = false;
        }

    };

    this.mappingModifications = function(modified) {
        var data = [];
        var modificationsWorkflow = {
            "DSRECEITA": "3",
            "DSRECES": "4",
            "DSRECIN": "5",
            "SGUNIDADE": "6",
            "IDATIVO": "7",
            "CDSETOR": "9",
            "CDCATEGORIA": "10",
            "CDCIA": "11",
            "NRNIVELDIFICULDADE": "12",
            "IDRECFLEXIVEL": "13",
            "NMVARIACAOREC": "20",
            "CDOPERADOR": "21",
            "QTRENDIMENTOREC": "22",
            "NRTEMPOPREPARO": "23"
        };
        Object.keys(modified).forEach(function(position) {
            for (let value in modificationsWorkflow) {
                if (position == value) {
                    var row = {};
                    row[value] = modificationsWorkflow[value];
                    data.push(row);
                }
            }
        });
        return data;
    };

    this.salvaEditReceita = function(widget, args) {
        var row = angular.copy(widget.currentRow);
        var backupRow = angular.copy(widget.backupRow);
        converteFieldsReceita(backupRow, "save");
        if (widget.isValid()) {
            if ((row.CDRECEITA === row.CDRECEITAPAI)) {
                MessageService.showMessageByCode("COC01400_RECIPE_EQUAL_SUB_RECIPE");
            } else {
                data[2].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                data[2].NRPROCESSOAPROVA = "1";
                coc01400_receitaService.solicitaEdicaoWorkFlow(data).then(function() {
                    return MessageService.showMessageByCode("WORKFLOW_SUCCESS").then(function() {
                        if (row.__is_new) {
                            widget.list(true);
                            InflightorGeneral.backOrCancel(args);
                            widget.reload();
                        } else {
                            widget.view();
                        }
                        ScreenService.closePopup();
                        widget.activate();
                        converteFieldsReceita(row, "find");
                        self.cancelModEdit(widget.container.getWidget("receita"), widget);
                        widget.currentRow = widget.backupRow;
                        self.abreDetalheReceita(widget);
                    });
                }, function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    ScreenService.showMessage(errorMessage);
                });
            }
        }
    };

    function compareRows(newRow, backupRow) {
        var row = {};
        if (angular.equals(newRow, backupRow))
            return false;
        else {
            for (let value in newRow) {
                if (newRow[value] != backupRow[value])
                    row[value] = newRow[value];
            }
            return row;
        }
    }

    this.salvarAlteracoesProd = function(widget) {

        var data = widget.dataSource.data;

        coc01400_receitaService.salvarAlteracaoVariacaoCompProd(data).then(function() {

            widget.commitInlineEdit();
            widget.list();
            if (widget.mode == "list") {
                widget.container.getWidget("variacoesReceita").reload();
            }

        }, function(err) {

            if (err.data) {

                ScreenService.showMessage(err.data.error, "error");

            } else {

                MessageService.showMessageByCode("NO_CONNECTION");

            }

        });

    };

    var workflow_composicao = {
        "PRORECEITAM": false,
        "SUBRECEITAM": false
    };

    this.controlaEditComposicao = function(widget) {
        var promises = [];
        promises.push(coc01400_receitaService.getProcessosWorkflow("1", "25"));
        promises.push(coc01400_receitaService.getProcessosWorkflow("1", "26"));
        ZHPromise.all(promises).then(function(callback) {
            if (callback[0].dataset.processos_workflow[0].IDATIVO == "N" &&
                callback[1].dataset.processos_workflow[0].IDATIVO == "N")
                coc01400_receitaController.salvarAlteracoesRec(widget, true);
            else {
                if (callback[0].dataset.processos_workflow[0].IDATIVO == "S" &&
                    callback[1].dataset.processos_workflow[0].IDATIVO == "S") {
                    workflow_composicao.PRORECEITAM = true;
                    workflow_composicao.SUBRECEITAM = true;
                } else if (callback[0].dataset.processos_workflow[0].IDATIVO == "S")
                    workflow_composicao.PRORECEITAM = true;
                else
                    workflow_composicao.SUBRECEITAM = true;
                self.solicitarAprovacaoReceita(widget, "edit_composicao");
            }
        }, function(error) {
            InflightorGeneral.callbackError(error);
        });
    };

    this.salvarAlteracoesRec = function(widget) {

        var widgetComposicao = widget.container.getWidget("composicaoprodutos");
        var data = widgetComposicao.dataSource.data;
        var receitas = [];
        var produtos = [];
        var promises = [];
        var situacao = true;
        for (let i in data) {
            if (data[i].ID == "S" && data[i].changed_QTD) {
                if (data[i].IDANALIZEATIV && data[i].IDANALIZEATIV != "A")
                    situacao = false;
                else {
                    var row = {
                        DSMOTIVOACAO: widget.currentRow.DSMOTIVOACAO,
                        NRITEMPROCAPROVA: "26", //add na tabela
                        NRPROCESSOAPROVA: "1", //fixed
                        IDSITWORKFLOW: "1",
                        DSINFORANTERIOR: data[i].changed_QTD,
                        DSINFORATUAL: InflightorGeneral.valueToFloat(data[i].QTD),
                        NMCAMPOWFLOW: "QTRENDSUBREC",
                        IDTIPOPERWFLOW: 1,
                        VRCUSTOPROREC: null,
                        NMTABELAWFLOW: "SUBRECEITAM",
                        UPDATEINTABLE: "subreceitam",
                        DSRECEITA: data[i].DSRECEITA,
                        CDSUBRECEITA: data[i].CODIGO,
                        CDRECEITA: data[i].CDRECEITA,
                        NRVARIACAOREC: data[i].NRVARIACAOREC,
                        NRORDSUBREC: data[i].ORDEM,
                        DTINCLUSAO: moment().format("DD/MM/YYYY"),
                        VRCUSTOSUBREC: null,
                        __is_new: false
                    };
                    if (workflow_composicao.SUBRECEITAM)
                        promises.push(coc01400_receitaService.solicitaWorkFlowVariacao(row));
                    else
                        promises.push(coc01400_receitaService.updateSubReceita(receitas));
                }
            } else if (data[i].ID == "P" && data[i].changed_QTD) {
                if (data[i].IDANALIZEATIV && data[i].IDANALIZEATIV != "A")
                    situacao = false;
                else {
                    var rowProd = {
                        DSMOTIVOACAO: widget.currentRow.DSMOTIVOACAO,
                        NRITEMPROCAPROVA: "25", //add na tabela
                        NRPROCESSOAPROVA: "1", //fixed
                        IDSITWORKFLOW: "1",
                        DSINFORANTERIOR: data[i].changed_QTD,
                        DSINFORATUAL: InflightorGeneral.valueToFloat(data[i].QTD),
                        NMCAMPOWFLOW: "QTPRODREC",
                        IDTIPOPERWFLOW: 1,
                        VRCUSTOPROREC: null,
                        NMTABELAWFLOW: "PRORECEITAM",
                        UPDATEINTABLE: "proreceitam",
                        DSRECEITA: data[i].DSRECEITA,
                        CDRECEITA: data[i].CDRECEITA,
                        NRVARIACAOREC: data[i].NRVARIACAOREC,
                        NRORDSUBREC: data[i].ORDEM,
                        DTINCLUSAO: moment().format("DD/MM/YYYY"),
                        CDPRODUTO: data[i].CODIGO,
                        NRORDPREC: data[i].ORDEM,
                        SGGRUPORECFLEX: data[i].SGGRUPORECFLEX,
                        __is_new: false
                    };
                    if (workflow_composicao.PRORECEITAM)
                        promises.push(coc01400_receitaService.solicitaWorkFlowVariacao(rowProd));
                    else
                        promises.push(coc01400_receitaService.updateSubProd(produtos));
                }
            }
        }
        ZHPromise.all(promises).then(function() {
            widgetComposicao.commitInlineEdit();
            self.setVisibilityCheckbox(widgetComposicao, true);
            ScreenService.closePopup(true);
            if (!situacao)
                MessageService.showMessageByCode("COC01400_COMPOSITION_WORKFLOW_PROCESS");
            widgetComposicao.list();
            self.setDataSourceForGrid(widgetComposicao, widget.container.getWidget("receitas").currentRow.CDRECEITA, widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC);
        }, function(err) {
            if (err.data) {
                ScreenService.showMessage(err.data.error, "error");
            } else {
                MessageService.showMessageByCode("NO_CONNECTION");
            }

        });

    };

    this.saveRendLiq = function(widget) {
        converteFieldsReceita(widget.currentRow, "save");
        widget.currentRow.QTRENDIMENTOREC = InflightorGeneral.valueToFloat(widget.currentRow.QTRENDIMENTOREC);
        var data = widget.currentRow;
        coc01400_receitaService.salvaEdicaoVariacaoRec(data).then(function() {
            widget.reload();
            ScreenService.closePopup();
            self.onEnterDadosVersao(widget.container.getWidget("composicaoprodutos"));
            widget.container.getWidget("composicaoprodutos").activate();
            widget.backupRow = undefined;
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.cancelaAlteracoes = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        self.setVisibilityCheckbox(widget, true);
    };

    this.addClassificacao = function(widget) {
        ScreenService.openPopup(widget);
        widget.activate();
        widget.reload();
    };

    this.deleteClassificacao = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            widget.currentRow = widget.getCheckedRows();
            coc01400_receitaService.deleteClassificacao(widget.currentRow).then(function() {
                widget.unSelectAll();
                widget.container.getWidget("classificacao").reload();
                widget.container.getWidget("classificacao").activate();
                widget.container.getWidget("detalheReceita").reload();
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                self.abreDetalheReceita(widget, false);
            });
        });
    };

    this.saveClassificacao = function(widget) {
        widget.currentRow = widget.getCheckedRows();
        widget.currentRow.forEach(function(row) {
            row.CDRECEITA = widget.container.getWidget("receitas").currentRow.CDRECEITA;
            row.__is_new = true;
        });
        coc01400_receitaService.saveClassificacao(widget.currentRow).then(function() {
            ScreenService.closePopup(true);
            widget.unSelectAll();
            widget.container.getWidget("classificacao").reload();
            widget.container.getWidget("classificacao").activate();
            widget.container.getWidget("detalheReceita").reload();
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
        }, InflightorGeneral.callbackError);
    };

    this.addUnidade = function(widget) {

        var dsfiltro = [{
            "name": "CDRECEITA",
            "operator": "=",
            "value": widget.container.getWidget("detalheReceita").currentRow.CDRECEITA
        }];
        ScreenService.openPopup(widget);
        widget.activate();
        widget.dataSourceFilter = dsfiltro;
        widget.reload();
    };

    this.salvaUnidade = function(widget) {

        if (widget.isValid()) {
            widget.currentRow.__is_new = true;
            widget.currentRow.DSMOTIVOACAO = widget.container.getWidget("detalhesvariacao").currentRow.DSMOTIVOACAO;
            widget.currentRow.NRITEMPROCAPROVA = "32";
            widget.currentRow.DSINFORANTERIOR = "N";
            widget.currentRow.DSINFORATUAL = "S";
            widget.currentRow.NMCAMPOWFLOW = "IDATIVO";
            widget.currentRow.IDTIPOPERWFLOW = 2; //INSERT
            widget.currentRow.NRPROCESSOAPROVA = "1"; //fixed
            widget.currentRow.IDANALIZEATIV = "P";
            widget.currentRow.IDSITWORKFLOW = "1";
            widget.currentRow.IDATIVO = "N";
            widget.currentRow.CDRECEITA = widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA;
            widget.currentRow.NRVARIACAOREC = widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC;
            widget.currentRow.NMTABELAWFLOW = "FILIRECEITA";
            widget.currentRow.UPDATEINTABLE = "filireceita";
            widget.currentRow.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;

            coc01400_receitaService.getDadoswFlow("CDRECEITA=" + widget.currentRow.CDRECEITA + ";NRORG=" + getLocalVar("INFLIGHTOR_NRORGTRAB") + ";NRVARIACAOREC=" + widget.currentRow.NRVARIACAOREC, "FILIRECEITA").then(function(callback) {
                var processos = callback.dataset.workflow_dadoswflow;
                var validate = true;
                if (processos.length) {
                    if (!validaExclusao(processos)) {
                        ScreenService.closePopup(true);
                        MessageService.showMessageByCode("COC01400_UNIT_WORKFLOW_PROCESS");
                        validate = false;
                        widget.container.getWidget("unidade").reload().then(function() {
                            widget.container.getWidget("unidade").activate();
                        });
                    }
                }
                if (validate) {
                    coc01400_receitaService.solicitaWorkFlowVariacao(widget.currentRow).then(function() {
                        widget.container.getWidget("unidade").reload().then(function() {
                            ScreenService.closePopup(true);
                            widget.container.getWidget("unidade").activate();
                        });
                    }, function(callback) {
                        InflightorGeneral.callbackError(callback);
                    });
                }
            });
        }
    };

    this.setUnMedida = function(widget) {
        var data = widget.dataSource.data;
        data.forEach(function(row) {
            row.SGUNIDADE = widget.container.getWidget("detalheReceita").currentRow.SGUNIDADE;
        });
    };

    this.addCompReceita = function(widget) {
        validaEIncluiComposicao(widget, "S");
    };

    this.addNovaCompReceita = function(widget) {

        if (widget.isValid()) {
            var novasReceitas = widget.currentRow;
            if (novasReceitas.IDATIVO == "N") {
                return MessageService.showMessageByCode("INACTIVE_RECIPE");
            } else {
                //Valida se a receita possui composição
                InflightorGeneral.callRestEngine("coc#/crud/coc01400_receitas_composicao", "FilterData", coc01400_receitaController.getDataSourceFilterComposicao(widget)).then(
                    function(callback) {
                        if (callback && callback.dataset.coc01400_receitas_composicao && callback.dataset.coc01400_receitas_composicao.length) {
                            var rowSubreceita = {
                                "CDSUBRECEITA": "",
                                "CDRECEITA": "",
                                "NRVARIACAOREC": "",
                                "QTRENDSUBREC": "",
                                "NRORG": ""
                            };

                            var rowVariacaoReceita = widget.container.getWidget("variacoesReceita").currentRow;
                            var rowReceita = widget.container.getWidget("receitas").currentRow;
                            for (let coluna in novasReceitas) {
                                if (rowSubreceita.hasOwnProperty(coluna))
                                    rowSubreceita[coluna] = novasReceitas[coluna];
                                if (coluna === "CDRECEITA")
                                    rowSubreceita.CDSUBRECEITA = novasReceitas[coluna];
                            }
                            rowSubreceita.QTRENDSUBREC = InflightorGeneral.valueToFloat(rowSubreceita.QTRENDSUBREC);
                            rowSubreceita.NRORDSUBREC = 1;

                            for (let coluna in rowVariacaoReceita) {
                                if (rowSubreceita.hasOwnProperty(coluna))
                                    rowSubreceita[coluna] = rowVariacaoReceita[coluna];
                            }

                            var rowToSave = {};
                            for (let propriedade in rowSubreceita) {
                                rowToSave[propriedade] = rowSubreceita[propriedade];
                            }

                            var arrayPrimaryKeys = {
                                "CDRECEITA": rowReceita.CDRECEITA,
                                "NRVARIACAOREC": rowVariacaoReceita.NRVARIACAOREC,
                                "CDSUBRECEITA": rowSubreceita.CDSUBRECEITA
                            };

                            rowSubreceita.IDANALIZEATIV = "P";
                            rowToSave.PRIMARYKEYS = arrayPrimaryKeys;
                            rowToSave.DSMOTIVOACAO = rowVariacaoReceita.DSMOTIVOACAO;
                            rowToSave.NMTABELA = "SUBRECEITAM";
                            rowToSave.IDTIPOPERWFLOW = "2";
                            rowToSave.NRPROCESSOAPROVA = "1";
                            rowToSave.NRITEMPROCAPROVA = "17";
                            rowToSave.CHANGEDITEMS = rowSubreceita;
                            rowToSave.NMITEM = rowReceita.DSRECEITA;
                            rowToSave.ID = "S";

                            coc01400_receitaService.solicitaWorkFlowComposicao(rowToSave).then(function() {
                                ScreenService.showMessage(callback.messages[0].message);
                                ScreenService.closePopup();
                                ScreenService.closePopup();
                            });
                        } else {
                            ScreenService.showMessage(ScreenService.i18n("Impossível adicionar uma receita sem composição!"), "error");
                        }
                    }, InflightorGeneral.callbackError);
            }
        }

    };

    this.removeRecCompVariacao = function(widget) {

        var deferred = ZHPromise.defer();

        var __rightActions = widget.currentRow.__rightActions;
        var __leftActions = widget.currentRow.__leftActions;
        delete widget.currentRow.__leftActions;
        delete widget.currentRow.__rightActions;
        var currentRow = angular.copy(widget.currentRow);
        widget.currentRow.__rightActions = __rightActions;
        widget.currentRow.__leftActions = __leftActions;
        currentRow.DSMOTIVOACAO = widget.container.getWidget("detalhesvariacao").currentRow.DSMOTIVOACAO;
        currentRow.NRITEMPROCAPROVA = "27"; //add na tabela
        currentRow.NRPROCESSOAPROVA = "1"; //fixed
        currentRow.IDSITWORKFLOW = "1";
        currentRow.DSINFORANTERIOR = currentRow.CODIGO;
        currentRow.DSINFORATUAL = currentRow.CODIGO;
        currentRow.CDSUBRECEITA = currentRow.CODIGO;
        currentRow.NMCAMPOWFLOW = "CDSUBRECEITA";
        currentRow.IDTIPOPERWFLOW = 3;
        currentRow.CDRECEITA = widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA;
        currentRow.NRVARIACAOREC = widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC;
        currentRow.NMTABELAWFLOW = "SUBRECEITAM";
        currentRow.__is_new = false;
        currentRow.UPDATEINTABLE = "subreceitam";
        currentRow.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;

        coc01400_receitaService.getDadoswFlow("CDSUBRECEITA=" + currentRow.CDSUBRECEITA, "SUBRECEITAM").then(function(callback) {
            var processos = callback.dataset.workflow_dadoswflow;
            if (processos.length) {
                if (!validaExclusao(processos)) {
                    ScreenService.closePopup();
                    MessageService.showMessageByCode("COC01400_COMPOSITION_WORKFLOW_PROCESS_ERROR");
                } else {
                    converteFieldsReceita(currentRow, "save");
                    coc01400_receitaService.solicitaWorkFlowVariacao(currentRow).then(function() {
                        widget.container.getWidget("composicaoprodutos").reload().then(function() {
                            self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), cdreceita, nrvariacaorec);
                            ScreenService.closePopup(true);
                            widget.container.getWidget("composicaoprodutos").activate();
                            deferred.resolve();
                        });
                    }, function(err) {
                        errorValidations(err.data.error);
                    });
                }
            } else {
                coc01400_receitaService.solicitaWorkFlowVariacao(currentRow).then(function() {
                    widget.container.getWidget("composicaoprodutos").reload().then(function() {
                        self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), cdreceita, nrvariacaorec);
                        ScreenService.closePopup(true);
                        widget.container.getWidget("composicaoprodutos").activate();
                        deferred.resolve();
                    });
                }, function(err) {
                    errorValidations(err.data.error);
                });
            }
        });
        deferred.reject();
    };

    this.adicionaVariacao = function(widget) {

        var row = widget.currentRow;
        if (widget.isValid()) {
            widget.currentRow.QTRENDIMENTOREC = widget.currentRow.QTRENDIMENTOREC ? widget.currentRow.QTRENDIMENTOREC : 0;
            validaProcessoWorkflow("1", "19").then(function() {
                row.IDATIVO = "N";
                row.IDANALIZEATIV = "S";
                row.NMVARIACAOREC = " ";
                if (widget.currentRow.NROPCOES < 1 && (widget.currentRow.IDRECFLEXIVEL == "M" || widget.currentRow.IDRECFLEXIVEL == "P")) {
                    MessageService.showMessageByCode("COC01400_NUMBER_OPTIONS");
                } else {
                    coc01400_receitaService.novaVariacao(cdreceita, row).then(function() {
                        widget.container.getWidget("variacoesReceita").reload().then(function() {
                            widget.container.getWidget("variacoesReceita").activate();
                            ScreenService.closeSwipe();
                        });
                    });
                }
            }, function() {
                //Não passa por processo de workflow
                row.NMVARIACAOREC = " ";
                if (widget.currentRow.NROPCOES < 1 && (widget.currentRow.IDRECFLEXIVEL == "M" || widget.currentRow.IDRECFLEXIVEL == "P")) {
                    MessageService.showMessageByCode("COC01400_NUMBER_OPTIONS");
                } else {
                    coc01400_receitaService.novaVariacao(cdreceita, row).then(function() {
                        widget.container.getWidget("variacoesReceita").reload().then(function() {
                            widget.container.getWidget("variacoesReceita").activate();
                            ScreenService.closeSwipe();
                            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                            self.abreDetalheReceita(widget, false);
                        });
                    });
                }
            });
        }
    };

    this.validateDeleteVariacao = function(widget) {
        var promises = [];
        promises.push(coc01400_receitaService.returnProduto(widget.currentRow.CDRECEITA, widget.currentRow.NRVARIACAOREC));
        promises.push(coc01400_receitaService.returnSubReceita(widget.currentRow.CDRECEITA, widget.currentRow.NRVARIACAOREC));
        var validate = true;
        return ZHPromise.all(promises).then(function(callback) {
            let produtos = callback[0].dataset.proreceitam_produto;
            let subreceitas = callback[1].dataset.subreceitam_receita;
            if (produtos.length || subreceitas.length)
                validate = false;
            return validate;
        }.bind(this), function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    this.removeVariacao = function(widget) {

        var variacaoRec = widget.currentRow;
        var data = [];
        variacaoRec.NRITEMPROCAPROVA = "24";
        variacaoRec.NRPROCESSOAPROVA = "1";
        variacaoRec.IDSITWORKFLOW = "1";
        variacaoRec.IDTIPOPERWFLOW = 3;
        data.push({
            CDRECEITA: variacaoRec.CDRECEITA,
            NRVARIACAOREC: variacaoRec.NRVARIACAOREC,
            NRORG: variacaoRec.NRORG,
            __is_new: false
        });

        coc01400_receitaService.getDadoswFlow(variacaoRec.NRVARIACAOREC, "VARIACAOREC").then(function(callback) {
            var processos = callback.dataset.workflow_dadoswflow;

            if (processos.length) {
                if (!validaExclusao(processos)) {
                    ScreenService.closePopup();
                    MessageService.showMessageByCode("COC01400_VARIATION_WORKFLOW_PROCESS");
                } else {
                    converteFieldsReceita(variacaoRec, "save");
                    coc01400_receitaService.solicitaWorkFlowVariacao(variacaoRec).then(function() {
                        widget.container.getWidget("variacoesReceita").reload().then(function() {
                            ScreenService.closePopup();
                            widget.container.getWidget("variacoesReceita").activate();
                        });
                    }, function(erro) {
                        if (erro.data && erro.data.error.indexOf("ORA-02292") !== -1) {
                            MessageService.showMessageByCode("COC01400_DEL_NOT_PERMISSION");
                        } else {
                            ScreenService.showMessage(erro.data.error);
                        }

                    });
                }
            } else {
                converteFieldsReceita(variacaoRec, "save");
                coc01400_receitaService.solicitaWorkFlowVariacao(variacaoRec).then(function() {
                    widget.container.getWidget("variacoesReceita").reload().then(function() {
                        ScreenService.closePopup();
                        widget.container.getWidget("variacoesReceita").activate();
                    });
                }, function(erro) {
                    if (erro.data && erro.data.error.indexOf("ORA-02292") !== -1) {
                        MessageService.showMessageByCode("COC01400_DEL_NOT_PERMISSION");
                    } else {
                        ScreenService.showMessage(erro.data.error);
                    }

                });
            }
        });

    };

    this.salvaEditVariacaoReceita = function(widget) {
        var row = angular.copy(widget.currentRow);
        var backupRow = angular.copy(widget.backupRow);
        converteFieldsReceita(backupRow, "save");
        data[0].DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
        if (data[2].QTRENDIMENTOREC)
            data[2].QTRENDIMENTOREC = InflightorGeneral.valueToFloat(data[2].QTRENDIMENTOREC);
        data[2].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        data[2].NRPROCESSOAPROVA = "1";
        coc01400_receitaService.solicitaEdicaoWorkFlowVariacao(data).then(function() {
            return MessageService.showMessageByCode("WORKFLOW_SUCCESS").then(function() {
                widget.view();
                ScreenService.closePopup();
                converteFieldsReceita(row, "find");
                self.cancelModEdit(widget.container.getWidget("versao"), widget);
                self.loadVersoes(widget);
                widget.container.getWidget("versao").currentWidget = "detalhesvariacao";
                widget.container.getWidget("detalhesvariacao").reload();
                widget.container.getWidget("detalhesvariacao").activate();
            });
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });

    };

    this.salvaEditVariacaoReceitaComp = function(widget) {
        var row = angular.copy(widget.currentRow);
        var backupRow = angular.copy(widget.backupRow);
        converteFieldsReceita(backupRow, "save");
        data[0].DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
        if (data[2].QTRENDIMENTOREC)
            data[2].QTRENDIMENTOREC = InflightorGeneral.valueToFloat(data[2].QTRENDIMENTOREC);
        data[2].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        data[2].NRPROCESSOAPROVA = "1";
        coc01400_receitaService.solicitaEdicaoWorkFlowVariacao(data).then(function() {
            return MessageService.showMessageByCode("WORKFLOW_SUCCESS").then(function() {
                widget.currentRow = backupRow;
                widget.container.getWidget("variacoesReceita").currentRow.QTRENDIMENTOREC = backupRow.QTRENDIMENTOREC;
                widget.container.getWidget("variacoesReceita").reload().then(function() {
                    widget.reload().then(function() {
                        ScreenService.closePopup(true);
                        self.onEnterDadosVersao(widget.container.getWidget("composicaoprodutos"));
                        widget.container.getWidget("composicaoprodutos").activate();
                        converteFieldsReceita(row, "find");
                    });
                });
            });
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });

    };

    var errorValidations = function(error) {

        /*
to do: faltam cercar alguns erros que podem surgir caso a variação seja fk de alguma outra tabela.
*/

        var errors = {
            FKRELATIONVARIACAO: {
                desc: "Unidade",
                message: "Não é possivel excluir uma variação vinculada a uma unidade."
            },
            FKRECSUB2VARIACAO: {
                desc: "receitas",
                message: "Não é possivel excluir a variação. Variação possui receitas em sua composição."
            },
            FKVARIPRORVARIACAO: {
                desc: "produtos",
                message: "Não é possivel excluir a variação. Variação possui produtos em sua composição."
            },
            FKRELATIONRECEITAM: {
                desc: "variacoes",
                message: "Não é possivel excluir a receita. Receita possui variações vinculadas."
            }
        };

        if ((error.search("FKRELATIONVARIACAO")) > 0) {

            ScreenService.showMessage(errors.FKRELATIONVARIACAO.message);
            return null;

        } else if ((error.search("FKRECSUB2VARIACAO")) > 0) {

            ScreenService.showMessage(errors.FKRECSUB2VARIACAO.message);
            return null;

        } else if ((error.search("FKVARIPRORVARIACAO")) > 0) {

            ScreenService.showMessage(errors.FKVARIPRORVARIACAO.message);
            return null;

        } else if ((error.search("FKRELATIONRECEITAM")) > 0) {

            ScreenService.showMessage(errors.FKRELATIONRECEITAM.message);
            return null;

        } else {

            ScreenService.showMessage(error);
            return null;

        }

    };

    this.salvaNovaReceita = function(widget) {

        var row = widget.currentRow;

        if (row.IDRECFLEXIVEL == "M" || row.IDRECFLEXIVEL == "P") {
            if (row.DSRECEITA.lastIndexOf("- FLEX") !== row.DSRECEITA.length - 6) {
                row.DSRECEITA += " - FLEX";
            }
            if (row.DSRECIN.lastIndexOf("- FLEX") !== row.DSRECIN.length - 6) {
                row.DSRECIN += " - FLEX";
            }
            if (row.DSRECES.lastIndexOf("- FLEX") !== row.DSRECES.length - 6) {
                row.DSRECES += " - FLEX";
            }
        }

        if (widget.isValid()) {
            row.__is_new = true;
            converteFieldsReceita(row, "save");
            validaProcessoWorkflow("1", "1").then(function() {
                row.IDANALIZEATIV = "S";
                row.IDATIVO = "N";
                coc01400_receitaService.novaReceita(row).then(function(callback) {
                    ScreenService.closeSwipe().then(
                        function() {
                            MessageService.showMessageByCode("COC01400_RECIPE_WAITING_APPROVAL");
                            widget.container.getWidget("receitas").reload();
                            widget.currentRow.CDRECEITA = callback.dataset.columnsToReturn[0].CDRECEITA;
                            self.abreDetalheReceita(widget);
                        }
                    );
                });
            }, function() {
                //Não passa por processo de workflow
                coc01400_receitaService.novaReceita(row).then(function(callback) {
                    ScreenService.closeSwipe().then(
                        function() {
                            widget.container.getWidget("receitas").reload();
                            widget.currentRow.CDRECEITA = callback.dataset.columnsToReturn[0].CDRECEITA;
                            self.abreDetalheReceita(widget);
                        }
                    );
                });
            });
        }
    };

    function validaProcessoWorkflow(processo, item) {
        var deferred = ZHPromise.defer();
        coc01400_receitaService.getProcessosWorkflow(processo, item).then(function(callback) {
            var processos = callback.dataset.processos_workflow;
            if (processos.length) {
                if (processos[0].IDATIVO == "S")
                    deferred.resolve();
                else
                    deferred.reject();
            } else
                deferred.reject();
        });
        return deferred.promise;
    }

    function validaExclusao(processos) {
        var validate = true;
        processos.forEach(function(processowflow) {
            if (processowflow.IDSITWORKFLOW == 1)
                validate = false;
        });
        return validate;
    }

    this.removeReceita = function(widget) {

        var currentRow = widget.currentRow;
        currentRow.NRITEMPROCAPROVA = "2";
        currentRow.IDSITWORKFLOW = "1";
        currentRow.IDTIPOPERWFLOW = 3;
        widget.currentRow.P_IDVIGENCIA = "T";
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            coc01400_receitaService.getDadoswFlow(currentRow.CDRECEITA, "RECEITAM").then(function(callback) {
                var processos = callback.dataset.workflow_dadoswflow;
                converteFieldsReceita(currentRow, "save");
                if (processos.length) {
                    if (!validaExclusao(processos)) {
                        ScreenService.closePopup();
                        MessageService.showMessageByCode("COC01400_RECIPE_WORKFLOW_PROCESS");
                    } else {
                        coc01400_receitaService.validateChangeSituation(widget.currentRow).then(function(callback) {
                            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                                InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, "Operação bloqueada");
                            } else {
                                coc01400_receitaService.excluiReceita(currentRow).then(function() {
                                    widget.container.getWidget("receitas").reload().then(function() {
                                        ScreenService.closePopup();
                                        widget.container.getWidget("receitas").activate();
                                    });
                                }, function(err) {
                                    if (err.data.dataset.vinculos) {
                                        var dataset_error = [];
                                        var dataset = err.data.dataset.vinculos;
                                        if (dataset.FLUXPRODREC[0].EXISTE || dataset.PADRAOPRODREC[0].EXISTE) {
                                            dataset_error.push("Receita utilizada na Parametrização da Produção.");
                                        }
                                        if (dataset.VARIACAOREC[0].EXISTE) {
                                            dataset_error.push("Receita possui Variações associadas.");
                                        }
                                        if (dataset.CATEGORIAREC[0].EXISTE) {
                                            dataset_error.push("Receita possui Classificações associadas.");
                                        }
                                        InflightorGeneral.showErrorMessages(widget, dataset_error, "Vínculos encontrados");
                                    } else
                                        errorValidations(err.data.error);
                                });
                            }
                        }, function(error) {
                            InflightorGeneral.callbackError(error);
                        });
                    }
                } else {
                    coc01400_receitaService.validateChangeSituation(widget.currentRow).then(function(callback) {
                        if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                            //Lista de erros gerados pela Procedure
                            InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, "Operação bloqueada");
                        } else {
                            //success   	
                            coc01400_receitaService.excluiReceita(currentRow).then(function() {
                                widget.container.getWidget("receitas").reload().then(function() {
                                    ScreenService.closePopup();
                                    widget.container.getWidget("detalheReceita").activate();
                                    converteFieldsReceita(currentRow, "find");
                                    self.cancelModEdit(widget.container.getWidget("receita"), widget);
                                    self.abreDetalheReceita(widget);
                                });
                            }, function(err) {
                                if (err.data.dataset.vinculos) {
                                    var dataset_error = [];
                                    var dataset = err.data.dataset.vinculos;
                                    if (dataset.FLUXPRODREC[0].EXISTE || dataset.PADRAOPRODREC[0].EXISTE) {
                                        dataset_error.push("Receita utilizada na Parametrização da Produção.");
                                    }
                                    if (dataset.VARIACAOREC[0].EXISTE) {
                                        dataset_error.push("Receita possui Variações associadas.");
                                    }
                                    if (dataset.CATEGORIAREC[0].EXISTE) {
                                        dataset_error.push("Receita possui Classificações associadas.");
                                    }
                                    InflightorGeneral.showErrorMessages(widget, dataset_error, "Vínculos encontrados");
                                } else
                                    errorValidations(err.data.error);
                            });
                        }
                    }, function(error) {
                        InflightorGeneral.callbackError(error);
                    });
                }
            });
        });
    };

    this.adicionarCategoria = function(field) {
        ScreenService.openPopup(field.widget.container.getWidget("novaCategoria"));
    };

    this.addNovaCategoria = function(widget) {
        var row = widget.currentRow;
        if (widget.isValid()) {
            coc01400_receitaService.addCategoria(row).then(function() {

                // ScreenService.successNotification(callback.messages[0].message);
                widget.container.getWidget("novaReceita").getField("NMCATEGORIA").reload().then(function() {
                    ScreenService.closePopup();
                });
            });
        }
    };

    this.addProduto = function(widget) {
        validaEIncluiComposicao(widget, "P");
    };

    this.addNovaCompProduto = function(widget) {

        if (widget.isValid()) {
            var novosProdutos = widget.currentRow;
            if (novosProdutos.IDATIVO == "N") {
                return MessageService.showMessageByCode("INACTIVE_PRODUCT");
            } else {

                var rowProduto = {
                    "CDPRODUTO": "",
                    "CDRECEITA": "",
                    "NRVARIACAOREC": "",
                    "QTPRODREC": "",
                    "SGGRUPORECFLEX": "",
                    "NRORG": ""
                };

                var rowVariacaoReceita = widget.container.getWidget("variacoesReceita").currentRow;
                var rowReceita = widget.container.getWidget("receitas").currentRow;
                for (let coluna in novosProdutos) {
                    if (rowProduto.hasOwnProperty(coluna))
                        rowProduto[coluna] = novosProdutos[coluna];
                }
                rowProduto.QTPRODREC = InflightorGeneral.valueToFloat(rowProduto.QTPRODREC);
                rowProduto.NRORDPREC = 1;

                for (let coluna in rowVariacaoReceita) {
                    if (rowProduto.hasOwnProperty(coluna))
                        rowProduto[coluna] = rowVariacaoReceita[coluna];
                }

                var rowToSave = {};
                for (let propriedade in rowProduto) {
                    rowToSave[propriedade] = rowProduto[propriedade];
                }

                var arrayPrimaryKeys = {
                    "CDRECEITA": rowProduto.CDRECEITA,
                    "NRVARIACAOREC": rowVariacaoReceita.NRVARIACAOREC,
                    "CDPRODUTO": rowProduto.CDPRODUTO
                };

                rowProduto.IDANALIZEATIV = "P";
                rowToSave.PRIMARYKEYS = arrayPrimaryKeys;
                rowToSave.DSMOTIVOACAO = rowVariacaoReceita.DSMOTIVOACAO;
                rowToSave.NMTABELA = "PRORECEITAM";
                rowToSave.IDTIPOPERWFLOW = "2";
                rowToSave.NRPROCESSOAPROVA = "1";
                rowToSave.NRITEMPROCAPROVA = "16";
                rowToSave.CHANGEDITEMS = rowProduto;
                rowToSave.NMITEM = rowReceita.DSRECEITA;
                rowToSave.ID = "P";

                coc01400_receitaService.solicitaWorkFlowComposicao(rowToSave).then(function(callback) {
                    ScreenService.showMessage(callback.messages[0].message);
                    ScreenService.closePopup();
                    ScreenService.closePopup();
                });

            }

        }

    };

    this.excluiProdCompVariacao = function(widget) {

        var __rightActions = widget.currentRow.__rightActions;
        var __leftActions = widget.currentRow.__leftActions;
        delete widget.currentRow.__leftActions;
        delete widget.currentRow.__rightActions;
        var currentRow = angular.copy(widget.currentRow);
        widget.currentRow.__rightActions = __rightActions;
        widget.currentRow.__leftActions = __leftActions;
        currentRow.DSMOTIVOACAO = widget.container.getWidget("detalhesvariacao").currentRow.DSMOTIVOACAO;
        currentRow.NRITEMPROCAPROVA = "27"; //add na tabela
        currentRow.NRPROCESSOAPROVA = "1"; //fixed
        currentRow.IDSITWORKFLOW = "1";
        currentRow.DSINFORANTERIOR = currentRow.CODIGO;
        currentRow.DSINFORATUAL = currentRow.CODIGO;
        currentRow.CDPRODUTO = currentRow.CODIGO;
        currentRow.NMCAMPOWFLOW = "CDPRODUTO";
        currentRow.IDTIPOPERWFLOW = 3;
        currentRow.CDRECEITA = widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA;
        currentRow.NRVARIACAOREC = widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC;
        currentRow.NMTABELAWFLOW = "PRORECEITAM";
        currentRow.__is_new = false;
        currentRow.UPDATEINTABLE = "proreceitam";
        currentRow.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
        return coc01400_receitaService.getDadoswFlow("CDPRODUTO=" + currentRow.CDPRODUTO, "PRORECEITAM").then(function(callback) {
            var processos = callback.dataset.workflow_dadoswflow;
            if (processos.length) {
                if (!validaExclusao(processos)) {
                    ScreenService.closePopup();
                    MessageService.showMessageByCode("COC01400_COMPOSITION_WORKFLOW_PROCESS_ERROR");
                } else {
                    converteFieldsReceita(currentRow, "save");
                    coc01400_receitaService.solicitaWorkFlowVariacao(currentRow).then(function() {
                        widget.container.getWidget("composicaoprodutos").reload().then(function() {
                            widget.container.getWidget("composicaoprodutos").activate();
                            self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC);
                            ScreenService.closePopup(true);
                        });
                    }, function(err) {
                        errorValidations(err.data.error);
                    });
                }
            } else {
                coc01400_receitaService.solicitaWorkFlowVariacao(currentRow).then(function() {
                    widget.container.getWidget("composicaoprodutos").reload().then(function() {
                        widget.container.getWidget("composicaoprodutos").activate();
                        self.setDataSourceForGrid(widget.container.getWidget("composicaoprodutos"), widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA, widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC);
                        ScreenService.closePopup(true);
                    });
                }, function(err) {
                    errorValidations(err.data.error);
                });
            }
        });

    };

    this.validateToSave = function(widget) {
        if (widget.dataSource.data.length) {
            if (widget.currentRow.DSMPREPRECHTML && widget.currentRow.DSMPREPRECHTML.length) {
                isNew = false;
            } else {
                isNew = true;
            }
        } else
            isNew = true;
        return isNew;
    };

    this.editComposicao = function(widget) {
        var actions_left = widget.currentRow.__leftActions;
        var actions_right = widget.currentRow.__rightActions;
        delete widget.currentRow.__leftActions;
        delete widget.currentRow.__rightActions;
        widget.backupRow = angular.copy(widget.currentRow);
        widget.getField("NOME").readOnly = true;
        widget.getField("QTD").readOnly = true;
        widget.getField("QTDEBRUTA").readOnly = true;
        widget.getField("UNIDADE").readOnly = true;
        widget.getField("CUSTO").readOnly = true;
        widget.getField("CDLEANCODE").readOnly = true;
        widget.edit();
        widget.currentRow.__leftActions = actions_left;
        widget.currentRow.__rightActions = actions_right;
    };

    this.cancelComposicao = function(widget) {
        var level = ScreenService.getSwipeLevel(widget);

        if (level == 1) {
            this.cancelaAlteracoes(widget);
        } else {
            widget.currentRow = widget.backupRow;
            widget.view();
            widget.activate();
        }
    };

    this.salvaEditModopreprecVariacao = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.__is_new = self.validateToSave(widget);
            widget.currentRow.DSMOTIVOACAO = widget.container.getWidget("detalhesvariacao").currentRow.DSMOTIVOACAO;
            if (widget.currentRow.__is_new) {
                widget.currentRow.NRITEMPROCAPROVA = "29";
                widget.currentRow.DSINFORANTERIOR = "N";
                widget.currentRow.DSINFORATUAL = "S";
                widget.currentRow.NMCAMPOWFLOW = "IDATIVO";
                widget.currentRow.IDTIPOPERWFLOW = 2; //INSERT
                widget.currentRow.IDATIVO = "N";
            } else {
                widget.currentRow.NRITEMPROCAPROVA = "30";
                widget.currentRow.DSINFORANTERIOR = widget.backupRow.DSMPREPRECHTML;
                widget.currentRow.DSINFORATUAL = widget.currentRow.DSMPREPRECHTML;
                widget.currentRow.NMCAMPOWFLOW = "DSMPREPRECHTML";
                widget.currentRow.IDTIPOPERWFLOW = 1; //UPDATE
                widget.currentRow.DSMPREPRECHTML = widget.backupRow.DSMPREPRECHTML;
            }
            widget.currentRow.NRPROCESSOAPROVA = "1"; //fixed
            widget.currentRow.IDANALIZEATIV = "P";
            widget.currentRow.IDSITWORKFLOW = "1";
            widget.currentRow.CDRECEITA = widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA;
            widget.currentRow.NRVARIACAOREC = widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC;
            widget.currentRow.NMTABELAWFLOW = "MODPREPRECM";
            widget.currentRow.UPDATEINTABLE = "modpreprecm";
            widget.currentRow.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;

            coc01400_receitaService.getDadoswFlow("CDRECEITA=" + widget.currentRow.CDRECEITA + ";NRORG=" + getLocalVar("INFLIGHTOR_NRORGTRAB") + ";NRVARIACAOREC=" + widget.currentRow.NRVARIACAOREC, "MODPREPRECM").then(function(callback) {
                var processos = callback.dataset.workflow_dadoswflow;
                var validate = true;
                if (processos.length) {
                    if (!validaExclusao(processos)) {
                        ScreenService.closePopup();
                        MessageService.showMessageByCode("COC01400_MODE_PREPARE_WORKFLOW");
                        validate = false;
                        widget.reload().then(function() {
                            self.cancelModEdit(widget.container.getWidget("versao"), widget);
                            widget.container.getWidget("versao").currentWidget = "detalhesvariacao";
                        });
                    }
                }
                if (validate) {
                    coc01400_receitaService.solicitaWorkFlowVariacao(widget.currentRow).then(function() {
                        widget.container.getWidget("mododepreparo").reload().then(function() {
                            ScreenService.closePopup(true);
                            widget.container.getWidget("mododepreparo").activate();
                            self.setCurrentForm(widget.container.getWidget("mododepreparo"));
                            widget.container.getWidget("mododepreparo").refreshWidget();
                            self.cancelModEdit(widget.container.getWidget("versao"), widget);
                            widget.container.getWidget("versao").currentWidget = "detalhesvariacao";
                        });
                    });
                }
            });
        }

    };

    this.duplicarVariacao = function(widget) {

        var row = widget.currentRow;
        row.CDOPERADOR_TO_SAVE = row.CDOPERADOR;
        converteFieldsReceita(row, "save");
        ScreenService.confirmMessage(ScreenService.i18n("Será criada uma nova variação. Confirmar operação?"), "question", function() {
            coc01400_receitaService.duplicaVariacaoReceita(row).then(function() {
                widget.container.getWidget("variacoesReceita").reload().then(function() {
                    ScreenService.closeSwipe();
                    widget.container.getWidget("variacoesReceita").activate();
                });
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                self.abreDetalheReceita(widget, false);
            }, function(err) {
                ScreenService.showMessage(err.message);
            });
        });

    };

    this.openPopupDuplicacao = function(widget) {
        var widgetPopup = widget.container.getWidget("nomeReceitaDuplicada");
        ScreenService.openPopup(widgetPopup);
        widgetPopup.getField("DSRECEITA").value(rowDuplicacao.DSRECEITA);
    };

    this.openPopupRendLiq = function(widget) {
        ScreenService.openPopup(widget);
        widget.activate();
        var rendLiquido = widget.container.getWidget("detalhesvariacao").currentRow.QTRENDIMENTOREC;
        widget.getField("QTRENDIMENTOREC").setValue(rendLiquido);
        widget.reload();
    };

    this.salvaRowTemp = function(row) {
        rowDuplicacao = row;
    };

    this.duplicarReceita = function(widget) {

        var row = widget.currentRow;
        rowDuplicacao.DSRECEITA = row.DSRECEITA;
        converteFieldsReceita(rowDuplicacao, "save");
        coc01400_receitaService.duplicaReceita(rowDuplicacao).then(function() {
            widget.container.getWidget("receitas").reload().then(function() {
                ScreenService.closePopup();
                ScreenService.closeSwipe();
                widget.container.getWidget("receitas").activate();
                rowDuplicacao = {};
            });
        }, function(err) {
            ScreenService.showMessage(err.message);
        });

    };

    this.salvaTesteReceita = function(widget) {
        var current = widget.container.getWidget("variacoesReceita").currentRow;
        widget.currentRow.CDRECEITA = current.CDRECEITA;
        widget.currentRow.NRVARIACAOREC = current.NRVARIACAOREC;

        InflightorGeneral.applyParamsNewCodeToRow(widget);

        var row = widget.currentRow;
        coc01400_receitaService.saveTesteReceita(row).then(function(callback) {
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
            widget.reload();
            MessageService.showMessageByCode("SAVE_SUCCESS");
        }, function(err) {
            ScreenService.showMessage(err.message);
        });
    };

    this.deleteTesteReceita = function(widget) {
        var row = widget.currentRow;
        coc01400_receitaService.deleteTesteReceita(row).then(function(callback) {
            widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
            self.abreDetalheReceita(widget, false);
            self.atualizaFormularioVariacao(widget);
            widget.reload();
            MessageService.showMessageByCode("DELETE_SUCCESS");
        }, function(err) {
            ScreenService.showMessage(err.message);
        });
    };

    this.onEnterDadosReceita = function(widget) {

        var row = widget.container.getWidget("receitas").currentRow;
        var cdreceita = row.CDRECEITA;

        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            },
        ];

        widget.dataSourceFilter = filtro;
        if (widget.mode == "list") {
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    this.filterManual = function(widget) {
        var newFilter = [];
        if (widget.currentRow.IDSITUAPROPCIA) {
            if (widget.currentRow.IDSITUAPROPCIA != "V") {
                newFilter = [
                    {
                        "name": "CDRECEITA",
                        "operator": "=",
                        "value": widget.container.getWidget("receitas").currentRow.CDRECEITA
                    },
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": getLocalVar("INFLIGHTOR_CDFILIAL")
                    },
                    {
                        "name": "IDVIGENCIA",
                        "operator": "=",
                        "value": "N"
                    },
                    {
                        "name": "IDSITUAPROPCIA",
                        "operator": "=",
                        "value": widget.currentRow.IDSITUAPROPCIA
                    }
                ];
            } else {
                newFilter = [
                    {
                        "name": "CDRECEITA",
                        "operator": "=",
                        "value": widget.container.getWidget("receitas").currentRow.CDRECEITA
                    },
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": getLocalVar("INFLIGHTOR_CDFILIAL")
                    },
                    {
                        "name": "IDVIGENCIA",
                        "operator": "=",
                        "value": "V"
                    }
                ];
            }
        }
        widget.container.getWidget("manualReceita").dataSourceFilter = newFilter;
        ScreenService.closePopup();
        widget.container.getWidget("manualReceita").reload();
        widget.container.getWidget("manualReceita").activate();
    };

    this.onEnterDadosManual = function(widget) {

        var row = widget.container.getWidget("receitas").currentRow;
        var cdreceita = row.CDRECEITA;
        var rowFilterManual = {};
        if (widget.container.getWidget("filtrarManuais"))
            rowFilterManual = widget.container.getWidget("filtrarManuais").currentRow;
        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": cdreceita
            },
            {
                "name": "IDVIGENCIA",
                "operator": "=",
                "value": "T"
            }
        ];
        if (rowFilterManual.IDSITUAPROPCIA) {
            var currentFilter = {
                "name": "IDSITUAPROPCIA",
                "operator": "=",
                "value": rowFilterManual.IDSITUAPROPCIA
            };
            if (filtro[3])
                filtro[3] = currentFilter;
            else
                filtro.push(currentFilter);
        }

        widget.dataSourceFilter = filtro;
        if (widget.mode == "list") {
            widget.reload().then(function() {
                widget.activate();
            });
        }
    };

    this.cancelarEditReceitaTeste = function(args) {

        var currentRow = args.owner.widget.currentRow;

        if (currentRow.__is_new) {
            InflightorGeneral.backOrCancel(args);
            args.owner.widget.reload().then(function() {
                args.owner.widget.list();
            });
        } else {
            InflightorGeneral.backOrCancel(args);
        }

    };

    this.calculaCusto = function(widget) {
        var current = widget.currentRow;
        var gridWidget = widget.container.getWidget("composicaoprodutos");
        var variacao = widget.container.getWidget("variacoesReceita");
        var currentPai = gridWidget.dataSource.data;
        var DATACUSTO = moment(current.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY");
        if (widget.isValid()) {

            var data = [];
            for (let i in currentPai) {
                var row = {
                    DATACUSTO: DATACUSTO,
                    TIPOCUSTO: current.TIPOCUSTO,
                    CDFILIAL: getLocalVar("INFLIGHTOR_CDFILIAL"),
                    CODIGO: currentPai[i].CODIGO,
                    NRVARIACAO: nrvariacaorec,
                    CDRECEITA: widget.container.getWidget("receitas").currentRow.CDRECEITA,
                    NRVARIACAOREC: nrvariacaorec,
                    ID: currentPai[i].ID
                };
                data.push(row);
            }
            converteFieldsReceita(variacao.currentRow, "save");
            coc01400_receitaService.findCusto(data).then(function(callback) {
                widget.currentRow.CDRECEITA = widget.container.getWidget('receitas').currentRow.CDRECEITA;
                widget.currentRow.NRVARIACAOREC = widget.container.getWidget('variacoesReceita').currentRow.NRVARIACAOREC;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);
                var custoAll = callback.dataset.composicao;
                self.updateVariacaoRec(current, variacao.currentRow);
                self.updateDataSourceforWidget(gridWidget, currentPai, custoAll);
            });
            self.findCusto(widget);
        }
    };

    //Find para o Custo Líquido do stripe footer
    this.findCusto = function(widget, atualizaWidget = true) {
        var DATACUSTO;
        var tipocusto;
        if (moment(widget.currentRow.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY") != "Invalid date") {
            DATACUSTO = moment(widget.currentRow.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY");
        } else {
            DATACUSTO = widget.container.getWidget("variacoesReceita").currentRow.DTCUSTOVARIACAO;
        }
        if (widget.container.getWidget("variacoesReceita").currentRow.IDTIPOCUSTOVAR) {
            tipocusto = widget.container.getWidget("variacoesReceita").currentRow.IDTIPOCUSTOVAR;
        } else {
            tipocusto = widget.container.getWidget("calculodecusto").currentRow.TIPOCUSTO;
        }
        var nrvariacaorec = widget.container.getWidget("variacoesReceita").currentRow.NRVARIACAOREC;
        var dataRec = [];
        var row = {
            DATACUSTO: DATACUSTO,
            TIPOCUSTO: tipocusto,
            CDFILIAL: getLocalVar("INFLIGHTOR_CDFILIAL"),
            CODIGO: widget.container.getWidget("receitas").currentRow.CDRECEITA,
            NRVARIACAO: nrvariacaorec,
            CDRECEITA: widget.container.getWidget("receitas").currentRow.CDRECEITA,
            NRVARIACAOREC: nrvariacaorec,
            ID: "S"
        };
        dataRec.push(row);
        coc01400_receitaService.findCusto(dataRec).then(function(callback) {
            if (atualizaWidget) {
                widget.currentRow.CDRECEITA = widget.container.getWidget('receitas').currentRow.CDRECEITA;
                widget.currentRow.NRVARIACAOREC = widget.container.getWidget('variacoesReceita').currentRow.NRVARIACAOREC;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);
            }
            var custoAll = callback.dataset.composicao;
            var custoUnitario = InflightorGeneral.valueToFloat(custoAll[0].CUSTO);
            var stripeFooter = widget.container.getWidget("stripefooter_composicao");
            var custoRend = 0;

            var gridWidget = widget.container.getWidget("composicaoprodutos");
            var currentPai = gridWidget.dataSource.data;
            currentPai.forEach(function(datasetRow) {
                custoRend += InflightorGeneral.valueToFloat(datasetRow.CUSTO);
            });

            stripeFooter.getField("CUSRENDLIQ").setValue(String(custoRend.toFixed(2)).replace(".", ","));
            stripeFooter.getField("CUSTOUN").setValue(String(custoUnitario.toFixed(2)).replace(".", ","));
            if (rendBruto != undefined) {
                stripeFooter.getField("QTRENDBRUTO").setValue(String(rendBruto.toFixed(3)).replace(".", ","));
            }
            if (rendLiquido != undefined) {
                stripeFooter.getField("QTPRODTCOMP").setValue(String(rendLiquido.toFixed(3)).replace(".", ","));
            }
            if (percentLiq != undefined) {
                stripeFooter.getField("QTPERCENTAPRO").setValue(String(percentLiq).replace(".", ","));
            }
            stripeFooter.reload();

        });
    };

    //Função com o cálculo do rendimento liquido do stripe
    this.updateDataSourceforWidget = function(gridWidget, currentPai, custoAll) {
        //Totalaux -> valor total da receita que será exibido no Stripe de rodapé
        var custoRend = 0;
        var stripeFooter = gridWidget.container.getWidget("stripefooter_composicao");
        currentPai.forEach(function(datasetRow) {

            var custoCodigo = custoAll.filter(function(row) {
                return row.CODIGO == datasetRow.CODIGO;
            });
            datasetRow.CUSTOUN = custoCodigo[0].CUSTO;
            if (datasetRow.ID == "P") {
                if (datasetRow.VRPEFATOCORR == 100) {
                    datasetRow.CUSTO = 0.00;
                } else {
                    let custo = parseFloat(((InflightorGeneral.valueToFloat(custoCodigo[0].CUSTO) / ((100 - InflightorGeneral.valueToFloat(datasetRow.VRPEFATOCORR)) / 100)) + 0.001).toFixed(2));
                    datasetRow.CUSTO = datasetRow.QTD * custo;
                }
            } else {
                let custo = parseFloat((InflightorGeneral.valueToFloat(custoCodigo[0].CUSTO) + 0.001).toFixed(2));
                datasetRow.CUSTO = InflightorGeneral.valueToFloat(custo) * InflightorGeneral.valueToFloat(datasetRow.QTD);
            }
            datasetRow.CUSTO = parseFloat((InflightorGeneral.valueToFloat(datasetRow.CUSTO) + 0.001).toFixed(2));
            custoRend += InflightorGeneral.valueToFloat(datasetRow.CUSTO);
        });

        stripeFooter.getField("CUSRENDLIQ").setValue(String(custoRend.toFixed(2)).replace(".", ","));

        gridWidget.dataSource.data = currentPai;
        self.updateSubRecandProd(currentPai);
        ScreenService.closePopup();
        gridWidget.activate();
    };

    this.updateSubRecandProd = function(data) {
        var dataRec = [];
        var dataProd = [];
        var promises = [];
        data.forEach(function(rows) {
            if (rows.ID == "S") {
                var row = {
                    CDSUBRECEITA: rows.CODIGO,
                    CDRECEITA: cdreceita,
                    NRVARIACAOREC: nrvariacaorec,
                    NRORDSUBREC: rows.ORDEM,
                    QTRENDSUBREC: rows.QTD,
                    VRCUSTOSUBREC: rows.CUSTO,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    __is_new: false
                };
                dataRec.push(row);
            } else {
                var rowprod = {
                    CDPRODUTO: rows.CODIGO,
                    CDRECEITA: cdreceita,
                    NRVARIACAOREC: nrvariacaorec,
                    QTPRODREC: rows.QTD,
                    NRORDPREC: rows.ORDEM,
                    SGGRUPORECFLEX: rows.SGGRUPORECFLEX,
                    VRCUSTOPROREC: rows.CUSTO,
                    DTINCLUSAO: moment().format("DD/MM/YYYY"),
                    __is_new: false
                };
                dataProd.push(rowprod);
            }
        });
        promises.push(coc01400_receitaService.updateSubReceita(dataRec));
        promises.push(coc01400_receitaService.updateSubProd(dataProd));

        ZHPromise.all(promises).then(null, function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    this.newVariacaorec = function(widget) {
        var validate = true;
        var currentRowReceita = widget.container.getWidget("detalheReceita").currentRow;
        coc01400_receitaService.getDadoswFlow(currentRowReceita.CDRECEITA, "RECEITAM").then(function(callback) {
            var workflow = callback.dataset.workflow_dadoswflow;
            if (workflow.length) {
                workflow.forEach(function(row) {
                    if ((row.NRITEMPROCAPROVA == 1 || row.NRITEMPROCAPROVA == 2) && row.IDSITWORKFLOW == 1) {
                        MessageService.showMessageByCode("COC01400_RECIPE_WORKFLOW_PROCESS");
                        validate = false;
                    }
                });
            }
            if (validate) {
                widget.container.getWidget("novaVariacao").newRow();
                ScreenService.openSwipe(widget.container.getWidget("novaVariacao"));
                verificaReceitaFlex(widget.container.getWidget("novaVariacao"));
                widget.container.getWidget("novaVariacao").edit();
            }
        });
    };

    this.updateVariacaoRec = function(current, currentVari) {

        var DATACUSTO = moment(current.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY");

        currentVari.DTCUSTOVARIACAO = DATACUSTO;
        currentVari.IDTIPOCUSTOVAR = current.TIPOCUSTO;

        coc01400_receitaService.updateVariacaoRec(currentVari).then(null, function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    this.searchReceita = function() {
        return true;
    };

    this.solicitarAprovacaoReceita = function(widget, process) {
        var motivoWidget = {
            "name": "motivoWidget",
            "label": "Justificativa",
            "template": "widget/form.html",
            "container": {},
            "process": process,
            "isVisible": true,
            "popupNoBlock": false,
            "events": [],
            "fields": [
                {
                    "name": "DSMOTIVOACAO",
                    "label": "Motivo",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                },
                {
                    "name": "IDSITOPERWFLOW",
                    "label": "",
                    "template": "",
                    "isVisible": false,
                    "readOnly": false,
                    "showOnForm": true,
                }],
            "actions": [
                {
                    "name": "cancelar",
                    "label": "Cancelar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": "Confirmar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "coc01400_receitaWorkflowController.solicitaAprovacao(args.owner.widget, args.owner.widget.process, args);"
                        }
                    ]
                }
            ]
        };
        motivoWidget.container = widget.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(motivoWidget);
        ScreenService.openPopup(motivoWidgetInitialized);
    };

    this.solicitaAprovacao = function(widget, process, args) {
        var widgetReceita = widget.container.getWidget("detalheReceita");
        widgetReceita.currentRow.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        widgetReceita.currentRow.NRPROCESSOAPROVA = "1";
        var widgetVariacao = widget.container.getWidget("detalhesvariacao");
        widgetVariacao.currentRow.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;

        switch (process) {
            case "new_receita":
                self.solicitaWorkFlow(widgetReceita);
                break;
            case "edit_receita":
                self.salvaEditReceita(widgetReceita, args);
                break;
            case "delete_receita":
                widgetReceita.currentRow.DSINFORANTERIOR = widgetReceita.currentRow.CDRECEITA;
                widgetReceita.currentRow.DSINFORATUAL = widgetReceita.currentRow.CDRECEITA;
                widgetReceita.currentRow.NMCAMPOWFLOW = "CDRECEITA";
                self.removeReceita(widgetReceita);
                break;
            case "new_variacao":
                self.solicitaWorkFlowVariacao(widgetVariacao);
                break;
            case "edit_variacao":
                self.salvaEditVariacaoReceita(widgetVariacao);
                break;
            case "edit_variacao_comp":
                self.salvaEditVariacaoReceitaComp(widgetVariacao);
                break;
            case "delete_variacao":
                widgetVariacao.currentRow.DSINFORANTERIOR = widgetVariacao.currentRow.NRVARIACAOREC;
                widgetVariacao.currentRow.DSINFORATUAL = widgetVariacao.currentRow.NRVARIACAOREC;
                widgetVariacao.currentRow.NMCAMPOWFLOW = "NRVARIACAOREC";
                widgetVariacao.currentRow.DSRECEITA = widgetReceita.currentRow.DSRECEITA;
                self.validateDeleteVariacao(widgetVariacao).then(function(callback) {
                    if (callback)
                        self.removeVariacao(widgetVariacao);
                    else
                        MessageService.showMessageByCode("DELETE_ERROR");
                }, function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    ScreenService.showMessage(errorMessage);
                });
                break;
            case "new_produto":
                self.addNovaCompProduto(widget.container.getWidget("selectProdutos"));
                break;
            case "new_subreceita":
                self.addNovaCompReceita(widget.container.getWidget("selectReceitas"));
                break;
            case "edit_composicao":
                self.salvarAlteracoesRec(widget);
                break;
            case "delete_composicao":
                self.directRemove(widget);
                break;
            case "new_modpreprecm":
                self.salvaEditModopreprecVariacao(widget.container.getWidget("mododepreparo"));
                break;
            case "new_unidade":
                self.salvaUnidade(widget.container.getWidget("selectUnidade"));
                break;
            case "delete_unidade":
                self.deleteUnidade(widget.container.getWidget("unidade"));
        }
    };

    this.solicitaWorkFlow = function(widgetReceita) {
        widgetReceita.currentRow.NRITEMPROCAPROVA = "1";
        widgetReceita.currentRow.IDSITWORKFLOW = "1";
        widgetReceita.currentRow.DSINFORANTERIOR = "N";
        widgetReceita.currentRow.DSINFORATUAL = "S";
        widgetReceita.currentRow.NMCAMPOWFLOW = "IDATIVO";
        widgetReceita.currentRow.IDTIPOPERWFLOW = 2;
        converteFieldsReceita(widgetReceita.currentRow, "save");
        coc01400_receitaService.solicitaWorkFlow(widgetReceita.currentRow).then(function() {
            MessageService.showMessageByCode("WORKFLOW_SUCCESS").then(function() {
                ScreenService.closePopup();
                self.abreDetalheReceita(widgetReceita);
            });
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.solicitaWorkFlowVariacao = function(widget) {
        widget.currentRow.NRPROCESSOAPROVA = "1";
        widget.currentRow.NRITEMPROCAPROVA = "19";
        widget.currentRow.IDSITWORKFLOW = "1";
        widget.currentRow.DSINFORANTERIOR = "N";
        widget.currentRow.DSINFORATUAL = "S";
        widget.currentRow.NMCAMPOWFLOW = "IDATIVO";
        widget.currentRow.IDTIPOPERWFLOW = 2;
        widget.currentRow.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
        converteFieldsReceita(widget, "save");
        widget.currentRow.IDATIVO = "N";
        coc01400_receitaService.solicitaWorkFlowVariacao(widget.currentRow).then(function() {
            ScreenService.closePopup();
            self.loadVersoes(widget);
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data)
                errorMessage = error.data.error;
            ScreenService.showMessage(errorMessage);
        });
    };

    this.deleteModpreprecm = function(widget) {
        coc01400_receitaService.getDadoswFlow("CDRECEITA=" + widget.currentRow.CDRECEITA + ";NRORG=" + getLocalVar("INFLIGHTOR_NRORGTRAB") + ";NRVARIACAOREC=" + widget.currentRow.NRVARIACAOREC, "MODPREPRECM").then(function(callback) {
            var processos = callback.dataset.workflow_dadoswflow;
            var validate = true;
            if (processos.length) {
                if (!validaExclusao(processos)) {
                    ScreenService.closePopup();
                    MessageService.showMessageByCode("COC01400_MODE_PREPARE_WORKFLOW");
                    validate = false;
                }
            }
            if (validate) {
                ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                    widget.removeRow().then(function() {
                        self.onEnterDadosVersao(widget);
                        widget.activate();
                        self.setCurrentForm(widget);
                        widget.getField("DSMPREPRECHTML").setValue("");
                    }, function(error) {
                        if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                            MessageService.showMessageByCode("DELETE_ERROR");
                        } else {
                            ScreenService.showMessage(error.data.error);
                        }
                    });
                });
            }
        });
    };

    this.applyInactiveStyleProduto = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            },
            {
                "condition": "row.IDPRODATIVPR == 'S' && row.PRODORCAMENTO == 'S'",
                "cssClass": "inf-orange"
            }
        ];
        field.selectWidget.events[0].code = "coc01400_receitaWorkflowController.inactiveBranchWarningProduto(args);";
    };

    this.inactiveBranchWarningProduto = function(args) {
        if (args.owner.currentRow.IDPRODATIVPR === "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.applyInactiveStyleReceita = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc01400_receitaWorkflowController.inactiveBranchWarningReceita(args);";
    };

    this.inactiveBranchWarningReceita = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_RECIPE");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.excluiUnidadeVariacao = function(widget) {

        var widgetVariacoes = widget.container.getWidget("variacoesReceita");
        var quantidadeVariacoes = widgetVariacoes.dataSource.data.length;
        var widgetSelectVariacoes = widget.container.getWidget("selectVariacoes");
        var unidadesSelecionadas = widget.dataSource.getCheckedRows();

        if (quantidadeVariacoes > 1) {

            var filtro = [
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": unidadesSelecionadas[0].CDRECEITA
                },
                {
                    "name": "NRVARIACAOREC",
                    "operator": "<>",
                    "value": unidadesSelecionadas[0].NRVARIACAOREC
                }
            ];

            widgetSelectVariacoes.dataSourceFilter = filtro;
            widgetSelectVariacoes.reload();

            ScreenService.openPopup(widgetSelectVariacoes);

        } else {
            MessageService.showMessageByCode("COC01400_DELETE_ERROR_VARIATION_RECIPE");
        }

    };

    this.motivoSolicitacaoWorkflowWidget = function(unidadesSelecionadasDelete, unidadesSelecionadasSalvar, widget) {
        var motivoWidget = {
            "name": "motivoWidget",
            "label": "Justificativa",
            "template": "widget/form.html",
            "container": {},
            "unidadesSelecionadasDelete": unidadesSelecionadasDelete,
            "unidadesSelecionadasSalvar": unidadesSelecionadasSalvar,
            "isVisible": true,
            "popupNoBlock": false,
            "events": [],
            "fields": [
                {
                    "name": "DSMOTIVOACAO",
                    "label": "Motivo",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                },
                {
                    "name": "IDSITOPERWFLOW",
                    "label": "",
                    "template": "",
                    "isVisible": false,
                    "readOnly": false,
                    "showOnForm": true,
                }
            ],
            "actions": [
                {
                    "name": "cancelar",
                    "label": "Cancelar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": "Confirmar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "coc01400_receitaWorkflowController.removeEAtribuiUnidade(args.owner.widget);"
                        }
                    ]
                }
            ]
        };
        motivoWidget.container = widget.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(motivoWidget);
        ScreenService.openPopup(motivoWidgetInitialized);
    };

    this.atribuiUnidadeDeletaUnidadeVariacao = function(widget) {
        var self = this;
        var widgetUnidade = widget.container.getWidget("unidade");
        var unidadesSelecionadasDelete = angular.copy(widgetUnidade.dataSource.getCheckedRows());
        var unidadesSelecionadasSalvar = angular.copy(widgetUnidade.dataSource.getCheckedRows());
        var rowParaAtualizacao = angular.copy(widgetUnidade.dataSource.getCheckedRows());

        for (let unidade in unidadesSelecionadasSalvar) {
            unidadesSelecionadasSalvar[unidade].NRVARIACAOREC = widget.currentRow.NRVARIACAOREC;
            unidadesSelecionadasSalvar[unidade].__is_new = true;
            rowParaAtualizacao[unidade].NRVARIACAOREC = widget.currentRow.NRVARIACAOREC;
            rowParaAtualizacao[unidade].__is_new = false;
            rowParaAtualizacao[unidade].changed = true;
            unidadesSelecionadasDelete[unidade].DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
            unidadesSelecionadasSalvar[unidade].DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;
        }

        validaProcessoWorkflow("1", "34").then(function() {
            self.motivoSolicitacaoWorkflowWidget(unidadesSelecionadasDelete, unidadesSelecionadasSalvar, widget);
        }, function() {
            coc01400_receitaService.atribuirUnidade(rowParaAtualizacao).then(function(callback) {
                ScreenService.closePopup();
                widgetUnidade.unSelectAll();
                widgetUnidade.reload();
                widget.currentRow.CDRECEITA = widget.currentRow.CDRECEITA || widget.container.getWidget('receitas').currentRow.CDRECEITA;
                widget.currentRow.NRVARIACAOREC = widget.container.getWidget("variacoesReceita").currentRow.NRVARIACAOREC;
                self.abreDetalheReceita(widget, false);
                self.atualizaFormularioVariacao(widget);
            });
        });

    };

    this.removeEAtribuiUnidade = function(widget) {
        if (widget.isValid()) {
            var filtro = widget.unidadesSelecionadasDelete[0].CDRECEITA;
            coc01400_receitaService.getDadoswFlow(filtro, "FILIRECEITA").then(function(callback) {
                if (callback.dataset.workflow_dadoswflow.length > 0) {
                    MessageService.showMessageByCode("COC01400_UNIT_WORKFLOW_PROCESS");
                    ScreenService.closePopup();
                } else {
                    var data = [
                        {
                            "unidadesDelete": widget.unidadesSelecionadasDelete
                        },
                        {
                            "unidadesSalvar": widget.unidadesSelecionadasSalvar
                        },
                        {
                            "row": widget.currentRow
                        }
                    ];
                    restEngine.post("coc#/solicita_alteracao_unidade_variacao", {
                        "requestType": "DataSet",
                        "dataset": data
                    }).then(function() {
                        ScreenService.closePopup();
                        ScreenService.closePopup();
                        widget.container.getWidget("unidade").unSelectAll();
                    });
                }
            });
        }
    };

    this.deleteUnidade = function(widget) {

        if (widget.isValid()) {

            var checkedRows = widget.dataSource.getCheckedRows();
            var validate = true;

            checkedRows.forEach(function(row) {

                row.__is_new = false;
                row.DSMOTIVOACAO = widget.container.getWidget("detalhesvariacao").currentRow.DSMOTIVOACAO;
                row.NRITEMPROCAPROVA = "34";
                row.DSINFORANTERIOR = row.CDFILIAL;
                row.DSINFORATUAL = row.CDFILIAL;
                row.NMCAMPOWFLOW = "CDFILIAL";
                row.IDTIPOPERWFLOW = 3; //DELETE
                row.NRPROCESSOAPROVA = "1"; //fixed
                row.IDANALIZEATIV = "P";
                row.UNIDADE = row.CDFILIAL;
                row.IDSITWORKFLOW = "1";
                row.CDRECEITA = widget.container.getWidget("detalhesvariacao").currentRow.CDRECEITA;
                row.NRVARIACAOREC = widget.container.getWidget("detalhesvariacao").currentRow.NRVARIACAOREC;
                row.NMTABELAWFLOW = "FILIRECEITA";
                row.UPDATEINTABLE = "filireceita";
                row.DSRECEITA = widget.container.getWidget("receitas").currentRow.DSRECEITA;

                coc01400_receitaService.getDadoswFlow("CDRECEITA=" + row.CDRECEITA + ";NRORG=" + getLocalVar("INFLIGHTOR_NRORGTRAB") + ";NRVARIACAOREC=" + row.NRVARIACAOREC + ";CDFILIAL=" + row.UNIDADE, "FILIRECEITA").then(function(callback) {
                    var processos = callback.dataset.workflow_dadoswflow;
                    if (processos.length) {
                        if (!validaExclusao(processos)) {
                            ScreenService.closePopup(true);
                            MessageService.showMessageByCode("COC01400_UNIT_WORKFLOW_PROCESS");
                            validate = false;
                            widget.container.getWidget("unidade").reload().then(function() {
                                widget.container.getWidget("unidade").activate();
                            });
                        }
                    }
                    if (validate) {
                        row.IDATIVO = "N";
                        coc01400_receitaService.solicitaWorkFlowVariacao(row).then(function() {
                            widget.container.getWidget("unidade").reload();
                        }, function(callback) {
                            InflightorGeneral.callbackError(callback);
                        });
                    }
                });
                ScreenService.closePopup();
                if (validate) {
                    MessageService.showMessageByCode("WORKFLOW_SUCCESS");
                }
                widget.unSelectAll();
                templateManager.updateTemplate();
                widget.container.getWidget("unidade").activate();

            });

        }

    };

    var verificaReceitaFlex = function(widget) {

        var rowWidgetReceita = widget.container.getWidget("receitas").currentRow;
        var fieldOpcoes = widget.getField("NROPCOES");

        if (rowWidgetReceita.IDRECFLEXIVEL === "M") {
            fieldOpcoes.isVisible = true;
            fieldOpcoes.readOnly = false;
            fieldOpcoes.validations = {
                "required": true
            };
            widget.currentRow.NROPCOES = widget.currentRow.NROPCOES > 1 ? widget.currentRow.NROPCOES : 1;
        } else if (rowWidgetReceita.IDRECFLEXIVEL === "P") {
            fieldOpcoes.isVisible = true;
            fieldOpcoes.readOnly = true;
            fieldOpcoes.validations = {};
            widget.currentRow.NROPCOES = 1;
        } else {
            fieldOpcoes.isVisible = false;
            fieldOpcoes.readOnly = true;
            fieldOpcoes.validations = {};
        }

    };

    var validaEIncluiComposicao = function(widget, iditemcomp) {
        var widgetReceita = widget.container.getWidget("receitas");
        if (widgetReceita.currentRow.IDRECFLEXIVEL === "M") {
            if (iditemcomp === "P")
                incluiNovoProdutoComposicao(widget);
            else if (iditemcomp === "S")
                incluiNovaSubreceitaComposicao(widget);
        } else if (widgetReceita.currentRow.IDRECFLEXIVEL === "P") {
            if (iditemcomp === "P") {
                restEngine.post("coc#/crud/coc01400_receitas_produtos", {
                    "requestType": "FilterData",
                    "filter": widget.dataSourceFilter
                }).then(function(callback) {
                    var contadorProdutos = 0;
                    var subreceitasEProdutos = callback.dataset.coc01400_receitas_produtos;
                    if (subreceitasEProdutos.length !== 0) {
                        subreceitasEProdutos.forEach(function(itemComp) {
                            if (itemComp.ID === "P")
                                contadorProdutos += 1;
                        });
                    }
                    if (contadorProdutos > 0)
                        MessageService.showMessageByCode("COC01400_NOT_ADD_PRODUCT");
                    else
                        incluiNovoProdutoComposicao(widget);
                });
            } else if (iditemcomp === "S") {
                var contadorSubreceita = 0;
                restEngine.post("coc#/crud/coc01400_receitas_produtos", {
                    "requestType": "FilterData",
                    "filter": widget.dataSourceFilter
                }).then(function(callback) {
                    var subreceitasEProdutos = callback.dataset.coc01400_receitas_produtos;
                    if (subreceitasEProdutos.length !== 0) {
                        subreceitasEProdutos.forEach(function(itemComp) {
                            if (itemComp.ID === "S")
                                contadorSubreceita += 1;
                        });
                    }
                    if (contadorSubreceita > 0)
                        MessageService.showMessageByCode("COC01400_NOT_ADD_SUB_RECIPE");
                    else
                        incluiNovaSubreceitaComposicao(widget);
                });
            }
        } else {
            if (iditemcomp === "P")
                incluiNovoProdutoComposicao(widget);
            else if (iditemcomp === "S")
                incluiNovaSubreceitaComposicao(widget);
        }
    };

    var incluiNovoProdutoComposicao = function(widget) {

        var produtoJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

        if (produtoJaInc.length > 0) {
            var dsfiltro = [
                {
                    "name": "IDPRODCOMISS",
                    "operator": "IS_NULL",
                    "value": ""
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "SGUNIDADE",
                    "operator": "IS_NOT_NULL",
                    "value": ""
                }
            ];
        }

        var validate = true;
        var currentRowReceita = widget.container.getWidget("detalheReceita").currentRow;
        coc01400_receitaService.getDadoswFlow(currentRowReceita.CDRECEITA, "RECEITAM").then(function(callback) {
            var workflow = callback.dataset.workflow_dadoswflow;
            if (workflow.length) {
                workflow.forEach(function(row) {
                    if ((row.NRITEMPROCAPROVA == 1 || row.NRITEMPROCAPROVA == 2) && row.IDSITWORKFLOW == 1) {
                        MessageService.showMessageByCode("COC01400_RECIPE_WORKFLOW_PROCESS");
                        validate = false;
                    }
                });
            }
            if (validate) {
                widget.getField("NMPRODUTO").dataSourceFilter = dsfiltro;
                widget.getField("NMPRODUTO").reload().then(function() {
                    widget.newRow();
                    ScreenService.openPopup(widget);
                });
            }
        });
    };

    var incluiNovaSubreceitaComposicao = function(widget) {

        var filtro = [];
        var receitaPai = widget.container.getWidget("receitas").currentRow.CDRECEITA;
        var receitaJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;
        filtro.push(receitaPai);
        if (receitaJaInc.length > 0 && receitaJaInc.ID == "S") {

            for (let i in receitaJaInc) {
                filtro.push(receitaJaInc[i].CODIGO);
            }

        }
        var dsfiltro = [
            {
                "name": "CDRECEITA",
                "operator": "NOT_IN",
                "value": filtro
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": "@getLocalVar('INFLIGHTOR_NRORGTRAB')@"
            }
        ];
        var validate = true;
        var currentRowReceita = widget.container.getWidget("detalheReceita").currentRow;

        coc01400_receitaService.getDadoswFlow(currentRowReceita.CDRECEITA, "RECEITAM").then(function(callback) {
            var workflow = callback.dataset.workflow_dadoswflow;
            if (workflow.length) {
                workflow.forEach(function(row) {
                    if ((row.NRITEMPROCAPROVA == 1 || row.NRITEMPROCAPROVA == 2) && row.IDSITWORKFLOW == 1) {
                        MessageService.showMessageByCode("COC01400_RECIPE_WORKFLOW_PROCESS");
                        validate = false;
                    }
                });
            }
            if (validate) {
                widget.getField("DSRECEITA").dataSourceFilter = dsfiltro;
                widget.getField("DSRECEITA").reload().then(function() {
                    widget.newRow();
                    ScreenService.openPopup(widget);
                });
            }
        });
    };

    this.definePreferencia = function(widget) {

        var data = [];

        widget.dataSource.getCheckedRows().forEach(function(row) {
            data.push({
                CDLEANCODE: row.CDLEANCODE,
                CDRECEITA: row.CDRECEITA,
                CODIGO: row.CODIGO,
                CODIGO_GRID: row.CODIGO_GRID,
                CUSTO: row.CUSTO,
                ID: row.ID,
                IDANALIZEATIV: row.IDANALIZEATIV,
                IDPREFERENCIA: "S",
                IDPREFERENCIA_OLD: row.IDPREFERENCIA,
                NOME: row.NOME,
                NRORG: row.NRORG,
                NRVARIACAOREC: row.NRVARIACAOREC,
                ORDEM: row.ORDEM,
                QTD: row.QTD,
                QTDEBRUTA: row.QTDEBRUTA,
                SGGRUPORECFLEX: row.SGGRUPORECFLEX,
                UNIDADE: row.UNIDADE,
                VRPEFATOCORR: row.VRPEFATOCORR,
                YIELD: row.YIELD,
                __isSelected: row.__isSelected,
                __is_new: row.__is_new,
                changed: true
            });
        });

        restEngine.post("coc#/composicao_preferencia", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(callback) {
            ScreenService.showMessage(callback.messages[0].message);
            widget.reload();
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });

    };

    this.removePreferencia = function(widget) {
        var data = [];

        widget.dataSource.getCheckedRows().forEach(function(row) {
            data.push({
                CDLEANCODE: row.CDLEANCODE,
                CDRECEITA: row.CDRECEITA,
                CODIGO: row.CODIGO,
                CODIGO_GRID: row.CODIGO_GRID,
                CUSTO: row.CUSTO,
                ID: row.ID,
                IDANALIZEATIV: row.IDANALIZEATIV,
                IDPREFERENCIA: "N",
                IDPREFERENCIA_OLD: row.IDPREFERENCIA,
                NOME: row.NOME,
                NRORG: row.NRORG,
                NRVARIACAOREC: row.NRVARIACAOREC,
                ORDEM: row.ORDEM,
                QTD: row.QTD,
                QTDEBRUTA: row.QTDEBRUTA,
                SGGRUPORECFLEX: row.SGGRUPORECFLEX,
                UNIDADE: row.UNIDADE,
                VRPEFATOCORR: row.VRPEFATOCORR,
                YIELD: row.YIELD,
                __isSelected: row.__isSelected,
                __is_new: row.__is_new,
                changed: true
            });
        });

        restEngine.post("coc#/composicao_remove_preferencia", {
            "requestType": "DataSet",
            "dataset": data
        }).then(function(callback) {
            ScreenService.showMessage(callback.messages[0].message);
            widget.reload();
        });
    };

    this.setVisibilityCheckbox = function(widget, visibility = true) {
        widget.showCheckbox = visibility;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01400_receitaWorkflowController", coc01400_receitaWorkflowController);
});

// FILE: ./assets/js/controllers/coc01401_categoriaReceitaController.js
var coc01401_categoriaReceitaController = function(InflightorGeneral, MessageService) {

    this.salvaClassificacaoReceita = function(widget) {
        var registroExistente = verificaExisteClassificacao(widget.dataSource.data, widget.currentRow, "NMCATEGORIA");

        if (!registroExistente && widget.currentRow.NMCATEGORIA.length > 0) {
            InflightorGeneral.saveCrud(widget);
        } else if (widget.currentRow.NMCATEGORIA.length == 0) {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        } else {
            MessageService.showMessageByCode("REGISTER_EXISTS");
        }
    };

    function verificaExisteClassificacao(dataSource, currentRow, fieldName) {
        for (let row in dataSource) {
            if (dataSource[row][fieldName].toUpperCase() == currentRow[fieldName].toUpperCase() && !dataSource[row].changed) {
                return true;
            }
        }

        return false;
    }

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01401_categoriaReceitaController", coc01401_categoriaReceitaController);
});

// FILE: ./assets/js/controllers/coc01402_produto_orcamento_controller.js
var coc01402_produto_orcamento_controller = function(ScreenService, InflightorGeneral) {

    this.openPopupAdd = function(widget) {

        var widgetPopupAdd = widget.container.getWidget('addProdForm');
        var cdfilial = widget.appliedFilter.FILTER_ROW.CDFILIAL;

        widgetPopupAdd.newRow();

        widgetPopupAdd.currentRow.CDFILIAL = cdfilial;

        ScreenService.openPopup(widgetPopupAdd);

    };

    this.saveProdOrcamento = function(widget) {

        var widgetCurrentRow = widget.currentRow;

        if (widget.isValid()) {
            InflightorGeneral.callRestEngine("coc#/prodorcainf_filial_produto/save", "DataSet", creatDatasetToSave(widgetCurrentRow)).then(function() {
                widget.container.getWidget('coc01402_produto_orcamento').reload();
                ScreenService.closePopup();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }

    };

    this.deleteProdOrca = function(widget) {

        var produtos = widget.dataSource.getCheckedRows();

        ScreenService.confirmMessage("Excluir o(s) registro(s) selecionado(s)?", "question", function() {
            InflightorGeneral.callRestEngine("coc#/crud/prodorcainf/remove", "DataSet", produtos).then(function() {
                widget.reload();
            });
        });

    };

    var creatDatasetToSave = function(widgetCurrentRow) {

        var produtos = widgetCurrentRow.NMPRODUTO;
        var datasetToSave = [];

        for (let i = 0; i < produtos.length; i++) {
            datasetToSave.push({
                "CDPRODUTO": produtos[i],
                "CDFILIAL": widgetCurrentRow.CDFILIAL,
                "__changed": true,
                "__is_new": true,
                "__cast_number": true,
                "__new_code_length": 10,
                "__new_code_pk": "NRPRODORCAINF",
                "__table_pk": "NRPRODORCAINF"
            });
        }

        return datasetToSave;

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01402_produto_orcamento_controller", coc01402_produto_orcamento_controller);
});

// FILE: ./assets/js/controllers/coc01410_cadastrosubgrupoController.js
var coc01410_cadastrosubgrupoController = function(ScreenService, MessageService, FilterUtil, coc01410_cadastrosubgrupoService, InflightorGeneral, restEngine, ZHPromise) {

    var parent = this;

    this.copySubgrupoFilterFromParent = function(widget) {
        if (widget.mode == "list") {
            var parentWidget = widget.parent;
            FilterUtil.updateFilter(widget, parentWidget.currentRow.CDSUBGRU, "CDSUBGRU");
            widget.reload();
        }
    };

    this.setCdSubgrupoInRow = function(widget) {
        var currentRow = widget.currentRow;
        if (currentRow.QTPRODSU)
            currentRow.QTPRODSU = InflightorGeneral.valueToFloat(currentRow.QTPRODSU);
        if (currentRow.QTRECSUB)
            currentRow.QTRECSUB = InflightorGeneral.valueToFloat(currentRow.QTRECSUB);
        if (!currentRow.CDSUBGRU) {
            currentRow.CDSUBGRU = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDSUBGRU");
        }
    };

    this.setVisibleforTabbed = function(widgets, value, exception) {
        widgets.forEach(function(widget) {
            if (exception) {
                if (exception.name != widget.name && widget.name != "composicaoprodutos")
                    widget.isVisible = value;
            } else {
                widget.isVisible = value;
            }
        });
    };

    this.filterRowsFromSelect = function(field, primaryKey) {
        var widget = field.widget;
        var data = widget.dataSource.data;
        var keyArray = data.map(function(currentItem) {
            return currentItem[primaryKey];
        });
        keyArray.pop();
        if (keyArray.length > 0) {
            FilterUtil.updateFilter(field, keyArray, primaryKey, "NOT_IN");
        }
        field.reload();
        //console.log(widget, data);
    };

    this.cancelInlineEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
    };

    this.checkDescFieldsRequirement = function(widget) {
        // restEngine.post("coc#/crud/pargeralcoc", {
        //     "requestType": "FilterData"
        // }).then(
        //     function(response) {
        //         var dataSetRow = response.dataset.pargeralcoc[0];
        //         if (dataSetRow.IDOBRIGANMEST === "S") {
        //             widget.getField("DSSUBGRUIN").validations = {
        //                 "required": true
        //             };
        //             widget.getField("DSSUBGRUES").validations = {
        //                 "required": true
        //             };
        //         }
        //     }
        // );
    };

    this.setDataSourceForGrid = function(widget) {

        var promises = [];
        var produtos = [];
        var receitas = [];
        var CDSUBGRU = widget.currentRow.CDSUBGRU;
        var positions = {
            PRODUTOS: {
                CODIGO: "CDPRODUTO",
                NOME: "NMPRODUTO",
                QTD: "QTPRODSU",
                UNIDADE: "SGUNIDADE",
                SUBGRU: "CDSUBGRU",
                ID: "P"
            },
            RECEITAS: {
                CODIGO: "CDRECEITA",
                NOME: "DSRECEITA",
                QTD: "QTRECSUB",
                UNIDADE: "SGUNIDADE",
                SUBGRU: "CDSUBGRU",
                ID: "S"
            }

        };
        var data = [];
        promises.push(coc01410_cadastrosubgrupoService.returnProduto(CDSUBGRU));
        promises.push(coc01410_cadastrosubgrupoService.returnReceita(CDSUBGRU));

        ZHPromise.all(promises).then(function(callback) {

            produtos = callback[0].dataset.prodsubgrum_produto;
            receitas = callback[1].dataset.recsubgrum_receitam;

            setRowByName(data, produtos, positions.PRODUTOS);
            setRowByName(data, receitas, positions.RECEITAS);
            var composicao = widget.container.getWidget("composicaoprodutos");
            composicao.dataSource.data = data;

        }.bind(this), function(callback) {
            InflightorGeneral.callbackError(callback);
        });
    };

    function setRowByName(dataSource, array, position) {
        array.forEach(function(object) {
            var row = {
                "CODIGO": object[position.CODIGO],
                "NOME": object[position.NOME],
                "SUBGRU": object[position.SUBGRU],
                "QTD": object[position.QTD],
                "UNIDADE": object[position.UNIDADE],
                "ID": position.ID
            };
            dataSource.push(row);
        });
    }

    this.addSubgrupo = function(widget) {
        widget.newRow();
        this.setVisibleforTabbed(widget.container.getWidget("cadastrosubgrupo").widgets, false, widget);
    };

    this.addProduto = function(widget) {

        var filtro = [];
        var produtoJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;

        if (produtoJaInc.length > 0) {

            for (let i in produtoJaInc) {
                if (produtoJaInc[i].ID == "P")
                    filtro.push(produtoJaInc[i].CODIGO);
            }

            var dsfiltro = [];

            if (filtro.length) {
                dsfiltro = [{
                        "name": "CDPRODUTO",
                        "operator": "NOT_IN",
                        "value": filtro
                },
                    {
                        "name": "IDPRODCOMISS",
                        "operator": "IS_NULL",
                        "value": ""
                },
                    {
                        "name": "SGUNIDADE",
                        "operator": "IS_NOT_NULL",
                        "value": ""
                }
                ];
            } else {
                dsfiltro = [
                    {
                        "name": "IDPRODCOMISS",
                        "operator": "IS_NULL",
                        "value": ""
                    },
                    {
                        "name": "SGUNIDADE",
                        "operator": "IS_NOT_NULL",
                        "value": ""
                    }
                ];
            }
            widget.getField("NMPRODUTO").dataSourceFilter = dsfiltro;
        }

        widget.getField("NMPRODUTO").reload().then(function() {
            widget.newRow();
            ScreenService.openPopup(widget);
        });

    };

    this.addReceita = function(widget) {

        var filtro = [];
        var receitaJaInc = widget.container.getWidget("composicaoprodutos").dataSource.data;
        var dsfiltro = {};
        if (receitaJaInc.length > 0) {
            receitaJaInc.forEach(function(receitas) {
                if (receitas.ID == "S")
                    filtro.push(receitas.CODIGO);
            });
        }
        if (filtro.length) {
            dsfiltro = {
                "name": "CDRECEITA",
                "operator": "NOT_IN",
                "value": filtro
            };
        }

        widget.getField("DSRECEITA").dataSourceFilter = dsfiltro;
        widget.getField("DSRECEITA").reload().then(function() {
            widget.newRow();
            ScreenService.openPopup(widget);
        });

    };

    this.directSave = function(widget) {
        widget.currentRow.__is_new = false;
        if (widget.currentRow.ID == "P") {
            widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
            widget.currentRow.QTPRODSU = widget.currentRow.QTD;
            parent.newProduto(widget);
        } else {
            widget.currentRow.CDRECEITA = widget.currentRow.CODIGO;
            widget.currentRow.QTRECSUB = widget.currentRow.QTD;
            parent.newReceita(widget);
        }
    };
    this.directRemove = function(widget) {
        var dataset = widget.getCheckedRows();
        var dataProduto = [];
        var dataReceita = [];
        dataset.forEach(function(rows) {
            if (rows.ID == "S") {
                let row = {
                    "CDSUBGRU": rows.SUBGRU,
                    "CDRECEITA": rows.CODIGO,
                    "QTRECSUB": InflightorGeneral.valueToFloat(rows.QTD),
                    "__is_new": false,
                    "changed": false
                };
                dataReceita.push(row);
            } else {
                let row = {
                    "CDSUBGRU": rows.SUBGRU,
                    "CDPRODUTO": rows.CODIGO,
                    "QTPRODSU": InflightorGeneral.valueToFloat(rows.QTD),
                    "__is_new": false,
                    "changed": false
                };
                dataProduto.push(row);
            }
        });
        if (dataProduto.length)
            parent.removeProduto(widget, dataProduto);
        if (dataReceita.length)
            parent.removeReceita(widget, dataReceita);
    };

    this.removeReceita = function(widget, data) {
        coc01410_cadastrosubgrupoService.removeReceita(data).then(function() {

            widget.container.getWidget("composicaoprodutos").reload().then(function() {
                // ScreenService.closeSwipe();
                widget.container.getWidget("composicaoprodutos").activate();
                parent.setDataSourceForGrid(widget.container.getWidget("cadastrosubgrupo"));
            });
        });

    };
    this.removeProduto = function(widget, data) {

        coc01410_cadastrosubgrupoService.removeProduto(data).then(function() {

            widget.container.getWidget("composicaoprodutos").reload().then(function() {
                // ScreenService.closeSwipe();
                widget.container.getWidget("composicaoprodutos").activate();
                parent.setDataSourceForGrid(widget.container.getWidget("cadastrosubgrupo"));
            });
        });

    };
    this.newProduto = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.IDATIVO == "N") {
                return MessageService.showMessageByCode("INACTIVE_PRODUCT");
            } else {
                var widgetComposicao = widget.container.getWidget("composicaoprodutos");
                var CDSUBGRU = widget.container.getWidget("cadastrosubgrupo").currentRow.CDSUBGRU;
                widgetComposicao.currentRow = widget.currentRow;
                var row = {
                    "CDSUBGRU": CDSUBGRU,
                    "CDPRODUTO": widgetComposicao.currentRow.CDPRODUTO,
                    "QTPRODSU": InflightorGeneral.valueToFloat(widgetComposicao.currentRow.QTPRODSU),
                    "__is_new": widget.currentRow.__is_new,
                    "changed": widget.currentRow.changed
                };
                coc01410_cadastrosubgrupoService.saveProduto(row).then(function() {
                    if (widgetComposicao.currentRow.__is_new) {
                        widgetComposicao.list(true);
                        ScreenService.closePopup();
                        parent.setDataSourceForGrid(widget.container.getWidget("cadastrosubgrupo")); //reload
                    } else {
                        widget.commitInlineEdit();
                        widgetComposicao.list();
                    }
                    widgetComposicao.activate();
                }, function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    ScreenService.showMessage(errorMessage);
                });
                widgetComposicao.dataSource.name = "";
                /*
                InflightorGeneral.saveCrud(widgetComposicao).then(function(){
                    
                    
                    widgetComposicao.activate();
                });
                */
            }
        }
    };

    this.newReceita = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.IDATIVO == "N") {
                return MessageService.showMessageByCode("INACTIVE_RECIPE");
            } else {
                var widgetComposicao = widget.container.getWidget("composicaoprodutos");
                var CDSUBGRU = widget.container.getWidget("cadastrosubgrupo").currentRow.CDSUBGRU;
                widgetComposicao.currentRow = widget.currentRow;
                widgetComposicao.currentRow.CDSUBGRU = CDSUBGRU;
                widgetComposicao.currentRow.QTRECSUB = InflightorGeneral.valueToFloat(widget.currentRow.QTRECSUB);
                var row = {
                    "CDSUBGRU": CDSUBGRU,
                    "CDRECEITA": widgetComposicao.currentRow.CDRECEITA,
                    "QTRECSUB": InflightorGeneral.valueToFloat(widgetComposicao.currentRow.QTRECSUB),
                    "__is_new": widget.currentRow.__is_new,
                    "changed": widget.currentRow.changed
                };
                coc01410_cadastrosubgrupoService.saveReceita(row).then(function() {
                    if (widgetComposicao.currentRow.__is_new) {
                        widgetComposicao.list(true);
                        ScreenService.closePopup();
                        parent.setDataSourceForGrid(widget.container.getWidget("cadastrosubgrupo")); //reload
                    } else {
                        widgetComposicao.commitInlineEdit();
                        widgetComposicao.list();
                    }
                    widgetComposicao.activate();
                }, function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                    }
                    ScreenService.showMessage(errorMessage);
                });
                widgetComposicao.dataSource.name = "";
                /*
                InflightorGeneral.saveCrud(widgetComposicao).then(function(){
                    
                    
                    widgetComposicao.activate();
                });
                */
            }
        }
    };

    this.selectInativoProduto = function(field) {
        if (field.widget.currentRow.IDPRODATIVPR == "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
            field.widget.currentRow.CDPRODUTO = "";
            field.widget.currentRow.IDPRODATIVPR = "";
            field.widget.currentRow.QTPRODSU = "";
            field.widget.currentRow.SGUNIDADE = "";
            field.widget.currentRow.NMPRODUTO = "";
        }
    };

    this.selectInativo = function(field) {
        if (field.widget.currentRow.IDATIVO == "N") {
            MessageService.showMessageByCode("INACTIVE_RECIPE");
            field.widget.currentRow.DSRECEITA = "";
            field.widget.currentRow.IDATIVO = "";
            field.widget.currentRow.QTRECSUB = "";
            field.widget.currentRow.SGUNIDADE = "";

        }

    };

    this.aplicaStylesGridSelect = function(fieldSelect) {
        fieldSelect.selectWidget.styles = [
            {
                condition: "row.IDATIVO==='N'",
                cssClass: "coc-inactive"
            }
        ];
        fieldSelect.selectWidget.reload();
    };

    this.aplicaStylesGridSelectProduto = function(fieldSelect) {
        fieldSelect.selectWidget.styles = [
            {
                condition: "row.IDPRODATIVPR==='N'",
                cssClass: "coc-inactive"
            }
        ];
        fieldSelect.selectWidget.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc01410_cadastrosubgrupoController", coc01410_cadastrosubgrupoController);
});

// FILE: ./assets/js/controllers/coc03300_manutencao_custo_produto_controller.js
var coc03300_manutencao_custo_produto_controller = function(ScreenService, InflightorGeneral, ManutencaoCustoProdutoService, restEngine, MessageService, templateManager, ZhFilterPreferences) {

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.limpaDatasourceData = function(widget) {
        var filtroFilialCia = filtroPreenchido(widget.dataSourceFilter);
        if (!filtroFilialCia.CDFILIAL || !filtroFilialCia.CDCIA) {
            widget.dataSource.data = [];
        }
    };

    var filtroPreenchido = function(dataSourceFilter) {
        var filialCompanhia = {
            CDFILIAL: false,
            CDCIA: false
        };
        dataSourceFilter.forEach(function(parametro) {
            if (parametro.name === "CDFILIAL") {
                filialCompanhia.CDFILIAL = temParametro(parametro, "name", "CDFILIAL");
            }
            if (parametro.name === "CDCIA") {
                filialCompanhia.CDCIA = temParametro(parametro, "name", "CDCIA");
            }
        });
        return filialCompanhia;
    };

    var temParametro = function(parametro, propriedade, valor) {
        return parametro.hasOwnProperty(propriedade) && parametro[propriedade] === valor;
    };

    this.corrigeFiltro = function(args) {
        var filterRow = args.e.preference.FILTER_ROW || {};
        var cia = "";

        if (filterRow.TCIA === "S" && filterRow.IDCUSPAD !== "P") {
            filterRow.CDCIA = "T";
        } else {
            if (Array.isArray(filterRow.CDCIA)) {
                cia = filterRow.CDCIA.join('_');
                filterRow.CDCIA = cia;
            }
        }

        delete filterRow.SM_NMCIA;
        delete filterRow.TCIA;

    };

    this.convertStringToArrayCdCia = function(args) {
        var filter = args.e.preference.FILTER_ROW;
        if (filter) {
            filter.CDCIA = filter.CDCIA.split("_");
        }
    };

    this.mudaVisibilidadeEFilter = function(field, fieldDependente, filtros) {

        var currentRow = field.widget.currentRow;

        if (currentRow[field.name]) {
            // habilitaField(fieldDependente, true);
            setaDatasourceFiltro(fieldDependente, filtros, true);
        } else {
            // habilitaField(fieldDependente, false);
            setaDatasourceFiltro(fieldDependente, filtros, false);
        }

    };

    var setaDatasourceFiltro = function(field, filtros, aplicaFiltro) {

        var filtroDataSource = [];
        field.value("");

        filtros.forEach(function(filtro) {
            let filtroObj = {
                "name": filtro,
                "operator": "=",
                "value": (aplicaFiltro) ? field.widget.currentRow[filtro] : null
            };
            let index;
            if (Array.isArray(field.dataSourceFilter)) {
                index = field.dataSourceFilter.findIndex(function(f) {
                    return f.name == filtro;
                });
            } else {
                field.dataSourceFilter = [];
            }
            if (field.dataSourceFilter[index]) {
                field.dataSourceFilter[index].value = filtroObj.value;
            } else {
                field.dataSourceFilter.push(filtroObj);
            }
        });
        if (!aplicaFiltro)
            field.value("");
    };

    this.saveProduto = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                this.saveNewProduto(widget);
            } else {
                this.salvaAlteracaoValores(widget);
            }
        }
    };

    this.salvaAlteracaoValores = function(widget) {
        if (widget.open) {
            this.saveNewProduto(widget);
        } else {
            widget.dataSource.data.forEach(function(row) {
                row.VRMEDBRUAP = row.hasOwnProperty("VRMEDBRUAP") && (!!row.VRMEDBRUAP) ? InflightorGeneral.valueToFloat(row.VRMEDBRUAP) : row.VRMEDBRUAP;
                row.VRMEDLIQAP = row.hasOwnProperty("VRMEDLIQAP") && (!!row.VRMEDLIQAP) ? InflightorGeneral.valueToFloat(row.VRMEDLIQAP) : row.VRMEDLIQAP;
                row.VRULTCLIQAP = row.hasOwnProperty("VRULTCLIQAP") && (!!row.VRULTCLIQAP) ? InflightorGeneral.valueToFloat(row.VRULTCLIQAP) : row.VRULTCLIQAP;
                row.VRULTCBRUAP = row.hasOwnProperty("VRULTCBRUAP") && (!!row.VRULTCBRUAP) ? InflightorGeneral.valueToFloat(row.VRULTCBRUAP) : row.VRULTCBRUAP;
            });

            ManutencaoCustoProdutoService.salvaManutencao(widget.dataSource.data).then(function() {
                widget.list();
                if (widget.mode == "list") {
                    widget.reload();
                }
            }, function(error) {
                var mensagemDeErro = error.data.dataset.errors[0].message;
                let erro = mensagemDeErro.split("#_#");
                MessageService.showMessageByCode(erro[0], {
                    'NMPRODUTO': erro[1]
                });
                widget.list();
                widget.reload();
            });
        }

    };

    this.saveNewProduto = function(widget) {
        if ((widget.currentRow.CUSTO_PADRAO == 'C') && (widget.currentRow.IDATIVO_CIA !== 'S')) {
            MessageService.showMessageByCode('INACTIVE_CIA');
            return;
        }
        if (widget.currentRow.IDSITUACAO === "S") {
            var dataSourceName = getDataSourceName(widget.currentRow);
            var row = widget.currentRow;
            var filterRow = widget.appliedFilter.FILTER_ROW;
            row.CDFILIAL = filterRow.CDFILIAL;
            row.CDOPIMPCUSTO = getLocalVar("CDOPERADOR");
            row.DTHRIMPCUSTO = moment().format("DD/MM/YYYY HH:mm:ss");
            row.DTCUSTOAP = filterRow.DTCUSTOAP;
            row.VRMEDBRUAP = row.VRMEDBRUAP ? InflightorGeneral.valueToFloat(row.VRMEDBRUAP) : 0;
            row.VRMEDLIQAP = row.VRMEDLIQAP ? InflightorGeneral.valueToFloat(row.VRMEDLIQAP) : 0;
            row.VRULTCLIQAP = row.VRULTCLIQAP ? InflightorGeneral.valueToFloat(row.VRULTCLIQAP) : 0;
            row.VRULTCBRUAP = row.VRULTCBRUAP ? InflightorGeneral.valueToFloat(row.VRULTCBRUAP) : 0;
            InflightorGeneral.callRestEngine("coc#/crud/" + dataSourceName + "/save", "Row", row).then(function() {
                widget.reload();
                closeSwipeNewProduto(widget);
            }, function() {
                MessageService.showMessageByCode("COC03300_UNIT_REF_OBRIGATORIOS");
                widget.list();
                widget.reload();
            });
        } else {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        }
    };

    this.enableCia = function(field) {
        var widget = field.widget;
        var fieldCia = widget.getField("NMCIA");
        var fieldSMCia = widget.getField("SM_NMCIA") || widget.getField("CDCIA");
        var fieldTCia = widget.getField("TCIA");
        var fieldProd = widget.getField("NMPRODUTO");
        if (field.value() == 'P')
            this.loadProdutosAdd();
        //fieldProd.clearValue();
        if (field.value() == "C" || field.value() == "T") {
            fieldCia.readOnly = false;
            fieldTCia.readOnly = false;
            fieldCia.validations = {
                required: true
            };
            fieldSMCia.readOnly = false;
            fieldSMCia.validations.required = true;
            fieldProd.readOnly = true;
            fieldProd.validations = {};
            if (fieldTCia.checked) {
                fieldSMCia.readOnly = true;
                fieldSMCia.clearValue();
                delete fieldSMCia.validations.required;
            }
        } else {
            fieldCia.readOnly = true;
            fieldSMCia.clearValue();
            fieldSMCia.readOnly = true;
            delete fieldSMCia.validations.required;
            fieldTCia.readOnly = true;
            fieldCia.clearValue();
            fieldCia.validations = {};
            fieldProd.readOnly = false;
            fieldProd.validations = {
                required: true
            };
        }

    };

    this.disableRequiredCompanhia = function(widget) {
        var fieldCompanhia = widget.getField('NMCIA');
        var fieldProduto = widget.getField("NMPRODUTO");
        if (fieldCompanhia.validations && fieldCompanhia.validations.required) {
            delete widget.getField('NMCIA').validations.required;
        }
        if (fieldProduto.validations) {
            widget.getField("NMPRODUTO").validations.required = true;
        }
    };

    function getDataSourceName(row) {
        var dataSourceName;
        if (row.SGCIA.length == 0 || row.SGCIA == "PADRAO") {
            dataSourceName = "produtoap";
        } else {
            dataSourceName = "produtoapcia";
        }
        return dataSourceName;
    }

    function closeSwipeNewProduto(widget) {
        ScreenService.closeSwipe().then(
            function() {
                widget.reload();
                widget.list();
                widget.activate();
            }
        );
    }

    this.delete = function(widget) {
        var row = widget.currentRow;
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question",
            function() {
                var dataSourceName = getDataSourceName(row);
                InflightorGeneral.callRestEngine("coc#/crud/" + dataSourceName + "/remove", "Row", row).then(
                    function() {
                        closeSwipeNewProduto(widget);
                    }.bind(this),
                    InflightorGeneral.callbackError
                );
            }.bind(this)
        );
    };

    this.cancelInlineEdit = function(widget) {
        if (widget.open) {
            ScreenService.closeSwipe().then(
                function() {
                    widget.dataSource.data.pop();
                    widget.list();
                }
            );
        } else {
            widget.rollbackInlineEdit();
            widget.list();
        }
    };

    this.applyFilter = function(field) {
        if (field.widget.appliedFilter) {
            field.widget.currentRow.NMCIA = "";
            field.value("");
            var filial = field.widget.appliedFilter.FILTER_ROW.CDFILIAL || field.widget.currentRow.CDFILIAL;
            var filtro = {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            };
            let filtroFilial;
            if (Array.isArray(field.dataSourceFilter)) {
                filtroFilial = field.dataSourceFilter.find(function(f) {
                    return f.name == 'CDFILIAL';
                });
            } else {
                field.dataSourceFilter = [];
            }

            if (filtroFilial) {
                filtroFilial.value = filtro.value;
            } else {
                field.dataSourceFilter.push(filtro);
            }
        }
    };

    this.loadProdutosAdd = function() {
        let widget = templateManager.container.getWidget('manutencaocustoproduto');
        let field = widget.getField('NMPRODUTO');
        var filterRow = widget.appliedFilter.FILTER_ROW;

        var filtroFilial = {
            "name": "CDFILIAL",
            "operator": "=",
            "value": filterRow.CDFILIAL
        };

        var filtroDtCusto = {
            "name": "DTCUSTOAP",
            "operator": "=",
            "value": filterRow.DTCUSTOAP
        };

        var filtroCia = {
            "name": "CDCIA",
            "operator": "=",
            "value": widget.currentRow.CDCIA
        };

        let fF;
        let fD;
        let fC;
        if (Array.isArray(field.dataSourceFilter)) {
            fF = field.dataSourceFilter.find(function(f) {
                return f.name == 'CDFILIAL';
            });
            fD = field.dataSourceFilter.find(function(f) {
                return f.name == 'DTCUSTOAP';
            });
            fC = field.dataSourceFilter.findIndex(function(f) {
                return f.name == 'CDCIA';
            });
        } else {
            field.dataSourceFilter = [];
        }

        if (fF) {
            fF.value = filtroFilial.value;
        } else {
            field.dataSourceFilter.push(filtroFilial);
        }

        if (fD) {
            fD.value = filtroDtCusto.value;
        } else {
            field.dataSourceFilter.push(filtroDtCusto);
        }

        if (widget.currentRow.CUSTO_PADRAO === "P") {
            field.dataSource.name = "coc#/crud/produto_produtoap";
            if (field.dataSourceFilter[fC]) {
                field.dataSourceFilter.splice(fC, 1);
            }
        } else {
            field.dataSource.name = "coc#/crud/produto_produtoapcia";
            if (!field.dataSourceFilter[fC]) {
                field.dataSourceFilter.push(filtroCia);
            } else {
                field.dataSourceFilter[fC].value = filtroCia.value;
            }

        }
    };

    this.enableProd = function(field) {
        var widget = field.widget;
        var fieldProd = widget.getField("NMPRODUTO");
        fieldProd.clearValue();
        if (widget.appliedFilter)
            this.loadProdutosAdd();
        if (field.value() == "") {
            fieldProd.readOnly = true;
            delete fieldProd.validations.required;
        } else {
            fieldProd.readOnly = false;
            fieldProd.validations.required = true;
        }
    };

    this.newProd = function(widget) {
        if (!widget.appliedFilter) {
            MessageService.showMessageByCode('COC03300_FILTRO_N_INF');
            return;
        }
        widget.newRow();
        this.enableCia(templateManager.container.getWidget('manutencaocustoproduto').getField('CUSTO_PADRAO'));
        this.applyFilter(widget.getField("NMCIA"));
        widget.getField("NMCIA").readOnly = true;
        widget.getField("NMPRODUTO").readOnly = false;
    };

    this.applyInactiveStyle = function(field, setEvent = true) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        if (setEvent)
            field.selectWidget.events[0].code = "coc03300_manutencao_custo_produto_controller.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.applyInactiveStyleProduto = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc03300_manutencao_custo_produto_controller.inactiveBranchWarningProduto(args);";
    };

    this.inactiveBranchWarningProduto = function(args) {
        if (args.owner.currentRow.IDPRODATIVPR === "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.abreFiltro = function(args) {
        let widget = templateManager.container.getWidget('manutencaocustoproduto');
        if (!widget.appliedFilter) {
            let row = {
                CDFILIAL: getLocalVar('CDFILIAL'),
                NMFILIAL: getLocalVar('NMFILIAL'),
                IDCUSPAD: "T",
                TCIA: 'S',
                DTCUSTOAP: moment().format('DD/MM/YYYY')
            };
            ZhFilterPreferences.openFilterWidget(widget, row);
        } else {
            ZhFilterPreferences.openFilterWidget(widget);
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03300_manutencao_custo_produto_controller", coc03300_manutencao_custo_produto_controller);
});

// FILE: ./assets/js/controllers/coc03301_relatorio_custo_produto_controller.js
var coc03301_relatorio_custo_produto_controller = function(InflightorGeneral, ScreenService, templateManager, restEngine, MessageService) {

    var self = this;

    this.onEnter = function(widget) {
        widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
        self.montaFiltroDependente(widget.getField('NMFILIAL'), ['CDFILIAL'], ['NMCIA']);
    };

    this.montaFiltroDependente = function(field, colunas, camposDependentes) {
        var currentRow = field.widget.currentRow;
        if (currentRow[field.name] != "") {
            for (let coluna in colunas) {
                var filtro = {
                    "name": colunas[coluna],
                    "operator": "=",
                    "value": currentRow[colunas[coluna]]
                };
                for (let campo in camposDependentes) {
                    let fieldDependente = field.widget.getField(camposDependentes[campo]);
                    let dtsFilterOriginal = fieldDependente.dataSourceFilter;
                    fieldDependente.dataSourceFilter = dtsFilterOriginal.filter(
                        function(condicao) {
                            return condicao.name != colunas[coluna];
                        }
                    );
                    fieldDependente.value();
                    currentRow[camposDependentes[campo]] = null;
                    fieldDependente.dataSourceFilter.push(filtro);
                    fieldDependente.reload();
                }
            }
        } else {
            for (let coluna in colunas) {
                for (let campo in camposDependentes) {
                    let fieldDependente = field.widget.getField(camposDependentes[campo]);
                    let dtsFilterOriginal = fieldDependente.dataSourceFilter;
                    fieldDependente.dataSourceFilter = dtsFilterOriginal.filter(
                        function(condicao) {
                            return condicao.name != colunas[coluna];
                        }
                    );
                    fieldDependente.value();
                    currentRow[camposDependentes[campo]] = null;
                }
            }
        }
    };

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.verificaFiltroCia = function(args, filtros) {
        var field = args.owner.field;
        var filtrosValidados = 0;
        for (let condicao in field.dataSourceFilter) {
            if (validaFiltro(field.dataSourceFilter[condicao], filtros))
                filtrosValidados++;
        }
        if (filtrosValidados < 1) {
            MessageService.showMessageByCode("FILL_UNIT");
            args.e.preventDefault();
        }
    };

    var validaFiltro = function(condicao, filtros) {
        for (let filtro in filtros) {
            if (condicao.name == filtros[filtro] && condicao.value != "" && condicao.value.length > 0)
                return true;
        }
    };

    this.relatorio = function(widget, formato) {
        if (widget.isValid()) {
            var row = ajustaRowParaRelatorio(widget.currentRow, formato);
            InflightorGeneral.openReport("coc#/relatoriocustoproduto", row.REPORTNAME, row).then(function() {
                ScreenService.closePopup();
                widget.activate();
                widget.reload();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data)
                    errorMessage = error.data.error;
                ScreenService.showMessage(errorMessage);
            });
        }
    };

    var ajustaRowParaRelatorio = function(row, formato) {

        var newRow = angular.copy(row);

        newRow.REPORTFORMAT = formato;

        if (row.TNMCIA == "T") {
            newRow.CDCIA = row.TNMCIA;
        } else {
            newRow.CDCIA = "_" + row.NMCIA.join("_") + "_";
        }

        if (row.TPRODUTO == "T") {
            newRow.CDPRODUTO = row.TPRODUTO;
        } else {
            newRow.CDPRODUTO = "_" + row.NMPRODUTO.join("_") + "_";
        }

        return newRow;

    };

    this.carregaProduto = function(field) {
        var currentRow = field.widget.currentRow;
        var parametros = {
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB"),
            "IDATIVO": "S",
            "IDPRODATIVPR": currentRow.DSPRODATIVPR ? currentRow.DSPRODATIVPR : "T"
        };

        InflightorGeneral.onClickDependentField(field, parametros);
    };

    this.habilitaCompanhia = function(field) {
        var fieldCia = field.widget.getField("NMCIA");
        var fieldTCia = field.widget.getField("TNMCIA");
        var currentRow = field.widget.currentRow;

        if (currentRow.IDCUSPAD === "P") {
            fieldTCia.readOnly = true;
            fieldTCia.setValue("T");
            currentRow.TNMCIA = "T";

            fieldCia.readOnly = true;
            fieldCia.setValue("");
            fieldCia.validations = {};
        } else {
            fieldTCia.readOnly = false;
        }

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03301_relatorio_custo_produto_controller", coc03301_relatorio_custo_produto_controller);
});

// FILE: ./assets/js/controllers/coc03500_historico_de_receitasController.js
var coc03500_historico_de_receitasController = function(InflightorGeneral, ScreenService, templateManager, restEngine, COCUtil, MessageService) {

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        COCUtil.updateFilterFields(fieldCia, 'CDFILIAL', rowFilter);
        COCUtil.blockDependentField(fieldUnit, 'NMCIA');
        COCUtil.clearFields(widget, 'NMCIA');
        this.onChangeCia(fieldCia);
    };

    this.onChangeCia = function(fieldCia) {
        const widget = fieldCia.widget;
        const fieldManual = widget.getField('DSMANUAL');
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });
        COCUtil.updateFilterFields(fieldManual, ['CDFILIAL', 'CDCIA'], rowFilter);
        COCUtil.blockDependentField(fieldCia, 'DSMANUAL');
        let fieldDependent = widget.getField('DSMANUAL');
        fieldDependent.setValue("");
        this.onChangeManual(fieldManual);
    };

    this.onChangeManual = function(fieldManual) {
        const widget = fieldManual.widget;
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA'
        });
        const fieldAndCheckbox = {
            'IDSERVICOCLASSE': 'TIDSERVICOCLASSE',
            'DSSUBGRU': 'TSUBGRU',
            'IDCICLO': 'TIDCICLO',
            'IT': 'TNRITAIRVISION'
        };
        const dependentFields = Object.keys(fieldAndCheckbox);
        COCUtil.updateFilterFields(dependentFields, ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowFilter, 'IDSERVICOCLASSE', widget);
        COCUtil.blockDependentField(fieldManual, dependentFields, true, false);
        widget.getField('IDSERVICOCLASSE').setValue("");
        widget.getField('DSSUBGRU').setValue("");
        widget.getField('IDCICLO').setValue("");
        widget.getField('IT').setValue("");
        widget.getField('TSUBGRU').setValue("T");
        widget.getField('TIDCICLO').setValue("T");
        widget.getField('TIDSERVICOCLASSE').setValue("T");
        widget.getField('TNRITAIRVISION').setValue("T");
    };

    this.fieldOnChangeServico = function(field) {

        var row = field.widget.currentRow;

        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            }
        ];

        field.dataSourceFilter = filtro;
        field.reload();

    };

    this.fieldOnChangeDssubgru = function(field) { //FOO

        var rowsSelecionadas = field.dataSource.getCheckedRows();
        var servicoClasse = [];
        for (let i in rowsSelecionadas) {
            servicoClasse.push(rowsSelecionadas[i].NRSERVICOCIAFILI + "-" + rowsSelecionadas[i].CDCLASSE);
        }
        var servicoClasseConcatenado = servicoClasse.join("_");
        if (servicoClasseConcatenado) {
            var filtro = field.dataSourceFilter;
            filtro.push({
                "name": "SERVICOCLASSE",
                "operator": "=",
                "value": servicoClasseConcatenado
            });
            field.widget.getField("DSSUBGRU").dataSourceFilter = filtro;
            field.reload();
        }
    };

    this.setValoresDefault = function(widget) {
        widget.currentRow.INFLACAO = 0;
        widget.currentRow.CDTIPOCUSTO = 3;
    };

    this.habilitaMoeda = function(field) {
        if (field.widget.currentRow.CDCIA && field.widget.currentRow.CDCIA !== "") {
            setMoedaPadrao(field.widget.currentRow, field.widget.currentRow.CDFILIAL, field.widget.currentRow.CDCIA);
            field.widget.getField("NMINDICE").readOnly = false;
        } else {
            field.widget.getField("NMINDICE").value("");
            field.widget.getField("NMINDICE").readOnly = true;
        }
    };

    var setMoedaPadrao = function(row, cdfilial, cdcia) {
        restEngine.post("coc#/crud/ciafili_indice", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": cdcia
                }
            ]
        }).then(function(callback) {
            if (callback.dataset.ciafili_indice.length > 0) {
                row.NMINDICE = callback.dataset.ciafili_indice[0].NMINDICE;
                row.CDINDICE = callback.dataset.ciafili_indice[0].CDINDICE;
            }
        });
    };

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        args.owner.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
    };

    this.limpaFieldMulti = function(widget, listaFieldName) {
        for (let fieldName in listaFieldName) {
            widget.getField(fieldName).setValue("");
            widget.getField(fieldName).readOnly = true;
            widget.getField(fieldName).validations = {};
            widget.getField(listaFieldName[fieldName]).checked = true;
            widget.getField(listaFieldName[fieldName]).setValue("T");
        }
    };

    this.report = function(widget, format) {

        if (widget.isValid()) {

            if (widget.currentRow.TSUBGRU != "T") {
                var subgrupos = listaSubgrupos(widget.getField("DSSUBGRU"));
                widget.currentRow.CDSUBGRU = subgrupos;
            } else {
                widget.currentRow.CDSUBGRU = "T";
            }

            if (widget.currentRow.TIDSERVICOCLASSE != "T") {
                widget.currentRow.SERVCLASSE = widget.currentRow.IDSERVICOCLASSE.join("_");
            } else {
                widget.currentRow.SERVCLASSE = "T";
            }

            if (widget.currentRow.TIDCICLO != "T") {
                widget.currentRow.CDCICLO = widget.currentRow.IDCICLO.join("_");
            } else {
                widget.currentRow.CDCICLO = "T";
            }
            if (widget.currentRow.TNRITAIRVISION != "T") {
                var rowIt = [];
                widget.currentRow.IT.forEach(function(row) {
                    rowIt.push(row.NRITAIRVISION + "|" + row.CDSUBGRU + "|" + row.NRSEQSUBGRUSEMAM + "|" + row.CDCICLO);
                });
                widget.currentRow.NRITAIRVISION = rowIt.join("_");
                widget.currentRow.FILTERIT = "T";
            } else {
                widget.currentRow.NRITAIRVISION = "T";
                widget.currentRow.FILTERIT = "F";
            }

            var dtCusto = widget.currentRow.DTCUSTOAP;
            dtCusto = dtCusto.split(" ");
            widget.currentRow.DTCUSTOAP = dtCusto[0];
            widget.currentRow.REPORTFORMAT = format;
            widget.currentRow.IDIOMA = "P";

            InflightorGeneral.openReport("coc#/historicoReceita", null, widget.currentRow).then(function() {
                ScreenService.closePopup();
                widget.activate();
                widget.reload();
            }, function(error) {
                if (error.data) {
                    ScreenService.showMessage(error.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        }
    };

    this.FieldBeforeSelectOpenDssubgru = function(args) {
        var filtro = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: args.row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: args.row.CDMANCIA
            }
        ];
        if (args.row.CDFILIAL == "" || typeof args.row.CDFILIAL === "undefined") {
            args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [];
            args.owner.owner.widget.getField("DSSUBGRU").reload();
            MessageService.showMessageByCode("MSG_COC_ESCOLHA_UNIDADE_SUBGRUPO");
            args.e.preventDefault();
            return;
        }
        if (args.row.CDCIA == "" || typeof args.row.CDCIA === "undefined") {
            args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [];
            args.owner.owner.widget.getField("DSSUBGRU").reload();
            MessageService.showMessageByCode("MSG_COC_ESCOLHA_CIA_SUBGRUPO");
            args.e.preventDefault();
            return;
        }
        if (args.row.CDMANCIA == "" || typeof args.row.CDMANCIA === "undefined") {
            args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [];
            args.owner.owner.widget.getField("DSSUBGRU").reload();
            MessageService.showMessageByCode("MSG_COC_MANUAL_SUBGRUPO");
            args.e.preventDefault();
            return;
        }
        if (args.row.TIDSERVICOCLASSE != "T") {
            if (args.row.IDSERVICOCLASSE == "") {
                args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [];
                args.owner.owner.widget.getField("DSSUBGRU").reload();
                MessageService.showMessageByCode("MSG_COC_ESCOLHA_SERVICO_CLASSE_SUBGRUPO");
                args.e.preventDefault();
                return;
            } else {
                filtro.push({
                    name: "NRSERVICOCLASSE",
                    operator: "IN",
                    value: args.row.IDSERVICOCLASSE
                });

            }
        }
        args.owner.field.dataSourceFilter = filtro;
        args.owner.field.reload();
    };

    var listaSubgrupos = function(field) {

        var lista = "";
        var rows = field.selectWidget.getCheckedRows();

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i].CDSUBGRU + "|" + rows[i].NRSEQSUBGRUSEMAM;
            } else {
                lista = lista + "_" + rows[i].CDSUBGRU + "|" + rows[i].NRSEQSUBGRUSEMAM;
            }
        }
        return lista;
    };

    this.filtraCiclo = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "T";

        if (currentRow.TSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = listaSubgrupos(field.widget.getField("DSSUBGRU"));
            } else {
                cdsubgru = "";
            }
        }

        var filtro = {
            "CDSUBGRU": cdsubgru,
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
        field.reload();
    };

    this.filtraIT = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "";
        var cdciclo = "";
        var dsservico = "";
        var cdclasse = "";

        var filtro = {
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        if (currentRow.TSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = "";
                field.widget.container.getWidget("coc03500_historico_de_receitas").getField("DSSUBGRU").selectWidget.getCheckedRows().forEach(function(subgru) {
                    cdsubgru += subgru.CDSUBGRU + "|" + subgru.NRSEQSUBGRUSEMAM + "_";
                });
            } else {
                cdsubgru = null;
            }
        }

        if (currentRow.TIDCICLO != "T") {
            if (currentRow.IDCICLO) {
                cdciclo = "";
                field.widget.container.getWidget("coc03500_historico_de_receitas").getField("IDCICLO").selectWidget.getCheckedRows().forEach(function(ciclo) {
                    cdciclo += ciclo.CDCICLO + "_";
                });
            } else {
                cdciclo = null;
            }
        }

        if (currentRow.TIDSERVICOCLASSE != "T") {
            if (currentRow.IDSERVICOCLASSE) {
                dsservico = "";
                cdclasse = "";
                field.widget.container.getWidget("coc03500_historico_de_receitas").getField("IDSERVICOCLASSE").selectWidget.getCheckedRows().forEach(function(servico) {
                    dsservico += servico.NRSERVICOCIAFILI + "_";
                    cdclasse += servico.CDCLASSE + "_";
                });
            } else {
                dsservico = null;
                cdclasse = null;
            }
        }

        filtro.CDCICLO = cdciclo || "T";
        filtro.CDSUBGRU = cdsubgru || "T";
        filtro.NRSERVICOCIAFILI = dsservico || "T";
        filtro.CDCLASSE = cdclasse || "T";

        InflightorGeneral.onClickDependentField(field, filtro);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03500_historico_de_receitasController", coc03500_historico_de_receitasController);
});

// FILE: ./assets/js/controllers/coc03600_menu_designController.js
var coc03600_menu_designController = function(FieldUtil, ScreenService, InflightorGeneral, ZHPromise, coc01001_servico_companhiaService, JSONContainerLoader, templateManager, coc03600_menu_designService, maskEngine, ZhUtil, coc01400_receitaService, restEngine, ZhFilterPreferences, metaDataFactory, MessageService) {
    var self = this;
    var maskAtual;
    var IDUTIAIRVISION;
    var menuGridPrimaryKeys = ["NRORG", "CDMANCIA", "CDCIA", "CDFILIAL"];
    var itemCiaPrimaryKeys = ["CDFILIAL", "CDCIA", "CDMANCIA", "NRSERVICOCIAFILI", "CDCLASSE", "CDSUBGRU", "NRSEQSUBGRUSEMAM", "CDPRODUTO"];
    var workflowItens = {
        NRPROCESSO: "4",
        MG_ALTERA_REPROVADO_DESENVOLVIMENTO: "2",
        MG_ALTERA_NOME: "3",
        MG_ALTERA_DATA_APRESENTACAO: "4",
        MG_ALTERA_LOCAL_APRESENTACAO: "5",
        SERVICO_INCLUSAO: "6",
        SERVICO_EXCLUSAO: "7",
        SUBGRUPO_ALTERA_VIGENCIA: "8",
        SUBGRUPO_ALTERA_PROPORCAO: "9",
        ITEMCIA_ALTERACAO_QTD: "10",
        ITEMCIA_INCLUSAO: "11",
        ITEMCIA_EXCLUSAO: "12",
        SUBGRUPO_INCLUSAO: "13",
        SUBGRUPO_EXCLUSAO: "14",
        ALTERA_INSTRUCAO_MONTAGEM: "15",
        PRODUTO_INCLUSAO: "16",
        PRODUTO_EXCLUSAO: "17",
        PRODUTO_ALTERACAO_QTD: "18",
        RECEITA_INCLUSAO: "19",
        RECEITA_EXCLUSAO: "20",
        RECEITA_ALTERACAO_QTD: "21",
        RECEITA_ALTERACAO_GRAMATURA: "22",
        APROVA_MENU_GRID: "23",
        MG_ALTERA_VIGENCIA_MANUAL: "25"
    };

    var filterCdfilial = null;
    var filterNmfilial = null;
    var filterCdcia = null;
    var filterNmcia = null;

    this.onChangeUnit = function(args) {
        const fieldUnit = args.owner.field;
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        fieldCia.clearValue();
        let filterUnit = fieldCia.dataSourceFilter.find(filter => filter.name === 'CDFILIAL');
        const CDFILIAL = fieldUnit.value() ? widget.currentRow.CDFILIAL : '';
        if (filterUnit) {
            filterUnit.value = CDFILIAL;
        }
    };

    this.verifyInactiveCia = function(fieldCia) {
        const widget = fieldCia.widget;
        const row = widget.currentRow;
        if (row.IDATIVO === 'N') {
            fieldCia.clearValue();
            MessageService.showMessageByCode("INACTIVE_CIA");
        }
    };

    this.clearCheckedRowsCiclo = function(widgetSubgrupo) {
        if (widgetSubgrupo.lastActiveWidget.name === widgetSubgrupo.name || widgetSubgrupo.container.getActiveWidget().name !== "widgetSubgrupo") {
            let parentWidget = widgetSubgrupo.getParent();
            let widgetCiclo = parentWidget.widgets.filter(
                function(widget) {
                    if (widget.name === "widgetCiclo") {
                        return widget;
                    }
                }
            );
            if (widgetCiclo[0]) {
                widgetCiclo[0].dataSource.clearCheckedRows();
            }
        }
    };

    this.isReturningFromReports = function(mainWidget) {
        return mainWidget.appliedFilter;
    };

    this.openFilterOnEnter = function(mainWidget) {
        if (!self.isReturningFromReports(mainWidget)) {
            let fieldToApplyDefaultValue = ['NMFILIAL', 'CDFILIAL', 'IDVIGENCIA', 'IDSITUAPROPCIA'];
            InflightorGeneral.openFilterOnEnter(mainWidget, fieldToApplyDefaultValue);
        }
    };

    this.verifyFilterWidgetOnEnter = function(mainWidget) {
        if (self.isReturningFromReports(mainWidget)) {
            filterCdcia = null;
            mainWidget.dataSourceFilter = [];
            mainWidget.dataSource.data = [];
            let filterWidget = mainWidget.getParent().getWidget('FILTER');
            if (filterWidget.fields) {
                filterWidget.fields.forEach(
                    function(field) {
                        field.label = "";
                    }
                );
            }
            ZhFilterPreferences.openFilterWidget(mainWidget, mainWidget.appliedFilter.FILTER_ROW);
        }
    };

    this.importarConfiguracao = function(widget) {

        var widgetImportarConfiguracaoCiclo = widget.container.getWidget("importarConfiguracaoCiclo");

        widgetImportarConfiguracaoCiclo.newRow();

        var widgetManual = widget.container.getWidget("grid_manual");
        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        var widgetServicos = widget.container.getWidget("grid_servicos");
        var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");

        widgetImportarConfiguracaoCiclo.currentRow.NMFILIALDES = appliedFilter.FILTER_ROW.NMFILIAL;
        widgetImportarConfiguracaoCiclo.currentRow.NMCIADES = appliedFilter.FILTER_ROW.NMCIA;
        widgetImportarConfiguracaoCiclo.currentRow.DSMANUALDES = widgetManual.currentRow.DSMANUAL;
        widgetImportarConfiguracaoCiclo.currentRow.SERVICO_CLASSEDES = widgetServicos.currentRow.DSSERVICOCIAFILI;
        widgetImportarConfiguracaoCiclo.currentRow.DSSUBGRUDES = widgetSubgrupo.currentRow.DSSUBGRU;
        widgetImportarConfiguracaoCiclo.currentRow.IDCICLODES = widget.currentRow.IDCICLO;

        ScreenService.openPopup(widgetImportarConfiguracaoCiclo);

    };

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.refresh = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.reload([], false).then(function() {
            self.ordenaCiclo(widget);
        });
    };

    function compare(a, b) {
        return a.IDCICLO - b.IDCICLO;
    }

    this.ordenaCiclo = function(widget) {
        widget.dataSource.data.sort(compare);
    };

    this.preparaCampoCiclo = function(widget) {
        var itens = [];

        widget.dataSource.data.forEach(function(row) {
            if (row.CDCICLO)
                itens.push(row.CDCICLO);
        });

        if (widget.dataSource.data.length > 1) {
            var filtro = {
                "name": "CDCICLO",
                "operator": "NOT_IN",
                "value": itens
            };
            widget.getField("IDCICLO").dataSourceFilter.push(filtro);
        }
        widget.getField("IDCICLO").reload();

    };

    this.callReportPopup = function(currentWidget, widgetLevel) {
        widgetLevel = ScreenService.getSwipeLevel(currentWidget);
        var data = {};
        if (filterCdcia) {
            data = {
                CDFILIAL: filterCdfilial,
                NMFILIAL: filterNmfilial,
                CDCIA: filterCdcia,
                NMCIA: filterNmcia
            };
            if (widgetLevel > -1) {
                data.CDMANCIA = currentWidget.container.getWidget("grid_manual").currentRow.CDMANCIA;
                data.DSMANUAL = currentWidget.container.getWidget("grid_manual").currentRow.DSMANUAL;
            }
            window.coc03602_params = data;
            ScreenService.openWindow("coc#coc03602_relatorios_do_manual");
        } else {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        }

    };

    this.calculaCustoMenuGrid = function(widgetCalculaCusto) {
        ScreenService.openPopup(widgetCalculaCusto);
        widgetCalculaCusto.newRow();
        widgetCalculaCusto.activate();
    };

    function getSituacao(widget, showMessage) {
        if (coc03600_menu_designService.getWorkflowPendenteAprovacaoMenuGrid()) {
            if (showMessage)
                MessageService.showMessageByCode("COC03600_MENU_GRID_APPROVE_PROCESS");
            return false;
        }
        return widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA;
    }

    //Possibilita a adição de novos serviços na própria tela de menu design
    //utilizando o fieldFactory
    this.adicionarServicoCiaFili = function(widget, field, widget_adicionaServ) {
        // ScreenService.closePopup();
        widget_adicionaServ.fields = [];
        JSONContainerLoader.loadContainer("coc#coc01001_servico_companhia").then(function(data) {
            // Caso queira dar load na widget inteira.
            // widgetServ = metaDataFactory.widgetFactory(data.widgets[0], templateManager.container);
            data.widgets[1].fields.forEach(function(field) {
                if (field.readOnlyOnEdit !== undefined && field.readOnlyOnEdit == true)
                    field.readOnlyOnEdit = false;
                metaDataFactory.fieldFactory(field, widget_adicionaServ, true);
            });
            updateFieldsNewService(widget_adicionaServ, widget);
        });
        ScreenService.openPopup(widget_adicionaServ);
    };

    var updateFieldsNewService = function(widget_adicionaServ, widget) {
        widget_adicionaServ.getField("IDTIPOSERVICO").dataSource.data = [
            {
                "label": "Item Menu Grid",
                "value": "ITEMMENUGRID"
            },
            {
                "label": "Ambos",
                "value": "AMBOS"
            }
        ];
        var fieldProduto = widget_adicionaServ.getField("NMPRODUTO");
        fieldProduto.widget.dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: widget.currentRow.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: widget.currentRow.CDCIA
            }
        ];
    };

    this.salvarSubGrup = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            row.CDARVPROD = "";
            row.DSSUBGRUES = "";
            row.CDSETOR = "";
            row.DSSUBGRUIN = "";
            row.__is_new = true;
            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                row.__using_unit = paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 10;
            }
            coc03600_menu_designService.saveSubgrupom([widget.currentRow]).then(
                function success() {
                    ScreenService.successNotification(ScreenService.i18n("Subgrupo salvo com sucesso!"));
                    widget.fields.forEach(function(field) {
                        field.clearValue();
                    });
                    ScreenService.getPopupOpen().lastActiveWidget = widget.container.getWidget("grid_subgrupos");
                    ScreenService.closePopup();
                    widget.container.getWidget("zh_crosstab").reload().then(function() {
                        widget.container.getWidget("zh_crosstab").activate();
                    });
                    widget.container.getWidget("widgetSubgrupo").getField("DSSUBGRU").reload();
                },
                function error(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente");
                        ScreenService.showMessage(msg);
                    }
                }
            );
        }
    };

    this.salvarServicoCiaFili = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            row.CDFILIAL = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
            row.CDCIA = widget.container.getWidget("grid_manual").currentRow.CDCIA;
            if (row.TXCOMENTASE == undefined)
                row.TXCOMENTASE = "";
            row.__is_new = true;
            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                row.__using_unit = paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
            }

            coc03600_menu_designService.saveServicociafili([widget.currentRow]).then(
                function success() {
                    ScreenService.closePopup();
                    widget.fields.forEach(function(field) {
                        field.clearValue();
                    });
                    ScreenService.successNotification(ScreenService.i18n("Serviço salvo com sucesso!"));
                    widget.parent.activate();
                    templateManager.updateTemplate();
                    widget.container.getWidget("grid_servicos").getField("DSSERVICOCIAFILI").reload();
                },
                function error(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente");
                        ScreenService.showMessage(msg);
                    }
                }
            );
        } else {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        }
    };

    this.adicionarServCiclo = function(widget, widgetCiclo) {
        if (widget.currentRow.NRSERVICOCIAFILI) {
            ScreenService.openPopup(widgetCiclo);
            //CDFILIAL
            widgetCiclo.getField("IDCICLO").dataSourceFilter[1].value = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
            widgetCiclo.getField("DSCLASSE").dataSourceFilter[1].value = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
            //CDCIA
            widgetCiclo.getField("IDCICLO").dataSourceFilter[2].value = widget.container.getWidget("grid_manual").currentRow.CDCIA;
            widgetCiclo.getField("DSCLASSE").dataSourceFilter[2].value = widget.container.getWidget("grid_manual").currentRow.CDCIA;
        } else {
            let msg = ScreenService.i18n("Escolha um Serviço para adicionar Classe/Ciclo!");
            ScreenService.showMessage(msg);
        }
    };

    //Cadastro automatico de ciclos dentro da adição de serviços
    this.saveCadAutoCiclos = function(widget) {
        if (widget.isValid()) {
            widget.getField("IDCICLO").selectWidget.getCheckedRows().forEach(function(row) {
                row.CDFILIAL = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
                row.CDCIA = widget.container.getWidget("grid_manual").currentRow.CDCIA;
                row.CDCLASSE = widget.currentRow.CDCLASSE;
                row.NRSERVICOCIAFILI = widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI;
                row.CDSERVICO = row.CDCICLO;
                row.CDARVSERV = row.CDCICLO;
                row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                row.IDATIVO = "S";
                row.__is_new = true;
            });

            var sendd = widget.getField("IDCICLO").selectWidget.getCheckedRows();
            coc01001_servico_companhiaService.cadAutoCiclos(sendd).then(
                function success() {
                    ScreenService.successNotification(ScreenService.i18n("Salvo com sucesso!"));
                    widget.fields.forEach(function(field) {
                        field.clearValue();
                    });
                    ScreenService.getPopupOpen().lastActiveWidget = widget.container.getWidget("grid_servicos");
                    ScreenService.closePopup();
                    widget.container.getWidget("grid_servicos").activate();
                    widget.container.getWidget("grid_servicos").getField("DSCLASSE").reload();
                },
                function error(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                }
            );
        }
    };

    this.voltarAdicaoSubGrup = function(widget) {
        widget.fields.forEach(function(field) {
            field.clearValue();
        });
        ScreenService.getPopupOpen().lastActiveWidget = widget.container.getWidget("widgetSubgrupo");
        ScreenService.closePopup();
        widget.container.getWidget("widgetSubgrupo").activate();
    };

    //Usa o metadataFactory para buildar os campos do cadastro de subgrupos
    this.adicionarSubGrup = function(widget, field, widget_adicionaSubGrup) {
        // ScreenService.closePopup();
        widget_adicionaSubGrup.fields = [];
        JSONContainerLoader.loadContainer("coc#coc01410_cadastrosubgrupo").then(function(data) {
            // Caso queira dar load na widget inteira.
            // widgetServ = metaDataFactory.widgetFactory(data.widgets[0], templateManager.container);
            data.widgets[1].fields.forEach(function(field) {
                metaDataFactory.fieldFactory(field, widget_adicionaSubGrup, true);
            });
        });
        ScreenService.openPopup(widget_adicionaSubGrup);
    };

    this.voltarAdicaoServCiaFili = function(widget) {
        widget.fields.forEach(function(field) {
            field.clearValue();
        });
        ScreenService.getPopupOpen().lastActiveWidget = widget.container.getWidget("grid_servicos");
        ScreenService.closePopup();
        widget.container.getWidget("grid_servicos").activate();
    };

    this.voltarAdicaoSubPrin = function(widget) {
        widget.fields.forEach(function(field) {
            field.clearValue();
        });
        ScreenService.getPopupOpen().lastActiveWidget = widget.container.getWidget("grid_subgruposPrincipais");
        ScreenService.closePopup();
        widget.container.getWidget("grid_subgruposPrincipais").activate();
    };

    function confirmaExclusao(widget) {
        coc03600_menu_designService.verificaTabelaPreco(widget.currentRow).then(function(tabelaPreco) {
            if (tabelaPreco.dataset.verifica_menugrid_tabelapreco.length === 0) {
                coc03600_menu_designService.deleteManual([widget.currentRow]).then(function() {
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                        widget.activate();
                    });
                }, function(error) {
                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                        MessageService.showMessageByCode("DELETE_ERROR");
                    } else {
                        ScreenService.showMessage(error.data.error);
                    }
                });
            } else {
                var mensagem = [];
                tabelaPreco.dataset.verifica_menugrid_tabelapreco.forEach(function(tabela) {
                    mensagem.push("A tabela de preço '" + tabela.NRTABPRECO + " - " + tabela.NMTABPRECO + "' está vinculada ao Menu Grid.");
                });
                InflightorGeneral.showErrorMessages(widget, mensagem, "Exclusão não permitida");
            }
        });
    }

    this.deleteManual = function(widget) {
        if (getSituacao(widget, true) == "P") {
            var row = retornaRowWorkflowPendenteMenuGrid(widget.currentRow, null, "1", null);
            coc03600_menu_designService.hasPendingWorkflow(row).then(function(qtd) {
                if (!workflowPendente(qtd, true)) {
                    ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                        confirmaExclusao(widget);
                    });
                }
            });
        } else if (getSituacao(widget, true) == "E") {
            ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                confirmaExclusao(widget);
            });
        } else {
            let msg = "Operação Bloqueada. Menu grid está inativo ou foi aprovado/reprovado.";
            ScreenService.showMessage(msg, "error");
        }
    };

    //Preenche as Labels de subgrupo, sendo que:
    //1 - 1 :
    //2 - não definido
    //3 - %
    //4 - 1 x
    this.afterLoadSubgrupos = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            switch (row.IDPROPSUBSEMAM) {
                case "1":
                    row.QTPROPSUBSEMAM = "0";
                    break;
                case "2":
                    row.QTPROPSUBSEMAM = Util.applyMask(row.QTPROPSUBSEMAM, {
                        "type": "currency",
                        "params": {
                            "symbol": "1:",
                            "thousands": "",
                            "decimal": ",",
                            "precision": "0"
                        }
                    }, maskEngine, ZhUtil);
                    break;
                case "3":
                    row.QTPROPSUBSEMAM = "0";
                    break;
                case "4":
                    row.QTPROPSUBSEMAM = Util.applyMask(row.QTPROPSUBSEMAM, {
                        "type": "currency",
                        "params": {
                            "symbol": "",
                            "thousands": "",
                            "decimal": ",",
                            "precision": "2"
                        }
                    }, maskEngine, ZhUtil);
                    break;
                case "5":
                    row.QTPROPSUBSEMAM = Util.applyMask(row.QTPROPSUBSEMAM, {
                        "type": "currency",
                        "params": {
                            "symbol": "",
                            "thousands": "",
                            "decimal": ",",
                            "precision": "0"
                        }
                    }, maskEngine, ZhUtil);
                    break;
            }
        });
    };

    //Salva um novo manual
    this.saveMenuDesign = function(widget) {
        if (widget.currentRow.__is_new) {
            if (!filterCdcia) {
                MessageService.showMessageByCode("COC03600_CHOOSE_CIA");
            } else {
                if (moment(widget.currentRow.DTINIVGMA, "DD/MM/YYYY") > moment(widget.currentRow.DTFIMVGMA, "DD/MM/YYYY")) {
                    MessageService.showMessageByCode("COC03600_DT_INI_FIN");
                } else {
                    //Apos verificar a situação do manual, entre no processo de save
                    if (widget.isValid()) {
                        widget.currentRow.CDFILIAL = filterCdfilial;
                        widget.currentRow.CDCIA = filterCdcia;
                        widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                        //Monta e salva a row
                        var dataR = [];
                        dataR.push({
                            CDFILIAL: widget.currentRow.CDFILIAL,
                            CDCIA: widget.currentRow.CDCIA,
                            DTINIVGMA: widget.currentRow.DTINIVGMA,
                            DTFIMVGMA: widget.currentRow.DTFIMVGMA,
                            DSMANUAL: widget.currentRow.DSMANUAL,
                            DTCUSTOMAN: widget.currentRow.DTCUSTOMAN,
                            CDMANCIABKP: widget.currentRow.CDMANCIABKP,
                            CDMANCIA: widget.currentRow.CDMANCIA,
                            VRCUSTOMAM: widget.currentRow.VRCUSTOMAM,
                            DTAPRESENTACIA: widget.currentRow.DTAPRESENTACIA,
                            DSLOCALAPCIA: widget.currentRow.DSLOCALAPCIA,
                            NRTIPONEGOCIOINF: widget.currentRow.NRTIPONEGOCIOINF,
                            __is_new: true,
                            NRORG: widget.currentRow.NRORG
                        });
                        coc03600_menu_designService.saveManual(dataR).then(function() {
                            widget.container.getWidget("grid_servicos").isVisible = true;
                            widget.list(true);
                            ScreenService.goBack();
                            widget.reload();
                            widget.activate();
                        }, function(error) {
                            var errorMessage = "Timeout";
                            if (error.data) {
                                errorMessage = error.data.error;
                            }
                            ScreenService.showMessage(errorMessage);
                        });
                    }
                }
            }
        } else {
            let row = widget.currentRow;
            row.DTINIVGMA = row.DTINIVGMA ? row.DTINIVGMA.split(' ')[0] : row.DTINIVGMA;
            row.DTFIMVGMA = row.DTFIMVGMA ? row.DTFIMVGMA.split(' ')[0] : row.DTFIMVGMA;
            //Tratamento de edição
            //Apos verificar a situação do manual, entre no processo de save
            if (widget.isValid()) {
                //verifica de o field de data de vigência foi alterado
                if (moment(widget.currentRow.DTINIVGMA, "DD/MM/YYYY") > moment(widget.currentRow.DTFIMVGMA, "DD/MM/YYYY")) {
                    MessageService.showMessageByCode("COC03600_DT_INI_FIN");
                } else {
                    if (validateMGAprove(widget)) {
                        if (getSituacao(widget, true) == "P") {
                            if (nomeDataLocalAlterado(widget)) {
                                self.solicitaCriacaoWorkflow(widget, {}, callbackWFNomeDataLocal);
                            } else {
                                salvaEdicaoManual(widget);
                            }
                        } else if (getSituacao(widget, true) == "E") {
                            salvaEdicaoManual(widget);
                        } else if (getSituacao(widget, true) == "A") {
                            // "value": currentWidgetMenuGrid.currentRow.DTINIVGMA
                            // "value": currentWidgetMenuGrid.currentRow.DTFIMVGMA
                            if (validateMGAprove(widget)) {
                                InflightorGeneral.callRestEngine("coc#/coc03600_save_menu", "Row", widget.currentRow).then(
                                    function(callback) {
                                        var data = callback.dataset;
                                        if (data && data.errorMessage) {
                                            InflightorGeneral.showErrorMessages(widget, data.errorMessage, "Subgrupos com datas sobrepostas");
                                        } else {
                                            alteraCampoVigencia(widget.currentRow);
                                            widget.container.getWidget("grid_servicos").isVisible = true;
                                            widget.view();
                                            ScreenService.closePopup(true);
                                        }
                                    }.bind(this), InflightorGeneral.callbackError
                                );
                                // salvaEdicaoManual(widget);
                            }
                        }

                    }
                }
            }
        }
    };

    function validateMGAprove(widget) {
        var row = widget.currentRow;
        var backupRow = widget.backupRow;

        var datasPreenhidas = row.DTINIVGMA && row.DTFIMVGMA;

        var msg;
        if (datasPreenhidas) {
            var dtfim = moment(row.DTFIMVGMA.substr(0, 10), "DD/MM/YYYY");
            var dtini = moment(row.DTINIVGMA.substr(0, 10), "DD/MM/YYYY");
            var backupDtIni = backupRow.DTINIVGMA ? backupRow.DTINIVGMA.substr(0, 10) : backupRow.DTINIVGMA;
            var backupDtFim = backupRow.DTFIMVGMA ? backupRow.DTFIMVGMA.substr(0, 10) : backupRow.DTFIMVGMA;
            var changeDtini = (row.DTINIVGMA.substr(0, 10) != backupDtIni);
            var changeDtfim = (row.DTFIMVGMA.substr(0, 10) != backupDtFim);
            var valid = dtini <= dtfim;
            if (!valid) {
                msg = "Data inicial menor que a data final.";
            } else if (changeDtini && valid && (dtini <= moment([]))) {
                msg = "A data incial deve ser maior que o dia de hoje.";
            } else if (changeDtfim && valid && (dtfim <= moment([]))) {
                msg = "A data final deve ser maior que o dia de hoje.";
            }
        } else if (!datasPreenhidas && getSituacao(widget, true) === "A") {
            msg = "Os campos de vigência devem ser preenchidos";
        }

        if (msg) {
            ScreenService.showMessage("Operação bloqueada!<br>" + msg);
            return false;
        } else {
            return true;
        }

    }

    function salvaEdicaoManual(widget) {
        return coc03600_menu_designService.saveManualEdit(widget.currentRow).then(
            function(callback) {
                var data = callback.dataset;
                if (data && data.errorMessage) {
                    InflightorGeneral.showErrorMessages(widget, data.errorMessage, "Subgrupos com datas sobrepostas");
                } else {
                    alteraCampoVigencia(widget.currentRow);
                    widget.container.getWidget("grid_servicos").isVisible = true;
                    widget.view();
                    ScreenService.closePopup(true);
                    widget.reload();
                }
            }, InflightorGeneral.callbackError
        );
    }

    function callbackWFNomeDataLocal(widget) {
        var parentWidget = widget.parentWidget;
        var promises = [];
        var dtfimvgma = parentWidget.currentRow.DTFIMVGMA ? parentWidget.currentRow.DTFIMVGMA : "";
        var dtinivgma = parentWidget.currentRow.DTINIVGMA ? parentWidget.currentRow.DTINIVGMA : "";
        var dtfimvgmabkp = parentWidget.backupRow.DTFIMVGMA ? parentWidget.backupRow.DTFIMVGMA : "";
        var dtinivgmabkp = parentWidget.backupRow.DTINIVGMA ? parentWidget.backupRow.DTINIVGMA : "";
        parentWidget.currentRow.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        if (parentWidget.usaWorkflow.NOME && parentWidget.currentRow.DSMANUAL != parentWidget.backupRow.DSMANUAL) {
            let rowToSave = retornaRowSalvarWFMG(widget.parentWidget, ["DSMANUAL"], workflowItens.MG_ALTERA_NOME);
            parentWidget.currentRow.DSMANUAL = parentWidget.backupRow.DSMANUAL;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        if (parentWidget.usaWorkflow.DATA && moment(parentWidget.currentRow.DTAPRESENTACIA, "DD/MM/YYYY").format("L") != moment(parentWidget.backupRow.DTAPRESENTACIA, "DD/MM/YYYY").format("L")) {
            let rowToSave = retornaRowSalvarWFMG(widget.parentWidget, ["DTAPRESENTACIA"], workflowItens.MG_ALTERA_DATA_APRESENTACAO);
            parentWidget.currentRow.DTAPRESENTACIA = parentWidget.backupRow.DTAPRESENTACIA;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        if (parentWidget.usaWorkflow.LOCAL && parentWidget.currentRow.DSLOCALAPCIA != parentWidget.backupRow.DSLOCALAPCIA) {
            let rowToSave = retornaRowSalvarWFMG(widget.parentWidget, ["DSLOCALAPCIA"], workflowItens.MG_ALTERA_LOCAL_APRESENTACAO);
            parentWidget.currentRow.DSLOCALAPCIA = parentWidget.backupRow.DSLOCALAPCIA;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        if (parentWidget.usaWorkflow.VIGENCIA && (dtfimvgma.substr(0, 10) != dtfimvgmabkp.substr(0, 10) || dtinivgma.substr(0, 10) != dtinivgmabkp.substr(0, 10))) {
            var changedItems = [];
            if (dtfimvgma.substr(0, 10) != dtfimvgmabkp.substr(0, 10)) {
                changedItems.push("DTFIMVGMA");
            }
            if (dtinivgma.substr(0, 10) != dtinivgmabkp.substr(0, 10)) {
                changedItems.push("DTINIVGMA");
            }
            let rowToSave = retornaRowSalvarWFMG(widget.parentWidget, changedItems, workflowItens.MG_ALTERA_VIGENCIA_MANUAL);
            dtinivgma = dtinivgmabkp;
            dtfimvgma = dtfimvgmabkp;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }

        if (promises.length) {
            ZHPromise.all(promises).then(function(callback) {
                salvaEdicaoManual(parentWidget).then(function() {
                    ScreenService.showMessage(callback[0].messages[0].message);
                    ScreenService.closePopup();
                    parentWidget.reload();
                });
            }, function(err) {
                if (err.data) {
                    ScreenService.showMessage(err.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        } else {
            salvaEdicaoManual(parentWidget);
        }
    }

    function nomeDataLocalAlterado(widget) {
        var nomeAlteado = widget.currentRow.DSMANUAL != widget.backupRow.DSMANUAL && widget.usaWorkflow.NOME;
        var dataAlteado = moment(widget.currentRow.DTAPRESENTACIA, "DD/MM/YYYY").format("L") != moment(widget.backupRow.DTAPRESENTACIA, "DD/MM/YYYY").format("L") && widget.usaWorkflow.DATA;
        var localAlteado = widget.currentRow.DSLOCALAPCIA != widget.backupRow.DSLOCALAPCIA && widget.usaWorkflow.LOCAL;
        var vigenciaAlterada = (widget.currentRow.DTINIVGMA != widget.backupRow.DTINIVGMA || widget.currentRow.DTFIMVGMA != widget.backupRow.DTFIMVGMA) && widget.usaWorkflow.VIGENCIA;
        return nomeAlteado || dataAlteado || localAlteado || vigenciaAlterada;
    }

    function retornaRowSalvarWFMG(widget, itensAlterados, NRITEMPROCAPROVA) {
        var tableName = "MANUALCIAM";
        var itemReferencia = "DSMANUAL";
        var primaryKeys = menuGridPrimaryKeys;
        var IDTIPOPERWFLOW = "1";
        var rowToSave = retornaRowSalvarWF(widget, tableName, NRITEMPROCAPROVA, itemReferencia, itensAlterados, primaryKeys, IDTIPOPERWFLOW);
        var itemManual = widget.backupRow && widget.backupRow[itemReferencia] ? widget.backupRow[itemReferencia] : widget.currentRow[itemReferencia];
        rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + widget.currentRow.NMCIA + " \nMenu Grid: " + itemManual;
        return rowToSave;
    }

    function copyRow(row) {
        var newRow = {};
        var itensIgnorados = ["__leftActions", "__rightActions"];
        for (let item in row) {
            if (!itemIgnorado(item, itensIgnorados))
                newRow[item] = row[item];
        }
        return newRow;
    }

    function itemIgnorado(item, itensIgnorados) {
        var itemEncontrado = itensIgnorados.indexOf(item);
        return itemEncontrado == -1 ? false : true;
    }

    function retornaRowSalvarWF(widget, tableName, NRITEMPROCESSO, NMITEM, CHANGEDITEMS, PRIMARYKEYS, IDTIPOPERWFLOW) {
        var rowToSave = copyRow(widget.currentRow);
        rowToSave.NRPROCESSOAPROVA = workflowItens.NRPROCESSO;
        rowToSave.NRITEMPROCAPROVA = NRITEMPROCESSO;
        rowToSave.NMITEM = widget.backupRow && widget.backupRow[NMITEM] ? widget.backupRow[NMITEM] : widget.currentRow[NMITEM];
        rowToSave.NMTABELA = tableName;
        rowToSave.IDTIPOPERWFLOW = IDTIPOPERWFLOW;
        rowToSave.CHANGEDITEMS = retornaItensPorRow(CHANGEDITEMS, rowToSave);
        rowToSave.PRIMARYKEYS = retornaItensPorRow(PRIMARYKEYS, rowToSave);
        return rowToSave;
    }

    function retornaItensPorRow(CHANGEDITEMS, row) {
        var itens = {};
        CHANGEDITEMS.forEach(function(item) {
            itens[item] = row[item];
        });
        return itens;
    }

    function alteraCampoVigencia(currentRow) {
        if (!currentRow.DTINIVGMA || !currentRow.DTFIMVGMA) {
            currentRow.VIGENCIA = "";
        } else if (moment(currentRow.DTFIMVGMA, "DD/MM/YYYY HH:mm:ss").isBefore(moment())) {
            currentRow.VIGENCIA = "NÃO VIGENTE";
        } else {
            currentRow.VIGENCIA = "VIGENTE";
        }
    }

    this.addManual = function(widget) {
        if (filterCdcia) {
            if (widget.getField('DTINIVGMA').validations) {
                delete widget.getField('DTINIVGMA').validations.required;
            }
            if (widget.getField('DTFIMVGMA').validations) {
                delete widget.getField('DTFIMVGMA').validations.required;
            }
            widget.container.getWidget("grid_servicos").isVisible = false;
            widget.edit();
            widget.activate();
            widget.newRow();
        } else {
            MessageService.showMessageByCode("COC03600_APPLY_FILTER");
        }
    };

    //cuida dos subgrupos e da proporção
    this.alteraProporcao = function(widget) {
        var fieldConfiguracao = widget.getField("QTPROPSUBSEMAM");

        fieldConfiguracao.template = "field/text-edit.html";
        fieldConfiguracao.label = "Configuração";
        fieldConfiguracao.isVisible = true;

        var novoValor = valorSemMask(fieldConfiguracao.value(), maskAtual);
        //DEFINE A NOVA MASK APENAS APÓS O RETORNO DA FUNCAO valorSemMask
        maskAtual = widget.currentRow.IDPROPSUBSEMAM;

        switch (maskAtual) {
            case "2":
                fieldConfiguracao.label = "Configuração (V)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "0"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                fieldConfiguracao.isVisible = true;
                break;
            case "3":
                novoValor = null;
                fieldConfiguracao.isVisible = false;
                break;
            case "4":
                fieldConfiguracao.label = "Configuração (%)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "2"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                break;
            case "5":
                fieldConfiguracao.label = "Configuração (S)";
                fieldConfiguracao.mask = {
                    "type": "currency",
                    "params": {
                        "symbol": "",
                        "thousands": "",
                        "decimal": ",",
                        "precision": "0"
                    }
                };
                novoValor = Util.applyMask(novoValor, fieldConfiguracao.mask, maskEngine, ZhUtil);
                break;
            default:
                fieldConfiguracao.isVisible = false;
        }

        var novoField = metaDataFactory.fieldFactory(fieldConfiguracao, widget);
        novoField.setValue(novoValor);
        widget.fields.splice(widget.fields.indexOf(fieldConfiguracao), 1, novoField);

        ScreenService.updateTemplate();
    };

    var valorSemMask = function(valor, mask) {
        if (!valor || valor === "0") {
            return "0";
        } else {
            return (mask == "3") ? 1 : valor;
        }
    };

    this.saveSubPrin = function(widget) {
        if (widget.isValid()) {
            var rows = widget.getField("DSSUBGRUESC").selectWidget.getCheckedRows();
            rows.forEach(function(row) {
                row.CDSUBGRUPRIN = row.CDSUBGRU;
                row.CDCICLO = widget.container.getWidget("widgetCiclo").currentRow.CDCICLO;
                row.NRSEQSUBGRUSEMAMPRIN = row.NRSEQSUBGRUSEMAM;
                row.CDSUBGRU = widget.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU;
                row.NRSEQSUBGRUSEMAM = widget.container.getWidget("widgetSubgrupo").currentRow.NRSEQSUBGRUSEMAM;
                row.__is_new = true;
            });
            coc03600_menu_designService.salvaSubPrin(rows).then(function() {
                ScreenService.closePopup();
                widget.container.getWidget("grid_subgruposPrincipais").activate();
                widget.container.getWidget("grid_subgruposPrincipais").reload();
            }, function(error) {
                if (error.data) {
                    let msg = ScreenService.i18n(error.data.error);
                    ScreenService.showMessage(msg);
                } else {
                    let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                    ScreenService.showMessage(msg);
                }
            });
        }
    };

    this.adicionaSubPrin = function(widget) {
        widget.container.getWidget("cad_autoSubGruPrin").getField("DSSUBGRUPRIN").defaultValue = widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU;
        widget.container.getWidget("cad_autoSubGruPrin").newRow();
        ScreenService.openPopup(widget.container.getWidget("cad_autoSubGruPrin"));
        widget.container.getWidget("cad_autoSubGruPrin").activate();
    };

    function alteraActionsParaAprovacao(widget) {
        switch (widget.currentRow.IDSITUAPROPCIA) {
            case "A":
                widget.getAction("propostaApr").isVisible = false;
                widget.getAction("aprovarApr").isVisible = false;
                widget.getAction("reprovarApr").isVisible = false;
                widget.getAction("desenvApr").isVisible = false;
                break;
            case "R":
                widget.getAction("propostaApr").isVisible = false;
                widget.getAction("aprovarApr").isVisible = false;
                widget.getAction("reprovarApr").isVisible = false;
                widget.getAction("desenvApr").isVisible = true;
                break;
            case "E":
                widget.getAction("propostaApr").isVisible = true;
                widget.getAction("aprovarApr").isVisible = false;
                widget.getAction("reprovarApr").isVisible = false;
                widget.getAction("desenvApr").isVisible = false;
                break;
            case "P":
                widget.getAction("propostaApr").isVisible = false;
                widget.getAction("aprovarApr").isVisible = true;
                widget.getAction("reprovarApr").isVisible = true;
                widget.getAction("desenvApr").isVisible = false;
                break;
        }
    }

    function alteraValidationDataVigencia(id, widget) {
        var validations = {
            required: true
        };
        var fields = ["DTINIVGMA", "DTFIMVGMA"];
        if (desenvolvimentoOuProposta(id, false)) {
            validations = null;
        }
        fields.forEach(function(name) {
            widget.getField(name).validations = validations;
        });
    }

    //Prepara a view seguida do afterMoveRow
    this.preparaView = function(widget) {

        if (widget.name == "grid_manual") {
            var appliedFilter = widget.appliedFilter || {};
            appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
            if (typeof(appliedFilter.FILTER_ROW) == "string") {
                appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
            }

            if (!widget.currentRow.__is_new) {
                widget.currentRow.CDFILIAL = appliedFilter.FILTER_ROW.CDFILIAL;
                var widgetServico = widget.container.getWidget("grid_servicos");

                widgetServico.isVisible = true;

                //aprovação/reprovação
                alteraActionsParaAprovacao(widget);
                alteraValidationDataVigencia(widget.currentRow.IDSITUAPROPCIA, widget);

                //Cuida da parte de filtros da aba de serviço
                InflightorGeneral.buildDataSourceFilter(widgetServico.dataSourceFilter, widget.currentRow);

                //seta os default values para agilizar no processo de save
                widgetServico.getField("CDMANCIA").defaultValue = widget.currentRow.CDMANCIA;
                widgetServico.getField("CDFILIAL").defaultValue = widget.currentRow.CDFILIAL;
                widgetServico.getField("NRORG").defaultValue = widget.currentRow.NRORG;
                widgetServico.getField("CDCIA").defaultValue = widget.currentRow.CDCIA;

                //reload para aplicar os filtros
                widgetServico.reload();
                coc03600_menu_designService.retornaFlagWorkflowPendenteAprovacaoMenuGrid(widget.currentRow);

            } else {
                widget.getField("NMFILIAL").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
                widget.getField("NMCIA").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
            }
        } else if (widget.name == "grid_servicos") {
            if (widget.currentRow.__is_new) {
                widget.getField("CDMANCIA").defaultValue = widget.dataSourceFilter[1].value;
                widget.getField("DSSERVICOCIAFILI").dataSourceFilter[1].value = widget.dataSourceFilter[2].value;
                widget.getField("CDFILIAL").defaultValue = widget.dataSourceFilter[2].value;
                widget.getField("DSSERVICOCIAFILI").dataSourceFilter[2].value = widget.dataSourceFilter[3].value;
                widget.getField("CDCIA").defaultValue = widget.dataSourceFilter[3].value;
                widget.getField("DSCLASSE").dataSourceFilter[1].value = widget.dataSourceFilter[2].value;
                widget.getField("DSCLASSE").dataSourceFilter[2].value = widget.dataSourceFilter[3].value;
                widget.container.getWidget("zh_crosstab").isVisible = false;
                widget.container.getWidget("gris_subgrupo").isVisible = false;
            } else {
                var primaryKeys = {
                    CDFILIAL: "",
                    CDCIA: "",
                    CDMANCIA: "",
                    NRSERVICOCIAFILI: "",
                    CDCLASSE: "",
                    NRORG: ""
                };
                var dataFilter = retornaRowWorkflowPentende(widget.currentRow, "7", "1", "3", Object.keys(primaryKeys));
                coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
                    function(qtde) {
                        if (qtde) {
                            MessageService.showMessageByCode("COC03600_SERVICE_PENDENT");
                            widget.container.getWidget("zh_crosstab").isVisible = false;
                            widget.container.getWidget("gris_subgrupo").isVisible = false;
                        } else {
                            widget.container.getWidget("zh_crosstab").isVisible = true;
                            widget.container.getWidget("gris_subgrupo").isVisible = true;
                            ScreenService.openSwipe(widget.container.getWidget("tabbed_servicos")).then(function() {
                                setTimeout(function() {
                                    self.expandSwipe();
                                    templateManager.updateTemplate();
                                }, 100);
                            });
                        }
                    }
                );
            }
        } else if (widget.name == "widgetCiclo") {
            if (!widget.currentRow.__is_new) {
                InflightorGeneral.buildDataSourceFilter(widget.container.getWidget("grid_subgruposPrincipais").dataSourceFilter, widget.currentRow);
                widget.container.getWidget("grid_subgruposPrincipais").reload();
                self.carregaDataSourceComposicao(widget.container.getWidget("grid_composicao"));
                self.carregaMontasubsemam(widget);
                self.alteraItairvision(widget);
            }
        }
    };

    this.backSubgrupo = function(widget) {
        ScreenService.closeSwipe();
        widget.container.getWidget("grid_servicos").activate();
        widget.container.getWidget("grid_servicos").unSelectAll();
    };

    this.checkRowServico = function(widget) {
        if (widget.getCheckedRows().length == 1) {
            widget.getAction("deleteservico").isVisible = true;
            widget.getAction("excluir").isVisible = true;
            widget.currentRow = widget.getCheckedRows()[0];
        } else {
            widget.getAction("deleteservico").isVisible = false;
            widget.getAction("excluir").isVisible = false;
        }
    };

    this.alteraItairvision = function(widget) {
        var fieldItairvision = widget.getField("NRITAIRVISION");
        var visible = IDUTIAIRVISION == "S";
        fieldItairvision.isVisible = visible;
    };

    this.setDescriptionOrder = function(widget) {
        var appliedFilter = widget.appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }

        widget.currentRow.NMFILIAL = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.currentRow.NMCIA = appliedFilter.FILTER_ROW.NMCIA;
        widget.currentRow.SGCIA = appliedFilter.FILTER_ROW.SGCIA;
        widget.computeDescriptionOrder();
    };

    //utilizada para setar os filtros do field de Ciclos na adição de serviços ao manual
    this.FieldOnChangeCiclos = function(args) {
        if (args.row.NRSERVICOCIAFILI) {
            args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[3].value = args.row.NRSERVICOCIAFILI;
            args.owner.owner.widget.getField("DSCLASSE").reload();
        } else {
            args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[3].value = "";
            args.owner.owner.widget.getField("DSCLASSE").reload();
            let msg = ScreenService.i18n("Escolha um Serviço para definir a Classe!");
            ScreenService.showMessage(msg);
        }
    };

    this.FieldOnChangeCopia = function(field) {
        //Cuida da parte de filtros da aba de serviço
        var rowFilter = field.widget.container.getWidget("widgetSubgrupo").currentRow;
        InflightorGeneral.onClickDependentField(field, rowFilter);
    };

    function desenvolvimentoOuProposta(id, showMessage) {
        if (coc03600_menu_designService.getWorkflowPendenteAprovacaoMenuGrid()) {
            if (showMessage)
                MessageService.showMessageByCode("COC03600_MENU_GRID_APPROVE_PROCESS");
            return false;
        } else if (id != "E" && id != "P") {
            if (showMessage)
                MessageService.showMessageByCode("COC03600_MENU_GRID_NO_CHANGE");
            return false;
        }
        return (id == "E" || id == "P");
    }

    function desenvolvimento(id) {
        return id == "E";
    }

    this.FieldOnChangeCopiaDest = function(args) {
        if (args.row.CDCICLOORIGEM) {
            //Cuida da parte de filtros da aba de serviço
            args.owner.owner.widget.getField("IDCICLODEST").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("widgetSubgrupo").currentRow.CDFILIAL
            }, {
                name: "CDCIA",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("widgetSubgrupo").currentRow.CDCIA
            }, {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("widgetSubgrupo").currentRow.NRSERVICOCIAFILI
            }, {
                name: "CDCLASSE",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("widgetSubgrupo").currentRow.CDCLASSE
            }, {
                name: "CDCICLO",
                operator: "<>",
                value: args.owner.owner.widget.currentRow.CDCICLOORIGEM
            }];
            args.owner.owner.widget.getField("IDCICLODEST").reload();
        } else {
            args.owner.owner.widget.getField("IDCICLODEST").reload();
            let msg = ScreenService.i18n("Escolha um Ciclo de Origem para definir os de destino!");
            ScreenService.showMessage(msg);
        }
    };

    this.replicaCiclo = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            if (widget.isValid()) {
                ScreenService.confirmMessage(ScreenService.i18n("Confirma replicação de Ciclos? As informações dos Ciclos existentes serão sobrescritas"), "question", function() {
                    var subgrupoRow = widget.container.getWidget("widgetSubgrupo").currentRow;
                    widget.getField("IDCICLODEST").selectWidget.getCheckedRows().forEach(function(row) {
                        row.CDFILIALORI = subgrupoRow.CDFILIAL;
                        row.CDCIAORI = subgrupoRow.CDCIA;
                        row.CDMANCIAORI = subgrupoRow.CDMANCIA;
                        row.NRSERVICOCIAFILIORI = subgrupoRow.NRSERVICOCIAFILI;
                        row.CDCLASSEORI = subgrupoRow.CDCLASSE;
                        row.CDSUBGRUORI = subgrupoRow.CDSUBGRU;
                        row.NRSEQSUBGRUSEMAMORI = subgrupoRow.NRSEQSUBGRUSEMAM;
                        row.CDCICLOORIGEM = widget.currentRow.CDCICLOORIGEM;

                        row.CDFILIALDES = subgrupoRow.CDFILIAL;
                        row.CDCIADES = subgrupoRow.CDCIA;
                        row.CDMANCIADES = subgrupoRow.CDMANCIA;
                        row.NRSERVICOCIAFILIDES = subgrupoRow.NRSERVICOCIAFILI;
                        row.CDCLASSEDES = subgrupoRow.CDCLASSE;
                        row.CDSUBGRUDES = subgrupoRow.CDSUBGRU;
                        row.NRSEQSUBGRUSEMAMDES = subgrupoRow.NRSEQSUBGRUSEMAM;
                        row.CDCICLODESTINO = row.CDCICLO;

                        row.COPIA_IMAGEM = "S";
                        row.COPIA_IT = "N";
                        row.SOBRESCREVER = "S";
                    });

                    coc03600_menu_designService.copiaCiclos(widget.getField("IDCICLODEST").selectWidget.getCheckedRows()).then(function() {
                        ScreenService.closePopup(true);
                        widget.container.getWidget("widgetCiclo").reload();
                        widget.container.getWidget("widgetCiclo").activate();
                    }, function(error) {
                        if (error.data) {
                            let msg = ScreenService.i18n(error.data.error);
                            ScreenService.showMessage(msg);
                        } else {
                            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                            ScreenService.showMessage(msg);
                        }
                    });

                });
            }
        }
    };

    this.replicaMenu = function(widget) {
        if (widget.isValid()) {
            ScreenService.confirmMessage(ScreenService.i18n("Confirma replicação do Menu?"), "question", function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }
                widget.currentRow.P_CDFILIALORI = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
                widget.currentRow.P_CDCIAORI = widget.container.getWidget("grid_manual").currentRow.CDCIA;
                widget.currentRow.P_CDMANCIAORI = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
                widget.currentRow.P_CDFILIALDES = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
                widget.currentRow.P_CDCIADES = widget.container.getWidget("grid_manual").currentRow.CDCIA;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                coc03600_menu_designService.copiaMenu([widget.currentRow]).then(function(callback) {
                    ScreenService.closePopup(true);
                    widget.container.getWidget("grid_manual").reload();
                    widget.container.getWidget("grid_manual").activate();
                    if (callback.dataset !== undefined) {
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                    } else {
                        ScreenService.closeSwipe().then(function() {
                            MessageService.showMessageByCode("COC03600_MENU_GRID_REPLICATED_SUCCESS");
                        });
                    }
                }, function(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        MessageService.showMessageByCode("NO_CONNECTION");
                    }
                });

            });
        }
    };

    this.exportaMenu = function(widget) {
        if (widget.isValid()) {
            ScreenService.confirmMessage(ScreenService.i18n("Confirma exportação do Menu?"), "question", function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }
                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIALORI;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIAORI;
                widget.currentRow.P_CDMANCIAORI = widget.currentRow.CDMANCIAORI;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIA;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                coc03600_menu_designService.copiaMenu([widget.currentRow]).then(function(callback) {
                    ScreenService.closePopup(true);
                    widget.container.getWidget("grid_manual").reload();
                    widget.container.getWidget("grid_manual").activate();
                    if (callback.dataset !== undefined) {
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                    } else {
                        MessageService.showMessageByCode("COC03600_MENU_GRID_EXPORT_SUCCESS");
                    }
                }, function(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                });
            });
        }
    };

    this.importaMenu = function(widget) {
        if (widget.isValid()) {
            ScreenService.confirmMessage(ScreenService.i18n("Confirma importação do Menu?"), "question", function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }
                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIA;
                var cdmanciaori = "_";
                widget.currentRow.DSMANUAL.forEach(function(codigo) {
                    cdmanciaori += codigo + "_";
                });
                widget.currentRow.P_CDMANCIAORI = cdmanciaori;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIALDES;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIADES;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                coc03600_menu_designService.copiaMenu([widget.currentRow]).then(function(callback) {
                    ScreenService.closePopup(true);
                    widget.container.getWidget("grid_manual").reload();
                    widget.container.getWidget("grid_manual").activate();
                    if (callback.dataset !== undefined) {
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                    } else {
                        MessageService.showMessageByCode("COC03600_MENU_GRID_IMPORT_SUCCESS");
                    }
                }, function(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                });
            });
        }
    };

    //carrega field rich text -- JuliaoMontas
    this.carregaMontasubsemam = function(widget) {
        //widget.currentRow.DSMONTASUBSEMAMHTML = widget.currentRow.DSMONTASUBSEMAMHTML ? coc03600_menu_designService.htmlEntityDecode(widget.currentRow.DSMONTASUBSEMAMHTML) : '';
        widget.currentRow.DSMONTASUBSEMAMHTMLBKP = widget.currentRow.EXIST_INSTRUCAO > 0 ? angular.copy(widget.currentRow.DSMONTASUBSEMAMHTML) : "new";
    };

    //trata a adição de novos serviços a um manual
    this.saveServManual = function(widget) {
        var beforeWorkflow = function(widget) {
            var dataSave = widget.rowToSave;

            for (let rowSave in dataSave) {
                dataSave[rowSave].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            }

            coc03600_menu_designService.insertServicoWF(dataSave).then(
                function(callback) {
                    ScreenService.closePopup();
                    var widgetServico = widget.container.getActiveWidget();
                    widgetServico.list();
                    widgetServico.reload();
                    widgetServico.activate();

                    if (callback.dataset.errors.length) {
                        if (callback.messages) {
                            MessageService.showMessageByCode("COC03600_WORKFLOW_ITEM_ERROR");
                        } else {
                            MessageService.showMessageByCode("COC03600_WORKFLOW_ERROR");
                        }
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.errors, "Erros");
                    } else {
                        MessageService.showMessageByCode("WORKFLOW_SUCCESS");
                    }
                }, InflightorGeneral.callbackError
            );
        };

        if (widget.isValid() && servicoCiaFiliAtivo(widget.currentRow)) {
            if (widget.currentRow.IDSERVICO !== "3") {
                if (getSituacao(widget, true) == "P") {
                    usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SERVICO_INCLUSAO).then(
                        function(usaWF) {
                            if (usaWF) {
                                var classes = widget.getField("DSCLASSE").selectWidget.getCheckedRows();
                                var dataSave = [];
                                for (let classe in classes) {
                                    var changedItems = {
                                        NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
                                        CDCLASSE: classes[classe].CDCLASSE,
                                        VRCUSTOSERVM: ""
                                    };
                                    var pks = {
                                        CDFILIAL: widget.currentRow.CDFILIAL,
                                        CDCIA: widget.currentRow.CDCIA,
                                        CDMANCIA: widget.currentRow.CDMANCIA,
                                        NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
                                        CDCLASSE: classes[classe].CDCLASSE,
                                        NRORG: widget.currentRow.NRORG
                                    };
                                    var variables = {
                                        nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                                        nmServico: widget.currentRow.DSSERVICOCIAFILI,
                                        dsClasse: classes[classe].DSCLASSE,
                                        unidade: getLocalVar("NMFILIAL"),
                                        companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA
                                    };
                                    var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__", variables);
                                    var workflowData = {
                                        CHANGEDITEMS: changedItems,
                                        PRIMARYKEYS: pks,
                                        NMITEM: translateNMITEM,
                                        NMTABELA: "SERVMANUALM",
                                        IDTIPOPERWFLOW: "2",
                                        NRITEMPROCAPROVA: workflowItens.SERVICO_INCLUSAO,
                                        NRPROCESSOAPROVA: workflowItens.NRPROCESSO /*JULIAOSAVE*/
                                    };
                                    dataSave.push(workflowData);
                                }
                                self.solicitaCriacaoWorkflow(widget, dataSave, beforeWorkflow);
                            } else {

                                //usa as rows do select multiple para montar o dataSet a ser mandado para o backend
                                widget.getField("DSCLASSE").selectWidget.getCheckedRows().forEach(function(row) {
                                    row.CDMANCIA = widget.getField("CDMANCIA").defaultValue;
                                    row.CDFILIAL = widget.getField("CDFILIAL").defaultValue;
                                    row.CDCIA = widget.getField("CDCIA").defaultValue;
                                    row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                                    row.IDATIVO = "S";
                                    row.CDSERVICOBKP = "";
                                    row.NRSERVICOCIAFILI = widget.currentRow.NRSERVICOCIAFILI;
                                    row.VRCUSTOSERVM = "";
                                    row.__is_new = true;
                                });

                                // Faz a seguinte verificação:
                                // "Não podemos ter dois serviços iguais vigentes, dentro do mesmo manual ou em manuais diferentes."
                                //Chama uma rota para enviar o array montado
                                coc03600_menu_designService.saveServManual(widget.getField("DSCLASSE").selectWidget.getCheckedRows()).then(
                                    function success() {
                                        //usa as mesmas ações do save crud
                                        widget.container.getWidget("zh_crosstab").isVisible = true;
                                        widget.getField("DSCLASSE").template = "field/text-edit.html";
                                        widget.list(true);
                                        ScreenService.goBack();
                                        widget.reload();
                                        widget.activate();
                                    },
                                    function error(error) {
                                        if (error.data) {
                                            let msg = ScreenService.i18n(error.data.error);
                                            ScreenService.showMessage(msg);
                                        } else {
                                            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                            ScreenService.showMessage(msg);
                                        }
                                    }
                                );
                            }
                        }
                    );
                } else if (getSituacao(widget, true) == "E") {
                    widget.getField("DSCLASSE").selectWidget.getCheckedRows().forEach(function(row) {
                        row.CDMANCIA = widget.getField("CDMANCIA").defaultValue;
                        row.CDFILIAL = widget.getField("CDFILIAL").defaultValue;
                        row.CDCIA = widget.getField("CDCIA").defaultValue;
                        row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                        row.IDATIVO = "S";
                        row.CDSERVICOBKP = "";
                        row.NRSERVICOCIAFILI = widget.currentRow.NRSERVICOCIAFILI;
                        row.VRCUSTOSERVM = "";
                        row.__is_new = true;
                    });

                    // Faz a seguinte verificação:
                    // "Não podemos ter dois serviços iguais vigentes, dentro do mesmo manual ou em manuais diferentes."
                    //Chama uma rota para enviar o array montado
                    coc03600_menu_designService.saveServManual(widget.getField("DSCLASSE").selectWidget.getCheckedRows()).then(
                        function success() {
                            //usa as mesmas ações do save crud
                            widget.container.getWidget("zh_crosstab").isVisible = true;
                            widget.getField("DSCLASSE").template = "field/text-edit.html";
                            widget.list();
                            widget.reload();
                            widget.activate();
                        },
                        function error(error) {
                            if (error.data) {
                                let msg = ScreenService.i18n(error.data.error);
                                ScreenService.showMessage(msg);
                            } else {
                                let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                ScreenService.showMessage(msg);
                            }
                        }
                    );
                }
            } else {
                MessageService.showMessageByCode("COC03600_HANDLING_NOT_ADD");
            }
        }
    };

    function servicoCiaFiliAtivo(currentRow) {
        if (currentRow.IDSITSERVICO == "2") {
            MessageService.showMessageByCode("INACTIVE_SERVICE");
            return false;
        }
        return true;
    }

    //carrega os dados de produto e receitas para montar o grid de composição
    this.carregaDataSourceComposicao = function(widget = templateManager.container.getWidget("grid_composicao")) {

        //pega a current row de subgrupos para preencher os filtros do request
        var row = widget.container.getWidget("widgetCiclo").currentRow;
        var cdfilial = row.CDFILIAL;
        var cdcia = row.CDCIA;
        var CDMANCIA = row.CDMANCIA;
        var nrservicociafili = row.NRSERVICOCIAFILI;
        var cdclasse = row.CDCLASSE;
        var cdciclo = row.CDCICLO;
        var nrseqsubgrusemam = row.NRSEQSUBGRUSEMAM;
        var cdsubgru = row.CDSUBGRU;

        //arrays para implantar os dados
        //solução para integrar duas tabelas em um mesmo grid
        //utilizando nomes genéricos
        var promises = [];
        var produtos = [];
        var receitas = [];
        var positions = {
            PRODUTOS: {
                CODIGO_LIST: "CDARVPROD",
                CODIGO: "CDPRODUTO",
                CDPRODUTO: "CDPRODUTO",
                NOME: "NMPRODUTO",
                QTD: "QTPRODSUMA",
                UNIDADE: "SGUNIDADE",
                COMENTARIO: "TXCOPRODSUMA",
                VRCUSTO: "VRCUSPRSUBSEMAM",
                CUSTOZEROREC: "VRCUSPRSUBSEMAM",
                IDRECFLEXIVEL: null,
                LEANCODE: "CDLEANCODE",
                NMLEANCODE: "NMLEANCODE",
                CDFILIAL: "CDFILIAL",
                CDCIA: "CDCIA",
                CDMANCIA: "CDMANCIA",
                NRSERVICOCIAFILI: "NRSERVICOCIAFILI",
                CDCLASSE: "CDCLASSE",
                CDCICLO: "CDCICLO",
                QTGRAMATURA: "QTGRAMATURA",
                VRPESOGRAMATURA: "VRPESOGRAMATURA",
                NRSEQSUBGRUSEMAM: "NRSEQSUBGRUSEMAM",
                CDSUBGRU: "CDSUBGRU",
                NRORG: "NRORG",
                NRORDEM: "NRORDEM",
                IDANALIZEATIV: "IDANALIZEATIV",
                ID: "P"
            },
            RECEITAS: {
                CODIGO_LIST: "CDARVPRODREC",
                CODIGO: "CDRECEITA",
                CDRECEITA: "CDRECEITA",
                NOME: "DSRECEITA",
                QTD: "QTRECSUMA",
                UNIDADE: "SGUNIDADE",
                COMENTARIO: "TXCOMRECSUMA",
                VRCUSTO: "VRCUSRESUBSEMAM",
                CUSTOZEROREC: "CUSTOZEROREC",
                IDRECFLEXIVEL: "IDRECFLEXIVEL",
                LEANCODE: null,
                NMLEANCODE: null,
                CDFILIAL: "CDFILIAL",
                CDCIA: "CDCIA",
                CDMANCIA: "CDMANCIA",
                NRSERVICOCIAFILI: "NRSERVICOCIAFILI",
                CDCLASSE: "CDCLASSE",
                CDCICLO: "CDCICLO",
                NRSEQSUBGRUSEMAM: "NRSEQSUBGRUSEMAM",
                CDSUBGRU: "CDSUBGRU",
                NRORG: "NRORG",
                NRORDEM: "NRORDEM",
                QTGRAMATURA: "QTGRAMATURA",
                VRPESOGRAMATURA: "VRPESOGRAMATURA",
                IDANALIZEATIV: "IDANALIZEATIV",
                ID: "S"
            }
        };
        var data = [];

        promises.push(coc03600_menu_designService.returnProduto(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru));
        promises.push(coc03600_menu_designService.returnReceita(cdfilial, cdcia, CDMANCIA, nrservicociafili, cdclasse, cdciclo, nrseqsubgrusemam, cdsubgru));

        ZHPromise.all(promises).then(function(callback) {

            produtos = callback[0].dataset.prodsubsemam_produto;
            receitas = callback[1].dataset.recsubsemam_receita;

            setRowByName(data, produtos, positions.PRODUTOS);
            setRowByName(data, receitas, positions.RECEITAS);

            widget.dataSource.data = data;

        }.bind(this), function(callback) {
            InflightorGeneral.callbackError(callback);
        });

    };

    //Seta as rows do dataSource gerado de composição
    function setRowByName(dataSource, array, position) {
        array.forEach(function(object) {
            var row = {
                "CODIGO_LIST": object[position.CODIGO_LIST],
                "CODIGO": object[position.CODIGO],
                "NOME": object[position.NOME],
                "QTD": object[position.QTD],
                "UNIDADE": object[position.UNIDADE],
                "COMENTARIO": object[position.COMENTARIO],
                "VRCUSTO": object[position.VRCUSTO],
                "LEANCODE": object[position.LEANCODE],
                "NMLEANCODE": object[position.NMLEANCODE],
                "CDFILIAL": object[position.CDFILIAL],
                "CDCIA": object[position.CDCIA],
                "CDMANCIA": object[position.CDMANCIA],
                "CUSTOZEROREC": object[position.CUSTOZEROREC],
                "NRSERVICOCIAFILI": object[position.NRSERVICOCIAFILI],
                "IDRECFLEXIVEL": object[position.IDRECFLEXIVEL],
                "CDCLASSE": object[position.CDCLASSE],
                "CDCICLO": object[position.CDCICLO],
                "NRSEQSUBGRUSEMAM": object[position.NRSEQSUBGRUSEMAM],
                "CDSUBGRU": object[position.CDSUBGRU],
                "NRORG": object[position.NRORG],
                "NRORDEM": object[position.NRORDEM],
                "QTGRAMATURA": object[position.QTGRAMATURA] || null,
                "VRPESOGRAMATURA": object[position.VRPESOGRAMATURA] || null,
                "IDANALIZEATIV": object[position.IDANALIZEATIV] || null,
                "CDPRODUTO": object[position.CDPRODUTO] || null,
                "CDRECEITA": object[position.CDRECEITA] || null,
                "ID": position.ID
            };
            if (row.NMLEANCODE == undefined)
                row.NMLEANCODE = " - ";
            dataSource.push(row);
        });
    }

    //faz o preenchimento dos dataSourceFilters do ZHFilter
    this.FieldOnChangeFiltro = function(args) {
        if (args.owner.field.name == "NMCIA") {
            if (args.row.CDFILIAL) {
                args.owner.owner.widget.getField("NMCIA").dataSourceFilter[1].value = args.row.CDFILIAL;
                args.owner.owner.widget.getField("NMCIA").reload();
            } else {
                args.owner.owner.widget.getField("NMCIA").dataSourceFilter[1].value = "";
                args.owner.owner.widget.getField("NMCIA").reload();
                let msg = ScreenService.i18n("Escolha um Unidade para definir a Companhia!");
                ScreenService.showMessage(msg);
            }
        }
        if (args.owner.field.name == "DSMANUAL") {
            if (args.row.CDFILIAL) {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = args.row.CDFILIAL;
                if (args.row.CDCIA) {
                    args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = args.row.CDCIA;
                    args.owner.owner.widget.getField("DSMANUAL").reload();
                } else {
                    args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = "";
                    args.owner.owner.widget.getField("DSMANUAL").reload();
                    let msg = ScreenService.i18n("Escolha uma Companhia para definir o Manual!");
                    ScreenService.showMessage(msg);
                }
            } else {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = "";
                args.owner.owner.widget.getField("DSMANUAL").reload();
                let msg = ScreenService.i18n("Escolha uma Unidade para definir o Manual!");
                ScreenService.showMessage(msg);
            }
        }
    };

    this.FieldOnChangeFiltroDSCLASSE = function(args) {
        if (args.row.CDFILIAL) {
            args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[1].value = args.row.CDFILIAL;
            if (args.row.CDCIA) {
                args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[2].value = args.row.CDCIA;
                args.owner.owner.widget.getField("DSCLASSE").reload();
            } else {
                args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[2].value = "";
                args.owner.owner.widget.getField("DSCLASSE").reload();
                let msg = ScreenService.i18n("Escolha uma Companhia para definir a Classe!");
                ScreenService.showMessage(msg);
            }
        } else {
            args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[1].value = "";
            args.owner.owner.widget.getField("DSCLASSE").reload();
            let msg = ScreenService.i18n("Escolha uma Unidade para definir a Classe!");
            ScreenService.showMessage(msg);
        }
    };

    function retornaMetodoDeleteIdWorkflow(widget) {
        var metodo = {
            params: widget.currentRow
        };
        metodo.primaryKeys = ["CDCICLO", "NRSEQSUBGRUSEMAM", "CDSUBGRU", "NRSERVICOCIAFILI", "CDCLASSE", "CDMANCIA", "CDCIA", "CDFILIAL", "NRORG"];
        if (widget.currentRow.ID == "S") {
            metodo.method = coc03600_menu_designService.removeReceita;
            metodo.idWorkflow = workflowItens.RECEITA_EXCLUSAO;
            metodo.tableName = "RECSUBSEMAM";
            metodo.primaryKeys.push("CDRECEITA");
            metodo.itemReferencia = "DSRECEITA";
        } else {
            metodo.method = coc03600_menu_designService.removeProduto;
            metodo.idWorkflow = workflowItens.PRODUTO_EXCLUSAO;
            metodo.tableName = "PRODSUBSEMAM";
            metodo.primaryKeys.push("CDPRODUTO");
            metodo.itemReferencia = "NMPRODUTO";
        }
        return metodo;
    }

    //encaminha o excluir do grid de composição juliaoComp
    this.deletaComposicao = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            verificaComposicaoPendenteNovoOuApagado(widget).then(function(composicaoValida) {
                if (composicaoValida) {
                    ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                        var metodo = retornaMetodoDeleteIdWorkflow(widget);
                        if (getSituacao(widget, false) == "P") {
                            widget.backupRow = copyRow(widget.currentRow);
                            var NRPROCESSOAPROVA = workflowItens.NRPROCESSO;
                            var NRITEMPROCAPROVA = metodo.idWorkflow;
                            usaWorkflow(NRPROCESSOAPROVA, NRITEMPROCAPROVA).then(function(idUsaWorkflow) {
                                if (idUsaWorkflow) {
                                    widget.currentRow.IDANALIZEATIV = "A";
                                    var itensAlterados = ["IDANALIZEATIV"];
                                    metodo.itemReferencia = "NOME";
                                    var rowToSave = retornaRowSalvarWFComposicao(widget, itensAlterados, metodo, "3");
                                    var variables = {
                                        nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                                        nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                                        nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                                        dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                                        unidade: getLocalVar("NMFILIAL"),
                                        companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                                        idCiclo: widget.parent.currentRow.IDCICLO,
                                        item: rowToSave.NOME
                                    };
                                    var label = widget.currentRow.ID == "S" ? "Receita" : "Produto";
                                    var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
                                    rowToSave.NMITEM = translateNMITEM;
                                    var afterSave = function(widget) {
                                        widget.parentWidget.currentRow.IDANALIZEATIV = "P";
                                        var metodoSave = retornaMetodoSaveIdWorkflow(widget.parentWidget);
                                        // metodoSave.itemReferencia = 'NOME';
                                        widget.parentWidget.currentRow.__is_new = false;
                                        salvaItemComposicao(metodoSave, widget.parentWidget).then(function() {
                                            widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                                            coc03600_menu_designService.salvaWorkflow(rowToSave).then(function(callback) {
                                                ScreenService.showMessage(callback.messages[0].message);
                                                ScreenService.closePopup();
                                                finalizaItemComposicao(widget.parentWidget);
                                            }, callbackErrorComposicao);
                                        }, callbackErrorComposicao);
                                    };
                                    self.solicitaCriacaoWorkflow(widget, rowToSave, afterSave);
                                } else {
                                    salvaItemComposicao(metodo).then(function() {
                                        widget.reload().then(function() {
                                            ScreenService.closeSwipe();
                                            widget.activate();
                                            self.carregaDataSourceComposicao(widget);
                                        });
                                    }, function(error) {
                                        if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                            MessageService.showMessageByCode("DELETE_ERROR");
                                        } else {
                                            ScreenService.showMessage(error.data.error);
                                        }
                                    });
                                }
                            });
                        } else {
                            salvaItemComposicao(metodo).then(function() {
                                widget.reload().then(function() {
                                    ScreenService.closeSwipe();
                                    widget.activate();
                                    self.carregaDataSourceComposicao(widget);
                                });
                            }, function(error) {
                                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                    MessageService.showMessageByCode("DELETE_ERROR");
                                } else {
                                    ScreenService.showMessage(error.data.error);
                                }
                            });
                        }
                    });
                }
            });
        }
    };

    function verificaOrdemComposicao(row) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": row.NRORG
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": row.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": row.CDCLASSE
            },
            {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": row.NRSEQSUBGRUSEMAM
            },
            {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": row.CDSUBGRU
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": row.CDCICLO
            }
        ];
        var promises = [];
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/prodsubsemam", "FilterData", filter));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/recsubsemam", "FilterData", filter));
        return ZHPromise.all(promises).then(function(callback) {
            return callback[0].dataset.prodsubsemam.concat(callback[1].dataset.recsubsemam);
        });
    }

    function validaOrdenacaoComposicao(itens, currentRow) {
        var ordenacaoValida = itens.filter(function(item) {
            return currentRow.NRORDEM == item.NRORDEM;
        });

        return !ordenacaoValida.length;
    }

    function modificaCurrentRowComposicao(widget, isNew) {
        InflightorGeneral.setFloatAttrOnCurrentRow(widget.currentRow, "QTD");
        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
        widget.currentRow.CDCIA = widget.getParent().currentRow.CDCIA;
        widget.currentRow.CDMANCIA = widget.getParent().currentRow.CDMANCIA;
        widget.currentRow.NRSERVICOCIAFILI = widget.getParent().currentRow.NRSERVICOCIAFILI;
        widget.currentRow.CDCLASSE = widget.getParent().currentRow.CDCLASSE;
        widget.currentRow.CDCICLO = widget.getParent().currentRow.CDCICLO;
        widget.currentRow.NRORG = widget.getParent().currentRow.NRORG;
        widget.currentRow.NRSEQSUBGRUSEMAM = widget.getParent().currentRow.NRSEQSUBGRUSEMAM;
        widget.currentRow.CDSUBGRU = widget.getParent().currentRow.CDSUBGRU;
        widget.currentRow.__is_new = isNew;
    }

    this.setNameToCurrentRow = function(widget) {
        if (widget.currentRow.ID == "S") {
            widget.currentRow.NOME = widget.currentRow.DSRECEITA;
            widget.currentRow.CODIGO = widget.currentRow.CDRECEITA;
        } else {
            widget.currentRow.NOME = widget.currentRow.NMPRODUTO;
            widget.currentRow.CODIGO = widget.currentRow.CDPRODUTO;
        }
    };

    function retornaMetodoSaveIdWorkflow(widget) {
        var metodo = {
            params: widget.currentRow
        };
        metodo.primaryKeys = ["CDCICLO", "NRSEQSUBGRUSEMAM", "CDSUBGRU", "NRSERVICOCIAFILI", "CDCLASSE", "CDMANCIA", "CDCIA", "CDFILIAL", "NRORG"];
        if (widget.currentRow.ID == "S") {
            metodo.method = coc03600_menu_designService.saveReceita;
            metodo.idWorkflow = workflowItens.RECEITA_INCLUSAO;
            metodo.tableName = "RECSUBSEMAM";
            metodo.primaryKeys.push("CDRECEITA");
            metodo.itemReferencia = "DSRECEITA";
        } else {
            metodo.method = coc03600_menu_designService.saveProduto;
            metodo.idWorkflow = workflowItens.PRODUTO_INCLUSAO;
            metodo.tableName = "PRODSUBSEMAM";
            metodo.primaryKeys.push("CDPRODUTO");
            metodo.itemReferencia = "NMPRODUTO";
        }
        return metodo;
    }

    function salvaItemComposicao(metodo) {
        return metodo.method.call(null, metodo.params);
    }

    function finalizaItemComposicao(widget) {
        ScreenService.goBack();
        widget.list(true);
        self.carregaDataSourceComposicao(widget);
        widget.activate();
        widget.getField("NOME").showOnForm = true;
        widget.getField("LEANCODE").showOnForm = true;
        widget.getField("DSRECEITA").isVisible = false;
        widget.getField("NMPRODUTO").isVisible = false;
        widget.getField("DSRECEITA").validations = null;
        widget.getField("NMPRODUTO").validations = null;
        widget.container.getWidget("grid_composicao").activate();
        widget.container.getWidget("grid_composicao").reload();
    }

    function retornaRowSalvarWFComposicao(widget, itensAlterados, metodo, tipoOperacao) {
        var tableName = metodo.tableName;
        var NRITEMPROCAPROVA = metodo.idWorkflow;
        var itemReferencia = metodo.itemReferencia;
        var primaryKeys = metodo.primaryKeys;
        var rowToSave = retornaRowSalvarWF(widget, tableName, NRITEMPROCAPROVA, itemReferencia, itensAlterados, primaryKeys, tipoOperacao);
        return rowToSave;
    }

    function callbackErrorComposicao(error) {
        if (error.data) {
            let msg = ScreenService.i18n(error.data.error);
            ScreenService.showMessage(msg);
        } else {
            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
            ScreenService.showMessage(msg);
        }
    }

    function validItemComposicaoExistente(composicao, novaComposicao) {
        var valid = true;
        for (let index in composicao) {
            var item = composicao[index];
            if (novaComposicao.ID == "S" && novaComposicao.CDRECEITA == item.CDRECEITA) {
                valid = false;
                break;
            } else if (novaComposicao.ID == "P" && novaComposicao.CDPRODUTO == item.CDPRODUTO) {
                valid = false;
                break;
            }
        }
        return valid;
    }

    function salvaNovaComposicao(widget) {
        modificaCurrentRowComposicao(widget, true);
        verificaOrdemComposicao(widget.currentRow).then(function(itens) {
            var ordenacaoValida = true;
            var itemInexistente = true;
            if (itens.length) {
                ordenacaoValida = validaOrdenacaoComposicao(itens, widget.currentRow);
                itemInexistente = validItemComposicaoExistente(itens, widget.currentRow);
            }
            if (!ordenacaoValida) {
                MessageService.showMessageByCode("COC03600_ORDER_NUMBER_DUPLICATED");
            } else if (!itemInexistente) {
                MessageService.showMessageByCode("COC03600_COMPOSITION_DUPLICATED");
            } else {
                var metodo = retornaMetodoSaveIdWorkflow(widget);
                if (getSituacao(widget, false) == "P") {
                    var NRPROCESSOAPROVA = workflowItens.NRPROCESSO;
                    var NRITEMPROCAPROVA = metodo.idWorkflow;
                    usaWorkflow(NRPROCESSOAPROVA, NRITEMPROCAPROVA).then(function(idUsaWorkflow) {
                        widget.metodo = metodo;
                        if (idUsaWorkflow) {
                            widget.currentRow.IDANALIZEATIV = "P";
                            var itensAlterados = ["CDRECEITA", "QTD", "QTGRAMATURA", "VRPESOGRAMATURA", "NRORDEM", "COMENTARIO", "IDANALIZEATIV"];
                            var rowToSave = retornaRowSalvarWFComposicao(widget, itensAlterados, metodo, "2");

                            var variables = {
                                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                                unidade: getLocalVar("NMFILIAL"),
                                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                                idCiclo: widget.parent.currentRow.IDCICLO,
                                item: rowToSave.NOME
                            };
                            var label = widget.currentRow.ID == "S" ? "Receita" : "Produto";
                            var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
                            rowToSave.NMITEM = translateNMITEM;
                            var afterSave = function(widget) {
                                var widgetComposicao = widget.container.getWidget("grid_composicao");
                                if (widgetComposicao.currentRow.ID == "S") {
                                    rowToSave.CHANGEDITEMS.QTRECSUMA = rowToSave.CHANGEDITEMS.QTD;
                                    rowToSave.CHANGEDITEMS.TXCOMRECSUMA = rowToSave.CHANGEDITEMS.COMENTARIO;
                                } else {
                                    rowToSave.CHANGEDITEMS.QTPRODSUMA = rowToSave.CHANGEDITEMS.QTD;
                                    rowToSave.CHANGEDITEMS.TXCOPRODSUMA = rowToSave.CHANGEDITEMS.COMENTARIO;

                                }
                                delete rowToSave.CHANGEDITEMS.QTD;
                                delete rowToSave.CHANGEDITEMS.COMENTARIO;
                                widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                                self.carregaDataSourceComposicao(widget.container.getWidget("grid_composicao"));
                                coc03600_menu_designService.salvaWorkflow(rowToSave).then(function(callback) {
                                    ScreenService.showMessage(callback.messages[0].message);
                                    ScreenService.closePopup();
                                    finalizaItemComposicao(widgetComposicao);
                                }, callbackErrorComposicao);
                            };
                            self.solicitaCriacaoWorkflow(widget, rowToSave, afterSave);

                        } else {
                            widget.currentRow.IDANALIZEATIV = "I";
                            salvaItemComposicao(metodo, widget).then(function() {
                                finalizaItemComposicao(widget);
                            }, callbackErrorComposicao);
                        }
                    });
                } else {
                    widget.currentRow.IDANALIZEATIV = "I";
                    salvaItemComposicao(metodo, widget).then(function() {
                        finalizaItemComposicao(widget);
                    }, callbackErrorComposicao);
                }
                // if(param) {
                // self.calculaCusto(widget.container.getWidget('calculodecusto'), param);
                // }
            }
        });
    }

    this.getDataSourceFilterComposicao = function(widget) {
        return [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": widget.currentRow.CDRECEITA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
    };

    function validateGramaturaComposicao(widget) {
        var currentRow = widget.currentRow;
        var qtd = InflightorGeneral.valueToFloat(currentRow.QTD) || 0;
        var porcao = InflightorGeneral.valueToFloat(currentRow.QTGRAMATURA) || 0;
        if (porcao)
            currentRow.VRPESOGRAMATURA = qtd / porcao;
        else
            currentRow.VRPESOGRAMATURA = 0;
    }

    this.salvaComposicao = function(widget) {
        var param = {};

        param.DTCUSTOMAN = widget.container.getWidget("grid_manual").currentRow.DTCUSTOMAN;
        param.IDTIPOCUSTOMAN = widget.container.getWidget("grid_manual").currentRow.IDTIPOCUSTOMAN;

        if (widget.currentRow.ID == "S") {
            widget.currentRow.DSRECEITA = widget.currentRow.NOME;
        } else {
            widget.currentRow.NMPRODUTO = widget.currentRow.NOME;
        }
        if (widget.isValid()) {
            validateGramaturaComposicao(widget);
            if (InflightorGeneral.valueToFloat(widget.getField("QTD").value()) == 0) {
                ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar uma composição com quantidade zerado."));
            } else {
                if (widget.currentRow.__is_new) {
                    if (produtoReceitaAtivo(widget.currentRow, widget.getField("ID").value())) {
                        if (widget.currentRow.ID == "S") {
                            var rowToFind = {};
                            rowToFind.CDRECEITA = widget.currentRow.CDRECEITA;
                            rowToFind.DSRECEITA = widget.currentRow.DSRECEITA;
                            rowToFind.IDRECFLEXIVEL = widget.currentRow.IDRECFLEXIVEL;
                            rowToFind.CDFILIAL = widget.container.getWidget("widgetCiclo").currentRow.CDFILIAL;
                            InflightorGeneral.callRestEngine("coc#/verificaComposicao", "Row", rowToFind).then(function(callback) {
                                if (callback.length == 0) {
                                    salvaNovaComposicao(widget, param);
                                }
                            }, function(error) {
                                ScreenService.showMessage(error.data.error);
                            });
                        } else {
                            salvaNovaComposicao(widget, param);
                        }
                    }
                } else {
                    if (widget.inEditMode()) {
                        InflightorGeneral.setFloatAttrOnCurrentRow(widget.currentRow, "QTD");
                        widget.currentRow.__is_new = false;
                        var metodo = retornaMetodoSaveIdWorkflow(widget);
                        if (widget.currentRow.ID == "S") {
                            if (qtdGramaturaReceitaAlterada(widget) && getSituacao(widget, false) == "P") {
                                if (widget.idWorkflowReceita.GRAMATURA) {
                                    self.solicitaCriacaoWorkflow(widget, {}, callbackWFReceita);
                                } else {
                                    salvaItemComposicao(metodo).then(function() {
                                        finalizaItemComposicao(widget);
                                    });
                                }
                            } else {
                                salvaItemComposicao(metodo).then(function() {
                                    finalizaItemComposicao(widget);
                                });
                            }
                        } else {
                            if (qtdGramaturaReceitaAlterada(widget) && getSituacao(widget, false) == "P") {
                                if (widget.idWorkflowProduto.QTD) {
                                    self.solicitaCriacaoWorkflow(widget, {}, callbackWFProduto);
                                } else {
                                    salvaItemComposicao(metodo).then(function() {
                                        finalizaItemComposicao(widget);
                                    });
                                }
                            } else {
                                salvaItemComposicao(metodo).then(function() {
                                    finalizaItemComposicao(widget);
                                });
                            }
                        }
                    } else {
                        var ordenacaoValida = alteraOrdenacao(widget.dataSource.data, "NRORDEM");
                        if (ordenacaoValida) {
                            var dataset = {
                                produtos: [],
                                receitas: []
                            };
                            widget.dataSource.data.forEach(function(row) {
                                row.__is_new = false;
                                InflightorGeneral.setFloatAttrOnCurrentRow(row, "QTD");
                                if (row.changed) {
                                    widget.currentRow = row;
                                    var metodo = retornaMetodoSaveIdWorkflow(widget);
                                    row.PKS = buildDSPrimaryKeys(metodo.primaryKeys, row);
                                    if (row.ID == "S")
                                        dataset.receitas.push(copyRow(row));
                                    else
                                        dataset.produtos.push(copyRow(row));
                                }
                            });
                            coc03600_menu_designService.verificaWorkflowPendenteReceitaProduto(dataset).then(function(callback) {
                                var produtosReceitas = callback.dataset.response;
                                var datasetProdutosReceita = produtosReceitas.produtos.concat(produtosReceitas.receitas);
                                var widgetDataSource = widget.dataSource.data;
                                for (let i = 0; i < datasetProdutosReceita.length; i++) {
                                    for (let j = 0; j < widgetDataSource.length; j++) {
                                        var sameKeys = true;
                                        let tipoProdKey = (datasetProdutosReceita[i].ID == "S") ? "CDRECEITA" : "CDPRODUTO";
                                        let keysArray = [
                                            "CDFILIAL",
                                            "CDCIA",
                                            "CDMANCIA",
                                            "NRSERVICOCIAFILI",
                                            "CDCLASSE",
                                            "CDSUBGRU",
                                            "NRSEQSUBGRUSEMAM",
                                            "CDCICLO",
                                            tipoProdKey,
                                            "NRORG"
                                        ];
                                        keysArray.forEach(function(key) {
                                            let keyIsEqual = datasetProdutosReceita[i][key] == widgetDataSource[j][key];
                                            sameKeys = (sameKeys && keyIsEqual);
                                        });
                                        if (sameKeys) {
                                            widgetDataSource[j] = datasetProdutosReceita[i];
                                        }
                                    }
                                }

                                var produtosPodemAlterar = produtosReceitas.produtos.filter(function(row) {
                                    return row.PODE_ALTERAR;
                                });

                                var receitasPodemAlterar = produtosReceitas.receitas.filter(function(row) {
                                    return row.PODE_ALTERAR;
                                });
                                if (((produtosPodemAlterar.length && produtosReceitas.usaWorkFlow.produto && quantidadeMudou(produtosPodemAlterar)) || (receitasPodemAlterar.length && produtosReceitas.usaWorkFlow.receita && quantidadeMudou(receitasPodemAlterar))) && getSituacao(widget, false) == "P") {
                                    self.solicitaCriacaoWorkflow(widget, produtosReceitas, callbackGridEditavelReceitaProduto);
                                } else {
                                    var promises = [];
                                    widget.dataSource.data.forEach(function(row) {
                                        row.__is_new = false;
                                        InflightorGeneral.setFloatAttrOnCurrentRow(row, "QTD");
                                        if (row.changed) {
                                            if (row.ID == "S") {
                                                promises.push(coc03600_menu_designService.saveReceita(row));
                                            } else {
                                                promises.push(coc03600_menu_designService.saveProduto(row));
                                            }
                                        }
                                    });
                                    ZHPromise.all(promises).then(function success() {
                                        widget.list(true);
                                        self.carregaDataSourceComposicao(widget);
                                        widget.activate();
                                        self.loadDataSource();
                                    }, function error(error) {
                                        if (error.data) {
                                            let msg = ScreenService.i18n(error.data.error);
                                            ScreenService.showMessage(msg);
                                        } else {
                                            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                            ScreenService.showMessage(msg);
                                        }
                                    });
                                }
                            });
                        }
                    }
                }
            }
        }
    };

    function callbackGridEditavelReceitaProduto(widget) {
        var produtosReceitas = widget.rowToSave;
        var parentWidget = widget.parentWidget;
        var promises = [];
        produtosReceitas.receitas.concat(produtosReceitas.produtos).forEach(function(row) {
            parentWidget.currentRow = row;
            var metodo = retornaMetodoSaveIdWorkflow(parentWidget);
            metodo.itemReferencia = "NOME";
            row.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            var itensAlterados = ["QTD"];
            var rowToSave = retornaRowSalvarWFComposicao(parentWidget, itensAlterados, metodo, "1");
            if (row.ID == "S" && row.PODE_ALTERAR && produtosReceitas.usaWorkFlow.receita) {
                rowToSave.CHANGEDITEMS.QTRECSUMA = rowToSave.CHANGEDITEMS.QTD;
                rowToSave.NRITEMPROCAPROVA = workflowItens.RECEITA_ALTERACAO_QTD;
            }

            if (row.ID == "P" && row.PODE_ALTERAR && produtosReceitas.usaWorkFlow.produto) {
                rowToSave.CHANGEDITEMS.QTPRODSUMA = rowToSave.CHANGEDITEMS.QTD;
                rowToSave.NRITEMPROCAPROVA = workflowItens.PRODUTO_ALTERACAO_QTD;
            }
            row.QTD = row.changed_QTD;
            delete rowToSave.CHANGEDITEMS.QTD;
            delete row.changed_QTD;
            delete row.QTD_inline;
            var variables = {
                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                unidade: getLocalVar("NMFILIAL"),
                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                idCiclo: parentWidget.parent.currentRow.IDCICLO,
                item: rowToSave.NOME
            };
            var label = parentWidget.currentRow.ID == "S" ? "Receita" : "Produto";
            var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
            rowToSave.NMITEM = translateNMITEM;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        });

        if (promises.length) {
            ZHPromise.all(promises).then(function(callback) {
                ScreenService.showMessage(callback[0].messages[0].message);
                ScreenService.closePopup();
                parentWidget.list(true);
                // self.carregaDataSourceComposicao(parentWidget);
                parentWidget.activate();
                parentWidget.getField("NOME").showOnForm = true;
                parentWidget.getField("DSRECEITA").isVisible = false;
                parentWidget.getField("NMPRODUTO").isVisible = false;
                parentWidget.getField("DSRECEITA").validations = null;
                parentWidget.getField("NMPRODUTO").validations = null;
                var promises = [];
                parentWidget.dataSource.data.forEach(function(row) {
                    row.__is_new = false;
                    InflightorGeneral.setFloatAttrOnCurrentRow(row, "QTD");
                    if (row.changed) {
                        if (row.ID == "S")
                            promises.push(coc03600_menu_designService.saveReceita(row));
                        else
                            promises.push(coc03600_menu_designService.saveProduto(row));
                    }
                });
                ZHPromise.all(promises).then(function success() {
                    //usa as mesmas ações do save crud
                    parentWidget.list(true);
                    self.carregaDataSourceComposicao(parentWidget);
                    parentWidget.activate();
                }, function error(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                });
            }, function(err) {
                if (err.data) {
                    ScreenService.showMessage(err.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        } else {
            let metodo = retornaMetodoSaveIdWorkflow(widget);
            salvaItemComposicao(metodo).then(function() {
                ScreenService.closePopup();
                widget.parentWidget.list(true);
                self.carregaDataSourceComposicao(widget.parentWidget);
                widget.parentWidget.activate();
                widget.parentWidget.getField("NOME").showOnForm = true;
                widget.parentWidget.getField("DSRECEITA").isVisible = false;
                widget.parentWidget.getField("NMPRODUTO").isVisible = false;
                widget.parentWidget.getField("DSRECEITA").validations = null;
                widget.parentWidget.getField("NMPRODUTO").validations = null;
            });
        }
    }

    function quantidadeMudou(dataset) {
        var mudou = false;
        dataset.forEach(function(row) {
            if (row.changed_QTD != row.QTD) {
                mudou = true;
            }
        });
        return mudou;
    }

    function callbackWFReceita(widget) {
        var parentWidget = widget.parentWidget;
        var metodo = retornaMetodoSaveIdWorkflow(parentWidget);
        var promises = [];
        metodo.itemReferencia = "NOME";
        parentWidget.currentRow.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        if (parentWidget.idWorkflowReceita.QTD && parentWidget.currentRow.QTD != parentWidget.backupRow.QTD) {
            let itensAlterados = ["QTD"];
            let rowToSave = retornaRowSalvarWFComposicao(parentWidget, itensAlterados, metodo, "1");
            rowToSave.CHANGEDITEMS.QTRECSUMA = rowToSave.CHANGEDITEMS.QTD;
            rowToSave.NRITEMPROCAPROVA = workflowItens.RECEITA_ALTERACAO_QTD;
            delete rowToSave.CHANGEDITEMS.QTD;
            parentWidget.currentRow.QTD = parentWidget.backupRow.QTD;
            let variables = {
                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                unidade: getLocalVar("NMFILIAL"),
                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                idCiclo: parentWidget.parent.currentRow.IDCICLO,
                item: rowToSave.NOME
            };
            let label = parentWidget.currentRow.ID == "S" ? "Receita" : "Produto";
            let translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
            rowToSave.NMITEM = translateNMITEM;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        if (parentWidget.idWorkflowReceita.GRAMATURA && parentWidget.currentRow.QTGRAMATURA != parentWidget.backupRow.QTGRAMATURA) {
            let itensAlterados = ["QTGRAMATURA", "VRPESOGRAMATURA"];
            let rowToSave = retornaRowSalvarWFComposicao(parentWidget, itensAlterados, metodo, "1");
            rowToSave.NRITEMPROCAPROVA = workflowItens.RECEITA_ALTERACAO_GRAMATURA;
            parentWidget.currentRow.QTGRAMATURA = parentWidget.backupRow.QTGRAMATURA;
            parentWidget.currentRow.VRPESOGRAMATURA = parentWidget.backupRow.VRPESOGRAMATURA;
            let variables = {
                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                unidade: getLocalVar("NMFILIAL"),
                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                idCiclo: parentWidget.parent.currentRow.IDCICLO,
                item: rowToSave.NOME
            };
            let label = parentWidget.currentRow.ID == "S" ? "Receita" : "Produto";
            let translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
            rowToSave.NMITEM = translateNMITEM;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        executaPromiseAlteracaoComposicao(promises, widget, metodo);
    }

    function executaPromiseAlteracaoComposicao(promises, widget, metodo) {
        if (promises.length) {
            ZHPromise.all(promises).then(function(callback) {
                salvaItemComposicao(metodo).then(function() {
                    ScreenService.showMessage(callback[0].messages[0].message);
                    ScreenService.closePopup();
                    finalizaItemComposicao(widget.parentWidget);
                });
            }, function(err) {
                if (err.data) {
                    ScreenService.showMessage(err.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        } else {
            salvaItemComposicao(metodo).then(function() {
                ScreenService.closePopup();
                finalizaItemComposicao(widget.parentWidget);
            });
        }
    }

    function callbackWFProduto(widget) {
        var parentWidget = widget.parentWidget;
        var metodo = retornaMetodoSaveIdWorkflow(parentWidget);
        var promises = [];
        metodo.itemReferencia = "NOME";
        parentWidget.currentRow.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        if (parentWidget.idWorkflowProduto.QTD && parentWidget.currentRow.QTD != parentWidget.backupRow.QTD) {
            var itensAlterados = ["QTD"];
            var rowToSave = retornaRowSalvarWFComposicao(parentWidget, itensAlterados, metodo, "1");
            rowToSave.CHANGEDITEMS.QTPRODSUMA = rowToSave.CHANGEDITEMS.QTD;
            rowToSave.NRITEMPROCAPROVA = workflowItens.PRODUTO_ALTERACAO_QTD;
            delete rowToSave.CHANGEDITEMS.QTD;
            parentWidget.currentRow.QTD = parentWidget.backupRow.QTD;
            var variables = {
                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                unidade: getLocalVar("NMFILIAL"),
                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                idCiclo: parentWidget.parent.currentRow.IDCICLO,
                item: rowToSave.NOME
            };
            var label = parentWidget.currentRow.ID == "S" ? "Receita" : "Produto";
            var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
            rowToSave.NMITEM = translateNMITEM;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }

        executaPromiseAlteracaoComposicao(promises, widget, metodo);
    }

    function qtdGramaturaReceitaAlterada(widget) {
        var qtdAlterada = widget.currentRow.QTD != widget.backupRow.QTD;
        var gramaturaAlterada = widget.currentRow.QTGRAMATURA != widget.backupRow.QTGRAMATURA;
        return qtdAlterada || gramaturaAlterada;
    }

    function produtoReceitaAtivo(currentRow, ID) {
        var IDATIVO = currentRow.IDATIVO;
        if (IDATIVO == "N") {
            if (ID == "S")
                MessageService.showMessageByCode("INACTIVE_RECIPE");
            else
                MessageService.showMessageByCode("INACTIVE_PRODUCT.");
            return false;
        } else if (cdciaReceitaInvalido(currentRow.CDCIA)) {
            MessageService.showMessageByCode("COC03600_RECIPE_NOT_BELONG_CIA");
            return false;
        }
        return true;
    }

    function cdciaReceitaInvalido(CDCIA) {
        return templateManager.container.getWidget("grid_manual").currentRow.CDCIA != CDCIA && CDCIA;
    }

    this.adicionarComposicao = function(widget, type) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            widget.type = type;
            widget.getField("NOME").showOnForm = false;
            widget.getField("ID").defaultValue = type;
            widget.getField("ID").setValue(type);
            widget.getField("QTD").readOnly = false;
            widget.getField("QTGRAMATURA").readOnly = false;

            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.container.getWidget("grid_manual").currentRow.CDFILIAL
                },
                {
                    "name": "DTCUSTO",
                    "operator": "=",
                    "value": widget.container.getWidget("grid_manual").currentRow.DTCUSTOMAN ? moment(widget.container.getWidget("grid_manual").currentRow.DTCUSTOMAN, "DD/MM/YYYY HH:mm:ss").format("DD/MM/YYYY") : null
                },
                {
                    "name": "TPCUSTO",
                    "operator": "=",
                    "value": widget.container.getWidget("grid_manual").currentRow.IDTIPOCUSTOMAN
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": widget.container.getWidget("widgetCiclo").currentRow.CDCIA
                }
            ];

            widget.getField("NMPRODUTO").dataSourceFilter = filter;
            widget.getField("DSRECEITA").dataSourceFilter = filter;

            if (widget.dataSource.data.length) {

                filter = [
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDFILIAL
                    },
                    {
                        "name": "CDCIA",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDCIA
                    },
                    {
                        "name": "CDMANCIA",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDMANCIA
                    },
                    {
                        "name": "NRSERVICOCIAFILI",
                        "operator": "=",
                        "value": widget.dataSource.data[0].NRSERVICOCIAFILI
                    },
                    {
                        "name": "CDCLASSE",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDCLASSE
                    },
                    {
                        "name": "CDCICLO",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDCICLO
                    },
                    {
                        "name": "NRSEQSUBGRUSEMAM",
                        "operator": "=",
                        "value": widget.dataSource.data[0].NRSEQSUBGRUSEMAM
                    },
                    {
                        "name": "CDSUBGRU",
                        "operator": "=",
                        "value": widget.dataSource.data[0].CDSUBGRU
                    },
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": widget.dataSource.data[0].NRORG
                    }
                ];

                if (type == "S") {
                    widget.getField("NMPRODUTO").validations = null;
                    widget.getField("NMPRODUTO").isVisible = false;
                    widget.getField("DSRECEITA").isVisible = true;
                    widget.getField("NMLEANCODE").isVisible = false;
                    widget.getField("DSRECEITA").validations = {
                        required: true
                    };
                    widget.getField("DSRECEITA").reload();
                    widget.newRow();
                    InflightorGeneral.callRestEngine("coc#/crud/max_produto_receita", "FilterData", filter).then(function(callback) {
                        if (callback.dataset.max_produto_receita) {
                            if (callback.dataset.max_produto_receita.length) {
                                var max = callback.dataset.max_produto_receita[0].MAXNRORDEM;
                                widget.currentRow.NRORDEM = max;
                            } else {
                                widget.currentRow.NRORDEM = 1;
                            }
                        }
                        widget.activate();
                        InflightorGeneral.justInteger("#NRORDEM, #QTGRAMATURA");
                    });

                } else {
                    widget.getField("DSRECEITA").validations = null;
                    widget.getField("DSRECEITA").isVisible = false;
                    widget.getField("NMPRODUTO").isVisible = true;
                    widget.getField("NMLEANCODE").isVisible = true;
                    widget.getField("NMPRODUTO").validations = {
                        required: true
                    };
                    widget.getField("NMPRODUTO").reload();
                    widget.newRow();
                    InflightorGeneral.callRestEngine("coc#/crud/max_produto_receita", "FilterData", filter).then(function(callback) {
                        if (callback.dataset.max_produto_receita) {
                            if (callback.dataset.max_produto_receita.length) {
                                var max = callback.dataset.max_produto_receita[0].MAXNRORDEM;
                                widget.currentRow.NRORDEM = max;
                            } else {
                                widget.currentRow.NRORDEM = 1;
                            }
                        }
                        widget.edit();
                        widget.activate();
                        InflightorGeneral.justInteger("#NRORDEM, #QTGRAMATURA");
                    });
                }
            } else {
                if (type == "S") {
                    widget.getField("NMPRODUTO").validations = null;
                    widget.getField("NMPRODUTO").isVisible = false;
                    widget.getField("DSRECEITA").isVisible = true;
                    widget.getField("NMLEANCODE").isVisible = false;
                    widget.getField("DSRECEITA").validations = {
                        required: true
                    };
                    widget.getField("DSRECEITA").reload();
                } else {
                    widget.getField("DSRECEITA").validations = null;
                    widget.getField("DSRECEITA").isVisible = false;
                    widget.getField("NMPRODUTO").isVisible = true;
                    widget.getField("NMLEANCODE").isVisible = true;
                    widget.getField("NMPRODUTO").validations = {
                        required: true
                    };
                    widget.getField("NMPRODUTO").reload();
                }

                widget.activate();
                widget.newRow();
                widget.currentRow.NRORDEM = 1;
                InflightorGeneral.justInteger("#NRORDEM, #QTGRAMATURA");

            }
        }
    };

    this.moveRowComposicao = function(widget) {
        if (widget.currentRow.ID == "S") {
            widget.getField("NMLEANCODE").isVisible = false;
        } else if (widget.currentRow.ID == "P") {
            widget.getField("NMLEANCODE").isVisible = true;
        }
    };

    function verificaComposicaoPendenteNovoOuApagado(widget) {
        var currentRow = widget.currentRow;
        var NRITEMPROCAPROVA = currentRow.ID == "S" ? workflowItens.RECEITA_INCLUSAO + "_" + workflowItens.RECEITA_EXCLUSAO : workflowItens.PRODUTO_INCLUSAO + "_" + workflowItens.PRODUTO_EXCLUSAO;
        var IDSITWORKFLOW = "1";
        var IDTIPOPERWFLOW = "_3_2_";
        if (currentRow.ID == "S")
            currentRow.CDRECEITA = currentRow.CODIGO;
        else
            currentRow.CDPRODUTO = currentRow.CODIGO;
        var pks = retornaMetodoSaveIdWorkflow(widget).primaryKeys;
        var row = retornaRowWorkflowPentende(currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, pks);
        return coc03600_menu_designService.hasPendingWorkflow(row).then(function(qtd) {
            if (qtd > 0) {
                MessageService.showMessageByCode("COC03600_WORKFLOW_PENDING");
                return false;
            }
            return true;
        });
    }

    this.editaComposicao = function(widget) {
        InflightorGeneral.justInteger("#NRORDEM, #QTGRAMATURA");
        if (widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA == "A") {
            widget.getField("QTD").readOnly = true;
            widget.getField("QTGRAMATURA").readOnly = true;
            widget.getField("COMENTARIO").readOnly = false;
        } else {
            widget.getField("QTD").isVisible = true;
            widget.getField("QTGRAMATURA").readOnly = false;
            widget.getField("COMENTARIO").readOnly = false;
        }
        verificaComposicaoPendenteNovoOuApagado(widget).then(function(composicaoValida) {
            if (composicaoValida) {
                var promises = [];
                var IDSITWORKFLOW = "1";
                var IDTIPOPERWFLOW = "1";
                if (widget.currentRow.ID == "S") {
                    promises.push(usaWorkflow(workflowItens.NRPROCESSO, workflowItens.RECEITA_ALTERACAO_QTD));
                    promises.push(usaWorkflow(workflowItens.NRPROCESSO, workflowItens.RECEITA_ALTERACAO_GRAMATURA));
                    ZHPromise.all(promises).then(function(dataSource) {
                        widget.idWorkflowReceita = {
                            QTD: dataSource[0],
                            GRAMATURA: dataSource[1]
                        };
                        promises = [];
                        widget.currentRow.CDRECEITA = widget.currentRow.CODIGO;
                        widget.getField("DSRECEITA").isVisible = false;
                        widget.getField("NMPRODUTO").isVisible = false;
                        widget.getField("NMLEANCODE").isVisible = false;
                        widget.getField("NOME").showOnForm = true;
                        var NRITEMPROCAPROVA = workflowItens.RECEITA_ALTERACAO_GRAMATURA;
                        var pks = retornaMetodoSaveIdWorkflow(widget).primaryKeys;
                        var row = retornaRowWorkflowPentende(widget.currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, pks);
                        promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                        row.NRITEMPROCAPROVA = workflowItens.RECEITA_ALTERACAO_QTD;
                        promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                        ZHPromise.all(promises).then(function(dataset) {
                            if (widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA == "A") {
                                widget.getField("QTD").readOnly = true;
                            } else {
                                widget.getField("QTD").readOnly = workflowPendente(dataset[0]);
                            }
                            widget.getField("QTGRAMATURA").readOnly = workflowPendente(dataset[0]);
                            InflightorGeneral.justInteger("#QTGRAMATURA");
                            widget.edit();
                            widget.backupRow = copyRow(widget.currentRow);
                        });
                    });
                } else {
                    promises.push(usaWorkflow(workflowItens.NRPROCESSO, workflowItens.PRODUTO_ALTERACAO_QTD));
                    ZHPromise.all(promises).then(function(dataSource) {
                        widget.idWorkflowProduto = {
                            QTD: dataSource[0]
                        };
                        promises = [];
                        widget.currentRow.CDPRODUTO = widget.currentRow.CODIGO;
                        widget.getField("NMPRODUTO").isVisible = false;
                        widget.getField("DSRECEITA").isVisible = false;
                        widget.getField("NMLEANCODE").isVisible = true;
                        widget.getField("NOME").showOnForm = true;
                        var NRITEMPROCAPROVA = workflowItens.PRODUTO_ALTERACAO_QTD;
                        var pks = retornaMetodoSaveIdWorkflow(widget).primaryKeys;
                        var row = retornaRowWorkflowPentende(widget.currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, pks);
                        promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                        ZHPromise.all(promises).then(function(dataset) {
                            if (widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA == "A") {
                                widget.getField("QTD").readOnly = true;
                            } else {
                                widget.getField("QTD").readOnly = workflowPendente(dataset[0]);
                            }
                            InflightorGeneral.justInteger("#QTGRAMATURA");
                            widget.edit();
                            widget.backupRow = copyRow(widget.currentRow);
                        });
                    });
                }
            }
        });
    };

    this.hideSaveChangesAction = function(widget, fields) {
        var actionsave = widget.getAction("save");
        if (widget.name === "grid_composicao") {
            actionsave.events[0].code = "coc03600_menu_designController.salvaComposicao(args.owner.widget)";
        } else if (widget.name === "widgetProdutocia") {
            actionsave.events[0].code = "coc03600_menu_designController.salvaEquipamentos(args.owner.widget)";
        }

        if (actionsave) {
            actionsave.isVisible = true;
        }
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA)) {
            fields.forEach(function(name) {
                widget.getField(name).inlineEditable = "number";
            });
        } else {
            fields.forEach(function(name) {
                delete widget.getField(name).inlineEditable;
            });
        }
        widget.activate();
    };

    this.cancelarComposicao = function(args, widget) {
        if (widget.getField("NOME").showOnForm == false) {
            widget.getField("NOME").showOnForm = true;
            widget.getField("LEANCODE").showOnForm = true;
            widget.getField("DSRECEITA").isVisible = false;
            widget.getField("NMPRODUTO").isVisible = false;
            widget.getField("NMPRODUTO").validations = null;
            widget.getField("DSRECEITA").validations = null;
            widget.currentRow = {};
            InflightorGeneral.backOrCancel(args);
        } else {
            InflightorGeneral.backOrCancel(args);
        }
        self.carregaDataSourceComposicao(widget);
    };

    this.calculaCusto = function(widget, param) {
        if (param) {
            widget.currentRow.DTCUSTOAP = param.DTCUSTOMAN;
            widget.currentRow.TIPOCUSTO = param.IDTIPOCUSTOMAN;
        }
        coc03600_menu_designService.pendencias_workflow(widget.container.getWidget("grid_manual").currentRow).then(
            function(qtde) {
                if (qtde) {
                    MessageService.showMessageByCode("COC03600_MENU_GRID_PENDING");
                } else {
                    if (widget.isValid()) {
                        var data = [];
                        var grid_manual = widget.container.getWidget("grid_manual");
                        var row = grid_manual.currentRow;
                        data.push({
                            CDFILIAL: row.CDFILIAL,
                            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
                            CDOPERADOR: getLocalVar("INFLIGHTOR_CDOPERADOR"),
                            TIPOCUSTO: widget.currentRow.TIPOCUSTO,
                            DATACUSTO: moment(widget.currentRow.DTCUSTOAP, "DD/MM/YYYY").format("DD/MM/YYYY"),
                            CDCIA: row.CDCIA,
                            CDMANCIA: row.CDMANCIA
                        });

                        coc03600_menu_designService.calculaCustoManual(data).then(function() {
                                ScreenService.closePopup(true);
                                widget.container.getWidget("grid_servicos").reload();
                                self.atualizaCustoMenu(grid_manual).then(function(callback) {
                                    grid_manual.currentRow.DTCUSTOMAN = callback.dataset.manualciam[0].DTCUSTOMAN;
                                    grid_manual.currentRow.IDTIPOCUSTOMAN = callback.dataset.manualciam[0].IDTIPOCUSTOMAN;
                                    grid_manual.currentRow.VRCUSTOMAM = callback.dataset.manualciam[0].VRCUSTOMAM;
                                    grid_manual.activate();
                                    MessageService.showMessageByCode("COC03600_COST_CALCULATED_SUCCESS");
                                });
                            },
                            function(error) {
                                let msg = (error.data && error.data.error) ? error.data.error : "Erro ao calcular o custo do manual";
                                ScreenService.showMessage(msg, "error");
                            });
                    }
                }
            }.bind(this),
            InflightorGeneral.callbackError
        );
    };

    this.atualizaCustoMenu = function(widget) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widget.currentRow.CDMANCIA
            }
        ];
        return InflightorGeneral.callRestEngine("coc#/crud/manualciam", "FilterData", filter);
    };

    /*---------- ESTRUTURA CRIADA PARA CONTROLAR A PROPORÇÃO (HENRIQUE) -------------*/
    this.adicionarProporcao = function(tabPropVar) {
        if (desenvolvimentoOuProposta(getSituacao(tabPropVar), true)) {
            var popupAddProporcao = tabPropVar.widgets[0];
            if (popupAddProporcao.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU == undefined || popupAddProporcao.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU == "") {
                MessageService.showMessageByCode("COC03600_CHOOSE_SUBGROUP");
            } else {
                popupAddProporcao.edit();
                popupAddProporcao.activate();
                popupAddProporcao.newRow();

                ScreenService.openPopup(popupAddProporcao);

                if (tabPropVar.dataSource.data && tabPropVar.dataSource.data.length > 0) {
                    var ultRow = angular.copy(tabPropVar.dataSource.data[0]);
                    tabPropVar.dataSource.data.forEach(function(row) {
                        if (row.QTPAXDE > ultRow.QTPAXDE) ultRow = angular.copy(row);
                    });
                    popupAddProporcao.currentRow.QTPAXDE = parseInt(ultRow.QTPAXATE) + 1;
                } else {
                    popupAddProporcao.currentRow.QTPAXDE = 1;
                }
            }
        }
    };

    this.excluiProporcao = function(tabPropVar) {
        if (desenvolvimentoOuProposta(getSituacao(tabPropVar), true)) {
            ScreenService.confirmMessage("Delete Register?", "question", function() {
                coc03600_menu_designService.removeProporcao(tabPropVar.dataSource.data[tabPropVar.dataSource.data.length - 1]).then(function() {
                    tabPropVar.dataSource.data.pop();
                    if (!tabPropVar.dataSource.data.length) {
                        tabPropVar.getAction("EXCLUIR").isVisible = false;
                    } else {
                        tabPropVar.getAction("EXCLUIR").isVisible = true;
                    }
                }, function(erro) {
                    var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao excluir a proporção";
                    ScreenService.showMessage(msg);
                });

            });
        }
    };

    this.validaPaxMax = function(tabPropVar) {
        if (tabPropVar.currentRow.QTPAXATE < tabPropVar.currentRow.QTPAXDE) {
            MessageService.showMessageByCode("COC03600_MAX_MIN");
            tabPropVar.currentRow.QTPAXATE = "";
        }
    };

    this.onEnterTabPropVar = function(tabPropVar) {
        if (tabPropVar.container.getWidget("widgetSubgrupo").currentRow.__is_new) {
            tabPropVar.dataSource = metaDataFactory.dataSourceFactory({
                "data": tabPropVar.dataSource.data
            });
        } else {
            tabPropVar.dataSource = metaDataFactory.dataSourceFactory({
                "name": "coc#/crud/propsubgruse",
                "rest": true,
                "lazyLoad": true
            });
            tabPropVar.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDFILIAL
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDCIA
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDMANCIA
            }, {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.NRSERVICOCIAFILI
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.NRSEQSUBGRUSEMAM
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDCLASSE
            }, {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": tabPropVar.container.getWidget("widgetSubgrupo").currentRow.NRORG
            }];
            tabPropVar.reload().then(function(callback) {
                if (tabPropVar.dataSource.data.length) {
                    tabPropVar.getAction("EXCLUIR").isVisible = true;
                } else {
                    tabPropVar.getAction("EXCLUIR").isVisible = false;
                }
            });
        }
    };

    this.salvaProporcaoLineEdit = function(tabPropVar) {
        if (desenvolvimentoOuProposta(getSituacao(tabPropVar), true)) {
            var invalidValues = false;
            tabPropVar.dataSource.data.forEach(function(row) {
                row.QTSUBGRUPO = (isNaN(row.QTSUBGRUPO)) ? 1 : row.QTSUBGRUPO;
            });

            if (!invalidValues) {
                if (tabPropVar.parent.currentRow.__is_new) {
                    tabPropVar.commitInlineEdit();
                    tabPropVar.list();
                    tabPropVar.activate();
                } else {
                    InflightorGeneral.saveCrud(tabPropVar).then(function() {
                        tabPropVar.commitInlineEdit();
                        tabPropVar.list();
                        tabPropVar.activate();
                    }, function(erro) {
                        var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao salvar a proporção";
                        ScreenService.showMessage(msg);
                    });
                }
            }
        } else {
            tabPropVar.rollbackInlineEdit();
            tabPropVar.list();
            tabPropVar.activate();
        }
    };

    this.finalizaProporcao = function(popupAddProporcao) {
        if (popupAddProporcao.isValid()) {
            var tabPropVar = popupAddProporcao.parent;
            var dataSourceToSave = "coc#/crud/propsubgruse/save";
            var currentSubGru = popupAddProporcao.container.getWidget("zh_crosstab").currentRow;

            popupAddProporcao.currentRow.CDFILIAL = currentSubGru.CDFILIAL;
            popupAddProporcao.currentRow.CDCIA = currentSubGru.CDCIA;
            popupAddProporcao.currentRow.CDMANCIA = currentSubGru.CDMANCIA;
            popupAddProporcao.currentRow.NRSERVICOCIAFILI = currentSubGru.NRSERVICOCIAFILI;
            popupAddProporcao.currentRow.NRSERVMANUALM = currentSubGru.NRSERVMANUALM;
            popupAddProporcao.currentRow.CDCLASSE = currentSubGru.CDCLASSE;
            popupAddProporcao.currentRow.CDCICLO = currentSubGru.CDCICLO;
            popupAddProporcao.currentRow.CDSUBGRU = currentSubGru.CDSUBGRU;

            InflightorGeneral.callRestEngine(dataSourceToSave, "Row", popupAddProporcao.currentRow).then(function() {
                ScreenService.closePopup();
                tabPropVar.activate();
                tabPropVar.getAction("EXCLUIR").isVisible = true;
            }, function(erro) {
                var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao salvar a proporção";
                ScreenService.showMessage(msg);
            });
        }
    };

    this.salvaProporcao = function(popupAddProporcao) {
        if (popupAddProporcao.isValid()) {
            var tabPropVar = popupAddProporcao.parent;
            var dataSourceToSave = "coc#/crud/propsubgruse/save";

            if (!tabPropVar.container.getWidget("widgetSubgrupo").currentRow.__is_new) {
                popupAddProporcao.currentRow.CDFILIAL = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDFILIAL;
                popupAddProporcao.currentRow.CDCIA = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDCIA;
                popupAddProporcao.currentRow.CDMANCIA = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDMANCIA;
                popupAddProporcao.currentRow.NRSERVICOCIAFILI = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.NRSERVICOCIAFILI;
                popupAddProporcao.currentRow.NRSEQSUBGRUSEMAM = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.NRSEQSUBGRUSEMAM;
                popupAddProporcao.currentRow.CDCLASSE = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDCLASSE;
                popupAddProporcao.currentRow.CDSUBGRU = tabPropVar.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU;
            }

            InflightorGeneral.callRestEngine(dataSourceToSave, "Row", popupAddProporcao.currentRow).then(function() {
                ScreenService.closePopup();
                tabPropVar.reload().then(function() {
                    tabPropVar.activate();
                    tabPropVar.getAction("EXCLUIR").isVisible = true;
                });
            }, function(erro) {
                var msg = (erro.data && erro.data.error) ? msg = erro.data.error : "Erro ao salvar a proporção";
                ScreenService.showMessage(msg);
            });
        }
    };

    this.cancelaLinhaProporcao = function(popupAddProporcao) {
        ScreenService.closePopup();
        popupAddProporcao.parent.activate();
    };

    this.onUploadPhoto = function(widget, photoUpload) {
        setTimeout(function() {
            if (isEmpty(photoUpload)) {
                self.onUploadPhoto(widget, photoUpload);
            }
            var validExt = widget.getField("PHOTO_UPLOAD").validations.extensions.value;
            var fileExt = "." + photoUpload[0].name.split(".").pop();
            if (validExt.indexOf(fileExt) != -1) {
                widget.currentRow.FGFOTOSUBSEMAM = photoUpload[0].b64File;
            } else {
                widget.currentRow.PHOTO_UPLOAD.splice(widget.currentRow.PHOTO_UPLOAD.indexOf(photoUpload), 1);
                widget.getField("PHOTO_UPLOAD").fileList = widget.currentRow.PHOTO_UPLOAD;
                widget.currentRow.FGFOTOSUBSEMAM = window.undefinedImage;
            }
            templateManager.updateTemplate();
        }, 500);
    };

    var isEmpty = function(value) {
        return !value || value.length === 0;
    };

    this.onDeselectPhoto = function(widget) {
        widget.currentRow.FGFOTOSUBSEMAM = window.undefinedImage;
        templateManager.updateTemplate();
    };

    this.cancelModEdit = function(widgetParent, widget) {
        widget.view();
        widgetParent.container.getWidget("grid_composicao").isVisible = true;
        widgetParent.container.getWidget("grid_subgruposPrincipais").isVisible = true;
        widgetParent.container.getWidget("grid_produtoscia").isVisible = true;
        widget.template = widget.formTemplate = widget.templateToView;
    };

    this.modEdit = function(widgetParent, widget) {
        InflightorGeneral.edit(widget);
        widgetParent.container.getWidget("grid_composicao").isVisible = false;
        widgetParent.container.getWidget("grid_subgruposPrincipais").isVisible = false;
        widgetParent.container.getWidget("grid_produtoscia").isVisible = false;
        widget.template = widget.formTemplate = widget.templateToForm;
    };

    this.authUser = function(widget) {
        if (widget.isValid()) {
            var user = widget.currentRow.user;
            var senha = widget.currentRow.senha;
            coc03600_menu_designService.autenticar(user, senha).then(function() {
                ScreenService.closePopup();
                if (angular.isFunction(widget.callback)) {
                    widget.callback.apply(null, widget.params);
                }
            }, function(callback) {
                InflightorGeneral.callbackError(callback);
            });
        }
    };

    this.confirmaProposta = function(widget, status) {
        var row = {
            NRORG: widget.currentRow.NRORG,
            CDMANCIA: widget.currentRow.CDMANCIA,
            CDFILIAL: widget.currentRow.CDFILIAL,
            CDCIA: widget.currentRow.CDCIA,
            NRPROCESSOAPROVA: 4
        };
        if (status == "E" && widget.currentRow.IDSITUAPROPCIA == "R") {
            var NRPROCESSOAPROVA = workflowItens.NRPROCESSO;
            var NRITEMPROCAPROVA = workflowItens.MG_ALTERA_REPROVADO_DESENVOLVIMENTO;
            var rowPendencias = {
                NRPROCESSOAPROVA: NRPROCESSOAPROVA,
                NRITEMPROCAPROVA: NRITEMPROCAPROVA,
                IDSITWORKFLOW: 1,
                IDTIPOPERWFLOW: null,
                DSPRIMARYKEY: "NRORG=" + row.NRORG + ";CDMANCIA=" + row.CDMANCIA + ";CDCIA=" + row.CDCIA + ";CDFILIAL=" + row.CDFILIAL + ";"
            };
            coc03600_menu_designService.hasPendingWorkflow(rowPendencias).then(function(qtd) {
                if (!workflowPendente(qtd, false)) {
                    usaWorkflow(NRPROCESSOAPROVA, NRITEMPROCAPROVA).then(function(idUsaWorkflow) {
                        widget.usaWorkflow = idUsaWorkflow;
                        exibeConfirmacaoAlteracaoMenuGrid(widget, status);
                    });
                } else {
                    MessageService.showMessageByCode("COC03600_WORKFLOW_PENDING_OPERATION");
                }
            });
        } else if (status == "A" || status == "R") {
            coc03600_menu_designService.verificaWorkflowPendenteMenuGrid(row).then(function(workflowPendente) {
                if (workflowPendente) {
                    MessageService.showMessageByCode("COC03600_WORKFLOW_PENDING_MENU_GRID");
                } else {
                    exibeConfirmacaoAlteracaoMenuGrid(widget, status);
                }
            });
        } else { // para proposta
            exibeConfirmacaoAlteracaoMenuGrid(widget, status);
        }
    };

    function exibeConfirmacaoAlteracaoMenuGrid(widget, status) {
        ScreenService.closePopup();
        widget.statusProposta = status;
        self.callAuthUser(self.alteraStatusMenu, [widget.container.getWidget("grid_manual")]);
    }

    this.callAuthUser = function(callback, params) {
        var widget_autenticar = templateManager.container.getWidget("widget_autenticar");
        widget_autenticar.callback = callback;
        widget_autenticar.params = params;
        widget_autenticar.newRow();
        ScreenService.openPopup(widget_autenticar);
    };

    this.closePopupAuth = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget("grid_manual").activate();
    };

    this.solicitaCriacaoWorkflow = function(widget, currentRow, saveMethod, afterSave) {
        /*JULIAOWF*/
        var motivoWidget = {
            "name": "motivoWidget",
            "label": "Justificativa",
            "template": "widget/form.html",
            "container": {},
            "rowToSave": currentRow,
            "saveMethod": saveMethod || null,
            "afterSave": afterSave || null,
            "parentWidget": widget,
            "isVisible": true,
            "popupNoBlock": false,
            "events": [],
            "fields": [
                {
                    "name": "DSMOTIVOACAO",
                    "label": "Motivo",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                },
                {
                    "name": "IDSITOPERWFLOW",
                    "label": "",
                    "template": "",
                    "isVisible": false,
                    "readOnly": false,
                    "showOnForm": true,
                }
            ],
            "actions": [
                {
                    "name": "cancelar",
                    "label": "Cancelar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": "Confirmar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "coc03600_menu_designController.validaMotivo(args.owner.widget);"
                        }
                    ]
                }
            ]
        };
        motivoWidget.container = widget.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(motivoWidget);
        ScreenService.openPopup(motivoWidgetInitialized);
    };

    this.validaMotivo = function(widget) {
        if (widget.isValid()) {
            if (widget.saveMethod) {
                widget.saveMethod(widget);
            } else {
                widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                coc03600_menu_designService.salvaWorkflow(widget.rowToSave).then(function(callback) {
                    ScreenService.showMessage(callback.messages[0].message);
                    ScreenService.closePopup();
                    if (widget.afterSave)
                        widget.afterSave(widget);
                }, function(err) {
                    if (err.data) {
                        ScreenService.showMessage(err.data.error);
                    } else {
                        MessageService.showMessageByCode("TIMEOUT");
                    }
                });
            }
        }
    };

    function retornaRowParaSalvarAprovacaoWF(widget) {
        var rowToSave = angular.copy(widget.currentRow);
        rowToSave.IDSITUAPROPCIA = widget.statusProposta;
        rowToSave.NRPROCESSOAPROVA = workflowItens.NRPROCESSO;
        rowToSave.NRITEMPROCAPROVA = workflowItens.APROVA_MENU_GRID;
        rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + widget.currentRow.NMCIA +
            " \nMenu Grid: " + widget.currentRow.DSMANUAL;
        rowToSave.NMTABELA = "MANUALCIAM";
        rowToSave.IDTIPOPERWFLOW = "1";
        rowToSave.CHANGEDITEMS = {
            IDSITUAPROPCIA: rowToSave.IDSITUAPROPCIA
        };
        rowToSave.PRIMARYKEYS = {
            NRORG: widget.currentRow.NRORG,
            CDMANCIA: widget.currentRow.CDMANCIA,
            CDCIA: widget.currentRow.CDCIA,
            CDFILIAL: widget.currentRow.CDFILIAL
        };
        return rowToSave;
    }

    function retornaRowParaSalvarReprovadoPropostaWF(widget) {
        var rowToSave = angular.copy(widget.currentRow);
        rowToSave.IDSITUAPROPCIA = widget.statusProposta;
        rowToSave.NRPROCESSOAPROVA = 4;
        rowToSave.NRITEMPROCAPROVA = 2;
        rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + widget.currentRow.NMCIA +
            " \nMenu Grid: " + widget.currentRow.DSMANUAL;
        rowToSave.NMTABELA = "MANUALCIAM";
        rowToSave.IDTIPOPERWFLOW = "1";
        rowToSave.CHANGEDITEMS = {
            IDSITUAPROPCIA: rowToSave.IDSITUAPROPCIA
        };
        rowToSave.PRIMARYKEYS = {
            NRORG: widget.currentRow.NRORG,
            CDMANCIA: widget.currentRow.CDMANCIA,
            CDCIA: widget.currentRow.CDCIA,
            CDFILIAL: widget.currentRow.CDFILIAL
        };
        return rowToSave;
    }

    function workflowPendente(qtd, showMessage) {
        if (qtd > 0) {
            if (showMessage) {
                MessageService.showMessageByCode("COC03600_WORKFLOW_PENDING");
            }
            return true;
        }
        return false;
    }

    function usaWorkflow(NRPROCESSOAPROVA, NRITEMPROCAPROVA) {
        return coc01400_receitaService.getProcessosWorkflow(NRPROCESSOAPROVA, NRITEMPROCAPROVA).then(function(callback) {
            var processos = callback.dataset.processos_workflow;
            if (processos.length && processos[0].IDATIVO == "S") {
                return true;
            }
            return false;
        });
    }

    function altaraMenuGridAprovado(widget) {
        coc03600_menu_designService.validaMenuGrid(widget.currentRow).then(function(callback) {
            var erros = callback.dataset.errosMenuGrid;
            if (erros.length) {
                var deferred = ZHPromise.defer();
                InflightorGeneral.showErrorMessages(widget, erros);
                deferred.reject("Erros encontrados");
                return deferred.promise;
            } else {
                return coc03600_menu_designService.validaInformacoesDependentesMenuGrid(widget.currentRow);
            }
        }).then(function(callback) {
            var erros;
            if (callback && callback.dataset)
                erros = callback.dataset.errosMenuGrid;
            if (erros && erros.length) {
                var deferred = ZHPromise.defer();
                InflightorGeneral.showErrorMessages(widget, erros);
                deferred.reject("Erros encontrados");
                return deferred.promise;
            } else {
                usaWorkflow(workflowItens.NRPROCESSO, workflowItens.APROVA_MENU_GRID).then(function(idUsaWorkflow) {
                    if (idUsaWorkflow) {
                        widget.idUsaWorkflow = idUsaWorkflow;
                        var afterSave = function(widget) {
                            ScreenService.closePopup();
                            widget.parentWidget.reload();
                            coc03600_menu_designService.retornaFlagWorkflowPendenteAprovacaoMenuGrid(widget.currentRow);
                        };
                        var rowToSave = retornaRowParaSalvarAprovacaoWF(widget);
                        rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + widget.currentRow.NMCIA +
                            " \nMenu Grid: " + widget.currentRow.DSMANUAL;
                        self.solicitaCriacaoWorkflow(widget, rowToSave, null, afterSave);
                    } else {
                        widget.currentRow.IDSITUAPROPCIA = widget.statusProposta;
                        coc03600_menu_designService.saveMenu([widget.currentRow]).then(function() {
                            widget.reload();
                            alteraActionsParaAprovacao(widget);
                            widget.activate();
                            coc03600_menu_designService.retornaFlagWorkflowPendenteAprovacaoMenuGrid(widget.currentRow);
                        }, InflightorGeneral.callbackError);
                    }
                }, InflightorGeneral.callbackError);
            }
        }).catch(function(err) {
            if (err && err.data && err.data.error) {
                ScreenService.showMessage(err.data.error);
            }
        });
    }

    function alteraSituacaoMenuGrid(widget) {
        widget.currentRow.IDSITUAPROPCIA = widget.statusProposta;
        coc03600_menu_designService.saveMenu([widget.currentRow]).then(function() {
            widget.reload();
            alteraActionsParaAprovacao(widget);
            widget.activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    }

    this.alteraStatusMenu = function(widget) {
        if (widget.statusProposta == "A") {
            altaraMenuGridAprovado(widget);
        } else if (widget.statusProposta == "E" && widget.currentRow.IDSITUAPROPCIA == "R") {
            if (widget.usaWorkflow) {
                var afterSave = function(widget) {
                    ScreenService.closePopup();
                    widget.parentWidget.reload();
                };
                var rowToSave = retornaRowParaSalvarReprovadoPropostaWF(widget);
                rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + widget.currentRow.NMCIA +
                    " \nMenu Grid: " + widget.currentRow.DSMANUAL;
                self.solicitaCriacaoWorkflow(widget, rowToSave, null, afterSave);
            } else {
                alteraSituacaoMenuGrid(widget);
            }
        } else {
            alteraSituacaoMenuGrid(widget);
        }
    };

    this.replicarCiclo = function(widgetCopiaCiclo) {
        if (desenvolvimentoOuProposta(getSituacao(widgetCopiaCiclo), true)) {
            this.buildFilterCopyCiclo(widgetCopiaCiclo.getField('IDCICLOORIGEM'));
            ScreenService.openPopup(widgetCopiaCiclo);
            widgetCopiaCiclo.newRow();
            widgetCopiaCiclo.activate();
        }
    };

    /*   --------------  HANDLE DE AÇÕES POR CONTA DO PROCESSO DE APROVAÇÃO  -------------   */
    this.adicionarServico = function(widget) {
        if (getSituacao(widget, true) == "P" || getSituacao(widget, true) == "E") {
            widget.getField("DSCLASSE").template = "field/select-multiple.html";
            widget.container.getWidget("zh_crosstab").isVisible = false;
            widget.dataSource.newRow();
            let row = widget.dataSource.data[widget.dataSource.data.length - 1];
            widget.currentRow = row;
            widget.currentRow.__index = widget.dataSource.data.indexOf(row);
            if (templateManager.project.showDescriptionOrder) {
                widget.computeDescriptionOrder();
            }
            widget.applyDefaultValue();
            let fieldService = widget.getField('DSSERVICOCIAFILI');
            let fieldClass = widget.getField('DSCLASSE');
            let filterRow = widget.container.getWidget('grid_manual').appliedFilter.FILTER_ROW;
            updateFilterFields(fieldService, ['CDFILIAL', 'CDCIA'], filterRow);
            updateFilterFields(fieldClass, ['CDFILIAL', 'CDCIA'], filterRow);
            widget.edit();
            // widget.activate();
            // ScreenService.openSwipe(widget);
        } else if (getSituacao(widget, true) == "A") {
            MessageService.showMessageByCode("COC03600_MENU_GRID_ON_APPROVAL");
        }
    };

    this.editServico = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            InflightorGeneral.edit(widget);
            widget.container.getWidget("zh_crosstab").isVisible = false;
        }
    };

    this.deleteServico = function(widget) {
        /*juliaoservico*/
        if (getSituacao(widget, true) == "P") {
            ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                var primaryKeys = {
                    CDFILIAL: "",
                    CDCIA: "",
                    CDMANCIA: "",
                    NRSERVICOCIAFILI: "",
                    CDCLASSE: "",
                    NRORG: ""
                };
                var dataFilter = retornaRowWorkflowPentende(widget.currentRow, "7", "1", "3", Object.keys(primaryKeys));
                coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
                    function(qtde) {
                        if (qtde) {
                            MessageService.showMessageByCode("COC03600_DELETE_ERROR_PROCESS_WORKFLOW");
                        } else {
                            //implementar função que tenta excluir dados em transaction
                            coc03600_menu_designService.tryDeleteWithTransaction(widget.currentRow).then(
                                function() {
                                    // criar metodo que implementa este delete
                                    implementDeleteServico(widget);
                                },
                                function(error) {
                                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                        MessageService.showMessageByCode("DELETE_ERROR");
                                    } else {
                                        ScreenService.showMessage(error.data.error, "error");
                                    }
                                }
                            );
                        }
                    }
                );
            });
        } else if (getSituacao(widget, true) == "E") {
            coc03600_menu_designService.deleteServico([widget.currentRow]).then(function() {
                widget.reload().then(function() {
                    widget.activate();
                    widget.unSelectAll();
                    MessageService.showMessageByCode("DELETE_SUCCESS");
                });
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error, "error");
                }
            });
        }
    };

    function implementDeleteServico(widget) {
        var afterWorkflow = function(widget) {
            /*juliaoWF*/
            widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            coc03600_menu_designService.salvaWorkflow(widget.rowToSave).then(
                function() {
                    ScreenService.closePopup();
                    MessageService.showMessageByCode("WORKFLOW_SUCCESS");
                    var widgetServico = widget.container.getActiveWidget();
                    widget.container.getWidget("zh_crosstab").isVisible = false;
                    widget.container.getWidget("gris_subgrupo").isVisible = false;
                    widgetServico.unSelectAll();
                    widgetServico.list();
                    widgetServico.activate();
                }, InflightorGeneral.callbackError
            );
        };

        usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SERVICO_EXCLUSAO).then(
            function(usaWF) {
                if (usaWF) {
                    var changedItems = {
                        VRCUSTOSERVM: " "
                    };
                    var pks = {
                        CDFILIAL: widget.currentRow.CDFILIAL,
                        CDCIA: widget.currentRow.CDCIA,
                        CDMANCIA: widget.currentRow.CDMANCIA,
                        NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
                        CDCLASSE: widget.currentRow.CDCLASSE,
                        NRORG: widget.currentRow.NRORG
                    };
                    var variables = {
                        nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                        nmServico: widget.currentRow.DSSERVICOCIAFILI,
                        dsClasse: widget.currentRow.DSCLASSE,
                        unidade: getLocalVar("NMFILIAL"),
                        companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA
                    };
                    var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__", variables);
                    var workflowData = {
                        CHANGEDITEMS: changedItems,
                        PRIMARYKEYS: pks,
                        NMITEM: translateNMITEM,
                        NMTABELA: "SERVMANUALM",
                        IDTIPOPERWFLOW: "3",
                        NRITEMPROCAPROVA: workflowItens.SERVICO_EXCLUSAO,
                        NRPROCESSOAPROVA: workflowItens.NRPROCESSO /*JULIAOSAVE*/
                    };

                    self.solicitaCriacaoWorkflow(widget, workflowData, afterWorkflow);
                } else {
                    coc03600_menu_designService.deleteServico([widget.currentRow]).then(function() {
                        widget.reload();
                        widget.activate();
                    }, function(error) {
                        if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                            MessageService.showMessageByCode("DELETE_ERROR");
                        } else {
                            ScreenService.showMessage(error.data.error);
                        }
                    });
                }
            }
        );
    }

    this.editSubgrupo = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            InflightorGeneral.edit(widget);
            widget.container.getWidget("view_montagem").isVisible = false;
            widget.container.getWidget("grid_composicao").isVisible = false;
            widget.container.getWidget("grid_subgruposPrincipais").isVisible = false;
            widget.container.getWidget("grid_produtoscia").isVisible = false;
            self.alteraProporcao(widget);
        }
    };

    this.excluirSubgrupo = function(widget) {
        widget.removeCheckedRows();
        widget.unSelectAll();
    };

    this.ajustaFiltro = function(widget) {
        // var removeSituation = false;
        if (!widget.dataSource.data) {
            widget.dataSource.data = [];
        }
        widget.dataSource.unSelectAll();
        var appliedFilter = widget.appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        IDUTIAIRVISION = appliedFilter.FILTER_ROW.IDUTIAIRVISION;
        filterCdfilial = appliedFilter.FILTER_ROW.CDFILIAL;
        filterNmfilial = appliedFilter.FILTER_ROW.NMFILIAL;
        filterCdcia = appliedFilter.FILTER_ROW.CDCIA;
        filterNmcia = appliedFilter.FILTER_ROW.NMCIA;
        // widget.dataSourceFilter.forEach(function(condicao) {
        //     if (condicao.name === "IDVIGENCIA") {
        //         condicao.operator = "=";
        //         condicao.value = condicao.value.replace(/%/g, "");
        //     } else if (condicao.name == "IDSITUAPROPCIA" && condicao.value.indexOf("T") != -1) {
        //         removeSituation = true;
        //     }
        // });
        // if (removeSituation) {
        //     widget.dataSourceFilter = widget.dataSourceFilter.filter(function(filterObj) {
        //         return filterObj.name != "IDSITUAPROPCIA";
        //     });
        // }
    };

    function verificaOrdemItensDaCompanhia(row) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": row.NRORG
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": row.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": row.CDCLASSE
            },
            {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": row.NRSEQSUBGRUSEMAM
            },
            {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": row.CDSUBGRU
            }
        ];
        return InflightorGeneral.callRestEngine("coc#/crud/prociasubsemam", "FilterData", filter);
    }

    function validaOrdenacaoItensCompanhia(itens, widget) {
        var novasOrdens = _chagendAndSave(widget, widget.currentRow.NRORDEM);
        return novasOrdens;
    }

    function instrucaoMontagemWF(widget) {
        //chama metodo que verifica o uso do workflow
        var montagem = widget.currentRow.DSMONTASUBSEMAMHTML;
        var afterWorkflow = function(widget) {
            widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            coc03600_menu_designService.salvaWorkflow(widget.rowToSave).then(
                function() {
                    afterSave();
                }
            );
        };
        var afterSave = function(callback) {
            var widget = templateManager.currentWidget;
            widget.container.getWidget("grid_composicao").isVisible = true;
            widget.container.getWidget("grid_subgruposPrincipais").isVisible = true;
            self.loadDataSource();
            if (widget.name == "motivoWidget") {
                ScreenService.showMessage(callback.messages[0].message);
                ScreenService.closePopup();
                widget = widget.container.getActiveWidget();
                if (widget.currentRow.__is_new) {
                    widget.currentRow.DSMONTASUBSEMAMHTML = "";
                } else {
                    widget.currentRow.DSMONTASUBSEMAMHTML = widget.currentRow.DSMONTASUBSEMAMHTMLBKP;
                }
            }
            if (widget.currentRow.__is_new) {
                widget.list(true);
                ScreenService.goBack();
                widget.reload();
            } else {
                widget.view();
            }
        };

        if (getSituacao(widget, false) == "P") {
            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.ALTERA_INSTRUCAO_MONTAGEM).then(
                function(usaWF) {
                    widget.currentRow.usaWF = usaWF;
                    if (usaWF) {
                        var changedItems = {
                            IDANALIZEATIV: "A",
                            //DSMONTASUBSEMAMHTML: coc03600_menu_designService.htmlentities(montagem)
                            DSMONTASUBSEMAMHTML: montagem
                        };
                        var pks = {
                            CDCICLO: widget.currentRow.CDCICLO,
                            NRSEQSUBGRUSEMAM: widget.currentRow.NRSEQSUBGRUSEMAM,
                            CDSUBGRU: widget.currentRow.CDSUBGRU,
                            CDCLASSE: widget.currentRow.CDCLASSE,
                            NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
                            CDMANCIA: widget.currentRow.CDMANCIA,
                            CDCIA: widget.currentRow.CDCIA,
                            CDFILIAL: widget.currentRow.CDFILIAL,
                            NRORG: widget.currentRow.NRORG
                        };
                        var variables = {
                            nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                            nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                            nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                            dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                            unidade: getLocalVar("NMFILIAL"),
                            companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                            idCiclo: widget.currentRow.IDCICLO
                        };
                        var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__", variables);
                        var workflowData = {
                            CHANGEDITEMS: changedItems,
                            PRIMARYKEYS: pks,
                            NMITEM: translateNMITEM,
                            NMTABELA: "MONTASUBSEMAM",
                            IDTIPOPERWFLOW: "1",
                            NRITEMPROCAPROVA: workflowItens.ALTERA_INSTRUCAO_MONTAGEM,
                            NRPROCESSOAPROVA: workflowItens.NRPROCESSO /*JULIAOSAVE*/
                        };
                        if (widget.currentRow.__is_new) {
                            coc03600_menu_designService.salvaIntMontagem([getDataSaveInstMontagem(widget)]).then(
                                function() {
                                    self.solicitaCriacaoWorkflow(widget, workflowData, afterWorkflow);
                                },
                                function(error) {
                                    ScreenService.showMessage(error.data.error);
                                }
                            );
                        } else {
                            self.solicitaCriacaoWorkflow(widget, workflowData, afterWorkflow);
                        }
                    } else {
                        var dataSaveBeforeWF = getDataSaveInstMontagem(widget);
                        dataSaveBeforeWF.IDANALIZEATIV = "A";
                        //dataSaveBeforeWF.DSMONTASUBSEMAMHTML = coc03600_menu_designService.htmlentities(montagem);
                        dataSaveBeforeWF.DSMONTASUBSEMAMHTML = montagem;
                        coc03600_menu_designService.salvaIntMontagem([dataSaveBeforeWF]).then(
                            function(callback) {
                                afterSave(callback);
                            },
                            function(error) {
                                ScreenService.showMessage(error.data.error);
                            }
                        );
                    }
                }
            );
        } else {
            var dataSaveBeforeWF = getDataSaveInstMontagem(widget);
            dataSaveBeforeWF.IDANALIZEATIV = "A";
            //dataSaveBeforeWF.DSMONTASUBSEMAMHTML = coc03600_menu_designService.htmlentities(montagem);
            dataSaveBeforeWF.DSMONTASUBSEMAMHTML = montagem;
            coc03600_menu_designService.salvaIntMontagem([dataSaveBeforeWF]).then(
                function(callback) {
                    afterSave(callback);
                },
                function(error) {
                    ScreenService.showMessage(error.data.error);
                }
            );
        }
    }

    function getDataSaveInstMontagem(widget) {
        var dataSaveBeforeWF = angular.copy(widget.currentRow);
        dataSaveBeforeWF.IDANALIZEATIV = "P";
        dataSaveBeforeWF.DSMONTASUBSEMAM1 = "";
        dataSaveBeforeWF.DSMONTASUBSEMAMHTML = "";
        dataSaveBeforeWF.DSMONTASUBSEMAM = "";
        return dataSaveBeforeWF;
    }

    this.salvaEquipamentos = function(widget) {
        widget.currentRow.DSMONTASUBSEMAMHTML = FieldUtil.changeTagRichText(widget.currentRow.DSMONTASUBSEMAMHTML);
        var cdsubgru, nrseqsubgrusemam;
        var montagem;
        var isNewMontagem = false;
        widget.dataSourceFilter.forEach(function(filter) {
            if (filter.name == "CDSUBGRU") {
                cdsubgru = filter.value;
            } else if (filter.name == "NRSEQSUBGRUSEMAM") {
                nrseqsubgrusemam = filter.value;
            }
        });
        widget.currentRow.CDFILIAL = widget.container.getWidget("grid_manual").currentRow.CDFILIAL;
        widget.currentRow.CDCIA = widget.container.getWidget("grid_manual").currentRow.CDCIA;
        widget.currentRow.CDMANCIA = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
        widget.currentRow.NRSERVICOCIAFILI = widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI;
        widget.currentRow.CDCLASSE = widget.container.getWidget("grid_servicos").currentRow.CDCLASSE;
        widget.currentRow.CDSUBGRU = cdsubgru;
        widget.currentRow.NRSEQSUBGRUSEMAM = nrseqsubgrusemam;
        widget.currentRow.NRORG = widget.container.getWidget("grid_servicos").currentRow.NRORG;
        widget.currentRow.PHOTO_UPLOAD = typeof widget.currentRow.PHOTO_UPLOAD == 'string' && !widget.currentRow.PHOTO_UPLOAD ? [] : widget.currentRow.PHOTO_UPLOAD;
        if (widget.name == "widgetCiclo") {
            if (!widget.currentRow.PHOTO_UPLOAD.length && widget.currentRow.FGFOTOSUBSEMAM == window.undefinedImage) {
                widget.currentRow.FGFOTOSUBSEMAM = null;
            }
            if ((widget.currentRow.EXIST_INSTRUCAO > 0 && widget.currentRow.DSMONTASUBSEMAMHTML !== widget.currentRow.DSMONTASUBSEMAMHTMLBKP) || (widget.currentRow.EXIST_INSTRUCAO == 0 && widget.currentRow.DSMONTASUBSEMAMHTML)) {
                montagem = widget.currentRow.DSMONTASUBSEMAMHTML;
                if (widget.currentRow.DSMONTASUBSEMAMHTMLBKP == "new")
                    isNewMontagem = true;
                // widget.currentRow.DSMONTASUBSEMAMHTMLBKP = "";
                coc03600_menu_designService.saveCicloSub([widget.currentRow]).then(function() {
                    widget.currentRow.__is_new = isNewMontagem;
                    instrucaoMontagemWF(widget);
                    if (!widget.currentRow.PHOTO_UPLOAD.length && widget.currentRow.FGFOTOSUBSEMAM == null) {
                        widget.currentRow.FGFOTOSUBSEMAM = window.undefinedImage;
                    }
                }, function(error) {
                    var errorMessage = "Timeout";
                    if (error.data) {
                        errorMessage = error.data.error;
                        if (errorMessage.indexOf("Impossivel")) {
                            errorMessage = "Impossivel adicionar. Ciclo já cadastrado para este subgrupo.";
                        }
                    }
                    ScreenService.showMessage(errorMessage);
                    widget.currentRow.DSMONTASUBSEMAMHTML = montagem;
                });
            } else {
                widget.currentRow.PHOTO_UPLOAD = widget.currentRow.PHOTO_UPLOAD ? widget.currentRow.PHOTO_UPLOAD : [];
                coc03600_menu_designService.saveCicloSub([widget.currentRow]).then(function() {
                    widget.currentRow.FGFOTOSUBSEMAM = widget.currentRow.FGFOTOSUBSEMAM != null ? widget.currentRow.FGFOTOSUBSEMAM : window.undefinedImage;
                    ScreenService.closeSwipe();
                    widget.reload();
                    if (widget.name == "widgetCiclo") {
                        widget.reload().then(function() {
                            self.refresh();
                        });
                    }
                    widget.container.getWidget("grid_composicao").isVisible = true;
                    widget.container.getWidget("grid_subgruposPrincipais").isVisible = true;
                });
            }
        } else if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            self.salvaItemCia(widget);
        }
    };

    this.replicaManual = function(widget) {
        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        ScreenService.openPopup(widget.container.getWidget("widgetCopiaMenu"));
        widget.container.getWidget("widgetCopiaMenu").getField("DSMANUALORI").defaultValue = widget.currentRow.DSMANUAL;
        widget.container.getWidget("widgetCopiaMenu").getField("DSMANUALDES").defaultValue = widget.currentRow.DSMANUAL;
        widget.container.getWidget("widgetCopiaMenu").getField("NMFILIALORI").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.container.getWidget("widgetCopiaMenu").getField("NMFILIALDES").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.container.getWidget("widgetCopiaMenu").getField("NMCIAORI").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        widget.container.getWidget("widgetCopiaMenu").getField("NMCIADES").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        widget.container.getWidget("widgetCopiaMenu").getField("CPFOTOSUBGRU").checked = false;
        widget.container.getWidget("widgetCopiaMenu").newRow();
        widget.container.getWidget("widgetCopiaMenu").activate();
    };

    this.exportaManual = function(widget) {
        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        ScreenService.openPopup(widget.container.getWidget("widgetExportMenu"));
        widget.container.getWidget("widgetExportMenu").getField("NMFILIALORI").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.container.getWidget("widgetExportMenu").getField("NMCIAORI").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        widget.container.getWidget("widgetExportMenu").newRow();
        widget.container.getWidget("widgetExportMenu").currentRow.CDFILIALORI = appliedFilter.FILTER_ROW.CDFILIAL;
        widget.container.getWidget("widgetExportMenu").currentRow.CDCIAORI = appliedFilter.FILTER_ROW.CDCIA;
        var manuais = "";
        var linhasMarcadas = widget.getCheckedRows();
        if (linhasMarcadas.length) {
            for (let i = 0; i < linhasMarcadas.length; i++) {
                manuais += linhasMarcadas[i].DSMANUAL;
                if (i !== linhasMarcadas.length - 1)
                    manuais += " - ";
                widget.container.getWidget("widgetExportMenu").currentRow.CDMANCIAORI += "_" + linhasMarcadas[i].CDMANCIA;
            }
            widget.container.getWidget("widgetExportMenu").getField("DSMANUALORI").setValue(manuais);
        } else {
            widget.container.getWidget("widgetExportMenu").currentRow.DSMANUALORI = widget.currentRow.DSMANUAL;
            widget.container.getWidget("widgetExportMenu").currentRow.CDMANCIAORI = widget.currentRow.CDMANCIA;
        }
        widget.container.getWidget("widgetExportMenu").activate();
        widget.container.getWidget("widgetExportMenu").getField("CPFOTOSUBGRU").checked = false;
    };

    this.importaManual = function(widget) {
        var appliedFilter = widget.appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        ScreenService.openPopup(widget.container.getWidget("widgetImportMenu"));
        widget.container.getWidget("widgetImportMenu").getField("NMFILIALDEST").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.container.getWidget("widgetImportMenu").getField("NMCIADEST").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        widget.container.getWidget("widgetImportMenu").newRow();
        widget.container.getWidget("widgetImportMenu").currentRow.CDFILIALDES = appliedFilter.FILTER_ROW.CDFILIAL;
        widget.container.getWidget("widgetImportMenu").currentRow.CDCIADES = appliedFilter.FILTER_ROW.CDCIA;
        widget.container.getWidget("widgetImportMenu").activate();
        widget.container.getWidget("widgetImportMenu").getField("CPFOTOSUBGRU").checked = false;
    };

    this.FieldOnChangeDsmanual = function(args) {
        if (args.row.CDFILIAL) {
            args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = args.row.CDFILIAL;
            if (args.row.CDCIA !== "") {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = args.row.CDCIA;
                args.owner.owner.widget.getField("DSMANUAL").reload();
            } else {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = "";
                args.owner.owner.widget.getField("DSMANUAL").reload();
                let msg = ScreenService.i18n("Escolha um Companhia para definir o Manual!");
                ScreenService.showMessage(msg);
            }
        } else {
            args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = "";
            args.owner.owner.widget.getField("DSMANUAL").reload();
            let msg = ScreenService.i18n("Escolha uma Unidade para definir o Manual!");
            ScreenService.showMessage(msg);
        }
    };

    var deleteCiclo = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
            var row = widget.currentRow;
            var primaryKeys = {
                CDCICLO: "",
                NRSEQSUBGRUSEMAM: "",
                CDSUBGRU: "",
                CDCLASSE: "",
                NRSERVICOCIAFILI: "",
                CDMANCIA: "",
                CDCIA: "",
                CDFILIAL: "",
                NRORG: ""
            };
            var dataFilter = retornaRowWorkflowPentende(widget.currentRow, "15", "1", "1", Object.keys(primaryKeys));
            coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
                function(qtde) {
                    if (qtde) {
                        MessageService.showMessageByCode("COC03600_ASSEMBLY_INSTRUCTION_PENDING");
                    } else {
                        coc03600_menu_designService.deleteMontagem([row]).then(
                            function() {
                                widget.removeRow().then(function() {
                                        widget.reload();
                                        if (widget.name == "widgetCiclo") {
                                            widget.reload().then(function() {
                                                self.refresh();
                                            });
                                        }
                                        ScreenService.closeSwipe().then(function() {
                                            widget.activate();
                                        });
                                    },
                                    function(error) {
                                        if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                            MessageService.showMessageByCode("DELETE_ERROR");
                                        } else {
                                            ScreenService.showMessage(error.data.error);
                                        }
                                    });
                            },
                            function(error) {
                                ScreenService.showMessage(error.data.error);
                            }
                        );
                    }
                }
            );
        });
    };

    this.excluiEquipamentos = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            if (widget.name == "widgetCiclo") {
                deleteCiclo(widget);
                // InflightorGeneral.deleteCrud(widget).then(function (sucess) {
                // 	coc03600_menu_designService.deleteMontagem([row]);
                // });
            } else {
                self.deletaItemCia(widget);
            }
        }
    };

    function retornaRowWorkflowPentende(currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, pks) {
        return {
            NRPROCESSOAPROVA: workflowItens.NRPROCESSO,
            NRITEMPROCAPROVA: NRITEMPROCAPROVA,
            DSPRIMARYKEY: buildDSPrimaryKeys(pks, currentRow),
            IDSITWORKFLOW: IDSITWORKFLOW,
            IDTIPOPERWFLOW: IDTIPOPERWFLOW
        };
    }

    function retornaRowWorkflowPendenteMenuGrid(currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW) {
        return retornaRowWorkflowPentende(currentRow, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, menuGridPrimaryKeys);
    }

    function buildDSPrimaryKeys(pks, currentRow) {
        var builddedPks = "";
        pks.forEach(function(pk) {
            builddedPks += pk + "=" + currentRow[pk] + ";";
        });
        return builddedPks;
    }

    this.editarManual = function(widget) {
        widget.getField("DTINIVGMA").readOnlyOnEdit = false;

        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }

        if (getSituacao(widget) == "P") {
            var promises = [];
            var NRPROCESSO = workflowItens.NRPROCESSO;
            promises.push(usaWorkflow(NRPROCESSO, workflowItens.MG_ALTERA_NOME));
            promises.push(usaWorkflow(NRPROCESSO, workflowItens.MG_ALTERA_DATA_APRESENTACAO));
            promises.push(usaWorkflow(NRPROCESSO, workflowItens.MG_ALTERA_LOCAL_APRESENTACAO));
            promises.push(usaWorkflow(NRPROCESSO, workflowItens.MG_ALTERA_VIGENCIA_MANUAL));
            ZHPromise.all(promises).then(function(dataset) {
                widget.usaWorkflow = {
                    NOME: dataset[0],
                    DATA: dataset[1],
                    LOCAL: dataset[2],
                    VIGENCIA: dataset[3]
                };
                promises = [];
                var row = retornaRowWorkflowPendenteMenuGrid(widget.currentRow, null, "1", "1");
                row.NRITEMPROCAPROVA = workflowItens.MG_ALTERA_NOME;
                promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                row.NRITEMPROCAPROVA = workflowItens.MG_ALTERA_DATA_APRESENTACAO;
                promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                row.NRITEMPROCAPROVA = workflowItens.MG_ALTERA_LOCAL_APRESENTACAO;
                promises.push(coc03600_menu_designService.hasPendingWorkflow(row));
                return ZHPromise.all(promises).then(function(dataset) {
                    var pendenteNome = workflowPendente(dataset[0]);
                    var pendenteData = workflowPendente(dataset[1]);
                    var pendenteLocal = workflowPendente(dataset[2]);
                    widget.getField("DSMANUAL").readOnly = widget.usaWorkflow.NOME && pendenteNome;
                    widget.getField("DTAPRESENTACIA").readOnly = widget.usaWorkflow.DATA && pendenteData;
                    widget.getField("DSLOCALAPCIA").readOnly = widget.usaWorkflow.LOCAL && pendenteLocal;
                    if (widget.currentRow.DSTIPONEGOCIOINF) {
                        widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = true;
                    } else {
                        widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = false;
                    }
                    InflightorGeneral.edit(widget);
                    widget.backupRow = angular.copy(widget.currentRow);
                    widget.container.getWidget("grid_servicos").isVisible = false;
                    widget.getField("NMFILIAL").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
                    widget.getField("NMCIA").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
                    widget.getField("NMCIA").applyDefaultValue();
                    widget.getField("NMFILIAL").applyDefaultValue();
                });
            });
        } else if (getSituacao(widget) == "E") {
            if (widget.currentRow.DSTIPONEGOCIOINF) {
                widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = true;
            } else {
                widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = false;
            }
            widget.getField("DSMANUAL").readOnly = false;
            widget.getField("DTAPRESENTACIA").readOnly = false;
            widget.getField("DSLOCALAPCIA").readOnly = false;
            InflightorGeneral.edit(widget);
            widget.container.getWidget("grid_servicos").isVisible = false;
            widget.getField("NMFILIAL").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
            widget.getField("NMCIA").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
            widget.getField("NMCIA").applyDefaultValue();
            widget.getField("NMFILIAL").applyDefaultValue();
        } else if (getSituacao(widget) == "A") {
            if (widget.currentRow.DSTIPONEGOCIOINF) {
                widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = true;
            } else {
                widget.getField("DSTIPONEGOCIOINF").readOnlyOnEdit = false;
            }
            if (widget.currentRow.DTINIVGMA || widget.currentRow.DTFIMVGMA) {

                if (widget.currentRow.DTINIVGMA == null || widget.currentRow.DTFIMVGMA == null) {

                    InflightorGeneral.edit(widget);

                } else {

                    var dtinivgma = moment(widget.currentRow.DTINIVGMA.substr(0, 10), "DD/MM/YYYY");
                    var dtfimvgma = moment(widget.currentRow.DTFIMVGMA.substr(0, 10), "DD/MM/YYYY");
                    var dtatual = moment([]);

                    if (dtinivgma < dtatual && dtfimvgma < dtatual) {
                        MessageService.showMessageByCode("COC03600_EXPIRED_TERM");
                    } else if (dtinivgma < dtatual && dtfimvgma >= dtatual) {
                        widget.getField("DTINIVGMA").readOnlyOnEdit = true;
                        InflightorGeneral.edit(widget);
                    } else if (dtinivgma >= dtatual && dtfimvgma >= dtatual) {
                        widget.getField("DTINIVGMA").readOnlyOnEdit = false;
                        InflightorGeneral.edit(widget);
                    }

                }
            } else {
                InflightorGeneral.edit(widget);
            }

        }
    };

    this.loadDataSource = function(widget = templateManager.container.getWidget("zh_crosstab")) {
        var deferred = ZHPromise.defer();
        InflightorGeneral.callRestEngine("coc#/loadDtsCrosstab", "FilterData", widget.dataSourceFilter).then(function(callback) {
            widget.dataSource.data = callback.dataset.view_aux_subgrupo.map(item => {
                if (item.html)
                    item.html = item.html.replace('UNDEFINED_IMAGE', window.undefinedImage);
                return item;
            });
            setTimeout(function() {
                widget.resizeGridColumns(true);
            }, 800);
            deferred.resolve();
        }, function(error) {
            ScreenService.showMessage(error.error);
            deferred.resolve();
        });
        return deferred;
    };

    this.afterMoveRow = function(widget, row) {
        /*juliaoAfter*/

        if (!row || Util.isEmptyObject(row)) return;
        if (!widget.currentRow.__is_new) {
            row = row.__row ? row.__row : row;
            var parentRow = widget.parent.currentRow;
            parentRow.CDSUBGRU = row.SUBGRUPO.substr(0, row.SUBGRUPO.indexOf("_"));
            parentRow.NRSEQSUBGRUSEMAM = row.SUBGRUPO.substr(row.SUBGRUPO.indexOf("_") + 1, row.SUBGRUPO.length);
            var primaryKeys = {
                NRSEQSUBGRUSEMAM: "",
                CDSUBGRU: "",
                CDCLASSE: "",
                NRSERVICOCIAFILI: "",
                CDMANCIA: "",
                CDCIA: "",
                CDFILIAL: "",
                NRORG: ""
            };
            var dataFilter = retornaRowWorkflowPentende(parentRow, workflowItens.SUBGRUPO_EXCLUSAO, "1", "3", Object.keys(primaryKeys));
            coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
                function(qtde) {
                    if (qtde) {
                        MessageService.showMessageByCode("COC03600_SUBGROUP_DELETE_WORKFLOW");
                        var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
                        widget.container.getWidget("tabPropVar").isVisible = false;
                        widget.container.getWidget("widgetProdutocia").isVisible = false;
                        widget.container.getWidget("widgetCiclo").isVisible = false;
                        var pkArray = row.SUBGRUPO.split("_");
                        var dataSourceFilter = angular.copy(widget.dataSourceFilter);
                        dataSourceFilter.push({
                            "name": "CDSUBGRU",
                            "operator": "=",
                            "value": pkArray[0]
                        }, {
                            "name": "NRSEQSUBGRUSEMAM",
                            "operator": "=",
                            "value": pkArray[1]
                        });
                        widgetSubgrupo.dataSourceFilter = dataSourceFilter;
                        widgetSubgrupo.isVisible = true;
                        widgetSubgrupo.reload().then(
                            function() {
                                widgetSubgrupo.currentRow = widgetSubgrupo.dataSource.data[0];
                                widgetSubgrupo.activate();
                                ScreenService.openSwipe(widgetSubgrupo.parent).then(
                                    function() {
                                        widgetSubgrupo.view();
                                    }
                                );
                            }
                        );
                    } else {
                        afterMoveRowNoHasDeppendingWf(widget, row);
                    }
                }
            );
        }
    };

    function afterMoveRowNoHasDeppendingWf(widget, row) {
        if (row.columnName) {
            if (!widget.currentRow.__is_new) {
                widget.container.getWidget("tabPropVar").isVisible = true;
            }
            var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
            var widgetPropvar = widget.container.getWidget("tabPropVar");
            var widgetProdutocia = widget.container.getWidget("widgetProdutocia");
            var widgetCiclo = widget.container.getWidget("widgetCiclo");
            var widgetSubgrPrinc = widget.container.getWidget("grid_subgruposPrincipais");

            var pkArray = row.SUBGRUPO.split("_");
            var dataSourceFilter = angular.copy(widget.dataSourceFilter);
            dataSourceFilter.push({
                "name": "CDSUBGRU",
                "operator": "=",
                "value": pkArray[0]
            }, {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": pkArray[1]
            });
            widgetSubgrupo.dataSourceFilter = dataSourceFilter;
            widgetProdutocia.dataSourceFilter = dataSourceFilter;
            widgetCiclo.dataSourceFilter = dataSourceFilter;

            widgetSubgrupo.view();
            widgetSubgrupo.isVisible = true;
            widgetProdutocia.isVisible = true;
            widgetCiclo.isVisible = true;

            var promisesArray = [];
            promisesArray.push(widgetSubgrupo.reload(null, false));
            promisesArray.push(widgetProdutocia.reload(null, false));
            promisesArray.push(widgetCiclo.reload(null, false));

            ZHPromise.all(promisesArray).then(function() {
                widgetSubgrupo.currentRow = widgetSubgrupo.dataSource.data[0];
                if (widgetSubgrupo.dataSource.data[0] !== undefined) {
                    self.alteraProporcao(widgetSubgrupo);
                    ScreenService.openSwipe(widgetSubgrupo.getParent()).then(function() {
                        switch (row.columnName) {
                            case "SUBGRUPO":
                                widgetSubgrupo.activate();
                                break;
                            case "PROPORCAO":
                                if (row.value == "T") {
                                    widgetPropvar.reload().then(function() {
                                        widgetPropvar.activate();
                                    });
                                } else widgetSubgrupo.activate();
                                break;
                            case "PRODUTOCIA":
                                widgetProdutocia.activate();
                                break;
                            default:
                                widgetCiclo.activate();
                                setCicloCurrentRow(widgetCiclo, row).then(function() {
                                    self.carregaMontasubsemam(widgetCiclo);
                                    self.carregaDataSourceComposicao();
                                    InflightorGeneral.buildDataSourceFilter(widgetSubgrPrinc.dataSourceFilter, widgetCiclo.currentRow);
                                    widgetSubgrPrinc.reload();
                                });
                        }
                    });
                } else {
                    widget.activate();
                    MessageService.showMessageByCode("COC03600_ERROR_LOAD_PRODUCT");
                }
            }, function(error) {
                ScreenService.showMessage(error.error);
            });
        }
    }

    var setCicloCurrentRow = function(widgetCiclo, row) {
        var deffered = ZHPromise.defer();
        var currentRowIsset = false;

        for (let i = 0; i < widgetCiclo.dataSource.data.length; i++) {
            var ciclo = widgetCiclo.dataSource.data[i];
            if (ciclo.CDCICLO == row.value) {
                widgetCiclo.currentRow = ciclo;
                currentRowIsset = true;
                deffered.resolve();
            }
        }
        if (!currentRowIsset) {
            deffered.reject();
        }
        return deffered;
    };

    function validateEditSubgrupo(response, widget) {
        /*juliaosubedit*/
        var hasWFExclude = response[0];
        var hasWFEditProporcao = response[1];
        var hasWFEditVigencia = response[2];
        if (hasWFExclude) {
            MessageService.showMessageByCode("COC03600_SUBGROUP_WORKFLOW_PROCESS");
        } else {
            if (hasWFEditProporcao) {
                widget.getField("IDPROPSUBSEMAM").readOnlyOnEdit = true;
            } else {
                widget.getField("IDPROPSUBSEMAM").readOnlyOnEdit = false;
            }
            if (hasWFEditVigencia) {
                widget.getField("DTINIVGSUMA").readOnlyOnEdit = true;
                widget.getField("DTFIMVGSUMA").readOnlyOnEdit = true;
            } else {
                widget.getField("DTINIVGSUMA").readOnlyOnEdit = false;
                widget.getField("DTFIMVGSUMA").readOnlyOnEdit = false;
            }
            var currentRow = widget.currentRow;
            currentRow.DTINIVGSUMA_BKP = currentRow.DTINIVGSUMA == null ? null : currentRow.DTINIVGSUMA.substr(0, 10);
            currentRow.DTFIMVGSUMA_BKP = currentRow.DTFIMVGSUMA == null ? null : currentRow.DTFIMVGSUMA.substr(0, 10);
            currentRow.IDPROPSUBSEMAM_BKP = currentRow.IDPROPSUBSEMAM;
            InflightorGeneral.edit(widget);
            widget.template = "widget/form-view-edit.html";
        }
    }

    this.swipeEdit = function(widget) {
        widget.dataSource.data = [];
        var fieldNames = ["NRORDSUBGRUSEMAM", "IDPROPSUBSEMAM", "QTPROPSUBSEMAM", "DTINIVGSUMA", "DTFIMVGSUMA"];
        fieldNames.forEach(function(fieldName) {
            widget.getField(fieldName).readOnlyOnEdit = false;
        });
        var primaryKeys = {
            NRSEQSUBGRUSEMAM: "",
            CDSUBGRU: "",
            CDCLASSE: "",
            NRSERVICOCIAFILI: "",
            CDMANCIA: "",
            CDCIA: "",
            CDFILIAL: "",
            NRORG: ""
        };
        if (getSituacao(widget) == "A") {
            var filterWorkflow = retornaRowWorkflowPentende(widget.currentRow, workflowItens.SUBGRUPO_ALTERA_VIGENCIA, "1", "1", Object.keys(primaryKeys));
            coc03600_menu_designService.hasPendingWorkflow(filterWorkflow).then(function(qtdWorkflowPendente) {
                if (qtdWorkflowPendente > 0) {
                    MessageService.showMessageByCode("COC03600_WORKFLOW_PENDING_OPERATION");
                } else {
                    var request = {
                        route: "coc#/crud/coc03600_max_dtinivgsuma_subgrusemam",
                        requestType: "FilterData",
                        filter: {
                            "CDFILIAL": widget.currentRow.CDFILIAL,
                            "CDCIA": widget.currentRow.CDCIA,
                            "CDMANCIA": widget.currentRow.CDMANCIA,
                            "NRSERVICOCIAFILI": widget.currentRow.NRSERVICOCIAFILI,
                            "CDCLASSE": widget.currentRow.CDCLASSE,
                            "CDSUBGRU": widget.currentRow.CDSUBGRU,
                            "NRORG": widget.currentRow.NRORG,
                        }
                    };
                    var newFilter = [];
                    for (let attr in request.filter) {
                        newFilter.push({
                            name: attr,
                            operator: "=",
                            value: request.filter[attr]
                        });
                    }
                    request.filter = newFilter;
                    InflightorGeneral.callRestEngine(request.route, request.requestType, request.filter).then(function(response) {
                        var format = "DD/MM/YYYY h:mm:ss";
                        var maxDtinivgsuma = response.dataset.coc03600_max_dtinivgsuma_subgrusemam[0].DTINIVGSUMA;
                        maxDtinivgsuma = moment(maxDtinivgsuma, format);
                        var dtinivgsuma = moment(widget.currentRow.DTINIVGSUMA, format);
                        if (maxDtinivgsuma.isSame(dtinivgsuma)) {
                            var fieldNames = ["NRORDSUBGRUSEMAM", "IDPROPSUBSEMAM", "QTPROPSUBSEMAM"];
                            fieldNames.forEach(function(fieldName) {
                                widget.getField(fieldName).readOnlyOnEdit = true;
                            });
                            var currentRow = widget.currentRow;
                            currentRow.DTINIVGSUMA_BKP = currentRow.DTINIVGSUMA == null ? null : currentRow.DTINIVGSUMA.substr(0, 10);
                            currentRow.DTFIMVGSUMA_BKP = currentRow.DTFIMVGSUMA == null ? null : currentRow.DTFIMVGSUMA.substr(0, 10);
                            currentRow.IDPROPSUBSEMAM_BKP = currentRow.IDPROPSUBSEMAM;
                            widget.getField("DTINIVGSUMA").readOnlyOnEdit = false;
                            widget.getField("DTFIMVGSUMA").readOnlyOnEdit = false;
                            widget.container.getWidget("widgetProdutocia").isVisible = false;
                            widget.container.getWidget("widgetCiclo").isVisible = false;
                            widget.container.getWidget("tabPropVar").isVisible = false;
                            InflightorGeneral.edit(widget);
                            widget.template = "widget/form-view-edit.html";
                        } else {
                            MessageService.showMessageByCode("COC03600_LAST_DT_VALIDITY");
                        }
                    }, InflightorGeneral.callbackError);
                }
            });
        } else if (!desenvolvimentoOuProposta(getSituacao(widget), true)) {
            return;
        }

        if (getSituacao(widget) == "P") {

            widget.currentRow.DTINIVGSUMA = widget.currentRow.DTINIVGSUMA == null ? null : widget.currentRow.DTINIVGSUMA.substr(0, 10);
            widget.currentRow.DTFIMVGSUMA = widget.currentRow.DTFIMVGSUMA == null ? null : widget.currentRow.DTFIMVGSUMA.substr(0, 10);
            widget.container.getWidget("widgetProdutocia").isVisible = false;
            widget.container.getWidget("widgetCiclo").isVisible = false;
            widget.container.getWidget("tabPropVar").isVisible = false;
            var dataFilter = {};
            var promises = [];
            //verifica se existe WF pendente de exclusão
            dataFilter = retornaRowWorkflowPentende(widget.currentRow, workflowItens.SUBGRUPO_EXCLUSAO, "1", "3", Object.keys(primaryKeys));
            promises.push(coc03600_menu_designService.hasPendingWorkflow(dataFilter));

            //verifica se existe WF pendente de alteração da Proporção
            dataFilter = retornaRowWorkflowPentende(widget.currentRow, workflowItens.SUBGRUPO_ALTERA_PROPORCAO, "1", "1", Object.keys(primaryKeys));
            promises.push(coc03600_menu_designService.hasPendingWorkflow(dataFilter));

            //verifica se existe WF pendente de alteração da Vigencia
            dataFilter = retornaRowWorkflowPentende(widget.currentRow, workflowItens.SUBGRUPO_ALTERA_VIGENCIA, "1", "1", Object.keys(primaryKeys));
            promises.push(coc03600_menu_designService.hasPendingWorkflow(dataFilter));

            ZHPromise.all(promises).then(
                function(response) {
                    validateEditSubgrupo(response, widget);
                }
            );
        } else if (getSituacao(widget) == "E") {
            widget.container.getWidget("widgetProdutocia").isVisible = false;
            widget.container.getWidget("widgetCiclo").isVisible = false;
            widget.container.getWidget("tabPropVar").isVisible = false;
            InflightorGeneral.edit(widget);
            widget.template = "widget/form-view-edit.html";
        }
    };

    function subgrupoAtivo(currentRow) {
        if (currentRow.__is_new) {
            if (currentRow.IDATIVO == "N") {
                MessageService.showMessageByCode("INACTIVE_SUB_GROUP");
                return false;
            }
            return true;
        }
        return true;
    }

    this.saveSubgrupo = function(widget) {
        if (validaSaveSubgru(widget)) {
            widget.currentRow.QTPROPSUBSEMAM = InflightorGeneral.valueToFloat(widget.currentRow.QTPROPSUBSEMAM);
            if (getSituacao(widget, true) == "A") {
                if (!widget.currentRow.DTINIVGSUMA || !widget.currentRow.DTFIMVGSUMA) {
                    ScreenService.showMessage("Preencha período de vigência corretamente.");
                    return;
                }
                var format = "DD/MM/YYYY h:mm:ss";
                var formatBR = "DD/MM/YYYY";
                var request = {
                    route: "coc#/crud/coc00022_subgrusemam_date_validate",
                    requestType: "FilterData",
                    filter: {
                        "CDFILIAL": "=",
                        "CDCIA": "=",
                        "CDMANCIA": "=",
                        "NRSERVICOCIAFILI": "=",
                        "CDCLASSE": "=",
                        "CDSUBGRU": "=",
                        "NRORG": "=",
                        "NRSEQSUBGRUSEMAM": "<>",
                        "DTINI": {
                            operator: "=",
                            value: moment(widget.currentRow.DTINIVGSUMA, format).format(formatBR)
                        },
                        "DTFIM": {
                            operator: "=",
                            value: moment(widget.currentRow.DTFIMVGSUMA, format).format(formatBR)
                        },
                    }
                };
                var newFilter = [];
                for (let attr in request.filter) {
                    newFilter.push({
                        name: attr,
                        operator: angular.isObject(request.filter[attr]) ? request.filter[attr].operator : request.filter[attr],
                        value: angular.isObject(request.filter[attr]) ? request.filter[attr].value : widget.currentRow[attr]
                    });
                }
                request.filter = newFilter;
                InflightorGeneral.callRestEngine(request.route, request.requestType, request.filter).then(function(response) {
                    var erros = response.dataset.coc00022_subgrusemam_date_validate;
                    if (erros.length) {
                        var datasConflito = erros.map(function(erro) {
                            return erro.DATA_ERROR;
                        });
                        ScreenService.showMessage("Não é possível realizar alteração, existem datas de vigência em conflito: " + datasConflito.join(","));
                    } else {
                        widget.currentRow.changed = false;
                        processWfSaveEditSubgrupo(widget);
                    }
                });
            } else if (getSituacao(widget, true) == "P") {
                if (widget.currentRow.__is_new) {
                    processWfSaveNewSubgrupo(widget);
                } else {
                    processWfSaveEditSubgrupo(widget);
                }
            } else if (getSituacao(widget, true) == "E") {
                saveSubgrupoNoWF(widget);
            }
        }
    };

    function validaSaveSubgru(widget) {
        return (
            widget.isValid() &&
            subgrupoAtivo(widget.currentRow) &&
            validaVigenciaSubgru(widget) &&
            validaConfiguracao(widget)
        );
    }

    function validaVigenciaSubgru(widget) {
        var currentRow = widget.currentRow;
        var widgetMenu = widget.container.getWidget("grid_manual");

        var iniMenu = widgetMenu.currentRow.DTINIVGMA;
        var dtIniMenu = moment(iniMenu, "DD/MM/YYYY hh:mm:ss");
        var fimMenu = widgetMenu.currentRow.DTFIMVGMA;
        var dtFimMenu = moment(fimMenu, "DD/MM/YYYY hh:mm:ss");

        var ini = currentRow.DTINIVGSUMA;
        var dtIni = moment(ini, "DD/MM/YYYY");

        var fim = currentRow.DTFIMVGSUMA;
        var dtFim = moment(fim, "DD/MM/YYYY");

        if ((ini && !fim) || (!ini && fim) || dtIni > dtFim) {
            ScreenService.showMessage("A vigência informada é inválida.", "error");
            return false;
        } else if ((iniMenu && fimMenu) && (dtIni < dtIniMenu || dtFim > dtFimMenu)) {
            ScreenService.showMessage("A vigência informada está fora do período de vigência do Menu.<br><br>(" + dtIniMenu.format("DD/MM/YYYY") + " - " + dtFimMenu.format("DD/MM/YYYY") + ")", "error");
            return false;
        }
        return true;
    }

    function validaConfiguracao(widget) {
        if (widget.getField("QTPROPSUBSEMAM").isVisible) {
            if (widget.currentRow.QTPROPSUBSEMAM == "0") {
                ScreenService.showMessage("Configuração deve ser maior que zero.", "error");
                return false;
            }
        }
        return true;
    }

    function saveSubgrupoNoWF(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            self.loadDataSource();
            widget.view();
            widget.template = "widget/form-view.html";
            widget.container.getWidget("widgetProdutocia").isVisible = true;
            widget.container.getWidget("widgetCiclo").isVisible = true;
            widget.container.getWidget("tabPropVar").isVisible = true;
            if (widget.currentRow.__is_new) {
                widget.container.getWidget("zh_crosstab").activate();
            } else {
                widget.container.getWidget("widgetSubgrupo").activate();
            }
        });
    }

    function changedVigencia(widget, changedIni, changedFin) {
        var itensVigencia = {};
        if (changedIni) {
            itensVigencia.DTINIVGSUMA = widget.currentRow.DTINIVGSUMA;
            widget.currentRow.DTINIVGSUMA = widget.currentRow.DTINIVGSUMA_BKP;
        }
        if (changedFin) {
            itensVigencia.DTFIMVGSUMA = widget.currentRow.DTFIMVGSUMA;
            widget.currentRow.DTFIMVGSUMA = widget.currentRow.DTFIMVGSUMA_BKP;
        }
        return itensVigencia;
    }

    function changedProporcao(widget) {
        var objProporcao = {
            IDPROPSUBSEMAM: widget.currentRow.IDPROPSUBSEMAM
        };
        widget.currentRow.IDPROPSUBSEMAM = widget.currentRow.IDPROPSUBSEMAM_BKP;
        return objProporcao;
    }

    function executeWfAlteraSubgrupo(widget, objDataWf) {
        var beforeWorkflow = function(widget) {
            var wfDataWidget = widget.rowToSave;
            var itensData = wfDataWidget.itensData;
            var promises = [];
            for (let index in itensData) {
                var workFlowData = {
                    CHANGEDITEMS: itensData[index].CHANGEDITEMS,
                    PRIMARYKEYS: wfDataWidget.PRIMARYKEYS,
                    NMITEM: wfDataWidget.NMITEM,
                    NMTABELA: "SUBGRUSEMAM",
                    IDTIPOPERWFLOW: "1",
                    NRITEMPROCAPROVA: itensData[index].NRITEMPROCAPROVA,
                    NRPROCESSOAPROVA: workflowItens.NRPROCESSO,
                    DSMOTIVOACAO: widget.currentRow.DSMOTIVOACAO
                };
                promises.push(coc03600_menu_designService.salvaWorkflow(workFlowData));
            }
            ZHPromise.all(promises).then(
                function(callback) {
                    /*juliaoSubsave*/
                    ScreenService.closePopup();
                    var widgetServico = widget.container.getActiveWidget();
                    widgetServico.view();
                    widgetServico.template = "widget/form-view.html";
                    widgetServico.container.getWidget("widgetProdutocia").isVisible = true;
                    widgetServico.container.getWidget("widgetCiclo").isVisible = true;
                    widget.container.getWidget("tabPropVar").isVisible = true;
                    ScreenService.showMessage(callback[0].messages[0].message);
                },
                function(error) {
                    ScreenService.showMessage(error.data.error);
                }
            );
        };

        var pks = {
            NRSEQSUBGRUSEMAM: widget.currentRow.NRSEQSUBGRUSEMAM,
            CDSUBGRU: widget.currentRow.CDSUBGRU,
            CDCLASSE: widget.currentRow.CDCLASSE,
            NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
            CDMANCIA: widget.currentRow.CDMANCIA,
            CDCIA: widget.currentRow.CDCIA,
            CDFILIAL: widget.currentRow.CDFILIAL,
            NRORG: widget.currentRow.NRORG
        };
        var variables = {
            nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
            nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
            nmSubgrupo: widget.currentRow.DSSUBGRU,
            dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
            unidade: getLocalVar("NMFILIAL"),
            companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA
        };
        var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__", variables);
        var workflowData = {
            itensData: objDataWf,
            PRIMARYKEYS: pks,
            NMITEM: translateNMITEM,
        };
        self.solicitaCriacaoWorkflow(widget, workflowData, beforeWorkflow);
    }

    function processWfSaveEditSubgrupo(widget) {
        var currentRow = widget.currentRow;
        var changeVigenciaInicial = currentRow.DTINIVGSUMA_BKP != currentRow.DTINIVGSUMA;
        var changeVigenciaFinal = currentRow.DTFIMVGSUMA_BKP != currentRow.DTFIMVGSUMA;
        var changeProporcao = currentRow.IDPROPSUBSEMAM_BKP != currentRow.IDPROPSUBSEMAM;
        if ((changeVigenciaInicial || changeVigenciaFinal) && changeProporcao) {
            var promises = [];
            promises.push(usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_ALTERA_PROPORCAO));
            promises.push(usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_ALTERA_VIGENCIA));
            ZHPromise.all(promises).then(
                function(usaWf) {
                    if (usaWf[0] && usaWf[1]) {
                        let objDataWf = [
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_VIGENCIA,
                                CHANGEDITEMS: changedVigencia(widget, changeVigenciaInicial, changeVigenciaFinal)
                            },
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_PROPORCAO,
                                CHANGEDITEMS: changedProporcao(widget)
                            }
                        ];
                        executeWfAlteraSubgrupo(widget, objDataWf); //criar
                    } else if (usaWf[0]) {
                        let objDataWf = [
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_PROPORCAO,
                                CHANGEDITEMS: changedProporcao(widget)
                            }
                        ];
                        executeWfAlteraSubgrupo(widget, objDataWf); //criar
                    } else if (usaWf[1]) {
                        let objDataWf = [
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_VIGENCIA,
                                CHANGEDITEMS: changedVigencia(widget, changeVigenciaInicial, changeVigenciaFinal)
                            }
                        ];
                        executeWfAlteraSubgrupo(widget, objDataWf); //criar
                    }
                    saveSubgrupoNoWF(widget);
                }
            );
        } else if (changeVigenciaInicial || changeVigenciaFinal) {
            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_ALTERA_VIGENCIA).then(
                function(usaWF) {
                    if (usaWF) {
                        let objDataWf = [
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_VIGENCIA,
                                CHANGEDITEMS: changedVigencia(widget, changeVigenciaInicial, changeVigenciaFinal)
                            }
                        ];
                        executeWfAlteraSubgrupo(widget, objDataWf); //criar
                    }
                    saveSubgrupoNoWF(widget);
                }
            );
        } else if (changeProporcao) {
            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_ALTERA_PROPORCAO).then(
                function(usaWF) {
                    if (usaWF) {
                        let objDataWf = [
                            {
                                NRITEMPROCAPROVA: workflowItens.SUBGRUPO_ALTERA_PROPORCAO,
                                CHANGEDITEMS: changedProporcao(widget)
                            }
                        ];
                        executeWfAlteraSubgrupo(widget, objDataWf); //criar
                    }
                    saveSubgrupoNoWF(widget);
                }
            );
        } else {
            saveSubgrupoNoWF(widget);
        }
    }

    function processWfSaveNewSubgrupo(widget) {

        var beforeWorkflow = function(widget) {
            widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            //implementar chamada da rota no service
            coc03600_menu_designService.insertSubgrupoWF(widget.rowToSave).then(
                function(callback) {
                    ScreenService.closePopup();
                    var widgetSubgrupo = widget.container.getActiveWidget();
                    ScreenService.closeSwipe().then(
                        function() {
                            widgetSubgrupo.view();
                            widgetSubgrupo.template = "widget/form-view.html";
                            widgetSubgrupo.container.getWidget("widgetProdutocia").isVisible = true;
                            widgetSubgrupo.container.getWidget("widgetCiclo").isVisible = true;
                            widget.container.getWidget("tabPropVar").isVisible = true;
                            widgetSubgrupo.container.getWidget("zh_crosstab").list();
                            widgetSubgrupo.container.getWidget("zh_crosstab").activate();
                        }
                    );
                    ScreenService.showMessage(callback.messages[0].message);
                },
                function(error) {
                    ScreenService.closePopup();
                    ScreenService.showMessage(error.data.error);
                }
            );
        };

        usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_INCLUSAO).then(
            function(usaWF) {
                if (usaWF) {
                    var changedItems = {
                        DTINIVGSUMA: widget.currentRow.DTINIVGSUMA,
                        DTFIMVGSUMA: widget.currentRow.DTFIMVGSUMA,
                        IDPROPSUBSEMAM: widget.currentRow.IDPROPSUBSEMAM,
                        QTPROPSUBSEMAM: widget.currentRow.QTPROPSUBSEMAM,
                        NRORDSUBGRUSEMAM: widget.currentRow.NRORDSUBGRUSEMAM
                    };
                    var pks = {
                        CDSUBGRU: widget.currentRow.CDSUBGRU,
                        CDCLASSE: widget.currentRow.CDCLASSE,
                        NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
                        CDMANCIA: widget.currentRow.CDMANCIA,
                        CDCIA: widget.currentRow.CDCIA,
                        CDFILIAL: widget.currentRow.CDFILIAL,
                        NRORG: widget.currentRow.NRORG
                    };
                    var variables = {
                        nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                        nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                        nmSubgrupo: widget.currentRow.DSSUBGRU,
                        dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                        unidade: getLocalVar("NMFILIAL"),
                        companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA
                    };
                    var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__", variables);
                    var workflowData = {
                        CHANGEDITEMS: changedItems,
                        PRIMARYKEYS: pks,
                        NMITEM: translateNMITEM,
                        NMTABELA: "SUBGRUSEMAM",
                        IDTIPOPERWFLOW: "2",
                        NRITEMPROCAPROVA: workflowItens.SUBGRUPO_INCLUSAO,
                        NRPROCESSOAPROVA: workflowItens.NRPROCESSO
                    };
                    self.solicitaCriacaoWorkflow(widget, workflowData, beforeWorkflow);
                } else {
                    saveSubgrupoNoWF(widget);
                }
            }
        );
    }

    this.closeSubgrupoSwipe = function(widget) {
        ScreenService.closeSwipe();
        widget.parent.parent.activate();
        self.loadDataSource();
        templateManager.updateTemplate();
    };

    this.cancelEdit = function(widget) {
        var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
        widget.template = "widget/form-view.html";
        if (widget.currentRow.__is_new) {
            widget.dataSource.data.pop();
            ScreenService.closeSwipe();
            widget.container.getWidget("zh_crosstab").activate();
        } else {
            widget.view();
            if (widget.backupRow) {
                widget.currentRow = angular.copy(widget.backupRow);
                widget.container.getWidget("widgetProdutocia").isVisible = true;
                widget.container.getWidget("widgetCiclo").isVisible = true;
                widget.container.getWidget("tabPropVar").isVisible = true;
                if (widget.backupRow.IDPROPSUBSEMAM !== "3") {
                    widgetSubgrupo.getField("QTPROPSUBSEMAM").isVisible = true;
                }
            }
        }
    };

    this.expandSwipe = function() {
        $(".zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level=\"2\"] ").addClass("sbg-swipe");
    };

    this.retrieveSwipe = function() {
        $(".zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level=\"2\"] ").removeClass("sbg-swipe");
    };

    this.adicionaProdutosCia = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            widget.getField("QTPRODUTOCIA").readOnly = false;
            widget.getField("NMPRODUTO").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "SGUNIDADE",
                operator: "IS_NOT_NULL",
                value: ""
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.container.getWidget("grid_manual").currentRow.CDFILIAL
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.container.getWidget("grid_manual").currentRow.CDCIA
            }];
            widget.newRow();
        }
    };

    this.setSubgrupoFilter = function(widgetServico) {
        var crosstabSubgrupo = widgetServico.container.getWidget("zh_crosstab");
        var gridSubgrupo = widgetServico.container.getWidget("gris_subgrupo");
        crosstabSubgrupo.dataSourceFilter = angular.copy(widgetServico.dataSourceFilter).filter(function(item) {
            return item.operator != "ORDER_BY";
        });
        crosstabSubgrupo.dataSourceFilter.push({
            "name": "NRSERVICOCIAFILI",
            "operator": "=",
            "value": widgetServico.currentRow.NRSERVICOCIAFILI
        }, {
            "name": "CDCLASSE",
            "operator": "=",
            "value": widgetServico.currentRow.CDCLASSE
        });
        gridSubgrupo.dataSourceFilter = crosstabSubgrupo.dataSourceFilter;
        gridSubgrupo.reload();
    };

    this.adicionaCiclos = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            widget.getField("IDCICLO").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }, {
                name: "CDFILIAL",
                operator: "=",
                value: widget.container.getWidget("grid_manual").currentRow.CDFILIAL
            }, {
                name: "CDCIA",
                operator: "=",
                value: widget.container.getWidget("grid_manual").currentRow.CDCIA
            }, {
                name: "CDCLASSE",
                operator: "=",
                value: widget.container.getWidget("grid_servicos").currentRow.CDCLASSE
            }, {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI
            }];
            // widget.edit();
            // widget.activate();
            widget.newRow();
            // ScreenService.openSwipe(widget);
            self.onDeselectPhoto(widget);
            widget.currentRow.DSMONTASUBSEMAMHTMLBKP = "new";
            widget.currentRow.EXIST_INSTRUCAO = 0; /*juliaomonta2*/
            widget.container.getWidget("grid_composicao").isVisible = false;
            widget.container.getWidget("grid_subgruposPrincipais").isVisible = false;
        }
    };

    this.editCiclos = function(widget) {
        /*sdds juliao*/
        widget.container.getWidget("grid_composicao").isVisible = false;
        widget.container.getWidget("grid_subgruposPrincipais").isVisible = false;
        removerImagemReadOnly(widget.currentRow.FGFOTOSUBSEMAM, widget.getField("REMOVE_IMAGE"));
        if (IDUTIAIRVISION == "S") {
            InflightorGeneral.justInteger("#NRITAIRVISION");
        }
        var primaryKeys = {
            CDCICLO: "",
            NRSEQSUBGRUSEMAM: "",
            CDSUBGRU: "",
            CDCLASSE: "",
            NRSERVICOCIAFILI: "",
            CDMANCIA: "",
            CDCIA: "",
            CDFILIAL: "",
            NRORG: ""
        };
        var dataFilter = retornaRowWorkflowPentende(widget.currentRow, "15", "1", "1", Object.keys(primaryKeys));
        coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
            function(qtde) {
                InflightorGeneral.edit(widget);
                if (qtde) {
                    widget.getField("DSMONTASUBSEMAMHTML").template = "zh-rich-text#field/rich-text-view.html";
                } else {
                    widget.getField("DSMONTASUBSEMAMHTML").template = "zh-rich-text#field/rich-text.html";
                }
            }
        );
    };

    function removerImagemReadOnly(FGFOTOSUBSEMAM, field) {
        if (FGFOTOSUBSEMAM === null || FGFOTOSUBSEMAM === window.undefinedImage) {
            field.readOnly = true;
        } else {
            field.readOnly = false;
        }
    }

    this.removerImagemCiclo = function(widget) {
        widget.currentRow.FGFOTOSUBSEMAM = window.undefinedImage;
        widget.getField("PHOTO_UPLOAD").fileList = [];
        widget.currentRow.PHOTO_UPLOAD = [];
        removerImagemReadOnly(widget.currentRow.FGFOTOSUBSEMAM, widget.getField("REMOVE_IMAGE"));
        templateManager.updateTemplate();
    };

    this.setCicloColor = function(element, row, columnIndex = 2) {
        if (row) {
            setTimeout(function() {
                Array.from(element.find(`.td:eq(${columnIndex})`)).forEach(element => {
                    if (element && element.style) {
                        element.style.setProperty('background-color', row.NRCORCICLO, 'important');
                    }
                });
            }, 100);
        }
    };

    this.setComposicaoColor = function(element, row) {
        if (row) {
            if (row.VRCUSTO == 0 || row.CUSTOZEROREC == "S") {
                setTimeout(function() {
                    element.find(".td:eq(7)").css("color", "#B51B3D");
                }, 100);
            }
        }
    };

    this.adicionarSubGrupo = function(widgetCrosstab) {
        if (desenvolvimentoOuProposta(getSituacao(widgetCrosstab), true)) {
            widgetCrosstab.container.getWidget("widgetProdutocia").isVisible = false;
            widgetCrosstab.container.getWidget("widgetCiclo").isVisible = false;
            widgetCrosstab.container.getWidget("tabPropVar").isVisible = false;

            var widgetSubgrupo = widgetCrosstab.container.getWidget("widgetSubgrupo");
            widgetSubgrupo.dataSource.data = [];
            widgetSubgrupo.isVisible = true;
            widgetSubgrupo.template = "widget/form-view-edit.html";
            widgetSubgrupo.edit();
            widgetSubgrupo.activate();
            widgetSubgrupo.newRow();

            var widgetServico = widgetSubgrupo.container.getWidget("grid_servicos");
            widgetSubgrupo.currentRow.CDFILIAL = widgetServico.currentRow.CDFILIAL;
            widgetSubgrupo.currentRow.CDCIA = widgetServico.currentRow.CDCIA;
            widgetSubgrupo.currentRow.CDMANCIA = widgetServico.currentRow.CDMANCIA;
            widgetSubgrupo.currentRow.NRSERVICOCIAFILI = widgetServico.currentRow.NRSERVICOCIAFILI;
            widgetSubgrupo.currentRow.CDCLASSE = widgetServico.currentRow.CDCLASSE;
            widgetSubgrupo.currentRow.NRORG = widgetServico.currentRow.NRORG;

            widgetSubgrupo.currentRow.DTINIVGSUMA = widgetSubgrupo.container.getWidget("grid_manual").currentRow.DTINIVGMA;
            widgetSubgrupo.currentRow.DTFIMVGSUMA = widgetSubgrupo.container.getWidget("grid_manual").currentRow.DTFIMVGMA;

            widgetSubgrupo.getField("QTPROPSUBSEMAM").isVisible = true;
            widgetSubgrupo.getField("QTPROPSUBSEMAM").label = "Configuração (V)";
            widgetSubgrupo.getField("QTPROPSUBSEMAM").mask = {
                "type": "currency",
                "params": {
                    "symbol": "",
                    "thousands": "",
                    "decimal": ",",
                    "precision": "0"
                }
            };

            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDMANCIA
                },
                {
                    "name": "NRSERVICOCIAFILI",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.NRSERVICOCIAFILI
                },
                {
                    "name": "CDCLASSE",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDCLASSE
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.NRORG
                }
            ];
            InflightorGeneral.callRestEngine("coc#/crud/subgrusemam_max_nrordsubgrusemam", "FilterData", filter).then(function(callback) {
                if (callback.dataset.subgrusemam_max_nrordsubgrusemam.length) {
                    var max = callback.dataset.subgrusemam_max_nrordsubgrusemam[0].MAXNRORDSUBGRUSEMAM;
                    widgetSubgrupo.currentRow.NRORDSUBGRUSEMAM = max;
                } else {
                    widgetSubgrupo.currentRow.NRORDSUBGRUSEMAM = 1;
                }
            });
            ScreenService.openSwipe(widgetSubgrupo.parent);
        }
    };

    this.validaOrdem = function(widget) {
        if (widget.currentRow.__is_new) {
            var widgetSubgrupo = widget.container.getWidget("widgetSubgrupo");
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDMANCIA
                },
                {
                    "name": "NRSERVICOCIAFILI",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.NRSERVICOCIAFILI
                },
                {
                    "name": "CDSUBGRU",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDSUBGRU
                },
                {
                    "name": "NRSEQSUBGRUSEMAM",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.NRSEQSUBGRUSEMAM
                },
                {
                    "name": "CDCLASSE",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.CDCLASSE
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": widgetSubgrupo.currentRow.NRORG
                }
            ];
            InflightorGeneral.callRestEngine("coc#/crud/prociasubsemam_max_nrordem", "FilterData", filter).then(function(callback) {
                if (callback.dataset.prociasubsemam_max_nrordem.length) {
                    var max = callback.dataset.prociasubsemam_max_nrordem[0].MAXNRORDEM;
                    widget.currentRow.NRORDEM = max;
                } else {
                    widget.currentRow.NRORDEM = 1;
                }
            });
        }
    };

    this.preencheFiltroSubgru = function(args) {
        args.owner.owner.widget.getField("DSSUBGRUESC").dataSourceFilter = [{
            name: "NRORG",
            operator: "=",
            value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        }, {
            name: "CDFILIAL",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDFILIAL
        }, {
            name: "CDCIA",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDCIA
        }, {
            name: "CDMANCIA",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDMANCIA
        }, {
            name: "NRSERVICOCIAFILI",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.NRSERVICOCIAFILI
        }, {
            name: "CDCLASSE",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDCLASSE
        }, {
            name: "CDSUBGRU",
            operator: "<>",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDSUBGRU
        }, {
            name: "NRSEQSUBGRUSEMAM",
            operator: "<>",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.NRSEQSUBGRUSEMAM
        }, {
            name: "CDCICLO",
            operator: "=",
            value: args.owner.owner.widget.container.getWidget("widgetCiclo").currentRow.CDCICLO
        }];
        args.owner.owner.widget.getField("DSSUBGRUESC").reload();
    };

    this.removeSubgrupo = function(widgetSubgrupo) {
        if (getSituacao(widgetSubgrupo, true) == "P") {
            var primaryKeys = {
                NRSEQSUBGRUSEMAM: "",
                CDSUBGRU: "",
                CDCLASSE: "",
                NRSERVICOCIAFILI: "",
                CDMANCIA: "",
                CDCIA: "",
                CDFILIAL: "",
                NRORG: ""
            };
            var dataFilter = retornaRowWorkflowPentende(widgetSubgrupo.currentRow, "8_9_14", "1", "1_3", Object.keys(primaryKeys));
            coc03600_menu_designService.hasPendingWorkflow(dataFilter).then(
                function(qtde) {
                    if (qtde) {
                        ScreenService.showMessage("Impossível excluir! <br>Existe workflow pendente para este serviço.");
                    } else {
                        usaWorkflow(workflowItens.NRPROCESSO, workflowItens.SUBGRUPO_EXCLUSAO).then(
                            function(usaWf) {
                                ScreenService.confirmMessage(ScreenService.i18n("Delete Register?"), "question", function() {
                                    if (usaWf) {
                                        subgrupoDeleteWf(widgetSubgrupo);
                                    } else {
                                        var widgetCrosstab = widgetSubgrupo.parent.parent;
                                        widgetSubgrupo.removeRow().then(function() {
                                            self.loadDataSource(widgetCrosstab);
                                            ScreenService.closeSwipe().then(function() {
                                                widgetCrosstab.activate();
                                            });
                                        }, function(error) {
                                            if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                                MessageService.showMessageByCode("DELETE_ERROR");
                                            } else {
                                                ScreenService.showMessage(error.data.error);
                                            }
                                        });
                                    }
                                });
                            }
                        );
                    }
                }
            );
        } else if (getSituacao(widgetSubgrupo, true) == "E") {
            var widgetCrosstab = widgetSubgrupo.parent.parent;
            widgetSubgrupo.removeRow().then(function() {
                self.loadDataSource(widgetCrosstab);
                ScreenService.closeSwipe().then(function() {
                    widgetCrosstab.activate();
                });
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }
    };

    function subgrupoDeleteWf(widget) {
        /*juliaosubdel*/
        var beforeWorkflow = function(widget) {
            widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            coc03600_menu_designService.subgrupoDeleteWf(widget.rowToSave).then(
                function(callback) {
                    ScreenService.closePopup();
                    var widgetSubgrupo = widget.container.getActiveWidget();
                    ScreenService.closeSwipe().then(
                        function() {
                            widgetSubgrupo.container.getWidget("zh_crosstab").list();
                            widgetSubgrupo.container.getWidget("zh_crosstab").activate();
                        }
                    );
                    ScreenService.showMessage(callback.messages[0].message);
                },
                function(error) {
                    if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                        MessageService.showMessageByCode("DELETE_ERROR");
                    } else {
                        ScreenService.showMessage(error.data.error);
                    }
                    ScreenService.closePopup();
                }
            );
        };

        var changedItems = {
            IDSITCUSTEIO: "P"
        };

        var pks = {
            CDSUBGRU: widget.currentRow.CDSUBGRU,
            CDCLASSE: widget.currentRow.CDCLASSE,
            NRSERVICOCIAFILI: widget.currentRow.NRSERVICOCIAFILI,
            CDMANCIA: widget.currentRow.CDMANCIA,
            CDCIA: widget.currentRow.CDCIA,
            CDFILIAL: widget.currentRow.CDFILIAL,
            NRORG: widget.currentRow.NRORG,
            NRSEQSUBGRUSEMAM: widget.currentRow.NRSEQSUBGRUSEMAM
        };

        var variables = {
            nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
            nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
            nmSubgrupo: widget.currentRow.DSSUBGRU,
            dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
            unidade: getLocalVar("NMFILIAL"),
            companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA
        };
        var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__", variables);
        var workflowData = {
            CHANGEDITEMS: changedItems,
            PRIMARYKEYS: pks,
            NMITEM: translateNMITEM,
            NMTABELA: "SUBGRUSEMAM",
            IDTIPOPERWFLOW: "3",
            NRITEMPROCAPROVA: workflowItens.SUBGRUPO_EXCLUSAO,
            NRPROCESSOAPROVA: workflowItens.NRPROCESSO
        };
        self.solicitaCriacaoWorkflow(widget, workflowData, beforeWorkflow);
    }

    this.importaProporcao = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            ScreenService.confirmMessage(ScreenService.i18n("Os dados serão substituidos pelo conteúdo do arquivo. Confima operação?"), "question", function() {
                setLocalVar("coc26004_nometela", "menugrid");
                var data = {
                    CDFILIAL: widget.container.getWidget("widgetSubgrupo").currentRow.CDFILIAL,
                    CDCIA: widget.container.getWidget("widgetSubgrupo").currentRow.CDCIA,
                    CDMANCIA: widget.container.getWidget("widgetSubgrupo").currentRow.CDMANCIA,
                    NRSERVICOCIAFILI: widget.container.getWidget("widgetSubgrupo").currentRow.NRSERVICOCIAFILI,
                    CDCLASSE: widget.container.getWidget("widgetSubgrupo").currentRow.CDCLASSE,
                    CDSUBGRU: widget.container.getWidget("widgetSubgrupo").currentRow.CDSUBGRU,
                    NRSEQSUBGRUSEMAM: widget.container.getWidget("widgetSubgrupo").currentRow.NRSEQSUBGRUSEMAM
                };
                setLocalVar("coc26004_data", data);
                ScreenService.openWindow("coc#coc26004_importacao_proporcao_variavel");
            });
        }
    };

    this.setDefaultReport = function(reportWidget) {
        if (!reportWidget.currentRow.REPORTNAME) {
            reportWidget.getField("REPORTNAME").applyDefaultValue();
        }
    };

    this.salvarOrdenacaoSubGrupos = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            widget.reload();
            widget.commitInlineEdit();
            widget.list();
            widget.activate();
        });
    };

    function alteraOrdenacao(dataset, attrOrdenacao) {
        var ordenacao = {};
        var ordenacaoInvalida = false;
        var ordenacaoExcedida = false;
        var ordenacaoValida = true;
        dataset.forEach(function(row) {
            if (parseInt(row[attrOrdenacao]) > 999 || parseInt(row[attrOrdenacao]) <= 0) {
                ordenacaoExcedida = true;
            } else {
                if (ordenacao.hasOwnProperty(row[attrOrdenacao])) {
                    ordenacaoInvalida = true;
                } else {
                    ordenacao[row[attrOrdenacao]] = true;
                }
            }
        });
        if (ordenacaoExcedida) {
            ScreenService.showMessage("Ordenação excedida. Ordenação vai de 1 à 999.");
            ordenacaoValida = false;
        } else if (ordenacaoInvalida) {
            ScreenService.showMessage("Ordenação inválida. Não é permitido o mesmo número de ordenação.");
            ordenacaoValida = false;
        }
        return ordenacaoValida;
    }

    this.atualizaPesoGramatura = function(widget) {
        var quantidade = InflightorGeneral.valueToFloat(widget.currentRow.QTD) || 0;
        var quantidadeGramatura = InflightorGeneral.valueToFloat(widget.currentRow.QTGRAMATURA) || 0;
        if (quantidadeGramatura > 0) {
            widget.currentRow.VRPESOGRAMATURA = quantidade / quantidadeGramatura;
        } else {
            widget.currentRow.VRPESOGRAMATURA = 0;
        }
    };

    this.exportaServico = function(widget) {
        var exportServicoPopup = widget.container.getWidget("widgetExportServico");
        ScreenService.openPopup(exportServicoPopup);

        widget.getParent().getField("DTINIVGMA").readOnlyOnEdit = false;
        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }

        exportServicoPopup.getField("NMFILIALORI").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        exportServicoPopup.getField("NMCIAORI").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        exportServicoPopup.getField("DSMANUALORI").defaultValue = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
        exportServicoPopup.newRow();
        var servicos = "";

        exportServicoPopup.currentRow.LISTASERVORI = [];
        var checkedRows = widget.getCheckedRows();
        for (let index in checkedRows) {
            servicos += checkedRows[index].DSSERVICOCIAFILI + "|" + checkedRows[index].DSCLASSE;
            if (index !== (checkedRows.length - 1)) {
                servicos += " - ";
            }
            exportServicoPopup.currentRow.LISTASERVORI.push(checkedRows[index].NRSERVICOCIAFILI + "|" + checkedRows[index].CDCLASSE);
        }
        exportServicoPopup.getField("DSSERVICOCIAFILIORI").setValue(servicos);
        exportServicoPopup.currentRow.CDFILIALORI = appliedFilter.FILTER_ROW.CDFILIAL;
        exportServicoPopup.currentRow.CDCIAORI = appliedFilter.FILTER_ROW.CDCIA;
        exportServicoPopup.currentRow.CDMANCIAORI = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
        exportServicoPopup.activate();
        exportServicoPopup.getField("CPFOTOSUBGRU").checked = false;
    };

    this.importaServico = function(widget) {
        if (widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA !== "R") {
            var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
            appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
            if (typeof(appliedFilter.FILTER_ROW) == "string") {
                appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
            }
            clearDataSourceFilter('widgetImportServico', ['NMFILIAL', 'NMCIA', 'DSMANUAL', 'DSSERVICOCIAFILI'], 'DSSERVICOCIAFILI');
            if (widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA == "A") {
                if (moment(widget.container.getWidget("grid_manual").currentRow.DTINIVGMA, "DD/MM/YYYY") <= moment() && moment(widget.container.getWidget("grid_manual").currentRow.DTFIMVGMA, "DD/MM/YYYY") >= moment()) {
                    ScreenService.openPopup(widget.container.getWidget("widgetImportServico"));
                    widget.container.getWidget("widgetImportServico").getField("NMFILIALDEST").defaultValue = widget.container.getWidget("grid_manual").appliedFilter.FILTER_ROW.NMFILIAL;
                    widget.container.getWidget("widgetImportServico").getField("NMCIADEST").defaultValue = widget.container.getWidget("grid_manual").appliedFilter.FILTER_ROW.NMCIA;
                    widget.container.getWidget("widgetImportServico").getField("DSMANUALDEST").defaultValue = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
                    widget.container.getWidget("widgetImportServico").newRow();
                    widget.container.getWidget("widgetImportServico").currentRow.CDFILIALDES = appliedFilter.FILTER_ROW.CDFILIAL;
                    widget.container.getWidget("widgetImportServico").currentRow.CDCIADES = appliedFilter.FILTER_ROW.CDCIA;
                    widget.container.getWidget("widgetImportServico").currentRow.CDMANCIADES = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
                    widget.container.getWidget("widgetImportServico").activate();
                    widget.container.getWidget("widgetImportServico").getField("CPFOTOSUBGRU").checked = false;
                } else {
                    ScreenService.showMessage("Operação Bloqueada. Não é possível importar serviços para menus aprovados fora do período vigência!");
                }
            } else {
                ScreenService.openPopup(widget.container.getWidget("widgetImportServico"));
                widget.container.getWidget("widgetImportServico").getField("NMFILIALDEST").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
                widget.container.getWidget("widgetImportServico").getField("NMCIADEST").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
                widget.container.getWidget("widgetImportServico").getField("DSMANUALDEST").defaultValue = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
                widget.container.getWidget("widgetImportServico").newRow();
                widget.container.getWidget("widgetImportServico").currentRow.CDFILIALDES = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.container.getWidget("widgetImportServico").currentRow.CDCIADES = appliedFilter.FILTER_ROW.CDCIA;
                widget.container.getWidget("widgetImportServico").currentRow.CDMANCIADES = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
                widget.container.getWidget("widgetImportServico").activate();
                widget.container.getWidget("widgetImportServico").getField("CPFOTOSUBGRU").checked = false;
            }
        } else {
            ScreenService.showMessage("Operação Bloqueada. Não é possível importar serviços para menus reprovados!");
        }
    };

    this.exportaService = function(widget) {
        if (widget.isValid()) {
            var executaExportacaoService = function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }

                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIALORI;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIAORI;
                widget.currentRow.P_CDMANCIAORI = widget.currentRow.CDMANCIAORI;
                widget.currentRow.P_SERVCLASSE = widget.currentRow.LISTASERVORI;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIA;
                widget.currentRow.P_CDMANCIADES = widget.currentRow.CDMANCIA;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.P_CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.P_NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");

                verificaWorkflowCopiaServico(widget.currentRow).then(function(callback) {
                    if (callback === false) {
                        coc03600_menu_designService.geraNovoCodigo().then(function(callback) {
                            var data = [];
                            widget.currentRow.P_NRLISTASERVORI = callback.dataset.lista[0];
                            var arrayLista = widget.currentRow.LISTASERVORI;
                            for (let index in arrayLista) {
                                var object = {};
                                object.NRLISTA = callback.dataset.lista[0];
                                object.CDLISTA = arrayLista[index];
                                object.__is_new = true;
                                data.push(object);
                            }
                            coc03600_menu_designService.saveLista(data).then(function() {
                                coc03600_menu_designService.copiaServico([widget.currentRow]).then(function(callback) {
                                    callbackCopiaServico(widget, callback, "Serviço exportado com sucesso!");
                                }, function(error) {
                                    if (error.data) {
                                        let msg = ScreenService.i18n(error.data.error);
                                        ScreenService.showMessage(msg);
                                    } else {
                                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                        ScreenService.showMessage(msg);
                                    }
                                });
                            }, function(error) {
                                if (error.data) {
                                    let msg = ScreenService.i18n(error.data.error);
                                    ScreenService.showMessage(msg);
                                } else {
                                    let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                    ScreenService.showMessage(msg);
                                }
                            });
                        }, function(error) {
                            if (error.data) {
                                let msg = ScreenService.i18n(error.data.error);
                                ScreenService.showMessage(msg);
                            } else {
                                let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                ScreenService.showMessage(msg);
                            }
                        });
                    } else {
                        solicitaWorkflowCopiaServico(widget);
                    }
                }, function(e) {
                    ScreenService.showMessage(e);
                });
            };
            var actions = [{
                label: ScreenService.i18n("Sobrepor"),
                "default": true,
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "S";
                    executaExportacaoService();
                    deferred.resolve(e);
                }
            }, {
                label: ScreenService.i18n("Manter"),
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "N";
                    executaExportacaoService();
                    deferred.reject(e);
                }
            }, {
                label: ScreenService.i18n("Cancelar"),
                code: function(e, deferred) {
                    deferred.reject(e);
                }
            }];
            ScreenService.showCustomDialog("Caso existam serviços semelhantes, qual operação deve ser realizada?<br>Atenção: A sobreposição, apagará os vínculos com itens de faturamento.",
                "CONFIRMATION", "question", actions);
        }
    };

    this.importaService = function(widget) {
        if (widget.isValid()) {
            var executaImportacaoService = function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }

                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIA;
                widget.currentRow.P_CDMANCIAORI = widget.currentRow.CDMANCIA;
                widget.currentRow.P_SERVCLASSE = widget.currentRow.DSSERVICOCIAFILI;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIALDES;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIADES;
                widget.currentRow.P_CDMANCIADES = widget.currentRow.CDMANCIADES;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.P_CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.P_NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");

                verificaWorkflowCopiaServico(widget.currentRow).then(function(callback) {
                    if (callback === false) {
                        coc03600_menu_designService.geraNovoCodigo().then(function(callback) {
                            var data = [];
                            var object = {};
                            widget.currentRow.P_NRLISTASERVORI = callback.dataset.lista[0];
                            widget.getField("DSSERVICOCIAFILI").selectWidget.getCheckedRows().forEach(function(row) {
                                object = {};
                                object.NRLISTA = callback.dataset.lista[0];
                                object.CDLISTA = row.NRSERVICOCIAFILI + "|" + row.CDCLASSE;
                                object.__is_new = true;
                                data.push(object);
                            });
                            coc03600_menu_designService.saveLista(data).then(function() {
                                coc03600_menu_designService.copiaServico([widget.currentRow]).then(function(callback) {
                                    callbackCopiaServico(widget, callback, "Serviço importado com sucesso!");
                                }, function(error) {
                                    if (error.data) {
                                        let msg = ScreenService.i18n(error.data.error);
                                        ScreenService.showMessage(msg);
                                    } else {
                                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                        ScreenService.showMessage(msg);
                                    }
                                });
                            }, function(error) {
                                if (error.data) {
                                    let msg = ScreenService.i18n(error.data.error);
                                    ScreenService.showMessage(msg);
                                } else {
                                    let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                    ScreenService.showMessage(msg);
                                }
                            });
                        }, function(error) {
                            if (error.data) {
                                let msg = ScreenService.i18n(error.data.error);
                                ScreenService.showMessage(msg);
                            } else {
                                let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                ScreenService.showMessage(msg);
                            }
                        });
                    } else {
                        solicitaWorkflowCopiaServico(widget);
                    }
                }, function(e) {
                    ScreenService.showMessage(e);
                });
            };
            var actions = [{
                label: ScreenService.i18n("Sobrepor"),
                "default": true,
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "S";
                    executaImportacaoService();
                    deferred.resolve(e);
                }
            }, {
                label: ScreenService.i18n("Manter"),
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "N";
                    executaImportacaoService();
                    deferred.reject(e);
                }
            }, {
                label: ScreenService.i18n("Cancelar"),
                code: function(e, deferred) {
                    deferred.reject(e);
                }
            }];

            ScreenService.showCustomDialog("Caso existam serviços semelhantes, qual operação deve ser realizada?<br>Atenção: A sobreposição, apagará os vínculos com itens de faturamento.",
                "CONFIRMATION", "question", actions);
        }
    };

    var callbackCopiaServico = function(widget, callback, callbackMsg) {
        ScreenService.closePopup(true);
        widget.container.getWidget("grid_servicos").reload();
        widget.container.getWidget("grid_servicos").activate();
        widget.container.getWidget("grid_servicos").unSelectAll();
        if (callback.dataset !== undefined) {
            InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
        } else {
            ScreenService.showMessage(callbackMsg);
        }
    };

    var verificaWorkflowCopiaServico = function(currentRow) {
        var deferred = ZHPromise.defer();
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.P_CDFILIALDES
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.P_CDCIADES
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": currentRow.P_CDMANCIADES
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": currentRow.P_NRORG
            }
        ];
        InflightorGeneral.callRestEngine("coc#/crud/workflowCopiaServico", "FilterData", filter).then(function(callback) {
            if (callback.dataset && callback.dataset.workflowCopiaServico) {
                var row = callback.dataset.workflowCopiaServico[0];
                if (row && row.PROCESSOATIVO) {
                    var ret = row.PROCESSOATIVO == "TRUE" ? true : false;
                    deferred.resolve(ret);
                } else {
                    deferred.reject("Erro ao verificar a ativação do Workflow.");
                }
            } else {
                deferred.reject("Erro ao verificar a ativação do Workflow.");
            }

        }, function(e) {
            deferred.reject(e);
        });
        return deferred;
    };

    var solicitaWorkflowCopiaServico = function(widget) {
        self.callAuthUser(geraWorkflowCopiaServico, [widget]);
    };

    var geraWorkflowCopiaServico = function(widget) {
        InflightorGeneral.callRestEngine("coc#/geraWorkflowCopiaServico", "Row", widget.currentRow).then(function(callback) {
            callbackCopiaServico(widget, callback, "Workflow gerado.");
        }, function(e) {
            ScreenService.showMessage(e);
        });
    };

    this.FieldOnChangeServciafili = function(field, operation) {
        var row = field.widget.currentRow;

        var menuGridIgual;
        if (operation == "E") {
            menuGridIgual = (row.CDFILIALORI == row.CDFILIAL && row.CDCIAORI == row.CDCIA && row.CDMANCIAORI == row.CDMANCIA);
        } else {
            menuGridIgual = (row.CDFILIALDES == row.CDFILIAL && row.CDCIADES == row.CDCIA && row.CDMANCIADES == row.CDMANCIA);
        }

        if (menuGridIgual) {
            var servicoRow = templateManager.container.getWidget("grid_servicos").currentRow;
            let filterService = field.dataSourceFilter.find(filter => filter.name === 'NRSERVICOCIAFILI');
            let filterClass = field.dataSourceFilter.find(filter => filter.name === 'CDCLASSE');
            field.dataSource.name = "coc#/crud/servmanualm_servicociafili_servicogen_classe_not_in_servico";
            if (filterService) {
                filterService.value = servicoRow.NRSERVICOCIAFILI;
            } else {
                field.dataSourceFilter.push({
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servicoRow.NRSERVICOCIAFILI
                });
            }
            if (filterClass) {
                filterClass.value = servicoRow.CDCLASSE;
            } else {
                field.dataSourceFilter.push({
                    name: "CDCLASSE",
                    operator: "=",
                    value: servicoRow.CDCLASSE
                });
            }
        } else {
            field.dataSource.name = "coc#/crud/servmanualm_servicociafili_servicogen_classe";
            removeFilterInField(field, ['NRSERVICOCIAFILI', 'CDCLASSE']);
        }
    };

    function removeFilterInField(field, nameFilters) {
        nameFilters = Array.isArray(nameFilters) ? nameFilters : [nameFilters];
        nameFilters.forEach(filterName => {
            let indexFilter = field.dataSourceFilter.findIndex(filter => filter.name == filterName);
            if (indexFilter > -1) {
                field.dataSourceFilter.splice(indexFilter, 1);
            }
        });
    }

    this.exportaSubgrupo = function(widget) {
        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }
        clearDataSourceFilter('widgetExportSubgrupo', ['NMFILIAL', 'NMCIA', 'DSMANUAL', 'DSSERVICOCIAFILI']);
        ScreenService.openPopup(widget.container.getWidget("widgetExportSubgrupo"));
        widget.container.getWidget("widgetExportSubgrupo").getField("NMFILIALORI").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
        widget.container.getWidget("widgetExportSubgrupo").getField("NMCIAORI").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
        widget.container.getWidget("widgetExportSubgrupo").getField("DSMANUALORI").defaultValue = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
        widget.container.getWidget("widgetExportSubgrupo").getField("DSSERVICOCIAFILIORI").defaultValue = widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI + " |" + widget.container.getWidget("grid_servicos").currentRow.DSCLASSE;
        widget.container.getWidget("widgetExportSubgrupo").newRow();
        widget.container.getWidget("widgetExportSubgrupo").currentRow.CDFILIALORI = appliedFilter.FILTER_ROW.CDFILIAL;
        widget.container.getWidget("widgetExportSubgrupo").currentRow.CDCIAORI = appliedFilter.FILTER_ROW.CDCIA;
        widget.container.getWidget("widgetExportSubgrupo").currentRow.CDMANCIAORI = widget.container.getWidget("grid_manual").currentRow.CDMANCIA;
        widget.container.getWidget("widgetExportSubgrupo").currentRow.NRSERVICOCIAFILIORI = widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI;
        widget.container.getWidget("widgetExportSubgrupo").currentRow.CDCLASSEORI = widget.container.getWidget("grid_servicos").currentRow.CDCLASSE;
        widget.container.getWidget("widgetExportSubgrupo").activate();
        widget.container.getWidget("widgetExportSubgrupo").getField("CPFOTOSUBGRU").checked = false;
    };

    this.importaSubgrupo = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
            appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
            if (typeof(appliedFilter.FILTER_ROW) == "string") {
                appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
            }
            clearDataSourceFilter('widgetImportSubgrupo', ['NMFILIAL', 'NMCIA', 'DSMANUAL', 'DSSERVICOCIAFILI']);
            ScreenService.openPopup(widget.container.getWidget("widgetImportSubgrupo"));
            widget.container.getWidget("widgetImportSubgrupo").getField("NMFILIALDEST").defaultValue = appliedFilter.FILTER_ROW.NMFILIAL;
            widget.container.getWidget("widgetImportSubgrupo").getField("NMCIADEST").defaultValue = appliedFilter.FILTER_ROW.NMCIA;
            widget.container.getWidget("widgetImportSubgrupo").getField("DSMANUALDEST").defaultValue = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
            widget.container.getWidget("widgetImportSubgrupo").getField("DSSERVICOCIAFILIDEST").defaultValue = widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI + " |" + widget.container.getWidget("grid_servicos").currentRow.DSCLASSE;
            widget.container.getWidget("widgetImportSubgrupo").newRow();
            widget.container.getWidget("widgetImportSubgrupo").currentRow.CDFILIALDES = appliedFilter.FILTER_ROW.CDFILIAL;
            widget.container.getWidget("widgetImportSubgrupo").currentRow.CDCIADES = appliedFilter.FILTER_ROW.CDCIA;
            widget.container.getWidget("widgetImportSubgrupo").currentRow.CDMANCIADES = widget.container.getWidget("grid_servicos").currentRow.CDMANCIA;
            widget.container.getWidget("widgetImportSubgrupo").currentRow.NRSERVICOCIAFILIDES = widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI;
            widget.container.getWidget("widgetImportSubgrupo").currentRow.CDCLASSEDES = widget.container.getWidget("grid_servicos").currentRow.CDCLASSE;
            widget.container.getWidget("widgetImportSubgrupo").activate();
            widget.container.getWidget("widgetImportSubgrupo").getField("CPFOTOSUBGRU").checked = false;
        }
    };

    this.FieldOnChangeDssubgru = function(args) {
        args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        },
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDFILIAL
        },
            {
                name: "CDCIA",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDCIA
        },
            {
                name: "CDMANCIA",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDMANCIA
        },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI
        },
            {
                name: "CDCLASSE",
                operator: "=",
                value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDCLASSE
        }];
        args.owner.owner.widget.getField("DSSUBGRU").reload();
    };

    this.FieldOnChangeDssubgrus = function(args) {
        args.owner.owner.widget.getField("DSSUBGRU").dataSourceFilter = [{
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        },
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL
        },
            {
                name: "CDCIA",
                operator: "=",
                value: args.row.CDCIA
        },
            {
                name: "CDMANCIA",
                operator: "=",
                value: args.row.CDMANCIA
        },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: args.row.NRSERVICOCIAFILI
        },
            {
                name: "CDCLASSE",
                operator: "=",
                value: args.row.CDCLASSE
        }];
        args.owner.owner.widget.getField("DSSUBGRU").reload();
    };

    this.FieldOnChangeIdciclo = function(args) {
        if (args.row.DSSUBGRU !== "") {
            args.owner.owner.widget.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDFILIAL
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDCIA
            },
                {
                    name: "CDMANCIA",
                    operator: "=",
                    value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDMANCIA
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.NRSERVICOCIAFILI
            },
                {
                    name: "CDCLASSE",
                    operator: "=",
                    value: args.owner.owner.widget.container.getWidget("grid_servicos").currentRow.CDCLASSE
            }];
            args.owner.owner.widget.getField("IDCICLO").reload();
        } else {
            args.owner.owner.widget.getField("IDCICLO").dataSourceFilter = [{
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: ""
            }, {
                name: "CDCLASSE",
                operator: "=",
                value: ""
            }];
            args.owner.owner.widget.getField("IDCICLO").reload();
            let msg = ScreenService.i18n("Escolha um Subgrupo para definir os Ciclos!");
            ScreenService.showMessage(msg);
        }
    };

    this.FieldOnChangeIdcicloS = function(args) {
        if (args.row.DSSUBGRU != "") {
            args.owner.owner.widget.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: args.row.CDFILIAL
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: args.row.CDCIA
            },
                {
                    name: "CDMANCIA",
                    operator: "=",
                    value: args.row.CDMANCIA
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: args.row.NRSERVICOCIAFILI
            },
                {
                    name: "CDCLASSE",
                    operator: "=",
                    value: args.row.CDCLASSE
            }];
            args.owner.owner.widget.getField("IDCICLO").reload();
        } else {
            args.owner.owner.widget.getField("IDCICLO").dataSourceFilter = [{
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: ""
            }, {
                name: "CDCLASSE",
                operator: "=",
                value: ""
            }];
            args.owner.owner.widget.getField("IDCICLO").reload();
            let msg = ScreenService.i18n("Escolha um Subgrupo para definir os Ciclos!");
            ScreenService.showMessage(msg);
        }
    };

    this.exportaSubgru = function(widget) {
        if (widget.isValid()) {
            var executaExportacaoSubgrupo = function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }

                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIALORI;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIAORI;
                widget.currentRow.P_CDMANCIAORI = widget.currentRow.CDMANCIAORI;
                widget.currentRow.P_NRSERVICOCIAFILIORI = widget.currentRow.NRSERVICOCIAFILIORI;
                widget.currentRow.P_CDCLASSEORI = widget.currentRow.CDCLASSEORI;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIA;
                widget.currentRow.P_CDMANCIADES = widget.currentRow.CDMANCIA;
                widget.currentRow.P_NRSERVICOCIAFILIDES = widget.currentRow.NRSERVICOCIAFILI;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.P_CDCLASSEDES = widget.currentRow.CDCLASSE;
                widget.currentRow.P_CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.P_NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                if (widget.currentRow.TCICLO == "S") {
                    widget.currentRow.P_TODOCICLO = true;
                } else {
                    widget.currentRow.P_TODOCICLO = false;
                    var ciclos = widget.getField("IDCICLO").selectWidget.getCheckedRows().map(function(row) {
                        return row.CDCICLO;
                    }).join("_");
                    widget.currentRow.P_LISTACICLOS = ciclos;
                }
                coc03600_menu_designService.geraNovoCodigo().then(function(callback) {
                    var data = [];
                    var object = {};
                    widget.currentRow.P_NRLISTASUBGRUORI = callback.dataset.lista[0];
                    if (widget.currentRow.TSUBGRU == "S") {
                        widget.currentRow.P_TODOSUBGRU = true;
                        data = [widget.currentRow];
                    } else {
                        widget.currentRow.P_TODOSUBGRU = false;
                        widget.getField("DSSUBGRU").selectWidget.getCheckedRows().forEach(function(row) {
                            object = {};
                            object.NRLISTA = callback.dataset.lista[0];
                            object.CDLISTA = row.CDSUBGRU + "|" + row.NRSEQSUBGRUSEMAM;
                            object.__is_new = true;
                            data.push(object);
                        });
                    }
                    coc03600_menu_designService.saveListaSub(data, widget.currentRow.P_TODOSUBGRU).then(function() {
                        coc03600_menu_designService.copiaSubgrupo([widget.currentRow]).then(function() {
                            ScreenService.closePopup(true);
                            widget.container.getWidget("zh_crosstab").reload();
                            widget.container.getWidget("zh_crosstab").activate();
                            if (callback.dataset && callback.dataset.ListaErros) {
                                InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                            } else {
                                MessageService.showMessageByCode("COC03600_SUBGROUP_EXPORT_SUCESS");
                            }
                        }, function(error) {
                            if (error.data) {
                                let msg = ScreenService.i18n(error.data.error);
                                ScreenService.showMessage(msg);
                            } else {
                                let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                                ScreenService.showMessage(msg);
                            }
                        });
                    }, function(error) {
                        if (error.data) {
                            let msg = ScreenService.i18n(error.data.error);
                            ScreenService.showMessage(msg);
                        } else {
                            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                            ScreenService.showMessage(msg);
                        }
                    });
                }, function(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                });
            };
            var actions = [
                {
                    label: ScreenService.i18n("Sobrepor"),
                    "default": true,
                    code: function(e, deferred) {
                        widget.currentRow.P_SOBRESCREVER = "S";
                        executaExportacaoSubgrupo();
                        deferred.resolve(e);
                    }
                }, {
                    label: ScreenService.i18n("Manter"),
                    code: function(e, deferred) {
                        widget.currentRow.P_SOBRESCREVER = "N";
                        executaExportacaoSubgrupo();
                        deferred.reject(e);
                    }
                }, {
                    label: ScreenService.i18n("Cancelar"),
                    code: function(e, deferred) {
                        deferred.reject(e);
                    }
                }
            ];
            ScreenService.showCustomDialog("Caso existam subgrupos semelhantes, qual operação deve ser realizada?<br>Atenção: A sobreposição, apagará os vínculos com itens de faturamento.", "CONFIRMATION", "question", actions);
        }
    };

    this.importaSubgru = function(widget) {
        if (widget.isValid()) {
            var executaImportacaoSubgrupo = function() {
                var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
                appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
                if (typeof(appliedFilter.FILTER_ROW) == "string") {
                    appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
                }

                widget.currentRow.P_CDFILIALORI = widget.currentRow.CDFILIAL;
                widget.currentRow.P_CDCIAORI = widget.currentRow.CDCIA;
                widget.currentRow.P_CDMANCIAORI = widget.currentRow.CDMANCIA;
                widget.currentRow.P_NRSERVICOCIAFILIORI = widget.currentRow.NRSERVICOCIAFILI;
                widget.currentRow.P_CDCLASSEORI = widget.currentRow.CDCLASSE;
                widget.currentRow.P_CDFILIPROC = appliedFilter.FILTER_ROW.CDFILIAL;
                widget.currentRow.P_CDFILIALDES = widget.currentRow.CDFILIALDES;
                widget.currentRow.P_CDCIADES = widget.currentRow.CDCIADES;
                widget.currentRow.P_CDMANCIADES = widget.currentRow.CDMANCIADES;
                widget.currentRow.P_COPIA_IMAGEM = widget.currentRow.CPFOTOSUBGRU;
                widget.currentRow.P_IMAGEMDEFAULT = null;
                widget.currentRow.P_NRSERVICOCIAFILIDES = widget.currentRow.NRSERVICOCIAFILIDES;
                widget.currentRow.P_CDCLASSEDES = widget.currentRow.CDCLASSEDES;
                widget.currentRow.P_CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
                widget.currentRow.P_NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                if (widget.currentRow.TCICLO == "S") {
                    widget.currentRow.P_TODOCICLO = true;
                } else {
                    widget.currentRow.P_TODOCICLO = false;
                    var ciclos = widget.getField("IDCICLO").dataSource.getCheckedRows().map(function(row) {
                        return row.CDCICLO;
                    }).join("_");
                    widget.currentRow.P_LISTACICLOS = ciclos;
                }
                coc03600_menu_designService.geraNovoCodigo().then(function(callback) {
                    var data = [];
                    var object = {};
                    widget.currentRow.P_NRLISTASUBGRUORI = callback.dataset.lista[0];
                    if (widget.currentRow.TSUBGRU == "S") {
                        widget.currentRow.P_TODOSUBGRU = true;
                        data = [widget.currentRow];
                    } else {
                        widget.currentRow.P_TODOSUBGRU = false;
                        widget.getField("DSSUBGRU").dataSource.getCheckedRows().forEach(function(row) {
                            object = {};
                            object.NRLISTA = callback.dataset.lista[0];
                            object.CDLISTA = row.CDSUBGRU + "|" + row.NRSEQSUBGRUSEMAM;
                            object.__is_new = true;
                            data.push(object);
                        });
                    }
                    coc03600_menu_designService.saveListaSub(data, widget.currentRow.P_TODOSUBGRU).then(function() {
                        coc03600_menu_designService.copiaSubgrupo([widget.currentRow]).then(function() {
                            ScreenService.closePopup(true);
                            self.loadDataSource().then(function() {
                                widget.container.getWidget("zh_crosstab").activate();
                                MessageService.showMessageByCode("COC03600_SUBGROUP_EXPORT_SUCESS");
                            });
                        }, function(error) {
                            if (error.data) {
                                let msg = ScreenService.i18n(error.data.error);
                                ScreenService.showMessage(msg);
                            } else {
                                MessageService.showMessageByCode("NO_CONNECTION");
                            }
                        });
                    }, function(error) {
                        if (error.data) {
                            let msg = ScreenService.i18n(error.data.error);
                            ScreenService.showMessage(msg);
                        } else {
                            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                            ScreenService.showMessage(msg);
                        }
                    });
                }, function(error) {
                    if (error.data) {
                        let msg = ScreenService.i18n(error.data.error);
                        ScreenService.showMessage(msg);
                    } else {
                        let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
                        ScreenService.showMessage(msg);
                    }
                });
            };
            var actions = [{
                label: ScreenService.i18n("Sobrepor"),
                "default": true,
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "S";
                    executaImportacaoSubgrupo();
                    deferred.resolve(e);
                }
            }, {
                label: ScreenService.i18n("Manter"),
                code: function(e, deferred) {
                    widget.currentRow.P_SOBRESCREVER = "N";
                    executaImportacaoSubgrupo();
                    deferred.reject(e);
                }
            }, {
                label: ScreenService.i18n("Cancelar"),
                code: function(e, deferred) {
                    deferred.reject(e);
                }
            }];
            ScreenService.showCustomDialog("Caso existam subgrupos semelhantes, qual operação deve ser realizada?<br>Atenção: A sobreposição, apagará os vínculos com itens de faturamento.", "CONFIRMATION", "question", actions);
        }
    };

    // WORKFLOW ITENS COMPANHIA
    this.editProdCia = function(widget) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            var nritemproc = workflowItens.ITEMCIA_ALTERACAO_QTD + "_" + workflowItens.ITEMCIA_INCLUSAO + "_" + workflowItens.ITEMCIA_EXCLUSAO;
            var row = retornaRowWorkflowPentende(widget.currentRow, nritemproc, "1", "_1_2_3_", itemCiaPrimaryKeys);
            coc03600_menu_designService.hasPendingWorkflow(row).then(function(pending) {
                widget.getField("QTPRODUTOCIA").readOnly = Boolean(pending);
                InflightorGeneral.edit(widget);
            });
        }
    };

    function changedOrdens(ordens) {
        let classes = [];
        classes = InflightorGeneral.sortArrayByProperty(ordens, ['NRORDEM']);
        for (let i = 0; i < classes.length; i++) {
            if (i) {
                let diff = classes[i].NRORDEM - classes[i - 1].NRORDEM;
                if (!diff) {
                    classes[i].NRORDEM++;
                    classes[i].__is_new = false;
                    classes[i].changed = true;
                }
            }
        }
        return classes;
    }

    function _chagendAndSave(widget, filter) {
        let classesChanged = [];
        let novasOrdens;
        classesChanged = widget.dataSource.data.filter(function(row) {
            return filter <= parseInt(row.NRORDEM);
        });
        if (filter) {
            classesChanged.unshift(classesChanged.pop());
        }
        novasOrdens = changedOrdens(classesChanged);
        return novasOrdens;
    }

    this.salvaItemCia = function(widget) {
        if (widget.isValid()) {
            verificaOrdemItensDaCompanhia(widget.currentRow).then(function(dataSource) {
                var itensCompanhia = dataSource.dataset.prociasubsemam;
                var ordenacaoValida = true;
                if (itensCompanhia.length) {
                    ordenacaoValida = validaOrdenacaoItensCompanhia(itensCompanhia, widget);
                }
                if (ordenacaoValida) {
                    if (widget.currentRow.__is_new) {
                        self.salvaItemCiaNovo(widget);
                    } else {
                        self.salvaItemCiaEditado(widget);
                    }
                } else {
                    ScreenService.showMessage("Número de ordem já cadastrado.");
                }
            });
        }
    };

    this.salvaItemCiaNovo = function(widget) {
        if (getSituacao(widget, false) == "P") {
            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.ITEMCIA_INCLUSAO).then(function(idUsaWorkflow) {
                if (idUsaWorkflow) {
                    InflightorGeneral.setFloatAttrOnCurrentRow(widget.currentRow, "QTPRODUTOCIA");
                    widget.currentRow.IDANALIZEATIV = "A";
                    var tableName = "PROCIASUBSEMAM";
                    var NMITEM = "NMPRODUTO";
                    var changedItems = ["IDANALIZEATIV", "QTPRODUTOCIA", "NRORDEM"];
                    var rowToSave = retornaRowSalvarWF(widget, tableName, workflowItens.ITEMCIA_INCLUSAO, NMITEM, changedItems, itemCiaPrimaryKeys, "2");
                    var menuName = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
                    var companhia = widget.container.getWidget("grid_manual").currentRow.NMCIA;
                    var servName = widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI;
                    var className = widget.container.getWidget("grid_servicos").currentRow.DSCLASSE;
                    var subgName = widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU;
                    var itemName = widget.currentRow.NMPRODUTO;
                    rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + companhia +
                        " \nMenu Grid: " + menuName + " \nServiço: " + servName + " \nClasse: " + className +
                        " \nSubgrupo: " + subgName + " \nItem: " + itemName;
                    var afterSave = function(widget) {
                        widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                        coc03600_menu_designService.salvaWorkflow(rowToSave).then(function(callback) {
                                ScreenService.closePopup();
                                var itensWidget = widget.container.getWidget("widgetProdutocia");
                                ScreenService.closeSwipe();
                                itensWidget.list();
                                itensWidget.activate();
                                itensWidget.reload().then(function() {
                                    ScreenService.showMessage(callback.messages[0].message);
                                });
                            },
                            callbackErrorItemCia);
                    };
                    self.solicitaCriacaoWorkflow(widget, rowToSave, afterSave);
                } else {
                    widget.currentRow.IDANALIZEATIV = "A";
                    InflightorGeneral.saveCrud(widget).then(function() {
                        widget.list();
                        widget.activate();
                    });
                }
            });
        } else {
            widget.currentRow.IDANALIZEATIV = "I";
            InflightorGeneral.saveCrud(widget).then(function() {
                widget.list();
                widget.activate();
            });
        }
    };

    this.salvaItemCiaEditado = function(widget) {
        if (getSituacao(widget, false) == "P") {
            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.ITEMCIA_ALTERACAO_QTD).then(function(idUsaWorkflow) {
                if (idUsaWorkflow) {
                    if (widget.backupRow && (widget.backupRow.QTPRODUTOCIA != widget.currentRow.QTPRODUTOCIA)) {
                        var tableName = "PROCIASUBSEMAM";
                        var NMITEM = "NMPRODUTO";
                        var changedItems = ["QTPRODUTOCIA"];
                        var rowToSave = retornaRowSalvarWF(widget, tableName, workflowItens.ITEMCIA_ALTERACAO_QTD, NMITEM, changedItems, itemCiaPrimaryKeys, "1");
                        var menuName = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
                        var companhia = widget.container.getWidget("grid_manual").currentRow.NMCIA;
                        var servName = widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI;
                        var className = widget.container.getWidget("grid_servicos").currentRow.DSCLASSE;
                        var subgName = widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU;
                        var itemName = widget.currentRow.NMPRODUTO;
                        rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + companhia +
                            " \nMenu Grid: " + menuName + " \nServiço: " + servName + " \nClasse: " + className +
                            " \nSubgrupo: " + subgName + " \nItem: " + itemName;
                        var afterSave = function(widget) {
                            var widgetProdutocia = widget.container.getWidget("widgetProdutocia");
                            widgetProdutocia.currentRow.IDANALIZEATIV = "P";
                            widgetProdutocia.currentRow.QTPRODUTOCIA = widgetProdutocia.backupRow.QTPRODUTOCIA;
                            InflightorGeneral.saveCrud(widgetProdutocia).then(function() {
                                widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                                coc03600_menu_designService.salvaWorkflow(rowToSave).then(function(callback) {
                                    ScreenService.showMessage(callback.messages[0].message);
                                    ScreenService.closePopup();
                                }, callbackErrorItemCia);
                            });
                        };
                        self.solicitaCriacaoWorkflow(widget, rowToSave, afterSave);
                    } else {
                        self.salvaItemCiaEditadoSemWF(widget);
                    }
                } else {
                    self.salvaItemCiaEditadoSemWF(widget);
                }
            });
        } else {
            self.salvaItemCiaEditadoSemWF(widget);
        }
    };

    this.changeField = function(field, arraySonFields, firstSonField) {
        for (var index in arraySonFields) {
            var fieldSon = field.widget.getField(arraySonFields[index]);
            fieldSon.clearValue();
            fieldSon.readOnly = true;
        }
        if (field.value()) {
            field.widget.getField(firstSonField).readOnly = false;
        } else {
            field.widget.getField(firstSonField).readOnly = true;
        }
    };

    this.salvaItemCiaEditadoSemWF = function(widget) {
        widget.dataSource.save().then(function() {
            if (widget.inEditMode()) {
                widget.view();
                widget.activate();
            } else {
                widget.reload().then(function() {
                    widget.inlineEditing = false;
                    widget.commitInlineEdit();
                    widget.list();
                    widget.activate();
                });
            }
        }, callbackErrorItemCia);
    };

    this.deletaItemCia = function(widget) {
        if (desenvolvimentoOuProposta(getSituacao(widget), true)) {
            var nritemproc = workflowItens.ITEMCIA_ALTERACAO_QTD + "_" + workflowItens.ITEMCIA_INCLUSAO + "_" + workflowItens.ITEMCIA_EXCLUSAO;
            var row = retornaRowWorkflowPentende(widget.currentRow, nritemproc, "1", "_1_2_3_", itemCiaPrimaryKeys);
            coc03600_menu_designService.hasPendingWorkflow(row).then(function(pending) {
                if (pending) {
                    ScreenService.showMessage("Workflow pendente para o item. Exclusão bloqueada.", "error");
                } else {
                    ScreenService.confirmMessage("Delete Register?").then(function() {
                        if (getSituacao(widget) == "P") {
                            usaWorkflow(workflowItens.NRPROCESSO, workflowItens.ITEMCIA_EXCLUSAO).then(function(idUsaWorkflow) {
                                if (idUsaWorkflow) {
                                    widget.currentRow.IDANALIZEATIV = "A";
                                    var tableName = "PROCIASUBSEMAM";
                                    var NMITEM = "NMPRODUTO";
                                    var changedItems = ["IDANALIZEATIV"];
                                    var rowToSave = retornaRowSalvarWF(widget, tableName, workflowItens.ITEMCIA_EXCLUSAO, NMITEM, changedItems, itemCiaPrimaryKeys, "3");
                                    var menuName = widget.container.getWidget("grid_manual").currentRow.DSMANUAL;
                                    var companhia = widget.container.getWidget("grid_manual").currentRow.NMCIA;
                                    var servName = widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI;
                                    var className = widget.container.getWidget("grid_servicos").currentRow.DSCLASSE;
                                    var subgName = widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU;
                                    var itemName = widget.currentRow.NMPRODUTO;
                                    rowToSave.NMITEM = "Unidade: " + getLocalVar("NMFILIAL") + " \nCompanhia: " + companhia +
                                        " \nMenu Grid: " + menuName + " \nServiço: " + servName + " \nClasse: " + className +
                                        " \nSubgrupo: " + subgName + " \nItem: " + itemName;

                                    var afterSave = function(widget) {
                                        var widgetProdutocia = widget.container.getWidget("widgetProdutocia");
                                        widgetProdutocia.currentRow.IDANALIZEATIV = "P";
                                        widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
                                        coc03600_menu_designService.salvaWorkflow(rowToSave).then(function(callback) {
                                            ScreenService.showMessage(callback.messages[0].message);
                                            ScreenService.closePopup();
                                            ScreenService.closeSwipe().then(
                                                function() {
                                                    widgetProdutocia.list();
                                                    widgetProdutocia.reload();
                                                }
                                            );
                                        }, callbackErrorItemCia);
                                    };
                                    self.solicitaCriacaoWorkflow(widget, rowToSave, afterSave);
                                } else {
                                    widget.removeRow().then(function() {
                                        ScreenService.closeSwipe().then(function() {
                                            widget.activate();
                                        });
                                    }, function(error) {
                                        if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                            MessageService.showMessageByCode("DELETE_ERROR");
                                        } else {
                                            ScreenService.showMessage(error.data.error);
                                        }
                                    });
                                }
                            });
                        } else {
                            widget.removeRow().then(function() {
                                ScreenService.closeSwipe().then(function() {
                                    widget.activate();
                                });
                            }, function(error) {
                                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                    MessageService.showMessageByCode("DELETE_ERROR");
                                } else {
                                    ScreenService.showMessage(error.data.error);
                                }
                            });
                        }
                    });
                }
            });
        }
    };

    function callbackErrorItemCia(error) {
        if (error.data) {
            let msg = ScreenService.i18n(error.data.error);
            ScreenService.showMessage(msg);
        } else {
            let msg = ScreenService.i18n("Verifique sua conexão e tente novamente!");
            ScreenService.showMessage(msg);
        }
    }

    this.validaMenuSemServico = function(widget, status) {
        InflightorGeneral.callRestEngine("coc#/validateTornarProposta", "Row", widget.currentRow)
            .then(function() {
                self.confirmaProposta(widget, status);
            }, function(err) {
                InflightorGeneral.showErrorMessages(widget, err.data.error);
            });
    };

    this.deleteMenuGrid = function(widget) {
        if (desenvolvimento(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA)) {
            var data = [];
            var message = "Serão excluídos o Menu Grid e todos os seus elementos. Confirma a operação?";
            var row = {
                "CDFILIAL": widget.currentRow.CDFILIAL,
                "CDCIA": widget.currentRow.CDCIA,
                "CDMANCIA": widget.currentRow.CDMANCIA
            };
            data.push(row);
            ScreenService.confirmMessage(message, "question", function() {
                var callbackAuthSuccess = function(widget) {
                    InflightorGeneral.callRestEngine("coc#/deleteMenuGrid", "DataSet", data).then(function() {
                        ScreenService.closePopup(true);
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload().then(function() {
                            MessageService.showMessageByCode("DELETE_SUCCESS");
                        });
                    }, function(err) {
                        ScreenService.showMessage(err.data.error, "error");
                    });
                };

                coc03600_menu_designService.verificaTabelaPreco(widget.currentRow).then(function(tabelaPreco) {
                    if (tabelaPreco.dataset.verifica_menugrid_tabelapreco.length === 0) {
                        self.callAuthUser(callbackAuthSuccess, [widget]);
                    } else {
                        var mensagem = [];
                        tabelaPreco.dataset.verifica_menugrid_tabelapreco.forEach(function(tabela) {
                            mensagem.push("A tabela de preço '" + tabela.NRTABPRECO + " - " + tabela.NMTABPRECO + "' está vinculada ao Menu Grid.");
                        });
                        InflightorGeneral.showErrorMessages(widget, mensagem, "Exclusão não permitida");
                    }
                });
            });
        } else {
            let msg = "Operação Bloqueada. Menu grid já está em proposta ou foi aprovado/reprovado.";
            ScreenService.showMessage(msg, "error");
        }
    };

    this.deleteServicoInteiro = function(widget) {
        if (desenvolvimento(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA)) {
            var data = [];
            var row = {
                "CDFILIAL": widget.currentRow.CDFILIAL,
                "CDCIA": widget.currentRow.CDCIA,
                "CDMANCIA": widget.currentRow.CDMANCIA,
                "NRSERVICOCIAFILI": widget.currentRow.NRSERVICOCIAFILI,
                "CDCLASSE": widget.currentRow.CDCLASSE
            };
            data.push(row);

            ScreenService.confirmMessage(ScreenService.i18n("Serão excluídos o Serviço e todos os seus elementos. Confirma a operação?"), "question", function() {
                var callbackAuthSuccess = function(widget) {
                    InflightorGeneral.callRestEngine("coc#/deleteServico", "DataSet", data).then(function() {
                        ScreenService.closePopup(true);
                        widget.activate();
                        widget.reload().then(function() {
                            widget.unSelectAll();
                            MessageService.showMessageByCode("DELETE_SUCCESS");
                        });
                    }, function(err) {
                        ScreenService.showMessage(err.data.error, "error");
                    });
                };
                self.callAuthUser(callbackAuthSuccess, [widget]);
            });
        } else {
            let msg = ScreenService.i18n("Operação Bloqueada. Menu grid já está em proposta ou foi aprovado/reprovado.");
            ScreenService.showMessage(msg, "error");
        }
    };

    this.deleteSubgrupoInteiro = function(widget) {
        if (desenvolvimento(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA) || widget.currentRow.IDALTERAAPROVADO == "S") {
            var data = [];
            var row = {
                "CDFILIAL": widget.currentRow.CDFILIAL,
                "CDCIA": widget.currentRow.CDCIA,
                "CDMANCIA": widget.currentRow.CDMANCIA,
                "NRSERVICOCIAFILI": widget.currentRow.NRSERVICOCIAFILI,
                "CDCLASSE": widget.currentRow.CDCLASSE,
                "CDSUBGRU": widget.currentRow.CDSUBGRU,
                "NRSEQSUBGRUSEMAM": widget.currentRow.NRSEQSUBGRUSEMAM
            };
            data.push(row);

            ScreenService.confirmMessage(ScreenService.i18n("Serão excluídos o Subgrupo e todos os seus elementos. Confirma a operação?"), "question", function() {
                var callbackAuthSuccess = function(widget) {
                    InflightorGeneral.callRestEngine("coc#/deleteSubgrupo", "DataSet", data).then(function() {
                        ScreenService.closePopup(true);
                        ScreenService.closeSwipe();
                        widget.container.getWidget("zh_crosstab").activate();
                        self.loadDataSource().then(function() {
                            MessageService.showMessageByCode("DELETE_SUCCESS");
                        });
                    }, function(err) {
                        ScreenService.showMessage(err.data.error, "error");
                    });
                };
                self.callAuthUser(callbackAuthSuccess, [widget]);
            });
        } else {
            let msg = ScreenService.i18n("Operação Bloqueada. Menu grid já está em proposta ou foi aprovado/reprovado.");
            ScreenService.showMessage(msg, "error");
        }
    };

    this.deleteCicloInteiro = function(widget) {
        if (desenvolvimento(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA) || widget.container.getWidget("widgetSubgrupo").currentRow.IDALTERAAPROVADO == "S") {
            var data = [];
            var row = {
                "CDFILIAL": widget.currentRow.CDFILIAL,
                "CDCIA": widget.currentRow.CDCIA,
                "CDMANCIA": widget.currentRow.CDMANCIA,
                "NRSERVICOCIAFILI": widget.currentRow.NRSERVICOCIAFILI,
                "CDCLASSE": widget.currentRow.CDCLASSE,
                "CDSUBGRU": widget.currentRow.CDSUBGRU,
                "NRSEQSUBGRUSEMAM": widget.currentRow.NRSEQSUBGRUSEMAM,
                "CDCICLO": widget.currentRow.CDCICLO
            };
            data.push(row);

            ScreenService.confirmMessage(ScreenService.i18n("Serão excluídos o Ciclo e todos os seus elementos. Confirma a operação?"), "question", function() {
                var callbackAuthSuccess = function(widget) {
                    InflightorGeneral.callRestEngine("coc#/deleteCiclo", "DataSet", data).then(function() {
                        ScreenService.closePopup(true);
                        ScreenService.closeSwipe();
                        widget.parent.activate();
                        widget.reload().then(function() {
                            self.refresh();
                            MessageService.showMessageByCode("DELETE_SUCCESS");
                        });
                    }, function(err) {
                        ScreenService.showMessage(err.data.error, "error");
                    });
                };
                self.callAuthUser(callbackAuthSuccess, [widget]);
            });
        } else {
            let msg = ScreenService.i18n("Operação Bloqueada. Menu grid já está em proposta ou foi aprovado/reprovado.");
            ScreenService.showMessage(msg, "error");
        }
    };

    this.classeNaoAssociada = function(field) {
        var servClasses = field.widget.dataSource.data.filter(function(item) {
            return item.NRSERVICOCIAFILI == field.widget.currentRow.NRSERVICOCIAFILI;
        });
        var classes = [];
        servClasses.forEach(function(classe) {
            if (classe.CDCLASSE.slice(0) != "")
                classes.push(classe.CDCLASSE.slice(0));
        });
        if (field.dataSourceFilter.length > 4)
            field.dataSourceFilter.pop();
        if (classes.length) {
            field.dataSourceFilter.push({
                "name": "CDCLASSE",
                "operator": "NOT_IN",
                "value": classes
            });
        }
        field.reload();
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc03600_menu_designController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.importarCiclo = function(widget) {
        widget.activate();
        if (desenvolvimentoOuProposta(getSituacao(widget), true)) {
            var popupImport = widget.container.getWidget("cicloImport");
            const fieldCia = popupImport.getField('NMCIA');
            popupImport.getField("DSMANUAL").readOnly = true;
            popupImport.getField("SERVICO_CLASSE").readOnly = true;
            popupImport.getField("DSSUBGRU").readOnly = true;
            popupImport.getField("IDCICLOORIGEM").readOnly = true;
            clearDataSourceFilter(popupImport, ['NMFILIAL', 'NMCIA', 'DSMANUAL', 'SERVICO_CLASSE', 'DSSUBGRU', 'IDCICLOORIGEM']);
            popupImport.getField('NMFILIAL').applyDefaultValue();
            fieldCia.readOnly = false;
            let filterUnit = fieldCia.dataSourceFilter.find(filter => filter.name == 'CDFILIAL');
            if (filterUnit) filterUnit.value = 'FILIAL_LOGADA';
            ScreenService.openPopup(popupImport);
            popupImport.newRow();
            popupImport.activate();
            setRowWithCurrentData(popupImport, "I");
        }
    };

    this.clickFieldSubgrupoOnImporConfigCiclo = function(field) {
        var row = field.widget.currentRow;
        var fieldFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: row.CDMANCIA
            },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: row.NRSERVICOCIAFILI
            },
            {
                name: "CDCLASSE",
                operator: "=",
                value: row.CDCLASSE
            }
        ];

        field.dataSource.name = "coc#/crud/subgrusemam_subgrupom";
        field.dataSourceFilter = fieldFilter;
        field.reload();
    };

    this.applyInactiveStyleReceita = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc03600_menu_designController.inactiveBranchWarningReceita(args);";
    };

    this.inactiveBranchWarningReceita = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_RECIPE");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.applyInactiveStyleProduto = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            },
            {
                "condition": "row.IDPRODATIVPR == 'S' && row.PRODORCAMENTO == 'S'",
                "cssClass": "inf-orange"
            }
        ];
        field.selectWidget.events[0].code = "coc03600_menu_designController.inactiveBranchWarningProduto(args);";
    };

    this.inactiveBranchWarningProduto = function(args) {
        if (args.owner.currentRow.IDPRODATIVPR === "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        } else {
            args.owner.setSelected(args);
        }
    };

    function executeOperationCiclo(widget) {
        var dataProcedure = [];
        var row = widget.currentRow;
        var complementoOri = row.operation == "E" ? "ORI" : "";
        var complementoDes = row.operation == "I" ? "DES" : "";

        for (let index in row.IDCICLOORIGEM) {
            var rowProcedure = {
                CDFILIALORI: row["CDFILIAL" + complementoOri],
                CDCIAORI: row["CDCIA" + complementoOri],
                CDMANCIAORI: row["CDMANCIA" + complementoOri],
                NRSERVICOCIAFILIORI: row["NRSERVICOCIAFILI" + complementoOri],
                CDCLASSEORI: row["CDCLASSE" + complementoOri],
                CDSUBGRUORI: row["CDSUBGRU" + complementoOri],
                NRSEQSUBGRUSEMAMORI: row["NRSEQSUBGRUSEMAM" + complementoOri],
                CDCICLOORIGEM: row.IDCICLOORIGEM[index],
                CDFILIALDES: row["CDFILIAL" + complementoDes],
                CDCIADES: row["CDCIA" + complementoDes],
                CDMANCIADES: row["CDMANCIA" + complementoDes],
                NRSERVICOCIAFILIDES: row["NRSERVICOCIAFILI" + complementoDes],
                CDCLASSEDES: row["CDCLASSE" + complementoDes],
                CDSUBGRUDES: row["CDSUBGRU" + complementoDes],
                NRSEQSUBGRUSEMAMDES: row["NRSEQSUBGRUSEMAM" + complementoDes],
                CDCICLODESTINO: row.IDCICLOORIGEM[index],
                COPIA_IMAGEM: row.COPIA_IMAGEM,
                COPIA_IT: row.COPIA_IT,
                SOBRESCREVER: row.SOBRESCREVER
            };
            dataProcedure.push(rowProcedure);
        }
        coc03600_menu_designService.copiaCiclos(dataProcedure).then(function() {
                ScreenService.closePopup(true);
                var widgetCiclo = widget.container.getWidget("widgetCiclo");
                if (widget.currentRow.operation == "E") {
                    widgetCiclo.unSelectAll();
                }
                widgetCiclo.reload();
                widgetCiclo.activate();
            },
            InflightorGeneral.callbackError
        );
    }

    this.importarConfiguracaoProcedure = function(widget) {

        if (widget.isValid()) {

            var dataProcedure = [];
            var widgetCiclo = widget.container.getWidget("widgetCiclo");
            dataProcedure.push({
                CDFILIALORI: widget.currentRow.CDFILIAL,
                CDCIAORI: widget.currentRow.CDCIA,
                CDMANCIAORI: widget.currentRow.CDMANCIA,
                NRSERVICOCIAFILIORI: widget.currentRow.NRSERVICOCIAFILI,
                CDCLASSEORI: widget.currentRow.CDCLASSE,
                CDSUBGRUORI: widget.currentRow.CDSUBGRU,
                NRSEQSUBGRUSEMAMORI: widget.currentRow.NRSEQSUBGRUSEMAM,
                CDCICLOORIGEM: widget.currentRow.CDCICLO,
                CDFILIALDES: widgetCiclo.currentRow.CDFILIAL,
                CDCIADES: widgetCiclo.currentRow.CDCIA,
                CDMANCIADES: widgetCiclo.currentRow.CDMANCIA,
                NRSERVICOCIAFILIDES: widgetCiclo.currentRow.NRSERVICOCIAFILI,
                CDCLASSEDES: widgetCiclo.currentRow.CDCLASSE,
                CDSUBGRUDES: widgetCiclo.currentRow.CDSUBGRU,
                NRSEQSUBGRUSEMAMDES: widgetCiclo.currentRow.NRSEQSUBGRUSEMAM,
                CDCICLODESTINO: widgetCiclo.currentRow.CDCICLO,
                COPIA_IMAGEM: widget.currentRow.COPIA_IMAGEM,
                COPIA_IT: widget.currentRow.COPIA_IT,
                SOBRESCREVER: "S"
            });

            coc03600_menu_designService.copiaCiclos(dataProcedure).then(function() {
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe();
                    widgetCiclo.reload();
                    widget.reload();
                    widgetCiclo.activate();
                },
                InflightorGeneral.callbackError
            );

        }

    };

    this.operationCiclo = function(widget, operation) {
        if (desenvolvimentoOuProposta(widget.container.getWidget("grid_manual").currentRow.IDSITUAPROPCIA, true)) {
            if (widget.isValid()) {
                widget.currentRow.operation = operation;
                var actions = [
                    {
                        label: ScreenService.i18n("Sobrepor"),
                        "default": true,
                        code: function(e, deferred) {
                            widget.currentRow.SOBRESCREVER = "S";
                            executeOperationCiclo(widget);
                            deferred.resolve(e);
                        }.bind(this)
                    },
                    {
                        label: ScreenService.i18n("Manter"),
                        code: function(e, deferred) {
                            widget.currentRow.SOBRESCREVER = "N";
                            executeOperationCiclo(widget);
                            deferred.reject(e);
                        }.bind(this)
                    },
                    {
                        label: ScreenService.i18n("Cancelar"),
                        code: function(e, deferred) {
                            deferred.reject(e);
                        }
                    }
                ];
                ScreenService.showCustomDialog("Atenção: Caso existam ciclos semelhantes, qual operação deve ser realizada?",
                    "CONFIRMATION", "question", actions);
            }
        }
    };

    this.exportarCiclo = function(widget) {
        widget.activate();
        if (desenvolvimentoOuProposta(getSituacao(widget), true)) {
            var popupExport = widget.container.getWidget("cicloExport");
            popupExport.getField("DSMANUAL").readOnly = true;
            popupExport.getField("SERVICO_CLASSE").readOnly = true;
            popupExport.getField("DSSUBGRU").readOnly = true;
            popupExport.getField("NMFILIAL").applyDefaultValue();
            clearDataSourceFilter(popupExport, ['NMFILIAL', 'NMCIA', 'DSMANUAL', 'SERVICO_CLASSE', 'DSSUBGRU']);
            const fieldCia = popupExport.getField('NMCIA');
            fieldCia.readOnly = false;
            let filterUnit = fieldCia.dataSourceFilter.find(filter => filter.name == 'CDFILIAL');
            if (filterUnit) filterUnit.value = 'FILIAL_LOGADA';
            ScreenService.openPopup(popupExport);
            popupExport.newRow();
            popupExport.currentRow.IDCICLOORIGEM = getCdCicloFromWidget(widget);
            popupExport.activate();
            setRowWithCurrentData(popupExport, "E");
        }
    };

    function getCdCicloFromWidget(widget) {
        var cdciclo = [];
        var checkedRows = widget.getCheckedRows();
        for (let index in checkedRows) {
            cdciclo.push(checkedRows[index].CDCICLO);
        }
        return cdciclo;
    }

    function setRowWithCurrentData(widget, operation) {
        var complemento = operation == "I" ? "DES" : "ORI";
        var row = widget.currentRow;

        var appliedFilter = widget.container.getWidget("grid_manual").appliedFilter || {};
        appliedFilter.FILTER_ROW = appliedFilter.FILTER_ROW || {};
        if (typeof(appliedFilter.FILTER_ROW) == "string") {
            appliedFilter.FILTER_ROW = JSON.parse(appliedFilter.FILTER_ROW);
        }

        var currentData = appliedFilter.FILTER_ROW;
        row["CDFILIAL" + complemento] = currentData.CDFILIAL;
        row["NMFILIAL" + complemento] = currentData.NMFILIAL;
        row["NMCIA" + complemento] = currentData.NMCIA;
        row["CDCIA" + complemento] = currentData.CDCIA;

        currentData = widget.container.getWidget("grid_manual").currentRow;
        row["DSMANUAL" + complemento] = currentData.DSMANUAL;
        row["CDMANCIA" + complemento] = currentData.CDMANCIA;

        currentData = widget.container.getWidget("grid_servicos").currentRow;
        row["SERVICO_CLASSE" + complemento] = currentData.SERVICO_CLASSE;
        row["CDCLASSE" + complemento] = currentData.CDCLASSE;
        row["NRSERVICOCIAFILI" + complemento] = currentData.NRSERVICOCIAFILI;

        currentData = widget.container.getWidget("widgetSubgrupo").currentRow;
        row["DSSUBGRU" + complemento] = currentData.DSSUBGRU;
        row["NRSEQSUBGRUSEMAM" + complemento] = currentData.NRSEQSUBGRUSEMAM;
        row["CDSUBGRU" + complemento] = currentData.CDSUBGRU;
    }

    this.openAutoDeleteCiclo = function(widgetCrosstab) {
        if (desenvolvimentoOuProposta(getSituacao(widgetCrosstab), true)) {
            var excludePopup = widgetCrosstab.container.getWidget("exclusaoCiclo");
            widgetCrosstab.activate();
            ScreenService.openPopup(excludePopup);
            excludePopup.newRow();
            excludePopup.getField("DSSUBGRU").readOnly = true;
            delete excludePopup.getField("DSSUBGRU").validations.required;
            excludePopup.getField("IDCICLO").readOnly = true;
            delete excludePopup.getField("IDCICLO").validations.required;
            excludePopup.currentRow = angular.copy(widgetCrosstab.container.getWidget("grid_servicos").currentRow);
            excludePopup.currentRow.ALL_CICLO = true;
            excludePopup.currentRow.ALL_SUBGRUPO = true;
        }
    };

    this.clickCicloOnMultipleDelete = function(field) {
        var currentRow = field.widget.currentRow;
        var rowFilter = {
            CDFILIAL: currentRow.CDFILIAL,
            CDCIA: currentRow.CDCIA,
            CDMANCIA: currentRow.CDMANCIA,
            NRSERVICOCIAFILI: currentRow.NRSERVICOCIAFILI,
            CDCLASSE: currentRow.CDCLASSE,
            SUBGRUPO: currentRow.ALL_SUBGRUPO ? "T" : (currentRow.DSSUBGRU ? currentRow.DSSUBGRU.join("_") : "")
        };
        InflightorGeneral.onClickDependentField(field, rowFilter);
    };

    this.multipleDeleteCiclo = function(widget) {
        if (widget.isValid()) {
            ScreenService.confirmMessage("Esta operaração é irreversível. Deseja excluir os ciclos selecionados?", "question",
                function() {
                    this.callAuthUser(this.executeMultipleDelete, [widget]);
                }.bind(this)
            );
        }
    };

    this.executeMultipleDelete = function(widget) {
        widget.currentRow.SUBGRUPO = widget.currentRow.ALL_SUBGRUPO ? "T" : widget.currentRow.DSSUBGRU.join("_");
        widget.currentRow.CICLO = widget.currentRow.ALL_CICLO ? "T" : widget.currentRow.IDCICLO.join("_");

        coc03600_menu_designService.multipleDeleteCiclo(widget.currentRow).then(
            function() {
                MessageService.showMessageByCode("DELETE_SUCCESS");
                ScreenService.closePopup(true);
                var crosstab = widget.container.getWidget("zh_crosstab");
                crosstab.activate();
                self.loadDataSource();
            }.bind(this),
            InflightorGeneral.callbackError
        );
    };

    this.applyCssClassOnCosultaITmodal = function() {
        $("#popup > span > section").addClass("it-modal");
    };

    this.removeCssClassOnCosultaITmodal = function() {
        $("#popup > span > section").removeClass("it-modal");
    };

    this.openSearchITPopup = function(importWidget) {
        var searchITpopup = importWidget.container.getWidget("searchITpopup");

        searchITpopup.dataSourceFilter = setSearchITDataSourceFilter(importWidget.currentRow);
        searchITpopup.showCheckbox = (searchITpopup.dataSourceFilter.length == 7);

        searchITpopup.reload().then(function() {
            ScreenService.openPopup(searchITpopup);
        });
    };

    this.closeSearchITPopup = function(searchITpopup) {
        searchITpopup.unSelectAll();
        ScreenService.closePopup();
        self.removeCssClassOnCosultaITmodal();
    };

    var setSearchITDataSourceFilter = function(currentRow) {
        var dataSourceFilter = [
            {
                "name": "NRITAIRVISION",
                "operator": "IS_NOT_NULL",
                "value": ""
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        var fieldsToCheck = [
            "CDFILIAL",
            "CDCIA",
            "CDMANCIA",
            "NRSERVICOCIAFILI",
            "CDCLASSE"
        ];

        for (let i in fieldsToCheck) {
            var fieldToCheck = fieldsToCheck[i];
            var fieldValue = currentRow[fieldToCheck];

            if (fieldValue) {
                dataSourceFilter.push({
                    "name": fieldToCheck,
                    "operator": "=",
                    "value": fieldValue
                });
            }
        }

        return dataSourceFilter;
    };

    this.setClickedRowToImportPopup = function(searchITpopup) {
        var importWidget = searchITpopup.container.getWidget("widgetImportSubgrupo");

        importWidget.currentRow.CDFILIAL = searchITpopup.currentRow.CDFILIAL;
        importWidget.currentRow.NMFILIAL = searchITpopup.currentRow.NMFILIAL;

        importWidget.currentRow.CDCIA = searchITpopup.currentRow.CDCIA;
        importWidget.currentRow.NMCIA = searchITpopup.currentRow.NMCIA;

        importWidget.currentRow.CDMANCIA = searchITpopup.currentRow.CDMANCIA;
        importWidget.currentRow.DSMANUAL = searchITpopup.currentRow.DSMANUAL;

        importWidget.currentRow.NRSERVICOCIAFILI = searchITpopup.currentRow.NRSERVICOCIAFILI;
        importWidget.currentRow.DSSERVICOCIAFILI = searchITpopup.currentRow.DSSERVICOCIAFILI;

        importWidget.currentRow.CDCLASSE = searchITpopup.currentRow.CDCLASSE;
        importWidget.currentRow.DSCLASSE = searchITpopup.currentRow.DSCLASSE;

        importWidget.currentRow.TCICLO = "N";
        importWidget.currentRow.TSUBGRU = "N";

        importWidget.currentRow.DSSUBGRU = [searchITpopup.currentRow.SUBGRUPO];
        var rowSbg = {
            "CDSUBGRU": searchITpopup.currentRow.CDSUBGRU,
            "DSSUBGRU": searchITpopup.currentRow.DSSUBGRU,
            "NRSEQSUBGRUSEMAM": searchITpopup.currentRow.NRSEQSUBGRUSEMAM,
            "SUBGRUPO": searchITpopup.currentRow.SUBGRUPO
        };
        importWidget.getField("DSSUBGRU").dataSource.data = [rowSbg];
        importWidget.getField("DSSUBGRU").dataSource.clearCheckedRows();
        importWidget.getField("DSSUBGRU").dataSource.addCheckedRows([rowSbg]);

        importWidget.currentRow.IDCICLO = [searchITpopup.currentRow.CDCICLO];
        var rowCiclo = {
            "CDCICLO": searchITpopup.currentRow.CDCICLO,
            "IDCICLO": searchITpopup.currentRow.IDCICLO
        };
        importWidget.getField("IDCICLO").dataSource.data = [rowCiclo];
        importWidget.getField("IDCICLO").dataSource.clearCheckedRows();
        importWidget.getField("IDCICLO").dataSource.addCheckedRows([rowCiclo]);

        self.closeSearchITPopup(searchITpopup);
    };

    this.setCheckedRowsToImportPopup = function(searchITpopup) {
        var importWidget = searchITpopup.container.getWidget("widgetImportSubgrupo");

        var checkedRows = searchITpopup.getCheckedRows();

        importWidget.currentRow.CDFILIAL = checkedRows[0].CDFILIAL;
        importWidget.currentRow.NMFILIAL = checkedRows[0].NMFILIAL;

        importWidget.currentRow.CDCIA = checkedRows[0].CDCIA;
        importWidget.currentRow.NMCIA = checkedRows[0].NMCIA;

        importWidget.currentRow.CDMANCIA = checkedRows[0].CDMANCIA;
        importWidget.currentRow.DSMANUAL = checkedRows[0].DSMANUAL;

        importWidget.currentRow.NRSERVICOCIAFILI = checkedRows[0].NRSERVICOCIAFILI;
        importWidget.currentRow.DSSERVICOCIAFILI = checkedRows[0].DSSERVICOCIAFILI;

        importWidget.currentRow.CDCLASSE = checkedRows[0].CDCLASSE;
        importWidget.currentRow.DSCLASSE = checkedRows[0].DSCLASSE;

        importWidget.currentRow.TCICLO = "N";
        importWidget.currentRow.TSUBGRU = "N";

        var sbgArr = [];
        var cdSbgArr = [];

        var cicloArr = [];
        var cdCicloArr = [];

        for (let i in checkedRows) {
            if (!cdSbgArr.includes(checkedRows[i].SUBGRUPO)) {
                cdSbgArr.push(checkedRows[i].SUBGRUPO);
                sbgArr.push({
                    "CDSUBGRU": checkedRows[i].CDSUBGRU,
                    "DSSUBGRU": checkedRows[i].DSSUBGRU,
                    "NRSEQSUBGRUSEMAM": checkedRows[i].NRSEQSUBGRUSEMAM,
                    "SUBGRUPO": checkedRows[i].SUBGRUPO
                });
            }

            if (!cdCicloArr.includes(checkedRows[i].CDCICLO)) {
                cdCicloArr.push(checkedRows[i].CDCICLO);
                cicloArr.push({
                    "CDCICLO": checkedRows[i].CDCICLO,
                    "IDCICLO": checkedRows[i].IDCICLO
                });
            }
        }

        importWidget.currentRow.DSSUBGRU = cdSbgArr;
        importWidget.getField("DSSUBGRU").dataSource.data = sbgArr;
        importWidget.getField("DSSUBGRU").dataSource.clearCheckedRows();
        importWidget.getField("DSSUBGRU").dataSource.addCheckedRows(sbgArr);

        importWidget.currentRow.IDCICLO = cdCicloArr;
        importWidget.getField("IDCICLO").dataSource.data = cicloArr;
        importWidget.getField("IDCICLO").dataSource.clearCheckedRows();
        importWidget.getField("IDCICLO").dataSource.addCheckedRows(cicloArr);

        self.closeSearchITPopup(searchITpopup);
    };

    this.clearCiaField = function(widget) {
        widget.currentRow.CDCIA = "";
        widget.currentRow.NMCIA = "";
        widget.getField('NMCIA').dataSource.data = [];
    };

    this.clearManciaField = function(widget) {
        widget.currentRow.CDMANCIA = "";
        widget.currentRow.DSMANUAL = "";
        widget.getField('DSMANUAL').dataSource.data = [];
    };

    this.clearServicoField = function(widget) {
        widget.currentRow.NRSERVICOCIAFILI = "";
        widget.currentRow.CDCLASSE = "";
        widget.currentRow.DSSERVICOCIAFILI = "";
        widget.currentRow.DSSERVICOCIAFILIDEST = "";
        widget.getField('DSSERVICOCIAFILI').dataSource.data = [];
    };

    this.clearDependentField = function(widget, dependentField, othersPropertiesEmptyRow = []) {
        dependentField.forEach(function(fieldName) {
            let field = widget.getField(fieldName);
            field.clearValue();
            widget.currentRow[fieldName] = "";
            field.reload();
        });

        othersPropertiesEmptyRow.forEach(function(property) {
            widget.currentRow[property] = "";
        });
    };

    this.deletaCheckedRowsComposicao = function(widget) {
        var situacao = getSituacao(widget, true);
        var checkedRows = widget.dataSource.getCheckedRows().slice();
        checkedRows = checkedRows.map(copyRow);
        if (situacao === "E") {
            coc03600_menu_designService.removeItensComposicao(checkedRows).then(
                function() {
                    widget.unSelectAll();
                    self.carregaDataSourceComposicao();
                    if (ScreenService.getSwipeLevel(widget) === 4) {
                        ScreenService.closeSwipe();
                    }
                    MessageService.showMessageByCode("DELETE_SUCCESS");
                },
                InflightorGeneral.callbackError
            );
        } else if (situacao === "P") {
            //verificaComposicaoPendenteNovoOuApagado
            validateDeleteComposicao(checkedRows).then(
                function() {
                    validaUsaWorkflowComposicao(checkedRows).then(
                        function(callback) {
                            var bothWorkflow = callback.S && callback.P;
                            var workflowItems = [];
                            var deleteItems = [];
                            if (callback.S || callback.P) {
                                if (bothWorkflow) {
                                    workflowItems = checkedRows;
                                } else if (callback.S) {
                                    workflowItems = checkedRows.filter(function(i) {
                                        return i.ID === "S";
                                    });
                                    deleteItems = checkedRows.filter(function(i) {
                                        return i.ID === "P";
                                    });
                                } else {
                                    workflowItems = checkedRows.filter(function(i) {
                                        return i.ID === "P";
                                    });
                                    deleteItems = checkedRows.filter(function(i) {
                                        return i.ID === "S";
                                    });
                                }
                                var rowToSave = {
                                    WFDATA: workflowItems,
                                    DELETEDATA: deleteItems
                                };
                                if (workflowItems.length) {
                                    self.solicitaCriacaoWorkflow(widget, rowToSave, self.generateWorkflowToMultipleDeleteComposicao);
                                } else {
                                    coc03600_menu_designService.removeItensComposicao(deleteItems).then(
                                        function() {
                                            widget.unSelectAll();
                                            self.carregaDataSourceComposicao();
                                            if (ScreenService.getSwipeLevel(widget) === 4) {
                                                ScreenService.closeSwipe();
                                            }
                                            MessageService.showMessageByCode("DELETE_SUCCESS");
                                        },
                                        InflightorGeneral.callbackError
                                    );
                                }
                            } else {
                                coc03600_menu_designService.removeItensComposicao(checkedRows).then(
                                    function() {
                                        widget.unSelectAll();
                                        self.carregaDataSourceComposicao();
                                        if (ScreenService.getSwipeLevel(widget) === 4) {
                                            ScreenService.closeSwipe();
                                        }
                                        MessageService.showMessageByCode("DELETE_SUCCESS");
                                    },
                                    InflightorGeneral.callbackError
                                );
                            }
                        }, InflightorGeneral.callbackError
                    );
                },
                function(error) {
                    InflightorGeneral.showErrorMessages(widget, error, "Operação não permitida:");
                }
            );
        } else {
            MessageService.showMessageByCode("COC03600_MENU_GRID_APPROVED");
        }
    };

    this.generateWorkflowToMultipleDeleteComposicao = function(widget) {
        /*juliaoultimo*/
        var wfItems = widget.rowToSave.WFDATA;
        var delItems = widget.rowToSave.DELETEDATA;
        var promises = [];
        for (let index in wfItems) {
            var item = wfItems[index];
            var param = {
                currentRow: item
            };
            var metodo = retornaMetodoDeleteIdWorkflow(param);
            item.IDANALIZEATIV = "A";
            var itensAlterados = ["IDANALIZEATIV"];
            metodo.itemReferencia = "NOME";
            var rowToSave = retornaRowSalvarWFComposicao(param, itensAlterados, metodo, "3");
            var variables = {
                nmMenuGrid: widget.container.getWidget("grid_manual").currentRow.DSMANUAL,
                nmServico: widget.container.getWidget("grid_servicos").currentRow.DSSERVICOCIAFILI,
                nmSubgrupo: widget.container.getWidget("widgetSubgrupo").currentRow.DSSUBGRU,
                dsClasse: widget.container.getWidget("grid_servicos").currentRow.DSCLASSE,
                unidade: getLocalVar("NMFILIAL"),
                companhia: widget.container.getWidget("grid_manual").currentRow.NMCIA,
                idCiclo: widget.parentWidget.parent.currentRow.IDCICLO,
                item: rowToSave.NOME
            };
            var label = widget.currentRow.ID === "S" ? "Receita" : "Produto";
            var translateNMITEM = ScreenService.i18n("Unidade: __unidade__ \nCompanhia: __companhia__ \nMenu: __nmMenuGrid__ \nServiço: __nmServico__ \nClasse: __dsClasse__ \nSubgrupo: __nmSubgrupo__ \nCiclo: __idCiclo__ \n" + label + ": __item__", variables);
            rowToSave.NMITEM = translateNMITEM;
            item.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
            rowToSave.DSMOTIVOACAO = item.DSMOTIVOACAO;
            promises.push(coc03600_menu_designService.salvaWorkflow(rowToSave));
        }
        ZHPromise.all(promises).then(
            function() {
                if (ScreenService.getSwipeLevel(widget) === 4) {
                    ScreenService.closeSwipe();
                }
                self.carregaDataSourceComposicao(widget.parentWidget);
                ScreenService.closePopup();
            },
            callbackErrorComposicao
        );
        if (delItems.length) {
            coc03600_menu_designService.removeItensComposicao(delItems).then(
                function() {
                    widget.unSelectAll();
                    self.carregaDataSourceComposicao();
                    if (ScreenService.getSwipeLevel(widget) === 4) {
                        ScreenService.closeSwipe();
                    }
                    MessageService.showMessageByCode("DELETE_SUCCESS");
                },
                InflightorGeneral.callbackError
            );
        }
    };

    function validateDeleteComposicao(rows) {
        var deferred = ZHPromise.defer();
        var index;
        var promises = [];
        for (index in rows) {
            var row = rows[index];
            var IDSITWORKFLOW = "1";
            var IDTIPOPERWFLOW = "_3_2_";
            var NRITEMPROCAPROVA;
            if (row.ID === "S") {
                NRITEMPROCAPROVA = workflowItens.RECEITA_INCLUSAO + "_" + workflowItens.RECEITA_EXCLUSAO;
                row.CDRECEITA = row.CODIGO;
            } else {
                NRITEMPROCAPROVA = workflowItens.PRODUTO_INCLUSAO + "_" + workflowItens.PRODUTO_EXCLUSAO;
                row.CDPRODUTO = row.CODIGO;
            }
            var params = {
                currentRow: row
            };
            var pks = retornaMetodoSaveIdWorkflow(params).primaryKeys;
            var rowWF = retornaRowWorkflowPentende(row, NRITEMPROCAPROVA, IDSITWORKFLOW, IDTIPOPERWFLOW, pks);
            promises.push(coc03600_menu_designService.hasPendingWorkflow(rowWF));
        }
        return ZHPromise.all(promises).then(
            function(responses) {
                var errorRows = [];
                for (let j = 0; j < responses.length; j++) {
                    if (responses[j])
                        errorRows.push(rows[j]);
                }
                if (!errorRows.length) {
                    deferred.resolve();
                } else {
                    var messages = errorRows.map(messageWorkflowRemoveRows);
                    deferred.reject(messages);
                }
                return deferred.promise;
            },
            InflightorGeneral.callbackError
        );

    }

    function messageWorkflowRemoveRows(row) {
        var msg = "Não foi possível realizar a exclusão! Existe workflow pendente para ";
        if (row.ID == "P") {
            msg += "o produto " + row.NOME + ".";
        } else {
            msg += "a receita " + row.NOME + ".";
        }
        return msg;
    }

    function validaUsaWorkflowComposicao() {
        var deferred = ZHPromise.defer();
        var promises = [];
        promises.push(usaWorkflow(4, 17));
        promises.push(usaWorkflow(4, 20));
        return ZHPromise.all(promises).then(function(responses) {
            if (responses.length) {
                var response = {
                    "P": responses[0],
                    "S": responses[1]
                };
                deferred.resolve(response);
            } else {
                deferred.reject("Erro inesperado na exclusão");
            }
            return deferred.promise;
        }, InflightorGeneral.callbackError);
    }

    this.cancelaGridServico = function(args) {
        const widget = args.owner.widget;
        widget.dataSource.data.pop();
        widget.currentRow = {};
        widget.list();
        widget.getField('DSCLASSE').template = 'field/text-edit.html';
        widget.container.getWidget('zh_crosstab').isVisible = true;
    };

    this.replaceUndefinedImage = function(widget, propertyToReplace, valueToUndefined = 'UNDEFINED_IMAGE') {
        widget.dataSource.data.forEach(item => {
            if (item[propertyToReplace] && item[propertyToReplace] === valueToUndefined) {
                item[propertyToReplace] = window.undefinedImage;
            }
        });
    };

    this.onChangeUnitImportExportSubGroup = function(fieldUnit) {
        const widgetImportExport = fieldUnit.widget;
        const fieldCia = widgetImportExport.getField('NMCIA');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL'
        });

        this.clearCiaField(widgetImportExport);

        updateFilterFields(fieldCia, ['CDFILIAL'], rowToFilter);

        this.onChangeCiaImportExportSubGroup(fieldCia);
    };

    this.onChangeCiaImportExportSubGroup = function(fieldCia) {
        const widgetImportExport = fieldCia.widget;
        const fieldManual = widgetImportExport.getField('DSMANUAL');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });

        this.clearManciaField(widgetImportExport);

        updateFilterFields(fieldManual, ['CDFILIAL', 'CDCIA'], rowToFilter);

        this.onChangeManualImportExportSubGroup(fieldManual);
    };

    this.onChangeManualImportExportSubGroup = function(fieldManual) {
        const widgetImportExport = fieldManual.widget;
        const isImport = isImportSubGroup(widgetImportExport);
        const fieldService = widgetImportExport.getField('DSSERVICOCIAFILI');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA'
        });
        const paramImportExport = isImport ? 'I' : 'E';
        this.clearServicoField(widgetImportExport);
        this.FieldOnChangeServciafili(fieldService, paramImportExport);
        if (isImport) {
            this.onChangeServiceImportSubGroup(fieldService);
        }
        updateFilterFields(fieldService, ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowToFilter);
    };

    this.onChangeServiceImportSubGroup = function(fieldService) {
        const widget = fieldService.widget;
        const fieldsToClear = ['DSSUBGRU', 'IDCICLO'];
        fieldsToClear.forEach(fieldName => {
            let cField = widget.getField(fieldName);
            if (cField) {
                widget.currentRow[cField.name] = [];
            }
        });
        widget.currentRow.DSSERVICOCIAFILIDEST = "";
    };

    function isImportSubGroup(widget) {
        return widget && widget.name === 'widgetImportSubgrupo';
    }

    this.onChangeUnitImportExportService = function(fieldUnit) {
        const widgetImportExport = fieldUnit.widget;
        const fieldCia = widgetImportExport.getField('NMCIA');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL'
        });

        this.clearCiaField(widgetImportExport);

        updateFilterFields(fieldCia, ['CDFILIAL'], rowToFilter);

        this.onChangeCiaImportExportService(fieldCia);
    };

    this.onChangeCiaImportExportService = function(fieldCia) {
        const widgetImportExport = fieldCia.widget;
        const widgetGridManual = templateManager.container.getWidget('grid_manual');
        const fieldManual = widgetImportExport.getField('DSMANUAL');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });
        let filterManual = fieldManual.dataSourceFilter.find(filter => filter.name === 'CDMANCIA' && filter.operator === '<>');
        var row = fieldManual.widget.currentRow;
        var ciafiliIgual = (
            row.CDFILIALDES == row.CDFILIAL && row.CDCIADES == row.CDCIA
        );

        this.clearManciaField(widgetImportExport);

        updateFilterFields(fieldManual, ['CDFILIAL', 'CDCIA'], rowToFilter);
        if (filterManual) {
            if (ciafiliIgual) {
                var rowManual = widgetGridManual.currentRow;
                filterManual.value = rowManual.CDMANCIA;
            } else {
                filterManual.value = 'T';
            }
        }
        if (widgetImportExport.name === 'widgetImportServico')
            this.onChangeManualImportService(fieldManual);
    };

    this.onChangeManualImportService = function(fieldManual) {
        const widgetImportExport = fieldManual.widget;
        const fieldService = widgetImportExport.getField('DSSERVICOCIAFILI');
        const rowToFilter = getValuesInRow(widgetImportExport, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA'
        });
        this.clearServicoField(widgetImportExport);
        updateFilterFields(fieldService, ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowToFilter);
        fieldService.reload();
    };

    this.onChangeUnitImportExportCiclo = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        this.clearCiaField(widget);

        updateFilterFields(fieldCia, ['CDFILIAL'], rowToFilter);

        blockDependentField(fieldUnit, 'NMCIA');

        this.onChangeCiaImportExportCiclo(fieldCia);
    };

    this.onChangeCiaImportExportCiclo = function(fieldCia) {
        const widget = fieldCia.widget;
        const fieldManual = widget.getField('DSMANUAL');
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });

        this.clearManciaField(widget);

        updateFilterFields(fieldManual, ['CDFILIAL', 'CDCIA'], rowToFilter);

        blockDependentField(fieldCia, 'DSMANUAL');

        this.onChangeManualImportExportCiclo(fieldManual);
    };

    this.onChangeManualImportExportCiclo = function(fieldManual) {
        const widget = fieldManual.widget;
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA'
        });
        const fieldService = widget.getField('SERVICO_CLASSE');
        this.clearFieldServiceImportExportCiclo(widget);

        updateFilterFields(fieldService, ['CDFILIAL', 'CDCIA', 'CDMANCIA'], rowToFilter);

        blockDependentField(fieldManual, 'SERVICO_CLASSE');

        this.onChangeServiceImportExportCiclo(fieldService);
    };

    this.onChangeServiceImportExportCiclo = function(fieldService) {
        const widget = fieldService.widget;
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            DSMANUAL: 'CDMANCIA',
            SERVICO_CLASSE: ['NRSERVICOCIAFILI', 'CDCLASSE']
        });
        const fieldSubGroup = widget.getField('DSSUBGRU');
        const isWidgetImportCiclo = isImportCiclo(widget);
        const isWidgetImportConfigCiclo = isImportConfigCiclo(widget);
        this.clearSubGroupImportExportCiclo(widget);

        updateFilterFields(fieldSubGroup, ['CDFILIAL', 'CDCIA', 'CDMANCIA', 'NRSERVICOCIAFILI', 'CDCLASSE'], rowToFilter);

        blockDependentField(fieldService, 'DSSUBGRU');

        if (isWidgetImportCiclo || isWidgetImportConfigCiclo) {
            this.onChangeSubGroupImportExportCiclo(fieldSubGroup);
        }
    };

    this.onChangeSubGroupImportExportCiclo = function(fieldSubGroup) {
        const widget = fieldSubGroup.widget;
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: "CDFILIAL",
            NMCIA: "CDCIA",
            DSMANUAL: "CDMANCIA",
            SERVICO_CLASSE: ["NRSERVICOCIAFILI", "CDCLASSE"],
            DSSUBGRU: ["CDSUBGRU", "NRSEQSUBGRUSEMAM"]
        });
        const nameFieldCicloOrigem = isImportCiclo(widget) ? 'IDCICLOORIGEM' : 'IDCICLO';
        const fieldCicloOrigem = widget.getField(nameFieldCicloOrigem);
        this.clearCicloOrigemImportCiclo(widget);
        updateFilterFields(fieldCicloOrigem, ['CDFILIAL', 'CDCIA', 'CDMANCIA', 'NRSERVICOCIAFILI', 'CDCLASSE', 'CDSUBGRU', 'NRSEQSUBGRUSEMAM'], rowToFilter);
        fieldCicloOrigem.reload();
        blockDependentField(fieldSubGroup, nameFieldCicloOrigem);
    };

    function isImportCiclo(widget) {
        return widget && widget.name === 'cicloImport';
    }

    function isImportConfigCiclo(widget) {
        return widget && widget.name === 'importarConfiguracaoCiclo';
    }

    this.clearCicloOrigemImportCiclo = function(widget) {
        const nameFieldCicloOrigem = isImportCiclo(widget) ? 'IDCICLOORIGEM' : 'IDCICLO';
        const fieldCicloOrigem = widget.getField(nameFieldCicloOrigem);
        let row = widget.currentRow;
        row.IDCICLO = "";
        row.CDCICLO = "";
        fieldCicloOrigem.dataSource.data = [];
    };

    this.clearSubGroupImportExportCiclo = function(widget) {
        const fieldSubGroup = widget.getField('DSSUBGRU');
        let row = widget.currentRow;
        row.CDSUBGRU = "";
        row.DSSUBGRU = "";
        row.NRSEQSUBGRUSEMAM = "";
        fieldSubGroup.dataSource.data = [];
    };

    this.clearFieldServiceImportExportCiclo = function(widget) {
        const fieldService = widget.getField('SERVICO_CLASSE');
        fieldService.dataSource.data = [];
        let row = widget.currentRow;
        row.CDCLASSE = "";
        row.DSCLASSE = "";
        row.DSSERVICOCIAFILI = "";
        row.NRSERVICOCIAFILI = "";
        row.SERVICO_CLASSE = "";
        row.SGCLASSE = "";
    };

    function blockDependentField(field, nameFields) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        const widget = field.widget;
        let firstNode = nameFields[0];
        const fieldValue = field.value();
        let fieldFirstNode = widget.getField(firstNode);
        nameFields.forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                cField.readOnly = true;
            }
        });
        if (fieldFirstNode) {
            fieldFirstNode.readOnly = !fieldValue;
        }
    }

    function updateFilterFields(fieldUpdate, nameFilters, rowToSet) {
        fieldUpdate.dataSourceFilter.forEach(filter => {
            if (nameFilters.indexOf(filter.name) > -1) {
                filter.value = rowToSet[filter.name];
            }
        });
    }

    function getValuesInRow(widget, fieldAndPropValue) { // {FIELDNAME: NAME_GET_ROW}
        let result = {};
        Object.keys(fieldAndPropValue).forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                let valueFields = Array.isArray(fieldAndPropValue[nameField]) ? fieldAndPropValue[nameField] : [fieldAndPropValue[nameField]];
                let fieldValue = cField.value();
                valueFields.forEach(valueField => {
                    result[valueField] = fieldValue ? widget.currentRow[valueField] : '';
                });
            }
        });
        return result;
    }

    function clearDataSourceFilter(widget, fields, fieldsToReload = [], flagIgnoreValueInFilter = 'T') {
        widget = typeof widget === 'string' ? templateManager.container.getWidget(widget) : widget;
        fieldsToReload = typeof fieldsToReload === 'string' ? [fieldsToReload] : fieldsToReload;
        fields = Array.isArray(fields) ? fields : [fields];
        fields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field && field.dataSourceFilter && field.dataSourceFilter.length) {
                field.dataSourceFilter.forEach(filter => {
                    if (flagIgnoreValueInFilter !== false) {
                        filter.value = filter.value === flagIgnoreValueInFilter ? filter.value : '';
                    } else {
                        filter.value = "";
                    }
                });
                if (fieldsToReload.indexOf(field.name) > -1) field.reload();
            }
        });
    }

    this.buildFilterCopyCiclo = function(fieldCicloOrigem) {
        const row = templateManager.container.getWidget("widgetSubgrupo").currentRow;
        fieldCicloOrigem.dataSourceFilter.forEach(filter => {
            filter.value = row[filter.name];
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03600_menu_designController", coc03600_menu_designController);
});

// FILE: ./assets/js/controllers/coc03602_relatorios_do_manualController.js
var coc03602_relatorios_do_manualController = function(InflightorGeneral, ScreenService, templateManager, coc03602_relatorios_do_manualService) {

    this.setDefaultValuesOnCurrentRow = function(widgetForm) {
        var params = window.coc03602_params;
        widgetForm.newRow();
        if (params) {
            widgetForm.getAction('voltar').isVisible = true;
            var outData = {
                "CDFILIAL": "CDFILIAL",
                "NMFILIAL": "NMFILIAL",
                "CDCIA": "CDCIA",
                "NMCIA": "NMCIA"
            };
            if (params.CDMANCIA && params.DSMANUAL) {
                outData.CDMANCIA = "CDMANCIA";
                outData.DSMANUAL = "DSMANUAL";
            }
            for (let i in outData) {
                widgetForm.currentRow[outData[i]] = params[i];
            }
        } else {
            widgetForm.getAction('voltar').isVisible = false;
            widgetForm.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
            widgetForm.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
        }
        this.onChangeReport(widgetForm);
    };

    function showUnitField(widget, isVisible) {
        widget.getField("NMFILIAL").isVisible = isVisible;
    }

    function showCiaField(widget, isVisibleSelect, isVisibleSelectMultiple) {
        widget.getField("NMCIA").isVisible = isVisibleSelect;
        widget.getField("SM_NMCIA").isVisible = isVisibleSelectMultiple;
        widget.getField("TCIA").isVisible = isVisibleSelectMultiple;
    }

    function showMenuField(widget, isVisibleSelect, isVisibleSelectMultiple) {
        widget.getField("DSMANUAL").isVisible = isVisibleSelect;
        widget.getField("SM_DSMANUAL").isVisible = isVisibleSelectMultiple;
        widget.getField("TMANUAL").isVisible = isVisibleSelectMultiple;
    }

    function showClassField(widget, isVisible) {
        widget.getField("IDSERVICOCLASSE").isVisible = isVisible;
        widget.getField("TIDSERVICOCLASSE").isVisible = isVisible;
    }

    function showSubgruField(widget, isVisible) {
        widget.getField("DSSUBGRU").isVisible = isVisible;
        widget.getField("TSUBGRU").isVisible = isVisible;
    }

    function showCicloField(widget, isVisible) {
        widget.getField("IDCICLO").isVisible = isVisible;
        widget.getField("TCICLO").isVisible = isVisible;
    }

    function showVersionField(widget, isVisible) {
        widget.getField("PARACLIENTE").isVisible = isVisible;
    }

    function showLangField(widget, isVisible) {
        widget.getField("IDIOMA").isVisible = isVisible;
    }

    function showImageField(widget, isVisible) {
        widget.getField("HIDEIMG").isVisible = isVisible;
    }

    function showOnlyVigSubField(widget, isVisible) {
        widget.getField("ONLYVIGSBG").isVisible = isVisible;
    }

    function showStatusField(widget, isVisible) {
        widget.getField("IDTIPOPERWFLOW").isVisible = isVisible;
    }

    function showReceitaField(widget, isVisible) {
        widget.getField("SUBRECEITASMENU").isVisible = isVisible;
    }

    function showSituacaoField(widget, isVisible) {
        widget.getField("IDSITUAPROPCIA").isVisible = isVisible;
    }

    function buildFormPrintReport(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showUnitField(widgetForm, true);
        showCiaField(widgetForm, true, false);
        showMenuField(widgetForm, true, false);
        showClassField(widgetForm, true);
        showSubgruField(widgetForm, true);
        showCicloField(widgetForm, true);
        showLangField(widgetForm, true);
        showImageField(widgetForm, true);
    }

    function toggleAllFieldsVisibility(widgetForm, isVisible) {
        showUnitField(widgetForm, isVisible);
        showCiaField(widgetForm, isVisible, isVisible);
        showMenuField(widgetForm, isVisible, isVisible);
        showClassField(widgetForm, isVisible);
        showSubgruField(widgetForm, isVisible);
        showCicloField(widgetForm, isVisible);
        showVersionField(widgetForm, isVisible);
        showLangField(widgetForm, isVisible);
        showImageField(widgetForm, isVisible);
        showStatusField(widgetForm, isVisible);
        showReceitaField(widgetForm, isVisible);
        showSituacaoField(widgetForm, isVisible);
        showOnlyVigSubField(widgetForm, isVisible);
    }

    function buildFormReportCoc03603(widgetForm) {
        buildFormPrintReport(widgetForm);
        showOnlyVigSubField(widgetForm, true);
    }

    function buildFormReportCoc03604(widgetForm) {
        buildFormPrintReport(widgetForm);
        showVersionField(widgetForm, true);
        showLangField(widgetForm, false);
        showOnlyVigSubField(widgetForm, false);
    }

    function buildFormReportCoc03602(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showUnitField(widgetForm, true);
        showCiaField(widgetForm, true, false);
        showMenuField(widgetForm, false, true);
        showReceitaField(widgetForm, true);
        showOnlyVigSubField(widgetForm, false);
    }

    function buildFormReportCocb3602(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showUnitField(widgetForm, true);
        showCiaField(widgetForm, true, false);
        showMenuField(widgetForm, true, false);
        showOnlyVigSubField(widgetForm, true);
    }

    function buildFormReportCocc3602(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showStatusField(widgetForm, true);
        showOnlyVigSubField(widgetForm, false);
    }

    function buildFormReportCocd3602(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showUnitField(widgetForm, true);
        showCiaField(widgetForm, false, true);
        showSituacaoField(widgetForm, true);
        showOnlyVigSubField(widgetForm, false);
    }

    function buildFormReportCoc03614(widgetForm) {
        toggleAllFieldsVisibility(widgetForm, false);
        showUnitField(widgetForm, true);
        showCiaField(widgetForm, false, true);
        showMenuField(widgetForm, false, true);
    }

    this.onChangeReport = function(widgetForm) {
        var selectedReport = widgetForm.currentRow.REPORTNAME;
        switch (selectedReport) {
            case "COC03603":
                buildFormReportCoc03603(widgetForm);
                break;
            case "COC03604":
                buildFormReportCoc03604(widgetForm);
                break;
            case "COC03602":
                buildFormReportCoc03602(widgetForm);
                break;
            case "COCB3602":
                buildFormReportCocb3602(widgetForm);
                break;
            case "COCC3602":
                buildFormReportCocc3602(widgetForm);
                break;
            case "COCD3602":
                buildFormReportCocd3602(widgetForm);
                break;
            case "COC03614":
                buildFormReportCoc03614(widgetForm);
                break;
        }
        buildFilterFieldsAutoComplete(widgetForm);
        templateManager.updateTemplate();
    };

    function buildFilterFieldsAutoComplete(widget, fieldsToUpdate = ['NMCIA', 'DSMANUAL']) {
        fieldsToUpdate = typeof fieldsToUpdate === 'string' ? [fieldsToUpdate] : fieldsToUpdate;
        if (Array.isArray(fieldsToUpdate)) {
            fieldsToUpdate.forEach(fieldName => {
                let field = widget.getField(fieldName);
                if (field && field.isVisible && Array.isArray(field.dataSourceFilter) && field.dataSourceFilter.length) {
                    field.dataSourceFilter.forEach(filter => {
                        if (filter.operator.indexOf('LIKE') == -1)
                            filter.value = widget.currentRow[filter.name];
                    });
                }
            });
        }
    }

    this.onChangeUnit = function(widgetForm) {
        widgetForm.getField("NMCIA").setValue("");
        widgetForm.getField("SM_NMCIA").setValue([]);
        buildFilterFieldsAutoComplete(widgetForm, 'NMCIA');
        this.onChangeCia(widgetForm);
    };

    this.onChangeCia = function(widgetForm) {
        widgetForm.getField("DSMANUAL").setValue("");
        widgetForm.getField("SM_DSMANUAL").setValue([]);
        if (widgetForm.currentRow.REPORTNAME && widgetForm.currentRow.REPORTNAME == "COC03614") {
            widgetForm.currentRow.CDCIA = widgetForm.currentRow.SM_NMCIA && widgetForm.currentRow.SM_NMCIA.length ? widgetForm.currentRow.SM_NMCIA : widgetForm.currentRow.CDCIA;
        }
        buildFilterFieldsAutoComplete(widgetForm, 'DSMANUAL');
        this.onChangeMenu(widgetForm);
    };

    this.onChangeMenu = function(widgetForm) {
        widgetForm.getField("IDSERVICOCLASSE").setValue([]);
        this.onChangeService(widgetForm);
    };

    this.onChangeService = function(widgetForm) {
        widgetForm.getField("DSSUBGRU").setValue([]);
        this.onChangeSubgru(widgetForm);
    };

    this.onChangeSubgru = function(widgetForm) {
        widgetForm.getField("IDCICLO").setValue([]);
    };

    function getDataSourceFilterDefault() {
        return [
            {
                "name": "NRORG",
                "operator": "=",
                "value": "@getLocalVar('INFLIGHTOR_NRORGTRAB')@"
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": ""
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": ""
            }
        ];
    }

    function getFilterByService() {
        return {
            "name": "NRSERVICOCLASSE",
            "operator": "IN",
            "value": ""
        };
    }

    function getFilterBySubgru() {
        return {
            "name": "CDSUBGRU",
            "operator": "=",
            "value": ""
        };
    }

    var listaSubgrupos = function(row) {

        var lista = "";
        var rowsArr = row.DSSUBGRU;
        var rows = [];
        for (let i in rowsArr) {
            rows.push(rowsArr[i].CDSUBGRU);
        }

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i];
            } else {
                lista = lista + "_" + rows[i];
            }
        }
        return lista;
    };

    var listaSubgruposSeq = function(row) {

        var lista = "";
        var rowsArr = row.DSSUBGRU;
        var rows = [];
        for (let i in rowsArr) {
            rows.push(rowsArr[i].CDSUBGRU + "|" + rowsArr[i].NRSEQSUBGRUSEMAM);
        }

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i];
            } else {
                lista = lista + "_" + rows[i];
            }
        }
        return lista;
    };

    function getDefaultObjectFilter(currentRow) {
        return {
            "CDFILIAL": currentRow.CDFILIAL,
            "CDCIA": currentRow.CDCIA,
            "CDMANCIA": currentRow.CDMANCIA
        };
    }

    this.onClickSubgru = function(fieldSubgru, currentRow) {
        fieldSubgru.dataSourceFilter = getDataSourceFilterDefault();
        var objFilter = getDefaultObjectFilter(currentRow);
        if (currentRow.TIDSERVICOCLASSE === "N") {
            fieldSubgru.dataSourceFilter.push(getFilterByService());
            objFilter.NRSERVICOCLASSE = currentRow.IDSERVICOCLASSE;
        }
        fieldSubgru.dataSourceFilter = InflightorGeneral.buildDataSourceFilter(fieldSubgru.dataSourceFilter, objFilter);
        fieldSubgru.reload();
    };

    this.onClickCiclo = function(fieldCiclo, currentRow) {
        fieldCiclo.dataSourceFilter = getDataSourceFilterDefault();
        fieldCiclo.dataSourceFilter.push(getFilterBySubgru());
        var objFilter = getDefaultObjectFilter(currentRow);
        var subgruValue = currentRow.TSUBGRU === "N" ? listaSubgruposSeq(currentRow) : "T";
        objFilter.CDSUBGRU = subgruValue;
        fieldCiclo.dataSourceFilter = InflightorGeneral.buildDataSourceFilter(fieldCiclo.dataSourceFilter, objFilter);
        fieldCiclo.reload();
    };

    function generateReportCoc03602(currentRow) {
        if (currentRow.TMANUAL === "N") {
            if (currentRow.SM_DSMANUAL && Array.isArray(currentRow.SM_DSMANUAL)) {
                currentRow.CDMANCIA = "_" + currentRow.SM_DSMANUAL.join("_") + "_";
            }
        } else {
            currentRow.CDMANCIA = "T";
        }
        InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
    }

    function setClassOnCurrentRow(widget, currentRow) {
        if (currentRow.TIDSERVICOCLASSE === "S") {
            currentRow.SERVICOCLASSE = "T";
            currentRow.SERVCLASSE = "T";
            currentRow.CDCLASSE = "T";
        } else if (currentRow.IDSERVICOCLASSE && Array.isArray(currentRow.IDSERVICOCLASSE)) {
            var servclasseArray = [];
            var cdclasseArray = [];
            var servclasseCheckedRows = widget.getField("IDSERVICOCLASSE").dataSource.getCheckedRows();
            servclasseCheckedRows.forEach(function(row) {
                servclasseArray.push(row.NRSERVICOCIAFILI + "-" + row.CDCLASSE);
                cdclasseArray.push(row.CDCLASSE);
            });
            currentRow.SERVICOCLASSE = "_" + currentRow.IDSERVICOCLASSE.join("_") + "_";
            currentRow.SERVCLASSE = servclasseArray.join("_");
            currentRow.CDCLASSE = "_" + cdclasseArray.join("_") + "_";
        } else {
            currentRow.SERVICOCLASSE = "";
            currentRow.SERVCLASSE = "";
            currentRow.CDCLASSE = "";
        }
    }

    function setSubgrudOnCurrentRow(widget, currentRow) {
        if (currentRow.REPORTNAME == "COC03603") {
            if (currentRow.TSUBGRU === "S") {
                currentRow.SUBGRU = "T";
                currentRow.SUBGRUPO = "T";
            } else if (currentRow.DSSUBGRU && Array.isArray(currentRow.DSSUBGRU)) {
                var lista = "";
                var rowsArr = currentRow.DSSUBGRU;
                var rows = [];
                for (let i in rowsArr) {
                    rows.push(rowsArr[i].CDSUBGRU + "-" + rowsArr[i].NRSEQSUBGRUSEMAM);
                }
                for (let i = 0; i < rows.length; i++) {
                    if (lista == "") {
                        lista = rows[i];
                    } else {
                        lista = lista + "_" + rows[i];
                    }
                }
                currentRow.SUBGRUPO = currentRow.SUBGRU = lista;
            } else {
                currentRow.SUBGRU = "";
                currentRow.SUBGRUPO = "";
            }
        } else {
            if (currentRow.TSUBGRU === "S") {
                currentRow.SUBGRU = "T";
                currentRow.SUBGRUPO = "T";
            } else if (currentRow.DSSUBGRU && Array.isArray(currentRow.DSSUBGRU)) {
                currentRow.SUBGRU = listaSubgrupos(currentRow);
                var subgruArray = [];
                var subgruCheckedRows = widget.getField("DSSUBGRU").dataSource.getCheckedRows();
                subgruCheckedRows.forEach(function(row) {
                    subgruArray.push(row.CDSUBGRU);
                });
                currentRow.SUBGRUPO = subgruArray.join("_");
            } else {
                currentRow.SUBGRU = "";
                currentRow.SUBGRUPO = "";
            }
        }
    }

    function setCicloOnCurrentRow(currentRow) {
        if (currentRow.TCICLO === "S") {
            currentRow.CICLO = "T";
        } else if (currentRow.IDCICLO && Array.isArray(currentRow.IDCICLO)) {
            currentRow.CICLO = "_" + currentRow.IDCICLO.join("_") + "_";
        } else {
            currentRow.CICLO = "";
        }
    }

    function generatePrinterReport(widget, currentRow) {
        coc03602_relatorios_do_manualService.verfImgCiclosubgrusemam(widget).then(function() {
            setClassOnCurrentRow(widget, currentRow);
            setSubgrudOnCurrentRow(widget, currentRow);
            setCicloOnCurrentRow(currentRow);
            currentRow.FILESERVPATH = window.filesPath;
            InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
        });
    }

    function generateReportCocd3602(currentRow) {
        if (currentRow.TCIA === "S") {
            currentRow.P_CDCIA = "T";
        } else {
            currentRow.P_CDCIA = "_" + currentRow.SM_NMCIA.join("_") + "_";
        }
        InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
    }

    function generateReportCoc03614(currentRow) {
        if (currentRow.TCIA === "S") {
            currentRow.CIA = "T";
        } else {
            currentRow.CIA = currentRow.SM_NMCIA.join("_");
        }
        if (currentRow.TMANUAL === "N") {
            if (currentRow.SM_DSMANUAL && Array.isArray(currentRow.SM_DSMANUAL)) {
                currentRow.MANUAL = currentRow.SM_DSMANUAL.join("_");
            }
        } else {
            currentRow.MANUAL = "T";
        }
        InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
    }

    this.report = function(widget, format) {
        if (!widget.isValid())
            return;

        var currentRow = widget.currentRow;
        currentRow.__format = format;
        currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");

        switch (currentRow.REPORTNAME) {
            case "COC03602":
                generateReportCoc03602(currentRow);
                break;
            case "COC03603":
                currentRow.__locale = currentRow.IDIOMA;
                generatePrinterReport(widget, currentRow);
                // this.openReportFormatsPopup(widget.container);
                break;
            case "COC03604":
                generatePrinterReport(widget, currentRow);
                break;
            case "COCB3602": //Produtos / Receitas Inativos
                InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
                break;
            case "COCC3602": //Alteração de Menu Grid
                InflightorGeneral.openReport("coc#/relatorioDoManual", currentRow.REPORTNAME, currentRow);
                break;
            case "COCD3602": //Alteração de Menu Grid
                generateReportCocd3602(currentRow);
                break;
            case "COC03614": //Alteração de Menu Grid
                generateReportCoc03614(currentRow);
                break;
            default:
                ScreenService.showMessage("Relatório Indisponível!");
                break;
        }
    };

    this.openReportFormatsPopup = function(container) {
        var formatsPopup = container.getWidget("reports_popup_type");
        ScreenService.openPopup(formatsPopup).then(function() {
            formatsPopup.activate();
        });
    };

    this.generateReportWithFormat = function(reportsWidget, formatsPopup) {
        var reportRow = reportsWidget.currentRow;
        reportRow.__format = formatsPopup.currentRow.REPORTFORMAT;
        this.report(reportsWidget.container.getWidget("coc03602_relatorios_do_manual"), reportRow.__format);
        ScreenService.closePopup();
        reportsWidget.activate();
    };

    this.goBackContainer = function() {
        delete window.coc03602_params;
        ScreenService.goBack();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03602_relatorios_do_manualController", coc03602_relatorios_do_manualController);
});

// FILE: ./assets/js/controllers/coc03603_relatorios_do_produtoController.js
var coc03603_relatorios_do_produtoController = function(InflightorGeneral, ScreenService, restEngine, templateManager) {
    var CDCIA = "";
    var CDMANUAL = "";
    var CDLEANCODE = "";
    var CDPRODUTO = "";

    this.report = function(widget, format) {
        if (widget.isValid()) {
            widget.currentRow.REPORTTYPE = format;
            widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
            //Ajustes dos multiselect's
            if (widget.currentRow.TLEANCODE === "S") {
                CDLEANCODE = "T";
            } else {
                if (widget.currentRow.NMLEANCODE !== "") {
                    CDLEANCODE = "_";
                    widget.currentRow.NMLEANCODE.forEach(function(row) {
                        CDLEANCODE += row + "_";
                    });
                }
            }
            widget.currentRow.CDLEANCODE = CDLEANCODE;
            if (widget.currentRow.TPRODUTO === "S") {
                CDPRODUTO = "T";
            } else {
                if (widget.currentRow.NMPRODUTO !== "") {
                    CDPRODUTO = "_";
                    widget.currentRow.NMPRODUTO.forEach(function(row) {
                        CDPRODUTO += row + "_";
                    });
                }
            }
            widget.currentRow.CDPRODUTO = CDPRODUTO;
            //Verificação do relatório 'COC000C5'. Este exige a execução de uma procedure antes de abrir o relatório
            if (widget.currentRow.REPORTNAME == "COC000C5") {
                //Verifica se o usuário entrou apenas com um parâmetro
                if (CDCIA === "T") {
                    let msg = ScreenService.i18n("Favor selecionar apenas um item no campo 'Companhia'.");
                    ScreenService.showMessage(msg);
                    return;
                }
                if (CDMANUAL === "T") {
                    let msg = ScreenService.i18n("Favor selecionar apenas um item no campo 'Manual'.");
                    ScreenService.showMessage(msg);
                    return;
                }
                var datas = [];
                datas.push({ //Array de parâmetros do Relatório
                    CDCIA: widget.currentRow.CDCIA,
                    CDFILIAL: widget.currentRow.CDFILIAL,
                    CDMANUAL: widget.currentRow.CDMANCIA,
                    NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB")
                });
                restEngine.post("coc#/produtosmacia", {
                        "requestType": "DataSet",
                        "dataset": datas
                    },
                    null, null, null, null, 60000).then(
                    function(callback) {
                        widget.currentRow.CDCONTROLE = callback.dataset.response.P_CDCONTROLE;
                        InflightorGeneral.openReport("coc#/relatorioProduto", widget.currentRow.REPORTNAME, widget.currentRow);
                    },
                    function(error) {
                        if (error.data) {
                            let msg = ScreenService.i18n("Erro ao gerar a lista de produtos do Menu Design: " + error.data.error);
                            ScreenService.showMessage(msg);
                        } else {
                            let msg = ScreenService.i18n("Erro ao gerar a lista de produtos do Menu Design: Timeout");
                            ScreenService.showMessage(msg);
                        }
                    }
                );
            } else { //Emissão dos demais relatórios
                InflightorGeneral.openReport("coc#/relatorioProduto", widget.currentRow.REPORTNAME, widget.currentRow);
            }
        } else {
            ScreenService.showMessage("Preencha os campos corretamente!");
            ScreenService.closePopup();
        }

    };

    this.FieldOnChangeReportName = function(widget, row) {
        switch (row.REPORTNAME) {
            case "COC000A5":
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                //widget.getField('TCIA').isVisible         = false;
                widget.getField("DSMANUAL").isVisible = false;
                //widget.getField('TMANUAL').isVisible      = false;
                widget.getField("NMLEANCODE").isVisible = true;
                widget.getField("TLEANCODE").isVisible = true;
                widget.getField("NMPRODUTO").isVisible = true;
                widget.getField("TPRODUTO").isVisible = true;
                widget.getField("DSPRODATIVPR").isVisible = false;
                break;
            case "COC000B5":
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                //widget.getField('TCIA').isVisible         = false;
                widget.getField("DSMANUAL").isVisible = false;
                //widget.getField('TMANUAL').isVisible      = false;
                widget.getField("NMLEANCODE").isVisible = true;
                widget.getField("TLEANCODE").isVisible = true;
                widget.getField("NMPRODUTO").isVisible = true;
                widget.getField("TPRODUTO").isVisible = true;
                widget.getField("DSPRODATIVPR").isVisible = true;
                break;
            case "COC000C5":
                widget.getField("NMFILIAL").isVisible = true;
                widget.getField("NMCIA").isVisible = true;
                //widget.getField('TCIA').isVisible         = false;
                widget.getField("DSMANUAL").isVisible = true;
                //widget.getField('TMANUAL').isVisible      = true;
                widget.getField("NMLEANCODE").isVisible = true;
                widget.getField("TLEANCODE").isVisible = true;
                widget.getField("NMPRODUTO").isVisible = true;
                widget.getField("TPRODUTO").isVisible = true;
                widget.getField("DSPRODATIVPR").isVisible = false;
                break;
            default:
                widget.getField("NMFILIAL").isVisible = false;
                widget.getField("NMCIA").isVisible = false;
                //widget.getField('TCIA').isVisible         = false;
                widget.getField("DSMANUAL").isVisible = false;
                //widget.getField('TMANUAL').isVisible      = false;
                widget.getField("NMLEANCODE").isVisible = false;
                widget.getField("TLEANCODE").isVisible = false;
                widget.getField("NMPRODUTO").isVisible = false;
                widget.getField("TPRODUTO").isVisible = false;
                widget.getField("DSPRODATIVPR").isVisible = false;
                break;
        }
        templateManager.updateTemplate();
    };

    this.onChangeStatusProduct = function(field) {
        field.widget.getField('NMPRODUTO').clearValue();
    };

    this.onChangeUnit = function(field) {
        const fieldCia = field.widget.getField('NMCIA');
        const widget = field.widget;
        const {
            CDFILIAL
        } = widget.currentRow;
        fieldCia.clearValue();
        let filterUnit = fieldCia.dataSourceFilter.find(filter => filter.name === 'CDFILIAL');
        if (filterUnit) {
            filterUnit.value = CDFILIAL;
        } else {
            fieldCia.dataSourceFilter.push({
                name: "CDFILIAL",
                operator: "=",
                value: CDFILIAL
            });
        }
        field.widget.getField('DSMANUAL').clearValue();
    };

    this.onChangeCia = function(field) {
        const widget = field.widget;
        const fieldManual = widget.getField('DSMANUAL');
        const {
            CDFILIAL,
            CDCIA
        } = widget.currentRow;
        fieldManual.clearValue();
        let filterUnit = fieldManual.dataSourceFilter.find(filter => filter.name === 'CDFILIAL');
        let filterCia = fieldManual.dataSourceFilter.find(filter => filter.name === 'CDCIA');

        if (filterUnit) {
            filterUnit.value = CDFILIAL;
        } else {
            fieldManual.dataSourceFilter.push({
                name: "CDFILIAL",
                operator: "=",
                value: CDFILIAL
            });
        }
        if (filterCia) {
            filterCia.value = CDCIA;
        } else {
            fieldManual.dataSourceFilter.push({
                name: "CDCIA",
                operator: "=",
                value: CDCIA
            });
        }

    };

    this.FieldOnChangeDsmanual = function(args) {
        if (args.row.CDFILIAL === "") {
            let msg = ScreenService.i18n("Escolha uma Unidade para definir o Manual.");
            ScreenService.showMessage(msg);
        }
        if (args.row.CDCIA === "") {
            let msg = ScreenService.i18n("Escolha uma Companhia para definir o Manual.");
            ScreenService.showMessage(msg);
        }
        var filter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: args.row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: args.row.CDCIA
            }
        ];
        args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter = filter;
        args.owner.owner.widget.getField("DSMANUAL").reload();
    };

    this.carregaProduto = function(field) {
        var currentRow = field.widget.currentRow;
        var parametros = {
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB"),
            "IDATIVO": "S",
            "IDPRODATIVPR": currentRow.DSPRODATIVPR ? currentRow.DSPRODATIVPR : "T"
        };

        InflightorGeneral.onClickDependentField(field, parametros);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03603_relatorios_do_produtoController", coc03603_relatorios_do_produtoController);
});

// FILE: ./assets/js/controllers/coc03604_relatorios_versionamento_menugridController.js
var coc03604_relatorios_versionamento_menugridController = function(InflightorGeneral, ScreenService) {

    this.setDefaultValuesOnCurrentRow = function(widgetForm) {
        var params = window.coc03604_params;
        widgetForm.newRow();
        if (params) {
            var outData = {
                "CDFILIAL": "CDFILIAL",
                "NMFILIAL": "NMFILIAL",
                "CDCIA": "CDCIA",
                "NMCIA": "NMCIA"
            };
            if (params.CDMANCIA && params.DSMANUAL) {
                outData.CDMANCIA = "CDMANCIA";
                outData.DSMANUAL = "DSMANUAL";
            }
            for (let i in outData) {
                widgetForm.currentRow[outData[i]] = params[i];
            }
        } else {
            widgetForm.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
            widgetForm.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
            widgetForm.getField("REPORTNAME").applyDefaultValue();
        }
        this.onChangeReport(widgetForm, params);
    };

    this.onChangeReport = function(widget, params) {
        var valueReportName = widget.currentRow.CDREPORTNAME || widget.getField("REPORTNAME").value();
        if (valueReportName) {
            widget.fields.forEach(function(field) {
                if (field.name !== "REPORTNAME") {
                    if (field.defaultValue) {
                        field.applyDefaultValue();
                    } else {
                        if (field.name !== "NMFILIAL" && field.name !== "NMCIA") {
                            field.setValue("");
                            field.readOnly = true;
                        } else if (field.name !== "NMCIA") {
                            field.setValue("");
                            field.widget.currentRow.NMCIA = "";
                        }
                    }
                }
            });
            widget.getField("IDSERVICOCLASSE").readOnly = true;
            widget.getField("DSSUBGRU").readOnly = true;
            widget.getField("IDCICLO").readOnly = true;
            if (valueReportName === "COC03800") {
                widget.getField("NRVERSAO").isVisible = true;
                widget.getField("NRVERSAO").class = 12;
                widget.getField("NRVERSAOF").isVisible = false;
                widget.getField("NRVERSAOF").showOnForm = false;
                widget.getField("NRVERSAOF").validations = {};
            } else if (valueReportName === "COC03801") {
                widget.getField("NRVERSAO").isVisible = true;
                widget.getField("NRVERSAO").class = 6;
                widget.getField("NRVERSAO").lastOfRow = false;
                widget.getField("NRVERSAOF").isVisible = true;
                widget.getField("NRVERSAOF").showOnForm = true;
                widget.getField("NRVERSAOF").validations = {
                    "required": true
                };
            }
            if (!params) {
                widget.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
                widget.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
            }
        }
    };

    var listaSubgrupos = function(row) {

        var lista = "";
        var rowsArr = row.DSSUBGRU;
        var rows = [];
        for (let i in rowsArr) {
            rows.push(rowsArr[i].CDSUBGRU);
        }

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i];
            } else {
                lista = lista + "_" + rows[i];
            }
        }
        return lista;
    };

    function getDefaultObjectFilter(currentRow) {
        return {
            "CDFILIAL": currentRow.CDFILIAL,
            "CDCIA": currentRow.CDCIA,
            "CDMANCIA": currentRow.CDMANCIA,
            "NRVERSAO": currentRow.NRVERSAO
        };
    }

    function getDataSourceFilterDefault() {
        return [
            {
                "name": "NRORG",
                "operator": "=",
                "value": "@getLocalVar('INFLIGHTOR_NRORGTRAB')@"
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": ""
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": ""
            },
            {
                "name": "NRVERSAO",
                "operator": "=",
                "value": ""
            }
        ];
    }

    this.onChangeUnit = function(widgetForm) {
        widgetForm.getField("NMCIA").setValue("");
        this.onChangeCia(widgetForm);
    };

    this.onChangeCia = function(widgetForm) {
        widgetForm.getField("DSMANUAL").setValue("");
        this.onChangeMenu(widgetForm);
    };

    this.onChangeMenu = function(widgetForm) {
        widgetForm.getField("NRVERSAO").setValue([]);
        widgetForm.getField("NRVERSAOF").setValue([]);
        this.onChangeVersao(widgetForm);
    };

    this.onChangeVersao = function(widgetForm) {
        widgetForm.getField("IDSERVICOCLASSE").setValue([]);
        widgetForm.getField("DSSUBGRU").setValue([]);
        widgetForm.getField("IDCICLO").setValue([]);
    };

    function getFilterByService() {
        return {
            "name": "NRSERVICOCLASSE",
            "operator": "IN",
            "value": ""
        };
    }

    function getFilterBySubgru() {
        return {
            "name": "CDSUBGRU",
            "operator": "=",
            "value": ""
        };
    }

    this.onClickSubgru = function(fieldSubgru, currentRow) {
        fieldSubgru.dataSourceFilter = getDataSourceFilterDefault();
        var objFilter = getDefaultObjectFilter(currentRow);
        if (currentRow.TIDSERVICOCLASSE === "N") {
            fieldSubgru.dataSourceFilter.push(getFilterByService());
            objFilter.NRSERVICOCLASSE = currentRow.IDSERVICOCLASSE;
        }
        fieldSubgru.dataSourceFilter = InflightorGeneral.buildDataSourceFilter(fieldSubgru.dataSourceFilter, objFilter);
        fieldSubgru.reload();
    };

    this.onClickCiclo = function(fieldCiclo, currentRow) {
        fieldCiclo.dataSourceFilter = getDataSourceFilterDefault();
        fieldCiclo.dataSourceFilter.push(getFilterBySubgru());
        var objFilter = getDefaultObjectFilter(currentRow);
        var subgruValue = currentRow.TSUBGRU === "N" ? listaSubgrupos(currentRow) : "T";
        objFilter.CDSUBGRU = subgruValue;
        fieldCiclo.dataSourceFilter = InflightorGeneral.buildDataSourceFilter(fieldCiclo.dataSourceFilter, objFilter);
        fieldCiclo.reload();
    };

    this.generateReportWithFormat = function(reportsWidget, formatsPopup) {
        var reportRow = reportsWidget.currentRow;
        reportRow.__format = formatsPopup.currentRow.REPORTFORMAT;
        generatePrinterReport(reportsWidget, reportRow);
        ScreenService.closePopup();
        reportsWidget.activate();
    };

    this.report = function(widget) {
        if (!widget.isValid())
            return;

        var currentRow = widget.currentRow;
        currentRow.__format = "pdf";
        currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
        this.openReportFormatsPopup(widget.container);
    };

    this.openReportFormatsPopup = function(container) {
        var formatsPopup = container.getWidget("reports_popup_type");
        ScreenService.openPopup(formatsPopup).then(function() {
            formatsPopup.activate();
        });
    };

    function generatePrinterReport(widget, currentRow) {
        setClassOnCurrentRow(widget, currentRow);
        setSubgrudOnCurrentRow(widget, currentRow);
        setCicloOnCurrentRow(currentRow);
        currentRow.FILESERVPATH = window.filesPath;
        InflightorGeneral.openReport("coc#/relatorioVersionamentoDoManual", currentRow.REPORTNAME, currentRow);
    }

    function setClassOnCurrentRow(widget, currentRow) {
        if (currentRow.TIDSERVICOCLASSE === "T") {
            currentRow.SERVICOCLASSE = "T";
            currentRow.SERVCLASSE = "T";
            currentRow.CDCLASSE = "T";
        } else if (currentRow.IDSERVICOCLASSE && Array.isArray(currentRow.IDSERVICOCLASSE)) {
            var servclasseArray = [];
            var cdclasseArray = [];
            var servclasseCheckedRows = widget.getField("IDSERVICOCLASSE").dataSource.getCheckedRows();
            servclasseCheckedRows.forEach(function(row) {
                servclasseArray.push(row.NRSERVICOCIAFILI + "-" + row.CDCLASSE);
                cdclasseArray.push(row.CDCLASSE);
            });
            currentRow.SERVICOCLASSE = "_" + currentRow.IDSERVICOCLASSE.join("_") + "_";
            currentRow.SERVCLASSE = servclasseArray.join("_");
            currentRow.CDCLASSE = "_" + cdclasseArray.join("_") + "_";
        } else {
            currentRow.SERVICOCLASSE = "";
            currentRow.SERVCLASSE = "";
            currentRow.CDCLASSE = "";
        }
    }

    function setSubgrudOnCurrentRow(widget, currentRow) {
        if (currentRow.TSUBGRU === "T") {
            currentRow.SUBGRU = "T";
            currentRow.SUBGRUPO = "T";
        } else if (currentRow.DSSUBGRU && Array.isArray(currentRow.DSSUBGRU)) {
            currentRow.SUBGRU = listaSubgrupos(currentRow);
            var subgruArray = [];
            var subgruCheckedRows = widget.getField("DSSUBGRU").dataSource.getCheckedRows();
            subgruCheckedRows.forEach(function(row) {
                subgruArray.push(row.CDSUBGRU);
            });
            currentRow.SUBGRUPO = subgruArray.join("_");
        } else {
            currentRow.SUBGRU = "";
            currentRow.SUBGRUPO = "";
        }
    }

    function setCicloOnCurrentRow(currentRow) {
        if (currentRow.TCICLO === "T") {
            currentRow.CICLO = "T";
        } else if (currentRow.IDCICLO && Array.isArray(currentRow.IDCICLO)) {
            currentRow.CICLO = "_" + currentRow.IDCICLO.join("_") + "_";
        } else {
            currentRow.CICLO = "";
        }
    }

    var MAP_TFIELD = {
        "IDSERVICOCLASSE": "TIDSERVICOCLASSE",
        "DSSUBGRU": "TSUBGRU",
        "IDCICLO": "TCICLO"
    };

    this.onChangeField = function(field, fieldsDependentes, fieldsDirRelacionados) {

        var widget = field.widget;
        if (fieldsDependentes.length > 0) {
            for (let i = 0; i < fieldsDependentes.length; i++) {

                widget.getField(fieldsDependentes[i]).readOnly = true;
                widget.getField(fieldsDependentes[i]).value('');

                if (typeof widget.getField(MAP_TFIELD[fieldsDependentes[i]]) !== "undefined") {

                    widget.getField(MAP_TFIELD[fieldsDependentes[i]]).value('T');
                    widget.currentRow[MAP_TFIELD[fieldsDependentes[i]]] = 'T';
                    widget.getField(fieldsDependentes[i]).validations = {};

                }
            }
        }
        for (let i = 0; i < fieldsDirRelacionados.length; i++) {
            if (field.value()) {
                if (typeof widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]) !== "undefined") {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = true;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).validations = {};
                    widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]).value('T');
                    widget.currentRow[MAP_TFIELD[fieldsDirRelacionados[i]]] = 'T';
                } else {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = false;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).reload();
                }
            } else {
                field.widget.getField(fieldsDirRelacionados[i]).value('');
                field.widget.getField(fieldsDirRelacionados[i]).readOnly = true;
            }
        }

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03604_relatorios_versionamento_menugridController", coc03604_relatorios_versionamento_menugridController);
});

// FILE: ./assets/js/controllers/coc03610_custeio_menu_gridController.js
var coc03610_custeio_menu_gridController = function(ScreenService, MessageService, InflightorGeneral, ZHPromise, coc03610_custeio_menu_gridService, coc23100_geracaocustomanualService, JSONContainerLoader, templateManager, maskEngine, ZhUtil, restEngine, General) {

    var self = this;

    this.aplicaReadonlyIdservicoclasse = function(widget) {
        setaValoresPadraoRelatorio(widget.container.getWidget('custeio_grid_manual'), widget);
        widget.redraw();
    };

    this.popUpOnEnter = function(widget) {
        var widgetFilter = widget.container.getWidget("custeio_grid_manual");
        var cdcia = widgetFilter.appliedFilter.FILTER_ROW.CDCIA;
        var cdfilial = widgetFilter.appliedFilter.FILTER_ROW.CDFILIAL;
        var cdmancia = widgetFilter.appliedFilter.FILTER_ROW.CDMANCIA;
        widget.dataSourceFilter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": cdmancia
            }
        ];
        widget.reload();
    };

    var restauraEstadoFields = function(widget) {
        var fields = [
            "DSSERVICOCIAFILI",
            "TNRSERVICOCLASSE",
            "DSSUBGRU",
            "TSUBGRU",
            "IDCICLO",
            "TIDCICLO",
            "IT",
            "TNRITAIRVISION"
        ];
        for (let i = 0; i < fields.length; i++) {
            if (widget.getField(fields[i]).template === "field/select-multiple.html") {
                widget.getField(fields[i]).readOnly = true;
                widget.getField(fields[i]).validations = {};
                widget.getField(fields[i]).value('');
            }
            if (widget.getField(fields[i]).template === "field/checkbox.html") {
                widget.getField(fields[i]).value('T');
            }
        }
    };

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.buildFilter = function(mainField, fieldNameToFilter, dependentFields) {
        var filterRow = mainField.widget.currentRow;
        dependentFields.forEach(function(field) {
            if (!mainField.widget.getField(field).dataSourceFilter) {
                mainField.widget.getField(field).dataSourceFilter = [];
            }
            if (mainField.widget.getField(field).dataSourceFilter.filter(condition => condition.name === fieldNameToFilter).length) {
                mainField.widget.getField(field).dataSourceFilter.filter(condition => condition.name === fieldNameToFilter)[0].value = filterRow[fieldNameToFilter];
            } else {
                mainField.widget.getField(field).dataSourceFilter.push({
                    "name": fieldNameToFilter,
                    "operator": "=",
                    "value": filterRow[fieldNameToFilter]
                });
            }
            mainField.widget.getField(field).clearValue();
            mainField.widget.getField(field).reload();
        });
    };

    //Seta os filtros dos fields do zh-grid-filter
    this.setFilterForFilterFields = function(field) {
        if (field.name === "NMCIA") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }];
            field.reload();
        } else if (field.name === "DSMANUAL" || field.name === "SGCLASSE") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": field.widget.currentRow.CDCIA
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }];
            field.reload();
        }
    };
    //Filtra a swipe de subgrupos pelo serviço escolhido da grid
    this.afterLoadServicos = function(args) {
        ScreenService.openSwipe(args.owner.container.getWidget("grid_subgrupos"));
        args.owner.container.getWidget("grid_subgrupos").dataSourceFilter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": args.row.NRORG
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": args.row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": args.row.CDCIA
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": args.row.CDCLASSE
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": args.row.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": args.row.NRSERVICOCIAFILI
            }
        ];
        args.owner.container.getWidget("grid_subgrupos").reload();
        let value = General.valueToFloat(args.row.VRCUSTOSERVM);
        args.owner.container.getWidget("footer_sub_grupo_menu_grid").currentRow.VRCUSTOSERVM = String(value.toFixed(2)).replace(".", ",");
    };
    //Filtra a swipe de ciclos pelo subgrupo escolhido da grid
    this.afterLoadSubgrupos = function(args) {
        ScreenService.openSwipe(args.owner.container.getWidget("grid_composicao"));
    };

    this.getCurrentFieldFilter = function(fieldName, widgetName) {
        var filterPopup = templateManager.container.getWidget(widgetName).appliedFilter;
        return filterPopup.FILTER_ROW[fieldName];
    };

    var reportFields = {
        "_CUSTO": {
            "QTPAXREF": true,
            "DSSERVICOCIAFILI": true,
            "TNRSERVICOCLASSE": true,
            "DSSUBGRU": true,
            "TSUBGRU": true,
            "IDCICLO": true,
            "TIDCICLO": true,
            "IT": true,
            "TNRITAIRVISION": true,
            "TIPOCUSTO": true,
            "DATAREF": true,
            "NMINDICE": true,
            "INFLACAO": true,
            "TDTVGSUMA": true
        },
        "COC03610": {
            "QTPAXREF": false,
            "DSSERVICOCIAFILI": true,
            "TNRSERVICOCLASSE": true,
            "DSSUBGRU": true,
            "TSUBGRU": true,
            "IDCICLO": true,
            "TIDCICLO": true,
            "IT": true,
            "TNRITAIRVISION": true,
            "TIPOCUSTO": true,
            "DATAREF": true,
            "NMINDICE": true,
            "INFLACAO": true,
            "TDTVGSUMA": true
        },
        "COC03613": {
            "QTPAXREF": false,
            "DSSERVICOCIAFILI": true,
            "TNRSERVICOCLASSE": true,
            "DSSUBGRU": true,
            "TSUBGRU": true,
            "IDCICLO": true,
            "TIDCICLO": true,
            "IT": true,
            "TNRITAIRVISION": true,
            "TIPOCUSTO": true,
            "DATAREF": true,
            "NMINDICE": true,
            "INFLACAO": true,
            "TDTVGSUMA": true
        }
    };

    this.onChangeReport = function(widget) {
        var fields = widget.fields;
        var nomeRelatorio = widget.currentRow.REPORTNAME;
        fields.forEach(function(field) {
            if (reportFields[nomeRelatorio].hasOwnProperty(field.name)) {
                field.isVisible = reportFields[nomeRelatorio][field.name];
            }
        });
        templateManager.updateTemplate();
    };

    this.filterSelectBeforeOpen = function(field) {
        field.widget.fields.forEach(function(selectField) {
            if (selectField.template && selectField.template.indexOf('select-multiple') !== -1) {
                field.widget.currentRow[selectField.valueField] = field.widget.currentRow[selectField.name] ? field.widget.currentRow[selectField.name].join('_') : "T";
            }
        });
        InflightorGeneral.onClickDependentField(field, field.widget.currentRow);
    };

    this.filterSelectOnChange = function(field) {
        field.widget.fields.forEach(function(selectField) {
            if (selectField.dataSourceFilter && selectField.dataSourceFilter.length) {
                selectField.dataSourceFilter.forEach(function(filter) {
                    if (filter.name === field.name || filter.name === field.valueField) {
                        selectField.applyDefaultValue();
                        if (selectField.template && selectField.template.indexOf('select-multiple')) {
                            if (selectField.toggleSelectDependecys) {
                                let fieldCheckBox = field.widget.getField(selectField.toggleSelectDependecys);
                                fieldCheckBox.applyDefaultValue();
                                fieldCheckBox.checked = true;
                                InflightorGeneral.toggleReadOnlySelect(fieldCheckBox, selectField);
                            }
                        }
                    }
                });
            }
        });
    };

    this.filtraCiclo = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "T";

        if (currentRow.TSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = listaSubgrupos(field.widget.getField("DSSUBGRU"));
            } else {
                cdsubgru = "";
            }
        }

        var filtro = {
            "CDSUBGRU": cdsubgru,
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
        field.reload();
        this.limpaFilhos(field);
    };

    var listaSubgrupos = function(field) {

        var lista = "";
        var rows = field.selectWidget.getCheckedRows();

        for (let i = 0; i < rows.length; i++) {
            if (lista == "") {
                lista = rows[i].CDSUBGRU + "|" + rows[i].NRSEQSUBGRUSEMAM;
            } else {
                lista = lista + "_" + rows[i].CDSUBGRU + "|" + rows[i].NRSEQSUBGRUSEMAM;
            }
        }
        return lista;
    };

    this.filtraIT = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "";
        var cdciclo = "";
        var dsservico = "";
        var cdclasse = "";

        var filtro = {
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        if (currentRow.TSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = "";
                field.widget.getField("DSSUBGRU").selectWidget.getCheckedRows().forEach(function(subgru) {
                    cdsubgru += subgru.CDSUBGRU + "|" + subgru.NRSEQSUBGRUSEMAM + "_";
                });
            } else {
                cdsubgru = null;
            }
        }

        if (currentRow.TIDCICLO != "T") {
            if (currentRow.IDCICLO) {
                cdciclo = "";
                field.widget.getField("IDCICLO").selectWidget.getCheckedRows().forEach(function(ciclo) {
                    cdciclo += ciclo.CDCICLO + "_";
                });
            } else {
                cdciclo = null;
            }
        }

        if (currentRow.TIDSERVICOCLASSE != "T") {
            if (currentRow.IDSERVICOCLASSE) {
                dsservico = "";
                cdclasse = "";
                field.widget.getField("NRSERVICOCLASSE").selectWidget.getCheckedRows().forEach(function(servico) {
                    dsservico += servico.NRSERVICOCIAFILI + "_";
                    cdclasse += servico.CDCLASSE + "_";
                });
            } else {
                dsservico = null;
                cdclasse = null;
            }
        }

        filtro.CDCICLO = cdciclo || "T";
        filtro.CDSUBGRU = cdsubgru || "T";
        filtro.NRSERVICOCIAFILI = dsservico || "T";
        filtro.CDCLASSE = cdclasse || "T";

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.validaAprovacaoManual = function(widget) {
        //Melhorar essa validação
        var now = widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW;
        var filter = [{
                name: "CDFILIAL",
                operator: "=",
                value: now.CDFILIAL
        },
            {
                name: "CDCIA",
                operator: "=",
                value: now.CDCIA
        },
            {
                name: "CDMANCIA",
                operator: "=",
                value: now.CDMANCIA
        },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
        }];
        var promises = [];
        var filter_pr = [{
            name: "VRCUSPRSUBSEMAM",
            operator: "=",
            value: 0
        }].concat(filter);
        var filter_rc = [{
            name: "VRCUSRESUBSEMAM",
            operator: "=",
            value: 0
        }].concat(filter);
        var filter_rc_pr = [{
            name: "IDCUSTOPRODZERO",
            operator: "=",
            value: "S"
        }].concat(filter);
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/prodsubsemam", "FilterData", filter_pr));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/recsubsemam", "FilterData", filter_rc));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/recsubsemam", "FilterData", filter_rc_pr));
        ZHPromise.all(promises).then(function(callback) {
            if (callback[0].dataset.prodsubsemam.length > 0 || callback[1].dataset.recsubsemam.length > 0 || callback[2].dataset.recsubsemam.length > 0) {
                var msg = "Existem produto(s) e/ou receita(s) com custo zero no menu grid! Deseja aprovar o custo do menu mesmo assim?";
                ScreenService.confirmMessage(ScreenService.i18n(msg), "question",
                    function() {
                        self.validaCusteio(widget);
                    },
                    function() {
                        widget.activate();
                    });
            } else {
                self.validaCusteio(widget);
            }
        }.bind(widget), function(err) {
            ScreenService.showMessage(err.data.error);
        });
    };
    //Abre a popup para aprovar ovo id da sitacao do manual
    this.actionManual = function(widget, actionID) {
        var msg = "Deseja aprovar o custeio do menu grid?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), "question",
            function() {
                widget.container.getWidget("widget_autenticar").currentRow.IDSITUAPROPCIA = actionID;
                ScreenService.openPopup(widget.container.getWidget("widget_autenticar"));
            },
            function() {
                MessageService.showMessageByCode("COC03610_CANCEL_OPERATION");
                widget.activate();
            });
    };
    //Chama o service que valida usuario e senha
    this.autenticarUsuario = function(widget) {
        if (widget.isValid()) {
            var user = widget.currentRow.user;
            var senha = widget.currentRow.senha;
            coc03610_custeio_menu_gridService.autenticar(user, senha)
                .then(function() {
                    self.aprovaCustoSemWorkflow(widget);
                    ScreenService.closePopup();
                }.bind(widget), function(callback) {
                    InflightorGeneral.callbackError(callback);
                });
        }
    };
    //Muda a situacao do manual OBS ::: MUDA APENAS A SITUACAO DO CUSTEIO
    this.aprovaCustoSemWorkflow = function(widget) {
        var row = {};
        var values = widget.parent.dataSourceFilter;
        values.forEach(function(obj) {
            row[obj.name] = obj.value;
        });
        row.IDSITUACAO = "A"; //id do historico de custo sem workflow
        InflightorGeneral.callRestEngine("coc#/crud/subgrusemam", "FilterData", [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            }, {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            }, {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }])
            .then(function(callback) {
                var newDataset = callback.dataset.subgrusemam;
                newDataset.forEach(function(subgrupo) {
                    subgrupo.DTHRAPROVACUSTO = moment().format("DD/MM/YYYY HH:mm:ss");
                    subgrupo.CDOPERAPROVACUSTO = getLocalVar("INFLIGHTOR_CDOPERADOR");
                    subgrupo.IDSITCUSTEIO = "A";
                    subgrupo.__is_new = false;
                });
                InflightorGeneral.callRestEngine("coc#/crud/subgrusemam/save", "DataSet", newDataset)
                    .then(function() {
                        coc03610_custeio_menu_gridService.aprovacusto(row)
                            .then(function() {
                                MessageService.showMessageByCode("COC03610_APPROVED_COST");
                                templateManager.container.getWidget("grid_subgrupos").reload();
                            }, function(err) {
                                ScreenService.showMessage(err.data.error);
                                widget.activate();
                            });
                    }.bind(row), function(err) {
                        ScreenService.showMessage(err.data.error);
                    });
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
    };
    //Carrega os dados de produto e receitas para montar o grid de composição
    this.carregaDataSourceComposicao = function(args) {

        //pega a current row de subgrupos para preencher os filtros do request
        var row = args.row;
        var tipocusto = {
            0: "Médio Bruto",
            1: "Médio Líquido",
            2: "Últ. Compra Bruto",
            3: "Últ. Compra Líquido"
        };
        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": row.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": row.CDCLASSE
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": row.CDCICLO
            },
            {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": row.NRSEQSUBGRUSEMAM
            },
            {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": row.CDSUBGRU
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("NRORG")
            }
        ];
        var grid_composicao = args.owner.container.getWidget("grid_composicao");
        var footer = grid_composicao.container.getWidget("footer_composicao_menu_grid");
        grid_composicao.dataSourceFilter = dataSourceFilter;
        grid_composicao.reload().then(function() {
            grid_composicao.dataSource.data.forEach(function(obj) {
                if (!obj.LEANCODE)
                    obj.LEANCODE = " - ";
            });
            //ScreenService.openSwipe(grid_composicao);
            ScreenService.openSwipe(args.owner.container.getWidget("tabbed_subgrupos"));
            //args.owner.container.getActiveWidget.parent.widgets[1].activate();
            let value = General.valueToFloat(args.row.VRCUSTOCICSEMAM);
            footer.currentRow.VRCUSTOCICSEMAM = String(value.toFixed(2)).replace(".", ",");
            footer.currentRow.DTCUSTO = args.row.DTCUSTO === null ? "-" : args.row.DTCUSTO.slice(0, -9);
            footer.currentRow.IDTIPOCUSTO = args.row.IDTIPOCUSTO === null ? "-" : tipocusto[args.row.IDTIPOCUSTO];
            footer.reload();
        });
    };

    this.setActiveWidget = function(args) {
        args.owner.container.getWidget("grid_composicao").activate();
    };

    //Abre Popup de Calculo de custo
    this.openCalculaCustoPopup = function(widget, idcalculo) { //se o calculo e do menu ou do subgrupo
        ScreenService.openPopup(widget.container.getWidget("calculodecusto"));
        widget.container.getWidget("calculodecusto").newRow();
        widget.container.getWidget("calculodecusto").currentRow.TOACTIVATE = widget.name;
        widget.container.getWidget("calculodecusto").currentRow.IDCALCULO = idcalculo;
    };
    //Chamada da procedure
    this.calculaCusto = function(widget) {
        if (widget.isValid() && widget.getField("DTCUSTOAP").value() != "" && widget.getField("TIPOCUSTO").value() != "") {
            var row = {};
            var currentRow = widget.currentRow;
            var datacusto = currentRow.DTCUSTOAP;
            var tipocusto = currentRow.TIPOCUSTO;
            var subgrupoRow = widget.container.getWidget("grid_subgrupos").currentRow;
            var values = widget.container.getWidget("custeio_grid_manual").dataSourceFilter;
            values.forEach(function(value) {
                row[value.name] = value.value;
            });
            row.IDCALCULO = currentRow.IDCALCULO;
            row.NRSERVICOCIAFILI = subgrupoRow.NRSERVICOCIAFILI;
            row.CDCLASSE = subgrupoRow.CDCLASSE;
            row.CDSUBGRU = subgrupoRow.CDSUBGRU;
            row.NRSEQSUBGRUSEMAM = subgrupoRow.NRSEQSUBGRUSEMAM;
            row.CDCICLO = subgrupoRow.CDCICLO;
            row.DATACUSTO = datacusto;
            row.TIPOCUSTO = tipocusto;
            row.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
            row.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
            coc03610_custeio_menu_gridService.runProcedureCustoManual(row)
                .then(function() {
                    var popupMsg = widget.currentRow.TOACTIVATE === "grid_composicao" ? "Cálculo de custo do subgrupo concluído!" : "Cálculo de custo do menu grid concluído!";
                    ScreenService.showMessage(popupMsg);
                    self.setFooterValues(widget.parent);
                    ScreenService.closePopup();
                    if (widget.currentRow.TOACTIVATE === "custeio_grid_manual") {
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).reload();
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).activate();
                        widget.reload();
                    } else if (widget.currentRow.TOACTIVATE === "grid_subgrupos") {
                        widget.container.getWidget("custeio_grid_manual").reload();
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).reload();
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).activate();
                        widget.reload();
                    } else {
                        widget.container.getWidget("custeio_grid_manual").reload();
                        widget.container.getWidget("grid_subgrupos").reload();
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).reload();
                        widget.container.getWidget(widget.currentRow.TOACTIVATE).activate();
                        widget.reload();
                    }
                    widget.container.getWidget(widget.currentRow.TOACTIVATE).reload();
                    widget.container.getWidget(widget.currentRow.TOACTIVATE).activate();
                    widget.reload();
                }.bind(widget), function(err) {
                    ScreenService.showMessage(err.data.error);
                });
        } else {
            ScreenService.showMessage("Preencha os campos corretamente!");
        }
    };
    //seta os valores do footer
    this.setFooterValues = function(owner) {

        var this_row = owner.appliedFilter.FILTER_ROW;
        var cdfilial = this_row.CDFILIAL;
        var cdcia = this_row.CDCIA;
        var cdmancia = this_row.CDMANCIA;
        var idsituapropcia = this_row.IDSITUAPROPCIA;

        coc03610_custeio_menu_gridService.getFooterValues(cdfilial, cdcia, cdmancia).then(function(callback) {
            if (callback && callback.dataset && callback.dataset.manualciam && callback.dataset.manualciam.length) {
                owner.container.getWidget("footer_custo_menu_grid").currentRow = callback.dataset.manualciam[0];
                var idtipocustodomanual = callback.dataset.manualciam[0].IDTIPOCUSTOMAN;
                var dtcustoman = callback.dataset.manualciam[0].DTCUSTOMAN;
                var footer = {
                    idtipocustodomanual: idtipocustodomanual,
                    dtcustoman: dtcustoman,
                    idsituapropcia: idsituapropcia
                };
                // if(owner.appliedFilter.FILTER_ROW.SGCLASSE.length) {
                self.totalizaRodape(owner, footer);
            }
            // } else {
            // owner.container.getWidget('footer_custo_menu_grid').currentRow.IDTIPOCUSTOMAN = tipocusto[idtipocustodomanual];
            // owner.container.getWidget('footer_custo_menu_grid').currentRow.IDSITUAPROPCIA = sitmenugrid[idsituapropcia];
            //     var fieldCusto = owner.container.getWidget('footer_custo_menu_grid').getField('VRCUSTO');
            //     // vrcustomam = Util.applyMask(vrcustomam, fieldCusto.mask, maskEngine, ZhUtil);
            //     var soma = 0;
            //     var fieldCusto = owner.container.getWidget('footer_custo_menu_grid').getField('VRCUSTO');
            //     owner.dataSource.data.forEach(function(servico){
            //         soma += servico.VRCUSTOSERVM;
            //     })
            //     //
            //     soma = Util.applyMask(soma, fieldCusto.mask, maskEngine, ZhUtil);
            //     owner.container.getWidget('footer_custo_menu_grid').currentRow.DTCUSTOMAN = moment(dtcustoman,"DD/MM/YYYY hh:mm:ss").format('DD/MM/YYYY');
            //     owner.container.getWidget('footer_custo_menu_grid').currentRow.VRCUSTO = vrcustomam;
            //     owner.container.getWidget('footer_custo_menu_grid').reload();
            // }
        }.bind(owner));
    };
    //apenas associa a action pra exibir icones da composicao a algum metodo
    this.doNothing = function() {};
    //  Verifica se gera workflow ou não
    this.validaCusteio = function(widget) {
        var promises = [];
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/processoaprova", "FilterData", [{
            "name": "NRPROCESSOAPROVA",
            "operator": "=",
            "value": 4
        }, {
            "name": "IDATIVO",
            "operator": "=",
            "value": "S"
        }]));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/itemprocaprova", "FilterData", [{
            "name": "NRPROCESSOAPROVA",
            "operator": "=",
            "value": 4
        }, {
            "name": "NRITEMPROCAPROVA",
            "operator": "=",
            "value": 1
        }, {
            "name": "IDATIVO",
            "operator": "=",
            "value": "S"
        }]));
        ZHPromise.all(promises).then(
            function(callback) {
                if (callback[0].dataset.processoaprova.length === 0) {
                    //processo não gera workflow
                    self.actionManual(widget, "A");
                } else {
                    if (callback[1].dataset.itemprocaprova.length === 0) {
                        //item não gera workflow
                        self.actionManual(widget, "A");
                    } else {
                        //gera workflow
                        var msg = "Atenção! <br> A operação gera workflow. Deseja continuar?";
                        ScreenService.confirmMessage(ScreenService.i18n(msg), "question",
                            function() {
                                self.openWorkflowPopup(widget);
                            },
                            function() {
                                widget.activate();
                            });
                    }
                }
            }.bind(widget),
            function(err) {
                ScreenService.showMessage(err.data.error);
            });
    };
    //  Verifica se o custo já foi calculado
    this.validaCustoNaoCalculado = function(datasets, widget) {
        var receitasCustoZero = datasets[0].dataset.recsubsemam;
        var produtosCustoZero = datasets[1].dataset.prodsubsemam;
        var manual = datasets[2].dataset.manualciam[0];
        if (manual.DTHRCALCCUSTO === null && manual.CDOPERCALCCUSTO === null) {
            ScreenService.showMessage("Operação não permitida! <br> O custo ainda não foi calculado!");
        } else if (receitasCustoZero.length !== 0) {
            ScreenService.showMessage("Operação não permitida! <br> Existem receitas sem custo calculado!");
        } else if (produtosCustoZero.length !== 0) {
            ScreenService.showMessage("Operação não permitida! <br> Existem produtos sem custo calculado!");
        } else {
            self.validaWorkflowPendente(widget);
        }
    };

    this.validaPendencia = function(widget) {
        var promises = [];
        var row = widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW;
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/recsubsemam", "FilterData", [{
            "name": "VRCUSRESUBSEMAM",
            "operator": "IS_NULL"
        }].concat(filter)));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/prodsubsemam", "FilterData", [{
            "name": "VRCUSPRSUBSEMAM",
            "operator": "IS_NULL"
        }].concat(filter)));
        promises.push(InflightorGeneral.callRestEngine("coc#/crud/manualciam", "FilterData", filter));
        ZHPromise.all(promises).then(function(callback) {
            this.validaCustoNaoCalculado(callback, widget);
        }.bind(this), function(err) {
            ScreenService.showMessage(err.data.error);
        });
    };
    //  Abre pop up da descrição do motivo do workflow
    this.openWorkflowPopup = function(widget) {
        var motivoWidget = {
            "name": "motivoWidget",
            "label": "Justificativa",
            "template": "widget/form.html",
            "container": {},
            "isVisible": true,
            "popupNoBlock": false,
            "events": [],
            "fields": [
                {
                    "name": "DSMOTIVOWFLOW",
                    "label": "Motivo",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                },
                {
                    "name": "IDSITOPERWFLOW",
                    "label": "",
                    "template": "",
                    "isVisible": false,
                    "readOnly": false,
                    "showOnForm": true
                }
            ],
            "actions": [
                {
                    "name": "cancelar",
                    "label": "Cancelar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": "Confirmar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "coc03610_custeio_menu_gridController.generateWorkflow(args.owner.widget);"
                        }
                    ]
                }
            ]
        };
        motivoWidget.container = widget.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(motivoWidget);
        ScreenService.openPopup(motivoWidgetInitialized);
    };
    //  Gera workflow
    this.generateWorkflow = function(widget) {
        if (widget.isValid() && widget.getField("DSMOTIVOWFLOW").value() != "") {
            var sourceRow = Object.assign({}, widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW);
            var rowWorkflow = sourceRow;
            var footerRow = widget.container.getWidget("footer_custo_menu_grid").currentRow;
            rowWorkflow.DSMOTIVOWFLOW = widget.currentRow.DSMOTIVOWFLOW;
            rowWorkflow.DTCUSTOMAN = footerRow.DTCUSTOMAN || widget.currentRow.DTCUSTOMAN;
            rowWorkflow.VRCUSTOMAM = footerRow.VRCUSTOMAM || widget.currentRow.VRCUSTOMAM;
            rowWorkflow.IDTIPOCUSTOMAN = footerRow.IDTIPOCUSTOMANVAL || widget.currentRow.IDTIPOCUSTOMAN;
            rowWorkflow.OPERADORES = [];
            InflightorGeneral.callRestEngine("coc#/crud/operitempaprova", "FilterData", [{
                    "name": "NRPROCESSOAPROVA",
                    "operator": "=",
                    "value": 4
                }, {
                    "name": "NRITEMPROCAPROVA",
                    "operator": "=",
                    "value": 1
                }, {
                    "name": "IDATIVO",
                    "operator": "=",
                    "value": "S"
                }])
                .then(function(callback) {
                    callback.dataset.operitempaprova.forEach(function(operador) {
                        rowWorkflow.OPERADORES.push({
                            "CDOPERADOR": operador.CDOPERADOR,
                            "NRDIASOPERAPRO": operador.NRDIASOPERAPRO
                        });
                    });
                    rowWorkflow.NMITEMWFLOW = "Unidade: " + getLocalVar("NMFILIAL") + "\nCompanhia: " + sourceRow.NMCIA + "\nMenu Grid: " + sourceRow.DSMANUAL;
                    rowWorkflow.VRCUSTOMAM = InflightorGeneral.valueToFloat(rowWorkflow.VRCUSTOMAM);
                    InflightorGeneral.callRestEngine("coc#/generate_workflow", "Row", rowWorkflow)
                        .then(function() {
                            ScreenService.showMessage("Workflow gerado com sucesso!");
                            ScreenService.closePopup();
                        }, function(err) {
                            ScreenService.showMessage(err.data.error);
                        });
                }, function(err) {
                    ScreenService.showMessage(err.data.error);
                });
        } else {
            ScreenService.showMessage("Preencha o motivo.");
        }
    };
    //  O cálculo do custo do menu grid aprovado só pode ser realizado se houverem subgrupos que ainda não foram custeados
    this.validaSubgrupoManualAprovado = function(widget) {
        if (widget.isValid() && widget.getField("DTCUSTOAP").value() != "") {
            var now = Object.assign({}, widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW);
            InflightorGeneral.callRestEngine("coc#/crud/subgrusemam", "FilterData", [{
                    name: "CDFILIAL",
                    operator: "=",
                    value: now.CDFILIAL
                },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: now.CDCIA
                },
                {
                    name: "CDMANCIA",
                    operator: "=",
                    value: now.CDMANCIA
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                },
                {
                    name: "IDSITCUSTEIO",
                    operator: "<>",
                    value: "A"
                }]).then(function() {
                self.validateWorkflowBeforeCusteio(widget);
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
        } else {
            ScreenService.showMessage("Preencha os campos corretamente!");
        }
    };
    // Valida se o existe processo de workflow para o manual antes calcular o custo
    this.validateWorkflowBeforeCusteio = function(widget) {
        var rowFilter = widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW;
        var filterDadosWf = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: rowFilter.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: rowFilter.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: rowFilter.CDMANCIA
            }];
        InflightorGeneral.callRestEngine("coc#/crud/pendencias_workflow", "FilterData", filterDadosWf)
            .then(function(callback) {
                if (callback.dataset.pendencias_workflow.length > 0) {
                    ScreenService.showMessage("Operação não permitida! <br> Existe processo de workflow para este Menu Grid!");
                    ScreenService.closePopup();
                } else {
                    self.calculaCusto(widget);
                }
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
    };
    // Abre a popup dos relatórios
    this.openReportsPopup = function(widget) {
        var reportsWidget = widget.container.getWidget("reports_popup");
        var widgetPrincipal = widget.container.getWidget("custeio_grid_manual");
        defineFiltroPadraoParaFields(widgetPrincipal, reportsWidget);
        restauraEstadoFields(reportsWidget);
        ScreenService.openPopup(reportsWidget);
        reportsWidget.newRow();
        this.onChangeReport(reportsWidget);
    };

    var defineFiltroPadraoParaFields = function(widget, popupWidget) {
        var fields = popupWidget.fields;
        var widgetFilter = widget.appliedFilter.FILTER_ROW;

        popupWidget.fields.forEach(function(f) {
            if (f.dataSourceFilter && f.dataSourceFilter.length > 0) {
                f.dataSourceFilter.forEach(function(condicao) {
                    if (widgetFilter.hasOwnProperty(condicao.name)) {
                        condicao.value = widgetFilter[condicao.name];
                    }
                });
            }
        });

    };

    var setaValoresPadraoRelatorio = function(widget, popupWidget) {
        popupWidget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        popupWidget.currentRow.CDCIA = widget.appliedFilter.FILTER_ROW.CDCIA;
        popupWidget.currentRow.CDMANCIA = widget.appliedFilter.FILTER_ROW.CDMANCIA;
    };
    // Validate report popup
    this.validateReportPopup = function(widget) {
        if (widget.isValid() && widget.getField("QTPAXREF") != 0) {
            ScreenService.openPopup(widget.container.getWidget("reports_popup_type"));
        } else {
            ScreenService.showMessage("Preencha os campos corretamente");
        }
    };
    // Generate report
    this.generateReport = function(widgetForm, widgetList) {
        var reportRow = Object.assign({}, widgetForm.currentRow, widgetForm.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW);
        var reportName = widgetForm.currentRow.REPORTNAME;
        reportName = (reportName == "_CUSTO") ? getReportName(widgetForm.container) : reportName;
        reportRow.DTCUSTOMAN = moment(templateManager.container.getWidget("footer_custo_menu_grid").currentRow.DTCUSTOMAN, "DD/MM/YYYY").format("DD/MM/YYYY");
        reportRow.REPORTNAME = reportName;
        reportRow.DTVIGENCIA = moment(reportRow.DTVIGENCIA, "YYYY-MM-DD").format("DD/MM/YYYY");
        reportRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
        reportRow.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");
        reportRow.REPORTFORMAT = widgetList.currentRow.REPORTFORMAT;
        reportRow.QTPAXREF = InflightorGeneral.valueToFloat(reportRow.QTPAXREF);
        reportRow.DATA = "30/10/2016";
        reportRow.NRSERVICOCLASSE = reportRow.TNRSERVICOCLASSE === "T" ? "T" : reportRow.DSSERVICOCIAFILI.join("_") + "_";
        reportRow.DSSUBGRU = reportRow.TSUBGRU === "T" ? "T" : reportRow.DSSUBGRU.join("_") + "_";
        reportRow.CDCICLO = reportRow.TIDCICLO === "T" ? "T" : reportRow.IDCICLO.join("_") + "_";
        reportRow.NRITAIRVISION = reportRow.TNRITAIRVISION === "T" ? "T" : concatenarIt(reportRow.IT);
        reportRow.INFLACAO = InflightorGeneral.valueToFloat(reportRow.INFLACAO);

        reportRow.SERVCLASSE = (reportName == "COC03613") ? this.getServClass() : "";

        InflightorGeneral.openReport("coc#/relatorio_custo_zero_manualciam", reportRow.REPORTNAME, reportRow).then(function() {
            ScreenService.closePopup();
        }, InflightorGeneral.callbackError);
    };

    var concatenarIt = function(valueObj) {
        var nritairvision = "";
        for (let i = 0; i < valueObj.length; i++) {
            nritairvision += valueObj[i].NRITAIRVISION + '-' + valueObj[i].CDSUBGRU + '-' + valueObj[i].NRSEQSUBGRUSEMAM + '-' + valueObj[i].CDCICLO + "_";
        }
        return nritairvision;
    };

    this.getServClass = function() {

        var widget = templateManager.container.getWidget("reports_popup");
        var currentRow = widget.currentRow;

        if (currentRow.TIDSERVICOCLASSE === "S") {
            return "T";
            /*currentRow.SERVICOCLASSE = "T";
currentRow.SERVCLASSE = "T";
currentRow.CDCLASSE = "T";*/
        } else if (currentRow.IDSERVICOCLASSE && Array.isArray(currentRow.IDSERVICOCLASSE)) {
            //var servclasseArray = [];
            //var cdclasseArray = [];
            //var servclasseCheckedRows = widget.getField("IDSERVICOCLASSE").dataSource.getCheckedRows();
            //servclasseCheckedRows.forEach(function(row) {
            //servclasseArray.push(row.NRSERVICOCIAFILI + "-" + row.CDCLASSE);
            //cdclasseArray.push(row.CDCLASSE);
            //});
            var SERVICOCLASSE = "_" + currentRow.IDSERVICOCLASSE.join("_") + "_";
            /*var SERVCLASSE = servclasseArray.join("_");
var CDCLASSE = "_" + cdclasseArray.join("_") + "_";*/
            return SERVICOCLASSE;
        } else {
            return "";
        }
    };

    this.clearIndice = function(field) {
        if (field.widget.currentRow.NMINDICE && field.widget.currentRow.DTCUSTOAP) {
            this.getVrIndice(field.widget);
        }
    };
    //Pega o valor da moeda e controla o readonly do valor
    this.getVrIndice = function(widget) {
        var cdindice = widget.currentRow.CDINDICE;
        var dtrefind = widget.currentRow.DTCUSTOAP;
        if (cdindice && dtrefind) {
            coc23100_geracaocustomanualService.checkCdIndice(cdindice).then(function(callback) {
                widget.getField("VRCUSTO").readOnly = callback.dataset.paramcomis.length > 0 ? true : false;
            });
            var vrindice = 0;
            coc23100_geracaocustomanualService.findVrIndice(cdindice, dtrefind).then(function(callback) {
                if (callback.dataset.movindice[0]) {
                    vrindice = callback.dataset.movindice[0].VRINDICE;
                    widget.getField("VRCUSTO").setValue(vrindice);
                } else {
                    widget.getField("VRCUSTO").setValue(0);
                }
            });
            widget.getField("VRCUSTO").reload();
        } else {
            widget.currentRow.DTCUSTOAP = 0;
        }
    };

    var getReportName = function(container) {
        var idutiairvision = container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW.IDUTIAIRVISION;
        return (idutiairvision === "S") ? "COC03612" : "COC03612";
    };
    // Valida idativo receita e produto
    this.validaIdAtivo = function(widget) {
        var row = widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW;
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": row.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": row.CDMANCIA
            },
            {
                "name": "",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        InflightorGeneral.callRestEngine("coc#/crud/coc03610_prodsubsemam_recsubsemam_idativo", "FilterData", filter)
            .then(function(callback) {
                if (callback.dataset.coc03610_prodsubsemam_recsubsemam_idativo.length > 0) {
                    ScreenService.showMessage("Operação não permitida! <br> Existem produtos ou receitas inativas no subgrupo.");
                } else {
                    self.validaAprovacaoManual(widget);
                }
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
    };
    //Rodape totalizador dinamico pelo filtro
    this.totalizaRodape = function(widget, footer) {
        var tipocusto = {
            0: "Médio Bruto",
            1: "Médio Líquido",
            2: "Últ. Compra Bruto",
            3: "Últ. Compra Líquido"
        };
        var sitmenugrid = {
            "P": "Em proposta",
            "A": "Aprovado",
            "E": "Em desenvolvimento",
            "R": "Reprovado"
        };
        var sum = 0;
        var fieldCusto = widget.container.getWidget("footer_custo_menu_grid").getField("VRCUSTO");
        widget.dataSource.data.forEach(function(servico) {
            sum += servico.VRCUSTOSERVM;
        });
        sum = Util.applyMask(sum, fieldCusto.mask, maskEngine, ZhUtil);
        widget.container.getWidget("footer_custo_menu_grid").currentRow.VRCUSTO = sum;
        widget.container.getWidget("footer_custo_menu_grid").currentRow.DTCUSTOMAN = moment(footer.dtcustoman, "DD/MM/YYYY hh:mm:ss").format("DD/MM/YYYY");
        widget.container.getWidget("footer_custo_menu_grid").currentRow.IDTIPOCUSTOMAN = tipocusto[footer.idtipocustodomanual];
        widget.container.getWidget("footer_custo_menu_grid").currentRow.IDSITUAPROPCIA = sitmenugrid[footer.idsituapropcia];
        templateManager.updateTemplate();
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc03610_custeio_menu_gridController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            ScreenService.showMessage("A companhia selecionada está inativa.");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.validaWorkflowPendente = function(widget) {
        var row = widget.container.getWidget("custeio_grid_manual").appliedFilter.FILTER_ROW;
        var filter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: row.CDMANCIA
            }
        ];
        InflightorGeneral.callRestEngine("coc#/crud/pendencias_workflow", "FilterData", filter).then(function(callback) {
            if (callback.dataset.pendencias_workflow.length > 0) {
                ScreenService.showMessage("Operação não permitida! <br> Não é possivel aprovar o custo com workflow pendente para o menu grid.");
            } else {
                self.validaIdAtivo(widget);
            }
        }, function() {
            ScreenService.showMessage("Ocorreu um erro durante a validação de workflow pendentes.");
        });
    };

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });

        clearFieldsInCurrentRow(widget, ['NMCIA', 'DSMANUAL', 'SGCLASSE']);
        updateFilterFields(fieldCia, ['CDFILIAL'], rowToFilter);
        blockDependentField(fieldUnit, 'NMCIA');
        this.onChangeCia(fieldCia);
    };

    this.onChangeCia = function(fieldCia) {
        const widget = fieldCia.widget;
        const fieldManual = widget.getField('DSMANUAL');
        const fieldClass = widget.getField('SGCLASSE');
        const rowToFilter = getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });
        clearFieldsInCurrentRow(widget, ['DSMANUAL', 'SGCLASSE']);
        updateFilterFields(fieldManual, ['CDFILIAL', 'CDCIA'], rowToFilter);
        updateFilterFields(fieldClass, ['CDFILIAL', 'CDCIA'], rowToFilter);
        blockDependentField(fieldCia, 'DSMANUAL');
        blockDependentField(fieldCia, 'SGCLASSE');
        fieldClass.reload();
    };

    function blockDependentField(field, nameFields) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        const widget = field.widget;
        let firstNode = nameFields[0];
        const fieldValue = field.value();
        let fieldFirstNode = widget.getField(firstNode);
        nameFields.forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                cField.readOnly = true;
            }
        });
        if (fieldFirstNode) {
            fieldFirstNode.readOnly = !fieldValue;
        }
    }

    function updateFilterFields(fieldUpdate, nameFilters, rowToSet) {
        fieldUpdate.dataSourceFilter.forEach(filter => {
            if (nameFilters.indexOf(filter.name) > -1) {
                filter.value = rowToSet[filter.name];
            }
        });
    }

    function getValuesInRow(widget, fieldAndPropValue) { // {FIELDNAME: NAME_GET_ROW}
        let result = {};
        Object.keys(fieldAndPropValue).forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                let valueFields = Array.isArray(fieldAndPropValue[nameField]) ? fieldAndPropValue[nameField] : [fieldAndPropValue[nameField]];
                let fieldValue = cField.value();
                valueFields.forEach(valueField => {
                    result[valueField] = fieldValue ? widget.currentRow[valueField] : '';
                });
            }
        });
        return result;
    }

    function clearFieldsInCurrentRow(widget, nameFields) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        nameFields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field) {
                let items = null;
                if (field.outData) items = Object.values(field.outData);
                if (field.valueFields && !items) items = field.valueFields;
                if (!items) items = [field.valueField, field.descriptionField];
                if (items) {
                    items.forEach(propToClear => {
                        widget.currentRow[propToClear] = '';
                    });
                }
            }
        });
    }

    this.changeField = function(field, arraySonFields, firstSonField) {
        if (field.name == 'NMCIA' && field.widget.currentRow.IDATIVO == 'N') {
            field.clearValue();
            ScreenService.showMessage("A companhia selecionada está inativa.");
            return;
        }
        for (var index in arraySonFields) {
            var fieldSon = field.widget.getField(arraySonFields[index]);
            fieldSon.clearValue();
            // fieldSon.reload();
            fieldSon.readOnly = true;
        }
        if (field.value()) {
            field.widget.getField(firstSonField).readOnly = false;
            field.widget.getField(firstSonField).reload();
        } else {
            field.widget.getField(firstSonField).readOnly = true;
        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register("coc03610_custeio_menu_gridController", coc03610_custeio_menu_gridController);
});

// FILE: ./assets/js/controllers/coc03688_substituicao_de_composicaoController.js
var coc03688_substituicao_de_composicaoController = function(ScreenService, InflightorGeneral, templateManager, MessageService) {

    var self = this;

    var PRODUCT = "P";
    var RECIPE = "R";

    this.trataErro = function(error) {
        var errorMessage = "COC03688_ERRO";
        let params = {};
        if (error.data && error.data.error)
            errorMessage = error.data.error;

        if (errorMessage.indexOf("ORA-02292") !== -1) {
            errorMessage = "DELETE_ERROR";
            MessageService.showMessageByCode(errorMessage);
            return;
        }
        if (errorMessage.indexOf("ORA-20501") !== -1) {
            var index = error.data.error.indexOf("ORA-20501: ") + 11;
            var newStr = error.data.error.slice(index);
            if (newStr.indexOf("ORA-") !== -1) {
                newStr = newStr.slice(0, newStr.indexOf("ORA-"));
            }
            errorMessage = newStr;

            ScreenService.showMessage(errorMessage);
            return;
        }
        if (errorMessage.indexOf("COC03688_") >= 0) {
            if (errorMessage.indexOf("COC03688_VALOR_CAMPO_N_ENCONTRADO") >= 0) {
                let msg_param = errorMessage.split("#_#");
                params.CAMPO = msg_param[1];
                MessageService.showMessageByCode("COC03688_VALOR_CAMPO_N_ENCONTRADO", params);
            } else
                MessageService.showMessageByCode(errorMessage);
        } else
            ScreenService.showMessage(errorMessage);
        errorMessage = ScreenService.i18n(errorMessage);

    };

    this.openFilterWidget = function(widgetMain) {

        widgetMain.openFilterWidget();
        widgetMain.activate();

    };

    this.openPopUp = function(widgetPopUp) {
        widgetPopUp.newRow();
        ScreenService.openPopup(widgetPopUp);
        widgetPopUp.activate();
    };

    this.closePopUp = function(widgetPopUp) {
        ScreenService.closePopup(widgetPopUp);
        widgetPopUp.getParent().activate();
    };

    this.popUpConfirm = function(widgetPopUp) {
        ScreenService.closePopup(widgetPopUp);
    };

    this.changeFieldProductAndRecipe = function(widgetFilterPopUp) {

        var changedValue = widgetFilterPopUp.currentRow.IDPRODUTORECEITA;

        var fieldRecipe = widgetFilterPopUp.getField("DSRECEITA");
        var fieldAllRecipe = widgetFilterPopUp.getField("TDSRECEITA");

        var fieldProduct = widgetFilterPopUp.getField("NMPRODUTO");
        var fieldAllProduct = widgetFilterPopUp.getField("TNMPRODUTO");

        if (changedValue == PRODUCT) {

            fieldRecipe.isVisible = false;
            fieldAllRecipe.isVisible = false;

            fieldProduct.isVisible = true;
            fieldAllProduct.isVisible = true;

        } else if (changedValue == RECIPE) {

            fieldRecipe.isVisible = true;
            fieldAllRecipe.isVisible = true;

            fieldProduct.isVisible = false;
            fieldAllProduct.isVisible = false;

        }

        var fieldGrammage = widgetFilterPopUp.getField("QTGRAMATURA");

        fieldGrammage.isVisible = true;

    };

    this.afterDeselectField = function(widgetFilterPopUp) {

        var fieldRecipe = widgetFilterPopUp.getField("DSRECEITA");
        var fieldAllRecipe = widgetFilterPopUp.getField("TDSRECEITA");
        var fieldProduct = widgetFilterPopUp.getField("NMPRODUTO");
        var fieldAllProduct = widgetFilterPopUp.getField("TNMPRODUTO");
        var fieldGrammage = widgetFilterPopUp.getField("QTGRAMATURA");

        fieldRecipe.isVisible = false;
        fieldAllRecipe.isVisible = false;
        fieldProduct.isVisible = false;
        fieldAllProduct.isVisible = false;
        fieldGrammage.isVisible = false;

    };

    this.changeMinDate = function(widget) {
        var dtEfetField = widget.getField("DTEFETIVACAO");
        if (widget.name === "zhFilterPreferencesSelectConditions") {
            dtEfetField.minDate = "";
            if (dtEfetField.calendarWidget) {
                delete dtEfetField.calendarWidget;
            }
        } else {
            dtEfetField.minDate = moment().add(1, "days").format("DD/MM/YYYY");
        }
    };

    this.saveSubscompmenu = function(subscompmenuWidget) {
        if (subscompmenuWidget.isValid()) {
            var rowToSave = angular.copy(subscompmenuWidget.currentRow);
            rowToSave.SUBGRUPO = (Array.isArray(rowToSave.CDFILIAL) && rowToSave.CDFILIAL.length) ? rowToSave.CDFILIAL.join('_') : "T";
            rowToSave.CIAFILI = (Array.isArray(rowToSave.CIAFILI) && rowToSave.CIAFILI.length) ? rowToSave.CIAFILI.join('_') : "T";
            rowToSave.MANCIA = (Array.isArray(rowToSave.MANCIA) && rowToSave.MANCIA.length) ? rowToSave.MANCIA.join('_') : "T";
            rowToSave.SERVCLASSE = (Array.isArray(rowToSave.SERVCLASSE) && rowToSave.SERVCLASSE.length) ? rowToSave.SERVCLASSE.join('_') : "T";
            rowToSave.SUBGRUPO = (Array.isArray(rowToSave.SUBGRUPO) && rowToSave.SUBGRUPO.length) ? rowToSave.SUBGRUPO.join('_') : "T";
            rowToSave.CICLO = (Array.isArray(rowToSave.CICLO) && rowToSave.CICLO.length) ? rowToSave.CICLO.join('_') : "T";

            var paramsNewCode = subscompmenuWidget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                rowToSave.__using_unit = paramsNewCode.__using_unit || false;
                rowToSave.__new_code_pk = paramsNewCode.__new_code_pk;
                rowToSave.__table_pk = paramsNewCode.__table_pk;
                rowToSave.__new_code_length = paramsNewCode.__new_code_length || 5;
                rowToSave.__cast_number = paramsNewCode.__cast_number || false;
            }
            rowToSave.__columns_to_return = subscompmenuWidget.__columns_to_return || [];

            InflightorGeneral.callRestEngine("coc#/saveSubscompmenu", "Row", rowToSave).then(function(callback) {
                delete subscompmenuWidget.backupRow;

                var widgets = subscompmenuWidget.widgets;
                InflightorGeneral.changeWidgetAcessibility(widgets.concat(subscompmenuWidget), true);
                subscompmenuWidget.view();

                var dataset = callback.dataset ? callback.dataset : {};
                var columnsToReturn = dataset.columnsToReturn ? dataset.columnsToReturn : [];
                var newCode = columnsToReturn[0] ? columnsToReturn[0].NRSUBSCOMPMENU : null;
                subscompmenuWidget.currentRow.NRORG = getLocalVar('NRORG');
                subscompmenuWidget.currentRow.NRSUBSCOMPMENU = newCode;
                subscompmenuWidget.reload(subscompmenuWidget.dataSourceFilter).then(function() {
                    if (newCode && subscompmenuWidget.dataSource && subscompmenuWidget.dataSource.data) {
                        self.aplicaVisibilidadeActions(subscompmenuWidget);
                    } else {
                        ScreenService.closeSwipe();
                        subscompmenuWidget.list();
                    }
                });
            }, function(error) {
                self.trataErro(error);
            });
        }
    };

    this.deleteSubscompmenu = function(subscompmenuWidget) {
        MessageService.confirmMessageByCode('COC03688_CONFIRM_DEL').then(function() {
            let row = {
                '__is_new': false,
                'NRSUBSCOMPMENU': subscompmenuWidget.currentRow.NRSUBSCOMPMENU,
                'NRORG': 1
            };
            InflightorGeneral.callRestEngine("coc#/deleteSubscompmenu", "Row", row).then(function() {
                ScreenService.closeSwipe().then(function() {
                    subscompmenuWidget.reload().then(function() {
                        subscompmenuWidget.list();
                        subscompmenuWidget.activate();
                    });
                });
            }).catch(function(error) {
                self.trataErro(error);
            });
        });
    };

    this.finalizaSubscompmenu = function(subscompmenuWidget) {
        var confirmMessage = "COC03688_CONFIRM_FINAL";

        let row = angular.copy(subscompmenuWidget.currentRow);
        delete row.CDFILIAL;

        MessageService.confirmMessageByCode(confirmMessage).then(function() {
            InflightorGeneral.callRestEngine("coc#/finalizaSubscompmenu", "Row", row).then(function(callback) {
                if (callback.dataset && callback.dataset.ListaErros) {
                    InflightorGeneral.showErrorMessages(subscompmenuWidget, callback.dataset.ListaErros);
                } else {
                    var msg = "COC03688_SUCCESS_FINAL";

                    var messages = callback.messages;
                    if (messages && messages[0] && messages[0].message) {
                        msg = messages[0].message;
                    }

                    if (msg.indexOf('COC03688_') >= 0)
                        MessageService.showMessageByCode(msg);
                    else
                        ScreenService.showMessage(msg);
                    ScreenService.closeSwipe().then(function() {
                        subscompmenuWidget.reload().then(function() {
                            subscompmenuWidget.list();
                            subscompmenuWidget.activate();
                        });
                    });
                }
            }, function(error) {
                self.trataErro(error);
            });
        });
    };

    this.saveEditSubscompmenu = function(popupEditComposicao) {
        if (popupEditComposicao.isValid()) {
            self.calculateVrpesogramatura(popupEditComposicao);
            var currentRow = popupEditComposicao.currentRow;
            currentRow.NRSUBSCOMPMENU = popupEditComposicao.getParent().getParent().currentRow.NRSUBSCOMPMENU;
            var checkedRows = popupEditComposicao.getParent().getCheckedRows();
            var rowsToEdit = [];

            for (let i = 0; i < checkedRows.length; i++) {
                var rowToEdit = angular.copy(checkedRows[i]);
                if (treatToEdit(rowToEdit, currentRow)) {
                    rowsToEdit.push(rowToEdit);
                }
            }

            if (rowsToEdit.length) {
                if (validaQtdPorcao(rowsToEdit)) {
                    InflightorGeneral.callRestEngine("coc#/crud/editsubscompmenu/save", "DataSet", rowsToEdit).then(function() {
                        MessageService.showMessageByCode("COC03688_ORD_EDIT_GER");
                        ScreenService.closePopup();
                        popupEditComposicao.getParent().reload().then(function() {
                            popupEditComposicao.getParent().dataSource.clearCheckedRows();
                            popupEditComposicao.getParent().activate();
                        });
                    }, function(error) {
                        self.trataErro(error);
                    });
                }
            }
        }
    };

    function treatToEdit(rowToEdit, currentRow) {
        if (currentRow && currentRow.QTPRODREC) {
            rowToEdit.QTPRODREC = InflightorGeneral.valueToFloat(currentRow.QTPRODREC);
            rowToEdit.QTGRAMATURA = InflightorGeneral.valueToFloat(currentRow.QTGRAMATURA);
            rowToEdit.VRPESOGRAMATURA = InflightorGeneral.valueToFloat(currentRow.VRPESOGRAMATURA);

            rowToEdit.IDPRODRECNOVO = rowToEdit.IDPRODREC;
            if (rowToEdit.IDPRODREC == "P") {
                rowToEdit.CDPRODUTO = rowToEdit.CODIGO;
                rowToEdit.CDPRODUTONOVO = rowToEdit.CODIGO;
            } else {
                rowToEdit.CDRECEITA = rowToEdit.CODIGO;
                rowToEdit.CDRECEITANOVO = rowToEdit.CODIGO;
            }

            rowToEdit.IDTIPOACAO = "E";

            rowToEdit.__is_new = true;
            rowToEdit.__using_unit = false;
            rowToEdit.__new_code_pk = "EDITSUBSCOMPMENU";
            rowToEdit.__table_pk = "NREDITSUBSCM";
            rowToEdit.__new_code_length = 5;
            rowToEdit.__cast_number = true;

            return true;
        } else {
            return false;
        }
    }

    this.excluirComposicao = function(prodrecWidget) {
        var checkedRows = prodrecWidget.getCheckedRows();
        var rowsToDelete = [];
        for (let i = 0; i < checkedRows.length; i++) {
            var rowToDelete = angular.copy(checkedRows[i]);
            treatToDelete(rowToDelete);
            rowsToDelete.push(rowToDelete);
        }
        if (rowsToDelete.length) {
            InflightorGeneral.callRestEngine("coc#/crud/editsubscompmenu/save", "DataSet", rowsToDelete).then(function() {
                MessageService.showMessageByCode("COC03688_ORD_REM_GER");
                prodrecWidget.reload().then(function() {
                    prodrecWidget.dataSource.clearCheckedRows();
                });
            }, function(error) {
                self.trataErro(error);
            });
        }
    };

    function treatToDelete(rowToDelete) {
        rowToDelete.IDPRODRECNOVO = rowToDelete.IDPRODREC;
        if (rowToDelete.IDPRODREC == "P") {
            rowToDelete.CDPRODUTO = rowToDelete.CODIGO;
            rowToDelete.CDPRODUTONOVO = rowToDelete.CODIGO;
        } else {
            rowToDelete.CDRECEITA = rowToDelete.CODIGO;
            rowToDelete.CDRECEITANOVO = rowToDelete.CODIGO;
        }

        rowToDelete.IDTIPOACAO = "D";

        rowToDelete.__is_new = true;
        rowToDelete.__using_unit = false;
        rowToDelete.__new_code_pk = "EDITSUBSCOMPMENU";
        rowToDelete.__table_pk = "NREDITSUBSCM";
        rowToDelete.__new_code_length = 5;
        rowToDelete.__cast_number = true;

        return true;
    }

    this.applyInactiveStyleProduto = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDPRODATIVPR == 'N'",
                "cssClass": "coc-inactive"
            },
            {
                "condition": "row.IDPRODATIVPR == 'S' && row.PRODORCAMENTO == 'S'",
                "cssClass": "inf-orange"
            }
        ];
        field.selectWidget.events[0].code = "coc03688_substituicao_de_composicaoController.inactiveBranchWarningProduto(args);";
    };

    this.inactiveBranchWarningProduto = function(args) {
        if (args.owner.currentRow.IDPRODATIVPR === "N") {
            MessageService.showMessageByCode("INACTIVE_PRODUCT");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.filtraComposicao = function(args) {
        let filterRow = args.e.preference.FILTER_ROW;
        if (filterRow && filterRow.IDSITUACAO == "T") {
            delete filterRow.IDSITUACAO;
        }
    };

    this.openPopUpAddComposicao = function(popupAddComposicao) {
        popupAddComposicao.isVisible = true;
        self.openPopUp(popupAddComposicao);
    };

    this.closePopUpAddComposicao = function(popupAddComposicao) {
        popupAddComposicao.isVisible = false;
        self.closePopUp(popupAddComposicao);
    };

    this.adicionarComposicao = function(popupAddComposicao) {
        if (popupAddComposicao.isValid()) {
            self.calculateVrpesogramatura(popupAddComposicao);
            var currentRow = popupAddComposicao.currentRow;
            currentRow.NRSUBSCOMPMENU = popupAddComposicao.getParent().getParent().currentRow.NRSUBSCOMPMENU;

            var rowToAdd = angular.copy(currentRow);
            if (treatToAdd(rowToAdd, currentRow)) {
                InflightorGeneral.callRestEngine("coc#/crud/editsubscompmenu/save", "Row", rowToAdd).then(function() {
                    MessageService.showMessageByCode("COC03688_ORD_ADD_GER");
                    self.closePopUpAddComposicao(popupAddComposicao);
                    popupAddComposicao.getParent().reload().then(function() {
                        popupAddComposicao.getParent().dataSource.clearCheckedRows();
                        popupAddComposicao.getParent().activate();
                    });
                }, InflightorGeneral.treatAndShowError);
            }
        }
    };

    function treatToAdd(rowToAdd, currentRow) {
        if (currentRow && currentRow.QTPRODREC) {
            rowToAdd.QTPRODREC = InflightorGeneral.valueToFloat(currentRow.QTPRODREC);
            rowToAdd.QTGRAMATURA = InflightorGeneral.valueToFloat(currentRow.QTGRAMATURA);
            rowToAdd.VRPESOGRAMATURA = InflightorGeneral.valueToFloat(currentRow.VRPESOGRAMATURA);

            if (rowToAdd.IDPRODRECNOVO == "P") {
                rowToAdd.CDPRODUTONOVO = rowToAdd.CDPRODUTO;
            } else {
                rowToAdd.CDRECEITANOVO = rowToAdd.CDRECEITA;
            }
            rowToAdd.CDPRODUTO = null;
            rowToAdd.CDRECEITA = null;

            rowToAdd.IDTIPOACAO = "A";

            rowToAdd.__is_new = true;
            rowToAdd.__using_unit = false;
            rowToAdd.__new_code_pk = "EDITSUBSCOMPMENU";
            rowToAdd.__table_pk = "NREDITSUBSCM";
            rowToAdd.__new_code_length = 5;
            rowToAdd.__cast_number = true;

            return true;
        } else {
            return false;
        }
    }

    this.openPopUpSubsComp = function(prodrecWidget) {
        var popupSubsComposicao = prodrecWidget.container.getWidget("popupSubsComposicao");
        var checkedRows = prodrecWidget.getCheckedRows();

        if (validateSubsComp(prodrecWidget, checkedRows)) {
            popupSubsComposicao.getField("DSRECEITA").isVisible = false;
            popupSubsComposicao.getField("NMPRODUTO").isVisible = true;
            self.openPopUp(popupSubsComposicao);
        } else {
            MessageService.showMessageByCode("COC03688_IT_MESMO_GRUPO");
        }

    };

    function validateSubsComp(popupSubsComposicao, checkedRows) {
        var ciclos = [];

        for (let i = 0; i < checkedRows.length; i++) {
            var checkedRow = checkedRows[i];
            var ciclo = checkedRow.CDFILIAL + "-" + checkedRow.CDCIA + "-" +
                checkedRow.CDMANCIA + "-" + checkedRow.NRSERVICOCIAFILI + "-" +
                checkedRow.CDCLASSE + "-" + checkedRow.CDSUBGRU + "-" +
                checkedRow.NRSEQSUBGRUSEMAM + "-" + checkedRow.CDCICLO;
            if (ciclos.indexOf(ciclo) == -1) {
                ciclos.push(ciclo);
            } else {
                return false;
            }
        }

        return true;
    }

    this.substituirComposicao = function(popupSubsComposicao) {
        if (popupSubsComposicao.isValid()) {
            self.calculateVrpesogramatura(popupSubsComposicao);
            var currentRow = popupSubsComposicao.currentRow;
            currentRow.NRSUBSCOMPMENU = popupSubsComposicao.getParent().getParent().currentRow.NRSUBSCOMPMENU;
            var checkedRows = popupSubsComposicao.getParent().getCheckedRows();
            var rowsToSubs = [];

            for (let i = 0; i < checkedRows.length; i++) {
                var rowToSubs = angular.copy(checkedRows[i]);
                if (treatToSubs(rowToSubs, currentRow)) {
                    rowsToSubs.push(rowToSubs);
                }
            }

            if (rowsToSubs.length) {
                if (validaQtdPorcao(rowsToSubs)) {
                    InflightorGeneral.callRestEngine("coc#/crud/editsubscompmenu/save", "DataSet", rowsToSubs).then(function() {
                        MessageService.showMessageByCode("COC03688_ORD_SUBS_GER");
                        ScreenService.closePopup();
                        popupSubsComposicao.getParent().reload().then(function() {
                            popupSubsComposicao.getParent().dataSource.clearCheckedRows();
                            popupSubsComposicao.getParent().activate();
                        });
                    }, function(error) {
                        self.trataErro(error);
                    });
                }
            }
        }
    };

    function validaQtdPorcao(row) {
        if (row[0].QTPRODREC > 0 && row[0].QTGRAMATURA > 0) {
            return true;
        } else {
            MessageService.showMessageByCode("COC03688_VALORES_NEGATIVOS");
            return false;
        }
    }

    function treatToSubs(rowToSubs, currentRow) {
        if (currentRow && currentRow.QTPRODREC) {
            rowToSubs.QTPRODREC = InflightorGeneral.valueToFloat(currentRow.QTPRODREC);
            rowToSubs.QTGRAMATURA = InflightorGeneral.valueToFloat(currentRow.QTGRAMATURA);
            rowToSubs.VRPESOGRAMATURA = InflightorGeneral.valueToFloat(currentRow.VRPESOGRAMATURA);
            rowToSubs.IDPRODRECNOVO = currentRow.IDPRODRECNOVO;

            if (rowToSubs.IDPRODREC == "P") {
                rowToSubs.CDPRODUTO = rowToSubs.CODIGO;
            } else {
                rowToSubs.CDRECEITA = rowToSubs.CODIGO;
            }

            if (rowToSubs.IDPRODRECNOVO == "P") {
                rowToSubs.CDPRODUTONOVO = currentRow.CDPRODUTO;
            } else {
                rowToSubs.CDRECEITANOVO = currentRow.CDRECEITA;
            }

            rowToSubs.IDTIPOACAO = "S";

            rowToSubs.__is_new = true;
            rowToSubs.__using_unit = false;
            rowToSubs.__new_code_pk = "EDITSUBSCOMPMENU";
            rowToSubs.__table_pk = "NREDITSUBSCM";
            rowToSubs.__new_code_length = 5;
            rowToSubs.__cast_number = true;

            return true;
        } else {
            return false;
        }
    }

    this.filterAndReload = function(fieldToBeFiltered, filterFieldsName) {
        var widget = fieldToBeFiltered.widget;
        var filterFields = [];
        fieldToBeFiltered.dataSourceFilter = [];

        if (typeof(filterFieldsName) == "string") {
            filterFieldsName = [filterFieldsName];
        }

        filterFieldsName.forEach(function(filterFieldName) {
            filterFields.push(widget.getField(filterFieldName));
        });

        filterFields.forEach(function(filterField) {
            if (filterField.createFilter) {
                var currentFilter = fieldToBeFiltered.dataSourceFilter;
                var newFilter = filterField.createFilter();

                fieldToBeFiltered.dataSourceFilter = currentFilter.concat(newFilter);
            }
        });

        fieldToBeFiltered.dataSourceFilter.push({
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
        });

        fieldToBeFiltered.reload();
    };

    this.filterProdrecWidget = function(prodrecWidget) {
        var parentWidget = templateManager.container.getWidget('mainWidget');
        prodrecWidget.dataSourceFilter = [
            {
                "name": "NRSUBSCOMPMENU",
                "operator": "=",
                "value": parentWidget.currentRow.NRSUBSCOMPMENU
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        prodrecWidget.reload().then(function() {
            prodrecWidget.dataSource.clearCheckedRows();
            prodrecWidget.activate();
        });
    };

    this.changeProdutoReceita = function(idprodrecField) {
        var widget = idprodrecField.widget;

        var produtoField = widget.getField("NMPRODUTO");
        var receitaField = widget.getField("DSRECEITA");

        if (produtoField && receitaField) {
            var isProduto = (idprodrecField.value() == "P");
            var isReceita = (idprodrecField.value() == "R");

            produtoField.isVisible = isProduto;
            produtoField.validations.required = isProduto;

            receitaField.isVisible = isReceita;
            receitaField.validations.required = isReceita;

            produtoField.setValue("");
            receitaField.setValue("");
        }
    };

    this.calculateVrpesogramatura = function(widget) {
        var qtprodrec = InflightorGeneral.valueToFloat(widget.currentRow.QTPRODREC);
        var qtgramatura = parseInt(widget.currentRow.QTGRAMATURA);

        if (qtprodrec && qtprodrec) {
            var vrpesogramatura = qtgramatura ? (qtprodrec / qtgramatura) : null;
            widget.currentRow.VRPESOGRAMATURA = vrpesogramatura;
        }
    };

    this.hideAndShowOlditemDetails = function(prodrecWidgetPendente) {
        var parentWidget = templateManager.container.getWidget('mainWidget');
        var currentRow = prodrecWidgetPendente.currentRow;
        var parentRow = parentWidget.currentRow;

        var itemEdicao = (currentRow.IDTIPOACAO == "E" || currentRow.IDTIPOACAO == "S");
        var subsAprovada = (parentRow.IDSITUACAO == "A");

        var isVisible = itemEdicao && !subsAprovada;

        var fieldGroups = prodrecWidgetPendente.fieldGroups;

        for (let i in fieldGroups) {
            if (fieldGroups[i].name == "OLDITEM") {
                fieldGroups[i].isVisible = isVisible;
            }
        }
    };

    this.aplicaVisibilidadeActions = function(subscompmenuWidget) {
        var currentRow = subscompmenuWidget.currentRow;
        var estadoSubscomp = currentRow.IDSITUACAO;
        var actionsAtivas = (estadoSubscomp == "E");

        function aplicaVisibilidade(widget, isVisible) {
            if (widget.actions && Array.isArray(widget.actions)) {
                var actionsArray = widget.actions;
                for (let i = 0; i < actionsArray.length; i++) {
                    if (actionsArray[i].apenasDesenvolvimento) {
                        actionsArray[i].isVisible = isVisible;
                    }
                }
            }

            if (widget.widgets && Array.isArray(widget.widgets)) {
                var widgetsArray = widget.widgets;
                for (let i = 0; i < widgetsArray.length; i++) {
                    aplicaVisibilidade(widgetsArray[i], isVisible);
                }
            }
        }

        aplicaVisibilidade(subscompmenuWidget, actionsAtivas);
    };

    this.aplicaVisibilidadeInfoaprova = function(subscompmenuWidget) {
        var currentRow = subscompmenuWidget.currentRow;
        var estadoSubscomp = currentRow.IDSITUACAO;
        var visibility = (estadoSubscomp == "A");

        subscompmenuWidget.fieldGroups.forEach(function(fieldGroup) {
            if (fieldGroup.name == "INFOAPROVA") {
                fieldGroup.isVisible = visibility;
            }
        });
    };

    // this.FILTRO = function(subscompmenuWidget){
    //      {
    //          "name": "FilterBeforeApply",
    //          "code": "coc03688_substituicao_de_composicaoController.teste(args.owner);"
    //      }
    // }

    this.expandSwipe = function() {
        $(".zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level=\"1\"] ").addClass("aem-composicao");
    };

    this.retrieveSwipe = function() {
        $(".zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level=\"1\"] ").removeClass("aem-composicao");
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03688_substituicao_de_composicaoController", coc03688_substituicao_de_composicaoController);
});

// FILE: ./assets/js/controllers/coc03700_vinculo_subgrupoController.js
var coc03700_vinculo_subgrupoController = function(ScreenService, InflightorGeneral, ZHPromise, Colors) {

    this.corrigeFiltro = function(args) {
        var filterRow = args.e.preference.FILTER_ROW;
        if (filterRow.IDSERVICO === "" || filterRow.IDSERVICO === "T") {
            delete filterRow.IDSERVICO;
        }
    };

    this.voltaValorPadrao = function(widget) {
        if (!widget.appliedFilter.FILTER_ROW.IDSERVICO) {
            widget.appliedFilter.FILTER_ROW.IDSERVICO = "T";
        }
    };

    this.setCicloColor = function(element, row) {
        if (row) {
            setTimeout(function() {
                Array.from(element.find(`.td:eq(1)`)).forEach(element => {
                    if (element && element.style) {
                        element.style.setProperty('background-color', row.NRCORCICLO, 'important');
                    }
                });
            }, 100);
        }
    };

    this.changeColor = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            row.NRCORCICLO = Colors.TColorToHex(row.NRCORCICLO);
        });
    };

    this.antesDeAplicarFiltro = function(widget) {

        widget.dataSourceFilter.forEach(function(condicao) {
            if (condicao.name === "IDSERVICO") {
                condicao.operator = "=";
                condicao.value = widget.appliedFilter.FILTER_ROW.IDSERVICO;
            }
        });

    };

    this.setFilialLogadaRow = function(args) {

        var filterRow = args.e.preference.FILTER_ROW;
        filterRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        filterRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
    };

    this.desabilitaTodosFields = function(widget, field) {
        if (widget.currentRow[field.name] === "") {
            widget.fields.forEach(function(fieldDependente) {
                if (fieldDependente.name !== field.name) {
                    fieldDependente.readOnly = true;
                    fieldDependente.value("");
                }
            });
        }
    };

    this.habilitaField = function(widget, field, dependentes) {
        if (dependentes.length > 1) {
            habilitaVariosField(widget, field, dependentes);
        } else {
            habilitaUmField(widget, field, dependentes);
        }
    };

    var habilitaUmField = function(widget, field, dependentes) {
        if (widget.currentRow[field] === "") {
            widget.getField(dependentes[0]).readOnly = true;
            widget.getField(dependentes[0]).value("");
        } else {
            if (widget.currentRow[dependentes[0]] !== "") {
                widget.getField(dependentes[0]).value("");
            }
            widget.getField(dependentes[0]).readOnly = false;
        }
    };

    var habilitaVariosField = function(widget, field, dependentes) {
        dependentes.forEach(function(dependente) {
            if (widget.currentRow[field] === "") {
                widget.getField(dependente).readOnly = true;
                widget.getField(dependente).value("");
            } else {
                if (widget.currentRow[dependente] !== "" && widget.currentRow[dependentes[0]] !== undefined) {
                    widget.getField(dependente).value("");
                }
                widget.getField(dependente).readOnly = false;
            }
        });
    };

    this.buildFilter = function(mainField, fieldToFilter, dependentFields) {
        var filterRow = mainField.widget.currentRow;
        dependentFields.forEach(function(field) {
            if (mainField.widget.getField(field).dataSourceFilter.filter(condition => condition.name === fieldToFilter).length) {
                mainField.widget.getField(field).dataSourceFilter.filter(condition => condition.name === fieldToFilter)[0].value = filterRow[fieldToFilter];
            } else {
                mainField.widget.getField(field).dataSourceFilter.push({
                    "name": fieldToFilter,
                    "operator": "=",
                    "value": filterRow[fieldToFilter]
                });
            }
            mainField.widget.getField(field).reload();
        });
    };

    this.filtraSubgrupos = function(widget) {

        var widgetSubgrupos = widget.container.getWidget("grid_subgrupos");

        widgetSubgrupos.dataSourceFilter.forEach(function(condicao) {
            if (condicao.name === "CDCICLO") {
                condicao.value = widget.currentRow.CDCICLO;
            }
        });

        widgetSubgrupos.reload();

    };

    this.montaFiltroCiclosAbreSwipe = function(widget) {

        var rowWidget = widget.currentRow;
        var widgetTabbed = widget.container.getWidget("tabbed");
        var widgetCiclosDisponiveis = widget.container.getWidget("grid_ciclos_disponiveis");

        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowWidget.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": rowWidget.CDCIA
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": rowWidget.CDCLASSE
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": rowWidget.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": rowWidget.NRSERVICOCIAFILI
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": rowWidget.NRSERVICOCIAFILI
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": rowWidget.NRORG
            }
        ];

        widgetCiclosDisponiveis.dataSourceFilter = filtro;
        widgetCiclosDisponiveis.reload();
        ScreenService.openSwipe(widgetTabbed);

    };

    this.carregaSubgrupos = function(widget) {

        var rowWidget = widget.currentRow;
        var widgetSubgrupos = widget.container.getWidget("grid_subgrupos");

        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowWidget.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": rowWidget.CDCIA
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": rowWidget.CDCLASSE
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": rowWidget.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": rowWidget.NRSERVICOCIAFILI
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": rowWidget.CDCICLO
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": rowWidget.NRORG
            }
        ];

        widgetSubgrupos.dataSourceFilter = filtro;
        widgetSubgrupos.reload();

        if (!ScreenService.isSwipeOpen(1)) {
            ScreenService.openSwipe(widget.container.getWidget("tabbed_subgrupos"));
        } else {
            widget.container.getWidget("grid_subgrupos").unSelectAll();
        }

    };

    this.afterLoadDataSourceSubgru = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.DTINIVGSUMA && row.DTFIMVGSUMA)
                row.VIGENCIA = moment(row.DTINIVGSUMA, "DD/MM/YYYY hh:ss:mm").format("DD/MM/YYYY") + " à " + moment(row.DTFIMVGSUMA, "DD/MM/YYYY hh:ss:mm").format("DD/MM/YYYY");
        });
    };

    this.fieldOnChangeFiltro = function(args, caso) {
        if (caso == "o Manual") {
            if (args.row.CDFILIAL) {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = args.row.CDFILIAL;
                if (args.row.CDCIA) {
                    args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = args.row.CDCIA;
                    args.owner.owner.widget.getField("DSMANUAL").reload();
                } else {
                    args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[2].value = "";
                    args.owner.owner.widget.getField("DSMANUAL").reload();
                    let msg = ScreenService.i18n("Escolha uma Companhia para definir " + caso + "!");
                    ScreenService.showMessage(msg);
                }
            } else {
                args.owner.owner.widget.getField("DSMANUAL").dataSourceFilter[1].value = "";
                args.owner.owner.widget.getField("DSMANUAL").reload();
                let msg = ScreenService.i18n("Escolha uma Unidade para definir " + caso + "!");
                ScreenService.showMessage(msg);
            }
        } else if (caso == "a Classe") {
            if (args.row.CDFILIAL) {
                args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[1].value = args.row.CDFILIAL;
                if (args.row.CDCIA) {
                    args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[2].value = args.row.CDCIA;
                    args.owner.owner.widget.getField("DSCLASSE").reload();
                } else {
                    args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[2].value = "";
                    args.owner.owner.widget.getField("DSCLASSE").reload();
                    let msg = ScreenService.i18n("Escolha uma Companhia para definir " + caso + "!");
                    ScreenService.showMessage(msg);
                }
            } else {
                args.owner.owner.widget.getField("DSCLASSE").dataSourceFilter[1].value = "";
                args.owner.owner.widget.getField("DSCLASSE").reload();
                let msg = ScreenService.i18n("Escolha uma Unidade para definir " + caso + "!");
                ScreenService.showMessage(msg);
            }
        }
    };

    this.limpaItem = function(widget) {
        var rows = widget.getCheckedRows();
        if (widget.getCheckedRows().length > 0) {
            var msg;
            if (rows.length == 1) {
                msg = "Deseja limpar o registro?";
            } else {
                msg = "Deseja limpar os registros?";
            }
            ScreenService.confirmMessage(msg, "question", function() {
                widget.removeCheckedRows();
            });
        } else {
            ScreenService.showMessage("Nenhum subgrupo selecionado!");
        }
    };

    this.adicionaItem = function(widget, caso) {
        var popupItem = widget.container.getWidget("popup_item");
        var rowWidgetCiclosDisponiveis = widget.container.getWidget("grid_ciclos_disponiveis").currentRow;
        var rows = widget.getCheckedRows();
        if (rows.length > 0) {
            var status = true;
            var msg;
            rows.forEach(function(row) {
                if (row.NRSERVICOCIAFILIPRECO) {
                    status = false;
                    msg = "Existem registros selecionados que já contêm Item de Faturamento!";
                }
            });

            if (status) {

                var filtro = [
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": rowWidgetCiclosDisponiveis.NRORG
                    },
                    {
                        "name": "CDCIA",
                        "operator": "=",
                        "value": rowWidgetCiclosDisponiveis.CDCIA
                    },
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": rowWidgetCiclosDisponiveis.CDFILIAL
                    },
                    {
                        "name": "CDCLASSE",
                        "operator": "=",
                        "value": rowWidgetCiclosDisponiveis.CDCLASSE
                    },
                    {
                        "name": "CDCICLO",
                        "operator": "=",
                        "value": rowWidgetCiclosDisponiveis.CDCICLO
                    }
                ];

                popupItem.currentRow = {};
                popupItem.dataSourceFilter = filtro;
                popupItem.reload();
                ScreenService.openPopup(popupItem);
                popupItem.activate();
                popupItem.caso = caso;
            } else {
                ScreenService.showMessage(msg);
            }

        } else {
            ScreenService.showMessage("Nenhum subgrupo selecionado!");
        }
    };

    this.salvaItem = function(popup, caso) {
        caso = (popup.caso);
        var paramsNewCode = popup.paramsNewCode;
        var widgetSubgru = popup.container.getWidget("grid_subgrupos");
        var widgetComposicao = popup.container.getWidget("grid_composicao");
        if (caso == "subgrupo") {
            ScreenService.confirmMessage(ScreenService.i18n("Vincular " + popup.currentRow.DSSERVICOCIAFILI + "?"), "question", function() {
                var rows = widgetSubgru.getCheckedRows();
                rows.forEach(function(row) {
                    row.NRSERVICOCIAFILIPRECO = popup.currentRow.NRSERVICOCIAFILI;
                    row.changed = true;
                    row.__is_new = true;
                    if (paramsNewCode && paramsNewCode.__is_new_code) {
                        row.__using_unit = paramsNewCode.__using_unit || false;
                        row.__new_code_pk = paramsNewCode.__new_code_pk;
                        row.__table_pk = paramsNewCode.__table_pk;
                        row.__new_code_length = paramsNewCode.__new_code_length || 5;
                        row.__cast_number = paramsNewCode.__cast_number || false;
                    }
                });

                InflightorGeneral.callRestEngine("coc#/crud/precosesubgru/save", "DataSet", rows).then(function() {
                        widgetSubgru.unSelectAll();
                        ScreenService.closePopup();
                        widgetSubgru.activate();
                        widgetSubgru.reload();
                    },
                    function(error) {
                        ScreenService.showMessage(error.data.error);
                    }
                );
            });
        }
        if (caso == "composicao") {
            var dataSetToSave = [];
            widgetComposicao.getCheckedRows().forEach(function(row) {
                var rowToSave = {};
                rowToSave.CDCIA = angular.copy(row.CDCIA);
                rowToSave.CDCICLO = angular.copy(row.CDCICLO);
                rowToSave.CDCLASSE = angular.copy(row.CDCLASSE);
                rowToSave.CDFILIAL = angular.copy(row.CDFILIAL);
                rowToSave.CDMANCIA = angular.copy(row.CDMANCIA);
                rowToSave.CDOPERINCLUSAO = angular.copy(row.CDOPERINCLUSAO);
                rowToSave.CDOPERULTATU = angular.copy(row.CDOPERULTATU);
                rowToSave.CDPRODUTO = angular.copy(row.CDPRODUTO);
                rowToSave.CDRECEITA = angular.copy(row.CDRECEITA);
                rowToSave.CDSUBGRU = angular.copy(row.CDSUBGRU);
                rowToSave.ID = angular.copy(row.ID);
                rowToSave.NRORG = angular.copy(row.NRORG);
                rowToSave.NRSEQSUBGRUSEMAM = angular.copy(row.NRSEQSUBGRUSEMAM);
                rowToSave.NRSERVICOCIAFILI = angular.copy(row.NRSERVICOCIAFILI);

                rowToSave.NRTABPRECO = angular.copy(popup.currentRow.NRTABPRECO);
                rowToSave.NRSERVICOCIAFILIPRECO = angular.copy(popup.currentRow.NRSERVICOCIAFILI);
                rowToSave.NRSEQPRECO = angular.copy(popup.currentRow.NRSEQPRECO);

                rowToSave.changed = true;
                rowToSave.__is_new = true;
                rowToSave.__is_new_code = true;
                rowToSave.__new_code_pk = "PRECOSESUBGRU";
                rowToSave.__new_code_length = 25;
                rowToSave.__table_pk = "NRSEQPRECOSESUBGRU";

                dataSetToSave.push(rowToSave);
            });

            InflightorGeneral.callRestEngine("coc#/crud/precosesubgru/save", "DataSet", dataSetToSave).then(function() {
                widgetComposicao.unSelectAll();
                ScreenService.closePopup();
                widgetComposicao.activate();
                widgetComposicao.reload();
            });
        }
    };

    this.setFilterComposicao = function(widget) {
        var widgetSubgru = widget.container.getWidget("grid_subgrupos");
        var widgetComposicao = widget.container.getWidget("grid_composicao");

        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": widgetSubgru.currentRow.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDCLASSE
            },
            {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDSUBGRU
            },
            {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": widgetSubgru.currentRow.NRSEQSUBGRUSEMAM
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": widgetSubgru.currentRow.CDCICLO
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widgetSubgru.currentRow.NRORG
            }
        ];
        widgetComposicao.dataSourceFilter = dataSourceFilter;
        widgetComposicao.reload();
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc03700_vinculo_subgrupoController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            ScreenService.showMessage("A companhia selecionada está inativa.");
        } else {
            args.owner.setSelected(args);
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc03700_vinculo_subgrupoController", coc03700_vinculo_subgrupoController);
});

// FILE: ./assets/js/controllers/coc21000_tabela_de_precos_controllers.js
var coc21000_tabela_de_precos_controllers = function(coc03600_menu_designService, ScreenService, ZHPromise, TabelaPrecosService, InflightorGeneral, MessageService, templateManager, metaDataFactory) {

    var self = this;

    function trataErros(message) {
        if (message && message.length) {
            if (message.indexOf("COC21000_") >= 0) {
                if (message.indexOf("COC21000_IT_SEM_VINCULO") >= 0) {
                    erro_params = message.split("#_#");
                    MessageService.showMessageByCode('COC21000_IT_SEM_VINCULO', {
                        DSSERVICOCIAFILI: erro_params[1],
                        SGCLASSE: erro_params[2],
                        IDCICLO: erro_params[3]
                    });
                } else if (message.indexOf("COC21000_MAIS_DE_UM_VINCULO") >= 0) {
                    erro_params = message.split("#_#");
                    MessageService.showMessageByCode('COC21000_MAIS_DE_UM_VINCULO', {
                        DSSERVICOCIAFILI: erro_params[1],
                        SGCLASSE: erro_params[2],
                        IDCICLO: erro_params[3]
                    });
                } else if (message.indexOf("COC21000_PLANO_VOO_JA_GERADO") >= 0) {
                    erro_params = message.split("#_#");
                    MessageService.showMessageByCode('COC21000_PLANO_VOO_JA_GERADO', {
                        LASTDATE: erro_params[1]
                    });
                } else {
                    MessageService.showMessageByCode(message);
                }
            } else {
                ScreenService.showMessage(message);
            }
        }
    }

    this.redimencionaCampo = function(field, size) {
        field.class = size;
    };

    this.setaServicosRow = function(field) {
        field.widget.currentRow.SERVICOS = field.dataSource.getCheckedRows();
    };

    this.deleteMult = function(widget) {
        InflightorGeneral.deleteCrudCheckedRows(widget);
    };

    this.gerarPrecoProporcionalItensSelecionados = function(widget) {
        InflightorGeneral.callRestEngine("coc#/gerarPrecoProporcionalItensSelecionados", "DataSet", widget.getCheckedRows());
    };

    this.ajustaFiltro = function(widget) {
        widget.currentRow.DATAREF = widget.appliedFilter.FILTER_ROW.DATAREF;
        widget.dataSourceFilter.forEach(function(condicao) {

            if (condicao.name === "IDVIGENCIA") {
                condicao.operator = "=";
                condicao.value = condicao.value.replace(/%/g, "");
            }

        });

    };
    this.importaProporcao = function(widget) {
        MessageService.confirmMessageByCode("COC21000_DADOS_SUBS_PELO_ARQ").then(function() {
            let widgetPopup;
            if (widget.name == "precoProporcional") {
                widgetPopup = widget.container.getWidget("widgetImportacaoProporcaoVariavel");
                widgetPopup.getField('FileUpload').setValue(null);
            } else if (widget.name == "proporcaoVariavel") {
                widgetPopup = widget.container.getWidget("widgetImportacaoPropVariavel");
                widgetPopup.getField('FileUpload').setValue(null);
            }
            ScreenService.openPopup(widgetPopup);
        });
    };

    this.habilitaPeriodo = function(widget) {

        if (widget.currentRow.IDVIGENCIA === "T") {
            widget.getField("DTINIVIGTAB").readOnly = false;
            // widget.getField('DTINIVIGTAB').validations = {"required":true};
            widget.getField("DTFIMVIGTAB").readOnly = false;
            // widget.getField('DTFIMVIGTAB').validations = {"required":true};
        } else {
            widget.getField("DTINIVIGTAB").readOnly = true;
            widget.getField("DTINIVIGTAB").value("");
            widget.getField("DTINIVIGTAB").validations = {};
            widget.getField("DTFIMVIGTAB").readOnly = true;
            widget.getField("DTFIMVIGTAB").value("");
            widget.getField("DTFIMVIGTAB").validations = {};
        }

    };

    //Exige que seja selecionada uma Unidade para exibir as Companhias
    this.fieldOnChangeFiltro = function(args) {
        var widget = templateManager.container.getWidget('tabeladeprecos');
        var field = widget.getField('NMCIA');
        if (args.row.CDFILIAL) {
            field.dataSourceFilter[0].value = args.row.CDFILIAL;
        } else {
            field.dataSourceFilter[0].value = "";
        }
    };

    this.onLoadDataSource = function(widget) {
        var dataRef = widget.appliedFilter ? widget.appliedFilter.FILTER_ROW.DATAREF : widget.container.getActiveWidget().getField('DATAREF').value();
        MessageService.getMessagesByCode(["COC21000_VIGENTE", "COC21000_N_VIGENTE"]).then(function(messages) {
            widget.dataSource.data.forEach(function(tabela) {
                if (moment(tabela.DTINIVIGTAB, "DD/MM/YYYY") <= moment(dataRef, "DD/MM/YYYY") && moment(tabela.DTFIMVIGTAB, "DD/MM/YYYY") >= moment(dataRef, "DD/MM/YYYY")) {
                    tabela.STATUS = messages[0];
                } else {
                    tabela.STATUS = messages[1];
                }
                tabela.DATAREF = dataRef;
            });
        });
    };

    this.setUnidadeFilter = function(widget) {
        widget.getField("NMCIA").dataSourceFilter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        }];
        widget.getField("NMCIA").reload();
    };

    this.novaTabPreco = function(widget) {
        widget.container.getWidget("widgetMenuGrids").isVisible = false;

        if (!widget.appliedFilter || !widget.appliedFilter.FILTER_ROW) {
            MessageService.showMessageByCode('FILTRO_INICIAL_DEVE');
            return;
        }
        var filterCurrentRow = widget.appliedFilter.FILTER_ROW;

        if (filterCurrentRow.CDFILIAL && filterCurrentRow.CDCIA) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": filterCurrentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": filterCurrentRow.CDCIA
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ];

            var fieldDsmanual = widget.getField("DSMANUAL");
            fieldDsmanual.dataSourceFilter = dataSourceFilter;
            fieldDsmanual.validations.required = filterCurrentRow.IDVINCULOMANUAL == "S" ? true : false;
            fieldDsmanual.readOnly = false;

            widget.getField("NMTABPRECO").class = 12;
            widget.getField("DTINIVIGTAB").validations = {
                "required": true
            };
            widget.getField("DTINIVIGTAB").readOnly = false;
            widget.getField("DTFIMVIGTAB").validations = {
                "required": true
            };
            widget.getField("DTFIMVIGTAB").readOnly = false;
            widget.newRow();
            widget.dataSourceFilter.forEach(function(filtro) {
                if (filtro.name === "CDFILIAL") {
                    widget.currentRow.CDFILIAL = filtro.value;
                } else if (filtro.name === "CDCIA") {
                    widget.currentRow.CDCIA = filtro.value;
                }
            });
            widget.container.getWidget("servicostabela").isVisible = false;
            widget.currentRow.CDOPERINCTAB = getLocalVar("INFLIGHTOR_CDOPERADOR");
            widget.currentRow.__new_code_pk = "TABELA/PRECO" + widget.currentRow.CDFILIAL + widget.currentRow.CDCIA;
            widget.currentRow.__is_new_code = widget.paramsNewCode.__is_new_code;
            widget.currentRow.__columns_to_return = widget.__columns_to_return;
            widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length;
            widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size;

        } else {
            MessageService.showMessageByCode("COC21000_INF_CIA");
        }

    };

    this.salvaNovaTabelaPreco = function(args) {
        var widget = args.owner.widget;

        var idvinculomanual = widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW.IDVINCULOMANUAL;

        if (idvinculomanual != "S") {

            if (widget.getField("DSMANUAL").validate) {
                delete(widget.getField("DSMANUAL").validate);
            }

        }

        if (widget.isValid()) {
            if (this.validaData(widget.currentRow.DTINIVIGTAB, widget.currentRow.DTFIMVIGTAB)) {
                TabelaPrecosService.getProcessosWorkflow("3", "1").then(function(callback) {
                    if (callback.dataset.processos_workflow_ativos.length && !widget.currentRow.__is_new) {
                        var modifiedinRows = compareDate(widget.currentRow, widget.backupRow, "TABPRECO");
                        if (modifiedinRows) {
                            //Gera Workflow
                            self.solicitaCriacaoWorkflow(widget, "edit_tabpreco", widget.currentRow, modifiedinRows);
                        }
                        if (widget.currentRow.NMTABPRECO != widget.backupRow.NMTABPRECO) {
                            widget.currentRow.DTINIVIGTAB = widget.backupRow.DTINIVIGTAB.slice(0, 10);
                            widget.currentRow.DTFIMVIGTAB = widget.backupRow.DTFIMVIGTAB.slice(0, 10);
                            TabelaPrecosService.saveTabelaPreco(widget.currentRow).then(function() {
                                widget.reload();
                                widget.view();
                                widget.activate();
                                widget.getField("NMTABPRECO").class = 6;
                                widget.getField("DTINIVIGTAB").validations = {};
                                widget.getField("DTFIMVIGTAB").validations = {};
                                widget.container.getWidget("servicostabela").isVisible = true;
                            }, function(error) {
                                if (error.data) {
                                    trataErros(error.data.error);
                                }
                            });
                        } else if (!modifiedinRows) {
                            widget.container.getWidget("servicostabela").isVisible = true;
                            if (widget.currentRow.__is_new) {
                                widget.view();
                            } else {
                                ScreenService.closeSwipe().then(
                                    function() {
                                        widget.list();
                                        widget.activate();
                                    }
                                );
                            }
                        }
                    } else {
                        TabelaPrecosService.saveTabelaPreco(widget.currentRow).then(function() {
                            if (widget.currentRow.__is_new) {
                                widget.list(true);
                                ScreenService.goBack();
                                widget.reload();
                            } else {
                                widget.reload();
                                widget.view();
                            }
                            widget.activate();
                            widget.getField("NMTABPRECO").class = 6;
                            widget.getField("DTINIVIGTAB").validations = {};
                            widget.getField("DTFIMVIGTAB").validations = {};
                            widget.container.getWidget("servicostabela").isVisible = true;
                        }, function(error) {
                            if (error.data) {
                                trataErros(error.data.error);
                            }
                        });
                    }
                });
            }
        }
    };

    this.widgetMenuGridsOnEnter = function(widget) {
        var widgetTabelaDePrecos = widget.container.getWidget("tabeladeprecos");
        widget.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.CDCIA
            },
            {
                "name": "NRTABPRECO",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.NRTABPRECO
            }
        ];
        widget.reload();
    };

    this.afterMoveRowServicos = function(widget) {
        var parentWidget = widget.container.getWidget("servicostabela");
        var subgrupoWidget = widget.container.getWidget("grid_subgrupos");

        if (subgrupoWidget.mode !== "list") {
            ScreenService.closeSwipe().then(function() {
                if (parentWidget.mode !== "list") {
                    ScreenService.closeSwipe();
                    subgrupoWidget.activate();
                }
            });
            ScreenService.closeSwipe();
            subgrupoWidget.activate();
        } else if (parentWidget.mode !== "list") {
            ScreenService.closeSwipe();
            parentWidget.activate();
        }
        parentWidget.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "NRTABPRECO",
                "operator": "=",
                "value": widget.currentRow.NRTABPRECO
            },
            {
                "name": "DATAREF",
                "operator": "=",
                "value": widget.currentRow.DATAREF
            }
        ];
        if (!widget.currentRow.__is_new) {
            if (typeof widget.currentWidget !== "undefined") {
                widget.currentWidget.reload().then(function() {
                    if (widget.currentWidget.name == "widgetMenuGrids") {
                        self.widgetMenuGridsOnEnter(widget.currentWidget);
                    }
                });
            } else {
                widget.reload();
            }

            if (widget.appliedFilter.FILTER_ROW.IDVINCULOMANUAL == "S") {
                widget.container.getWidget("widgetMenuGrids").isVisible = true;
            } else {
                widget.container.getWidget("widgetMenuGrids").isVisible = false;
            }

            widget.getField("DSMANUAL").isVisible = false;
        } else {
            widget.getField("DSMANUAL").isVisible = true;
        }
    };

    this.filterServTabela = function(widget) {
        var parentWidget = widget.container.getWidget("tabeladeprecos");
        if (widget.mode == "list") {
            widget.dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": parentWidget.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": parentWidget.currentRow.CDCIA
                },
                {
                    "name": "NRTABPRECO",
                    "operator": "=",
                    "value": parentWidget.currentRow.NRTABPRECO
                },
                {
                    "name": "DATAREF",
                    "operator": "=",
                    "value": parentWidget.currentRow.DATAREF
                }
            ];

            widget.getField("SERVICOS").dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": parentWidget.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": parentWidget.currentRow.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": parentWidget.currentRow.CDMANCIA
                }
            ];
            if (!widget.currentRow.__is_new) {
                widget.reload();
            }
            widget.activate();
        }

    };

    this.filtraComposicao = function(widget) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.getParent().currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widget.getParent().currentRow.CDMANCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": widget.getParent().currentRow.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCLASSE
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCICLO
            },
            {
                "name": "CDSUBGRU",
                "operator": "=",
                "value": widget.getParent().currentRow.CDSUBGRU
            },
            {
                "name": "NRSEQSUBGRUSEMAM",
                "operator": "=",
                "value": widget.getParent().currentRow.NRSEQSUBGRUSEMAM
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.filtraTabSubgrupos = function(widget) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.getParent().currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCIA
            },
            {
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": widget.getParent().currentRow.NRSERVICOCIAFILI
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCLASSE
            },
            {
                "name": "CDCICLO",
                "operator": "=",
                "value": widget.getParent().currentRow.CDCICLO
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widget.getParent().currentRow.CDMANCIA
            },
            {
                "name": "NRSEQPRECO",
                "operator": "=",
                "value": widget.getParent().currentRow.NRSEQPRECO
            },
            {
                "name": "NRTABPRECO",
                "operator": "=",
                "value": widget.getParent().currentRow.NRTABPRECO
            },
            // {
            //     "name": "DATAREF",
            //     "operator": "=",
            //     "value": widget.container.getWidget('tabeladeprecos').appliedFilter.FILTER_ROW.DATAREF 
            // },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    // Funções para serviços da tabela de preços

    this.novoServTab = function(widget) {
        widget.newRow();
        widget.fields[0].isVisible = true;
        widget.fields[0].class = 12;
        widget.fields[1].showOnForm = false;
        widget.fields[1].class = 0;
        self.alteraProporcao(widget);
        widget.getField("SGCLASSE").showOnForm = false;
        widget.getField("IDCICLO").showOnForm = false;
        widget.getField("DTINIVIGSE").readOnly = false;
        widget.getField("DTFIMVIGSE").readOnly = false;
        widget.getField("IDUTILLOADIGUAL").readOnly = false;
        widget.getField("IDCALCTABPROP").readOnly = false;

        widget.getField('CDSEQPURCHASEORDER').isVisible = false;
        widget.fieldGroups.filter(function(fieldGroup) {
            return fieldGroup.name == "INTEGRACAO";
        })[0].isVisible = false;

        widget.getField("DTINIVIGSE").setValue(widget.container.getWidget("tabeladeprecos").currentRow.DTINIVIGTAB);
        widget.getField("DTFIMVIGSE").setValue(widget.container.getWidget("tabeladeprecos").currentRow.DTFIMVIGTAB);
        widget.dataSourceFilter.forEach(function(filtro) {
            if (filtro.name === "CDFILIAL") {
                widget.currentRow.CDFILIAL = filtro.value;
            } else if (filtro.name === "CDCIA") {
                widget.currentRow.CDCIA = filtro.value;
            } else if (filtro.name === "NRTABPRECO") {
                widget.currentRow.NRTABPRECO = filtro.value;
            }
        });

        widget.getField("SERVICOS").dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "NRTABPRECO",
                "operator": "=",
                "value": widget.currentRow.NRTABPRECO
            }
        ];

        widget.container.getWidget("grid_subgrupos").isVisible = false;
        widget.widgets[1].isVisible = false;
    };

    this.onEditTabPreco = function(widget) {
        widget.getField("DTINIVIGTAB").readOnly = false;
        widget.getField("DSMANUAL").readOnly = true;
        widget.getField("DTINIVIGTAB").validations = {
            "required": true
        };
        widget.getField("DTFIMVIGTAB").readOnly = false;
        widget.getField("DTFIMVIGTAB").validations = {
            "required": true
        };
        permiteEdicao("NRTABPRECO=" + widget.currentRow.NRTABPRECO, "TABPRECOSEM").then(function(callback) {
            if (callback) {
                widget.getField("NMTABPRECO").class = 12;
                widget.currentRow.BACKUPROW = angular.copy(widget.currentRow);
                InflightorGeneral.edit(widget);
                widget.container.getWidget("servicostabela").isVisible = false;
                TabelaPrecosService.getDadoswFlow(widget.currentRow.NRTABPRECO, "TABPRECOSEM").then(function(callback) {
                    var processos = callback.dataset.workflow_dadoswflow;
                    if (processos.length) {
                        processos.forEach(function(field) {
                            if ((field.NMCAMPOWFLOW.search("DTINIVIGTAB") != -1 || field.NMCAMPOWFLOW.search("DTFIMVIGTAB") != -1) && field.IDSITWORKFLOW == 1) {
                                widget.getField("DTINIVIGTAB").readOnly = true;
                                widget.getField("DTFIMVIGTAB").readOnly = true;
                            }
                        });
                    }
                });
            } else {
                MessageService.showMessageByCode("COC21000_TAB_IN_EXCLUSAO");
            }
        });
    };

    var currentItemDeFaturamento;

    this.onEditPreco = function(widget) {

        currentItemDeFaturamento = angular.copy(widget.currentRow);

        if (widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW.IDVINCULOMANUAL == "S" && widget.currentRow.IDSERVICO != "3" && widget.currentRow.IDSERVICO != "5") {
            widget.getField("IDPROPORCAO").readOnly = true;
            delete widget.getField("IDPROPORCAO").validations;
            widget.getField("QTPROPORCAO").readOnly = true;
            delete widget.getField("QTPROPORCAO").validations;
        } else {
            widget.getField("IDPROPORCAO").readOnly = false;
            widget.getField("IDPROPORCAO").validations = {
                required: true
            };
            if (currentItemDeFaturamento.IDPROPORCAO === "1" || currentItemDeFaturamento.IDPROPORCAO === "3") {
                widget.getField("QTPROPORCAO").readOnly = true;
                delete widget.getField("QTPROPORCAO").validations;
            } else {
                widget.getField("QTPROPORCAO").readOnly = false;
                widget.getField("QTPROPORCAO").validations = {
                    required: true
                };
            }
        }

        widget.getField("VRPRESERV").readOnly = false;
        widget.getField("VRPRESERVD").readOnly = false;
        widget.getField("DTINIVIGSE").readOnly = false;
        widget.getField("DTFIMVIGSE").readOnly = false;
        widget.getField("INTERVAL").readOnly = false;
        widget.getField("IDCICLO").readOnly = true;
        widget.getField("IDCALCTABPROP").readOnly = true;
        widget.getField("IDUTILLOADIGUAL").readOnly = true;
        widget.getField("SGCLASSE").readOnly = true;
        widget.getField("DSSERVICOCIAFILI").readOnly = true;
        widget.container.getWidget("grid_subgrupos").isVisible = false;
        widget.widgets[1].isVisible = false;
        permiteEdicao("NRTABPRECO=" + widget.currentRow.NRTABPRECO + ";NRSERVICOCIAFILI=" + widget.currentRow.NRSERVICOCIAFILI +
            ";NRSEQPRECO=" + widget.currentRow.NRSEQPRECO + ";CDCLASSE=" + widget.currentRow.CDCLASSE + ";CDCICLO=" + widget.currentRow.CDCICLO, "PRECOSECIAM").then(function(callback) {
            if (callback) {
                InflightorGeneral.edit(widget);
                TabelaPrecosService.getDadoswFlow("NRTABPRECO=" + widget.currentRow.NRTABPRECO + ";NRSERVICOCIAFILI=" + widget.currentRow.NRSERVICOCIAFILI +
                    ";NRSEQPRECO=" + widget.currentRow.NRSEQPRECO + ";CDCLASSE=" + widget.currentRow.CDCLASSE + ";CDCICLO=" + widget.currentRow.CDCICLO, "PRECOSECIAM").then(function(callback) {
                    var processos = callback.dataset.workflow_dadoswflow;
                    if (processos.length) {
                        processos.forEach(function(field) {
                            if ((field.NMCAMPOWFLOW.search("DTINIVIGSE") != -1 || field.NMCAMPOWFLOW.search("DTFIMVIGSE") != -1) && field.IDSITWORKFLOW == 1) {
                                widget.getField("DTINIVIGSE").readOnly = true;
                                widget.getField("DTFIMVIGSE").readOnly = true;
                            }
                            if ((field.NMCAMPOWFLOW.search("VRPRESERV") != -1 || field.NMCAMPOWFLOW.search("VRPRESERVD") != -1) && field.IDSITWORKFLOW == 1) {
                                widget.getField("VRPRESERV").readOnly = true;
                                widget.getField("VRPRESERVD").readOnly = true;
                            }
                        });
                    }
                });
            } else {
                MessageService.showMessageByCode("COC21000_IT_IN_EXCLUSAO");
            }
        });
    };

    this.onCancelEditPreco = function(args) {
        var widget = args.owner.widget;

        widget.getField('CDSEQPURCHASEORDER').isVisible = true;
        widget.fieldGroups.filter(function(fieldGroup) {
            return fieldGroup.name == "INTEGRACAO";
        })[0].isVisible = true;

        widget.getField("VRPRESERV").readOnly = true;
        widget.getField("VRPRESERVD").readOnly = true;
        widget.getField("DTINIVIGSE").readOnly = true;
        widget.getField("DTFIMVIGSE").readOnly = true;
        widget.getField("DSSERVICOCIAFILI").readOnly = false;
        widget.container.getWidget("grid_subgrupos").isVisible = true;
        widget.fields[0].isVisible = false;
        widget.fields[0].class = 0;
        widget.fields[1].showOnForm = true;
        widget.fields[1].class = 6;
        widget.getField("SGCLASSE").showOnForm = true;
        widget.getField("IDCICLO").showOnForm = true;
        self.setTabsVisibility(widget);
        InflightorGeneral.backOrCancel(args);
    };

    this.applyStylesGridSelectProduto = function(fieldSelect) {
        fieldSelect.selectWidget.styles = [
            {
                condition: "row.IDSITSERVICO === 1",
                cssClass: "coc-inactive"
            },
            {
                condition: "row.IDVINCULOMANUAL === 'S' && row.IDSERVICO !== '3' && row.IDSERVICO !== '5' && (row.CDMANCIA == null || row.QT_MANUAIS > 1)",
                cssClass: "coc-inactive"
            }
        ];
        fieldSelect.selectWidget.reload();
    };

    this.applyStyleSubgrupo = function(widget) {
        var dataref = widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW.DATAREF;
        var momentRef = moment(dataref, "DD/MM/YYYY");
        widget.dataSource.data.forEach(function(row) {
            if (row.DTINIVGSUMA && row.DTFIMVGSUMA && row.IDSITUAPROPCIA != "P") {
                if (!(momentRef.unix() >= moment(row.DTINIVGSUMA, "DD/MM/YYYY").unix() && momentRef.unix() <= moment(row.DTFIMVGSUMA, "DD/MM/YYYY").unix())) {
                    //Subgrupo não vigênte na data de referência
                    row.STATUS = "NV";
                } else {
                    row.STATUS = "V";
                }
            }
        });
    };

    this.onChangeServico = function(field) {
        if (field.widget.currentRow.SERVICOS.length) {
            field.widget.getField("VRPRESERVD").readOnly = false;
            field.widget.getField("VRPRESERV").readOnly = false;
            field.widget.getField("DTINIVIGSE").readOnly = false;
            field.widget.getField("DTFIMVIGSE").readOnly = false;
        }
    };

    this.onChangeClasse = function(field) {
        if (field.widget.currentRow.CDCLASSE && field.widget.currentRow.CDCLASSE !== "") {
            field.widget.getField("IDCICLO").readOnly = false;
            field.widget.getField("IDCICLO").dataSourceFilter = [{
                "name": "NRSERVICOCIAFILI",
                "operator": "=",
                "value": field.widget.currentRow.NRSERVICOCIAFILI
            }, {
                "name": "CDCLASSE",
                "operator": "=",
                "value": field.widget.currentRow.CDCLASSE
            }, {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            }, {
                "name": "CDCIA",
                "operator": "=",
                "value": field.widget.currentRow.CDCIA
            }];
            field.widget.getField("IDCICLO").reload();
        } else {
            field.widget.getField("IDCICLO").readOnly = true;
            field.widget.getField("IDCICLO").setValue("");
        }
    };

    this.setPeriodoVigen = function(field) {
        if (field.widget.currentRow.INTERVAL.start && field.widget.currentRow.INTERVAL.end) {
            field.widget.getField("DTINIVIGSE").setValue(field.widget.currentRow.INTERVAL.start);
            field.widget.getField("DTFIMVIGSE").setValue(field.widget.currentRow.INTERVAL.end);
            field.widget.currentRow.DTFIMVIGSE = field.widget.currentRow.INTERVAL.end;
            field.widget.currentRow.DTINIVIGSE = field.widget.currentRow.INTERVAL.start;
        }
    };

    this.setPeriodoVigenTabPreco = function(field) {
        if (field.widget.currentRow.INTERVAL_TAB.start && field.widget.currentRow.INTERVAL_TAB.end) {
            field.widget.getField("DTINIVIGTAB").setValue(field.widget.currentRow.INTERVAL_TAB.start);
            field.widget.getField("DTFIMVIGTAB").setValue(field.widget.currentRow.INTERVAL_TAB.end);
            field.widget.currentRow.DTFIMVIGTAB = field.widget.currentRow.INTERVAL_TAB.end;
            field.widget.currentRow.DTINIVIGTAB = field.widget.currentRow.INTERVAL_TAB.start;
        }
    };

    this.onChangeCiclo = function(field) {
        if (field.widget.currentRow.IDCICLO && field.widget.currentRow.IDCICLO !== "") {
            field.widget.getField("VRPRESERV").readOnly = false;
            field.widget.getField("VRPRESERVD").readOnly = false;
            field.widget.getField("DTINIVIGSE").readOnly = false;
            field.widget.getField("DTFIMVIGSE").readOnly = false;
        } else {
            field.widget.getField("VRPRESERV").readOnly = true;
            field.widget.getField("VRPRESERV").setValue("");
            field.widget.getField("VRPRESERVD").readOnly = true;
            field.widget.getField("VRPRESERVD").setValue("");
            field.widget.getField("DTINIVIGSE").readOnly = true;
            field.widget.getField("DTINIVIGSE").setValue("");
            field.widget.getField("DTFIMVIGSE").readOnly = true;
            field.widget.getField("DTFIMVIGSE").setValue("");
        }
    };

    this.setTabsVisibility = function(widget) {
        if (widget.currentRow.IDPROPORCAO == "3") {
            widget.container.getWidget("proporcaoVariavel").isVisible = true;
        } else {
            widget.container.getWidget("proporcaoVariavel").isVisible = false;
        }
        widget.container.getWidget("precoProporcional").isVisible = true;
    };

    var backupRow;

    this.salvaNovaTabelaDePrecos = function(widget) {

        var counter = 1;
        var promises = [];
        backupRow = angular.copy(widget.currentRow);
        var dataToSave = [];

        if (self.validaData(widget.currentRow.DTINIVIGSE, widget.currentRow.DTFIMVIGSE)) {

            TabelaPrecosService.getProcessosWorkflow("3", "5").then(function(callback) {

                var servicos = widget.getField("SERVICOS");

                servicos.reload().then(function() {

                    var dateFinal = widget.currentRow.DTFIMVIGSE;
                    var dateInit = widget.currentRow.DTINIVIGSE;
                    var valueR = widget.currentRow.VRPRESERV;
                    var valueD = widget.currentRow.VRPRESERVD;
                    var proprocao = widget.currentRow.IDPROPORCAO;
                    var value_proporcao = widget.currentRow.QTPROPORCAO;

                    servicos.dataSource.getCheckedRows().forEach(function(servico) {

                        if (servico.IDVINCULOMANUAL == "S" && servico.IDSERVICO != "3" && servico.IDSERVICO != "5") {

                            if (servico.CDMANCIA == null) {
                                MessageService.showMessageByCode("COC21000_IT_SEM_VINCULO", {
                                    DSSERVICOCIAFILI: servico.DSSERVICOCIAFILI,
                                    SGCLASSE: servico.SGCLASSE,
                                    IDCICLO: servico.IDCICLO
                                });
                                throw "";
                            }

                            if (servico.QT_MANUAIS > 1) {
                                MessageService.showMessageByCode("COC21000_MAIS_DE_UM_VINCULO", {
                                    DSSERVICOCIAFILI: servico.DSSERVICOCIAFILI,
                                    SGCLASSE: servico.SGCLASSE,
                                    IDCICLO: servico.IDCICLO
                                });
                                throw "";
                            }

                        }

                        if (servico.IDSITSERVICO != 1) {
                            MessageService.showMessageByCode("COC21000_IT_INATIVO");
                            throw "";
                        }

                        var rowToSave = {};
                        rowToSave.__is_new = widget.currentRow.__is_new;
                        rowToSave.changed = true;
                        rowToSave.__new_code_pk = "PRECOSECIAM" + servico.CDFILIAL + servico.CDCIA + widget.container.getWidget("tabeladeprecos").currentRow.NRTABPRECO;
                        rowToSave.__is_new_code = widget.paramsNewCode.__is_new_code;
                        rowToSave.__table_pk = widget.paramsNewCode.__table_pk;
                        rowToSave.__new_code_length = widget.paramsNewCode.__new_code_length;
                        rowToSave.__columns_to_return = ["NRSEQPRECO"];
                        rowToSave.IDCALCTABPROP = widget.currentRow.IDCALCTABPROP;
                        rowToSave.IDUTILLOADIGUAL = widget.currentRow.IDUTILLOADIGUAL;
                        rowToSave.SAVE_SUBGRUPO = true;
                        rowToSave.DTFIMVIGSE = dateFinal;
                        rowToSave.IDSITSERVICO = servico.IDSITSERVICO;
                        rowToSave.DTINIVIGSE = dateInit;
                        rowToSave.QT_MANUAIS = servico.QT_MANUAIS;
                        rowToSave.IDVINCULOMANUAL = servico.IDVINCULOMANUAL;
                        rowToSave.QT_SERVICOS = servico.QT_SERVICOS;
                        rowToSave.IDTPPRECO = widget.currentRow.IDTPPRECO;
                        rowToSave.IDSERVICO = servico.IDSERVICO;
                        rowToSave.CDMANCIA = servico.CDMANCIA;
                        rowToSave.VRPRESERV = InflightorGeneral.valueToFloat(valueR);
                        rowToSave.VRPRESERVD = InflightorGeneral.valueToFloat(valueD);
                        rowToSave.CDSERVICO = servico.CDSERVICO;
                        rowToSave.CDFILIAL = servico.CDFILIAL;
                        rowToSave.CDCIA = servico.CDCIA;
                        rowToSave.NRTABPRECO = widget.container.getWidget("tabeladeprecos").currentRow.NRTABPRECO;
                        rowToSave.NRSERVICOCIAFILI = servico.NRSERVICOCIAFILI;
                        rowToSave.CDCLASSE = servico.CDCLASSE;
                        rowToSave.CDCICLO = servico.CDCICLO;
                        rowToSave.CDCLASSE = servico.CDCLASSE;
                        rowToSave.IDPROPORCAO = proprocao;
                        rowToSave.QTPROPORCAO = value_proporcao;
                        rowToSave.NRORG = servico.NRORG;
                        rowToSave.DSSERVICOCIAFILI = servico.DSSERVICOCIAFILI;

                        if (callback.dataset.processos_workflow_ativos.length) {
                            dataToSave.push(rowToSave);
                        } else {
                            promises.push(TabelaPrecosService.saveServTab(rowToSave));
                        }

                    });

                    if (callback.dataset.processos_workflow_ativos.length) {
                        self.solicitaCriacaoWorkflow(widget, "new_itemFaturamento", dataToSave, null);
                    } else {
                        self.salvaDataServicos(widget, promises, counter);
                    }

                });

            });

        }

    };

    this.salvaServicoTabela = function(widget) {

        try {

            if (InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERV) == 0 ||
                InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERVD) == 0) {

                if (widget.currentRow.__is_new) {

                    MessageService.showMessageByCode("COC21000_PRECO_ZERADO");

                } else {

                    var modifiedinRows_preco = comparePrecos(widget.currentRow, widget.backupRow, "ITEMFAT");

                    if (modifiedinRows_preco) {

                        MessageService.showMessageByCode("COC21000_PRECO_ZERADO");

                    } else {

                        return self.salvaEdicaoServicoTabela(widget);

                    }

                }

            } else {

                if (widget.currentRow.__is_new) {

                    return self.salvaNovaTabelaDePrecos(widget);

                } else {

                    return self.salvaEdicaoServicoTabela(widget);

                }

            }

        } catch (exception) {
            trataErros(exception);
        }

    };

    this.salvaDataServicos = function(widget, promises, counter) {
        ZHPromise.all(promises).then(function() {
            if (widget.currentRow.__is_new) {
                widget.list();
                if (counter)
                    ScreenService.goBack();
                counter = 0;
                widget.reload();
            }
            self.setTabsVisibility(widget);

            widget.getField('CDSEQPURCHASEORDER').isVisible = true;
            widget.fieldGroups.filter(function(fieldGroup) {
                return fieldGroup.name == "INTEGRACAO";
            })[0].isVisible = true;

            widget.getField("VRPRESERV").readOnly = true;
            widget.getField("VRPRESERVD").readOnly = true;
            widget.getField("DTINIVIGSE").readOnly = true;
            widget.getField("DTFIMVIGSE").readOnly = true;
            widget.activate();
            widget.fields[0].isVisible = false;
            widget.fields[0].class = 0;
            widget.fields[1].showOnForm = true;
            widget.fields[1].class = 6;
            widget.getField("SGCLASSE").showOnForm = true;
            widget.getField("IDCICLO").showOnForm = true;
        }.bind(this), function(callback) {
            if (callback.data && callback.data.error) {
                trataErros(callback.data.error);
            }
            widget.currentRow = backupRow;
            self.alteraProporcao(widget);
            widget.reload();
        });
    };

    this.salvaEdicaoServicoTabela = function(widget) {
        var salva_edicao_sem_workflow = true;
        var promises = [];
        var workflow_controle = true;
        if (this.validaData(widget.currentRow.DTINIVIGSE, widget.currentRow.DTFIMVIGSE)) {
            //Valida se existem Plano de Voo já gerado para o intervalo de Vigência do Item
            var modifiedinRows_dates = compareDate(widget.currentRow, widget.backupRow, "ITEMFAT");
            var modifiedinRows_preco = comparePrecos(widget.currentRow, widget.backupRow, "ITEMFAT");
            var changedDate;
            if (modifiedinRows_dates && !modifiedinRows_preco)
                changedDate = true;
            else
                changedDate = false;
            searchPlanoVooGerados(widget, changedDate).then(function() {
                promises.push(TabelaPrecosService.getProcessosWorkflow("3", "2"));
                promises.push(TabelaPrecosService.getProcessosWorkflow("3", "3"));
                ZHPromise.all(promises).then(function(callback) {
                    var edit_vigencia = callback[0].dataset.processos_workflow_ativos;
                    var edit_preco = callback[1].dataset.processos_workflow_ativos;
                    var array_modified = [];
                    array_modified.push(modifiedinRows_preco);
                    array_modified.push(modifiedinRows_dates);
                    if ((edit_vigencia.length && modifiedinRows_dates) && (edit_preco.length && modifiedinRows_preco)) {
                        self.solicitaCriacaoWorkflow(widget, "edit_itemFaturamento", widget.currentRow, array_modified);
                        widget.currentRow.DTINIVIGSE = widget.backupRow.DTINIVIGSE.slice(0, 10);
                        widget.currentRow.DTFIMVIGSE = widget.backupRow.DTFIMVIGSE.slice(0, 10);
                        widget.currentRow.VRPRESERV = widget.backupRow.VRPRESERV;
                        widget.currentRow.VRPRESERVD = widget.backupRow.VRPRESERVD;
                        salva_edicao_sem_workflow = false;
                        workflow_controle = false;
                    } else if (edit_vigencia.length && modifiedinRows_dates) {
                        //Gera Workflow para a alteração da vigência
                        self.solicitaCriacaoWorkflow(widget, "edit_itemFaturamento", widget.currentRow, array_modified);
                        widget.currentRow.DTINIVIGSE = widget.backupRow.DTINIVIGSE.slice(0, 10);
                        widget.currentRow.DTFIMVIGSE = widget.backupRow.DTFIMVIGSE.slice(0, 10);
                        workflow_controle = false;
                        salva_edicao_sem_workflow = true;
                    } else if (edit_preco.length && modifiedinRows_preco) {
                        self.solicitaCriacaoWorkflow(widget, "edit_itemFaturamento", widget.currentRow, array_modified);
                        widget.currentRow.VRPRESERV = widget.backupRow.VRPRESERV;
                        widget.currentRow.VRPRESERVD = widget.backupRow.VRPRESERVD;
                        salva_edicao_sem_workflow = true;
                        workflow_controle = false;
                    }
                    if (salva_edicao_sem_workflow && workflow_controle) {
                        widget.currentRow.VRPRESERVD = InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERVD);
                        widget.currentRow.VRPRESERV = InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERV);
                        widget.currentRow.VRPRESERV_BKP = currentItemDeFaturamento.VRPRESERV;
                        widget.currentRow.VRPRESERVD_BKP = currentItemDeFaturamento.VRPRESERVD;
                        TabelaPrecosService.saveServTab(widget.currentRow).then(function() {
                            if (workflow_controle) {
                                widget.view();
                                self.setTabsVisibility(widget);

                                widget.getField('CDSEQPURCHASEORDER').isVisible = true;
                                widget.fieldGroups.filter(function(fieldGroup) {
                                    return fieldGroup.name == "INTEGRACAO";
                                })[0].isVisible = true;

                                widget.getField("VRPRESERV").readOnly = true;
                                widget.getField("VRPRESERVD").readOnly = true;
                                widget.getField("DTINIVIGSE").readOnly = true;
                                widget.getField("DTFIMVIGSE").readOnly = true;
                                widget.container.getWidget("grid_subgrupos").isVisible = true;
                                widget.activate();
                                widget.fields[0].isVisible = false;
                                widget.fields[0].class = 0;
                                widget.fields[1].showOnForm = true;
                                widget.fields[1].class = 6;
                                widget.getField("SGCLASSE").showOnForm = true;
                                widget.getField("IDCICLO").showOnForm = true;
                            }
                        }, function(error) {
                            if (error.data) {
                                errorMessage = error.data.error;
                                trataErros(error.data.error);
                            }
                        });
                    }
                });
            }, function(error) {
                if (typeof error == "string")
                    trataErros(error);
                else
                    InflightorGeneral.callbackError(error);
            });
        }
    };
    this.validaData = function(dtIni, dtFin) {
        if (moment(dtIni, "DD/MM/YYYY").unix() > moment(dtFin, "DD/MM/YYYY").unix() || moment(dtFin, "DD/MM/YYYY").unix() < moment(dtIni, "DD/MM/YYYY").unix()) {
            MessageService.showMessageByCode("COC21000_PERIODO_VIGENCIA_INV");
            return false;
        } else {
            return true;
        }

    };

    this.filterServicos = function(widget) {
        var currentRow = widget.currentRow;
        var gridWidget = widget.container.getWidget("servicostabela");
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": gridWidget.dataSourceFilter[0].value
        }, {
            "name": "CDCIA",
            "operator": "=",
            "value": gridWidget.dataSourceFilter[1].value
        }, {
            "name": "NRTABPRECO",
            "operator": "=",
            "value": gridWidget.dataSourceFilter[2].value
        }];
        if (currentRow.IDSERVICO) {
            filter.push({
                "name": "IDSERVICO",
                "operator": "=",
                "value": currentRow.IDSERVICO
            });
        }
        if (currentRow.CDCLASSE) {
            filter.push({
                "name": "CDCLASSE",
                "operator": "=",
                "value": currentRow.CDCLASSE
            });
        }
        if (currentRow.CDCICLO) {
            filter.push({
                "name": "CDCICLO",
                "operator": "=",
                "value": currentRow.CDCICLO
            });
        }
        if (currentRow.IDTIPMANUAL) {
            filter.push({
                "name": "IDTIPMANUAL",
                "operator": "=",
                "value": currentRow.IDTIPMANUAL
            });
        }
        gridWidget.dataSourceFilter = filter;
        ScreenService.closePopup(true);
        gridWidget.reload().then(function() {
            gridWidget.activate();
        });
    };

    this.applyFilterField = function(widget) {
        var swipeWidget = widget.container.getWidget("servicostabela");
        var dataSourceFilter = swipeWidget.dataSourceFilter;
        var fieldClasse = widget.container.getWidget("filtrarServicos").getField("DSCLASSE");
        var fieldCiclo = widget.container.getWidget("filtrarServicos").getField("IDCICLO");

        fieldClasse.dataSourceFilter[0].value = dataSourceFilter[0].value;
        fieldCiclo.dataSourceFilter[0].value = dataSourceFilter[0].value;
        fieldClasse.dataSourceFilter[1].value = dataSourceFilter[1].value;
        fieldCiclo.dataSourceFilter[1].value = dataSourceFilter[1].value;
    };

    function mountRowPercent(parentWidget, currentWidget) {
        /*sjuliao*/
        currentWidget.currentRow.NMTABPRECO = parentWidget.currentRow.NMTABPRECO;
        currentWidget.currentRow.NRTABPRECO = parentWidget.currentRow.NRTABPRECO;
        currentWidget.currentRow.NMTABPRECO_BKP = parentWidget.currentRow.NMTABPRECO;
        currentWidget.currentRow.CDMANCIA_BKP = parentWidget.currentRow.CDMANCIA;
        currentWidget.currentRow.DSMANUAL_BKP = parentWidget.currentRow.DSMANUAL;
        currentWidget.currentRow.DSMANUAL = parentWidget.currentRow.DSMANUAL;
        currentWidget.currentRow.DTINIVIGTAB = parentWidget.currentRow.DTINIVIGTAB.substr(0, 10);
        currentWidget.currentRow.DTINIVIGTAB_BKP = parentWidget.currentRow.DTINIVIGTAB.substr(0, 10);
        currentWidget.currentRow.DTFIMVIGTAB = parentWidget.currentRow.DTFIMVIGTAB.substr(0, 10);
        currentWidget.currentRow.DTFIMVIGTAB_BKP = parentWidget.currentRow.DTFIMVIGTAB.substr(0, 10);
    }

    this.applyFilterPercent = function(widget) {
        var tabPrecosWidget = widget.container.getWidget("tabeladeprecos");
        var filterServicos = angular.copy(widget.container.getWidget("servicostabela").dataSourceFilter);
        var filterTodosServicos = [];
        var widgetPercentual = widget.container.getWidget("aumentaPercen");

        for (let i = 0; i < filterServicos.length; i++) {
            if (filterServicos[i].name == 'CDFILIAL' || filterServicos[i].name == 'CDCIA' || filterServicos[i].name == 'NRTABPRECO')
                filterTodosServicos.push(filterServicos[i]);
        }

        //mountRowPercent(tabPrecosWidget, widgetPercentual);
        var fieldServicos = widgetPercentual.getField("DSSERVICOCIAFILI");

        fieldServicos.dataSourceFilter = filterTodosServicos;
    };

    this.openFieldGroup = function(field) {
        let widget = field.widget;
        let widgetTabPreco = templateManager.container.getWidget('tabeladeprecos');
        if (field.value()) {
            widget.getField('NMTABPRECO').validations = {
                required: true
            };
            let dtProxDtIni = moment(widgetTabPreco.currentRow.DTFIMVIGTAB, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
            widget.getField('DTINIVIGTAB').setValue(dtProxDtIni);
            widget.getField('DTFIMVIGTAB').setValue(null);
            widget.getField('DTFIMVIGTAB').clearValue(widget.getField('DTFIMVIGTAB'));
            field.widget.fieldGroups[1].opened = true;
            setReadOnlyGroup(widget.fields, false);
        } else {
            widget.getField('NMTABPRECO').validations = {};
            widget.getField('DTINIVIGTAB').setValue(null);
            widget.getField('DTINIVIGTAB').clearValue(widget.getField('DTINIVIGTAB'));
            widget.getField('DTFIMVIGTAB').setValue(null);
            widget.getField('DTFIMVIGTAB').clearValue(widget.getField('DTFIMVIGTAB'));
            field.widget.fieldGroups[1].opened = false;
            setReadOnlyGroup(widget.fields, true);
        }
        templateManager.updateTemplate();
    };

    function setReadOnlyGroup(fields, value) {
        fields.forEach(function(field) {
            if (field.fieldGroup == "NOVATABELA") {
                field.readOnly = value;
                if (!value) {
                    if (field.name == "DSMANUAL") {
                        var filterCurrentRow = templateManager.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW;

                        var idvinculomanual = filterCurrentRow.IDVINCULOMANUAL;

                        if (idvinculomanual == "S") {

                            field.validations = {
                                required: true
                            };
                        }
                    } else
                        field.validations = {
                            required: true
                        };
                } else {
                    field.validations = {};
                }
            }
        });
    }

    this.aumentaPercentualTodos = function(fieldTodos) {
        var fieldServico = fieldTodos.widget.getField("DSSERVICOCIAFILI");
        var fieldGerarTabela = fieldTodos.widget.getField("GERARTAB");

        if (fieldTodos.checked) {
            fieldServico.readOnly = true;
            fieldServico.validations = {};
            fieldServico.setValue("");

            fieldGerarTabela.checked = true;
            fieldGerarTabela.setValue(true);
            fieldGerarTabela.readOnly = true;

            MessageService.showMessageByCode("COC21000_NECE_NOVA_TAB_AUMENTO_TODOS_ITENS");
        } else {
            fieldServico.readOnly = false;
            fieldServico.validations = {
                required: true
            };

            fieldGerarTabela.checked = false;
            fieldGerarTabela.setValue(false);
            fieldGerarTabela.readOnly = false;
        }
        this.openFieldGroup(fieldGerarTabela);
    };

    this.afterMoveRowPrecos = function(widget) {
        var subgrupoWidget = widget.container.getWidget("grid_subgrupos");

        if (subgrupoWidget.mode !== "list") {
            ScreenService.closeSwipe();
            subgrupoWidget.activate();
        }

        widget.container.getWidget("servicostabela").unSelectAll();
        widget.container.getWidget("servicostabela").activate();
        if (widget.currentRow.IDSERVICO !== "3" && widget.currentRow.IDSERVICO !== "5") {
            subgrupoWidget.isVisible = true;
        } else {
            subgrupoWidget.isVisible = false;
        }

        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        }, {
            "name": "CDCIA",
            "operator": "=",
            "value": widget.currentRow.CDCIA
        }, {
            "name": "NRTABPRECO",
            "operator": "=",
            "value": widget.currentRow.NRTABPRECO
        }, {
            "name": "NRSERVICOCIAFILI",
            "operator": "=",
            "value": widget.currentRow.NRSERVICOCIAFILI
        }, {
            "name": "CDCLASSE",
            "operator": "=",
            "value": widget.currentRow.CDCLASSE
        }, {
            "name": "CDCICLO",
            "operator": "=",
            "value": widget.currentRow.CDCICLO
        }, {
            "name": "NRSEQPRECO",
            "operator": "=",
            "value": widget.currentRow.NRSEQPRECO
        }];

        var parentWidget = widget.container.getWidget("precoProporcional");
        var propVariavelWidget = widget.container.getWidget("proporcaoVariavel");
        var subGruposWidget = widget.container.getWidget("grid_subgrupos");
        if (widget.currentWidget && widget.currentWidget.isVisible) {
            self.setTabsVisibility(widget);
            self.alteraProporcao(widget);
            propVariavelWidget.dataSourceFilter = filter;
            parentWidget.dataSourceFilter = filter;
            if (widget.currentWidget.name == "precoProporcional") {
                if (!parentWidget.currentRow.__is_new) {
                    widget.selectDetailOnMoveRow = false;
                    parentWidget.reload();
                }
            } else if (widget.currentWidget.name == "grid_subgrupos") {
                if (!parentWidget.currentRow.__is_new) {
                    self.filtraTabSubgrupos(subGruposWidget);
                }
            } else if (widget.currentWidget.name == "proporcaoVariavel") {
                if (!propVariavelWidget.isVisible) {
                    widget.selectDetailOnMoveRow = true;
                    widget.currentWidget = widget;
                    widget.activate();
                } else {
                    if (!propVariavelWidget.currentRow.__is_new)
                        propVariavelWidget.reload().then(function() {
                            if (propVariavelWidget.dataSource.data && propVariavelWidget.dataSource.data.length) {
                                propVariavelWidget.getAction("ZH_DELETE").isVisible = true;
                            } else {
                                propVariavelWidget.getAction("ZH_DELETE").isVisible = false;
                            }
                        });
                }
            }
        } else {
            parentWidget.dataSourceFilter = filter;
            if (!parentWidget.currentRow.__is_new)
                parentWidget.reload();

            propVariavelWidget.dataSourceFilter = filter;
            self.setTabsVisibility(widget);
            self.alteraProporcao(widget);
            if (!propVariavelWidget.currentRow.__is_new && propVariavelWidget.isVisible)
                propVariavelWidget.reload().then(function() {
                    if (propVariavelWidget.dataSource.data && propVariavelWidget.dataSource.data.length) {
                        propVariavelWidget.getAction("ZH_DELETE").isVisible = true;
                    } else {
                        propVariavelWidget.getAction("ZH_DELETE").isVisible = false;
                    }
                });
            if (!propVariavelWidget.isVisible) {
                if (widget.currentWidget) {
                    if (widget.currentWidget.name == "proporcaoVariavel" || widget.currentWidget.name == "grid_subgrupos") {
                        widget.selectDetailOnMoveRow = true;
                        widget.currentWidget = widget;
                        widget.activate();
                    }
                }
            } else
                widget.selectDetailOnMoveRow = false;
        }
    };

    this.filterPrecPromocional = function(widget) {
        var parentWidget = widget.container.getWidget("servicostabela");
        widget.actions[4].isVisible = false;
        widget.dataSourceFilter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": parentWidget.currentRow.CDFILIAL
        }, {
            "name": "CDCIA",
            "operator": "=",
            "value": parentWidget.currentRow.CDCIA
        }, {
            "name": "NRTABPRECO",
            "operator": "=",
            "value": parentWidget.currentRow.NRTABPRECO
        }, {
            "name": "NRSERVICOCIAFILI",
            "operator": "=",
            "value": parentWidget.currentRow.NRSERVICOCIAFILI
        }, {
            "name": "CDCLASSE",
            "operator": "=",
            "value": parentWidget.currentRow.CDCLASSE
        }, {
            "name": "CDCICLO",
            "operator": "=",
            "value": parentWidget.currentRow.CDCICLO
        }, {
            "name": "NRSEQPRECO",
            "operator": "=",
            "value": parentWidget.currentRow.NRSEQPRECO
        }];
        if (widget.mode != "edit")
            widget.reload();
    };

    //Proporção Variável
    this.filterPropVariavel = function(widget) {
        var parentWidget = widget.container.getWidget("servicostabela");
        widget.dataSourceFilter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": parentWidget.currentRow.CDFILIAL
        }, {
            "name": "CDCIA",
            "operator": "=",
            "value": parentWidget.currentRow.CDCIA
        }, {
            "name": "NRTABPRECO",
            "operator": "=",
            "value": parentWidget.currentRow.NRTABPRECO
        }, {
            "name": "NRSERVICOCIAFILI",
            "operator": "=",
            "value": parentWidget.currentRow.NRSERVICOCIAFILI
        }, {
            "name": "CDCLASSE",
            "operator": "=",
            "value": parentWidget.currentRow.CDCLASSE
        }, {
            "name": "CDCICLO",
            "operator": "=",
            "value": parentWidget.currentRow.CDCICLO
        }, {
            "name": "NRSEQPRECO",
            "operator": "=",
            "value": parentWidget.currentRow.NRSEQPRECO
        }];
        if (widget.mode != "edit")
            widget.reload();
    };

    this.adicionarPropVariavel = function(widget) {
        var popupAddProp = widget.widgets[0];
        popupAddProp.edit();
        popupAddProp.activate();
        popupAddProp.newRow();
        ScreenService.openPopup(popupAddProp);
        if (widget.dataSource.data && widget.dataSource.data.length > 0) {
            var ultRow = angular.copy(widget.dataSource.data[0]);
            widget.dataSource.data.forEach(function(row) {
                if (row.QTPAXDE > ultRow.QTPAXDE) ultRow = angular.copy(row);
            });
            popupAddProp.currentRow.QTPAXDE = parseInt(ultRow.QTPAXATE) + 1;
        } else {
            popupAddProp.currentRow.QTPAXDE = 1;
        }
    };

    this.excluiPropVariavel = function(widget) {
        MessageService.confirmMessageByCode("CONFIRM_DEL").then(function() {
            TabelaPrecosService.removePropVariavel(widget.dataSource.data[widget.dataSource.data.length - 1]).then(function() {
                widget.dataSource.data.pop();
                if (!widget.dataSource.data.length) {
                    widget.getAction("ZH_DELETE").isVisible = false;
                } else {
                    widget.getAction("ZH_DELETE").isVisible = true;
                }
                MessageService.showMessageByCode("DEL_SUCCESS");
            }, function(erro) {
                trataErros(erro.data.error);
            });

        });
    };

    this.salvaPropVariavel = function(widget) {
        var currentParent = widget.container.getWidget("servicostabela").currentRow;
        var propVariavelWidget = widget.container.getWidget("proporcaoVariavel");

        if (widget.isValid()) {
            if (widget.currentRow.QTPAXDE <= widget.currentRow.QTPAXATE) {
                if (widget.name == "popupAddProporcao") {
                    widget.currentRow.CDFILIAL = currentParent.CDFILIAL;
                    widget.currentRow.QTSERVICO = parseInt(widget.currentRow.QTSERVICO);
                    widget.currentRow.CDCIA = currentParent.CDCIA;
                    widget.currentRow.NRTABPRECO = currentParent.NRTABPRECO;
                    widget.currentRow.NRSERVICOCIAFILI = currentParent.NRSERVICOCIAFILI;
                    widget.currentRow.CDCLASSE = currentParent.CDCLASSE;
                    widget.currentRow.CDCICLO = currentParent.CDCICLO;
                    widget.currentRow.NRSEQPRECO = currentParent.NRSEQPRECO;
                    propVariavelWidget.dataSource.data.push(widget.currentRow);
                }
                propVariavelWidget.dataSource.save().then(function() {
                    if (widget.currentRow.__is_new) {
                        ScreenService.closePopup();
                        propVariavelWidget.reload();
                        propVariavelWidget.getAction("ZH_DELETE").isVisible = true;
                    } else {
                        propVariavelWidget.commitInlineEdit();
                    }
                    propVariavelWidget.list();
                    propVariavelWidget.activate();
                });
            } else {
                MessageService.showMessageByCode("COC21000_PAX_MIN_MENOR_PAX_MAX");
            }
        }
    };

    this.goBack = function(widget) {
        var backWidget;
        if (widget.mode == "list" && widget.getParent()) {
            backWidget = widget.getParent();
        } else {
            backWidget = widget;
        }

        return ScreenService.closeSwipe().then(function() {
            delete backWidget.currentWidget;
            backWidget.activate();
        });
    };

    this.cancelEdit = function(args) {
        var widget = args.owner.widget;
        if (!widget.currentRow.__is_new) {
            //Cancela Edição no modo InLineEdit
            widget.rollbackInlineEdit();
            widget.activate();
            widget.list();
        } else {
            //Cancela adição de um novo Preço Promocional
            InflightorGeneral.backOrCancel(args);
        }
    };

    this.alteraProporcao = function(widget) {
        if (widget.currentRow.IDPROPORCAO == "3" || widget.currentRow.IDPROPORCAO == "1") {
            widget.getField("QTPROPORCAO").readOnly = true;
            widget.getField("QTPROPORCAO").value("");
            widget.getField("QTPROPORCAO").validations = {};
        } else {
            widget.getField("QTPROPORCAO").readOnly = false;
            widget.getField("QTPROPORCAO").validations = {
                required: true
            };
        }
    };

    this.aumentaPercServ = function(widget) {
        /*juliao*/
        var currentRow = widget.currentRow;
        var tabelaPrecoRow = widget.container.getWidget("tabeladeprecos").currentRow;
        var tabelaPreco = {};

        var idvinculomanual = widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW.IDVINCULOMANUAL;

        if (idvinculomanual != "S") {

            if (widget.getField("DSMANUAL").validate) {
                delete(widget.getField("DSMANUAL").validate);
            }

        }

        if (widget.isValid()) {
            if (this.validaData(currentRow.DTINIVIGTAB, currentRow.DTFIMVIGTAB)) {
                tabelaPreco = {
                    CDFILIAL: tabelaPrecoRow.CDFILIAL,
                    CDCIA: tabelaPrecoRow.CDCIA,
                    NRTABPRECO: tabelaPrecoRow.NRTABPRECO,
                    CDMANCIA: widget.getField("DSMANUAL").dataSource.getCheckedRows(),
                    GERATABELA: currentRow.GERARTAB ? "S" : "N",
                    NMTABPRECO: currentRow.NMTABPRECO,
                    NRSERVICOCIAFILI: currentRow.TGRUPSERV ? "T" : "_" + currentRow.DSSERVICOCIAFILI.join("_") + "_",
                    VRPERCENTUAL: InflightorGeneral.valueToFloat(widget.currentRow.PERCENT),
                    DTINIVIGTAB: currentRow.DTINIVIGTAB,
                    DTFIMVIGTAB: currentRow.DTFIMVIGTAB
                };
                TabelaPrecosService.aumentaPorcent([tabelaPreco]).then(function(callback) {
                    if (callback.dataset !== undefined) {
                        MessageService.getMessageByCode("COC21000_IT_NAO_IMPORT").then(function(msg) {
                            InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros, ScreenService.i18n(msg));
                        });
                    } else {
                        ScreenService.closePopup();
                        MessageService.showMessageByCode("OPER_SUCCESS");
                        widget.container.getWidget("servicostabela").reload();
                        // if (currentRow.GERARTAB) {
                        //     var mainWidget = widget.container.getWidget("tabeladeprecos");
                        //     if (mainWidget.dataSourceFilter.length) {
                        //         mainWidget.dataSource.reload();
                        //     }
                        // }
                        // widget.container.getWidget("servicostabela").activate();
                    }
                }, function(error) {
                    if (error.data) {
                        if (!InflightorGeneral.mensagemErroProcedure(error.data.error))
                            trataErros(error.data.error);
                    }
                });
            }
        } else {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        }
    };

    function savePrecPromo(widget, __is_new) {
        var currentParent = widget.container.getWidget("servicostabela").currentRow;
        widget.currentRow.VRPRESERVPRO = InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERVPRO);
        widget.currentRow.VRPRESERVDPRO = InflightorGeneral.valueToFloat(widget.currentRow.VRPRESERVDPRO);
        widget.currentRow.CDFILIAL = currentParent.CDFILIAL;
        widget.currentRow.CDCIA = currentParent.CDCIA;
        widget.currentRow.NRTABPRECO = currentParent.NRTABPRECO;
        widget.currentRow.NRSERVICOCIAFILI = currentParent.NRSERVICOCIAFILI;
        widget.currentRow.CDCLASSE = currentParent.CDCLASSE;
        widget.currentRow.CDCICLO = currentParent.CDCICLO;
        widget.currentRow.QTPAXINICIAL = parseInt(widget.currentRow.QTPAXINICIAL);
        widget.currentRow.QTPAXFINAL = parseInt(widget.currentRow.QTPAXFINAL);
        widget.currentRow.NRSEQPRECO = currentParent.NRSEQPRECO;
        if (__is_new) {
            if (widget.currentRow.QTPAXINICIAL == 0 || widget.currentRow.QTPAXFINAL == 0) {
                MessageService.showMessageByCode("COC21000_QT_PASSAGEIROS_INVALIDA");
            } else {
                if (widget.currentRow.QTPAXINICIAL <= widget.currentRow.QTPAXFINAL) {
                    TabelaPrecosService.savePrecPromo(widget.currentRow).then(function() {
                        widget.reload().then(function() {
                            ScreenService.closeSwipe();
                            widget.activate();
                        });
                        widget.list();
                    }, function(error) {
                        if (error.data) {
                            trataErros(error.data.error);
                        }
                    });
                } else MessageService.showMessageByCode("COC21000_QTPAXFIN_MENOR_QTPAXINI");
            }
        } else {
            //edit in line
            var promises = [];
            var grid_data = widget.dataSource.data;
            grid_data.forEach(function(row) {
                if (row.changed_VRPRESERVPRO != null || row.changed_VRPRESERVDPRO != null) {
                    var row_alter = {
                        CDCIA: currentParent.CDCIA,
                        CDCICLO: currentParent.CDCICLO,
                        CDCLASSE: currentParent.CDCLASSE,
                        CDFILIAL: currentParent.CDFILIAL,
                        NRSEQPRECO: currentParent.NRSEQPRECO,
                        NRSERVICOCIAFILI: currentParent.NRSERVICOCIAFILI,
                        NRTABPRECO: currentParent.NRTABPRECO,
                        QTPAXFINAL: row.QTPAXFINAL,
                        QTPAXINICIAL: row.QTPAXINICIAL,
                        VRPRESERVDPRO: InflightorGeneral.valueToFloat(row.VRPRESERVDPRO),
                        VRPRESERVPRO: InflightorGeneral.valueToFloat(row.VRPRESERVPRO),
                        __is_new: row.__is_new
                    };
                    promises.push(InflightorGeneral.callRestEngine("coc#/crud/proprecosem/save", "Row", row_alter));
                }
            });
            ZHPromise.all(promises).then(function() {
                widget.commitInlineEdit();
                widget.activate();
                widget.list();
            }, function(error) {
                if (error.data) {
                    trataErros(error.data.error);
                }
            });
        }
    }
    this.salvaPrecPromocional = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                //Novo registro não passa por Workflow
                savePrecPromo(widget, true);
            } else {
                var currentParent = widget.container.getWidget("servicostabela").currentRow;
                permiteEdicao("NRTABPRECO=" + currentParent.NRTABPRECO + ";NRSERVICOCIAFILI=" + currentParent.NRSERVICOCIAFILI +
                    ";NRSEQPRECO=" + currentParent.NRSEQPRECO + ";CDCLASSE=" + currentParent.CDCLASSE + ";CDCICLO=" + currentParent.CDCICLO, "PRECOSECIAM").then(function(callback) {
                    if (callback) {
                        TabelaPrecosService.getProcessosWorkflow("3", "4").then(function(callback) {
                            if (callback.dataset.processos_workflow_ativos.length) {
                                var dataToSave = [];
                                widget.dataSource.data.forEach(function(row) {
                                    if ((row.changed_VRPRESERVPRO != null && row.changed_VRPRESERVPRO != row.VRPRESERVPRO) || (row.changed_VRPRESERVDPRO != null && row.changed_VRPRESERVDPRO != row.VRPRESERVDPRO)) {
                                        var row_alter = {
                                            CDCIA: currentParent.CDCIA,
                                            CDCICLO: currentParent.CDCICLO,
                                            CDCLASSE: currentParent.CDCLASSE,
                                            CDFILIAL: currentParent.CDFILIAL,
                                            NRORG: currentParent.NRORG,
                                            NRSEQPRECO: currentParent.NRSEQPRECO,
                                            NRSERVICOCIAFILI: currentParent.NRSERVICOCIAFILI,
                                            NRTABPRECO: currentParent.NRTABPRECO,
                                            QTPAXFINAL: row.QTPAXFINAL,
                                            QTPAXINICIAL: row.QTPAXINICIAL,
                                            VRPRESERVDPRO: InflightorGeneral.valueToFloat(row.VRPRESERVDPRO),
                                            VRPRESERVPRO: InflightorGeneral.valueToFloat(row.VRPRESERVPRO),
                                            __is_new: row.__is_new
                                        };
                                        dataToSave.push(row_alter);
                                    }
                                });
                                self.solicitaCriacaoWorkflow(widget, "edit_precoProporcional", dataToSave, null);
                            } else
                                //Processo de workflow desativado
                                savePrecPromo(widget, false);
                        });
                    } else {
                        MessageService.showMessageByCode("COC21000_IT_FAT_EM_EXCLUSAO");
                    }
                });
            }
        }
    };

    this.openPopupPercent = function(widget) {

        var currentRowTabPreco = widget.container.getWidget("tabeladeprecos").currentRow;
        let popupAumentaPercent = templateManager.container.getWidget("aumentaPercen");
        TabelaPrecosService.getItensFaturamento(currentRowTabPreco.CDFILIAL, currentRowTabPreco.CDCIA, currentRowTabPreco.NRTABPRECO).then(function(callback) {
            if (callback.dataset.precoseciam_servicociafili_servicogen_classe_ciclo.length) {
                popupAumentaPercent.newRow();
                self.applyFilterPercent(widget);
                popupAumentaPercent.getField("TGRUPSERV").setValue(false);
                popupAumentaPercent.getField("TGRUPSERV").checked = false;
                popupAumentaPercent.getField("TGRUPSERV").readOnly = false;
                self.aumentaPercentualTodos(popupAumentaPercent.getField("TGRUPSERV"));
                popupAumentaPercent.getField("GERARTAB").setValue(false);
                popupAumentaPercent.getField("GERARTAB").checked = false;
                self.openFieldGroup(popupAumentaPercent.getField("GERARTAB"));

                var filterCurrentRow = widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW;

                popupAumentaPercent.getField("DSMANUAL").dataSourceFilter = [
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": filterCurrentRow.CDFILIAL
                    },
                    {
                        "name": "CDCIA",
                        "operator": "=",
                        "value": filterCurrentRow.CDCIA
                    }
                ];

                ScreenService.openPopup(popupAumentaPercent).then(function() {
                    popupAumentaPercent.activate();
                });
            } else {
                MessageService.showMessageByCode("COC21000_N_HA_IT_TAB");
            }
        });
    };

    this.openPopupImport = function(widget) {
        var tabela_preco = widget.container.getWidget("tabeladeprecos");
        var popup_import = widget.container.getWidget("widgetImportTabela");
        popup_import.newRow();
        ScreenService.openPopup(popup_import).then(function() {
            popup_import.getField("DTINIVIGTAB").setValue(tabela_preco.getField("DTINIVIGTAB").value().slice(0, 10));
            popup_import.getField("DTFIMVIGTAB").setValue(tabela_preco.getField("DTFIMVIGTAB").value().slice(0, 10));
            popup_import.reload();
            popup_import.activate();
        });
    };

    this.validaPeriodo = function(widget) {
        var dtInicial = moment(widget.currentRow.DTINIVIGTAB, "DD/MM/YYYY");
        var dtFinal = moment(widget.currentRow.DTFIMVIGTAB, "DD/MM/YYYY");
        var dtIniTab = moment(widget.getParent().currentRow.DTINIVIGTAB, "DD/MM/YYYY hh:mm:ss");
        var dtFinTab = moment(widget.getParent().currentRow.DTFIMVIGTAB, "DD/MM/YYYY hh:mm:ss");
        if (dtInicial <= dtFinal) {
            if (dtInicial >= dtIniTab && dtFinal <= dtFinTab) {
                return true;
            } else {
                MessageService.showMessageByCode("COC21000_PERIODO_FORA_VIG_TAB");
                return false;
            }
        } else {
            MessageService.showMessageByCode("COC21000_DTFINVIG_MENOR_DTINIVIG");
            return false;
        }
    };

    this.uploadTabela = function(widget) {
        if (widget.isValid()) {
            var file = widget.currentRow.FileUpload;
            var servicosTab = widget.container.getWidget("servicostabela");
            if (file) {
                if (this.validaPeriodo(widget)) {
                    if (file.length) {
                        var filterRow = widget.container.getWidget("tabeladeprecos").appliedFilter.FILTER_ROW;
                        widget.currentRow.CDCIA = filterRow.CDCIA;
                        widget.currentRow.CDFILIAL = filterRow.CDFILIAL;
                        widget.currentRow.TABELA = widget.getParent().currentRow.NRTABPRECO;

                        return MessageService.getMessagesByCode(["YES", "NO", "CANCEL"]).then(function(messages) {
                            let config = {
                                type: "question",
                                icon: "question",
                                actions: [
                                    {
                                        label: messages[0],
                                        code: function() {
                                            self.callUploadMethod(file, "S", widget.currentRow, servicosTab);
                                        },
                                        "default": true
                                    },
                                    {
                                        label: messages[1],
                                        code: function() {
                                            self.callUploadMethod(file, "N", widget.currentRow, servicosTab);
                                        }
                                    },
                                    {
                                        label: messages[2]
                                    }
                                ]
                            };
                            return MessageService.showCustomDialogByCode("COC21000_PODEM_EXISTIR_PRECOS_MESMA_VIG", {}, config);
                        });

                    }
                }
            } else {
                MessageService.showMessageByCode("COC21000_ESCOLHA_ARQUIVO");
            }
        }
    };

    this.callUploadMethod = function(file, idSubs, currentRow, widget) {
        TabelaPrecosService.uploadCsvFile(file[0], idSubs, currentRow).then(
            function() {
                widget.reload().then(function() {
                    ScreenService.closePopup(true);
                    widget.activate();
                    MessageService.showMessageByCode("COC21000_IMPORT_IT_SUCCESS");
                });
            }
        ).catch(
            function(error) {
                errorMessage = "";

                if (error.data) {
                    errorMessage = error.data.error;
                    errorCode = error.data.errorCode;
                }

                if (errorCode == 800 && errorMessage.length) {
                    MessageService.getMessagesByCode(['COC21000_PLANO_VOO_EXISTENTE', 'COC21000_ERROS_NO_ARQUIVO', 'COC21000_IT_IMPORT_COM_PLANOVOO', 'COC21000_IT_IMPORT_VOO', 'COC21000_ERRO_CONTEUDO_ARQUIVO']).then(function(mensagens) {
                        let firstMessage = errorMessage[0];
                        let titulo = "";
                        if (firstMessage.indexOf('COC21000_IT_IMPORT_COM_PLANOVOO') >= 0 || firstMessage.indexOf('COC21000_IT_IMPORT_VOO') >= 0) {
                            titulo = ScreenService.i18n(mensagens[0]);
                        } else if (firstMessage.indexOf('COC21000_ERRO_CONTEUDO_ARQUIVO') >= 0) {
                            titulo = ScreenService.i18n(mensagens[1]);
                        }

                        for (let i = 0; i < errorMessage.length; i++) {
                            if (errorMessage[i].indexOf('COC21000_IT_IMPORT_COM_PLANOVOO') >= 0) {
                                let erro_params = errorMessage[i].split("#_#");
                                errorMessage[i] = ScreenService.i18n(mensagens[2], {
                                    DSSERVICOCIAFILI: erro_params[1],
                                    SGCLASSE: erro_params[2],
                                    IDCICLO: erro_params[3]
                                });
                            } else if (errorMessage[i].indexOf('COC21000_IT_IMPORT_VOO') >= 0) {
                                let erro_params = errorMessage[i].split("#_#");
                                errorMessage[i] = ScreenService.i18n(mensagens[3], {
                                    NRVOO: erro_params[1],
                                    DTHRPLAVOOPV: erro_params[2]
                                });
                            } else if (errorMessage[i].indexOf('COC21000_ERRO_CONTEUDO_ARQUIVO') >= 0) {
                                let erro_params = errorMessage[i].split("#_#");
                                errorMessage[i] = ScreenService.i18n(mensagens[4], {
                                    SUBGRUPO: erro_params[1],
                                    DSSERVICOCIAFILI: erro_params[2],
                                    SGCLASSE: erro_params[3],
                                    IDCICLO: erro_params[4],
                                    LINHA: erro_params[5]
                                });
                            }
                        }

                        widget.reload().then(function() {
                            InflightorGeneral.showErrorMessages(widget, errorMessage, titulo);
                        });

                    });
                } else {
                    trataErros(errorMessage);
                }
            }
        );
    };

    this.cancelarTabelaDePrecos = function(widget) {
        ScreenService.closeSwipe().then(function() {
            var widgetTabelaDePrecos = widget.container.getWidget("tabeladeprecos");
            widgetTabelaDePrecos.activate();
        });
    };

    this.cancelarAdicaoDoMenuGrid = function(widget) {
        ScreenService.closePopup();
        var widgetMenuGrids = widget.container.getWidget("widgetMenuGrids");
        widgetMenuGrids.activate();
    };

    this.salvarMenuGrid = function(widget) {

        if (widget.isValid()) {

            var dataSetToSave = [];

            widget.getField("DSMANUAL").dataSource.getCheckedRows().forEach(function(checkedRow) {
                checkedRow.__is_new = true;
                dataSetToSave.push(checkedRow);
            });

            InflightorGeneral.callRestEngine("coc#/salvarMenuGrid", "DataSet", dataSetToSave).then(function() {

                var widgetMenuGrids = widget.container.getWidget("widgetMenuGrids");
                widgetMenuGrids.reload().then(function() {
                    ScreenService.closePopup();
                    widgetMenuGrids.activate();
                });

            }, function(error) {
                if (typeof error.data.errorCode !== "undefined") {
                    trataErros(error.data.error);
                } else {
                    MessageService.showMessageByCode('ERRO_INESPERADO');
                }
            });

        }

    };

    this.adicionarMenuGrid = function(widget) {
        var widgetAdicionarMenuGrid = widget.container.getWidget("widgetAdicionarMenuGrid");
        var widgetTabelaDePrecos = widget.container.getWidget("tabeladeprecos");
        widgetAdicionarMenuGrid.getField("DSMANUAL").setValue("");
        widgetAdicionarMenuGrid.getField("DSMANUAL").dataSourceFilter = [
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.CDCIA
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.CDFILIAL
            },
            {
                "name": "NRTABPRECO",
                "operator": "=",
                "value": widgetTabelaDePrecos.currentRow.NRTABPRECO
            }
        ];
        widgetAdicionarMenuGrid.activate();
        ScreenService.openPopup(widgetAdicionarMenuGrid);
    };

    this.excluirMenuGridsTabelaDePrecos = function(widget) {

        var dataSetToSave = [];

        widget.dataSource.getCheckedRows().forEach(function(checkedRow) {
            checkedRow.__is_new = true;
            dataSetToSave.push(checkedRow);
        });

        InflightorGeneral.callRestEngine("coc#/excluirMenuGridsTabelaDePrecos", "DataSet", dataSetToSave).then(function() {

            widget.reload().then(function() {
                widget.activate();
                widget.unSelectAll();
            });

        }, function(error) {

            if (error.data && error.data.error) {
                if (Array.isArray(error.data.error)) {
                    let msg = "";
                    error.data.error.forEach(function(erro) {
                        msg += erro.DSMANUAL + " / ";
                    });
                    MessageService.showMessageByCode('COC21000_MENU_GRID_N_EXCL_VINC_ITENS', {
                        MENUS: msg
                    });

                    widget.reload().then(function() {
                        widget.activate();
                        widget.unSelectAll();
                    });
                } else {
                    trataErros(error.data.error);
                }
            } else {
                MessageService.showMessageByCode("ERRO_INESPERADO");
            }

        });

    };

    function getMessagesErroItem(mensagem, itens) {
        var errorMessages = [];
        itens.forEach(function(item, index) {
            if (index) {
                var msg = ScreenService.i18n(mensagem, item);
                errorMessages.push(msg);
            }
        });
        return errorMessages;
    }

    function validateRemoveItem(widget) {
        var deferred = ZHPromise.defer();
        var promises = [];
        if (widget.mode == "view") {
            searchPlanoVooGerados(widget).then(function() {
                deferred.resolve();
                return deferred.promise;
            }, function(error) {
                if (typeof error == "string") {
                    trataErros(error);
                    deferred.reject();
                } else {
                    InflightorGeneral.callbackError(error);
                    deferred.reject();
                }
                return deferred.promise;
            });
        } else {
            widget.dataSource.getCheckedRows().forEach(
                function(item) {
                    promises.push(InflightorGeneral.callRestEngine("coc#/crud/planodevoo_between_vigencia", "FilterData", getFilterPlanoVooVigencia(item, item)));
                }
            );
            ZHPromise.all(promises).then(function(callback) {
                callback.forEach(function(item, index) {
                    if (item.dataset && item.dataset.planodevoo_between_vigencia.length) {
                        widget.dataSource.getCheckedRows()[index].__isSelected = false;
                        widget.dataSource.getCheckedRows()[index].blocked = true;
                    }
                });
                deferred.resolve();
            });
        }
        return deferred.promise;
    }

    this.deleteItemFat = function(widget) {
        var row = widget.currentRow;
        row.NRPROCESSOAPROVA = "3";
        row.IDSITWORKFLOW = "1";
        row.NRITEMPROCAPROVA = null;
        row.IDTIPOPERWFLOW = null;
        row.DSPRIMARYKEY = "CDCIA=" + widget.currentRow.CDCIA + ";" + "CDFILIAL=" + widget.currentRow.CDFILIAL + ";" + "NRORG=" + widget.currentRow.NRORG + ";" + "NRTABPRECO=" + widget.currentRow.NRTABPRECO + ";" + "NRSERVICOCIAFILI=" + widget.currentRow.NRSERVICOCIAFILI + ";" + "NRSEQPRECO=" + widget.currentRow.NRSEQPRECO + ";" + "CDCLASSE=" + widget.currentRow.CDCLASSE + ";" + "CDCICLO=" + widget.currentRow.CDCICLO + ";";
        validateRemoveItem(widget).then(function() {
            coc03600_menu_designService.hasPendingWorkflow(row).then(function(callback) {
                if (callback) {
                    //Existem processos
                    MessageService.showMessageByCode("COC21000_IT_FAT_PROCESS_WORKFLOW");
                } else {
                    //Não Existem processos pendentes e permite Exclusão
                    TabelaPrecosService.getProcessosWorkflow("3", "6").then(function(callback) {
                        if (callback.dataset.processos_workflow_ativos.length) {
                            if (widget.mode != "view") {
                                MessageService.showMessageByCode('COC21000_EXCL_MULTI_COM_WORKFLOW');
                            } else {
                                self.solicitaCriacaoWorkflow(widget, "delete_itemFaturamento", widget.currentRow, null);
                            }
                        } else {
                            //Não passa por Workflow
                            if (widget.mode == "view") {

                                var filtro = [
                                    {
                                        "name": "CDCIA",
                                        "operator": "=",
                                        "value": row.CDCIA
                                    },
                                    {
                                        "name": "CDFILIAL",
                                        "operator": "=",
                                        "value": row.CDFILIAL
                                    },
                                    {
                                        "name": "NRSEQPRECO",
                                        "operator": "=",
                                        "value": row.NRSEQPRECO
                                    },
                                    {
                                        "name": "CDCICLO",
                                        "operator": "=",
                                        "value": row.CDCICLO
                                    },
                                    {
                                        "name": "CDCLASSE",
                                        "operator": "=",
                                        "value": row.CDCLASSE
                                    },
                                    {
                                        "name": "NRSERVICOCIAFILI",
                                        "operator": "=",
                                        "value": row.NRSERVICOCIAFILI
                                    },
                                    {
                                        "name": "NRTABPRECO",
                                        "operator": "=",
                                        "value": row.NRTABPRECO
                                    }
                                ];

                                InflightorGeneral.callRestEngine("coc#/crud/proprecosem", "FilterData", filtro).then(function(callback) {
                                    if (callback.dataset.proprecosem.length > 0) {
                                        MessageService.confirmMessageByCode("COC21000_IT_FAT_POSSUI_PRECO_PROP").then(function() {
                                            var dataSetToSave = [];
                                            dataSetToSave.push(widget.currentRow);
                                            InflightorGeneral.callRestEngine("coc#/deleteItemFaturamento", "DataSet", dataSetToSave).then(function() {
                                                widget.reload();
                                                ScreenService.closeSwipe();
                                            }, function(error) {
                                                if (error.data) {
                                                    MessageService.getMessagesByCode(['COC21000_IT_A_EXCL_POSSUI_PLANOVOO', 'COC21000_ERRO_AO_EXCLUIR']).then(function(msg) {
                                                        if (Array.isArray(error.data.error)) {
                                                            for (let i = 0; i < error.data.error.length; i++) {
                                                                error.data.error[i] = ScreenService.i18n(msg[0], error.data.error[i]);
                                                            }
                                                            InflightorGeneral.showErrorMessages(widget, error.data.error, ScreenService.i18n(msg[1]));
                                                        }
                                                        widget.reload();
                                                        widget.activate();
                                                        widget.unSelectAll();
                                                    });
                                                }
                                            });
                                        });
                                    } else {
                                        filtro = [
                                            {
                                                "name": "CDFILIAL",
                                                "operator": "=",
                                                "value": row.CDFILIAL
                                            },
                                            {
                                                "name": "CDCIA",
                                                "operator": "=",
                                                "value": row.CDCIA
                                            },
                                            {
                                                "name": "NRSERVICOCIAFILI",
                                                "operator": "=",
                                                "value": row.NRSERVICOCIAFILI
                                            },
                                            {
                                                "name": "CDCLASSE",
                                                "operator": "=",
                                                "value": row.CDCLASSE
                                            },
                                            {
                                                "name": "CDCICLO",
                                                "operator": "=",
                                                "value": row.CDCICLO
                                            },
                                            {
                                                "name": "CDMANCIA",
                                                "operator": "=",
                                                "value": row.CDMANCIA
                                            },
                                            {
                                                "name": "NRSEQPRECO",
                                                "operator": "=",
                                                "value": row.NRSEQPRECO
                                            },
                                            {
                                                "name": "NRTABPRECO",
                                                "operator": "=",
                                                "value": row.NRTABPRECO
                                            },
                                            {
                                                "name": "NRORG",
                                                "operator": "=",
                                                "value": row.NRORG
                                            }
                                        ];
                                        InflightorGeneral.callRestEngine("coc#/crud/subgrutabprecom_subgrupo_servico", "FilterData", filtro).then(function(callback) {
                                            if (callback.dataset.subgrutabprecom_subgrupo_servico.length > 0) {

                                                var dataSetToSave = [];
                                                dataSetToSave.push(widget.currentRow);
                                                InflightorGeneral.callRestEngine("coc#/deleteItemFaturamento", "DataSet", dataSetToSave).then(function() {
                                                    widget.reload();
                                                    ScreenService.closeSwipe();
                                                }, function(error) {
                                                    if (error.data) {
                                                        MessageService.getMessagesByCode(['COC21000_IT_A_EXCL_POSSUI_PLANOVOO', 'COC21000_ERRO_AO_EXCLUIR']).then(function(msg) {
                                                            if (Array.isArray(error.data.error)) {
                                                                for (let i = 0; i < error.data.error.length; i++) {
                                                                    error.data.error[i] = ScreenService.i18n(msg[0], error.data.error[i]);
                                                                }
                                                                InflightorGeneral.showErrorMessages(widget, error.data.error, ScreenService.i18n(msg[1]));
                                                            }
                                                            widget.reload();
                                                            widget.activate();
                                                            widget.unSelectAll();
                                                        });
                                                    }
                                                });
                                            } else {
                                                InflightorGeneral.deleteCrud(widget);
                                            }
                                        });
                                    }
                                });

                            } else {

                                MessageService.confirmMessageByCode("CONFIRM_DEL_SELECTED").then(
                                    function() {
                                        var dataSetToSave = [];
                                        var itensNaoExcluidos = [];

                                        widget.dataSource.getCheckedRows().forEach(function(checkedRow) {
                                            if (checkedRow.__isSelected) {
                                                checkedRow.__is_new = true;
                                                dataSetToSave.push(checkedRow);
                                            } else if (checkedRow.blocked) {
                                                itensNaoExcluidos.push(checkedRow);
                                            }
                                        });

                                        InflightorGeneral.callRestEngine("coc#/proprecosem_verifica_filhos", "DataSet", dataSetToSave).then(function(callback) {
                                            if (callback.dataset.registroscompreprop[0] > 0) {
                                                MessageService.confirmMessageByCode("COC21000_IT_COM_REGISTROS_FILHOS", {
                                                    ITENS: callback.dataset.registroscompreprop[0]
                                                }).then(function() {
                                                    InflightorGeneral.callRestEngine("coc#/deleteItemFaturamento", "DataSet", dataSetToSave).then(function() {
                                                        widget.reload().then(function() {
                                                            widget.activate();
                                                            widget.unSelectAll();
                                                            if (itensNaoExcluidos.length > 1) {
                                                                MessageService.getMessagesByCode(['COC21000_ITEM', 'COC21000_ITENS_N_EXCLUIDOS']).then(function(mensagens) {
                                                                    InflightorGeneral.showErrorMessages(widget, getMessagesErroItem(mensagens[0], itensNaoExcluidos), ScreenService.i18n(mensagens[1]));
                                                                });
                                                            }
                                                        });
                                                    }, function(error) {
                                                        if (error.data) {
                                                            MessageService.getMessagesByCode(['COC21000_IT_A_EXCL_POSSUI_PLANOVOO', 'COC21000_ERRO_AO_EXCLUIR']).then(function(msg) {
                                                                if (Array.isArray(error.data.error)) {
                                                                    for (let i = 0; i < error.data.error.length; i++) {
                                                                        error.data.error[i] = ScreenService.i18n(msg[0], error.data.error[i]);
                                                                    }
                                                                    InflightorGeneral.showErrorMessages(widget, error.data.error, ScreenService.i18n(msg[1]));
                                                                }
                                                                widget.reload();
                                                                widget.activate();
                                                                widget.unSelectAll();
                                                            });
                                                        }
                                                    });
                                                });
                                            } else {
                                                InflightorGeneral.callRestEngine("coc#/deleteItemFaturamento", "DataSet", dataSetToSave).then(function() {
                                                    widget.reload().then(function() {
                                                        widget.activate();
                                                        widget.unSelectAll();
                                                        if (itensNaoExcluidos.length > 1) {
                                                            MessageService.getMessagesByCode(['COC21000_ITEM', 'COC21000_ITENS_N_EXCLUIDOS']).then(function(mensagens) {
                                                                InflightorGeneral.showErrorMessages(widget, getMessagesErroItem(mensagens[0], itensNaoExcluidos), ScreenService.i18n(mensagens[1]));
                                                            });
                                                        }
                                                    });
                                                }, function(error) {
                                                    if (error.data) {
                                                        MessageService.getMessagesByCode(['COC21000_IT_A_EXCL_POSSUI_PLANOVOO', 'COC21000_ERRO_AO_EXCLUIR']).then(function(msg) {
                                                            if (Array.isArray(error.data.error)) {
                                                                for (let i = 0; i < error.data.error.length; i++) {
                                                                    error.data.error[i] = ScreenService.i18n(msg[0], error.data.error[i]);
                                                                }
                                                                InflightorGeneral.showErrorMessages(widget, error.data.error, ScreenService.i18n(msg[1]));
                                                            }
                                                            widget.reload();
                                                            widget.activate();
                                                            widget.unSelectAll();
                                                        });
                                                    }
                                                });
                                            }
                                        });

                                    });
                            }
                        }
                    });
                }
            });
        });
    };

    //Adição de metodos para controle do Workflow na Tabela de Preços
    function comparePrecos(newRow, backupRow) {
        var row = {};
        backupRow = {
            VRPRESERV: backupRow.VRPRESERV,
            VRPRESERVD: backupRow.VRPRESERVD
        };
        newRow = {
            VRPRESERV: newRow.VRPRESERV,
            VRPRESERVD: newRow.VRPRESERVD
        };
        if (angular.equals(newRow, backupRow))
            return false;
        else {
            for (let value in newRow) {
                if ((newRow[value] != backupRow[value]) && (value != "DTINIVIGSE" || value != "DTFIMVIGSE"))
                    if (value == "VRPRESERVD" || value == "VRPRESERV")
                        row[value] = InflightorGeneral.valueToFloat(newRow[value]);
                    else row[value] = newRow[value];
            }
            delete row.DTFIMVIGSE;
            delete row.DTINIVIGSE;
            delete row.changed;
            return !angular.equals({}, row) ? row : false;
        }
    }

    function compareDate(currentRow, backupRow, id_swipe) {
        var row = {};
        if (id_swipe == "TABPRECO") {
            if (currentRow.DTINIVIGTAB != backupRow.DTINIVIGTAB.slice(0, 10))
                row.DTINIVIGTAB = currentRow.DTINIVIGTAB;
            if (currentRow.DTFIMVIGTAB != backupRow.DTFIMVIGTAB.slice(0, 10))
                row.DTFIMVIGTAB = currentRow.DTFIMVIGTAB;
        } else {
            if (currentRow.DTINIVIGSE != backupRow.DTINIVIGSE.slice(0, 10))
                row.DTINIVIGSE = currentRow.DTINIVIGSE;
            if (currentRow.DTFIMVIGSE != backupRow.DTFIMVIGSE.slice(0, 10))
                row.DTFIMVIGSE = currentRow.DTFIMVIGSE;
        }
        return !angular.equals({}, row) ? row : false;
    }

    this.solicitaCriacaoWorkflow = function(widget, id_workflow, currentRow, modifiedinRows) {
        var motivoWidget = {
            "name": "motivoWidget",
            "label": "Justificativa",
            "template": "widget/form.html",
            "container": {},
            "parentWidget": widget,
            "id_workflow": id_workflow || null,
            "rowToSave": currentRow,
            "modifiedinRows": modifiedinRows || null,
            "isVisible": true,
            "popupNoBlock": false,
            "events": [],
            "fields": [
                {
                    "name": "DSMOTIVOACAO",
                    "label": "Motivo",
                    "template": "field/textarea.html",
                    "isVisible": true,
                    "readOnly": false,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    }
                },
                {
                    "name": "IDSITOPERWFLOW",
                    "label": "",
                    "template": "",
                    "isVisible": false,
                    "readOnly": false,
                    "showOnForm": true,
                }],
            "actions": [
                {
                    "name": "cancelar",
                    "label": "Cancelar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": "Confirmar",
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "coc21000_tabela_de_precos_controllers.validaMotivo(args.owner.widget);"
                        }
                    ]
                }
            ]
        };
        motivoWidget.container = widget.container;
        var motivoWidgetInitialized = metaDataFactory.widgetFactory(motivoWidget);
        ScreenService.openPopup(motivoWidgetInitialized);
    };

    var itens_workflow = {
        edit_tabpreco: {
            "function": solicitaAlterTabPreco,
            "IDTIPOPERWFLOW": "1",
            "NRITEMPROCAPROVA": "1"
        },
        edit_itemFaturamento: {
            "function": solicitaAlterItemFat,
            "IDTIPOPERWFLOW": "1",
            "NRITEMPROCAPROVA": ["3", "2"]
        },
        edit_precoProporcional: {
            "function": solicitaAlterPrecoPromo,
            "IDTIPOPERWFLOW": "1",
            "NRITEMPROCAPROVA": "4"
        },
        new_itemFaturamento: {
            "function": solicitaCriacaoItemFat,
            "IDTIPOPERWFLOW": "2",
            "NRITEMPROCAPROVA": "5"
        },
        delete_itemFaturamento: {
            "function": solicitaExclusaoItemFat,
            "IDTIPOPERWFLOW": "3",
            "NRITEMPROCAPROVA": "6"
        }
    };

    this.validaMotivo = function(widget) {
        if (widget.isValid()) {
            let itenWorkflowFunction = itens_workflow[widget.id_workflow].function;
            if (itenWorkflowFunction && typeof(itenWorkflowFunction) == "function") {
                return itenWorkflowFunction(widget, itens_workflow[widget.id_workflow].IDTIPOPERWFLOW, itens_workflow[widget.id_workflow].NRITEMPROCAPROVA);
            } else {
                return null;
            }
        }
    };

    function solicitaAlterTabPreco(widget, IDTIPOPERWFLOW, NRITEMPROCAPROVA) {
        widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        widget.rowToSave.IDTIPOPERWFLOW = IDTIPOPERWFLOW;
        widget.rowToSave.NRITEMPROCAPROVA = NRITEMPROCAPROVA;
        widget.rowToSave.NRPROCESSOAPROVA = "3";
        var data = [];
        data.push(widget.modifiedinRows);
        data.push(widget.rowToSave);
        InflightorGeneral.callRestEngine("coc#/tabpreco_workflow", "DataSet", data).then(function() {
            return MessageService.showMessageByCode("COC21000_SOL_ALT_TAB_GER", {
                NMTABPRECO: widget.parentWidget.currentRow.NMTABPRECO
            }).then(function() {
                ScreenService.closePopup();
                if (widget.parentWidget.currentRow.__is_new) {
                    widget.parentWidget.list(true);
                    ScreenService.goBack();
                    widget.parentWidget.reload();
                } else {
                    widget.parentWidget.currentRow.DTFIMVIGTAB = widget.parentWidget.backupRow.DTFIMVIGTAB.slice(0, 10);
                    widget.parentWidget.currentRow.DTINIVIGTAB = widget.parentWidget.backupRow.DTINIVIGTAB.slice(0, 10);
                    widget.parentWidget.reload();
                    ScreenService.goBack();
                }
                widget.parentWidget.activate();
                widget.parentWidget.getField("NMTABPRECO").class = 6;
                widget.parentWidget.getField("DTINIVIGTAB").validations = {};
                widget.parentWidget.getField("DTFIMVIGTAB").validations = {};
                widget.parentWidget.container.getWidget("servicostabela").isVisible = true;
            });
        }, function(error) {
            if (error.data) {
                trataErros(error.data.error);
            }
        });
    }

    function solicitaAlterItemFat(widget, IDTIPOPERWFLOW) {
        widget.rowToSave.DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        widget.rowToSave.IDTIPOPERWFLOW = IDTIPOPERWFLOW;
        if (widget.modifiedinRows[0] && !widget.modifiedinRows[1]) {
            widget.rowToSave.NRITEMPROCAPROVA = ["3"];
        } else if (!widget.modifiedinRows[0] && widget.modifiedinRows[1]) {
            widget.rowToSave.NRITEMPROCAPROVA = ["2"];
        } else if (widget.modifiedinRows[0] && widget.modifiedinRows[1]) {
            widget.rowToSave.NRITEMPROCAPROVA = ["3", "2"];
        } else {
            widget.rowToSave.NRITEMPROCAPROVA = [];
        }
        widget.rowToSave.NRPROCESSOAPROVA = "3";
        var data = [];
        var edit_rows = [];
        widget.modifiedinRows.forEach(function(row) {
            if (row)
                edit_rows.push(row);
        });
        data.push(edit_rows);
        data.push(widget.rowToSave);

        InflightorGeneral.callRestEngine("coc#/itemfat_workflow", "DataSet", data).then(function() {
            return MessageService.showMessageByCode("COC21000_SOL_ALT_IT_GER", {
                DSSERVICOCIAFILI: widget.parentWidget.currentRow.DSSERVICOCIAFILI
            }).then(function() {
                ScreenService.closePopup();
                ScreenService.goBack();

                widget.parentWidget.getField('CDSEQPURCHASEORDER').isVisible = true;
                widget.parentWidget.fieldGroups.filter(function(fieldGroup) {
                    return fieldGroup.name == "INTEGRACAO";
                })[0].isVisible = true;

                widget.parentWidget.getField("VRPRESERV").readOnly = true;
                widget.parentWidget.getField("VRPRESERVD").readOnly = true;
                widget.parentWidget.getField("DTINIVIGSE").readOnly = true;
                widget.parentWidget.getField("DTFIMVIGSE").readOnly = true;
                widget.parentWidget.activate();
                widget.parentWidget.fields[0].isVisible = false;
                widget.parentWidget.fields[0].class = 0;
                widget.parentWidget.fields[1].showOnForm = true;
                widget.parentWidget.fields[1].class = 6;
                widget.parentWidget.getField("SGCLASSE").showOnForm = true;
                widget.parentWidget.getField("IDCICLO").showOnForm = true;
                widget.parentWidget.widgets[1].isVisible = true;
            });
        }, function(error) {
            if (error.data) {
                trataErros(error.data.error);
            }
        });
    }

    function solicitaCriacaoItemFat(widget, IDTIPOPERWFLOW, NRITEMPROCAPROVA) {
        widget.rowToSave[0].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        widget.rowToSave[0].IDTIPOPERWFLOW = IDTIPOPERWFLOW;
        widget.rowToSave[0].NRITEMPROCAPROVA = NRITEMPROCAPROVA;
        widget.rowToSave[0].NRPROCESSOAPROVA = "3";
        InflightorGeneral.callRestEngine("coc#/itemfat_new_workflow", "DataSet", widget.rowToSave).then(function(callback) {
            widget.parentWidget.reload().then(function() {
                ScreenService.closePopup();
                widget.parentWidget.list();
                ScreenService.goBack();

                widget.parentWidget.getField('CDSEQPURCHASEORDER').isVisible = true;
                widget.parentWidget.fieldGroups.filter(function(fieldGroup) {
                    return fieldGroup.name == "INTEGRACAO";

                })[0].isVisible = true;

                widget.parentWidget.getField("VRPRESERV").readOnly = true;
                widget.parentWidget.getField("VRPRESERVD").readOnly = true;
                widget.parentWidget.getField("DTINIVIGSE").readOnly = true;
                widget.parentWidget.getField("DTFIMVIGSE").readOnly = true;
                widget.parentWidget.activate();
                widget.parentWidget.fields[0].isVisible = false;
                widget.parentWidget.fields[0].class = 0;
                widget.parentWidget.fields[1].showOnForm = true;
                widget.parentWidget.fields[1].class = 6;
                widget.parentWidget.getField("SGCLASSE").showOnForm = true;
                widget.parentWidget.getField("IDCICLO").showOnForm = true;
                widget.parentWidget.widgets[1].isVisible = true;
                trataErros(callback.messages[0].message);
            });
        }, function(error) {
            if (error.data) {
                trataErros(error.data.error);
            }
        });
    }

    function solicitaExclusaoItemFat(widget, IDTIPOPERWFLOW, NRITEMPROCAPROVA) {
        var data_item = [];
        data_item.push(widget.rowToSave);
        data_item[0].DSMOTIVOACAO = widget.currentRow.DSMOTIVOACAO;
        data_item[0].IDTIPOPERWFLOW = IDTIPOPERWFLOW;
        data_item[0].NRITEMPROCAPROVA = NRITEMPROCAPROVA;
        data_item[0].NRPROCESSOAPROVA = "3";
        data_item[0].NMCAMPOWFLOW = "NRSEQPRECO";
        InflightorGeneral.callRestEngine("coc#/itemfat_delete_workflow", "DataSet", data_item).then(function() {
            ScreenService.closePopup();
            widget.parentWidget.reload();
            ScreenService.closeSwipe().then(function() {
                MessageService.showMessageByCode("COC21000_SOL_EXC_IT_GER", {
                    DSSERVICOCIAFILI: widget.parentWidget.currentRow.DSSERVICOCIAFILI
                });
                widget.parentWidget.activate();
            });
        });
    }

    function solicitaAlterPrecoPromo(widget, IDTIPOPERWFLOW, NRITEMPROCAPROVA) {
        var row_workflow = {
            DSMOTIVOACAO: widget.currentRow.DSMOTIVOACAO,
            IDTIPOPERWFLOW: IDTIPOPERWFLOW,
            NRITEMPROCAPROVA: NRITEMPROCAPROVA,
            NRPROCESSOAPROVA: "3",
            NMTABPRECO: widget.container.getWidget("tabeladeprecos").currentRow.NMTABPRECO
        };
        widget.rowToSave.push(row_workflow);
        InflightorGeneral.callRestEngine("coc#/precoPromo_workflow", "DataSet", widget.rowToSave).then(function() {
            ScreenService.closePopup();
            widget.parentWidget.reload().then(function() {
                MessageService.showMessageByCode("COC21000_SOL_ALT_PRECO_PROP");
                widget.parentWidget.activate();
                widget.parentWidget.list();
            });
        });
    }

    function permiteEdicao(primaryKeys, tabela) {
        return TabelaPrecosService.getDadoswFlow(primaryKeys, tabela).then(function(callback) {
            var validate = true;
            var processos = callback.dataset.workflow_dadoswflow;
            if (processos.length) {
                processos.forEach(function(field) {
                    if ((field.NRITEMPROCAPROVA == 6) && field.IDSITWORKFLOW == 1)
                        validate = false;
                });
            }
            return validate;
        });
    }

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc21000_tabela_de_precos_controllers.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.deleteAllProp = function(widget) {
        var servicostabela = widget.container.getWidget("servicostabela");
        MessageService.confirmMessageByCode("CONFIRM_DEL_ALL").then(function() {
            var request = {
                route: "coc#/" + widget.name.toLowerCase() + "/remove_all",
                type: "Row",
                params: servicostabela.currentRow
            };
            InflightorGeneral.callRestEngine(request.route, request.type, request.params).then(function() {
                ScreenService.closePopup();
                widget.activate();
                widget.reload().then(function() {
                    MessageService.showMessageByCode("DEL_SUCCESS", "success");
                });
            }, InflightorGeneral.callbackError);
        });
    };

    this.carregaUrlFoto = function(widgetCiclo) {
        var dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDCIA
            },
            {
                name: "CDMANCIA",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDMANCIA
            },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.NRSERVICOCIAFILI
            },
            {
                name: "CDCLASSE",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDCLASSE
            },
            {
                name: "CDSUBGRU",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDSUBGRU
            },
            {
                name: "NRSEQSUBGRUSEMAM",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.NRSEQSUBGRUSEMAM
            },
            {
                name: "CDCICLO",
                operator: "=",
                value: widgetCiclo.getParent().currentRow.CDCICLO
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        widgetCiclo.dataSourceFilter = dataSourceFilter;
        widgetCiclo.reload().then(function() {
            var data = widgetCiclo.dataSource.data;
            if (data && data.length > 0) {
                let currentRow = data[0];
                currentRow.FGFOTOSUBSEMAM = currentRow.FGFOTOSUBSEMAM ? (window.filesPath + currentRow.FGFOTOSUBSEMAM) : window.undefinedImage;
                widgetCiclo.currentRow = currentRow;
            }
        });
    };

    this.chageFieldIndiceFinanceiro = function(field) {
        var widget = field.widget;
        var valueField = field.value();
        var fieldsReference = {
            VRPRESERVD: "VRPRESERV",
            VRPRESERV: "VRPRESERVD"
        };
        var fieldName = field.name;
        var otherFieldName = fieldsReference[fieldName];

        if (valueField) {
            valueField = InflightorGeneral.valueToFloat(valueField);
            InflightorGeneral.callRestEngine("coc#/crud/coc21000_movindice", "FilterData", []).then(
                function(callback) {
                    var indice = callback.dataset.coc21000_movindice;
                    if (indice.length) {
                        indice = indice[0].VRINDICE;
                        if (fieldName == "VRPRESERV") {
                            widget.currentRow[otherFieldName] = valueField / indice;
                        } else {
                            widget.currentRow[otherFieldName] = valueField * indice;
                        }
                    }
                },
                InflightorGeneral.callbackError
            );
        } else {
            widget.currentRow[otherFieldName] = "";
        }
    };
    this.backSubgrupo = function(args, widget) {
        InflightorGeneral.backOrCancel(args).then(function() {
            widget.container.getWidget("widgetServicos").activate();
        });
    };

    this.atualizaVinculoSubgrupo = function(widget) {
        var widgetItemFaturamento = widget.container.getWidget("servicostabela");
        MessageService.confirmMessageByCode("COC21000_CONFIRM_ATUALIZACAO_SUBGRUPOS").then(function() {
            if (widgetItemFaturamento.currentRow.CDMANCIA) {
                InflightorGeneral.callRestEngine("coc#/atualizaVinculoSubgrupo", "Row", widgetItemFaturamento.currentRow).then(function() {
                    widget.reload().then(function() {
                        MessageService.showMessageByCode("COC21000_SUBGRUPOS_ATUALIZADOS").then(function() {
                            widget.activate();
                        });
                    });
                }, function(error) {
                    if (error.data) {
                        trataErros(error.data.error);
                    }
                });
            } else {
                InflightorGeneral.callRestEngine("coc#/atribuiMenuGrid", "Row", widgetItemFaturamento.currentRow).then(function() {
                    widget.reload().then(function() {
                        MessageService.showMessageByCode("COC21000_SUBGRUPOS_ATUALIZADOS").then(function() {
                            widget.activate();
                        });
                    });
                }, function(error) {
                    if (error.data) {
                        trataErros(error.data.error);
                    }
                });
            }
        });
    };

    function getFilterPlanoVooVigencia(row, backupRow) {
        return [
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.CDCIA
            },
            {
                name: "NRSERVICOCIAFILI",
                operator: "=",
                value: row.NRSERVICOCIAFILI
            },
            {
                name: "DTINIHRPLAVOOPV",
                operator: "=",
                value: backupRow.DTINIVIGSE.slice(0, 10)
            },
            {
                name: "DTFIMHRPLAVOOPV",
                operator: "=",
                value: backupRow.DTFIMVIGSE.slice(0, 10)
            },
            {
                name: "CDCLASSE",
                operator: "=",
                value: row.CDCLASSE
            },
            {
                name: "CDCICLO",
                operator: "=",
                value: row.CDCICLO
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
    }

    function searchPlanoVooGerados(widget, changedDate) {
        var deferred = ZHPromise.defer();
        InflightorGeneral.callRestEngine("coc#/crud/planodevoo_between_vigencia", "FilterData", getFilterPlanoVooVigencia(widget.currentRow, widget.currentRow)).then(
            function(callback) {
                if (callback.dataset && callback.dataset.planodevoo_between_vigencia.length) {
                    //Encontrou planos de Voos já gerados para o intervalo informado
                    var lastDate = callback.dataset.planodevoo_between_vigencia[callback.dataset.planodevoo_between_vigencia.length - 1].DTHRPLAVOOPV;
                    var firstDate = callback.dataset.planodevoo_between_vigencia[0].DTHRPLAVOOPV;
                    if (changedDate && validDate(firstDate, lastDate, widget.currentRow)) {
                        //Caso apenas a data foi alterada e é maior que a data do ultimo plano de Voo gerado
                        //a edição é permitida.
                        deferred.resolve();
                    } else {
                        let msg = "COC21000_PLANO_VOO_JA_GERADO#_#" + lastDate.slice(0, 10);
                        deferred.reject(msg);
                    }
                } else {
                    deferred.resolve();
                }
                return deferred.promise;
            },
            function(error) {
                deferred.reject(error);
                return deferred.promise;
            }
        );
        return deferred.promise;
    }

    function validDate(firstDate, lastDate, currentRow) {
        if (moment(firstDate, "DD/MM/YYYY").unix() >= moment(currentRow.DTINIVIGSE, "DD/MM/YYYY").unix() && moment(lastDate, "DD/MM/YYYY").unix() <= moment(currentRow.DTFIMVIGSE, "DD/MM/YYYY").unix())
            return true;
        else
            return false;
    }

    this.validaFiltro = function(args) {
        let filter = args.e.preference.FILTER_ROW;
        if (filter.CIAATIVA !== 'S') {
            MessageService.showMessageByCode("INACTIVE_CIA");
            args.e.preventDefault();
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc21000_tabela_de_precos_controllers", coc21000_tabela_de_precos_controllers);
});

// FILE: ./assets/js/controllers/coc21001_preco_padrao_preco_proporcional_controller.js
var coc21001_preco_padrao_preco_proporcional_controller = function(InflightorGeneral, ScreenService, COCUtil, MessageService) {

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const fieldCia = widget.getField('NMCIA');
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        COCUtil.updateFilterFields(fieldCia, 'CDFILIAL', rowFilter);
        COCUtil.blockDependentField(fieldUnit, 'NMCIA');
        COCUtil.clearFields(widget, 'NMCIA');
        this.onChangeCia(fieldCia);
    };

    this.onChangeCia = function(fieldCia) {
        const widget = fieldCia.widget;
        const fieldTabela = widget.getField('NMTABPRECO');
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA'
        });
        const fieldAndCheckbox = {
            'DSSERVICOCIAFILI': 'TIDSERVICO',
            'SGCLASSE': 'TSGCLASSE',
            'IDCICLO': 'TIDCICLO'
        };

        const dependentFields = ['NMTABPRECO', 'DSSERVICOCIAFILI', 'SGCLASSE', 'IDCICLO'];
        COCUtil.updateFilterFields(dependentFields, ['CDFILIAL', 'CDCIA'], rowFilter, false, widget);
        COCUtil.blockDependentField(fieldCia, dependentFields);
        COCUtil.clearFields(widget, ['NMTABPRECO', 'DSSERVICOCIAFILI', 'SGCLASSE', 'IDCICLO'], fieldAndCheckbox);
        this.onChangeTabela(fieldTabela);
    };

    this.onChangeTabela = function(fieldTabela) {
        const widget = fieldTabela.widget;
        const rowFilter = COCUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL',
            NMCIA: 'CDCIA',
            NMTABPRECO: 'NRTABPRECO'
        });
        COCUtil.updateFilterFields('DSSERVICOCIAFILI', ['CDFILIAL', 'CDCIA', 'NRTABPRECO'], rowFilter, true, widget);
        COCUtil.blockDependentField(fieldTabela, 'DSSERVICOCIAFILI', true, false);
        COCUtil.clearFields(widget, 'DSSERVICOCIAFILI', {
            'DSSERVICOCIAFILI': 'TIDSERVICO'
        });
    };

    this.validaParametrosServico = function(args) {
        let filters = args.owner.field.dataSourceFilter;
        let isValid = filters.every(filter => filter.value);
        if (!isValid) {
            args.e.preventDefault();
            MessageService.showMessageByCode("MSG_COC_UNID_CIA_TABELA");
        }
    };

    this.validaParametrosClasseCiclo = function(args) {
        let filters = args.owner.field.dataSourceFilter;
        let isValid = filters.every(filter => filter.value);
        if (!isValid) {
            args.e.preventDefault();
            MessageService.showMessageByCode("MSG_COC_UNID_CIA");
        }
    };

    this.relatorio = function(widget, formato) {
        if (widget.isValid()) {
            var row = ajustaRowParaRelatorio(widget.currentRow, formato);
            InflightorGeneral.openReport("coc#/precopadraoprecoproporcional", row.REPORTNAME, row).then(function() {
                ScreenService.closePopup();
                widget.activate();
                widget.reload();
            }, function(error) {
                if (error.data) {
                    ScreenService.showMessage(error.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        }
    };

    var ajustaRowParaRelatorio = function(row, formato) {

        var newRow = angular.copy(row);

        newRow.REPORTFORMAT = formato;

        if (row.TIDSERVICO === "T") {
            newRow.NRSERVICOCIAFILI = "T";
        } else {
            newRow.NRSERVICOCIAFILI = row.DSSERVICOCIAFILI.join("_");
        }

        if (row.TSGCLASSE === "T") {
            newRow.CDCLASSE = "T";
        }

        if (row.TIDCICLO === "T") {
            newRow.CDCICLO = "T";
        }

        return newRow;

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc21001_preco_padrao_preco_proporcional_controller", coc21001_preco_padrao_preco_proporcional_controller);
});

// FILE: ./assets/js/controllers/coc21002_listagem_tabela_precos_controller.js
var coc21002_listagem_tabela_precos_controller = function(InflightorGeneral, ScreenService, MessageService) {

    var _fieldsByReport = {
        "COC21002": {
            "REPORTNAME": true,
            "IDVIGENCIA": true,
            "NMFILIAL": true,
            "TNMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "EXIB_PRECO_PROP": false
        },
        "COC21003": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "TNMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DSSTATUS": true,
            "DATAINI": true,
            "DATAFIN": true,
            "EXIB_PRECO_PROP": false
        },
        "COC21004": {
            "REPORTNAME": true,
            "S_NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DTREFERENCIA": true,
            "SOMENTE_VIGENTES": true,
            "NMTABPRECO": true,
            "TNMTABPRECO": true,
            "NRSERVICOCIAFILI": true,
            "TNRSERVICOCIAFILI": true,
            "IDCICLO": true,
            "TCICLO": true,
            "CDCLASSE": true,
            "TCLASSE": true,
            "EXIB_PRECO_PROP": false
        },
        "COC21006": {
            "REPORTNAME": true,
            "S_NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DTREFERENCIA": true,
            "SOMENTE_VIGENTES": true,
            "NMTABPRECO": true,
            "TNMTABPRECO": true,
            "NRSERVICOCIAFILI": true,
            "TNRSERVICOCIAFILI": true,
            "IDCICLO": true,
            "TCICLO": true,
            "CDCLASSE": true,
            "TCLASSE": true,
            "EXIB_PRECO_PROP": true
        },
        "COC21005": {
            "REPORTNAME": true,
            "S_NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "NMTABPRECO": true,
            "TNMTABPRECO": true,
            "SOMENTE_VIGENTES": true,
            "DTREFERENCIA": true,
            "NRSERVICOCIAFILI": true,
            "TNRSERVICOCIAFILI": true,
            "IDCICLO": true,
            "TCICLO": true,
            "CDCLASSE": true,
            "TCLASSE": true,
            "EXIB_PRECO_PROP": false
        }
    };

    function fieldScope(field) {
        // var field = $scope.field;
        var defaultBuildParams = {
            fields: field.fields,
            fieldsSeparator: "-",
            valuesSeparator: "_"
        };

        var buildParams = field.concatenatedValueString ? field.concatenatedValueString : defaultBuildParams;
        buildParams.fields = Util.isArray(buildParams.fields) ? buildParams.fields : defaultBuildParams.fields;
        buildParams.fieldsSeparator = !Util.isArray(buildParams.fieldsSeparator) ? buildParams.fieldsSeparator : defaultBuildParams.fieldsSeparator;
        buildParams.valuesSeparator = !Util.isArray(buildParams.valuesSeparator) ? buildParams.valuesSeparator : defaultBuildParams.valuesSeparator;

        field.valueStrBuildParams = buildParams;
        return buildParams;
    }

    function buildMultipleSelectValueStr(field, rows, buildParams) {
        var valuesArray = [];
        rows.forEach(function(item) {
            var itemArray = [];
            // buildParams.fields.forEach(function(field){
            itemArray.push(item[buildParams.fields[0].name]);
            // });
            var itemString = itemArray.join(buildParams.fieldsSeparator);
            valuesArray.push(itemString);
        });
        return valuesArray.length ? valuesArray.join(buildParams.valuesSeparator) : null;
    }

    function createFilter(field, buildParams) {
        if (buildParams && buildParams.fields && buildParams.fields.length) {
            var valueStr = field.value();

            if (valueStr != field.valueToAll) {
                var valuesSeparator = buildParams.valuesSeparator;
                var valueArray = typeof(valueStr) == "string" ? valueStr.split(valuesSeparator) : valueStr;
                var filterName = field.valueField;

                var filter = {
                    "name": filterName,
                    "operator": "IN",
                    "value": valueArray
                };
                return [filter];
            } else {
                return [];
            }
        } else {
            return [];
        }
    }

    this.montaFiltroField = function(field, fieldDependente) {
        var currentRow = field.widget.currentRow;
        if (currentRow.NMFILIAL.length > 0) {
            var filtro = [
                {
                    "name": "CDFILIAL",
                    "operator": "IN",
                    "value": currentRow.NMFILIAL
                }
            ];
            fieldDependente.dataSourceFilter = filtro;
        } else {
            fieldDependente.dataSourceFilter = [];
        }
    };

    function _validateAllSelect(widget) {
        widget.fields.forEach(function(field) {
            if (field.template.indexOf("checkbox") !== -1 && (field.value() == "S" || field.value() == "T")) {
                var dependentField = widget.fields.filter(function(fieldselect) {
                    return fieldselect.template.indexOf("field/select-multiple") !== -1 && fieldselect.name.indexOf(field.name.substr(1, field.name.length)) !== -1;
                });
                if (dependentField[0] && dependentField[0] !== undefined) {
                    dependentField[0].readOnly = true;
                    dependentField[0].validations = {};
                }
            }
        });
    }

    this.controlaVisibilidadePeloTipoRelatorio = function(field) {

        var report = field.widget.currentRow.REPORTNAME;
        field.widget.newRow();
        field.widget.fields.forEach(function(field) {
            if (_fieldsByReport[report][field.name])
                field.isVisible = true;
            else
                field.isVisible = false;
            field.applyDefaultValue();
        });
        _validateAllSelect(field.widget);
        field.widget.getField("REPORTNAME").setValue(report);
    };

    this.limpaFiltroFieldDependente = function(field, fieldDependente) {
        var currentRow = field.widget.currentRow;
        if (currentRow.TNMFILIAL == "T")
            fieldDependente.dataSourceFilter = [];
    };

    this.relatorio = function(widget, formato) {
        if (widget.isValid()) {
            if (isValidDates(widget.currentRow)) {
                MessageService.showMessageByCode("MSG_COC_PERIODO_INVALIDO");
            } else {
                var row = ajustaRowParaRelatorio(widget.currentRow, formato, widget);
                InflightorGeneral.openReport("coc#/listagemtabelaprecos", row.REPORTNAME, row).then(function() {
                    ScreenService.closePopup();
                    widget.activate();
                    widget.reload();
                }, function(error) {
                    if (error.data) {
                        ScreenService.showMessage(error.data.error);
                    } else {
                        MessageService.showMessageByCode("TIMEOUT");
                    }
                });
            }
        }
    };

    var isValidDates = function(currentRow) {
        var format = "DD/MM/YYYY";
        return moment(currentRow.DATAFIN, format).isBefore(moment(currentRow.DATAINI, format));
    };

    var ajustaRowParaRelatorio = function(row, formato, widget) {

        var newRow = angular.copy(row);

        newRow.REPORTFORMAT = formato;

        if (newRow.REPORTNAME == "COC21004" || newRow.REPORTNAME == "COC21005" || newRow.REPORTNAME == "COC21006") {
            if (!newRow.NRSERVICOCIAFILI.length)
                newRow.NRSERVICOCIAFILIPRECO = "T";
            else
                newRow.NRSERVICOCIAFILIPRECO = newRow.NRSERVICOCIAFILI.join("_");
            if (!newRow.CDCLASSE.length || newRow.TCLASSE == "T")
                newRow.CDCLASSE = "T";
            else
                newRow.CDCLASSE = newRow.CDCLASSE.join("_");
            if (!newRow.IDCICLO.length || newRow.TCICLO == "T")
                newRow.CDCICLO = "T";
            else
                newRow.CDCICLO = newRow.IDCICLO.join("_");
            if (!newRow.NMCIA.length)
                newRow.CDCIA = "T";
            else
                newRow.CDCIA = newRow.NMCIA.join("_");
            if (!newRow.NMTABPRECO.length)
                newRow.NRTABPRECO = "T";
            else {
                var nrtabpreco = "";
                widget.getField("NMTABPRECO").selectWidget.getCheckedRows().forEach(function(tabpreco) {
                    nrtabpreco += tabpreco.NRTABPRECO + "-" + tabpreco.CDCIA + "|";
                });
                newRow.NRTABPRECO = nrtabpreco;
            }
        } else {
            if (row.TNMFILIAL === "T") {
                newRow.CDFILIAL = "T";
            } else {
                newRow.CDFILIAL = row.NMFILIAL.join("_");
                newRow.CDFILIAL = "_" + newRow.CDFILIAL + "_";
            }

            if (row.TNMCIA === "T") {
                newRow.CDCIA = "T";
            } else {
                newRow.CDCIA = row.NMCIA.join("_");
                newRow.CDCIA = "_" + newRow.CDCIA + "_";
            }

            if (newRow.REPORTNAME == "COC21003") {
                newRow.STATUS = row.DSSTATUS.join("_");
                newRow.STATUS = "_" + newRow.STATUS + "_";
            }
        }

        return newRow;

    };

    var setFilterValues = function(currentRow, fieldForm) {
        var fieldFilter = fieldForm.dataSourceFilter;
        if (typeof fieldFilter != "undefined" && fieldFilter != null) {
            for (let i = 0; i < fieldFilter.length; i++) {
                if (currentRow.hasOwnProperty(fieldFilter[i].name)) {
                    fieldFilter[i].value = currentRow[fieldFilter[i].name];
                }
            }
        }
    };

    this.updateFilterDependents = function(field) {
        var widgetForm = field.widget;
        widgetForm.fields.forEach(function(formField) {
            setFilterValues(widgetForm.currentRow, formField);
        });
    };

    this.onChangeUnit = function(widgetForm) {
        widgetForm.getField("S_NMCIA").setValue("");
        setFilterValues(widgetForm, ["IDCICLO", "CDCLASSE"], "CDFILIAL");
        this.onChangeCia(widgetForm);
    };

    this.onChangeCia = function(widgetForm) {
        widgetForm.getField("NMTABPRECO").setValue("");
        this.onChangeTabPreco(widgetForm);
    };

    this.onChangeTabPreco = function(widgetForm) {
        widgetForm.getField("NRSERVICOCIAFILI").setValue([]);
        this.onChangeService(widgetForm);
    };

    this.onChangeService = function(widgetForm) {
        widgetForm.getField("CDCLASSE").setValue([]);
        this.onChangeClasse(widgetForm);
    };

    this.onChangeClasse = function(widgetForm) {
        widgetForm.getField("IDCICLO").setValue([]);
    };

    this.onChangeNMTABPRECO = function(widgetForm) {
        if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== []) {
            widgetForm.getField("NMTABPRECO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "CDCIA",
                    operator: "IN",
                    value: widgetForm.currentRow.NMCIA
            }];
            widgetForm.getField("NMTABPRECO").reload();
        } else {
            widgetForm.getField("NMTABPRECO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            }];
            widgetForm.getField("NMTABPRECO").reload();
        }
    };

    this.onChangeNRSERVICOCIAFILI = function(widgetForm) {
        var tabPreco;
        if (widgetForm.currentRow.NMTABPRECO !== "") {
            tabPreco = widgetForm.currentRow.NMTABPRECO.join("_");
        } else {
            tabPreco = "T";
        }

        if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== [] && widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("NRSERVICOCIAFILI").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "CDCIA",
                    operator: "IN",
                    value: widgetForm.currentRow.NMCIA
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            }];
            widgetForm.getField("NRSERVICOCIAFILI").reload();
        } else if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== []) {
            widgetForm.getField("NRSERVICOCIAFILI").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "CDCIA",
                    operator: "IN",
                    value: widgetForm.currentRow.NMCIA
            }];
            widgetForm.getField("NRSERVICOCIAFILI").reload();
        } else if (widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("NRSERVICOCIAFILI").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            }];
            widgetForm.getField("NRSERVICOCIAFILI").reload();
        } else {
            widgetForm.getField("NRSERVICOCIAFILI").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            }];
            widgetForm.getField("NRSERVICOCIAFILI").reload();
        }
    };

    this.filterAndReload = function(fieldToBeFiltered, filterFieldsName) {
        var widget = fieldToBeFiltered.widget;
        var filterFields = [];
        fieldToBeFiltered.dataSourceFilter = [];

        if (typeof(filterFieldsName) == "string") {
            filterFieldsName = [filterFieldsName];
        }

        filterFieldsName.forEach(function(filterFieldName) {
            filterFields.push(widget.getField(filterFieldName));
        });

        filterFields.forEach(function(filterField) {
            var buildParams = fieldScope(filterField);
            fieldToBeFiltered.multipleSelectValueStr = buildMultipleSelectValueStr(fieldToBeFiltered, filterField.dataSource.getCheckedRows(), buildParams);
            if (createFilter(filterField, buildParams)) {
                var currentFilter = fieldToBeFiltered.dataSourceFilter;
                var newFilter = createFilter(filterField, buildParams);

                if (newFilter[0].value[0]) {
                    fieldToBeFiltered.dataSourceFilter = currentFilter.concat(newFilter);
                }
            }
        });

        fieldToBeFiltered.dataSourceFilter.push({
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar("INFLIGHTOR_NRORG")
        }, {
            "name": "CDFILIAL",
            "operator": "=",
            "value": fieldToBeFiltered.widget.currentRow.CDFILIAL
        });

        fieldToBeFiltered.reload();
    };

    this.onChangeIDCICLO = function(widgetForm) {
        var tabPreco;
        if (widgetForm.currentRow.NMTABPRECO !== "") {
            tabPreco = widgetForm.currentRow.NMTABPRECO.join("_");
        } else {
            tabPreco = "T";
        }

        var companhia;
        if (widgetForm.currentRow.NMCIA !== "") {
            companhia = widgetForm.currentRow.NMCIA.join("_");
        } else {
            companhia = "T";
        }

        var servico;
        if (widgetForm.currentRow.NRSERVICOCIAFILI !== "") {
            servico = widgetForm.currentRow.NRSERVICOCIAFILI.join("_");
        } else {
            servico = "T";
        }

        if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== [] && widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("IDCICLO").reload();
        } else if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== []) {
            widgetForm.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("IDCICLO").reload();
        } else if (widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("IDCICLO").reload();
        } else {
            widgetForm.getField("IDCICLO").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("IDCICLO").reload();
        }
    };

    this.onChangeCDCLASSE = function(widgetForm) {
        var tabPreco;
        if (widgetForm.currentRow.NMTABPRECO !== "") {
            tabPreco = widgetForm.currentRow.NMTABPRECO.join("_");
        } else {
            tabPreco = "T";
        }

        var companhia;
        if (widgetForm.currentRow.NMCIA !== "") {
            companhia = widgetForm.currentRow.NMCIA.join("_");
        } else {
            companhia = "T";
        }

        var servico;
        if (widgetForm.currentRow.NRSERVICOCIAFILI !== "") {
            servico = widgetForm.currentRow.NRSERVICOCIAFILI.join("_");
        } else {
            servico = "T";
        }

        if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== [] && widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("CDCLASSE").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("CDCLASSE").reload();
        } else if (widgetForm.currentRow.NMCIA !== "" && widgetForm.currentRow.NMCIA !== []) {
            widgetForm.getField("CDCLASSE").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("CDCLASSE").reload();
        } else if (widgetForm.currentRow.NMTABPRECO !== "" && widgetForm.currentRow.NMTABPRECO !== []) {
            widgetForm.getField("CDCLASSE").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("CDCLASSE").reload();
        } else {
            widgetForm.getField("CDCLASSE").dataSourceFilter = [{
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: widgetForm.currentRow.CDFILIAL
            },
                {
                    name: "NRTABPRECO",
                    operator: "=",
                    value: tabPreco
            },
                {
                    name: "CDCIA",
                    operator: "=",
                    value: companhia
            },
                {
                    name: "NRSERVICOCIAFILI",
                    operator: "=",
                    value: servico
            }];
            widgetForm.getField("CDCLASSE").reload();
        }
    };

    this.substrDate = function(widget) {
        if (widget.dataSource.data.length) {
            widget.dataSource.data.forEach(function(row) {
                row.DTINIVIGTAB = row.DTINIVIGTAB.substr(0, 10);
                row.DTFIMVIGTAB = row.DTFIMVIGTAB.substr(0, 10);
            });
        }
    };

    var MAP_TFIELD = {
        "NMFILIAL": "TNMFILIAL",
        "NMCIA": "TNMCIA",
        "NMTABPRECO": "TNMTABPRECO",
        "NRSERVICOCIAFILI": "TNRSERVICOCIAFILI",
        "IDCICLO": "TCICLO",
        "CDCLASSE": "TCLASSE"
    };

    this.onChangeField = function(field, fieldsDependentes, fieldsDirRelacionados) {

        var widget = field.widget;
        if (fieldsDependentes.length > 0) {
            for (let i = 0; i < fieldsDependentes.length; i++) {

                widget.getField(fieldsDependentes[i]).readOnly = true;
                widget.getField(fieldsDependentes[i]).value('');

                if (typeof widget.getField(MAP_TFIELD[fieldsDependentes[i]]) !== "undefined") {

                    widget.getField(MAP_TFIELD[fieldsDependentes[i]]).value('T');
                    widget.currentRow[MAP_TFIELD[fieldsDependentes[i]]] = 'T';
                    widget.getField(fieldsDependentes[i]).validations = {};

                }

            }
        }

        for (let i = 0; i < fieldsDirRelacionados.length; i++) {
            if (field.value()) {
                if (typeof widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]) !== "undefined") {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = true;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).validations = {};
                    widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]).value('T');
                    widget.currentRow[MAP_TFIELD[fieldsDirRelacionados[i]]] = 'T';
                } else {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = false;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).reload();
                }
            } else {
                field.widget.getField(fieldsDirRelacionados[i]).value('');
                field.widget.getField(fieldsDirRelacionados[i]).readOnly = true;
            }
        }

    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc21002_listagem_tabela_precos_controller", coc21002_listagem_tabela_precos_controller);
});

// FILE: ./assets/js/controllers/coc21010_controle_de_precificacaoController.js
var coc21010_controle_de_precificacaoController = function(ScreenService, InflightorGeneral, MessageService) {

    var MAP_TFIELD = {
        "DSSERVICOCIAFILI": "TDSERVICO",
        "DSMANUAL": "TDSMANUAL",
        "SGCLASSE": "TDCLASSE",
        "IDCICLO": "TDCICLO"
    };

    this.onChangeField = function(field, fieldsDependentes, fieldsDirRelacionados) {

        var widget = field.widget;

        for (let i = 0; i < fieldsDependentes.length; i++) {

            widget.getField(fieldsDependentes[i]).readOnly = true;
            widget.getField(fieldsDependentes[i]).value('');

            if (typeof widget.getField(MAP_TFIELD[fieldsDependentes[i]]) !== "undefined") {

                widget.getField(MAP_TFIELD[fieldsDependentes[i]]).value('T');
                widget.currentRow[MAP_TFIELD[fieldsDependentes[i]]] = 'T';
                widget.getField(fieldsDependentes[i]).validations = {};

            }

        }
        for (let i = 0; i < fieldsDirRelacionados.length; i++) {
            if (field.value()) {
                if (typeof widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]) !== "undefined") {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = true;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).validations = {};
                    widget.getField(MAP_TFIELD[fieldsDirRelacionados[i]]).value('T');
                    widget.currentRow[MAP_TFIELD[fieldsDirRelacionados[i]]] = 'T';
                } else {
                    widget.getField(fieldsDirRelacionados[i]).readOnly = false;
                    widget.getField(fieldsDirRelacionados[i]).value('');
                    widget.getField(fieldsDirRelacionados[i]).reload();
                }
            } else {
                field.widget.getField(fieldsDirRelacionados[i]).value('');
                field.widget.getField(fieldsDirRelacionados[i]).readOnly = true;
            }
        }

    };

    this.setFilterForFilterFields = function(field) {
        if (field.name === "NMCIA") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }];
            field.reload();
        } else if (field.name === "DSMANUAL") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": field.widget.currentRow.CDCIA
            },
                {
                    "name": "IDSITUAPROPCIA",
                    "operator": "IN",
                    "value": [
                    "A",
                    "P"
                ]
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }];
            field.reload();
        } else if (field.name === "SGCLASSE" || field.name === "IDCICLO") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": field.widget.currentRow.CDCIA
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }];
            field.reload();
        } else if (field.name === "DSSERVICOCIAFILI") {
            field.dataSourceFilter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": field.widget.currentRow.CDFILIAL
            },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": field.widget.currentRow.CDCIA
            },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            },
                {
                    "name": "IDTIPOSERVICO",
                    "operator": "<>",
                    "value": "ITEMMENUGRID"
            }
            ];
            field.reload();
        }
    };

    this.export = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.CIAATIVA !== 'S') {
                MessageService.showMessageByCode("INACTIVE_CIA");
                return;
            }
            var serviceUrlModulo = window.serviceUrls.filter(function(row) {
                return row.package == "coc";
            })[0].baseUrl;
            var date = moment().format("DD_MM_YYYY");
            var fileName = "EXPORTACAO_ITEMFAT_" + widget.currentRow.CDFILIAL + "_" + widget.currentRow.CDCIA + "_" + date;

            formatParametrs(widget.currentRow);

            InflightorGeneral.callRestEngine("coc#/exportItemFat", "DataSet", [widget.currentRow]).then(function() {
                window.open(serviceUrlModulo + "/downloadExportItemFat?" + "&cdfilial=" + widget.currentRow.CDFILIAL + "&cdcia=" + widget.currentRow.CDCIA + "&cdmancia=" + widget.currentRow.CDMANCIA + "&nrorg=" + getLocalVar("INFLIGHTOR_NRORGTRAB") + "&dtinivg=" + widget.currentRow.DTINIVG + "&dtfimvg=" + widget.currentRow.DTFIMVG + "&nrservicociafili=" + widget.currentRow.NRSERVICOCIAFILI + "&cdclasse=" + widget.currentRow.CDCLASSE + "&cdciclo=" + widget.currentRow.CDCICLO + "&filename=" + fileName);
            }, function(error) {
                if (error.data) {
                    InflightorGeneral.showErrorMessages(widget, error.data.error, "Erros na exportação");
                }
            });
        }
    };

    function formatParametrs(currentRow) {
        currentRow.CDMANCIA = currentRow.DSMANUAL ? currentRow.DSMANUAL.join("_") : "T";
        currentRow.NRSERVICOCIAFILI = currentRow.DSSERVICOCIAFILI ? currentRow.DSSERVICOCIAFILI.join("_") : "T";
        currentRow.CDCLASSE = currentRow.SGCLASSE ? currentRow.SGCLASSE.join("_") : "T";
        currentRow.CDCICLO = currentRow.IDCICLO ? currentRow.IDCICLO.join("_") : "T";
    }

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc21010_controle_de_precificacaoController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

    this.changedDate = function(fieldChanged, fieldCompl, msg) {
        var momentChanged = moment(fieldChanged.value(), "DD/MM/YYYY");
        var momentCompl = moment(fieldCompl.value(), "DD/MM/YYYY");
        var showError = false;
        if (fieldCompl.value()) {
            if (fieldChanged.name === "DTINIVG") {
                if (momentChanged.unix() > momentCompl.unix())
                    showError = true;
            } else if (fieldChanged.name === "DTFIMVG") {
                if (momentChanged.unix() < momentCompl.unix())
                    showError = true;
            }
            if (showError) {
                fieldChanged.widget.currentRow[fieldChanged.name] = "";
                fieldChanged.setValue("");
                MessageService.showMessageByCode(msg, "error");
            }
        }
    };

    this.changeField = function(field, arraySonFields, firstSonField) {
        for (var index in arraySonFields) {
            var fieldSon = field.widget.getField(arraySonFields[index]);
            fieldSon.clearValue();
            // fieldSon.reload();
            fieldSon.readOnly = true;
        }
        if (field.value()) {
            field.widget.getField(firstSonField).readOnly = false;
            field.widget.getField(firstSonField).reload();
        } else {
            field.widget.getField(firstSonField).readOnly = true;
        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register("coc21010_controle_de_precificacaoController", coc21010_controle_de_precificacaoController);
});

// FILE: ./assets/js/controllers/coc23100_geracaocustomanual_controller.js
var coc23100_geracaocustomanual_controller = function(ScreenService, InflightorGeneral, coc23100_geracaocustomanualService, restEngine, COCUtil, templateManager) {

    this.onChangeUnitCia = function(field) {
        const widget = field.widget;
        const fieldNameAndCheckbox = {
            DSSERVICOCIAFILI: 'TNMSERVICO',
            DSSUBGRU: 'TDSSUBGRU',
            IDCICLO: 'TIDCICLO',
            IT: 'TNRITAIRVISION'
        };
        let arrayFields = ['NMFILIAL', 'NMCIA', 'DSMANUAL'];
        let childrenField = 'NMCIA';
        arrayFields = arrayFields.splice(arrayFields.indexOf(field.name) + 1);
        const fieldsToClear = arrayFields.concat(Object.keys(fieldNameAndCheckbox));
        let objFilter = {
            NMFILIAL: 'CDFILIAL'
        };
        if (field.name === 'NMCIA') {
            objFilter.NMCIA = 'CDCIA';
            childrenField = 'DSMANUAL';
        }

        COCUtil.blockDependentField(field, fieldsToClear);
        COCUtil.clearFields(widget, fieldsToClear, fieldNameAndCheckbox);
        const rowFilter = COCUtil.getValuesInRow(widget, objFilter);
        COCUtil.updateFilterFields(childrenField, Object.keys(rowFilter), rowFilter, false, widget);
    };

    this.carregaDatasReferencia = function(field) {
        var mes = (field.calendarWidget.getMonth() + 1 < 10) ? "0" + (field.calendarWidget.getMonth() + 1) : "" + (field.calendarWidget.getMonth() + 1);
        var ano = field.calendarWidget.getYear();
        var diaMesAno = "01/" + mes + "/" + ano;
        var dataInicialMes = moment(diaMesAno, "DD/MM/YYYY").startOf("month").format("DD/MM/YYYY");
        var dataFinalMes = moment(diaMesAno, "DD/MM/YYYY").endOf("month").format("DD/MM/YYYY");
        restEngine.post("coc#/crud/date_produtoap", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "DTINICIAL",
                    "operator": "=",
                    "value": dataInicialMes
                },
                {
                    "name": "DTFINAL",
                    "operator": "=",
                    "value": dataFinalMes
                }
            ]
        }).then(function(callback) {
            field.dataSource.data = callback.dataset.date_produtoap;
        });
    };

    this.limpaFieldMulti = function(widget, listaFieldName) {
        for (let fieldName in listaFieldName) {
            widget.getField(fieldName).setValue("");
            widget.getField(fieldName).readOnly = true;
            widget.getField(fieldName).validations = {};
            widget.getField(listaFieldName[fieldName]).checked = true;
            widget.getField(listaFieldName[fieldName]).setValue("T");
        }
    };

    //Quando o valor do field que cria os filtros mudam, limpa os campos filtrados
    this.clearField = function(field) {
        if (field.name === "NMFILIAL") {
            field.widget.getField("NMCIA").value("");
            field.widget.getField("DSMANUAL").value("");

            field.widget.getField("NMINDICE").value("");
            field.widget.getField("VRCUSTO").applyDefaultValue();

            this.limpaFieldMulti(field.widget, {
                "DSSERVICOCIAFILI": "TNMSERVICO",
                "DSSUBGRU": "TDSSUBGRU",
                "IDCICLO": "TIDCICLO"
            });

        } else if (field.name === "NMCIA") {
            field.widget.getField("DSMANUAL").value("");
            this.limpaFieldMulti(field.widget, {
                "DSSERVICOCIAFILI": "TNMSERVICO",
                "DSSUBGRU": "TDSSUBGRU",
                "IDCICLO": "TIDCICLO"
            });
            if (field.widget.currentRow.NMINDICE && field.widget.currentRow.DTCUSTOAP)
                this.getVrIndice(field.widget);

        } else if (field.name === "DSMANUAL") {
            this.limpaFieldMulti(field.widget, {
                "DSSERVICOCIAFILI": "TNMSERVICO",
                "DSSUBGRU": "TDSSUBGRU",
                "IDCICLO": "TIDCICLO"
            });

        } else if (field.name === "DSSERVICOCIAFILI" || field.name === "TNMSERVICO") {
            this.limpaFieldMulti(field.widget, {
                "DSSUBGRU": "TDSSUBGRU",
                "IDCICLO": "TIDCICLO"
            });

        } else if (field.name === "TDSSUBGRU" || field.name === "DSSUBGRU") {
            this.limpaFieldMulti(field.widget, {
                "IDCICLO": "TIDCICLO"
            });

        } else if (field.name === "IDSERVICO") {
            field.widget.getField("DSSERVICOCIAFILI").value("");
            field.widget.getField("DSSERVICOCIAFILI").isVisible = false;
            field.widget.getField("DSSUBGRU").value("");
            field.widget.getField("DSSUBGRU").isVisible = false;
        }
        templateManager.updateTemplate();
    };

    //Quando os campos que criam filtro estão zerados, os campos filtrados vão ser readonly
    this.blockField = function(field) {
        if (field.name === "NMFILIAL") {
            field.widget.getField("NMCIA").readOnly = field.value() === "" ? true : false;
            field.widget.getField("DSMANUAL").readOnly = field.value() === "" ? true : true;
            field.widget.getField("DSSERVICOCIAFILI").readOnly = field.value() === "" ? true : true;
            field.widget.getField("DSSUBGRU").readOnly = field.value() === "" ? true : true;
        } else if (field.name === "NMCIA") {
            field.widget.getField("DSMANUAL").readOnly = field.value() === "" ? true : false;
            field.widget.getField("DSSERVICOCIAFILI").readOnly = field.value() === "" ? true : true;
            field.widget.getField("DSSUBGRU").readOnly = field.value() === "" ? true : true;
        }
    };

    //Data sempre limpa a moeda selecionada
    this.clearIndice = function(field) {
        if (field.widget.currentRow.NMINDICE && field.widget.currentRow.DTCUSTOAP) {
            this.getVrIndice(field.widget);
        }
    };

    //Pega o valor da moeda e controla o readonly do valor
    this.getVrIndice = function(widget) {
        var cdindice = widget.currentRow.CDINDICE;
        var dtrefind = widget.currentRow.DTCUSTOAP;
        if (cdindice && dtrefind) {
            coc23100_geracaocustomanualService.checkCdIndice(cdindice).then(function(callback) {
                widget.getField("VRCUSTO").readOnly = callback.dataset.paramcomis.length > 0 ? true : false;
            });
            var vrindice = 0;
            coc23100_geracaocustomanualService.findVrIndice(cdindice, dtrefind).then(function(callback) {
                if (callback.dataset.movindice[0]) {
                    vrindice = callback.dataset.movindice[0].VRINDICE;
                    widget.getField("VRCUSTO").setValue(vrindice);
                } else {
                    widget.getField("VRCUSTO").setValue(0);
                }
            });
            widget.getField("VRCUSTO").reload();
        } else {
            widget.currentRow.DTCUSTOAP = 0;
        }
    };

    //Adicina os parametros no filtro para os serviços
    this.filterServico = function(field) {
        var filterRow = field.widget.currentRow;
        var filter = filterRow.IDSERVICO != "T" ? [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filterRow.CDFILIAL
                },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": filterRow.CDCIA
                },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": filterRow.CDMANCIA
                },
            {
                "name": "IDSERVICO",
                "operator": "=",
                "value": filterRow.IDSERVICO
                }
            ] : [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filterRow.CDFILIAL
                },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": filterRow.CDCIA
                },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": filterRow.CDMANCIA
                }
            ];
        field.dataSourceFilter = filter;
        field.reload();
    };

    //Adiciona o filtro para o Sub Grupos
    this.filterSubGru = function(field) {
        var filterRow = field.widget.currentRow;
        var filter = [];
        if (filterRow.TNMSERVICO === "T") {
            filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": filterRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": filterRow.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": filterRow.CDMANCIA
                }
            ];
        } else {
            var nrservicoclasse = [];
            if (filterRow.DSSERVICOCIAFILI) {
                filterRow.DSSERVICOCIAFILI.forEach(function(servico) {
                    nrservicoclasse.push(servico.NRSERVICOCLASSE);
                });
            }
            filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": filterRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": filterRow.CDCIA
                },
                {
                    "name": "CDMANCIA",
                    "operator": "=",
                    "value": filterRow.CDMANCIA
                },
                {
                    "name": "NRSERVICOCLASSE",
                    "operator": "IN",
                    "value": nrservicoclasse
                }
            ];
        }
        field.dataSourceFilter = filter;
        field.reload();
    };

    //Imprime o Relatorio
    this.report = function(widget, format) {
        var rowMontada = {};
        if (widget.isValid()) {
            switch (widget.currentRow.REPORT_NAME) {
                case "COC23100":
                    var servclasse = widget.currentRow.TNMSERVICO;
                    var subgrupo = widget.currentRow.TDSSUBGRU;
                    if (widget.currentRow.TNMSERVICO !== "T") {
                        servclasse = "";
                        widget.currentRow.DSSERVICOCIAFILI.forEach(function(row) {
                            servclasse += row.NRSERVICOCIAFILI + "-" + row.CDCLASSE + "_";
                        });
                    }
                    if (widget.currentRow.TDSSUBGRU !== "T") {
                        subgrupo = "";
                        widget.currentRow.DSSUBGRU.forEach(function(row) {
                            subgrupo += row.CDSUBGRU + "-" + row.NRSEQSUBGRUSEMAM + "_";
                        });
                    }
                    if (widget.currentRow.TIDCICLO != "T") {
                        widget.currentRow.CDCICLO = widget.currentRow.IDCICLO.join("_");
                    } else {
                        widget.currentRow.CDCICLO = "T";
                    }
                    if (widget.currentRow.TNRITAIRVISION != "T") {
                        var rowIt = [];
                        widget.currentRow.IT.forEach(function(row) {
                            rowIt.push(row.NRITAIRVISION + "|" + row.CDSUBGRU + "|" + row.NRSEQSUBGRUSEMAM + "|" + row.CDCICLO);
                        });
                        widget.currentRow.NRITAIRVISION = rowIt.join("_");
                    } else {
                        widget.currentRow.NRITAIRVISION = "T";
                    }

                    rowMontada.CDFILIAL = widget.currentRow.CDFILIAL;
                    rowMontada.CDMANCIA = widget.currentRow.CDMANCIA;
                    rowMontada.CDCIA = widget.currentRow.CDCIA;
                    rowMontada.SERVCLASSE = servclasse;
                    rowMontada.TIPOCUSTO = widget.currentRow.TIPOCUSTO;
                    rowMontada.IDIOMA = widget.currentRow.IDIOMA;
                    rowMontada.__locale = getLocale(widget.currentRow.IDIOMA);
                    rowMontada.CDINDICE = widget.currentRow.CDINDICE;
                    rowMontada.CDSUBGRU = subgrupo;
                    rowMontada.QTPAXREF = widget.currentRow.QTPAXREF;
                    rowMontada.DATACUSTO = widget.currentRow.DTCUSTOAP;
                    rowMontada.ONLYVIGSBG = widget.currentRow.ONLYVIGSBG;
                    rowMontada.INFLACAO = widget.currentRow.INFLACAO;
                    rowMontada.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                    rowMontada.REPORTNAME = widget.currentRow.REPORT_NAME;
                    rowMontada.REPORTTYPE = format;
                    rowMontada.CDCICLO = widget.currentRow.CDCICLO;
                    rowMontada.NRITAIRVISION = widget.currentRow.NRITAIRVISION;
                    InflightorGeneral.openReport("coc#/GeracaoCustoManual", widget.currentRow.REPORT_NAME, rowMontada);
                    break;
                default:
                    ScreenService.showMessage("Relatório Indisponível!");
                    break;
            }
            ScreenService.closePopup();
            widget.activate();
        }
    };

    var getLocale = function(idioma) {
        var locale = "";

        switch (idioma) {
            case "1":
                locale = "pt_BR";
                break;
            case "2":
                locale = "en_US";
                break;
            case "3":
                locale = "es";
                break;
            default:
                locale = "pt_BR";
                break;
        }

        return locale;
    };

    this.filtraServico = function(field) {
        var currentRow = field.widget.currentRow;

        var filtro = {
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.filtraIT = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "";
        var cdciclo = "";
        var dsservico = "";
        var cdclasse = "";

        var filtro = {
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        if (currentRow.TDSSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = "";
                field.widget.container.getWidget("coc23100_geracaocustomanual").getField("DSSUBGRU").selectWidget.getCheckedRows().forEach(function(subgru) {
                    cdsubgru += subgru.CDSUBGRU + "|" + subgru.NRSEQSUBGRUSEMAM + "_";
                });
            } else {
                cdsubgru = null;
            }
        }

        if (currentRow.TIDCICLO != "T") {
            if (currentRow.IDCICLO) {
                cdciclo = "";
                field.widget.container.getWidget("coc23100_geracaocustomanual").getField("IDCICLO").selectWidget.getCheckedRows().forEach(function(ciclo) {
                    cdciclo += ciclo.CDCICLO + "_";
                });
            } else {
                cdciclo = null;
            }
        }

        if (currentRow.TNMSERVICO != "T") {
            if (currentRow.DSSERVICOCIAFILI) {
                dsservico = "";
                cdclasse = "";
                field.widget.container.getWidget("coc23100_geracaocustomanual").getField("DSSERVICOCIAFILI").selectWidget.getCheckedRows().forEach(function(servico) {
                    dsservico += servico.NRSERVICOCIAFILI + "_";
                    cdclasse += servico.CDCLASSE + "_";
                });
            } else {
                dsservico = null;
                cdclasse = null;
            }
        }

        filtro.CDCICLO = cdciclo || "T";
        filtro.CDSUBGRU = cdsubgru || "T";
        filtro.NRSERVICOCIAFILI = dsservico || "T";
        filtro.CDCLASSE = cdclasse || "T";

        InflightorGeneral.onClickDependentField(field, filtro);
    };

    this.filtraCiclo = function(field) {
        var currentRow = field.widget.currentRow;
        var cdsubgru = "T";

        if (currentRow.TDSSUBGRU != "T") {
            if (currentRow.DSSUBGRU) {
                cdsubgru = "";
                currentRow.DSSUBGRU.forEach(function(subgru) {
                    cdsubgru += subgru.CDSUBGRU + "|" + subgru.NRSEQSUBGRUSEMAM + "_";
                });
            } else {
                cdsubgru = "";
            }
        }

        var filtro = {
            "CDSUBGRU": cdsubgru,
            "CDMANCIA": currentRow.CDMANCIA,
            "CDCIA": currentRow.CDCIA,
            "CDFILIAL": currentRow.CDFILIAL,
            "NRORG": getLocalVar("INFLIGHTOR_NRORGTRAB")
        };

        InflightorGeneral.onClickDependentField(field, filtro);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc23100_geracaocustomanual_controller", coc23100_geracaocustomanual_controller);
});

// FILE: ./assets/js/controllers/coc26000_cicloController.js
var coc26000_cicloController = function(ScreenService, InflightorGeneral, templateManager, Colors, ZHPromise, coc26000_cicloService) {

    this.setCicloColor = function(element, row) {
        if (row) {
            setTimeout(function() {
                element.find(".td:eq(1)").css("background-color", row.NRCORCICLO);
            }, 100);
        }
    };

    this.afterMoveRow = function(row) {
        setTimeout(function() {
            if (row.NRCORCICLO) {
                if (row.NRCORCICLO.toString().charAt(0) != "#")
                    row.NRCORCICLO = Colors.TColorToHex(row.NRCORCICLO);
                $("#CORCICLO").css("background-color", row.NRCORCICLO);
            }
        }, 500);
    };

    this.validaCiclo = function(widget) {
        if (widget.currentRow.__is_new) {
            var idciclo = widget.currentRow.IDCICLO;
            var nrorg = getLocalVar("INFLIGHTOR_NRORGTRAB");
            coc26000_cicloService.validaIdCiclo(idciclo, nrorg).then(function(callback) {
                if (callback.dataset.ciclo.length != 0) {
                    ScreenService.showMessage("Favor informar outro ciclo. Ciclo " + idciclo + " já cadastrado!");
                } else {
                    this.saveCiclo(widget);
                }
            }.bind(this));
        } else {
            this.saveCiclo(widget);
        }
    };

    this.saveCiclo = function(widget) {

        if (widget.isValid()) {
            var r, g, b;
            if (widget.currentRow.NRCORCICLO.toString().charAt(0) != "#") {
                r = widget.currentRow.NRCORCICLO._r.toString();
                g = widget.currentRow.NRCORCICLO._g.toString();
                b = widget.currentRow.NRCORCICLO._b.toString();
                if (r.indexOf(".") != -1) {
                    r = r.substr(0, r.indexOf("."));
                }
                if (g.indexOf(".") != -1) {
                    g = g.substr(0, g.indexOf("."));
                }
                if (b.indexOf(".") != -1) {
                    b = b.substr(0, b.indexOf("."));
                }
                r = parseInt(r);
                g = parseInt(g);
                b = parseInt(b);

                widget.currentRow.NRCORCICLO = r | (g << 8) | (b << 16);
            } else {
                var h = widget.currentRow.NRCORCICLO.substring(1, 7);
                r = parseInt((h).substring(0, 2), 16);
                g = parseInt((h).substring(2, 4), 16);
                b = parseInt((h).substring(4, 6), 16);
                widget.currentRow.NRCORCICLO = r | (g << 8) | (b << 16);
            }
            this.saveCrud(widget).then(function() {
                widget.activate();
                widget.reload();
            });
        }

    };

    this.saveCrud = function(widget) {
        var deferred = ZHPromise.defer();
        if (widget.isValid()) {
            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
                widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
            }

            widget.saveRow().then(function() {
                widget.list(true);
                ScreenService.goBack();
                widget.reload();
                widget.activate();
                deferred.resolve();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
                deferred.reject(errorMessage);
            });
        } else {
            // @todo create messages class
            deferred.reject("Preencha todos os campos corretamente!");
        }

        return deferred.promise;
    };

    this.RGBToTColor = function(r, g, b) {
        var Tcolor = r | (g << 8) | (b << 16);
        return Tcolor;
    };

    this.changeColor = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.NRCORCICLO.toString().charAt(0) != '#') {
                row.NRCORCICLO = Colors.TColorToHex(row.NRCORCICLO);
            }
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26000_cicloController", coc26000_cicloController);
});

// FILE: ./assets/js/controllers/coc26001_importacaoCustoController.js
function coc26001_importacaoCustoController(ScreenService, InflightorGeneral, coc26001_importacaoCustoService, templateManager, MessageService) {

    this.uploadCsvFile = function(file) {
        if (templateManager.container.getActiveWidget().isValid()) {
            if (file && file.length > 0) {
                MessageService.confirmMessageByCode('COC26001_AVISO_PROD_JA_CAD').then(function() {
                    callUploadMethod(file, "S");
                }).catch(function() {
                    callUploadMethod(file, "N");
                });

            } else {
                MessageService.showMessagebyCode("COC26001_INFORME_ARQUIVO");
            }
        }
    };

    var callUploadMethod = function(file, idSubs) {
        coc26001_importacaoCustoService.uploadCsvFile(file[0], idSubs).then(
            function(callback) {
                var data = callback.dataset;
                if (data && data.errorMessage) {
                    MessageService.getMessageByCode("COC26001_VERIFICAR_COD_PROD").then(function(msg) {
                        InflightorGeneral.showErrorMessages(templateManager.container.getActiveWidget(), data.errorMessage, ScreenService.i18n(msg));
                    });
                } else {
                    MessageService.showMessageByCode("COC26001_SUCCESS");
                }
            },
            function(error) {
                //var errorMessage = "Timeout";
                if (error.data) {
                    let errorMessage = error.data.error;
                    if (errorMessage.indexOf('COC26001_PRODUTOS_DUP_ARQ') >= 0) {
                        let produtos = errorMessage.split('#_#');
                        let produtosString = "";
                        produtos.splice(0, 1);
                        let msgProd = "";
                        MessageService.getMessageByCode('COC26001_PROD_DUPLICATAS').then(function(message) {
                            msgProd = message;
                            for (let i = 0; i < produtos.length; i++) {
                                let produto = produtos[i].split('@@@');
                                produtosString += ScreenService.i18n(msgProd, {
                                    PRODUTO: produto[0],
                                    DUPLICATAS: produto[1]
                                });
                            }
                            MessageService.showMessageByCode('COC26001_PRODUTOS_DUP_ARQ', {
                                PRODUTOS: produtosString
                            });
                        });
                    } else if (errorMessage.indexOf('COC26001_PROD_N_ENCONTRADOS') >= 0) {
                        let message = errorMessage.split('#_#');
                        MessageService.showMessageByCode(message[0], {
                            LINES: message[1]
                        });
                    } else if (errorMessage.indexOf('COC26001_') >= 0) {
                        MessageService.showMessageByCode(errorMessage);
                    } else {
                        console.log(error);
                    }
                } else {
                    console.error(error);
                }
            }
        );
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26001_importacaoCustoController", coc26001_importacaoCustoController);
});

// FILE: ./assets/js/controllers/coc26003_integracaoSAPController.js
var coc26003_integracaoSAPController = function(ScreenService, MessageService, InflightorGeneral, coc26003_integracaoSAPService, ZHPromise) {

    this.limpaField = function(field) {
        field.widget.currentRow.CDCICLO = "";
        field.widget.currentRow.DSCLASSE = "";
    };

    this.corrigeFiltro = function(widget) {
        var filtros = widget.dataSourceFilter;

        filtros.forEach(function(filtro) {
            if (filtro.name === "CDINTEGRASAP") {
                filtro.operator = filtro.value.replace(new RegExp("%", "g"), "");
                delete filtro.value;
            }
        });
    };

    this.geraCodigoSAP = function(widget) {
        var rowsSelecionadas = widget.getCheckedRows();
        var promises = [];
        var objSelecionados = {};
        var prefixoSAP;

        if (!widget.dataSource.data[0].CDPREFIXOSAP) {
            ScreenService.showMessage("Não existe prefixo cadastrado para a Companhia!");
            widget.unSelectAll();
        } else {
            for (let j = 0; j < rowsSelecionadas.length; j++) {
                prefixoSAP = rowsSelecionadas[j].CDPREFIXOSAP + rowsSelecionadas[j].CDPREFIXOSAPCLASSE;
                if (!objSelecionados.hasOwnProperty(prefixoSAP)) {
                    objSelecionados[prefixoSAP] = [];
                }
                objSelecionados[prefixoSAP].push(rowsSelecionadas[j]);
            }
            for (let prefixoSAP in objSelecionados) {
                promises.push(geraCodigoSAPCurrentRow(objSelecionados[prefixoSAP], prefixoSAP));
            }

            ZHPromise.all(promises).then(function() {
                widget.dataSource.save().then(function() {
                    widget.activate();
                    widget.reload();
                });
            });
        }
    };

    function geraCodigoSAPCurrentRow(rowsSelecionadas, prefixoSAP) {
        var precisaGerarCodigo = rowsSelecionadas.filter(function(rowSelecionada) {
            return !rowSelecionada.CDINTEGRASAP;
        });

        if (precisaGerarCodigo.length) {

            var filter = [
                {
                    "name": "PREFIXO",
                    "operator": "=",
                    "value": prefixoSAP
                }
            ];

            return coc26003_integracaoSAPService.retornaCodigoMaximoIntegracao(filter).then(function(callback) {

                if (callback.dataset.codigo_maximo_integracao.length) {

                    var maiorSAP = callback.dataset.codigo_maximo_integracao[0].MAXSEQ;

                    maiorSAP = parseInt(maiorSAP);
                    maiorSAP++;

                    for (let j = 0; j < rowsSelecionadas.length; j++) {

                        if (!rowsSelecionadas[j].CDINTEGRASAP) {

                            var maiorSAPString = maiorSAP.toString();
                            var fill = "0000";
                            var sequencial = fill.substring(0, fill.length - maiorSAPString.length) + maiorSAPString;
                            var novoSAP = prefixoSAP + sequencial;

                            rowsSelecionadas[j].CDINTEGRASAP = novoSAP;
                            rowsSelecionadas[j].changed = true;
                            maiorSAP++;

                        }

                    }

                    return rowsSelecionadas;

                }

            });
        }
    }

    this.limpaCodigoSAP = function(widget) {

        var SAPvazio = false;
        var rowsSelecionadas = widget.getCheckedRows();

        for (let j = 0; j <= rowsSelecionadas.length - 1; j++) {
            widget.currentRow = rowsSelecionadas[j];
            if (widget.currentRow.CDINTEGRASAP == "" || widget.currentRow.CDINTEGRASAP == undefined || widget.currentRow.CDINTEGRASAP == null) {
                SAPvazio = true;
            }

            widget.currentRow.__is_new = false;
            widget.currentRow.changed = true;
            widget.currentRow.CDINTEGRASAP = null;
        }

        if (SAPvazio == true) {
            ScreenService.showMessage("Não é possível limpar Código SAP inexistente!");
            widget.unSelectAll();
            widget.activate();
            widget.reload();
        } else {

            InflightorGeneral.saveCrud(widget, null, false).then(function() {
                widget.activate();
                widget.reload();
            });
        }
    };

    this.unidadeToCompanhia = function(field) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        field.widget.getField("NMCIA").dataSourceFilter = filter;
        field.widget.getField("NMCIA").setValue("");
    };

    this.applyInactiveStyle = function(field) {
        field.selectWidget.styles = [
            {
                "condition": "row.IDATIVO == 'N'",
                "cssClass": "coc-inactive"
            }
        ];
        field.selectWidget.events[0].code = "coc26003_integracaoSAPController.inactiveBranchWarning(args);";
    };

    this.inactiveBranchWarning = function(args) {
        if (args.owner.currentRow.IDATIVO === "N") {
            MessageService.showMessageByCode("INACTIVE_CIA");
        } else {
            args.owner.setSelected(args);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26003_integracaoSAPController", coc26003_integracaoSAPController);
});

// FILE: ./assets/js/controllers/coc26004_exportacaoSAPController.js
var coc26004_exportacaoSAPController = function(InflightorGeneral, ScreenService, templateManager, coc26004_exportacaoSAPService, MessageService) {

    var self = this;

    var FILTROCLEAR = [
        {
            name: "NRORG",
            operator: "=",
            value: "-1"
        }
    ];

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");
        args.owner.currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
    };

    this.limpaField = function(field) {

        if (field.name == "DTFINAL") {
            field.widget.currentRow.NMCIA = "";
            field.widget.currentRow.DESCRIPTION = "";
        } else if (field.name == "DTINICIAL") {
            field.widget.currentRow.NMCIA = "";
            field.widget.currentRow.DESCRIPTION = "";
        } else if (field.name == "NMFILIAL") {
            field.widget.currentRow.NMCIA = "";
            field.widget.currentRow.DESCRIPTION = "";
        }

    };

    this.unidadeToCompanhia = function(field) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": field.widget.currentRow.DTINICIAL
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": field.widget.currentRow.DTFINAL
            },
            {
                "name": "IDDELIEXPSAP",
                "operator": "=",
                "value": field.widget.currentRow.EXPORTNAME == "01" ? "N" : "S"
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        if (!field.widget.currentRow.CDFILIAL) {
            field.dataSourceFilter = FILTROCLEAR;
            field.reload();
            MessageService.showMessageByCode('COC26004_ESCOLHA_FILIAL');
            return;
        }

        field.widget.getField("NMCIA").dataSourceFilter = filter;
        field.widget.getField("NMCIA").reload();
    };

    this.companhiaToVooTrecho = function(field) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": field.widget.currentRow.CDCIA
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": field.widget.currentRow.DTINICIAL
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": field.widget.currentRow.DTFINAL
            },
            {
                "name": "IDDELIEXPSAP",
                "operator": "=",
                "value": field.widget.currentRow.EXPORTNAME == "01" ? "N" : "S"
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar("INFLIGHTOR_NRORGTRAB")
            }
        ];
        if (!field.widget.currentRow.CDCIA) {
            field.dataSourceFilter = FILTROCLEAR;
            field.reload();
            MessageService.showMessageByCode("COC26004_ESCOLHA_CIA");
            return;
        }

        field.widget.getField("DESCRIPTION").dataSourceFilter = filter;
        field.widget.getField("DESCRIPTION").reload();
    };

    var validaDias = function(widget) {
        var dataInicial = widget.currentRow.DTINICIAL;
        dataInicial = moment(dataInicial, "DD/MM/YYYY");
        widget.getField("DTFINAL").minDate = moment(dataInicial, "DD/MM/YYYY");
    };

    this.habilitaFieldData = function(widget) {
        widget.currentRow.DTFINAL = "";
        widget.getField("DTFINAL").readOnly = true;
        widget.getField("DTFINAL").reload();

        if (widget.currentRow.DTINICIAL) {
            widget.getField("DTFINAL").readOnly = false;
            validaDias(widget);
        }
    };

    this.fieldOnChangeTipo = function(widget) {
        var sgcia = widget.getField("SGCIA");
        var ciaField = widget.getField("NMCIA");

        /*
        widget.fields.forEach(function(field) {
            if (field.template && field.template.indexOf("calendar.html") !== -1) {
                field.setValue("");
            } else if (field.name !== "EXPORTNAME") {
                field.clearValue();
            }
            if (field.defaultValue && field.name !== "EXPORTNAME") {
                field.applyDefaultValue();
                if (field.template && field.template.indexOf("checkbox.html") !== -1) {
                    var boolChecked = false;
                    if (field.value() == field.valueToTrue) {
                        boolChecked = true;
                    }
                    field.checked = boolChecked;
                    if (field.parentField) {
                        InflightorGeneral.toggleReadOnlySelect(field, widget.getField(field.parentField));
                    }
                }
            }

        });
        
        */

        if (widget.currentRow.EXPORTNAME == "01") {
            ciaField.validations = {
                required: true
            };
            ciaField.isVisible = true;
            widget.getField("DTINICIAL").isVisible = true;
            widget.getField("DTFINAL").isVisible = true;
            widget.getField("DESCRIPTION").isVisible = true;
            widget.getField("TVOOTRECHO").isVisible = true;

            sgcia.isVisible = false;
            sgcia.validations = {};
            widget.getField("ALL_CIA").isVisible = false;
            widget.getField("DTULTATU").isVisible = false;
            widget.getField("DSSERVICOCIAFILI").isVisible = false;
            widget.getField("ALL_SERVICO").isVisible = false;

            let fC = widget.getField("NMCIA");
            fC.setValue(null);
            self.changeCia(fC);

        } else if (widget.currentRow.EXPORTNAME == "02") {
            ciaField.isVisible = false;
            ciaField.validations = {};
            widget.getField("DTINICIAL").isVisible = false;
            widget.getField("DTFINAL").isVisible = false;
            widget.getField("DESCRIPTION").isVisible = false;
            widget.getField("TVOOTRECHO").isVisible = false;

            sgcia.isVisible = true;
            widget.getField("ALL_CIA").isVisible = true;
            widget.getField("DTULTATU").isVisible = true;
            widget.getField("DSSERVICOCIAFILI").isVisible = true;
            widget.getField("ALL_SERVICO").isVisible = true;
            this.changeFilial(widget.getField("NMFILIAL"));

        } else if (widget.currentRow.EXPORTNAME == "03") {
            ciaField.validations = {
                required: true
            };
            ciaField.isVisible = true;

            widget.getField("DTINICIAL").isVisible = true;
            widget.getField("DTFINAL").isVisible = true;
            widget.getField("DESCRIPTION").isVisible = true;
            widget.getField("TVOOTRECHO").isVisible = true;

            sgcia.isVisible = false;
            sgcia.validations = {};
            widget.getField("ALL_CIA").isVisible = false;
            widget.getField("DTULTATU").isVisible = false;
            widget.getField("DSSERVICOCIAFILI").isVisible = false;
            widget.getField("ALL_SERVICO").isVisible = false;

            let fC = widget.getField("NMCIA");
            fC.setValue(null);
            self.changeCia(fC);

        }
        templateManager.updateTemplate().then(
            function() {
                templateManager.updateTemplate();
            }
        );
    };

    function verificaDeliveryExp(currentRow) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            },
            {
                "name": "DTEMISDEL",
                "operator": ">=",
                "value": currentRow.DTINICIAL
            },
            {
                "name": "DTEMISDEL",
                "operator": "<=",
                "value": currentRow.DTFINAL
            },
            {
                "name": "IDSITDEL",
                "operator": "<>",
                "value": "3"
            },
            {
                "name": "IDDELIEXPSAP",
                "operator": "=",
                "value": "N"
            }
        ];

        return coc26004_exportacaoSAPService.verificaDeliveryExp(filter);
    }

    function verificaDeliveryReexp(currentRow) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            },
            {
                "name": "DTEMISDEL",
                "operator": ">=",
                "value": currentRow.DTINICIAL
            },
            {
                "name": "DTEMISDEL",
                "operator": "<=",
                "value": currentRow.DTFINAL
            },
            {
                "name": "IDSITDEL",
                "operator": "<>",
                "value": "3"
            },
            {
                "name": "IDDELIEXPSAP",
                "operator": "=",
                "value": "S"
            }
        ];

        return coc26004_exportacaoSAPService.verificaDeliveryReexp(filter);
    }

    function escolheLayout(widget) {
        MessageService.getMessagesByCode(["COC26004_LAYOUT_PADRAO", "COC26004_LAYOUT_CUSTOM", "COC26004_LAYOUT_CANCEL"]).then(function(messages) {
            let config = {
                type: "question",
                icon: "question",
                actions: [
                    {
                        label: messages[0],
                        code: function() {
                            self.callExport(widget, "0");
                        }
                },
                    {
                        label: messages[1],
                        code: function() {
                            self.callExport(widget, "1");
                        }
                },
                    {
                        label: messages[2]
                }]
            };
            MessageService.showCustomDialogByCode("COC26004_ESCOLHA_LAYOUT", {}, config);
        });
    }

    this.export = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.EXPORTNAME == "01") {
                verificaDeliveryExp(widget.currentRow).then(function(deliveryExp) {
                    if (deliveryExp.dataset.deliverym.length) {
                        escolheLayout(widget);
                    } else {
                        MessageService.showMessageByCode("COC26004_N_HA_DELIVERY");
                    }
                });
            } else if (widget.currentRow.EXPORTNAME == "03") {
                verificaDeliveryReexp(widget.currentRow).then(function(deliveryExp) {
                    if (deliveryExp.dataset.deliverym.length) {
                        escolheLayout(widget);
                        // função para reexportação 'coc26004_exportacaoSAPService.reexportaDelivery(widget.currentRow);'
                    } else {
                        MessageService.showMessageByCode("COC26004_N_HA_DELIVERY");
                    }
                });
            } else {
                escolheLayout(widget);
            }
        }
    };

    this.callExport = function(widget, id) {
        var serviceUrlModulo = window.serviceUrls.filter(function(row) {
            return row.package == "coc";
        })[0].baseUrl;

        if (widget.currentRow.EXPORTNAME == "01" || widget.currentRow.EXPORTNAME == "03") {
            var rows;
            var dtInicio = widget.currentRow.DTINICIAL;
            var dtFim = widget.currentRow.DTFINAL;

            if (widget.currentRow.TVOOTRECHO == "N") {
                rows = widget.getField("DESCRIPTION").selectWidget.getCheckedRows();
                rows.forEach(function(row) {
                    row.TVOOTRECHO = "N";
                    row.DTINICIO = dtInicio;
                    row.DTFIM = dtFim;
                    row.NRORG = widget.currentRow.NRORG;
                });
            } else {
                widget.currentRow.DTINICIO = dtInicio;
                widget.currentRow.DTFIM = dtFim;
                rows = [widget.currentRow];
            }
            rows[0].EXPORTNAME = rows[0].EXPORTNAME ? rows[0].EXPORTNAME : widget.currentRow.EXPORTNAME;
            coc26004_exportacaoSAPService.exportaDelivery(rows).then(function(callback) {

                if (callback.dataset.dataset.DATA_ERROR.length) {
                    MessageService.getMessageByCode('COC26004_SERV_SEM_COD_INT').then(function(message) {
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.dataset.DATA_ERROR, message);
                    });

                } else {

                    var fileName = "EXPORT_DELIVERY_" + widget.currentRow.CDFILIAL;
                    //abre a guia do download
                    window.open(
                        serviceUrlModulo + "/downloadExport?" +
                        "cdfilial=" + widget.currentRow.CDFILIAL +
                        "&cdcia=" + widget.currentRow.CDCIA +
                        "&dtinicio=" + dtInicio +
                        "&dtfim=" + dtFim +
                        "&nrlista=" + callback.dataset.dataset.NRLISTA +
                        "&nrorg=" + getLocalVar("INFLIGHTOR_NRORGTRAB") +
                        "&filename=" + fileName +
                        "&id=" + id +
                        "&exportname=" + rows[0].EXPORTNAME

                    );

                }

            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        } else if (widget.currentRow.EXPORTNAME == "02") {
            if (validData(widget.currentRow)) {
                veryfyServico(widget.currentRow).then(
                    function(data_error) {
                        if (data_error.length) {
                            MessageService.getMessageByCode('COC26004_SERV_SEM_COD_INT').then(function(message) {
                                InflightorGeneral.showErrorMessages(widget, data_error, message);
                            });
                        } else {
                            widget.currentRow.NRORG = getLocalVar("INFLIGHTOR_NRORGTRAB");
                            var fileName = "fileName=EXPORT_SERVICO_" + widget.currentRow.CDFILIAL;
                            var servico = widget.currentRow.ALL_SERVICO ? "T" : "_" + widget.currentRow.DSSERVICOCIAFILI.join("_") + "_";
                            var dtultatu = widget.currentRow.DTULTATU ? widget.currentRow.DTULTATU : null;
                            var cdcia = widget.currentRow.ALL_CIA ? "T" : widget.currentRow.SGCIA.join("_");
                            //abre a guia do download
                            window.open(
                                serviceUrlModulo + "/downloadExportServ?" +
                                "cdfilial=" + widget.currentRow.CDFILIAL +
                                "&" + fileName +
                                "&nrorg=" + getLocalVar("INFLIGHTOR_NRORGTRAB") +
                                "&id=" + id +
                                "&dtultatu=" + dtultatu +
                                "&cdcia=" + cdcia +
                                "&servico=" + servico
                            );
                        }
                    }
                );
            } else {
                MessageService.showMessageByCode("COC26004_OPER_BLOQ_INFO");
            }
        }
    };

    function validData(row) {
        var valid = row.CDFILIAL ? true : false;
        valid = valid && row.ALL_CIA ? true : (row.SGCIA ? true : false);
        valid = valid && row.ALL_SERVICO ? true : (row.DSSERVICOCIAFILI ? true : false);

        return valid;
    }

    function veryfyServico(row) {
        var rowFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: row.CDFILIAL
            },
            {
                name: "CDCIA",
                operator: "=",
                value: row.ALL_CIA ? "T" : row.SGCIA.join("_")
            },
            {
                name: "DTULTATU",
                operator: "=",
                value: row.DTULTATU ? row.DTULTATU : null
            },
            {
                name: "SERVICO",
                operator: "=",
                value: row.ALL_SERVICO ? "T" : "_" + row.DSSERVICOCIAFILI.join("_") + "_"
            }
        ];

        return InflightorGeneral.callRestEngine("coc#/crud/coc26004_verify_servicos", "FilterData", rowFilter).then(
            function(callback) {
                var data = callback.dataset.coc26004_verify_servicos;
                var dataError = [];
                for (let index in data) {
                    dataError.push(data[index].DATA_ERROR);
                }
                return dataError;

            }, InflightorGeneral.callbackError
        );
    }

    this.changeFilial = function(field) {
        var widget = field.widget;
        var ciaField = widget.getField("SGCIA");
        ciaField.clearValue();
        if (widget.currentRow.EXPORTNAME == "02") {
            var checkbox = widget.getField("ALL_CIA");
            checkbox.checked = true;
            checkbox.value(true);
            if (field.value()) {
                checkbox.readOnly = false;
            } else {
                checkbox.readOnly = true;
            }
            InflightorGeneral.toggleReadOnlySelect(checkbox, ciaField);
            this.changeSgcia(ciaField);
        }
    };

    this.clickCia = function(field) {
        field.dataSourceFilter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: field.widget.currentRow.CDFILIAL
            }
        ];
        field.reload();
    };

    this.changeCia = function(field) {
        var widget = field.widget;
        widget.getField("DESCRIPTION").clearValue();
        self.companhiaToVooTrecho(widget.getField("DESCRIPTION"));
    };

    this.changeSgcia = function(field) {
        var widget = field.widget;
        var checkbox = widget.getField("ALL_SERVICO");
        checkbox.value(true);
        checkbox.checked = true;
        var servico = widget.getField("DSSERVICOCIAFILI");
        InflightorGeneral.toggleReadOnlySelect(checkbox, servico);
    };

    this.clickServico = function(field) {
        var widget = field.widget;
        var rowFilter = {
            CDFILIAL: widget.currentRow.CDFILIAL,
            CDCIA: widget.currentRow.ALL_CIA ? "T" : (widget.currentRow.SGCIA ? widget.currentRow.SGCIA.join("_") : ""),
            DTULTATU: widget.currentRow.DTULTATU ? widget.currentRow.DTULTATU : null
        };
        InflightorGeneral.onClickDependentField(field, rowFilter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26004_exportacaoSAPController", coc26004_exportacaoSAPController);
});

// FILE: ./assets/js/controllers/coc26004_importacaoProporcaoVariavelController.js
function coc26004_importacaoProporcaoVariavelController(ScreenService, InflightorGeneral, coc26004_importacaoProporcaoVariavelService, MessageService, templateManager) {
    function trataErros(message) {
        if (message && message.length) {
            if (message.indexOf("COC21004_") >= 0) {
                if (message.indexOf("COC21004_QTPAX_INV") >= 0) {
                    let msg_params = message.split("#_#");
                    MessageService.showMessageByCode("COC21004_QTPAX_INV", {
                        DE: msg_params[1],
                        ATE: msg_params[2],
                        DE2: msg_params[3],
                        ATE2: msg_params[4]
                    });
                } else {
                    MessageService.showMessageByCode(message);
                }
            } else {
                ScreenService.showMessage(message);
            }
        }
    }

    var idSubs;
    this.getMainVar = function(widget) {
        var tela = getLocalVar("coc26004_nometela");
        if (tela == "tabelapreco") {
            widget.currentRow.TELA = 1;
        } else if (tela == "menugrid") {
            widget.currentRow.TELA = 2;
        } else if (tela == "configuracaoserv") {
            widget.currentRow.TELA = 3;
        }
    };

    this.uploadCsvFile = function(file, widget, id) {
        idSubs = null;
        idSubs = id ? id : widget.currentRow.TELA;
        if (!idSubs) {
            MessageService.showMessageByCode("COC21004_ORI_INV");
        } else {
            if (file) {
                callUploadMethod(file, idSubs, widget);
            } else {
                MessageService.showMessageByCode("COC21004_ARQ_N_INF");
            }
        }
    };

    var callUploadMethod = function(file, idSubs, widget) {
        var data = {};
        if (idSubs == 1 || idSubs == 4) {
            data = {
                "CDFILIAL": widget.container.getWidget("servicostabela").currentRow.CDFILIAL,
                "CDCIA": widget.container.getWidget("servicostabela").currentRow.CDCIA,
                "NRTABPRECO": widget.container.getWidget("servicostabela").currentRow.NRTABPRECO,
                "NRSEQPRECO": widget.container.getWidget("servicostabela").currentRow.NRSEQPRECO,
                "NRSERVICOCIAFILI": widget.container.getWidget("servicostabela").currentRow.NRSERVICOCIAFILI,
                "CDCLASSE": widget.container.getWidget("servicostabela").currentRow.CDCLASSE,
                "CDCICLO": widget.container.getWidget("servicostabela").currentRow.CDCICLO
            };
        } else {
            data = getLocalVar("coc26004_data");
        }

        coc26004_importacaoProporcaoVariavelService.uploadCsvFile(file, idSubs, data).then(function() {
            afterImporting(widget, file, idSubs);
        }, function(error) {
            if (error.data && error.data.error) {
                trataErros(error.data.error);
            }
        });
    };

    function afterImporting(widget, file, idSubs) {
        ScreenService.closePopup(true);
        file = [];
        var sourceWidget = widget;

        if (idSubs == 1) {
            sourceWidget = widget.container.getWidget("precoProporcional");
        } else if (idSubs == 4) {
            sourceWidget = widget.container.getWidget("proporcaoVariavel");
        }

        sourceWidget.activate();
        sourceWidget.reload().then(function() {
            MessageService.showMessageByCode("COC21004_IMPORT_SUCCES");
            if (idSubs == 4) {
                if (sourceWidget.dataSource.data && sourceWidget.dataSource.data.length) {
                    sourceWidget.getAction("ZH_DELETE").isVisible = true;
                } else {
                    sourceWidget.getAction("ZH_DELETE").isVisible = false;
                }
            }
        }, InflightorGeneral.callbackError);
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register("coc26004_importacaoProporcaoVariavelController", coc26004_importacaoProporcaoVariavelController);
});

// FILE: ./assets/js/controllers/coc27000_familiaReceitaController.js
var coc27000_familiaReceitaController = function(InflightorGeneral, MessageService) {

    this.salvaFamiliaReceita = function(widget) {
        var registroExistente = verificaExisteFamilia(widget.dataSource.data, widget.currentRow, "NMFAMILIAREC");

        if (!registroExistente && widget.currentRow.NMFAMILIAREC.length > 0) {
            InflightorGeneral.saveCrud(widget);
        } else if (widget.currentRow.NMFAMILIAREC.length == 0) {
            MessageService.showMessageByCode("MISSING_REQUIRED_FIELDS");
        } else {
            MessageService.showMessageByCode("REGISTER_EXISTS");
        }
    };

    function verificaExisteFamilia(dataSource, currentRow, fieldName) {
        for (let row in dataSource) {
            if (dataSource[row][fieldName].toUpperCase() == currentRow[fieldName].toUpperCase() && !dataSource[row].changed) {
                return true;
            }
        }

        return false;
    }

};

Configuration(function(ContextRegister) {
    ContextRegister.register("coc27000_familiaReceitaController", coc27000_familiaReceitaController);
});

// FILE: ./assets/js/controllers/CompanhiaAereaController.js
var CompanhiaAereaController = function(ScreenService, MessageService, FilterUtil, FieldUtil, InflightorGeneral, templateManager, General) {

    var self = this;
    var imgIndef = window.undefinedImage;

    this.setEndeclieFilter = function(widget) {
        if (!widget.getParent().currentRow.__is_new) {
            widget.dataSourceFilter = [];

            var parentRow = widget.getParent().currentRow;

            FilterUtil.updateFilter(widget, parentRow.CDCLIENTE, "CDCLIENTE");

            if (widget.mode != "edit" && widget.mode != "view") {
                widget.reload();
            }
        }
    };

    this.filterAfterApply = function(args) {
        const stripeWidget = templateManager.container.getWidget('FILTER');
        stripeWidget.currentRow = angular.copy(args.e.preference.FILTER_ROW);
    };

    this.deleteLogo = function(args, widget) {
        var logo = angular.copy(widget.currentRow.FGLOGOCIA);

        if (logo && logo != "http://midiatest.teknisa.com/files/") {
            widget.currentRow.FGLOGOCIA = null;
            ScreenService.confirmMessage(ScreenService.i18n("Deletar Logo da Companhia?"), "question", function() {
                self.salvaCia(widget, true);
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        } else {
            ScreenService.showMessage("A companhia não possui uma logo definida!");
        }
    };

    this.setEndeclieFilterOnGridClick = function(widget) {

        if (widget.swipeParent) {
            var swipeWidget = widget.swipeParent.currentWidget;
            if (swipeWidget.name != widget.name) {
                if (widget.mode != "list") {
                    if (swipeWidget.name == "enderecoCliente") {

                        swipeWidget.dataSourceFilter = [];

                        FilterUtil.updateFilter(swipeWidget, widget.currentRow.CDCLIENTE, "CDCLIENTE");

                        if (swipeWidget.mode != "edit" && swipeWidget.mode != "view") {
                            swipeWidget.reload();
                        }
                    }
                }
                swipeWidget.reload();
            }
        }
    };

    this.setCurrentRowForSave = function(widget) {
        self.copyCdclienteFromFilter(widget);
        self.setTipoendeclieField(widget);
    };

    this.verificaEnderecos = function(widget) {

        if (widget.dataSource.data.length == 4) {
            widget.getAction("Add").isVisible = false;
        } else {
            widget.getAction("Add").isVisible = true;
        }
    };

    this.copyCdclienteFromFilter = function(widget) {
        var filteredCdCliente = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDCLIENTE");
        widget.currentRow.CDCLIENTE = filteredCdCliente;
    };

    this.gridAfterMoveRow = function(widget) {
        if (!widget.currentRow.FGLOGOCIA) widget.currentRow.FGLOGOCIA = imgIndef;
        var tabs = widget.widgets;
        tabs.forEach(function(tab) {
            if (tab.active) self.setCiaFilter(tab);
        });
    };

    this.editCompanhia = function(widget) {
        InflightorGeneral.edit(widget);
        widget.currentRow.PHOTO_UPLOAD = [];
        setVisibilityInSonWidgets(widget, false);
    };

    function setVisibilityInSonWidgets(widget, isVisible) {
        var sonWidgets = ["enderecoCliente", "contatocia", "leancodecia", "ciafili"];
        for (let index in sonWidgets) {
            widget.container.getWidget(sonWidgets[index]).showAsTab = isVisible;
        }
    }

    this.setCiaFilter = function(widget) {
        var parentRow = widget.getParent().currentRow;
        FilterUtil.updateFilter(widget, parentRow.CDCIA, "CDCIA");
        if (widget.mode != "edit" && widget.mode != "view" && widget.name !== "enderecoCliente") {
            widget.reload();
        }
    };

    this.copyCdciaFromFilter = function(widget) {
        var filteredCdCia = FilterUtil.getFilterValueByName(widget.dataSourceFilter, "CDCIA");

        if (filteredCdCia) {
            widget.currentRow.CDCIA = filteredCdCia;
        }
    };

    this.enableEstadoField = function(widget) {
        var paisField = widget.getField("CDPAIS");
        var estadoField = widget.getField("NMESTADO");
        var municipioField = widget.getField("NMMUNICIPIO");

        if (widget.currentRow.__is_new) {
            FieldUtil.clearField(estadoField);
            FieldUtil.toggleField(municipioField, false);
            FieldUtil.clearField(municipioField);
        }

        if (paisField.value()) {
            FieldUtil.toggleField(estadoField, true);
            FilterUtil.updateFilter(estadoField, paisField.value(), "CDPAIS");
            estadoField.reload();
        } else {
            FieldUtil.toggleField(estadoField, false);
        }
    };

    this.enableMunicipioField = function(widget) {
        var paisField = widget.getField("CDPAIS");
        var estadoField = widget.getField("SGESTADO");
        var municipioField = widget.getField("NMMUNICIPIO");
        if (widget.currentRow.__is_new) {
            FieldUtil.clearField(municipioField);
        }

        if (estadoField.value()) {
            FieldUtil.toggleField(municipioField, true);
            FilterUtil.updateFilter(municipioField, paisField.value(), "CDPAIS");
            FilterUtil.updateFilter(municipioField, estadoField.value(), "SGESTADO");
            municipioField.reload();
        } else {
            FieldUtil.toggleField(municipioField, false);
        }
    };

    this.filterDataRowsFromSelect = function(field, primaryKey) {
        var widget = field.widget;
        var data = widget.dataSource.data;
        var newArray = data.filter(function(currentItem) {
            return currentItem[primaryKey];
        });
        var keyArray = newArray.map(function(currentItem) {
            return currentItem[primaryKey];
        });
        if (keyArray.length > 0) {
            FilterUtil.updateFilter(field, keyArray, primaryKey, "NOT_IN");
            field.reload();
        }
    };

    this.setTipoendeclieField = function(widget) {
        var editFieldValue = widget.getField("IDTPENDECLIE_EDIT").value();
        if (editFieldValue) {
            widget.currentRow.IDTPENDECLIE = editFieldValue;
        }
    };

    this.popDataToCurrentRow = function(widget) {
        var data = widget.dataSource.data;
        if (data)
            widget.currentRow = data[data.length - 1];
    };

    this.filterDataRowsFromFieldDataByTipo = function(widget) {

        var field = widget.getField("IDTPENDECLIE_EDIT");
        var widgetData = widget.dataSource.data;
        var fieldData = field.dataSource.data;

        var newData = fieldData.filter(function(fieldDataItem) {

            var dataToRemove = widgetData.find(function(widgetDataItem) {
                if (fieldDataItem.value == widgetDataItem.IDTPENDECLIE) {
                    return true;
                }
                return false;
            });

            if (dataToRemove) return false;
            else return true;
        });

        field.dataSource.data = newData;
    };

    this.enderecoNewRow = function(enderecoWidget, fromNewCia = false) {
        enderecoWidget.newRow();
        if (fromNewCia) {
            enderecoWidget.getAction("ZH_SAVE").isVisible = false;
            enderecoWidget.getAction("ZH_SAVE_CLICIAEND").isVisible = true;
            enderecoWidget.getAction("ZH_CANCEL").isVisible = false;
            enderecoWidget.getAction("ZH_CANCEL_CLICIAEND").isVisible = true;
        } else {
            self.setTpendeclieVisibility(enderecoWidget, true);
            self.filterDataRowsFromFieldDataByTipo(enderecoWidget);
            InflightorGeneral.justInteger('#NRCEPCLIE');
            InflightorGeneral.justInteger('#NRRAMALCLIE');
        }
    };

    this.setTpendeclieVisibility = function(widget, enable) {
        if (enable) {
            widget.getField("IDTPENDECLIE_EDIT").isVisible = true;
            widget.getField("IDTPENDECLIE").showOnForm = false;
        } else {
            widget.getField("IDTPENDECLIE_EDIT").isVisible = false;
            widget.getField("IDTPENDECLIE").showOnForm = true;
        }
    };

    this.restoreTipoFieldData = function(widget) {
        var field = widget.getField("IDTPENDECLIE_EDIT");

        field.dataSource.data = [
            {
                "name": "Comercial",
                "value": "P"
            },
            {
                "name": "Cobrança",
                "value": "C"
            },
            {
                "name": "Internacional",
                "value": "E"
            },
            {
                "name": "Exportação",
                "value": "X"
            }
        ];
    };

    this.abrePopup = function(popup) {
        popup.dataSourceFilter[0].value = getMainWidget().currentRow.CDCIA;
        popup.activate();
        popup.reload();
        popup.unSelectAll();
    };

    this.vinculaCompanhiaFilial = function(popupUnidade) {
        var datasetToSave = [];
        popupUnidade.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            datasetToSave.push(row);
        });
        InflightorGeneral.callRestEngine("coc#/crud/ciafili/save", "DataSet", datasetToSave).then(function() {
            ScreenService.closePopup();
            popupUnidade.container.getWidget("ciafili").reload();
            popupUnidade.container.getWidget("ciafili").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.vinculaCompanhiaLeanCode = function(popupLeanCode) {
        var datasetToSave = [];
        popupLeanCode.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            row.IDBLOQUEIA = "N";
            datasetToSave.push(row);
        });
        InflightorGeneral.callRestEngine("coc#/crud/leancodecia/save", "DataSet", datasetToSave).then(function() {
            ScreenService.closePopup();
            popupLeanCode.container.getWidget("leancodecia").reload();
            popupLeanCode.container.getWidget("leancodecia").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.validaActionLeanCode = function(widgetLeanCode) {
        if (widgetLeanCode.getCheckedRows().length > 1) {
            widgetLeanCode.getAction("BLOQUEIA").isVisible = true;
            widgetLeanCode.getAction("DESBLOQUEIA").isVisible = true;
            widgetLeanCode.getAction("ALTERNA").isVisible = false;
        } else {
            widgetLeanCode.getAction("ALTERNA").isVisible = true;
            widgetLeanCode.getAction("BLOQUEIA").isVisible = false;
            widgetLeanCode.getAction("DESBLOQUEIA").isVisible = false;
        }
    };

    this.alteraLeanCode = function(widgetLeanCode, valor) {
        var datasetToSave = [];

        widgetLeanCode.getCheckedRows().forEach(function(row) {
            row.IDBLOQUEIA = valor;
            datasetToSave.push(row);
        });

        InflightorGeneral.callRestEngine("coc#/crud/leancodecia/save", "DataSet", datasetToSave).then(function() {
            widgetLeanCode.reload();
            widgetLeanCode.activate();
            widgetLeanCode.unSelectAll();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.alternaLeanCode = function(widgetLeanCode) {
        var row = widgetLeanCode.getCheckedRows()[0];

        if (row.IDBLOQUEIA == "S") {
            row.IDBLOQUEIA = "N";
        } else {
            row.IDBLOQUEIA = "S";
        }
        row.changed = true;
        InflightorGeneral.callRestEngine("coc#/crud/leancodecia/save", "Row", row).then(function() {
            widgetLeanCode.reload();
            widgetLeanCode.activate();
            widgetLeanCode.unSelectAll();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };
    this.salvaCia = function(widget, logo) {
        var widgetEndereco = widget.container.getWidget("enderecoCliente");
        if (widget.currentRow.__is_new) {
            widget.currentRow.IDENTRSAIDOP = "S";
            if (widget.isValid()) {
                widgetEndereco.dataSource.data = [];
                InflightorGeneral.callRestEngine("coc#/companhia_cliente_operador/validateSigla", "Row", widget.currentRow).then(function() {
                    self.enableTabs(widget);
                    widgetEndereco.activate();
                    self.enderecoNewRow(widgetEndereco, true);
                }, function(err) {
                    ScreenService.showMessage(err.data.error, "error");
                });
            }
        } else {
            InflightorGeneral.callRestEngine("coc#/companhia_cliente_operador/save", "Row", widget.currentRow).then(function() {
                if (logo) {
                    widget.list();
                    logo = false;
                } else {
                    widget.view();
                }
                widget.reload();
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
        }

    };

    this.salvaClienteEnderecoCompanhia = function(widget) {
        if (!widget.isValid()) return;
        var widgetEndereco = widget.container.getWidget("enderecoCliente");
        var rowEndereco = widget.currentRow;
        rowEndereco.IDTPENDECLIE = rowEndereco.IDTPENDECLIE_EDIT;
        var rowClienteCompanhia = widget.getParent().currentRow;
        var widgetClienteCompanhia = widget.getParent();

        InflightorGeneral.callRestEngine("coc#/companhia_cliente_operador/save", "Row", rowClienteCompanhia).then(function(callback) {
            rowEndereco.CDCLIENTE = callback.dataset.novacompanhia[0].CDCLIENTE;
            InflightorGeneral.saveCrud(widget).then(function() {
                widget.dataSourceFilter = [
                    {
                        name: "CDCLIENTE",
                        operator: "=",
                        value: rowEndereco.CDCLIENTE
                    }
                ];
                widgetClienteCompanhia.view();
                widgetClienteCompanhia.currentRow.CDCIA = callback.dataset.novacompanhia[0].CDCIA;
                widgetClienteCompanhia.reload().then(function() {
                    widget.reload().then(function() {
                        widget.getAction("ZH_SAVE").isVisible = true;
                        widget.getAction("ZH_SAVE_CLICIAEND").isVisible = false;
                        widgetEndereco.getAction("ZH_CANCEL").isVisible = true;
                        widgetEndereco.getAction("ZH_CANCEL_CLICIAEND").isVisible = false;
                        widgetEndereco.list();
                    });
                });
            });
        }, function(err) {
            ScreenService.showMessage(err.data.error);
        });
    };

    function getMainWidget() {
        return templateManager.container.getWidget('cadastroDeCompanhia');
    }

    this.cancelaNovoRegistro = function(widget) {
        const mainWidget = getMainWidget();
        var widgetEndereco = widget.container.getWidget("enderecoCliente");
        widget.currentRow = {};
        widget.list();
        if (mainWidget.currentRow.__is_new) {
            mainWidget.dataSource.data.pop();
        }
        widget.getAction("ZH_SAVE").isVisible = true;
        widget.getAction("ZH_SAVE_CLICIAEND").isVisible = false;
        widgetEndereco.getAction("ZH_CANCEL").isVisible = true;
        widgetEndereco.getAction("ZH_CANCEL_CLICIAEND").isVisible = false;
        ScreenService.closeSwipe().then(function() {
            widget.getParent().currentWidget = widget.getParent();
            mainWidget.activate();
        });
    };

    this.vinculaCompanhiaLeanCode = function(popupLeanCode) {
        var datasetToSave = [];
        popupLeanCode.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            row.IDBLOQUEIA = "N";
            datasetToSave.push(row);
        });
        InflightorGeneral.callRestEngine("coc#/crud/leancodecia/save", "DataSet", datasetToSave).then(function() {
            ScreenService.closePopup();
            popupLeanCode.container.getWidget("leancodecia").reload();
            popupLeanCode.container.getWidget("leancodecia").activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    };

    this.desvinculaCompanhiaFilial = function(widgetUnidade) {
        var msg = "Deseja desvincular da companhia a(s) unidades(es) selecionada(s)?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), msg, function() {
            InflightorGeneral.callRestEngine("coc#/crud/ciafili/remove", "DataSet", widgetUnidade.getCheckedRows()).then(function() {
                widgetUnidade.reload();
                widgetUnidade.activate();
                widgetUnidade.dataSource.clearCheckedRows();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    this.desvinculaCompanhiaLeanCode = function(widgetLeanCode) {
        var msg = "Deseja desvincular da companhia o(s) Lean Code(s) selecionado(s)?";
        ScreenService.confirmMessage(ScreenService.i18n(msg), msg, function() {
            InflightorGeneral.callRestEngine("coc#/crud/leancodecia/remove", "DataSet", widgetLeanCode.getCheckedRows()).then(function() {
                widgetLeanCode.reload();
                widgetLeanCode.activate();
                widgetLeanCode.unSelectAll();
            }, function(error) {
                if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                    MessageService.showMessageByCode("DELETE_ERROR");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
        widgetLeanCode.activate();
    };

    this.onUploadPhoto = function(widget, photoUpload) {
        setTimeout(function() {
            if (isEmpty(photoUpload)) {
                self.onUploadPhoto(widget, photoUpload);
            }
            var validExt = widget.getField("PHOTO_UPLOAD").validations.extensions.value;
            var fileExt = "." + photoUpload[0].name.split(".").pop();
            if (validExt.indexOf(fileExt) != -1) {
                widget.currentRow.FGLOGOCIA = photoUpload[0].b64File;
            } else {
                widget.currentRow.PHOTO_UPLOAD.splice(widget.currentRow.PHOTO_UPLOAD.indexOf(photoUpload), 1);
                widget.getField("PHOTO_UPLOAD").fileList = widget.currentRow.PHOTO_UPLOAD;
                widget.currentRow.FGLOGOCIA = imgIndef;
            }
            templateManager.updateTemplate();
        }, 500);
    };

    this.onDeselectPhoto = function(widget) {
        widget.currentRow.FGLOGOCIA = imgIndef;
        templateManager.updateTemplate();
    };

    this.adicionarCompanhia = function(widget) {
        widget.newRow();
        widget.currentRow.FGLOGOCIA = imgIndef;
        widget.currentRow.PHOTO_UPLOAD = [];
        // templateManager.updateTemplate();
    };

    var isEmpty = function(value) {
        return !value || value.length === 0;
    };

    this.disableTabs = function(widget) {
        widget.widgets.forEach(function(childWidget) {
            childWidget.showAsTab = false;
        });
    };

    this.enableTabs = function(widget) {
        widget.widgets.forEach(function(childWidget) {
            childWidget.showAsTab = true;
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("CompanhiaAereaController", CompanhiaAereaController);
});

// FILE: ./assets/js/controllers/WorkflowController.js
var WorkflowController = function(ScreenService, InflightorGeneral, WorkflowService, ZHPromise, MessageService, templateManager) {
    var self = this;

    this.parentAferMoveRow = function(widgetParent) {
        if (widgetParent.currentRow.PARENT) {
            var NRITEMPROCAPROVA = widgetParent.currentRow.PRIMARY.slice(1);
            var dataSourceFilter = [
                {
                    name: "NRPROCESSOAPROVA",
                    operator: "=",
                    value: widgetParent.currentRow.PARENT
                },
                {
                    name: "NRITEMPROCAPROVA",
                    operator: "=",
                    value: NRITEMPROCAPROVA
                },
                {
                    name: "NRORG",
                    operator: "=",
                    value: getLocalVar("INFLIGHTOR_NRORGTRAB")
                }
            ];

            var widgetItem = widgetParent.widgets[0].widgets[0];
            var widgetOperador = widgetParent.widgets[0].widgets[1];

            widgetItem.dataSourceFilter = dataSourceFilter;
            widgetOperador.dataSourceFilter = dataSourceFilter;

            widgetItem.reload().then(function() {
                widgetItem.currentRow = widgetItem.dataSource.data[0];
                self.itemView(widgetItem);
                ScreenService.openSwipe(widgetItem.parent);
            });
            widgetOperador.reload();
        } else ScreenService.closeSwipe(0);
    };

    this.itemView = function(widgetItem) {
        var widgetOperador = widgetItem.container.getWidget("widgetOperador");
        widgetOperador.isVisible = true;
        widgetItem.template = widgetItem.templateToView;
        widgetItem.view();
    };

    this.itemEdit = function(widgetItem) {
        var widgetOperador = widgetItem.container.getWidget("widgetOperador");
        widgetOperador.isVisible = false;
        widgetItem.template = widgetItem.templateToForm;
        InflightorGeneral.edit(widgetItem);
    };

    this.saveItem = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            self.itemView(widget);
        });
    };

    this.vinculaOperador = function(popupOperador) {
        var datasetToSave = [];
        var filter = [
            {
                name: "NRPROCESSOAPROVA",
                operator: "=",
                value: popupOperador.container.getWidget("widgetParent").currentRow.PARENT
            },
            {
                name: "NRITEMPROCAPROVA",
                operator: "=",
                value: popupOperador.container.getWidget("widgetParent").currentRow.PRIMARY.slice(1)
            }
        ];
        InflightorGeneral.callRestEngine("coc#/crud/itemprocaprova", "FilterData", filter).then(function(callback) {
            var row = callback.dataset.itemprocaprova[0];
            if (row.NRDIASPROCAPROVA) {
                var nrdiasoperapro = row.NRDIASPROCAPROVA;
            }

            popupOperador.getCheckedRows().forEach(function(row) {
                row.__is_new = true;
                row.NRDIASOPERAPRO = nrdiasoperapro;
                row.NRPTOACAOOPER = 0;
                row.IDTIPOACAOOPER = "A";
                row.CDOPER = row.CDOPERADOR;
                datasetToSave.push(row);
            });

            InflightorGeneral.callRestEngine("coc#/vinculaOperItems", "DataSet", datasetToSave).then(function() {
                popupOperador.unSelectAll();
                ScreenService.closePopup();
                popupOperador.container.getWidget("widgetOperador").reload();
                popupOperador.container.getWidget("widgetOperador").activate();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data) {
                    errorMessage = error.data.error;
                    ScreenService.showMessage(errorMessage);
                }
            });
        });
    };

    this.concatCdOperador = function(rows) {
        var cdoperConcat = "";
        for (let index in rows) {
            cdoperConcat += rows[index].CDOPERADOR;
        }
        return cdoperConcat;
    };

    this.desvinculaOperador = function(widgetOperador) {
        MessageService.confirmMessageByCode("COC00003_CONFIRM_DESVINC_OPER").then(function() {
            var filterWf = [
                {
                    name: "CDOPER",
                    operator: "=",
                    value: self.concatCdOperador(widgetOperador.getCheckedRows())
                }
            ];
            InflightorGeneral.callRestEngine("coc#/crud/coc00003_operworkflow_workflow_pendencia", "FilterData", filterWf).then(
                function(callback) {
                    if (callback.dataset.coc00003_operworkflow_workflow_pendencia[0].QTDE > 0) {
                        MessageService.showMessageByCode("COC00003_OPER_PROC_PEND");
                    } else {
                        let dataSourceToSave = [];
                        widgetOperador.getCheckedRows().forEach(function(row) {
                            row.CDOPER = row.CDOPERADOR;
                            dataSourceToSave.push(row);
                        });
                        InflightorGeneral.callRestEngine("coc#/desvinculaOperItems", "DataSet", dataSourceToSave).then(function() {
                            widgetOperador.unSelectAll();
                            widgetOperador.reload();
                            widgetOperador.activate();
                        }, function(error) {
                            if (error.data && error.data.error.indexOf("ORA-02292") !== -1) {
                                MessageService.showMessageByCode("DELETE_ERROR");
                            } else {
                                ScreenService.showMessage(error.data.error);
                            }
                        });
                    }
                },
                function(error) {
                    ScreenService.showMessage(error.data.error);
                }
            );

        });
    };

    this.abrePopup = function(popup) {
        popup.dataSourceFilter[0].value = popup.container.getWidget("widgetParent").currentRow.PARENT;
        popup.dataSourceFilter[1].value = popup.container.getWidget("widgetParent").currentRow.PRIMARY.slice(1);
        popup.isVisible = true;
        popup.activate();
        popup.reload();
    };

    this.validaOperador = function(widgetOperador) {
        if (widgetOperador.isValid()) {
            var nrprocessoaprova = widgetOperador.container.getWidget("widgetParent").currentRow.PARENT;
            var nritemprocaprova = widgetOperador.container.getWidget("widgetParent").currentRow.PRIMARY.slice(1);

            WorkflowService.getNrdiasoperaproDefault(nrprocessoaprova, nritemprocaprova).then(function(callback) {
                if (callback && callback.dataset && callback.dataset.itemprocaprova[0]) {
                    if (parseInt(callback.dataset.itemprocaprova[0].NRDIASPROCAPROVA) >= parseInt(widgetOperador.currentRow.NRDIASOPERAPRO)) {
                        let row = angular.copy(widgetOperador.currentRow);
                        row.CDOPER = row.CDOPERADOR;
                        InflightorGeneral.callRestEngine("coc#/atualizaOperItem", "Row", row).then(function() {
                            widgetOperador.reload().then(function() {
                                widgetOperador.view();
                            });
                        }).catch(function(error) {
                            MessageService.showMessageByCode('ERRO_INESPERADO');
                            console.log(error);
                        });
                    } else {
                        MessageService.showMessageByCode("COC00003_NUM_DIAS_OPER_MAIOR_ITEM");
                    }
                } else {
                    MessageService.showMessageByCode("ERRO_INESPERADO");
                }
            });
        }
    };

    this.validaActionAtivaInativa = function(widgetParent) {
        if (widgetParent.getCheckedRows().length > 1) {
            widgetParent.getAction("ATIVA").isVisible = true;
            widgetParent.getAction("DESATIVA").isVisible = true;
            widgetParent.getAction("ALTERNA").isVisible = false;
        } else {
            widgetParent.getAction("ALTERNA").isVisible = true;
            widgetParent.getAction("ATIVA").isVisible = false;
            widgetParent.getAction("DESATIVA").isVisible = false;
        }
        widgetParent.activate();
    };

    this.alteraAtivaInativa = function(widgetParent, valor) {
        var promises = [];

        var datasetProcessoAprova = [];
        var datasetItemProcAprova = [];

        widgetParent.getCheckedRows().forEach(function(row) {
            var rowToSave = angular.copy(row);
            rowToSave.IDATIVO = valor;
            if (rowToSave.PARENT) {
                rowToSave.NRPROCESSOAPROVA = rowToSave.PARENT;
                rowToSave.NRITEMPROCAPROVA = rowToSave.PRIMARY.slice(1);
                datasetItemProcAprova.push(rowToSave);

                widgetParent.dataSource.data.forEach(function(datasetRow) {
                    if (datasetRow.PRIMARY == rowToSave.PARENT && datasetRow.IDATIVO != valor) {
                        datasetRow.NRPROCESSOAPROVA = rowToSave.PARENT;
                        datasetRow.IDATIVO = valor;
                        InflightorGeneral.callRestEngine("coc#/crud/processoaprova/save", "Row", datasetRow).then(function() {}, function(error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    }
                });

            } else {
                rowToSave.NRPROCESSOAPROVA = rowToSave.PRIMARY;
                datasetProcessoAprova.push(rowToSave);
            }
        });

        if (datasetProcessoAprova.length)
            promises.push(InflightorGeneral.callRestEngine("coc#/crud/processoaprova/save", "DataSet", datasetProcessoAprova));

        if (datasetItemProcAprova.length)
            promises.push(InflightorGeneral.callRestEngine("coc#/crud/itemprocaprova/save", "DataSet", datasetItemProcAprova));

        ZHPromise.all(promises).then(function() {
            widgetParent.reload();
            widgetParent.activate();
        }, function(error) {
            var errorMessage = "Timeout";
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });

    };

    this.alternaAtivaInativa = function(widgetParent) {
        var rows = widgetParent.getCheckedRows();
        var rowPai;
        rows.forEach(function(rowToSave) {
            var dataSourceToSave;

            if (rowToSave.IDATIVO == "S") {
                rowToSave.IDATIVO = "N";
            } else {
                rowToSave.IDATIVO = "S";
            }
            if (rowToSave.PARENT) {
                rowToSave.NRPROCESSOAPROVA = rowToSave.PARENT;
                rowToSave.NRITEMPROCAPROVA = rowToSave.PRIMARY.slice(1);
                dataSourceToSave = "coc#/crud/itemprocaprova/save";
            } else {
                rowToSave.NRPROCESSOAPROVA = rowToSave.PRIMARY;
                dataSourceToSave = "coc#/crud/processoaprova/save";
            }

            if (rowToSave.PARENT) {
                let idativo = "N";
                widgetParent.dataSource.data.forEach(function(procs) {
                    if (procs.PRIMARY === rowToSave.NRPROCESSOAPROVA) {
                        rowPai = procs;
                        rowPai.NRPROCESSOAPROVA = rowToSave.NRPROCESSOAPROVA;
                    }
                    if (procs.PARENT == rowToSave.PARENT) {
                        if (procs.IDATIVO == "S") {
                            idativo = "S";
                        }
                    }
                });
                rowPai.IDATIVO = idativo;
                InflightorGeneral.callRestEngine("coc#/crud/processoaprova/save", "Row", rowPai).then(function() {}).catch(function(err) {
                    ScreenService.showMessage(err.data.error);
                });
            }

            InflightorGeneral.callRestEngine(dataSourceToSave, "Row", rowToSave).then(function() {
                widgetParent.reload();
                widgetParent.activate();
            }, function(error) {
                var errorMessage = "Timeout";
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        });
    };

    this.verificaNecesPop = function(widgetMain) {
        if (widgetMain.dataSource && widgetMain.dataSource.data && widgetMain.dataSource.data.length) {
            let mensagens = widgetMain.dataSource.data.map(row => row.NAME);
            MessageService.getMessagesByCode(mensagens).then(function(msgs) {
                for (let i = 0; i < widgetMain.dataSource.data.length; i++) {
                    widgetMain.dataSource.data[i].NAME = msgs[i];
                    widgetMain.dataSource.data[i].__is_new = false;
                }
            });
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("WorkflowController", WorkflowController);
});
